// @(#) ENIB/LI2 (c) - oRis - Version 2.3
//--------------------------------------------------------------------------//
// file        : listCd.h                                                   //
// description : header file for the ENIB/LI2 List codes                    //
// author(s)   : F. Harrouet                                                //
// copyright   : Laboratoire d'Informatique Industrielle (LI2)              //
//               Ecole Nationale d'Ingenieurs de Brest (ENIB)               //
//               CP 30815 - 29608 Brest Cedex - FRANCE                      //
//               Tel: (33) 02.98.05.66.31 , Fax: (33) 02.98.05.66.29        //
//               e-mail: li2@enib.fr                                        //
//--------------------------------------------------------------------------//
// Copyright (c) 1996-2000                                                  //
// Laboratoire d'Informatique Industrielle (LI2)                            //
// Ecole Nationale d'Ingenieurs de Brest (ENIB)                             //
//                                                                          //
// Permission to use, copy, modify, distribute and sell this software       //
// and its documentation for any purpose is hereby granted without fee,     //
// provided that the above copyright notice appear in all copies and        //
// that both that copyright notice and this permission notice appear        //
// in supporting documentation.                                             //
// ENIB/LI2 makes no representations about the suitability of this          //
// software for any purpose.                                                //
// It is provided "as is" without express or implied warranty.              //
//--------------------------------------------------------------------------//

#ifndef LISTCD_H
#define LISTCD_H

#include "oRis/Codes/smaCd.h"
#include "oRis/Main/value.h"

//--------------------------------------------------------------------------//
// class BackRefCode                                                        //
//--------------------------------------------------------------------------//

class BackRefCode : public SmaCode
{
public:
// Allocators
  BackRefCode(const SmaNode * node);
  virtual ~BackRefCode(void);
// Evaluation
  virtual void eval(Context * context) const;
};


//--------------------------------------------------------------------------//
// class BackCopyCode                                                       //
//--------------------------------------------------------------------------//

class BackCopyCode : public SmaCode
{
public:
// Allocators
  BackCopyCode(const SmaNode * node);
  virtual ~BackCopyCode(void);
// Evaluation
  virtual void eval(Context * context) const;
};


//--------------------------------------------------------------------------//
// class FrontRefCode                                                       //
//--------------------------------------------------------------------------//

class FrontRefCode : public SmaCode
{
public:
// Allocators
  FrontRefCode(const SmaNode * node);
  virtual ~FrontRefCode(void);
// Evaluation
  virtual void eval(Context * context) const;
};


//--------------------------------------------------------------------------//
// class FrontCopyCode                                                      //
//--------------------------------------------------------------------------//

class FrontCopyCode : public SmaCode
{
public:
// Allocators
  FrontCopyCode(const SmaNode * node);
  virtual ~FrontCopyCode(void);
// Evaluation
  virtual void eval(Context * context) const;
};


//--------------------------------------------------------------------------//
// class Index_S_I_Code                                                     //
//--------------------------------------------------------------------------//

class Index_S_I_Code : public SmaCode
{
public:
// Allocators
  Index_S_I_Code(const SmaNode * node);
  virtual ~Index_S_I_Code(void);
// Evaluation
  virtual void eval(Context * context) const;
};


//--------------------------------------------------------------------------//
// class Index_S_R_Code                                                     //
//--------------------------------------------------------------------------//

class Index_S_R_Code : public SmaCode
{
public:
// Allocators
  Index_S_R_Code(const SmaNode * node);
  virtual ~Index_S_R_Code(void);
// Evaluation
  virtual void eval(Context * context) const;
};


//--------------------------------------------------------------------------//
// class IndexRef_I_Code                                                    //
//--------------------------------------------------------------------------//

class IndexRef_I_Code : public SmaCode
{
public:
// Allocators
  IndexRef_I_Code(const SmaNode * node);
  virtual ~IndexRef_I_Code(void);
// Evaluation
  virtual void eval(Context * context) const;
};


//--------------------------------------------------------------------------//
// class IndexRef_R_Code                                                    //
//--------------------------------------------------------------------------//

class IndexRef_R_Code : public SmaCode
{
public:
// Allocators
  IndexRef_R_Code(const SmaNode * node);
  virtual ~IndexRef_R_Code(void);
// Evaluation
  virtual void eval(Context * context) const;
};


//--------------------------------------------------------------------------//
// class IndexCopy_I_Code                                                   //
//--------------------------------------------------------------------------//

class IndexCopy_I_Code : public SmaCode
{
public:
// Allocators
  IndexCopy_I_Code(const SmaNode * node);
  virtual ~IndexCopy_I_Code(void);
// Evaluation
  virtual void eval(Context * context) const;
};


//--------------------------------------------------------------------------//
// class IndexCopy_R_Code                                                   //
//--------------------------------------------------------------------------//

class IndexCopy_R_Code : public SmaCode
{
public:
// Allocators
  IndexCopy_R_Code(const SmaNode * node);
  virtual ~IndexCopy_R_Code(void);
// Evaluation
  virtual void eval(Context * context) const;
};


//--------------------------------------------------------------------------//
// class EraseCode                                                          //
//--------------------------------------------------------------------------//

class EraseCode : public SmaCode
{
public:
// Allocators
  EraseCode(const SmaNode * node);
  virtual ~EraseCode(void);
// Evaluation
  virtual void eval(Context * context) const;
};


//--------------------------------------------------------------------------//
// class SortCode                                                           //
//--------------------------------------------------------------------------//

class SortCode : public SmaCode
{
public:
// Allocators
  SortCode(const SmaNode * node);
  virtual ~SortCode(void);
// Evaluation
  virtual void eval(Context * context) const;
};


//--------------------------------------------------------------------------//
// class MinPosCode                                                         //
//--------------------------------------------------------------------------//

class MinPosCode : public SmaCode
{
public:
// Allocators
  MinPosCode(const SmaNode * node);
  virtual ~MinPosCode(void);
// Evaluation
  virtual void eval(Context * context) const;
};


//--------------------------------------------------------------------------//
// class MaxPosCode                                                         //
//--------------------------------------------------------------------------//

class MaxPosCode : public SmaCode
{
public:
// Allocators
  MaxPosCode(const SmaNode * node);
  virtual ~MaxPosCode(void);
// Evaluation
  virtual void eval(Context * context) const;
};


//--------------------------------------------------------------------------//
// class Insert_same_I_Code                                                 //
//--------------------------------------------------------------------------//

class Insert_same_I_Code : public SmaCode
{
public:
// Allocators
  Insert_same_I_Code(const SmaNode * node);
  virtual ~Insert_same_I_Code(void);
// Evaluation
  virtual void eval(Context * context) const;
};


//--------------------------------------------------------------------------//
// class Insert_same_R_Code                                                 //
//--------------------------------------------------------------------------//

class Insert_same_R_Code : public SmaCode
{
public:
// Allocators
  Insert_same_R_Code(const SmaNode * node);
  virtual ~Insert_same_R_Code(void);
// Evaluation
  virtual void eval(Context * context) const;
};


//--------------------------------------------------------------------------//
// class Insert_diff_I_Code                                                 //
//--------------------------------------------------------------------------//

class Insert_diff_I_Code : public SmaCode
{
public:
// Allocators
  Insert_diff_I_Code(const SmaNode * node,const ValueType & listElemType);
  virtual ~Insert_diff_I_Code(void);
// Evaluation
  virtual void eval(Context * context) const;
protected:
// Attributes
  ValueType _listElemType;
};


//--------------------------------------------------------------------------//
// class Insert_diff_R_Code                                                 //
//--------------------------------------------------------------------------//

class Insert_diff_R_Code : public SmaCode
{
public:
// Allocators
  Insert_diff_R_Code(const SmaNode * node,const ValueType & listElemType);
  virtual ~Insert_diff_R_Code(void);
// Evaluation
  virtual void eval(Context * context) const;
protected:
// Attributes
  ValueType _listElemType;
};


//--------------------------------------------------------------------------//
// class PushBack_same_Code                                                 //
//--------------------------------------------------------------------------//

class PushBack_same_Code : public SmaCode
{
public:
// Allocators
  PushBack_same_Code(const SmaNode * node);
  virtual ~PushBack_same_Code(void);
// Evaluation
  virtual void eval(Context * context) const;
};


//--------------------------------------------------------------------------//
// class PushBack_diff_Code                                                 //
//--------------------------------------------------------------------------//

class PushBack_diff_Code : public SmaCode
{
public:
// Allocators
  PushBack_diff_Code(const SmaNode * node,const ValueType & listElemType);
  virtual ~PushBack_diff_Code(void);
// Evaluation
  virtual void eval(Context * context) const;
protected:
// Attributes
  ValueType _listElemType;
};


//--------------------------------------------------------------------------//
// class PushFront_same_Code                                                //
//--------------------------------------------------------------------------//

class PushFront_same_Code : public SmaCode
{
public:
// Allocators
  PushFront_same_Code(const SmaNode * node);
  virtual ~PushFront_same_Code(void);
// Evaluation
  virtual void eval(Context * context) const;
};


//--------------------------------------------------------------------------//
// class PushFront_diff_Code                                                 //
//--------------------------------------------------------------------------//

class PushFront_diff_Code : public SmaCode
{
public:
// Allocators
  PushFront_diff_Code(const SmaNode * node,const ValueType & listElemType);
  virtual ~PushFront_diff_Code(void);
// Evaluation
  virtual void eval(Context * context) const;
protected:
// Attributes
  ValueType _listElemType;
};


//--------------------------------------------------------------------------//
// class Remove_I_Code                                                      //
//--------------------------------------------------------------------------//

class Remove_I_Code : public SmaCode
{
public:
// Allocators
  Remove_I_Code(const SmaNode * node);
  virtual ~Remove_I_Code(void);
// Evaluation
  virtual void eval(Context * context) const;
};


//--------------------------------------------------------------------------//
// class Remove_R_Code                                                      //
//--------------------------------------------------------------------------//

class Remove_R_Code : public SmaCode
{
public:
// Allocators
  Remove_R_Code(const SmaNode * node);
  virtual ~Remove_R_Code(void);
// Evaluation
  virtual void eval(Context * context) const;
};


//--------------------------------------------------------------------------//
// class RemoveVoid_I_Code                                                  //
//--------------------------------------------------------------------------//

class RemoveVoid_I_Code : public SmaCode
{
public:
// Allocators
  RemoveVoid_I_Code(const SmaNode * node);
  virtual ~RemoveVoid_I_Code(void);
// Evaluation
  virtual void eval(Context * context) const;
};


//--------------------------------------------------------------------------//
// class RemoveVoid_R_Code                                                  //
//--------------------------------------------------------------------------//

class RemoveVoid_R_Code : public SmaCode
{
public:
// Allocators
  RemoveVoid_R_Code(const SmaNode * node);
  virtual ~RemoveVoid_R_Code(void);
// Evaluation
  virtual void eval(Context * context) const;
};


//--------------------------------------------------------------------------//
// class PopBackCode                                                        //
//--------------------------------------------------------------------------//

class PopBackCode : public SmaCode
{
public:
// Allocators
  PopBackCode(const SmaNode * node);
  virtual ~PopBackCode(void);
// Evaluation
  virtual void eval(Context * context) const;
};


//--------------------------------------------------------------------------//
// class PopBackVoidCode                                                    //
//--------------------------------------------------------------------------//

class PopBackVoidCode : public SmaCode
{
public:
// Allocators
  PopBackVoidCode(const SmaNode * node);
  virtual ~PopBackVoidCode(void);
// Evaluation
  virtual void eval(Context * context) const;
};


//--------------------------------------------------------------------------//
// class PopFrontCode                                                       //
//--------------------------------------------------------------------------//

class PopFrontCode : public SmaCode
{
public:
// Allocators
  PopFrontCode(const SmaNode * node);
  virtual ~PopFrontCode(void);
// Evaluation
  virtual void eval(Context * context) const;
};


//--------------------------------------------------------------------------//
// class PopFrontVoidCode                                                   //
//--------------------------------------------------------------------------//

class PopFrontVoidCode : public SmaCode
{
public:
// Allocators
  PopFrontVoidCode(const SmaNode * node);
  virtual ~PopFrontVoidCode(void);
// Evaluation
  virtual void eval(Context * context) const;
};


//--------------------------------------------------------------------------//
// class Length_V_Code                                                      //
//--------------------------------------------------------------------------//

class Length_V_Code : public SmaCode
{
public:
// Allocators
  Length_V_Code(const SmaNode * node);
  virtual ~Length_V_Code(void);
// Evaluation
  virtual void eval(Context * context) const;
};


//--------------------------------------------------------------------------//
// class Length_S_Code                                                      //
//--------------------------------------------------------------------------//

class Length_S_Code : public SmaCode
{
public:
// Allocators
  Length_S_Code(const SmaNode * node);
  virtual ~Length_S_Code(void);
// Evaluation
  virtual void eval(Context * context) const;
};


//--------------------------------------------------------------------------//
// class Seek_same_V_Code                                                   //
//--------------------------------------------------------------------------//

class Seek_same_V_Code : public SmaCode
{
public:
// Allocators
  Seek_same_V_Code(const SmaNode * node,bool deep);
  virtual ~Seek_same_V_Code(void);
// Evaluation
  virtual void eval(Context * context) const;
protected:
// Attributes
  bool _deep;
};


//--------------------------------------------------------------------------//
// class Seek_diff_V_Code                                                   //
//--------------------------------------------------------------------------//

class Seek_diff_V_Code : public SmaCode
{
public:
// Allocators
  Seek_diff_V_Code(const SmaNode * node,const ValueType & listElemType);
  virtual ~Seek_diff_V_Code(void);
// Evaluation
  virtual void eval(Context * context) const;
protected:
// Attribute
  ValueType _listElemType;
};


//--------------------------------------------------------------------------//
// class Seek_S_Code                                                        //
//--------------------------------------------------------------------------//

class Seek_S_Code : public SmaCode
{
public:
// Allocators
  Seek_S_Code(const SmaNode * node);
  virtual ~Seek_S_Code(void);
// Evaluation
  virtual void eval(Context * context) const;
};


#endif // LISTCD_H

