// @(#) ENIB/LI2 (c) - oRis - Version 2.3
//--------------------------------------------------------------------------//
// file        : moduleCd.h                                                 //
// description : header file for the ENIB/LI2 class ...ModuleCode           //
// author(s)   : F. Harrouet                                                //
// copyright   : Laboratoire d'Informatique Industrielle (LI2)              //
//               Ecole Nationale d'Ingenieurs de Brest (ENIB)               //
//               CP 30815 - 29608 Brest Cedex - FRANCE                      //
//               Tel: (33) 02.98.05.66.31 , Fax: (33) 02.98.05.66.29        //
//               e-mail: li2@enib.fr                                        //
//--------------------------------------------------------------------------//
// Copyright (c) 1996-2000                                                  //
// Laboratoire d'Informatique Industrielle (LI2)                            //
// Ecole Nationale d'Ingenieurs de Brest (ENIB)                             //
//                                                                          //
// Permission to use, copy, modify, distribute and sell this software       //
// and its documentation for any purpose is hereby granted without fee,     //
// provided that the above copyright notice appear in all copies and        //
// that both that copyright notice and this permission notice appear        //
// in supporting documentation.                                             //
// ENIB/LI2 makes no representations about the suitability of this          //
// software for any purpose.                                                //
// It is provided "as is" without express or implied warranty.              //
//--------------------------------------------------------------------------//

#ifndef MODULECD_H
#define MODULECD_H

#include "oRis/Codes/smaCd.h"
#include "oRis/Main/value.h"
#include "oRis/Main/dLibInfo.h"

class ExecModule;

//--------------------------------------------------------------------------//
// class GlobalModuleCode                                                   //
//--------------------------------------------------------------------------//

class GlobalModuleCode : public SmaCode
{
public:
// Allocators
  GlobalModuleCode(const SmaNode * node,const Word & moduleName);
  virtual ~GlobalModuleCode(void);
// Evaluation
  virtual void eval(Context * context) const;
// Attributes
protected:
  Word _moduleName;
};


//--------------------------------------------------------------------------//
// class StartCode                                                          //
//--------------------------------------------------------------------------//

class StartCode : public SmaCode
{
public:
// Allocators
  StartCode(const SmaNode * node,ExecModule * module,bool returnID);
  virtual ~StartCode(void);
// Evaluation
  virtual void eval(Context * context) const;
// Attributes
protected:
  ExecModule * _module;
  bool _returnID;
};


//--------------------------------------------------------------------------//
// class ModelModuleCode                                                    //
//--------------------------------------------------------------------------//

class ModelModuleCode : public SmaCode
{
public:
// Allocators
  ModelModuleCode(const SmaNode * node,const Word & moduleName,
                  Model * model);
  virtual ~ModelModuleCode(void);
// Evaluation
  virtual void eval(Context * context) const;
// Attributes
protected:
  Word _moduleName;
  Model * _model;
};


//--------------------------------------------------------------------------//
// class AgentModuleCode                                                    //
//--------------------------------------------------------------------------//

class AgentModuleCode : public SmaCode
{
public:
// Allocators
  AgentModuleCode(const SmaNode * node,const Word & moduleName);
  virtual ~AgentModuleCode(void);
// Evaluation
  virtual void eval(Context * context) const;
// Attributes
protected:
  Word _moduleName;
};


//--------------------------------------------------------------------------//
// class ConstrModuleCode                                                   //
//--------------------------------------------------------------------------//

class ConstrModuleCode : public SmaCode
{
public:
// Allocators
  ConstrModuleCode(const SmaNode * node,Model * model);
  virtual ~ConstrModuleCode(void);
// Evaluation
  virtual void eval(Context * context) const;
// Attributes
protected:
  Model * _model;
};


//--------------------------------------------------------------------------//
// class DestrModuleCode                                                    //
//--------------------------------------------------------------------------//

class DestrModuleCode : public SmaCode
{
public:
// Allocators
  DestrModuleCode(const SmaNode * node,Model * model);
  virtual ~DestrModuleCode(void);
// Evaluation
  virtual void eval(Context * context) const;
// Attributes
protected:
  Model * _model;
};


//--------------------------------------------------------------------------//
// class FarModelModuleCode                                                 //
//--------------------------------------------------------------------------//

class FarModelModuleCode : public SmaCode
{
public:
// Allocators
  FarModelModuleCode(const SmaNode * node,const Word & moduleName,
                     Model * model);
  virtual ~FarModelModuleCode(void);
// Evaluation
  virtual void eval(Context * context) const;
// Attributes
protected:
  Word _moduleName;
  Model * _model;
};


//--------------------------------------------------------------------------//
// class FarAgentModuleCode                                                 //
//--------------------------------------------------------------------------//

class FarAgentModuleCode : public SmaCode
{
public:
// Allocators
  FarAgentModuleCode(const SmaNode * node,const Word & moduleName);
  virtual ~FarAgentModuleCode(void);
// Evaluation
  virtual void eval(Context * context) const;
// Attributes
protected:
  Word _moduleName;
};


//--------------------------------------------------------------------------//
// class NewCode                                                            //
//--------------------------------------------------------------------------//

class NewCode : public SmaCode
{
public:
// Allocators
  NewCode(const SmaNode * node,Model * model,
          bool isTransient,bool suffixGiven);
  virtual ~NewCode(void);
// Evaluation
  virtual void eval(Context * context) const;
protected:
// Attributes
  Model * _model;
  bool _isTransient;
  bool _suffixGiven;
};


//--------------------------------------------------------------------------//
// class DeleteCode                                                         //
//--------------------------------------------------------------------------//

class DeleteCode : public SmaCode
{
public:
// Allocators
  DeleteCode(const SmaNode * node);
  virtual ~DeleteCode(void);
// Evaluation
  virtual void eval(Context * context) const;
};


//--------------------------------------------------------------------------//
// class NativeCode                                                         //
//--------------------------------------------------------------------------//

class NativeCode : public SmaCode
{
public:
// Allocators
  NativeCode(const SmaNode * node,ExternFunction function);
  virtual ~NativeCode(void);
// Evaluation
  virtual void eval(Context * context) const;
protected:
// Attributes
  ExternFunction _function;
};


//--------------------------------------------------------------------------//
// class ProtoCode                                                          //
//--------------------------------------------------------------------------//

class ProtoCode : public SmaCode
{
public:
// Allocators
  ProtoCode(const SmaNode * node);
  virtual ~ProtoCode(void);
// Evaluation
  virtual void eval(Context * context) const;
};

#endif // MODULECD_H

