// @(#) ENIB/LI2 (c) - oRis - Version 2.3
//--------------------------------------------------------------------------//
// file        : utilCd.h                                                   //
// description : header file for the ENIB/LI2 Code utilities                //
// author(s)   : F. Harrouet                                                //
// copyright   : Laboratoire d'Informatique Industrielle (LI2)              //
//               Ecole Nationale d'Ingenieurs de Brest (ENIB)               //
//               CP 30815 - 29608 Brest Cedex - FRANCE                      //
//               Tel: (33) 02.98.05.66.31 , Fax: (33) 02.98.05.66.29        //
//               e-mail: li2@enib.fr                                        //
//--------------------------------------------------------------------------//
// Copyright (c) 1996-2000                                                  //
// Laboratoire d'Informatique Industrielle (LI2)                            //
// Ecole Nationale d'Ingenieurs de Brest (ENIB)                             //
//                                                                          //
// Permission to use, copy, modify, distribute and sell this software       //
// and its documentation for any purpose is hereby granted without fee,     //
// provided that the above copyright notice appear in all copies and        //
// that both that copyright notice and this permission notice appear        //
// in supporting documentation.                                             //
// ENIB/LI2 makes no representations about the suitability of this          //
// software for any purpose.                                                //
// It is provided "as is" without express or implied warranty.              //
//--------------------------------------------------------------------------//

#ifndef UTILCD_H
#define UTILCD_H

#include "oRis/Codes/smaCd.h"
#include "oRis/Main/value.h"

//--------------------------------------------------------------------------//
// class PopCode                                                            //
//--------------------------------------------------------------------------//

class PopCode : public SmaCode
{
public:
// Allocators
  PopCode(const SmaNode * node);
  virtual ~PopCode(void);
// Evaluation
  virtual void eval(Context * context) const;
};


//--------------------------------------------------------------------------//
// class PushCode                                                           //
//--------------------------------------------------------------------------//

class PushCode : public SmaCode
{
public:
// Allocators
  PushCode(const SmaNode * node,const ValueType & valueType);
  virtual ~PushCode(void);
// Evaluation
  virtual void eval(Context * context) const;
protected:
// Attributes
  ValueType _valueType;
};


//--------------------------------------------------------------------------//
// class JumpCode                                                           //
//--------------------------------------------------------------------------//

class JumpCode : public SmaCode
{
public:
// Allocators
  JumpCode(const SmaNode * node,int offset);
  virtual ~JumpCode(void);
// Inspectors
  virtual int jumpsTo(void) const;
// Evaluation
  virtual void eval(Context * context) const;
protected:
// Attributes
  int _offset;
};


//--------------------------------------------------------------------------//
// class JumpIfTrueCode                                                     //
//--------------------------------------------------------------------------//

class JumpIfTrueCode : public SmaCode
{
public:
// Allocators
  JumpIfTrueCode(const SmaNode * node,int offset);
  virtual ~JumpIfTrueCode(void);
// Evaluation
  virtual void eval(Context * context) const;
protected:
// Attributes
  int _offset;
};


//--------------------------------------------------------------------------//
// class JumpIfTrueNoPopCode                                                //
//--------------------------------------------------------------------------//

class JumpIfTrueNoPopCode : public SmaCode
{
public:
// Allocators
  JumpIfTrueNoPopCode(const SmaNode * node,int offset);
  virtual ~JumpIfTrueNoPopCode(void);
// Evaluation
  virtual void eval(Context * context) const;
protected:
// Attributes
  int _offset;
};


//--------------------------------------------------------------------------//
// class JumpIfFalseCode                                                    //
//--------------------------------------------------------------------------//

class JumpIfFalseCode : public SmaCode
{
public:
// Allocators
  JumpIfFalseCode(const SmaNode * node,int offset);
  virtual ~JumpIfFalseCode(void);
// Evaluation
  virtual void eval(Context * context) const;
protected:
// Attributes
  int _offset;
};


//--------------------------------------------------------------------------//
// class JumpIfFalseNoPopCode                                               //
//--------------------------------------------------------------------------//

class JumpIfFalseNoPopCode : public SmaCode
{
public:
// Allocators
  JumpIfFalseNoPopCode(const SmaNode * node,int offset);
  virtual ~JumpIfFalseNoPopCode(void);
// Evaluation
  virtual void eval(Context * context) const;
protected:
// Attributes
  int _offset;
};


//--------------------------------------------------------------------------//
// class NotCode                                                            //
//--------------------------------------------------------------------------//

class NotCode : public SmaCode
{
public:
// Allocators
  NotCode(const SmaNode * node);
  virtual ~NotCode(void);
// Evaluation
  virtual void eval(Context * context) const;
};


//--------------------------------------------------------------------------//
// class ReturnCode                                                         //
//--------------------------------------------------------------------------//

class ReturnCode : public SmaCode
{
public:
// Allocators
  ReturnCode(const SmaNode * node,int noCBLevel,int criticalLevel);
  virtual ~ReturnCode(void);
// Inspectors
  virtual bool isReturn(void) const;
// Evaluation
  virtual void eval(Context * context) const;
protected:
// Attributes
  int _noCBLevel;
  int _criticalLevel;
};


//--------------------------------------------------------------------------//
// class BeginCriticalCode                                                  //
//--------------------------------------------------------------------------//

class BeginCriticalCode : public SmaCode
{
public:
// Allocators
  BeginCriticalCode(const SmaNode * node);
  virtual ~BeginCriticalCode(void);
// Evaluation
  virtual void eval(Context * context) const;
};


//--------------------------------------------------------------------------//
// class EndCriticalCode                                                    //
//--------------------------------------------------------------------------//

class EndCriticalCode : public SmaCode
{
public:
// Allocators
  EndCriticalCode(const SmaNode * node);
  virtual ~EndCriticalCode(void);
// Evaluation
  virtual void eval(Context * context) const;
};


//--------------------------------------------------------------------------//
// class BeginNoCBCode                                                      //
//--------------------------------------------------------------------------//

class BeginNoCBCode : public SmaCode
{
public:
// Allocators
  BeginNoCBCode(const SmaNode * node);
  virtual ~BeginNoCBCode(void);
// Evaluation
  virtual void eval(Context * context) const;
};


//--------------------------------------------------------------------------//
// class EndNoCBCode                                                        //
//--------------------------------------------------------------------------//

class EndNoCBCode : public SmaCode
{
public:
// Allocators
  EndNoCBCode(const SmaNode * node);
  virtual ~EndNoCBCode(void);
// Evaluation
  virtual void eval(Context * context) const;
};


//--------------------------------------------------------------------------//
// class CopyCode                                                           //
//--------------------------------------------------------------------------//

class CopyCode : public SmaCode
{
public:
// Allocators
  CopyCode(const SmaNode * node);
  virtual ~CopyCode(void);
// Evaluation
  virtual void eval(Context * context) const;
};


//--------------------------------------------------------------------------//
// class CastCode                                                           //
//--------------------------------------------------------------------------//

class CastCode : public SmaCode
{
public:
// Allocators
  CastCode(const SmaNode * node,const ValueType & resultType);
  virtual ~CastCode(void);
// Evaluation
  virtual void eval(Context * context) const;
protected:
// Attributes
  ValueType _baseType;
  ValueType _resultType;
};


//--------------------------------------------------------------------------//
// class FormatCode                                                         //
//--------------------------------------------------------------------------//

class FormatCode : public SmaCode
{
public:
// Allocators
  FormatCode(const SmaNode * node,int nbArgs);
  virtual ~FormatCode(void);
// Evaluation
  virtual void eval(Context * context) const;
protected:
// Attributes
  int _nbArgs;
};


//--------------------------------------------------------------------------//
// class PrintCode                                                          //
//--------------------------------------------------------------------------//

class PrintCode : public SmaCode
{
public:
// Allocators
  PrintCode(const SmaNode * node,int nbArgs,bool newLine);
  virtual ~PrintCode(void);
// Evaluation
  virtual void eval(Context * context) const;
protected:
// Attributes
  int _nbArgs;
  bool _newLine;
};


//--------------------------------------------------------------------------//
// class ErrorCode                                                          //
//--------------------------------------------------------------------------//

class ErrorCode : public SmaCode
{
public:
// Allocators
  ErrorCode(const SmaNode * node,int nbArgs,bool warning);
  virtual ~ErrorCode(void);
// Evaluation
  virtual void eval(Context * context) const;
protected:
// Attributes
  int _nbArgs;
  bool _warning;
};


#endif // UTILCD_H

