// @(#) ENIB/LI2 (c) - oRis - Version 2.3
//--------------------------------------------------------------------------//
// file        : agent.h                                                    //
// description : header file for the ENIB/LI2 class Agent                   //
// author(s)   : F. Harrouet                                                //
// copyright   : Laboratoire d'Informatique Industrielle (LI2)              //
//               Ecole Nationale d'Ingenieurs de Brest (ENIB)               //
//               CP 30815 - 29608 Brest Cedex - FRANCE                      //
//               Tel: (33) 02.98.05.66.31 , Fax: (33) 02.98.05.66.29        //
//               e-mail: li2@enib.fr                                        //
//--------------------------------------------------------------------------//
// Copyright (c) 1996-2000                                                  //
// Laboratoire d'Informatique Industrielle (LI2)                            //
// Ecole Nationale d'Ingenieurs de Brest (ENIB)                             //
//                                                                          //
// Permission to use, copy, modify, distribute and sell this software       //
// and its documentation for any purpose is hereby granted without fee,     //
// provided that the above copyright notice appear in all copies and        //
// that both that copyright notice and this permission notice appear        //
// in supporting documentation.                                             //
// ENIB/LI2 makes no representations about the suitability of this          //
// software for any purpose.                                                //
// It is provided "as is" without express or implied warranty.              //
//------------------------------------------------------------------------- //

#ifndef AGENT_H
#define AGENT_H

#include "Utility/stlMap.h"
#include "Utility/stlList.h"
#include "Utility/smplVect.h"

#include "oRis/Main/agtName.h"
#include "oRis/Main/execMod.h"

//---------------------------------------------------------------------------//
// class Agent                                                               //
//---------------------------------------------------------------------------//

class Agent;

typedef StlMap<AgentName,Agent *> AgentMap;
typedef StlList<Agent *> AgentList;
typedef SimpleVector<Agent *> AgentVect;

class Agent : public BlockAllocatedObject
{
public:
// Allocators
  Agent(Model * model,long instNum);
  virtual ~Agent(void);
// Inspectors
  inline const Model * getModel(void) const;
  inline const AgentName & getName(void) const;
  inline bool isTransient(void) const;
  inline bool isAlive(void) const;
  inline bool isSuspended(void) const;
  inline int getNbAttributes(void) const;
  inline const AttrMap & getAttrMap(void) const;
  inline const Attribute * getAttribute(const Word & name) const;
  Error * tryToSetAttribute(const Attribute & attr) const;
  inline int getNbModules(void) const;
  inline const ExecModMap & getModuleMap(void) const;
  inline const ExecModule * getMainModule(void) const;
  inline const ExecModule * getNewModule(void) const;
  inline const ExecModule * getDeleteModule(void) const;
  const ExecModule * getModule(const Word & name) const;
  Error * tryToSetModule(const ExecModule * newModule) const;
// Comparisons
  inline bool operator==(const Agent & agent) const;
  inline bool operator!=(const Agent & agent) const;
  inline bool operator<(const Agent & agent) const;
  inline bool operator>(const Agent & agent) const;
  inline bool operator<=(const Agent & agent) const;
  inline bool operator>=(const Agent & agent) const;
// Input/output
  friend ostream& operator<<(ostream& stream, const Agent& agent);
  Strings print(int column=0) const;
  Strings printInstanciation(int column=0) const;
  Strings printDeclarations(int column=0) const;
  Strings printDefinitions(int column=0) const;
  Strings printInitialisations(int column=0) const;
  virtual Strings userSave(int column=0) const;
// Modifiers
  inline Model * accessModel(void);
  inline void setName(const AgentName & name);
  inline void reference(void);
  void unReference(void);
  virtual void clearReferences(bool strongClear=false);
  inline void setTransient(bool isTransient);
  void kill(void);
  inline void newBirth(void);
  void suspend(void);
  void resume(void);
  void setAttribute(const Attribute & attr,bool notify);
  inline AttrMap & accessAttrMap(void);
  inline Attribute * accessAttribute(const Word & name);
  void setModule(ExecModule * newModule,bool notify);
  inline ExecModMap & accessModuleMap(void);
  inline ExecModule * accessMainModule(void);
  inline ExecModule * accessNewModule(void);
  inline ExecModule * accessDeleteModule(void);
  ExecModule * accessModule(const Word & name);
// Native code
  
  // native string Object::name(void);
  //
  static void stName(Context * context);

  // native bool Object::isA(string className);
  //
  static void stIsA(Context * context);

  // native string Object::className(void);
  //
  static void stClassName(Context * context);

  // native int Object::suffix(void);
  //
  static void stSuffix(Context * context);

  // native void Object::suspend(void);
  //
  static void stSuspend(Context * context);

  // native void Object::resume(void);
  //
  static void stResume(Context * context);

  // native bool Object::isSuspended(void);
  //
  static void stIsSuspended(Context * context);

  // native void Object::setTransient(bool isTransient);
  //
  static void stSetTransient(Context * context);

  // native bool Object::isTransient(void);
  //
  static void stIsTransient(Context * context);

// Communication
  // native void Agent::getSensitivity(string[] & msgTypes,
  //                                   string[] & callbacks);
  //
  static void stGetSensitivity(Context * context);

  // native void Agent::setSensitivity(string msgType,string callback);
  //
  static void stSetSensitivity(Context * context);

  // native void Agent::cancelSensitivity(string msgType);
  //
  static void stCancelSensitivity(Context * context);

  // native int Agent::getNbMessages(void);
  //
  static void stGetNbMessages(Context * context);

  // native Message Agent::peekMessage(int pos);
  //
  static void stPeekMessage(Context * context);

  // native Message Agent::getNextMessage(void);
  //
  static void stGetNextMessage(Context * context);

  // native void Agent::putBackMessage(Message message);
  //
  static void stPutBackMessage(Context * context);

  // native void Agent::asynchronousReception(Message message);
  //
  static void stAsynchronousReception(Context * context);

  // native void Message::sendTo(Agent receiver);
  //
  static void stMsgSendTo(Context * context);

  // native void Message::broadcast(void);
  //
  static void stMsgBroadcast(Context * context);

protected:
  void _notifyChange(const Word & method,const Word & param);
// Attributes
  Model * _model;
  AgentName _name;
  AttrMap _attrMap;
  ExecModMap _moduleMap;
  ExecModule * _mainModule;
  ExecModule * _newModule;
  ExecModule * _deleteModule;
  bool _isTransient;
  bool _isAlive;
  int _nbRefs;
  bool _isSuspended;
  AgentList _msgList;
  static Word _notifyAttributeWord;
  static Word _notifyModuleWord;
  static Word _messageWord;
  static Word _onMessageWord;
  static Word _objectDescriptionWord;
  static Word _objectWord;
};

#include "oRis/Main/agent.Ci"

#endif // AGENT_H

