// @(#) ENIB/LI2 (c) - oRis - Version 2.3
//--------------------------------------------------------------------------//
// file        : attrVal.h                                                  //
// description : header file for the ENIB/LI2 class AttrValue               //
// author(s)   : F. Harrouet                                                //
// copyright   : Laboratoire d'Informatique Industrielle (LI2)              //
//               Ecole Nationale d'Ingenieurs de Brest (ENIB)               //
//               CP 30815 - 29608 Brest Cedex - FRANCE                      //
//               Tel: (33) 02.98.05.66.31 , Fax: (33) 02.98.05.66.29        //
//               e-mail: li2@enib.fr                                        //
//--------------------------------------------------------------------------//
// Copyright (c) 1996-2000                                                  //
// Laboratoire d'Informatique Industrielle (LI2)                            //
// Ecole Nationale d'Ingenieurs de Brest (ENIB)                             //
//                                                                          //
// Permission to use, copy, modify, distribute and sell this software       //
// and its documentation for any purpose is hereby granted without fee,     //
// provided that the above copyright notice appear in all copies and        //
// that both that copyright notice and this permission notice appear        //
// in supporting documentation.                                             //
// ENIB/LI2 makes no representations about the suitability of this          //
// software for any purpose.                                                //
// It is provided "as is" without express or implied warranty.              //
//--------------------------------------------------------------------------//

#ifndef ATTRVAL_H
#define ATTRVAL_H

#include "oRis/Main/value.h"

//---------------------------------------------------------------------------//
// class ValueCallback                                                       //
//---------------------------------------------------------------------------//

class AttrValue;

class ValueCallback : public BlockAllocatedObject
{
public:
// Static methods
  inline static void mask(void);
  inline static void unmask(void);
  inline static int getMask(void);
// Allocators
  inline ValueCallback(void);
  inline ValueCallback(const ValueCallback & callback);
  inline virtual ~ValueCallback(void);
// Inspectors
// Comparisons
// Input/output
// Modifiers
  inline void connect(AttrValue * value);
  inline void disconnect(void);
  virtual void before(void)=0;
  virtual void after(void)=0;
protected:
// Attributes
  AttrValue * _value;
  static int _mask;
};

typedef SimpleVector<ValueCallback *> CallbackVect;

//---------------------------------------------------------------------------//
// class AttrValue                                                           //
//---------------------------------------------------------------------------//

class AttrValue : public Value
{
public:
  friend class ValueCallback;
// Allocators
  inline AttrValue(const ValueType & type);
  inline virtual ~AttrValue(void);
// Inspectors
  inline virtual bool isLeftValue(void) const;
  inline bool isSeen(void) const;
// Comparisons
// Input/output
// Modifiers
  inline virtual void reference(void);
  virtual void relax(void);
  virtual void clearReferences(bool strongClear=false);
  void seeMore(void);
  void seeLess(void);
  virtual bool preset(void);
  virtual void postset(void);
  void disconnect(void);
  inline virtual void setInteger(long integer);
  inline virtual void setReal(double real);
  inline virtual void setStrings(const Strings & string);
  inline virtual void setAgent(Agent * agent);
  inline virtual void setValueVectorCopy(const Value & valueVector);
  inline virtual void setValueVectorCopyI2R(const SubValueVect & valueVector);
  inline virtual void setValueVectorCopyR2I(const SubValueVect & valueVector);
  inline virtual void setValueVector(SubValueVect * valueVector);
  inline virtual void insert(SubValue * value,int pos);
  inline virtual void insert(const Value & value,int pos);
  inline virtual void pushFront(SubValue * value);
  inline virtual void pushFront(const Value & value);
  inline virtual void pushBack(SubValue * value);
  inline virtual void pushBack(const Value & value);
  inline virtual void erase(int pos);
  inline virtual void readErase(Value & result,int pos);
  inline virtual void popFront(void);
  inline virtual void readPopFront(Value & result);
  inline virtual void popBack(void);
  inline virtual void readPopBack(Value & result);
  inline virtual void appendVector(const SubValueVect & valueVector);
  inline virtual void sortVector(void);
  inline virtual Value & operator=(const Value & value);
  inline virtual Value & affectCast(const Value & value);
protected:
// Attributes
  int _nbRefs;
  CallbackVect _callbackVect;
  int _seen;
};

#include "oRis/Main/attrVal.Ci"

#endif // ATTRVAL_H

