// @(#) ENIB/LI2 (c) - oRis - Version 2.3
//--------------------------------------------------------------------------//
// file        : attrib.h                                                   //
// description : header file for the ENIB/LI2 class  Attribute              //
// author(s)   : F. Harrouet                                                //
// copyright   : Laboratoire d'Informatique Industrielle (LI2)              //
//               Ecole Nationale d'Ingenieurs de Brest (ENIB)               //
//               CP 30815 - 29608 Brest Cedex - FRANCE                      //
//               Tel: (33) 02.98.05.66.31 , Fax: (33) 02.98.05.66.29        //
//               e-mail: li2@enib.fr                                        //
//--------------------------------------------------------------------------//
// Copyright (c) 1996-2000                                                  //
// Laboratoire d'Informatique Industrielle (LI2)                            //
// Ecole Nationale d'Ingenieurs de Brest (ENIB)                             //
//                                                                          //
// Permission to use, copy, modify, distribute and sell this software       //
// and its documentation for any purpose is hereby granted without fee,     //
// provided that the above copyright notice appear in all copies and        //
// that both that copyright notice and this permission notice appear        //
// in supporting documentation.                                             //
// ENIB/LI2 makes no representations about the suitability of this          //
// software for any purpose.                                                //
// It is provided "as is" without express or implied warranty.              //
//--------------------------------------------------------------------------//

#ifndef ATTRIB_H
#define ATTRIB_H

#include "Utility/stlMap.h"
#include "Utility/stlVect.h"
#include "Utility/word.h"

#include "oRis/Main/attrVal.h"

//--------------------------------------------------------------------------//
// class Attribute                                                          //
//--------------------------------------------------------------------------//

class Attribute : public BlockAllocatedObject
{
public:
// Allocators
  Attribute(void);
  Attribute(const Attribute & attr);
  Attribute(const Word & name,const ValueType & type,
            Model * model,Agent * owner);
  virtual ~Attribute(void);
// Inspectors
  inline const Value * getValue(void) const;
  inline const Model * getModel(void) const;
  inline const Agent * getOwner(void) const;
  inline const Word & getName(void) const;
  inline const ValueType & getType(void) const;
// Comparisons
// Input/output
  friend ostream & operator<<(ostream & stream,const Attribute & attr);
  Strings print(int column=0) const;
// Modifiers
  void clearReferences(bool strongClear=false);
  inline Model * accessModel(void);
  inline Agent * accessOwner(void);
  inline void setOwner(Agent * owner);
  inline Value * accessValue(void);
  Attribute & operator=(const Attribute & attr);
private:
// Attributes 
  ValueType _type;
  Word _name;
  AttrValue * _value;
  Model * _model;
  Agent * _owner;
};

typedef StlMap<Word,Attribute> AttrMap;
typedef StlVector<Attribute> AttrVect;

#include "oRis/Main/attrib.Ci"

#endif // ATTRIB_H

