// @(#) ENIB/LI2 (c) - oRis - Version 2.3
//--------------------------------------------------------------------------//
// file        : context.h                                                  //
// description : header file for the ENIB/LI2 class Context                 //
// author(s)   : F. Harrouet                                                //
// copyright   : Laboratoire d'Informatique Industrielle (LI2)              //
//               Ecole Nationale d'Ingenieurs de Brest (ENIB)               //
//               CP 30815 - 29608 Brest Cedex - FRANCE                      //
//               Tel: (33) 02.98.05.66.31 , Fax: (33) 02.98.05.66.29        //
//               e-mail: li2@enib.fr                                        //
//--------------------------------------------------------------------------//
// Copyright (c) 1996-2000                                                  //
// Laboratoire d'Informatique Industrielle (LI2)                            //
// Ecole Nationale d'Ingenieurs de Brest (ENIB)                             //
//                                                                          //
// Permission to use, copy, modify, distribute and sell this software       //
// and its documentation for any purpose is hereby granted without fee,     //
// provided that the above copyright notice appear in all copies and        //
// that both that copyright notice and this permission notice appear        //
// in supporting documentation.                                             //
// ENIB/LI2 makes no representations about the suitability of this          //
// software for any purpose.                                                //
// It is provided "as is" without express or implied warranty.              //
//------------------------------------------------------------------------- //

#ifndef CONTEXT_H
#define CONTEXT_H

#include "oRis/Main/value.h"
#include "oRis/Main/agent.h"
#include "oRis/Codes/smaCd.h"

//---------------------------------------------------------------------------//
// class Context                                                             //
//---------------------------------------------------------------------------//

class ExecModule;
class Execution;

class Context : public BlockAllocatedObject
{
public:
// Static methods
  static Context * getContext(ExecModule * module,Agent * owner,
                              Context * previous);
  inline static void relaxContext(Context * context);
  static void clearContextStock(void);
  inline static ValueVect * getValueVect(void);
  inline static void relaxValueVect(ValueVect * vect);
  static void clearValueVectStock(void);
// Allocators
protected:                  // use static methods
  Context(void);
  virtual ~Context(void);
public:
// Inspectors
  inline const Execution * getExecution(void) const;
  inline const Context * getPrevious(void) const;
  inline const Agent * getOwner(void) const;
  inline const ExecModule * getModule(void) const;
  inline const ValueVect * getLocals(void) const;
  inline const Value * getLocal(int rank) const;
  inline long getInteger(int rank) const;
  inline double getReal(int rank) const;
  inline const Strings & getStrings(int rank) const;
  inline Agent * getAgent(int rank) const;
  inline const SubValueVect * getValueVector(int rank) const;
  void getStack(AgentVect & callers,StringsVect & modules,int depth=-1) const;
  inline bool started(void) const;
  bool endReached(void) const;
// Comparisons
// Input/output
  Strings printUser(void) const;
  Strings printStack(void) const;
// Modifiers
  void enterExecution(Execution * execution);
  void leaveExecution(void);
  inline Execution * accessExecution(void);
  inline void eval(void);
  inline void moveCounter(int offset);
  inline void moveCounterToEnd(void);
  inline Context * accessPrevious(void);
  inline Agent * accessOwner(void);
  inline ExecModule * accessModule(void);
  inline ValueVect * accessLocals(void);
  inline Value * accessLocal(int rank);
  void returnValue(Value * value);
  void returnInteger(long integer);
  void returnReal(double real);
  void returnStrings(const Strings & str);
  void returnAgent(Agent * agent,Model * model);
  inline void setInteger(int rank,long integer);
  inline void setReal(int rank,double real);
  inline void setStrings(int rank,const Strings & string);
  inline void setAgent(int rank,Agent * agent,Model * model);
  inline void setValueVector(int rank,SubValueVect * vect,
                             const ValueType & type);
private:
// Attributes
  Execution * _execution;
  int _counter;
  ExecModule * _module;
  ValueVect * _locals;
  Agent * _owner;
  const CodeVect * _code;
  Context * _previous;
  static SimpleVector<Context *> _contextStock;
  static SimpleVector<ValueVect *> _valueVectStock;
};

#include "oRis/Main/context.Ci"

#endif // CONTEXT_H

