// @(#) ENIB/LI2 (c) - oRis - Version 2.3
//--------------------------------------------------------------------------//
// file        : errors.h                                                   //
// description : header file for the ENIB/LI2 class Error                   //
// author(s)   : F. Harrouet                                                //
// copyright   : Laboratoire d'Informatique Industrielle (LI2)              //
//               Ecole Nationale d'Ingenieurs de Brest (ENIB)               //
//               CP 30815 - 29608 Brest Cedex - FRANCE                      //
//               Tel: (33) 02.98.05.66.31 , Fax: (33) 02.98.05.66.29        //
//               e-mail: li2@enib.fr                                        //
//--------------------------------------------------------------------------//
// Copyright (c) 1996-2000                                                  //
// Laboratoire d'Informatique Industrielle (LI2)                            //
// Ecole Nationale d'Ingenieurs de Brest (ENIB)                             //
//                                                                          //
// Permission to use, copy, modify, distribute and sell this software       //
// and its documentation for any purpose is hereby granted without fee,     //
// provided that the above copyright notice appear in all copies and        //
// that both that copyright notice and this permission notice appear        //
// in supporting documentation.                                             //
// ENIB/LI2 makes no representations about the suitability of this          //
// software for any purpose.                                                //
// It is provided "as is" without express or implied warranty.              //
//------------------------------------------------------------------------- //

#ifndef ERRORS_H
#define ERRORS_H

#include "Utility/strings.h"

//--------------------------------------------------------------------------//
// class Error                                                              //
//--------------------------------------------------------------------------//

class Error : public BlockAllocatedObject
{
public:
  enum ErrorType {BUG, SYNTAX, SEMANTICS, RUNTIME, WARNING, DEVICE};
// Allocators
  Error(ErrorType type,const Strings & message);
  virtual ~Error();
// Inspectors
  Error::ErrorType getType(void) const;
  const Strings & getMessage(void) const;
  int getLineNumber(void) const;
  const Strings & getFileName(void) const;
// Input/Output
  friend ostream & operator<<(ostream & stream,const Error & error);
  Strings print(void) const;
// Modifiers
  void setLineNumber(int line);
  void setFileName(const Strings & file);
private:
// Attributes
  int _line;
  Strings _file;
  ErrorType _type;
  Strings _message;
};

#endif // ERRORS_H

