// @(#) ENIB/LI2 (c) - oRis - Version 2.3
//--------------------------------------------------------------------------//
// file        : exec.h                                                     //
// description : header file for the ENIB/LI2 class Execution               //
// author(s)   : F. Harrouet                                                //
// copyright   : Laboratoire d'Informatique Industrielle (LI2)              //
//               Ecole Nationale d'Ingenieurs de Brest (ENIB)               //
//               CP 30815 - 29608 Brest Cedex - FRANCE                      //
//               Tel: (33) 02.98.05.66.31 , Fax: (33) 02.98.05.66.29        //
//               e-mail: li2@enib.fr                                        //
//--------------------------------------------------------------------------//
// Copyright (c) 1996-2000                                                  //
// Laboratoire d'Informatique Industrielle (LI2)                            //
// Ecole Nationale d'Ingenieurs de Brest (ENIB)                             //
//                                                                          //
// Permission to use, copy, modify, distribute and sell this software       //
// and its documentation for any purpose is hereby granted without fee,     //
// provided that the above copyright notice appear in all copies and        //
// that both that copyright notice and this permission notice appear        //
// in supporting documentation.                                             //
// ENIB/LI2 makes no representations about the suitability of this          //
// software for any purpose.                                                //
// It is provided "as is" without express or implied warranty.              //
//------------------------------------------------------------------------- //

#ifndef EXEC_H
#define EXEC_H

#include "Utility/stlList.h"
#include "oRis/Main/agent.h"

//--------------------------------------------------------------------------//
// class Execution                                                          //
//--------------------------------------------------------------------------//

class Execution;
class ThreadedNative;

typedef SimpleVector<Execution *> ExecutionVect;
typedef StlList<Execution *> ExecutionList;

class Execution : public BlockAllocatedObject
{
public:
// Static Methods
  static Execution * getExecution(Agent * owner,
                             ExecModule::SpecialType type=ExecModule::NORMAL);
  static void relaxExecution(Execution * execution);
  static void clearExecutionStock(void);
// Allocators
  Execution(void);
  virtual ~Execution(void);
// Inspectors
  inline long getID(void) const;
  inline const Agent * getOwner(void) const;
  inline const Context * getContext(void) const;
  inline bool goOn(void) const;
  inline bool isFinished(void) const;
  inline bool isAborted(void) const;
  inline ExecModule::SpecialType getType(void) const;
  inline bool canRestart(void) const;
  inline bool isSuspended(void) const;
  inline int getCriticalLevel(void) const;
  inline int getStackSize(void) const;
  inline const Value * getValue(int rank) const;
  inline int getDepth(void) const;
  void getStack(AgentVect & callers,StringsVect & modules,int depth=-1) const;
  bool usesAgent(const Agent * agent) const;
  bool started(void) const;
  bool endReached(void) const;
  inline const ThreadedNative * getThreadedNative(void) const;
// Comparisons
// Input/output
  Strings print(void) const;
  Strings printStack(void) const;
// Modifiers
  inline Agent * accessOwner(void);
  inline Context * accessContext(void);
  inline void setGoOn(bool goOn);
  inline void mustDisappear(void);
  inline void avoidRestart(void);
  inline void allowRestart(void);
  inline void setSuspended(bool isSuspended);
  inline void beginCritical(void);
  void endCritical(int level=1);
  void enterContext(ExecModule * module,Agent * contextOwner);
  void leaveContext(void);
  void abort(bool canDestroy);
  inline void cancelAbort(void);
  inline void pushValue(Value * value);
  inline Value * popValue(void);
  inline Value * lastValue(void);
  inline Value * accessValue(int rank);
  inline void setThreadedNative(ThreadedNative * thNative);
  inline ThreadedNative * accessThreadedNative(void);
private:
// Attributes
  long _id;
  Agent * _owner;
  ExecModule::SpecialType _type;
  bool _canRestart;
  ValueVect _values;
  Context * _lastContext;
  int _criticalLevel;
  bool _goOn;
  bool _isFinished;
  bool _isSuspended;
  bool _isAborted;
  int _depth;
  ThreadedNative * _thNative;
  static ExecutionVect _execStock;
  static long _lastID;
};

#include "oRis/Main/exec.Ci"

#endif // EXEC_H

