// @(#) ENIB/LI2 (c) - oRis - Version 2.3
//--------------------------------------------------------------------------//
// file        : execMod.h                                                  //
// description : header file for the ENIB/LI2 class ExecModule              //
// author(s)   : F. Harrouet                                                //
// copyright   : Laboratoire d'Informatique Industrielle (LI2)              //
//               Ecole Nationale d'Ingenieurs de Brest (ENIB)               //
//               CP 30815 - 29608 Brest Cedex - FRANCE                      //
//               Tel: (33) 02.98.05.66.31 , Fax: (33) 02.98.05.66.29        //
//               e-mail: li2@enib.fr                                        //
//--------------------------------------------------------------------------//
// Copyright (c) 1996-2000                                                  //
// Laboratoire d'Informatique Industrielle (LI2)                            //
// Ecole Nationale d'Ingenieurs de Brest (ENIB)                             //
//                                                                          //
// Permission to use, copy, modify, distribute and sell this software       //
// and its documentation for any purpose is hereby granted without fee,     //
// provided that the above copyright notice appear in all copies and        //
// that both that copyright notice and this permission notice appear        //
// in supporting documentation.                                             //
// ENIB/LI2 makes no representations about the suitability of this          //
// software for any purpose.                                                //
// It is provided "as is" without express or implied warranty.              //
//------------------------------------------------------------------------- //

#ifndef EXECMOD_H
#define EXECMOD_H

#include "Utility/stlMap.h"

#include "oRis/Main/attrib.h"
#include "oRis/Main/param.h"
#include "oRis/Main/localVal.h"
#include "oRis/Codes/smaCd.h"

//---------------------------------------------------------------------------//
// class ExecModule                                                          //
//---------------------------------------------------------------------------//

class ProtoModule;
class Execution;

class ExecModule : public BlockAllocatedObject
{
public:
  enum ModuleType {PROCEDURE, PROTOTYPE, EXTERN};
  enum SpecialType {EXECUTE, START, NORMAL, BEHAVIOR, CONSTRUCTOR, DESTRUCTOR};
// Allocators
  ExecModule(const Word & name,
             SpecialType specialType,
             const ValueType & resultType,
             Model * model,Agent * owner);
  ExecModule(const ExecModule & module);
  virtual ExecModule * duplicate(void) const;
  virtual ~ExecModule(void);
// Inspectors
  inline const Model * getModel(void) const;
  inline const Agent * getOwner(void) const;
  inline const Word & getName(void) const;
  Strings getCompleteName(void) const;
  inline const ValueType & getResultType(void) const;
  inline virtual ExecModule::ModuleType getType(void) const;
  inline ExecModule::SpecialType getSpecialType(void) const;
  inline bool isCritical(void) const;
  inline int getNbParams(void) const;
  inline const Parameter * getParam(int rank) const;
  inline int getNbLocals(void) const;
  int getLocalRank(const Word & name) const;
  inline const Attribute * getLocal(int rank) const;
  inline const AttrVect & getLocals(void) const;
  bool isCompatible(const ExecModule & module) const;
  Error * tryToCallFromNode(const SmaNode * node,int nodeOffset=0) const;
  Error * tryToCall(const Execution * exec,int nbArgs) const;
  ProtoModule * buildPrototype(void) const;
  inline const CodeVect * getCodeVect(void) const;
// Comparisons
// Input/output
  friend ostream & operator<<(ostream & stream,const ExecModule & execModule);
  virtual Strings print(int column=0) const;
  Strings printBody(int column=0) const;
  Strings printLocals(int column=0) const;
  Strings printCode(int column=0) const;
  Strings printProto(int column=0) const;
  Strings printModelProto(int column=0) const;
// Modifiers
  inline Model * accessModel(void);
  inline Agent * accessOwner(void);
  inline void setOwner(Agent * owner);
  inline bool & mustDisappear(void);
  inline void setCritical(bool isCritical);
  inline void reference(void);
  inline void unReference(void);
  void setParam(const Parameter & param);
  inline void setLocal(const Attribute & attr);
  void setNode(SmaNode * node);
protected:
// Private Methods
  Strings _printParams(void) const;
// Attributes
  ExecModule::SpecialType _specialType;
  AttrVect _locals;
  Model * _model;
  Agent * _owner;
  Word _name;
  ValueType _resultType;
  int _nbRefs;
  ParamVect _params;
  bool _mustDisappear;
  bool _isCritical;
  SmaNode * _node;
  CodeVect * _codeVect;
};

typedef StlMap<Word,ExecModule *> ExecModMap;

typedef SimpleVector<ExecModule *> ExecModVect;

#include "oRis/Main/execMod.Ci"

#endif // EXECMOD_H

