// @(#) ENIB/LI2 (c) - oRis - Version 2.3
//--------------------------------------------------------------------------//
// file        : param.h                                                    //
// description : header file for the ENIB/LI2 class Parameter               //
// author(s)   : F. Harrouet                                                //
// copyright   : Laboratoire d'Informatique Industrielle (LI2)              //
//               Ecole Nationale d'Ingenieurs de Brest (ENIB)               //
//               CP 30815 - 29608 Brest Cedex - FRANCE                      //
//               Tel: (33) 02.98.05.66.31 , Fax: (33) 02.98.05.66.29        //
//               e-mail: li2@enib.fr                                        //
//--------------------------------------------------------------------------//
// Copyright (c) 1996-2000                                                  //
// Laboratoire d'Informatique Industrielle (LI2)                            //
// Ecole Nationale d'Ingenieurs de Brest (ENIB)                             //
//                                                                          //
// Permission to use, copy, modify, distribute and sell this software       //
// and its documentation for any purpose is hereby granted without fee,     //
// provided that the above copyright notice appear in all copies and        //
// that both that copyright notice and this permission notice appear        //
// in supporting documentation.                                             //
// ENIB/LI2 makes no representations about the suitability of this          //
// software for any purpose.                                                //
// It is provided "as is" without express or implied warranty.              //
//--------------------------------------------------------------------------//

#ifndef PARAM_H
#define PARAM_H

#include "Utility/stlVect.h"
#include "Utility/word.h"

#include "oRis/Main/value.h"

//---------------------------------------------------------------------------//
// class Parameter                                                           //
//---------------------------------------------------------------------------//

class Parameter : public BlockAllocatedObject
{
public:
  enum ParamMode {COPY, REFERENCE};
// Allocators
  inline Parameter(void);
  inline Parameter(ParamMode paramMode,const ValueType & type,
                   const Word & name);
  inline Parameter(const Parameter & param);
  inline virtual ~Parameter(void);
// Inspectors
  inline Parameter::ParamMode getMode(void) const;
  inline const ValueType & getType(void) const;
  inline const Word & getName(void) const;
  inline bool isCompatible(const Parameter & param) const;
// Comparisons
  inline bool operator==(const Parameter & param) const;
  inline bool operator!=(const Parameter & param) const;
// Input/output
  Strings print(void) const;
  friend ostream & operator<<(ostream & stream,const Parameter & param);
// Modifiers
private:
// Attributes
  Parameter::ParamMode _paramMode;
  ValueType _type;
  Word _name;
};

typedef StlVector<Parameter> ParamVect;

#include "oRis/Main/param.Ci"

#endif // PARAM_H

