// @(#) ENIB/LI2 (c) - oRis - Version 2.3
//--------------------------------------------------------------------------//
// file        : smaNd.h                                                    //
// description : header file for the ENIB/LI2 class SmaNode                 //
// author(s)   : F. Harrouet                                                //
// copyright   : Laboratoire d'Informatique Industrielle (LI2)              //
//               Ecole Nationale d'Ingenieurs de Brest (ENIB)               //
//               CP 30815 - 29608 Brest Cedex - FRANCE                      //
//               Tel: (33) 02.98.05.66.31 , Fax: (33) 02.98.05.66.29        //
//               e-mail: li2@enib.fr                                        //
//--------------------------------------------------------------------------//
// Copyright (c) 1996-2000                                                  //
// Laboratoire d'Informatique Industrielle (LI2)                            //
// Ecole Nationale d'Ingenieurs de Brest (ENIB)                             //
//                                                                          //
// Permission to use, copy, modify, distribute and sell this software       //
// and its documentation for any purpose is hereby granted without fee,     //
// provided that the above copyright notice appear in all copies and        //
// that both that copyright notice and this permission notice appear        //
// in supporting documentation.                                             //
// ENIB/LI2 makes no representations about the suitability of this          //
// software for any purpose.                                                //
// It is provided "as is" without express or implied warranty.              //
//--------------------------------------------------------------------------//

#ifndef SMAND_H
#define SMAND_H

#include "oRis/Main/tmpVal.h"
#include "oRis/Codes/smaCd.h"

//--------------------------------------------------------------------------//
// class SmaNode                                                            //
//--------------------------------------------------------------------------//
class SmaNode;

typedef SimpleVector<SmaNode *> NodeVect;

class SmaNode : public BlockAllocatedObject
{
public:
// Allocators
  SmaNode(int nbNodes,int lineNumber,const Strings & fileName,
          const Strings & name,bool isInstruction);
  SmaNode(const SmaNode & node);
  virtual SmaNode * duplicate(void) const=0;
  virtual ~SmaNode(void);
// Inspectors
  inline int getLineNumber(void) const;
  inline const Strings & getFileName(void) const;
  inline const Strings & getName(void) const;
  inline int getNbNodes(void) const;
  inline SmaNode * getNode(int nodeNumber) const;
  inline const ValueType & getType(void) const;
  inline bool isInstruction(void) const;
  inline virtual bool containsReturn(void) const;
  inline virtual bool buildsParent(const Model * parent) const;
  inline virtual bool destroysParent(const Model * parent) const;
  inline virtual bool isLeftValue(void) const;
  inline virtual bool isLocalValue(void) const;
  inline virtual bool isConst(void) const;
  inline bool usesParenthesis(void) const;
  inline bool isChecked(void) const;
  inline bool fail(void) const;
// Input/Output
  friend ostream & operator<<(ostream & stream,const SmaNode & node);
  virtual Strings print(int column=0) const;
// Modifiers
  inline virtual void setInstruction(bool isInstruction);
  inline void useParenthesis(bool usesParenthesis);
  virtual void setOwner(Agent * owner);
  void insideCritical(void);
  void insideNoAttrCB(void);
  void addNode(SmaNode * node);
  void setNode(int nodeNumber,SmaNode * node);
// Evaluation
  virtual bool check(const Context * context,const Model * model);
  virtual SmaNode * optimize(void);
  virtual CodeVect * generateCode(void);
  virtual CodeVect * generateVoidCode(void);
// Error management
  void manageError(Error * error) const;
protected:
  virtual Strings _print(int column=0) const;
// Attributes
  ValueType _type;
  bool _fail;
  int _criticalLevel;
  int _noCBLevel;
private:
  NodeVect _nodes;
  int _lineNumber;
  Strings _fileName;
  Strings _name;
  bool _isInstruction;
  bool _usesParenthesis;
  bool _isChecked;
};

#include "oRis/Nodes/smaNd.Ci"

#endif // SMAND_H

