// @(#) ENIB/LI2 (c) - oRis - Version 2.3
//--------------------------------------------------------------------------//
// file        : orisCtrl.h                                                 //
// description : header file for the ENIB/LI2 class OrisControl             //
// author(s)   : F. Harrouet                                                //
// copyright   : Laboratoire d'Informatique Industrielle (LI2)              //
//               Ecole Nationale d'Ingenieurs de Brest (ENIB)               //
//               CP 30815 - 29608 Brest Cedex - FRANCE                      //
//               Tel: (33) 02.98.05.66.31 , Fax: (33) 02.98.05.66.29        //
//               e-mail: li2@enib.fr                                        //
//--------------------------------------------------------------------------//
// Copyright (c) 1996-2000                                                  //
// Laboratoire d'Informatique Industrielle (LI2)                            //
// Ecole Nationale d'Ingenieurs de Brest (ENIB)                             //
//                                                                          //
// Permission to use, copy, modify, distribute and sell this software       //
// and its documentation for any purpose is hereby granted without fee,     //
// provided that the above copyright notice appear in all copies and        //
// that both that copyright notice and this permission notice appear        //
// in supporting documentation.                                             //
// ENIB/LI2 makes no representations about the suitability of this          //
// software for any purpose.                                                //
// It is provided "as is" without express or implied warranty.              //
//--------------------------------------------------------------------------//

#ifndef ORISCTRL_H
#define ORISCTRL_H

#include "guiTrans.h"

#include "Utility/strings.h"
#include "oRis/Main/textOut.h"

//--------------------------------------------------------------------------//
// class OrisControl                                                        //
//--------------------------------------------------------------------------//

class Simulator;

class OrisControl : public BlockAllocatedObject
{
public:
  enum State {BAD, RUN, ONESTEP, STOP};
// Friend functions
  friend OrisControl * getOrisControl(void);
// Allocators
  OrisControl(const StringsVect & commandLine);
  virtual ~OrisControl(void);
// Views Management
  void buildPanel(void);
  void displayClock(void);
  void displayState(void);
  void singleMessage(const Strings & message);
  bool singleQuestion(const Strings & question);
// Simulation control
  void mainTask(void);
  void registerTask(void);
  void newSimul(void);
  void load(const Strings & fileName);
  void save(const Strings & fileName);
// Callback Methods
  static Strings controlCB(Simulator * simul,void * controller,
                           const Strings & command,const Strings & arg);
  static void aboutCB(GuiElement * calling,void * data);
  static void manualCB(GuiElement * calling,void * data);
  static void quitCB(GuiElement * calling,void * data);
  static void runStopCB(GuiElement * calling,void * data);
  static void oneStepCB(GuiElement * calling,void * data);
  static void newSimulCB(GuiElement * calling,void * data);
  static void includeCB(GuiElement * calling,void * data);
  static void loadCB(GuiElement * calling,void * data);
  static void reloadCB(GuiElement * calling,void * data);
  static void saveCB(GuiElement * calling,void * data);
  static void parseCB(GuiElement * calling,void * data);
  static void clearCB(GuiElement * calling,void * data);
  static void execCB(GuiElement * calling,void * data);
  static void startCB(GuiElement * calling,void * data);
  static void editCB(GuiElement * calling,void * data);
  static void upStepCB(GuiElement * calling,void * data);
  static void downStepCB(GuiElement * calling,void * data);
  static void chooseStepCB(GuiElement * calling,void * data);
  static void browserCB(GuiElement * calling,void * data);
  static void scene2dCB(GuiElement * calling,void * data);
#ifdef OPEN_GL
  static void scene3dCB(GuiElement * calling,void * data);
#endif
  static void grapherCB(GuiElement * calling,void * data);
  static void remoteCtrlCB(GuiElement * calling,void * data);
  static void readMailCB(GuiElement * calling,void * data);
  static void resizeCB(GuiElement * calling,void * data);
  static void abortSignal(int sigNum);
protected:
  static void _mainTask(void *);
  static void _quitTask(void *);
// Attributes
  StringsVect _commandLine;
  Strings _fileName;
  Strings _currentPath;
  OrisControl::State _state;
  OrisControl::State _oldState;
  bool _quit;
  int _step;
  int _oldStep;
  unsigned long _oldClock;
  TextOutput _textOutput;
  TextOutput _textError;
  GuiWindow * _mainWindow;
  GuiMenuBar * _mainMenu;
  GuiMenuItem * _runStopItm;
  GuiButton * _runStopBtn;
  GuiButton * _oneStepBtn;
  GuiLabel * _stateLabel;
  GuiLabel * _clockLabel;
  GuiButton * _parseBtn;
  GuiButton * _clearBtn;
  GuiButton * _execBtn;
  GuiButton * _startBtn;
  GuiEditor * _editor;
  void (*_prevTask)(void *);
  void * _prevTaskData;
  static OrisControl * _orisControl;
};

#endif // ORISCTRL_H

