// @(#) ENIB/LI2 (c) - oRis - Version 2.3
//--------------------------------------------------------------------------//
// file        : attCbPkg.cpp                                               //
// description : source file for the ENIB/LI2 oRis package attrCB.pkg       //
// author(s)   : F. Harrouet                                                //
// copyright   : Laboratoire d'Informatique Industrielle (LI2)              //
//               Ecole Nationale d'Ingenieurs de Brest (ENIB)               //
//               CP 30815 - 29608 Brest Cedex - FRANCE                      //
//               Tel: (33) 02.98.05.66.31 , Fax: (33) 02.98.05.66.29        //
//               e-mail: li2@enib.fr                                        //
//--------------------------------------------------------------------------//
// Copyright (c) 1996-2000                                                  //
// Laboratoire d'Informatique Industrielle (LI2)                            //
// Ecole Nationale d'Ingenieurs de Brest (ENIB)                             //
//                                                                          //
// Permission to use, copy, modify, distribute and sell this software       //
// and its documentation for any purpose is hereby granted without fee,     //
// provided that the above copyright notice appear in all copies and        //
// that both that copyright notice and this permission notice appear        //
// in supporting documentation.                                             //
// ENIB/LI2 makes no representations about the suitability of this          //
// software for any purpose.                                                //
// It is provided "as is" without express or implied warranty.              //
//--------------------------------------------------------------------------//

#include "AttrCB/attCbPkg.h"
#include "oRis/Main/simul.h"
#include "oRis/Main/context.h"

#ifndef __MSDOS__
#define EXPORT_PREFIX extern "C"
#else
#include <windows.h>
#define EXPORT_PREFIX extern "C" __declspec(dllexport)
#define openingLibrary    openingAttrCbLibrary
#define closingLibrary    closingAttrCbLibrary
#define getExternFunction getExternAttrCbFunction
#define getNewAgent       getNewAttrCbAgent
#endif

//--------------------------------------------------------------------------//
// Opening and closing the library                                          //
//--------------------------------------------------------------------------//
 
EXPORT_PREFIX void openingLibrary(void)
{
  // nothing to do
}
 
EXPORT_PREFIX void closingLibrary(void)
{
  // nothing to do
}

//--------------------------------------------------------------------------//
// LookUp table for agents                                                  //
//--------------------------------------------------------------------------//

static Word attrCBWord(Word::create("AttrCB"));

EXPORT_PREFIX Agent * getNewAgent(Model * model,long instNum)
{
if(model->isA(::attrCBWord)) return(new AttrCB(model,instNum));
return((Agent *)0); // not found
}

//--------------------------------------------------------------------------//
// AttrCB functions                                                         //
//--------------------------------------------------------------------------//

// native void AttrCB::connect(Object obj,string attrName);
//
EXPORT_PREFIX void wrap_AttrCB_connect(Context * context)
{
AttrCB * attrCB=(AttrCB *)context->accessOwner();
if(attrCB->getAgent()) attrCB->accessAgent()->unReference();
attrCB->setAgent(context->getAgent(0));
attrCB->accessAttr()=context->getStrings(1);
bool done=false;
if(attrCB->getAgent()&&attrCB->getAgent()->isAlive())
  {
  Word attrName(Word::create(attrCB->getAttr()));
  Attribute * attribute=attrCB->accessAgent()->accessAttribute(attrName);
  if(attribute)
    {
    attrCB->accessAgent()->reference();
    attrCB->ValueCallback::connect((AttrValue *)attribute->accessValue());
    done=true;
    }
  }
if(!done)
  {
  attrCB->setAgent((Agent *)0);
  attrCB->accessAttr()=Strings();
  }
}

// native void AttrCB::disconnect(void);
//
void wrap_AttrCB_disconnect(Context * context)
{
AttrCB * attrCB=(AttrCB *)context->accessOwner();
if(attrCB->getAgent()) attrCB->accessAgent()->unReference();
attrCB->setAgent((Agent *)0);
attrCB->accessAttr()=Strings();
attrCB->ValueCallback::disconnect();
}

// native void AttrCB::getRef(Object & obj,string & attrName);
//
void wrap_AttrCB_getRef(Context * context)
{
AttrCB * attrCB=(AttrCB *)context->accessOwner();
if(attrCB->getAgent()&&attrCB->getAgent()->isAlive())
  {
  context->setAgent(0,attrCB->accessAgent(),
                      attrCB->accessAgent()->accessModel());
  context->setStrings(1,attrCB->getAttr());
  }
else
  {
  context->setAgent(0,(Agent *)0,Model::getBaseModel());
  context->setStrings(1,Strings());
  }
}

//--------------------------------------------------------------------------//
// LookUp table for functions                                               //
//--------------------------------------------------------------------------//
 
EXPORT_PREFIX ExternFunction getExternFunction(const Strings & name)
{
if(name==Strings("AttrCB::connect()"))
      return(wrap_AttrCB_connect);
if(name==Strings("AttrCB::disconnect()"))
      return(wrap_AttrCB_disconnect);
if(name==Strings("AttrCB::getRef()"))
      return(wrap_AttrCB_getRef);
return((ExternFunction)0); // not found
}

//--------------------------------------------------------------------------//
// class AttrCB                                                             //
//--------------------------------------------------------------------------//

Word AttrCB::_beforeWord(Word::create("before")); // static
Word AttrCB::_afterWord(Word::create("after"));   // static

AttrCB::AttrCB(Model * model,int instNum)
 : Agent(model,instNum), ValueCallback(),
   _agent((Agent *)0), _attribute()
{}

AttrCB::~AttrCB(void)
{
if(_agent) _agent->unReference();
}

Strings AttrCB::userSave(int column) const // virtual
{
Strings str;
if(_agent&&_agent->isAlive())
  {
  for(int i=0;i<column;i++) str+=" ";
  str+=getName().print()+"->connect("+_agent->getName().print()+","
                                     +_attribute.dump()+");\n";
  }
return(str);
}

void AttrCB::clearReferences(bool strongClear) // virtual
{
Agent::clearReferences(strongClear);
if(_agent&&!strongClear) _agent->unReference();
_agent=(Agent *)0;
}

void AttrCB::before(void) // virtual
{
ExecModule * module=accessModule(_beforeWord);
if(module->getType()!=ExecModule::PROTOTYPE)
  {
  getSimul()->executeModule(module,this);
  }
}

void AttrCB::after(void) // virtual
{
ExecModule * module=accessModule(_afterWord);
if(module->getType()!=ExecModule::PROTOTYPE)
  {
  getSimul()->executeModule(module,this);
  }
}

