// @(#) ENIB/LI2 (c) - oRis - Version 2.3
//--------------------------------------------------------------------------//
// file        : cntnrPkg.cpp                                               //
// description : source file for the ENIB/LI2 oRis package contain.pkg      //
// author(s)   : F. Harrouet                                                //
// copyright   : Laboratoire d'Informatique Industrielle (LI2)              //
//               Ecole Nationale d'Ingenieurs de Brest (ENIB)               //
//               CP 30815 - 29608 Brest Cedex - FRANCE                      //
//               Tel: (33) 02.98.05.66.31 , Fax: (33) 02.98.05.66.29        //
//               e-mail: li2@enib.fr                                        //
//--------------------------------------------------------------------------//
// Copyright (c) 1996-2000                                                  //
// Laboratoire d'Informatique Industrielle (LI2)                            //
// Ecole Nationale d'Ingenieurs de Brest (ENIB)                             //
//                                                                          //
// Permission to use, copy, modify, distribute and sell this software       //
// and its documentation for any purpose is hereby granted without fee,     //
// provided that the above copyright notice appear in all copies and        //
// that both that copyright notice and this permission notice appear        //
// in supporting documentation.                                             //
// ENIB/LI2 makes no representations about the suitability of this          //
// software for any purpose.                                                //
// It is provided "as is" without express or implied warranty.              //
//--------------------------------------------------------------------------//

#include "Contain/cntnrPkg.h"
#include "oRis/Main/simul.h"
#include "oRis/Main/context.h"
#include "oRis/Main/tmpVal.h"

#ifndef __MSDOS__
#define EXPORT_PREFIX extern "C"
#else
#include <windows.h>
#define EXPORT_PREFIX extern "C" __declspec(dllexport)
#define openingLibrary    openingCntnrLibrary
#define closingLibrary    closingCntnrLibrary
#define getExternFunction getExternCntnrFunction
#define getNewAgent       getNewCntnrAgent
#endif

//--------------------------------------------------------------------------//
// Opening and closing the library                                          //
//--------------------------------------------------------------------------//
 
EXPORT_PREFIX void openingLibrary(void)
{
  // nothing to do
}
 
EXPORT_PREFIX void closingLibrary(void)
{
  // nothing to do
}

//--------------------------------------------------------------------------//
// LookUp table for functions                                               //
//--------------------------------------------------------------------------//

EXPORT_PREFIX ExternFunction getExternFunction(const Strings & name)
{
if(name==Strings("Set::getSize()"))        return(OSet::getSize);
if(name==Strings("Set::erase()"))          return(OSet::erase);
if(name==Strings("Set::getElems()"))       return(OSet::getElems);
if(name==Strings("Set::hasElem()"))        return(OSet::hasElem);
if(name==Strings("Set::setElem()"))        return(OSet::setElem);
if(name==Strings("Set::remove()"))         return(OSet::remove);
if(name==Strings("Map::getSize()"))        return(OMap::getSize);
if(name==Strings("Map::erase()"))          return(OMap::erase);
if(name==Strings("Map::getElems()"))       return(OMap::getElems);
if(name==Strings("Map::getKeys()"))        return(OMap::getKeys);
if(name==Strings("Map::getElem()"))        return(OMap::getElem);
if(name==Strings("Map::setElem()"))        return(OMap::setElem);
if(name==Strings("Map::readRemove()"))     return(OMap::readRemove);
if(name==Strings("Map::remove()"))         return(OMap::remove);
return((ExternFunction)0); // not found
}

//--------------------------------------------------------------------------//
// LookUp table for agents                                                  //
//--------------------------------------------------------------------------//

static Word mapWord(Word::create("Map"));
static Word setWord(Word::create("Set"));

EXPORT_PREFIX Agent * getNewAgent(Model * model,long instNum)
{
if(model->isA(::mapWord))  return(new OMap(model,instNum));
if(model->isA(::setWord))  return(new OSet(model,instNum));
return((Agent *)0); // not found
}

//--------------------------------------------------------------------------//
// Set functions                                                            //
//--------------------------------------------------------------------------//
 
// native int Set::getSize(void);
//
void OSet::getSize(Context * context) // static
{
const OSet * set=(const OSet *)context->getOwner();
context->returnInteger(set->_set.size());
}
 
// native void Set::erase(void);
//
void OSet::erase(Context * context) // static
{
OSet * set=(OSet *)context->accessOwner();
Value * elem;
while(set->_set.size())
  {
  elem=*set->_set.begin();
  set->_set.erase(set->_set.begin());
  elem->relax();
  }
}

// native ELEM_TYPE[] Set::getElems(void);
//
void OSet::getElems(Context * context) // static
{
const OSet * set=(const OSet *)context->getOwner();
Value * result=TmpValue::getTmpValue(context->getModule()->getResultType());
SubValueVect * vect=SubValue::getSubValueVect();
OSet::SetType::const_iterator it=set->_set.begin();
for(;it!=set->_set.end();it++)
  {
  vect->pushBack(SubValue::getSubValue((*it)->getType(),result));
  (*vect->accessBack())=*(*it);
  }
result->setValueVector(vect);
context->returnValue(result);
}

// native bool Set::hasElem(ELEM_TYPE elem);
//
void OSet::hasElem(Context * context) // static
{
OSet * set=(OSet *)context->accessOwner();
OSet::SetType::iterator it=set->_set.find(context->accessLocal(0));
if(it==set->_set.end()) context->returnInteger(0);
else context->returnInteger(1);
}

// native void Set::setElem(ELEM_TYPE elem);
//
void OSet::setElem(Context * context) // static
{
OSet * set=(OSet *)context->accessOwner();
Value * elem=TmpValue::getTmpValue(context->getLocal(0)->getType());
(*elem)=*context->getLocal(0);
OSet::SetType::iterator it=set->_set.find(context->accessLocal(0));
if(it!=set->_set.end())
  {
  Value * oldElem=*it;
  set->_set.erase(it);
  set->_set.insert(elem);
  oldElem->relax();
  }
else set->_set.insert(elem);
}

// native void Set::remove(ELEM_TYPE elem);
//
void OSet::remove(Context * context) // static
{
OSet * set=(OSet *)context->accessOwner();
OSet::SetType::iterator it=set->_set.find(context->accessLocal(0));
if(it!=set->_set.end())
  {
  Value * elem=*it;
  set->_set.erase(it);
  elem->relax();
  }
}

//--------------------------------------------------------------------------//
// Map functions                                                            //
//--------------------------------------------------------------------------//

// native int Map::getSize(void);
//
void OMap::getSize(Context * context) // static
{
const OMap * map=(const OMap *)context->getOwner();
context->returnInteger(map->_map.size());
}

// native void Map::erase(void);
//
void OMap::erase(Context * context) // static
{
OMap * map=(OMap *)context->accessOwner();
Value * key;
Value * elem;
while(map->_map.size())
  {
  key=(*map->_map.begin()).first;
  elem=(*map->_map.begin()).second;
  map->_map.erase(map->_map.begin());
  key->relax();
  elem->relax();
  }
}

// native ELEM_TYPE[] Map::getElems(void);
//
void OMap::getElems(Context * context) // static
{
const OMap * map=(const OMap *)context->getOwner();
Value * result=TmpValue::getTmpValue(context->getModule()->getResultType());
SubValueVect * vect=SubValue::getSubValueVect();
OMap::MapType::const_iterator it=map->_map.begin();
for(;it!=map->_map.end();it++)
  {
  vect->pushBack(SubValue::getSubValue((*it).second->getType(),result));
  (*vect->accessBack())=*(*it).second;
  }
result->setValueVector(vect);
context->returnValue(result);
}

// native KEY_TYPE[] Map::getKeys(void);
//
void OMap::getKeys(Context * context) // static
{
const OMap * map=(const OMap *)context->getOwner();
Value * result=TmpValue::getTmpValue(context->getModule()->getResultType());
SubValueVect * vect=SubValue::getSubValueVect();
OMap::MapType::const_iterator it=map->_map.begin();
for(;it!=map->_map.end();it++)
  {
  vect->pushBack(SubValue::getSubValue((*it).first->getType(),result));
  (*vect->accessBack())=*(*it).first;
  }
result->setValueVector(vect);
context->returnValue(result);
}

// native bool Map::getElem(KEY_TYPE key,ELEM_TYPE & found);
//
void OMap::getElem(Context * context) // static
{
OMap * map=(OMap *)context->accessOwner();
OMap::MapType::iterator it=map->_map.find(context->accessLocal(0));
if(it==map->_map.end()) context->returnInteger(0);
else
  {
  (*context->accessLocal(1))=*(*it).second;
  context->returnInteger(1);
  }
}

// native void Map::setElem(KEY_TYPE key,ELEM_TYPE elem);
//
void OMap::setElem(Context * context) // static
{
OMap * map=(OMap *)context->accessOwner();
Value * key=TmpValue::getTmpValue(context->getLocal(0)->getType());
(*key)=*context->getLocal(0);
Value * elem=TmpValue::getTmpValue(context->getLocal(1)->getType());
(*elem)=*context->getLocal(1);
StlPair<OMap::MapType::iterator,bool>
       p(map->_map.insert(OMap::MapType::value_type(key,elem)));
if(!p.second)
  {
  Value * oldElem=(*p.first).second;
  (*p.first).second=elem;
  key->relax();
  oldElem->relax();
  }
}

// native bool Map::readRemove(KEY_TYPE key,ELEM_TYPE & found);
//
void OMap::readRemove(Context * context) // static
{
OMap * map=(OMap *)context->accessOwner();
OMap::MapType::iterator it=map->_map.find(context->accessLocal(0));
if(it==map->_map.end()) context->returnInteger(0);
else
  {
  Value * key=(*it).first;
  Value * elem=(*it).second;
  map->_map.erase(it);
  (*context->accessLocal(1))=*elem;
  key->relax();
  elem->relax();
  context->returnInteger(1);
  }
}

// native void Map::remove(KEY_TYPE key);
//
void OMap::remove(Context * context) // static
{
OMap * map=(OMap *)context->accessOwner();
OMap::MapType::iterator it=map->_map.find(context->accessLocal(0));
if(it!=map->_map.end()) 
  {
  Value * key=(*it).first;
  Value * elem=(*it).second;
  map->_map.erase(it);
  key->relax();
  elem->relax();
  }
}

//--------------------------------------------------------------------------//
// class OSet                                                               //
//--------------------------------------------------------------------------//
 
OSet::OSet(Model * model,int instNum)
 : Agent(model,instNum),
   _set()
{}
 
OSet::~OSet(void)
{}
 
void OSet::clearReferences(bool strongClear) // virtual
{
Agent::clearReferences(strongClear);
Value * elem;
while(_set.size())
  {
  elem=*_set.begin();
  _set.erase(_set.begin());
  if(strongClear)
    {
    elem->clearReferences(strongClear);
    }
  elem->relax();
  }
}
 
Strings OSet::userSave(int /* column */) const // virtual
{
Strings str;
return(str);
}

//--------------------------------------------------------------------------//
// class OMap                                                               //
//--------------------------------------------------------------------------//

OMap::OMap(Model * model,int instNum)
 : Agent(model,instNum),
   _map()
{}

OMap::~OMap(void)
{}

void OMap::clearReferences(bool strongClear) // virtual
{
Agent::clearReferences(strongClear);
Value * key;
Value * elem;
while(_map.size())
  {
  key=(*_map.begin()).first;
  elem=(*_map.begin()).second;
  _map.erase(_map.begin());
  if(strongClear)
    {
    key->clearReferences(strongClear);
    elem->clearReferences(strongClear);
    }
  key->relax();
  elem->relax();
  }
}

Strings OMap::userSave(int /* column */) const // virtual
{
Strings str;
return(str);
}

