// @(#) ENIB/LI2 (c) - oRis - Version 2.3
//--------------------------------------------------------------------------//
// file        : convPkg.cpp                                                //
// description : source file for the ENIB/LI2 oRis package convert.pkg      //
// author(s)   : F. Harrouet                                                //
// copyright   : Laboratoire d'Informatique Industrielle (LI2)              //
//               Ecole Nationale d'Ingenieurs de Brest (ENIB)               //
//               CP 30815 - 29608 Brest Cedex - FRANCE                      //
//               Tel: (33) 02.98.05.66.31 , Fax: (33) 02.98.05.66.29        //
//               e-mail: li2@enib.fr                                        //
//--------------------------------------------------------------------------//
// Copyright (c) 1996-2000                                                  //
// Laboratoire d'Informatique Industrielle (LI2)                            //
// Ecole Nationale d'Ingenieurs de Brest (ENIB)                             //
//                                                                          //
// Permission to use, copy, modify, distribute and sell this software       //
// and its documentation for any purpose is hereby granted without fee,     //
// provided that the above copyright notice appear in all copies and        //
// that both that copyright notice and this permission notice appear        //
// in supporting documentation.                                             //
// ENIB/LI2 makes no representations about the suitability of this          //
// software for any purpose.                                                //
// It is provided "as is" without express or implied warranty.              //
//--------------------------------------------------------------------------//

#include "Convert/convPkg.h"
#include "oRis/Main/simul.h"
#include "oRis/Main/context.h"

#ifndef __MSDOS__
#define EXPORT_PREFIX extern "C"
#else
#include <windows.h>
#define EXPORT_PREFIX extern "C" __declspec(dllexport)
#define openingLibrary    openingConvLibrary
#define closingLibrary    closingConvLibrary
#define getExternFunction getExternConvFunction
#define getNewAgent       getNewConvAgent
#endif

//--------------------------------------------------------------------------//
// Opening and closing the library                                          //
//--------------------------------------------------------------------------//
 
EXPORT_PREFIX void openingLibrary(void)
{
  // nothing to do
}
 
EXPORT_PREFIX void closingLibrary(void)
{
  // nothing to do
}


//--------------------------------------------------------------------------//
// LookUp table for functions                                               //
//--------------------------------------------------------------------------//

EXPORT_PREFIX ExternFunction getExternFunction(const Strings & name)
{
if(name==Strings("IntConv::read()"))           return(IntConv::read);
if(name==Strings("IntConv::intValue()"))       return(IntConv::intValue);
if(name==Strings("IntConv::floatValue()"))     return(IntConv::floatValue);
if(name==Strings("IntConv::stringValue()"))    return(IntConv::stringValue);

if(name==Strings("FloatConv::read()"))         return(FloatConv::read);
if(name==Strings("FloatConv::intValue()"))     return(FloatConv::intValue);
if(name==Strings("FloatConv::floatValue()"))   return(FloatConv::floatValue);
if(name==Strings("FloatConv::stringValue()"))  return(FloatConv::stringValue);

if(name==Strings("StringConv::read()"))        return(StringConv::read);
if(name==Strings("StringConv::fail()"))        return(StringConv::fail);
if(name==Strings("StringConv::intValue()"))    return(StringConv::intValue);
if(name==Strings("StringConv::floatValue()"))  return(StringConv::floatValue);
if(name==Strings("StringConv::stringValue()")) return(StringConv::stringValue);
if(name==Strings("StringConv::objectValue()")) return(StringConv::objectValue);
if(name==Strings("StringConv::lookup()"))      return(StringConv::lookup);
if(name==Strings("StringConv::subString()"))   return(StringConv::subString);
if(name==Strings("StringConv::substitute()"))  return(StringConv::substitute);
if(name==Strings("StringConv::substituteAll()"))
                                            return(StringConv::substituteAll);
if(name==Strings("StringConv::baseName()"))    return(StringConv::baseName);
if(name==Strings("StringConv::extension()"))   return(StringConv::extension);
if(name==Strings("StringConv::dirName()"))     return(StringConv::dirName);
if(name==Strings("StringConv::toUpper()"))     return(StringConv::toUpper);
if(name==Strings("StringConv::toLower()"))     return(StringConv::toLower);
if(name==Strings("StringConv::toUpperChar()")) return(StringConv::toUpperChar);
if(name==Strings("StringConv::toLowerChar()")) return(StringConv::toLowerChar);
if(name==Strings("StringConv::length()"))      return(StringConv::length);

if(name==Strings("ObjectConv::read()"))        return(ObjectConv::read);
if(name==Strings("ObjectConv::stringValue()")) return(ObjectConv::stringValue);
if(name==Strings("ObjectConv::objectValue()")) return(ObjectConv::objectValue);
return((ExternFunction)0); // not found
}

//--------------------------------------------------------------------------//
// LookUp table for agents                                                  //
//--------------------------------------------------------------------------//

static Word objectConvWord(Word::create("ObjectConv"));
static Word stringConvWord(Word::create("StringConv"));
static Word floatConvWord(Word::create("FloatConv"));
static Word intConvWord(Word::create("IntConv"));

EXPORT_PREFIX Agent * getNewAgent(Model * model,long instNum)
{
if(model->isA(::objectConvWord)) return(new ObjectConv(model,instNum));
if(model->isA(::stringConvWord)) return(new StringConv(model,instNum));
if(model->isA(::floatConvWord))  return(new FloatConv(model,instNum));
if(model->isA(::intConvWord))    return(new IntConv(model,instNum));
return((Agent *)0); // not found
}

//--------------------------------------------------------------------------//
// IntConv functions                                                        //
//--------------------------------------------------------------------------//

// native void IntConv::read(int i);
//
void IntConv::read(Context * context) // static
{
((IntConv *)context->accessOwner())->_intValue=context->getInteger(0);
}

// native int IntConv::intValue(void);
//
void IntConv::intValue(Context * context) // static
{
context->returnInteger(((const IntConv *)context->getOwner())->_intValue);
}

// native float IntConv::floatValue(void);
//
void IntConv::floatValue(Context * context) // static
{
context->returnReal((double)((const IntConv *)context->getOwner())->_intValue);
}

// native string IntConv::stringValue(void);
//
void IntConv::stringValue(Context * context) // static
{
context->returnStrings(Strings(((const IntConv *)context->getOwner())
                               ->_intValue));
}

//--------------------------------------------------------------------------//
// FloatConv functions                                                      //
//--------------------------------------------------------------------------//

// native void FloatConv::read(float f);
//
void FloatConv::read(Context * context) // static
{
((FloatConv *)context->accessOwner())->_floatValue=context->getReal(0);
}

// native int FloatConv::intValue(void);
//
void FloatConv::intValue(Context * context) // static
{
context->returnInteger((long)((const FloatConv *)context->getOwner())
                                                        ->_floatValue);
}

// native float FloatConv::floatValue(void);
//
void FloatConv::floatValue(Context * context) // static
{
context->returnReal(((const FloatConv *)context->getOwner())->_floatValue);
}

// native string FloatConv::stringValue(void);
//
void FloatConv::stringValue(Context * context) // static
{
context->returnStrings(Strings(((const FloatConv *)context->getOwner())
                               ->_floatValue));
}

//--------------------------------------------------------------------------//
// StringConv functions                                                     //
//--------------------------------------------------------------------------//

// native void StringConv::read(string s);
//
void StringConv::read(Context * context) // static
{
((StringConv *)context->accessOwner())->_fail=false;
((StringConv *)context->accessOwner())->_stringValue=context->getStrings(0);
}

// native bool StringConv::fail(void);
//
void StringConv::fail(Context * context) // static
{
context->returnInteger(((const StringConv *)context->getOwner())->_fail);
}

// native int StringConv::intValue(void);
//
void StringConv::intValue(Context * context) // static
{
long i;
if(sscanf(((const StringConv *)context->getOwner())->_stringValue,
          "%ld",&i)==1)
  {
  ((StringConv *)context->accessOwner())->_fail=false;
  context->returnInteger(i);
  }
else
  {
  ((StringConv *)context->accessOwner())->_fail=true;
  context->returnInteger(0);
  }
}

// native float StringConv::floatValue(void);
//
void StringConv::floatValue(Context * context) // static
{
double f;
if(sscanf(((const StringConv *)context->getOwner())->_stringValue,
          "%lf",&f)==1)
  {
  ((StringConv *)context->accessOwner())->_fail=false;
  context->returnReal(f);
  }
else
  {
  ((StringConv *)context->accessOwner())->_fail=true;
  context->returnReal(0.0);
  }
}

// native string StringConv::stringValue(void);
//
void StringConv::stringValue(Context * context) // static
{
((StringConv *)context->accessOwner())->_fail=false;
context->returnStrings(((const StringConv *)context->getOwner())
                                                   ->_stringValue);
}

// native Object StringConv::objectValue(void);
//
void StringConv::objectValue(Context * context) // static
{
AgentName agtName(((const StringConv *)context->getOwner())->_stringValue);
Agent * agt=getSimul()->accessAgent(agtName);
if(agt)
  {
  ((StringConv *)context->accessOwner())->_fail=false;
  context->returnAgent(agt,agt->accessModel());
  }
else
  {
  ((StringConv *)context->accessOwner())->_fail=true;
  context->returnAgent(agt,Model::getBaseModel());
  }
}

// native int StringConv::lookup(string str);
//
void StringConv::lookup(Context * context) // static
{
((StringConv *)context->accessOwner())->_fail=false;
int result=((const StringConv *)context->getOwner())
           ->_stringValue.lookUp(context->getStrings(0));
context->returnInteger(result);
}

// native string StringConv::subString(int pos,int len);
//
void StringConv::subString(Context * context) // static
{
((StringConv *)context->accessOwner())->_fail=false;
context->returnStrings(((const StringConv *)context->getOwner())
                       ->_stringValue.subString(context->getInteger(0),
                                                context->getInteger(1)));
}

// native int StringConv::substitute(string str1,string str2);
//
void StringConv::substitute(Context * context) // static
{
((StringConv *)context->accessOwner())->_fail=false;
int result=((StringConv *)context->accessOwner())
           ->_stringValue.subst(context->getStrings(0),
                                context->getStrings(1));
context->returnInteger(result);
}

// native int StringConv::substituteAll(string str1,string str2);
//
void StringConv::substituteAll(Context * context) // static
{
((StringConv *)context->accessOwner())->_fail=false;
int result=((StringConv *)context->accessOwner())
           ->_stringValue.substAll(context->getStrings(0),
                                   context->getStrings(1));
context->returnInteger(result);
}

// native string StringConv::baseName(void);
//
void StringConv::baseName(Context * context) // static
{
((StringConv *)context->accessOwner())->_fail=false;
context->returnStrings(((const StringConv *)context->getOwner())
                       ->_stringValue.basename());
}

// native string StringConv::extension(void);
//
void StringConv::extension(Context * context) // static
{
((StringConv *)context->accessOwner())->_fail=false;
context->returnStrings(((const StringConv *)context->getOwner())
                       ->_stringValue.extension());
}

// native string StringConv::dirName(void);
//
void StringConv::dirName(Context * context) // static
{
((StringConv *)context->accessOwner())->_fail=false;
context->returnStrings(((const StringConv *)context->getOwner())
                       ->_stringValue.dirname());
}

// native bool StringConv::toUpperChar(int pos);
//
void StringConv::toUpperChar(Context * context) // static
{
((StringConv *)context->accessOwner())->_fail=false;
context->returnInteger(((StringConv *)context->accessOwner())
                       ->_stringValue.toUpper(context->getInteger(0)));
}
 
// native bool StringConv::toLowerChar(int pos);
//
void StringConv::toLowerChar(Context * context) // static
{
((StringConv *)context->accessOwner())->_fail=false;
context->returnInteger(((StringConv *)context->accessOwner())
                       ->_stringValue.toLower(context->getInteger(0)));
}
 
// native void StringConv::toUpper(void);
//
void StringConv::toUpper(Context * context) // static
{
((StringConv *)context->accessOwner())->_fail=false;
((StringConv *)context->accessOwner())->_stringValue.toUpper();
}
 
// native void StringConv::toLower(void);
//
void StringConv::toLower(Context * context) // static
{
((StringConv *)context->accessOwner())->_fail=false;
((StringConv *)context->accessOwner())->_stringValue.toLower();
}

// native int StringConv::length(void);
//
void StringConv::length(Context * context) // static
{
((StringConv *)context->accessOwner())->_fail=false;
context->returnInteger(((const StringConv *)context->getOwner())
                       ->_stringValue.len());
}

//--------------------------------------------------------------------------//
// ObjectConv functions                                                     //
//--------------------------------------------------------------------------//

// native void ObjectConv::read(Object o);
//
void ObjectConv::read(Context * context) // static
{
ObjectConv * objConv=(ObjectConv *)context->accessOwner();
if(objConv->_objectValue) objConv->_objectValue->unReference();
objConv->_objectValue=context->getAgent(0);
if(objConv->_objectValue) objConv->_objectValue->reference();
}

// native string ObjectConv::stringValue(void);
//
void ObjectConv::stringValue(Context * context) // static
{
const ObjectConv * objConv=(const ObjectConv *)context->getOwner();
if(objConv->_objectValue)
  context->returnStrings(objConv->_objectValue->getName().print());
else
  context->returnStrings("NONE");
}

// native Object ObjectConv::objectValue(void);
//
void ObjectConv::objectValue(Context * context) // static
{
const ObjectConv * objConv=(const ObjectConv *)context->getOwner();
if(objConv->_objectValue)
  context->returnAgent(objConv->_objectValue,
                       objConv->_objectValue->accessModel());
else
  context->returnAgent((Agent *)0,Model::getBaseModel());
}

//--------------------------------------------------------------------------//
// class IntConv                                                            //
//--------------------------------------------------------------------------//

IntConv::IntConv(Model * model,int instNum)
 : Agent(model,instNum), _intValue(0)
{}

IntConv::~IntConv(void)
{}

Strings IntConv::userSave(int column) const // virtual
{
Strings str;
for(int i=0;i<column;i++) str+=" ";
str+=getName().print()+"->read("+Strings(_intValue)+");\n";
return(str);
}

//--------------------------------------------------------------------------//
// class FloatConv                                                          //
//--------------------------------------------------------------------------//

FloatConv::FloatConv(Model * model,int instNum)
 : Agent(model,instNum), _floatValue(0.0)
{}

FloatConv::~FloatConv(void)
{}

Strings FloatConv::userSave(int column) const // virtual
{
Strings str;
for(int i=0;i<column;i++) str+=" ";
str+=getName().print()+"->read("+Strings(_floatValue)+");\n";
return(str);
}

//--------------------------------------------------------------------------//
// class StringConv                                                         //
//--------------------------------------------------------------------------//

StringConv::StringConv(Model * model,int instNum)
 : Agent(model,instNum), _stringValue(), _fail(false)
{}

StringConv::~StringConv(void)
{}

Strings StringConv::userSave(int column) const // virtual
{
Strings str;
for(int i=0;i<column;i++) str+=" ";
str+=getName().print()+"->read("+_stringValue.dump()+");\n";
return(str);
}

//--------------------------------------------------------------------------//
// class ObjectConv                                                         //
//--------------------------------------------------------------------------//

ObjectConv::ObjectConv(Model * model,int instNum)
 : Agent(model,instNum), _objectValue((Agent *)0)
{}

ObjectConv::~ObjectConv(void)
{}

void ObjectConv::clearReferences(bool strongClear) // virtual
{
Agent::clearReferences(strongClear);
if(_objectValue&&!strongClear) _objectValue->unReference();
_objectValue=(Agent *)0;
}

Strings ObjectConv::userSave(int column) const // virtual
{
Strings str;
for(int i=0;i<column;i++) str+=" ";
str+=getName().print()+"->read(";
if(_objectValue&&_objectValue->isAlive())
  str+=_objectValue->getName().print();
else
  str+="NONE";
str+=");\n";
return(str);
}

