// @(#) ENIB/LI2 (c) - oRis - Version 2.3
//--------------------------------------------------------------------------//
// file        : fcm.C                                                      //
// description : Implementation fuzzy cognitive maps in oRis                //
// author(s)   : E. Maffre                                                  //
// modified    : O. Ansquer             			            //
// copyright   : Laboratoire d'Informatique Industrielle (LI2)              //
//               Ecole Nationale d'Ingenieurs de Brest (ENIB)               //
//               CP 30815 - 29608 Brest Cedex - FRANCE                      //
//               Tel: (33) 02.98.05.66.31 , Fax: (33) 02.98.05.66.29        //
//               e-mail: li2@enib.fr                                        //
//--------------------------------------------------------------------------//
// Copyright (c) 1996-2000                                                  //
// Laboratoire d'Informatique Industrielle (LI2)                            //
// Ecole Nationale d'Ingenieurs de Brest (ENIB)                             //
//                                                                          //
// Permission to use, copy, modify, distribute and sell this software       //
// and its documentation for any purpose is hereby granted without fee,     //
// provided that the above copyright notice appear in all copies and        //
// that both that copyright notice and this permission notice appear        //
// in supporting documentation.                                             //
// ENIB/LI2 makes no representations about the suitability of this          //
// software for any purpose.                                                //
// It is provided "as is" without express or implied warranty.              //
//--------------------------------------------------------------------------//

#include "fcm.h"

#include <stdlib.h>
#include <fstream>
using namespace std;

#include "oRis/Main/simul.h"
#include "oRis/Main/context.h"
#include "oRis/Main/tmpVal.h"
#include "oRis/Main/simul.h"

#include <dlfcn.h>

#define replace FCM_EPSILON by 0.001 
#define EXPORT_PREFIX extern "C"

EXPORT_PREFIX void openingLibrary(void){
  cerr << "### Fcm Library Loaded" << endl;
} 

EXPORT_PREFIX void closingLibrary(void){
  cerr << "### Fcm Library unloaded" << endl;
} 

//----------------------------------------------------------------------//
// LookUp table for fonctions
//----------------------------------------------------------------------// 

EXPORT_PREFIX ExternFunction getExternFunction( const Strings & name){
  //  cout << name << endl;
  if ( name==Strings("Fcm::new()"))
    return (NFcm::N_Fcm);
  if ( name==Strings("Fcm::affect()"))
    return (NFcm::affect);

  if ( name==Strings("Fcm::setName()"))
    return (NFcm::setName);
  if ( name==Strings("Fcm::setBiValued()"))
    return (NFcm::setBiValued);
  if ( name==Strings("Fcm::setTriValued()"))
  return (NFcm::setTriValued);
  if ( name==Strings("Fcm::isBiValued()"))
    return (NFcm::isBiValued);
  if ( name==Strings("Fcm::isTriValued()"))
    return (NFcm::isTriValued);
  if ( name==Strings("Fcm::setCrisp()"))
    return (NFcm::setCrisp);
  if ( name==Strings("Fcm::setFuzzy()"))
    return (NFcm::setFuzzy);  
  if ( name==Strings("Fcm::setThreshold()"))
    return (NFcm::setThreshold);
  if ( name==Strings("Fcm::setNormalisationBeforeForcing()"))
    return (NFcm::setNormalisationBeforeForcing);
  if ( name==Strings("Fcm::isCrisp()"))
    return (NFcm::isCrisp);
  if ( name==Strings("Fcm::setDelta()"))
    return (NFcm::setDelta);
  if ( name==Strings("Fcm::setA0()"))
    return (NFcm::setA0);  
  if ( name==Strings("Fcm::getDelta()"))
    return (NFcm::getDelta);
  if ( name==Strings("Fcm::getA0()"))
    return (NFcm::getA0);
  if ( name==Strings("Fcm::getK()"))
    return (NFcm::getK);
  if ( name==Strings("Fcm::setK()"))
    return (NFcm::setK);
  if ( name==Strings("Fcm::getThreshold()"))
    return (NFcm::getThreshold);
  if ( name==Strings("Fcm::getNormalisationBeforeForcing()"))
    return (NFcm::getNormalisationBeforeForcing);

  if ( name==Strings("Fcm::addConcepts()"))
    return (NFcm::addConcepts) ;
  if ( name==Strings("Fcm::addConceptsFromFile()"))
    return (NFcm::addConceptsFromFile) ;
  if ( name==Strings("Fcm::addRandomizedConceptsFromFile()"))
    return (NFcm::addRandomizedConceptsFromFile) ;
  if ( name==Strings("Fcm::removeConcepts()"))
    return (NFcm::removeConcepts) ;
  if ( name==Strings("Fcm::connect()"))
    return (NFcm::connect) ;
  if ( name==Strings("Fcm::disconnect()"))
    return (NFcm::disconnect) ;
  if ( name==Strings("Fcm::merge()"))
    return (NFcm::merge) ;

  if ( name==Strings("Fcm::display()"))
    return (NFcm::display) ;
  if ( name==Strings("Fcm::_statusTextValues()"))
    return (NFcm::_statusTextValues) ;

  if ( name==Strings("Fcm::eval()"))
    return (NFcm::eval) ;
  if ( name==Strings("Fcm::cycle()"))
    return (NFcm::cycle) ;

  if ( name==Strings("Fcm::setValues()"))
    return (NFcm::setValues) ;
  if ( name==Strings("Fcm::setForcing()"))
    return (NFcm::setForcing) ;
  if ( name==Strings("Fcm::_getValues()"))
    return (NFcm::_getValues) ;
  if ( name==Strings("Fcm::_getForcing()"))
    return (NFcm::_getForcing) ;
  if ( name==Strings("Fcm::setValue()"))
    return (NFcm::setValue) ;
  if ( name==Strings("Fcm::getValue()"))
    return (NFcm::getValue) ;
  if ( name==Strings("Fcm::setNamedForcing()"))
    return (NFcm::setNamedForcing) ;
  if ( name==Strings("Fcm::getNamedForcing()"))
    return (NFcm::getNamedForcing) ;

  if ( name==Strings("Fcm::has()"))
    return (NFcm::has) ;
  if ( name==Strings("Fcm::getWeight()"))
    return (NFcm::getWeight) ;
  if ( name==Strings("Fcm::getSize()"))
    return (NFcm::getSize) ;
  if ( name==Strings("Fcm::getName()"))
    return (NFcm::getName) ;

  if ( name==Strings("Fcm::SetGFct()"))
    return (NFcm::SetGFct);
  if ( name==Strings("Fcm::GetGFct()"))
    return (NFcm::GetGFct);
  if ( name==Strings("Fcm::GetGFctParams()"))
    return (NFcm::GetGFctParams);
  if ( name==Strings("Fcm::SetDefaultGFct()"))
    return (NFcm::SetDefaultGFct);
  if ( name==Strings("Fcm::GetDefaultGFct()"))
    return (NFcm::GetDefaultGFct);
  if ( name==Strings("Fcm::GetDefaultGFctParams()"))
    return (NFcm::GetDefaultGFctParams);
  if ( name==Strings("Fcm::SetNumberOfEvals()"))
    return (NFcm::SetNumberOfEvals);
  if ( name==Strings("Fcm::GetNumberOfEvals()"))
    return (NFcm::GetNumberOfEvals);
    
  if ( name==Strings("Fcm::statusTextValues()"))
    return (NFcm::statusTextValues);
  if ( name==Strings("Fcm::getValues()"))
    return (NFcm::getValues);
  if ( name==Strings("Fcm::getForcing()"))
    return (NFcm::getForcing);
  if ( name==Strings("Fcm::getTextWeight()"))
    return (NFcm::getTextWeight);
  if ( name==Strings("Fcm::setTextWeight()"))
    return (NFcm::setTextWeight);
  if ( name==Strings("Fcm::main()"))
    return (NFcm::main);

  return ((ExternFunction)0);
}

//----------------------------------------------------------------------//
// LookUp table for fonctions
//----------------------------------------------------------------------// 

static Word FcmWord(Word::create("Fcm"));
static Word LearnFcmWord(Word::create("LearnFcm"));
static Word CorrelationFcmWord(Word::create("CorrelationFcm"));
static Word HebbianFcmWord(Word::create("HebbianFcm"));

EXPORT_PREFIX Agent * getNewAgent(Model * model, long instNum){
  if(model->isA(::FcmWord))
    return (new NFcm(model,instNum));
  /*  if(model->isA(::LearnFcmWord))
      return (new NLearnFcm(model,instNum));
      if(model->isA(::CorrelationFcmWord))
      return (new NCorrelationFcm(model,instNum));
      if(model->isA(::HebbianFcmWord))
      return (new NHebbianFcm(model,instNum));*/
  return((Agent*)0);
}

//----------------------------------------------------------------------//
NFcm::NFcm ( Model * model  , int instNum ) : Agent(model,instNum) {
}

NFcm::~NFcm(void) {
  if (_nbConcepts > 0  ){
    delete[] _matrix;
    delete[] _conceptVector;
    delete[] _forcingVector;
    delete[] _gFunctions;
  }
}
//native void Fcm::new(void) ;
void 
NFcm::N_Fcm(Context * context){
  NFcm * obj = (NFcm*)context->getOwner();
  obj->_nbConcepts = 0;
  obj->_conceptVector = NULL;
  obj->_forcingVector = NULL;
  obj->_matrix = NULL;
  obj->_gFunctions = NULL;
  obj->_biValued = true;
  obj->_triValued = false;
  obj->_fuzzy = true ;
  obj->_threshold = 0.5 ;
  obj->_k = 5;
  obj->_delta = 0;
  obj->_a0 = 0.5;
  obj->_defaultGFunction = "sum";
  obj->_numberOfEvals = 4;
  // obj->_normalisationBeforeForcing = false ;
}

//native void Fcm::setName(string name) ;
void 
NFcm::setName(Context * context){
  NFcm * obj = (NFcm*)context->getOwner();
  obj->_name = context->getStrings(0); 
}

//native void Fcm::affect(Object) ;
void 
NFcm::affect(Context * context){
  NFcm * obj = (NFcm*)context->getOwner();
  NFcm * model = (NFcm*)context->getAgent(0);

  obj->_name = model->_name;
  obj->_biValued = model->_biValued;
  obj->_triValued = model->_triValued;
  obj->_fuzzy =  model->_fuzzy;
  obj->_threshold =  model->_threshold;
  obj->_k = model->_k;
  obj->_delta = model->_delta;
  obj->_a0 =model ->_a0; 
  obj->_normalisationBeforeForcing = model->_normalisationBeforeForcing;
  obj->_defaultGFunction = model->_defaultGFunction;
  obj->_defaultGParams = model->_defaultGParams;
  obj->_numberOfEvals = model->_numberOfEvals;
  
  obj->_nbConcepts = model->_nbConcepts;
  obj->_conceptsName = model->_conceptsName;
  obj->_conceptVector = new float[obj->_nbConcepts];
  obj->_forcingVector = new float[obj->_nbConcepts];
  obj->_matrix = new float[obj->_nbConcepts * obj->_nbConcepts];
  obj->_gFunctions = new GFunction[obj->_nbConcepts];
  memcpy (obj->_conceptVector , model->_conceptVector , obj->_nbConcepts * sizeof(float) );
  memcpy (obj->_forcingVector , model->_forcingVector , obj->_nbConcepts * sizeof(float) );
  memcpy (obj->_matrix , model->_matrix , obj->_nbConcepts * obj->_nbConcepts * sizeof(float) );
  for (size_t i=0;i<obj->_nbConcepts;i++)
     obj->_gFunctions[i].affect(model->_gFunctions[i]);
}

//native void Fcm::setBiValued(void) ;
void 
NFcm::setBiValued(Context * context) {
  NFcm * obj = (NFcm*)context->getOwner();
  obj->_biValued = true;
  obj->_triValued = false;
}
//native void Fcm::setTriValued(void)
void 
NFcm::setTriValued(Context * context) {
  NFcm * obj = (NFcm*)context->getOwner();
  obj->_biValued = false;
  obj->_triValued = true;
}
//native bool Fcm::isBiValued(void)
void 
NFcm::isBiValued(Context * context) {
  NFcm * obj = (NFcm*)context->getOwner();
  context->returnInteger( obj->_biValued ) ;
}
//native bool Fcm::isTriValued(void)
void 
NFcm::isTriValued(Context * context) {
  NFcm * obj = (NFcm*)context->getOwner();
  context->returnInteger( obj->_triValued ) ;
}

//native void Fcm::setFuzzy(float k)
void 
NFcm::setFuzzy(Context * context) {
  NFcm * obj = (NFcm*)context->getOwner();
  obj->_fuzzy = true;
  obj->_k = context->getReal(0);
}
//native void Fcm::setCrisp(void)
void 
NFcm::setCrisp(Context * context) {
  NFcm * obj = (NFcm*)context->getOwner();
  obj->_fuzzy = false;
}
//native void Fcm::setThreshold(float threshold)
void 
NFcm::setThreshold(Context * context) {
  NFcm * obj = (NFcm*)context->getOwner();
  obj->_threshold = false;
}

//native void Fcm::setNormalisationBeforeForcing(bool norm);
void 
NFcm::setNormalisationBeforeForcing(Context * context) {
  NFcm * obj = (NFcm*)context->getOwner();
  obj->_normalisationBeforeForcing = context->getInteger(0);
}

//native bool Fcm::getNormalisationBeforeForcing(void);
void NFcm::getNormalisationBeforeForcing(Context * context) {
   NFcm * obj = (NFcm*)context->getOwner(); 
   context->returnInteger( (obj->_normalisationBeforeForcing) ) ;
}

//native float Fcm::getK(void);
void NFcm::getK(Context * context){
   NFcm * obj = (NFcm*)context->getOwner(); 
   context->returnReal( (obj->_k) ) ;
}

//native void Fcm::setK(float);
void NFcm::setK(Context * context){
   NFcm * obj = (NFcm*)context->getOwner(); 
   obj->_k = context->getReal(0); ;
}

//native float Fcm::getThreshold(void);
void NFcm::getThreshold(Context * context){
   NFcm * obj = (NFcm*)context->getOwner(); 
   context->returnReal( (obj->_threshold) ) ;
}

//native bool Fcm::isCrisp(void)
void 
NFcm::isCrisp(Context * context) {
  NFcm * obj = (NFcm*)context->getOwner();
  context->returnInteger( !(obj->_fuzzy) ) ;
}

//native void Fcm::SetGFct(string concept,string gFunction,float[] params);
void
NFcm::SetGFct(Context * context)
{
  NFcm * obj = (NFcm*)context->getOwner();
  Strings concept = context->getStrings(0);
  int  i = _seek(concept,obj);
  if ( i < 0 ) 
    cerr << "Fcm::SetGFunction : don't have any concept named " << concept << " ."  << endl;
  else
  {
    const SubValueVect * subValueVect = context->getValueVector(2);
    vector<float> params;
    for (size_t ii = 0 ; ii < subValueVect->getSize() ; ii ++ )
       params.push_back( ((*subValueVect)[ii])->getReal() );
    
    obj->_gFunctions[i].init(context->getStrings(1),params);
  }
}
  
//native string Fcm::GetGFct(string concept);
void
NFcm::GetGFct(Context * context)
{
  NFcm * obj = (NFcm*)context->getOwner();
  Strings concept = context->getStrings(0);
  int  i = _seek(concept,obj);
  if ( i < 0 ) 
    cerr << "Fcm::SetGFunction : don't have any concept named " << concept << " ."  << endl;
  else
    context->returnStrings(obj->_gFunctions[i].getName());
}

//native float[] Fcm::GetGFctParams(string concept);
void
NFcm::GetGFctParams(Context * context)
{
  NFcm * obj = (NFcm*)context->getOwner();
  Strings concept = context->getStrings(0);
  int  i = _seek(concept,obj);
  if ( i < 0 ) 
    cerr << "Fcm::SetGFunction : don't have any concept named " << concept << " ."  << endl;
  else
  {
    SubValueVect * realVect=SubValue::getSubValueVect();
    Value * result=TmpValue::getTmpValue(context->getModule()->getResultType());
    SubValue * value;
    StlList<SubValue *> realList;
    StlList<SubValue *>::iterator realIter;
    for ( size_t i = 0 ; i < (size_t)obj->_gFunctions[i].GetParamCount() ; i++)
    {
      value=SubValue::getSubValue(ValueType(ValueType::REAL),result);
      value->setReal(obj->_gFunctions[i].GetParam(i));
      realList.push_back(value);
    }

    realVect->resizeIfNeeded(realList.size());
    realIter=realList.begin();
    for(size_t j=0;j<realList.size();j++,realIter++)
      realVect->pushBack(*realIter);
    
    result->setValueVector(realVect);
    context->returnValue(result);
  }  
}
  
//native void Fcm::SetDefaultGFct(string gFunction,float[] params);
void
NFcm::SetDefaultGFct(Context * context)
{
  NFcm * obj = (NFcm*)context->getOwner();
  obj->_defaultGFunction = context->getStrings(0);
  const SubValueVect * subValueVect = context->getValueVector(1);
  obj->_defaultGParams.clear();
  for (size_t i = 0 ; i < subValueVect->getSize() ; i ++ )
     obj->_defaultGParams.push_back( ((*subValueVect)[i])->getReal() );
}
  
//native string Fcm::GetDefaultGFct()
void
NFcm::GetDefaultGFct(Context * context)
{
  NFcm * obj = (NFcm*)context->getOwner();
  context->returnStrings(obj->_defaultGFunction);
}

//native float[] Fcm::GetDefaultGFctParams();
void
NFcm::GetDefaultGFctParams(Context * context)
{
  NFcm * obj = (NFcm*)context->getOwner();
 
  SubValueVect * realVect=SubValue::getSubValueVect();
  Value * result=TmpValue::getTmpValue(context->getModule()->getResultType());
  SubValue * value;
  StlList<SubValue *> realList;
  StlList<SubValue *>::iterator realIter;
  for ( size_t i = 0 ; i < obj->_defaultGParams.size() ; i++)
  {
    value=SubValue::getSubValue(ValueType(ValueType::REAL),result);
    value->setReal(obj->_defaultGParams[i]);
    realList.push_back(value);
  }

  realVect->resizeIfNeeded(realList.size());
  realIter=realList.begin();
  for(size_t j=0;j<realList.size();j++,realIter++)
    realVect->pushBack(*realIter);
    
  result->setValueVector(realVect);
  context->returnValue(result); 
}

//native void Fcm::SetNumberOfEvals(int count);
void
NFcm::SetNumberOfEvals(Context * context)
{
  NFcm * obj = (NFcm*)context->getOwner();
  obj->_numberOfEvals = context->getInteger(0);
}

//native int Fcm::GetNumberOfEvals();
void
NFcm::GetNumberOfEvals(Context * context)
{
  NFcm * obj = (NFcm*)context->getOwner();
  context->returnInteger(obj->_numberOfEvals);
}

//native void Fcm::addConcepts(string[] concept)
void 
NFcm::addConcepts(Context * context) {
  NFcm * obj = (NFcm*)context->getOwner();
  const SubValueVect * subValueVect = context->getValueVector(0);
  StringsVect vect;
  for (size_t i = 0 ; i < subValueVect->getSize() ; i ++ ) {
    vect.push_back( ((*subValueVect)[i])->getStrings() );
  }
  NFcm::_addConcepts( vect , obj ) ;

}

void 
NFcm::_addConcepts(StringsVect & vect,NFcm * obj) {
  size_t oldNbConcepts = obj->_nbConcepts ;
  for (size_t i = 0 ; i < vect.size() ; i ++ ) {
    Strings conceptName = vect[i];
    _normName( conceptName );
     if (  ! NFcm::_has(conceptName,obj) ) {
      obj->_conceptsName.push_back(conceptName);
      obj->_nbConcepts ++;
    }
    else
      cerr << "Fcm::addConcepts : " + conceptName + " is already a concept of the fcm" << endl; 
  }
  
  float * newConceptVector;
  float * newForcingVector;
  float * newMatrix;
  GFunction * newGFunctions;
  // resize different vector
  // create new space
  size_t n = obj->_nbConcepts * sizeof(float);
  newConceptVector = (float * )malloc ( n );
  newForcingVector = (float * )malloc ( n );
  newMatrix = (float * )malloc ( obj->_nbConcepts * n );
  newGFunctions = new GFunction[obj->_nbConcepts];
  // refill the new space
  memset ( newConceptVector , 0 , n );
  memset ( newForcingVector , 0 , n );
  memset ( newMatrix , 0 , n * obj->_nbConcepts );
  float * ptr = obj->_matrix;
  int widthDest = obj->_nbConcepts;
  for ( size_t i1 = 0 ; i1 < oldNbConcepts ; i1 ++ ){
    memcpy((void*)&newMatrix[i1 * widthDest] , (void*)&ptr[i1 * oldNbConcepts] , oldNbConcepts *
sizeof(float));
  }
  memcpy ( newConceptVector ,  obj->_conceptVector , oldNbConcepts * sizeof(float) );
  memcpy ( newForcingVector ,  obj->_forcingVector , oldNbConcepts * sizeof(float) );
 
  for ( size_t i2 = 0 ; i2< oldNbConcepts ; i2 ++ )
     newGFunctions[i2].affect(obj->_gFunctions[i2]);
  for ( size_t i3 = oldNbConcepts ; i3 < obj->_nbConcepts ; i3 ++ )
     newGFunctions[i3].init(obj->_defaultGFunction,obj->_defaultGParams);

  // free old vector
  if (oldNbConcepts != 0){
    free(obj->_conceptVector);
    free(obj->_forcingVector);
    free(obj->_matrix);
  }
  // swap pointer
  obj->_conceptVector = newConceptVector;
  obj->_forcingVector = newForcingVector;
  obj->_matrix = newMatrix;
  obj->_gFunctions = newGFunctions;
}

//native void Fcm::addConceptsFromFile(string filename)
void NFcm::addConceptsFromFile(Context * context) {
  NFcm * obj = (NFcm*)context->getOwner();
  Strings lex ;
  Strings fileName = getSimul()->locateFile(context->getStrings(0)); 
  ifstream * file = new ifstream( fileName );
  if ( !file->fail() )
  {
    bool endOfLine;
    enum Zone { none,concept,defaultg,gfunctions,connect };
    Zone zone = none;
    eatWhites(file);
    while (NFcm::getWord(file,lex,endOfLine))
    {
       if (lex.len()>=2 && lex.subString(0,2) ==(Strings)"//")
       {
         //cerr << fileName << " : commentaire" << endl;
         while (endOfLine==false)
	   NFcm::getWord(file,lex,endOfLine);
       }
       else if (lex ==(Strings)"<CONCEPT>")
       {
         //cerr << fileName << " : zone concept" << endl;
	 zone = concept;
	 continue;
       }
       else if (lex ==(Strings)"<DEFAULT_G>")
       {
         //cerr << fileName << " : zone defaultg" << endl;
	 zone = defaultg;
	 continue;
       }
       else if (lex ==(Strings)"<G_FUNCTIONS>")
       {
         //cerr << fileName << " : zone gfunctions" << endl;
	 zone = gfunctions;
	 continue;
       }
       else if (lex ==(Strings)"<CONNECT>")
       {
         //cerr << fileName << " : zone connect" << endl;
	 zone = connect;
	 continue;
       }
       else
       {
         StringsVect words;
	 words.push_back(lex);
         while (endOfLine==false)
	 {
	   NFcm::getWord(file,lex,endOfLine);
	   words.push_back(lex);
	 }
	
         switch (zone)
	 {
	 case none:
	   cerr << "Fcm::addConceptFromFile : syntax error in '" << fileName << "'" << endl;
	   break;
	 case defaultg:
	   {
	     obj->_defaultGFunction = words[0];
             obj->_defaultGParams.clear();
  	     for (size_t i = 1;i < words.size();i++)
                obj->_defaultGParams.push_back((float)atof(words[i]));
	     zone = none;
           }
	   break;
	 case concept:
	   // add concept List
    	   _addConcepts( words , obj );
	   break;
	 case gfunctions:
	   if (words.size()<2)
	     cerr << "Fcm::addConceptFromFile : syntax error in '" << fileName << "'" << endl;
	   else
	   {
	     Strings concept = words[0];
	     _normName(concept);
             Strings g = words[1];
	     vector<float> params;
  	     for (size_t i = 2;i < words.size();i++)
                params.push_back((float)atof(words[i]));
	     
             int index = _seek(concept,obj);
             if ( index < 0 ) 
                cerr << "Fcm::addConceptFromFile : don't have any concept named " << concept << " ."  << endl;
             else
                obj->_gFunctions[index].init(g,params);
           }
	   break;
	 case connect:
	   if (words.size()!=3)
	     cerr << "Fcm::addConceptFromFile : syntax error in '" << fileName << "'" << endl;
	   else
	   {
	     StringsVect fromConcepts;
             StringsVect toConcepts;
             float weight;
             fromConcepts.push_back(words[0]);
             toConcepts.push_back(words[1]);
	     weight = (float)atof(words[2]);
             // add connections      
             NFcm::_connect( fromConcepts , toConcepts , weight , obj );
           }
	   break;
	 }
       }
    }
  }
  else
    cerr << "Fcm::addConceptFromFile failed while opening '" << fileName << "'" << endl;

  delete file;
}


//native void Fcm::addRandomizedConceptsFromFile(string filename, float delta)
void NFcm::addRandomizedConceptsFromFile(Context * context) {
  NFcm * obj = (NFcm*)context->getOwner();
  Strings lex ;
  Strings fileName = getSimul()->locateFile(context->getStrings(0));
  float delta = context->getReal(1); 
  ifstream * file = new ifstream( fileName );
  if ( !file->fail() )
  {
    bool endOfLine;
    enum Zone { none,concept,defaultg,gfunctions,connect };
    Zone zone = none;
    eatWhites(file);
    while (NFcm::getWord(file,lex,endOfLine))
    {
       if (lex.len()>=2 && lex.subString(0,2) ==(Strings)"//")
       {
         //cerr << fileName << " : commentaire" << endl;
         while (endOfLine==false)
	   NFcm::getWord(file,lex,endOfLine);
       }
       else if (lex ==(Strings)"<CONCEPT>")
       {
         //cerr << fileName << " : zone concept" << endl;
	 zone = concept;
	 continue;
       }
       else if (lex ==(Strings)"<DEFAULT_G>")
       {
         //cerr << fileName << " : zone defaultg" << endl;
	 zone = defaultg;
	 continue;
       }
       else if (lex ==(Strings)"<G_FUNCTIONS>")
       {
         //cerr << fileName << " : zone gfunctions" << endl;
	 zone = gfunctions;
	 continue;
       }
       else if (lex ==(Strings)"<CONNECT>")
       {
         //cerr << fileName << " : zone connect" << endl;
	 zone = connect;
	 continue;
       }
       else
       {
         StringsVect words;
	 words.push_back(lex);
         while (endOfLine==false)
	 {
	   NFcm::getWord(file,lex,endOfLine);
	   words.push_back(lex);
	 }
	
         switch (zone)
	 {
	 case none:
	   cerr << "Fcm::addRandomizedConceptsFromFile : syntax error in '" << fileName << "'" << endl;
	   break;
	 case defaultg:
	   {
	     obj->_defaultGFunction = words[0];
             obj->_defaultGParams.clear();
  	     for (size_t i = 1;i < words.size();i++)
                obj->_defaultGParams.push_back((float)atof(words[i]));
	     zone = none;
           }
	   break;
	 case concept:
	   // add concept List
    	   _addConcepts( words , obj );
	   break;
	 case gfunctions:
	   if (words.size()<2)
	     cerr << "Fcm::addRandomizedConceptsFromFile : syntax error in '" << fileName << "'" << endl;
	   else
	   {
	     Strings concept = words[0];
	     _normName(concept);
             Strings g = words[1];
	     vector<float> params;
  	     for (size_t i = 2;i < words.size();i++)
                params.push_back((float)atof(words[i]));
	     
             int index = _seek(concept,obj);
             if ( index < 0 )
                cerr << "Fcm::addRandomizedConceptsFromFile : don't have any concept named " << concept << " ."  << endl;
             else
                obj->_gFunctions[index].init(g,params);
           }
	   break;
	 case connect:
	   if (words.size()!=3)
	     cerr << "Fcm::addRandomizedConceptsFromFile : syntax error in '" << fileName << "'" << endl;
	   else
	   {
	     StringsVect fromConcepts;
             StringsVect toConcepts;
             float weight;
             fromConcepts.push_back(words[0]);
             toConcepts.push_back(words[1]);
	     weight = (float)atof(words[2]);
             //cerr << words[0] << " " << words[1] << " " << words[2] << " " << endl;
             // add connections      
             NFcm::_connect( fromConcepts , toConcepts , weight +
                 (weight*delta*((double)rand()/(double)RAND_MAX-0.5)) , obj );
           }
	   break;
	 }
       }
    }
  }
  else
    cerr << "Fcm::addRandomizedConceptsFromFile failed while opening '" << fileName << "'" << endl;

  delete file;
}

void
NFcm::eatWhites(ifstream * file)
{
  char c;
  while (1)
  {
    c = file->peek();
    if (!file->good()) 
      break; 
    if ( (c == ' ') || (c == '\t') || (c == '\n') || (c == '\r') )
      file->get(c);
    else
      break;
  }
}

bool
NFcm::getWord(ifstream * file, Strings & str, bool& endOfLine) {
  endOfLine = false;
  str.raz();
  char c;
  while(1)
  {
    c = file->peek();
    if ( (c == (char)EOF) || (c == ' ') || (c == '\t') || (c == '\n') || (c == '\r') )
      break;
    else
      file->get(c); 
    str += c;
  }
  
  while (1)
  {
    c = file->peek();
    if (c == (char)EOF)
    {
      endOfLine = true;
      break;
    }
    else if (c == '\n')
    {
      file->get(c);
      endOfLine = true;
    }
    else if ((c == ' ') || (c == '\t') || (c == '\r'))
      file->get(c);
    else
      break;
  }

  return str.len()!=0;
}

//native void Fcm::removeConcepts(string[] concept) 
void NFcm::removeConcepts(Context * context) {
  NFcm * obj = (NFcm*)context->getOwner();
  
  const SubValueVect * subValueVect = context->getValueVector(0);
  size_t oldNbConcepts = obj->_nbConcepts ;
  bool * validConcept = new bool[oldNbConcepts];
  for (size_t i0 = 0 ; i0 < oldNbConcepts ; i0++) validConcept[i0]=true;

  for (size_t i = 0 ; i < subValueVect->getSize() ; i ++ ) {
    Strings conceptName = (*subValueVect)[i]->getStrings();
    _normName( conceptName );
    int i = NFcm::_seek(conceptName,obj);
    if ( i >= 0 ){
      vector<Strings>::iterator iter = obj->_conceptsName.begin();
      advance(iter,i);
      obj->_conceptsName.erase(iter);
      validConcept[i] = false;
      obj->_nbConcepts --;
    }
    else 
      cerr << "Fcm::removeConcepts : " + conceptName + " isn't a  concept of the fcm" << endl; 
  }

  if ( obj->_nbConcepts != oldNbConcepts ){
    float * newConceptVector;
    float * newForcingVector;
    float * newMatrix;
    GFunction * newGFunctions;
    // resize different vector
    // create new space
    newConceptVector = (float * )malloc ( obj->_nbConcepts * sizeof(float));
    newForcingVector = (float * )malloc ( obj->_nbConcepts * sizeof(float));
    newMatrix = (float * )malloc ( obj->_nbConcepts * obj->_nbConcepts * sizeof(float));
    newGFunctions = new GFunction[obj->_nbConcepts];
    // refill the new space
    int widthDest = obj->_nbConcepts;
    size_t i1d = 0;
    for ( size_t i1 = 0 ; i1 < oldNbConcepts ; i1 ++ ){
      if (validConcept[i1]){
        size_t i2d = 0;
        for ( size_t i2 = 0 ; i2 < oldNbConcepts ; i2 ++ ){
          if ( validConcept[i2]){
            newMatrix[i1d + i2d * widthDest] = obj->_matrix[i1 + i2 * oldNbConcepts];  
            i2d++;
          }     
        }
        newConceptVector[i1d ] = obj->_conceptVector[i1];
        newForcingVector[i1d ] = obj->_forcingVector[i1];
        newGFunctions[i1d].affect(obj->_gFunctions[i1]);
        i1d++;
      }
    }  
    // free old vector
    if (oldNbConcepts != 0){
      free(obj->_conceptVector);
      free(obj->_forcingVector);
      free(obj->_matrix);
      delete[] obj->_gFunctions;
    }
    // swap pointer
    obj->_conceptVector = newConceptVector;
    obj->_forcingVector = newForcingVector;
    obj->_matrix = newMatrix;
    obj->_gFunctions = newGFunctions;
    
    delete validConcept;
  }

}

//native void Fcm::connect(string[] from, string[] to, float weight)
void 
NFcm::connect(Context * context) {
  NFcm * obj = (NFcm*)context->getOwner();
  const SubValueVect * fromValueVect = context->getValueVector(0);
  const SubValueVect * toValueVect = context->getValueVector(1);
  float weight = context->getReal(2);

  StringsVect fromVect;
  StringsVect toVect;
  size_t i;
  for (i = 0 ; i < fromValueVect->getSize() ; i ++ ) {
    fromVect.push_back( ((*fromValueVect)[i])->getStrings() );
  }
  for (i = 0 ; i < toValueVect->getSize() ; i ++ ) {
    toVect.push_back( ((*toValueVect)[i])->getStrings() );
  }
  NFcm::_connect(fromVect,toVect,weight,obj);

}

void 
NFcm::_connect(StringsVect fromVect,StringsVect toVect,float weight,NFcm * obj) {
  if ( fromVect.size() == toVect.size() ){
    for (size_t index = 0 ; index < fromVect.size() ; index ++ ) {
      Strings normFrom = fromVect[index];
      Strings normTo = toVect[index];
      _normName( normFrom );
      _normName( normTo );   
      int i = _seek( normFrom , obj ) ;
      int j = _seek( normTo , obj ) ;
      if ( i < 0 )
        cerr <<  "Fcm::connect : " << normFrom << " isn't a concept of this FCM" << endl;
      else if ( j < 0 )
        cerr << "Fcm::connect : " << normTo << " isn't a concept of this FCM" << endl;
      else 
        obj->_matrix[j + obj->_nbConcepts * i ] = weight;
    }
  }
  else
    cerr << "Fcm::connect : 'from' array Must be of the same size of 'to' array" << endl;
}

//native void Fcm::disconnect(string[] from, string[] to) 
void 
NFcm::disconnect(Context * context) {
  NFcm * obj = (NFcm*)context->getOwner();
  const SubValueVect * fromValueVect = context->getValueVector(0);
  const SubValueVect * toValueVect = context->getValueVector(1);
  
  if ( fromValueVect->getSize() == toValueVect->getSize() ){
    for (size_t index = 0 ; index < fromValueVect->getSize() ; index ++ ) {
      int i = _seek( (Strings&)((*fromValueVect)[index])->getStrings() , obj ) ;
      int j = _seek( (Strings&)((*toValueVect)[index])->getStrings() , obj ) ;
      if ( i < 0 )
        cerr <<  "Fcm::disconnect : " << ((*fromValueVect)[index])->getStrings() << " isn't a concept of this FCM" << endl;
      else if ( j < 0 )
        cerr << "Fcm::disconnect : " << ((*toValueVect)[index])->getStrings() << "isn't a concept of this FCM" << endl;
      else 
        obj->_matrix[j + obj->_nbConcepts * i ] = 0;
    }
  }
  else
    cerr << "Fcm::disconnect : 'from' array Must be of the same size of 'to' array" << endl;

}

//native void Fcm::merge(Fcm c)
void 
NFcm::merge(Context * context) {
  NFcm * obj = (NFcm*)context->getOwner();
  NFcm * fcm = (NFcm*)context->getAgent(0);
  StringsVect vect;
  size_t i;
  for ( i = 0 ; i < fcm->_nbConcepts ; i ++ ){
    if ( !_has( fcm->_conceptsName[i], obj ) ){
      vect.push_back( fcm->_conceptsName[i] );
    }
  }
  _addConcepts(vect,obj);

  for ( i = 0 ; i < obj->_nbConcepts ; i ++ ){
    for ( size_t j = 0 ; j < obj->_nbConcepts ; j ++ ){
      obj->_matrix[j + obj->_nbConcepts * i ] =
NFcm::_getWeight(obj->_conceptsName[i],obj->_conceptsName[j],obj)
                                               +
NFcm::_getWeight(obj->_conceptsName[i],obj->_conceptsName[j],fcm) ;
    }
  }
}

bool NFcm::_has(Strings & conceptName , NFcm * obj){
  return ( NFcm::_seek(conceptName,obj) >= 0 );
}

int NFcm::_seek(Strings& conceptName, NFcm * obj) {
  int end = obj->_conceptsName.size();
  int i = 0;
  if ( !end )
    return -1;
  while( obj->_conceptsName[i] != conceptName ){
    i++;
    if ( i == end )
      return -1;
  }
  return i;
}

void 
NFcm::_normName(Strings & conceptName) {
  conceptName.toLower();
}

void
NFcm::_g(float * x , float * y , size_t size, NFcm * obj){
  for ( size_t i = 0 ; i < size ; i ++ ){
    y[i] = obj->_gFunctions[i].eval(x[i],y[i]);
    //y[i] = x[i] + y[i];
  }
}

float 
NFcm::_sigma(float x,float delta,float a0,float k){
  return ( (1+delta)/(1+exp(-k*(x-a0))) - delta );
} 

//native int[][] Fcm::cycle(int[] startValue, int[] forcing)
void 
NFcm::cycle(Context * /*context*/) {
  cerr << "cycle isn't implemented !" << endl;
}

//native void Fcm::main(void)
void
NFcm::main(Context * context)
{
  NFcm * obj = (NFcm*)context->getOwner();
  for (int i=0;i<obj->_numberOfEvals;i++)
     NFcm::_eval(obj);
}

//native void Fcm::eval(void)
void 
NFcm::eval(Context * context)
{
   NFcm::_eval((NFcm*)context->getOwner());
}

void
NFcm::_eval(NFcm * obj)
{
  float * result = new float[ obj->_nbConcepts ];
  float * resultLow = new float[ obj->_nbConcepts ];
  float * resultWidth = new float[ obj->_nbConcepts ];  
  memset ( resultLow , 0 , sizeof(float) * obj->_nbConcepts  ) ;
  memset ( resultWidth , 0 , sizeof(float) * obj->_nbConcepts  ) ;

  // sigma
  for (size_t i = 0 ; i <  obj->_nbConcepts ; i ++ ){
    result[i] = 0;
    for ( size_t j = 0 ; j < obj->_nbConcepts ; j++ ){
      float coef = obj->_matrix[ i + j * obj->_nbConcepts  ];
      if ( coef < 0 ) {
        resultLow[i] += coef;
        resultWidth[i] -= coef;
      }
      else{
        resultWidth[i] += coef;
      } 
      result[i] += obj->_conceptVector[j] * coef;
    }
  }

  // normalisation
  if ( obj->_normalisationBeforeForcing ) {
    if ( obj->_fuzzy ){
      if (  obj->_biValued ){  // Fuzzy mode
        for (size_t i = 0 ; i <  obj->_nbConcepts ; i ++ ){
          if ( resultWidth[i] )
            result[i] = ( result[i] - resultLow[i] ) / resultWidth[i] ;
        }
      }
      else if (  obj->_triValued ){
        for (size_t i = 0 ; i <  obj->_nbConcepts ; i ++ ){
          if ( resultWidth[i] )
            result[i] = ( 2 * ( result[i] - resultLow[i] ) / resultWidth[i] ) -1 ;
        }     
      }
    }
    else if ( obj->_biValued ){  // Crisp mode
      for (size_t i = 0 ; i <  obj->_nbConcepts ; i ++ ){
        if ( (( result[i] - resultLow[i] ) / resultWidth[i]) < 0.5 )
          result[i] = 0 ;
        else 
          result[i] = 1 ;
      }   
    }
    else{
      for (size_t i = 0 ; i <  obj->_nbConcepts ; i ++ ){
        if ( ( (2*( result[i] - resultLow[i] ) / resultWidth[i])-1) < -0.5 )
          result[i] = -1 ;
        else if ( ( (2*( result[i] - resultLow[i] ) / resultWidth[i])-1) < 0.5 )
          result[i] = 0 ;
        else
          result[i] = 1 ; 
      }   
    }
  }
  // g
  NFcm::_g(obj->_forcingVector, result, obj->_nbConcepts, obj);

  // normalisation
  if ( obj->_fuzzy ){
    if (  obj->_biValued ){
      for (size_t i = 0 ; i <  obj->_nbConcepts ; i ++ ){
        result[i] = NFcm::_sigma( result[i] , 0 , 0.5 , obj->_k );
      }
    }
    else if (  obj->_triValued ){
      for (size_t i = 0 ; i <  obj->_nbConcepts ; i ++ )
        result[i] = NFcm::_sigma( result[i] , 1 , 0 , obj->_k );
    }
  }
  else if ( obj->_biValued ){
    for (size_t i = 0 ; i <  obj->_nbConcepts ; i ++ ){
      if ( NFcm::_sigma( result[i] , 0 , 0.5 , obj->_k ) > 0.5 )
        result[i] = 1;
      else
        result[i] = 0;
    }
  }
  else
    for (size_t i = 0 ; i <  obj->_nbConcepts ; i ++ ){
      if ( NFcm::_sigma( result[i] , 1 , 0 , obj->_k ) <= -0.5 )
        result[i] = -1;
      else if  ( NFcm::_sigma( result[i] , 1 , 0 , obj->_k ) <= 0.5 )
        result[i] = 0;
      else
        result[i] = 1;
    }
  memcpy ( obj->_conceptVector , result , obj->_nbConcepts * sizeof(float) );
  delete result;
  delete resultLow;
  delete resultWidth;
}

//native void Fcm::display(void)
void 
NFcm::display(Context * context) {
  NFcm * obj = (NFcm*)context->getOwner();
  size_t i;
  size_t j;
  cerr << endl;
  for ( i = 0 ; i < obj->_conceptsName.size() ; i ++ )
    cerr << obj->_conceptsName[i] << " ";
  cerr << endl;
  
  for ( i = 0 ; i < obj->_nbConcepts ; i ++ ){
    for ( j = 0 ; j < obj->_nbConcepts ; j ++ ){
      cerr << obj->_matrix[j + i * obj->_nbConcepts] << " " ;
    }
    cerr << endl;
  }
}

//native string Fcm::_statusTextValues(int i) 
void 
NFcm::_statusTextValues(Context * context) {
  NFcm * obj = (NFcm*)context->getOwner();
  size_t i = context->getInteger(0);
  if ( i < obj->_nbConcepts )
    context->returnStrings( obj->_conceptsName[i] );
  else{
    cerr << "Fcm::_statusTextValues : argument must be inferior to concept number" << endl;
    context->returnStrings( (Strings)"" );
  }
}

//native float FCM::_getValues(int i)
void 
NFcm::_getValues(Context * context){
  NFcm * obj = (NFcm*)context->getOwner();
  size_t i = context->getInteger(0);
  context->returnReal(NFcm::_getValue(i,obj));
}

float
NFcm::_getValue(size_t i , NFcm * obj)
{
  if ( i < obj->_nbConcepts )
    return obj->_conceptVector[i];
  else
  {
    cerr << "Fcm::_getValue : argument must be inferior to concept number" << endl;
    return 0;
  }
}

//native float FCM::_getForcing(int i)
void 
NFcm::_getForcing(Context * context){
  NFcm * obj = (NFcm*)context->getOwner();
  size_t i = context->getInteger(0);
  if ( i < obj->_nbConcepts )
    context->returnReal( obj->_forcingVector[i] );
  else{
    cerr << "Fcm::_getForcing : argument must be inferior to concept number" << endl;
    context->returnReal( 0 );
  }
}

//native void FCM::setValues(float[] values)
void 
NFcm::setValues(Context * context){
  NFcm * obj = (NFcm*)context->getOwner();
  const SubValueVect * valueVect = context->getValueVector(0);
  if ( valueVect->getSize() != (size_t)obj->_nbConcepts ){
    cerr << "Fcm::setValues : value array must have the same size as concepts set" << endl;
  }
  else {
    for (size_t i = 0 ; i < valueVect->getSize() ; i ++ ){
      obj->_conceptVector[i] = ((*valueVect)[i])->getReal();
    }  
  }
}

//native void FCM::setValue(string concept,float values)
void 
NFcm::setValue(Context * context){
  NFcm * obj = (NFcm*)context->getOwner();
  Strings concept = context->getStrings(0);
  float value = context->getReal(1);
  int  i = _seek(concept,obj);
  if ( i < 0 ) 
    cerr << "Fcm::setValue : don't have any concept named " << concept << " ." ;
  else
    obj->_conceptVector[i] = value;;
}

//native float FCM::getValue(string concept)
void 
NFcm::getValue(Context * context){
  NFcm * obj = (NFcm*)context->getOwner();
  Strings concept = context->getStrings(0);
  int  i = _seek(concept,obj);
  if ( i < 0 ){ 
    cerr << "Fcm::getValue : don't have any concept named " << concept << " ." ;
    context->returnReal(0);
  }
  else
    context->returnReal(obj->_conceptVector[i]) ;
  
}

//native void FCM::setNamedForcing(string concept,float[] values)
void 
NFcm::setNamedForcing(Context * context){
  NFcm * obj = (NFcm*)context->getOwner();
  Strings concept = context->getStrings(0);
  float value = context->getReal(1);
  int  i = _seek(concept,obj);
  if ( i < 0 ) 
    cerr << "Fcm::setNamedForcing : don't have any concept named " << concept << " ." ;
  else
    obj->_forcingVector[i] = value;;
}

//native float FCM::getNamedForcing(string concept)
void 
NFcm::getNamedForcing(Context * context){
  NFcm * obj = (NFcm*)context->getOwner();
  Strings concept = context->getStrings(0);
  int  i = _seek(concept,obj);
  if ( i < 0 ){ 
    cerr << "Fcm::getNamedForcing : don't have any concept named " << concept << " ." ;
    context->returnReal(0);
  }
  else
    context->returnReal(obj->_forcingVector[i]) ;
  
}

//native void FCM::setForcing(float[] values)
void 
NFcm::setForcing(Context * context){
  NFcm * obj = (NFcm*)context->getOwner();
  const SubValueVect * forcingVect = context->getValueVector(0);
  if ( forcingVect->getSize() != (size_t)obj->_nbConcepts ){
    cerr << "Fcm::setForcing : forcing value array must have the same size as concepts set" << endl;
  }
  else {
    for (size_t i = 0 ; i < forcingVect->getSize() ; i ++ ){
      obj->_forcingVector[i] = ((*forcingVect)[i])->getReal();
    }  
  }
}

//native bool Fcm::has(string concept)
void 
NFcm::has(Context * context) {
  context->returnInteger( NFcm::_has( (Strings&)context->getStrings(0)  , (NFcm*)context->getOwner() ) ) ;  
}
//native float Fcm::getWeight(string from , string to)
void 
NFcm::getWeight(Context * context) {
  NFcm * obj = (NFcm*)context->getOwner();
  Strings from;
  Strings to;
  from = context->getStrings(0);
  to = context->getStrings(1);
  int i = _seek( from , obj ) ;
  int j = _seek( to , obj ) ;
  if ( i < 0 ){
    cerr << "Fcm::getWeight : " << from << " isn't a concept of this Fcm." << endl;
    context->returnReal( 0 ) ;
  }
  else if ( j < 0 ){
    cerr << "Fcm::getWeight : " << to << " isn't a concept of this Fcm." << endl;
    context->returnReal( 0 ) ;
  }
  else
    context->returnReal( NFcm::_getWeight(from,to,obj) );
}

float 
NFcm::_getWeight(Strings & from, Strings & to , NFcm * obj ) {
  int i = _seek( from , obj ) ;
  int j = _seek( to , obj ) ;
  if ( i < 0 )
    return ( 0 ) ;
  else if ( j < 0 )
    return ( 0 ) ;
  else{ 
    return ( obj->_matrix[j + obj->_nbConcepts * i ] ) ;
  }
}

//native int Fcm::getSize(void)
void 
NFcm::getSize(Context * context) {
  context->returnInteger( ((NFcm*)context->getOwner())->_nbConcepts  ) ;  
}

void 
NFcm::getName(Context * context) {
  context->returnStrings( ((NFcm*)context->getOwner())->_name  ) ;  
}

//native void Fcm::setDelta(float delta)
void 
NFcm::setDelta(Context * context) {
  NFcm * obj = (NFcm*)context->getOwner();
  obj->_delta = context->getReal(0);
} 
//native void Fcm::setA0(float a)
void 
NFcm::setA0(Context * context){
  NFcm * obj = (NFcm*)context->getOwner();
  obj->_a0 = context->getReal(0);
}

//native float Fcm::getDelta(void)
void 
NFcm::getDelta(Context * context) {
  NFcm * obj = (NFcm*)context->getOwner();
  context->returnReal( obj->_delta ) ;
} 
//native float Fcm::getA0(void)
void 
NFcm::getA0(Context * context){
  NFcm * obj = (NFcm*)context->getOwner();
  context->returnReal ( obj->_a0 ) ;
}

//native string[] Fcm::statusTextValues(void)
void
NFcm::statusTextValues(Context * context)
{
  NFcm * obj = (NFcm*)context->getOwner();
  SubValueVect * strVect=SubValue::getSubValueVect();
  Value * result=TmpValue::getTmpValue(context->getModule()->getResultType());
  SubValue * value;
  StlList<SubValue *> strList;
  StlList<SubValue *>::iterator strIter;
  
  for ( size_t i = 0 ; i < obj->_nbConcepts ; i++)
  {
     value=SubValue::getSubValue(ValueType(ValueType::STRING),result);
     value->setStrings(obj->_conceptsName[i]);
     strList.push_back(value);
  }
  
  strVect->resizeIfNeeded(strList.size());
  strIter=strList.begin();
  for(size_t j=0;j<strList.size();j++,strIter++)
    strVect->pushBack(*strIter);
    
  result->setValueVector(strVect);
  context->returnValue(result);
}

//native float[] Fcm::getValues(void)
void
NFcm::getValues(Context * context)
{
  NFcm * obj = (NFcm*)context->getOwner();
  SubValueVect * realVect=SubValue::getSubValueVect();
  Value * result=TmpValue::getTmpValue(context->getModule()->getResultType());
  SubValue * value;
  StlList<SubValue *> realList;
  StlList<SubValue *>::iterator realIter;
  
  for ( size_t i = 0 ; i < obj->_nbConcepts ; i++)
  {
     value=SubValue::getSubValue(ValueType(ValueType::REAL),result);
     value->setReal(obj->_conceptVector[i]);
     realList.push_back(value);
  }
  
  realVect->resizeIfNeeded(realList.size());
  realIter=realList.begin();
  for(size_t j=0;j<realList.size();j++,realIter++)
    realVect->pushBack(*realIter);
    
  result->setValueVector(realVect);
  context->returnValue(result);
}
  
//native float[] Fcm::getForcing()
void
NFcm::getForcing(Context * context)
{
  NFcm * obj = (NFcm*)context->getOwner();
  SubValueVect * realVect=SubValue::getSubValueVect();
  Value * result=TmpValue::getTmpValue(context->getModule()->getResultType());
  SubValue * value;
  StlList<SubValue *> realList;
  StlList<SubValue *>::iterator realIter;
  
  for ( size_t i = 0 ; i < obj->_nbConcepts ; i++)
  {
     value=SubValue::getSubValue(ValueType(ValueType::REAL),result);
     value->setReal(obj->_forcingVector[i]);
     realList.push_back(value);
  }
  
  realVect->resizeIfNeeded(realList.size());
  realIter=realList.begin();
  for(size_t j=0;j<realList.size();j++,realIter++)
    realVect->pushBack(*realIter);
    
  result->setValueVector(realVect);
  context->returnValue(result);
}

//native string[] Fcm::getTextWeight(void)
void
NFcm::getTextWeight(Context * context)
{
  NFcm * obj = (NFcm*)context->getOwner();
  SubValueVect * strVect=SubValue::getSubValueVect();
  Value * result=TmpValue::getTmpValue(context->getModule()->getResultType());
  SubValue * value;
  StlList<SubValue *> strList;
  StlList<SubValue *>::iterator strIter;
  
  for ( size_t i = 0 ; i < obj->_nbConcepts ; i++)
  {
    for ( size_t j = 0 ; j < obj->_nbConcepts ; j++)
    {
       value=SubValue::getSubValue(ValueType(ValueType::STRING),result);
       value->setStrings(Strings(obj->_matrix[i*obj->_nbConcepts+j]));
       strList.push_back(value);
    }
  }
  
  strVect->resizeIfNeeded(strList.size());
  strIter=strList.begin();
  for(size_t j=0;j<strList.size();j++,strIter++)
    strVect->pushBack(*strIter);
    
  result->setValueVector(strVect);
  context->returnValue(result);
}

//native void Fcm::setTextWeight(string[] str)
void
NFcm::setTextWeight(Context * context)
{
  NFcm * obj = (NFcm*)context->getOwner();
  const SubValueVect * subValueVect = context->getValueVector(0);
  StringsList vect;
  for (size_t ii = 0 ; ii < subValueVect->getSize() ; ii ++ )
    vect.push_back( ((*subValueVect)[ii])->getStrings() );
  
  size_t size = obj->_nbConcepts;
  for ( size_t i = 0 ; i < size ; i++)
  {
    for ( size_t j = 0 ; j < size ; j++)
    {
      obj->_matrix[i*obj->_nbConcepts+j] = atof(vect.front());
      vect.pop_front();
    }
  }
}

//*********************************************************
//
//
// class GFunction
//
//
//*********************************************************

GFunction::GFunction()
{
   _name = "";
   _gFunction = NULL;
   _pOpenLib = NULL;
   _params = NULL;
   _count = 0;
}

GFunction::~GFunction()
{ 
   _clean();
}

void
GFunction::_clean()
{
   if (_pOpenLib != NULL)
   {
      dlclose(_pOpenLib);
      _pOpenLib = NULL;
   }
   if (_params != NULL)
   {
      delete[] _params;
      _params = NULL;
      _count = 0;
   }
}

void
GFunction::init(Strings name,vector<float> params)
{
  _clean();

  _name = name;

  if (_name==Strings("min"))
     _gFunction = GFunction::_min;
  else if (_name==Strings("max"))
     _gFunction = GFunction::_max;
  else if (_name==Strings("sum"))
     _gFunction = GFunction::_sum;
  else if (_name==Strings("punderateSum"))
     _gFunction = GFunction::_punderateSum;
  else if (_name==Strings("multiplication"))
     _gFunction = GFunction::_multiplication;
  else
  {
     Strings lib;
     Strings fonction;
     int index = _name.lookUp(Strings("."));
     if (index <= 0 || index >= static_cast<int>(_name.len())-1)
	cerr << name << " isn't a correct name for a g function." << endl;
     else
     {
        lib = _name.subString(0,index);
        fonction = _name.subString(index+1,name.len()-index-1);
        void* pLib = dlopen("lib" + lib + ".so",RTLD_LAZY);
        if (pLib == NULL)
	   cerr << "library " << lib << ".so couldn't be loaded !" << endl;
        else
        {
           _gFunction = (PFUNCTION)dlsym(pLib,fonction);  
           if (_gFunction == NULL)
           {
              dlclose(pLib);
              cerr << fonction << " in library " << lib << ".so couldn't be loaded !" << endl;
           }
           else
	      // the function exist !!!
              _pOpenLib = pLib;
        }
     }
  }

  if (!params.empty())
  {
     _count = params.size();
     _params = new float[_count];
     for (int i=0;i<_count;i++)
        _params[i] = params[i];
  }
}

void
GFunction::affect(GFunction& src)
{
   _clean();
   _name = src._name;
   _gFunction = src._gFunction;
   _count = src._count;
   if (_count!=0)
   {
      _params = new float[_count];
      for (int i=0;i<_count;i++)
         _params[i] = src._params[i];
   }
}

float
GFunction::eval(float x,float y)
{
   if (_gFunction != NULL)
      return _gFunction(x,y,_params,_count);
   else
   {
      cerr << "The g function couldn't be evaluate !!!" << endl;
      return 0;
   }
}

float
GFunction::_min(float x,float y,float* /*params*/,int /*count*/)
{
   return x<y?x:y;
}

float
GFunction::_max(float x,float y,float* /*params*/,int /*count*/)
{
   return x<y?y:x;
}

float
GFunction::_sum(float x,float y,float* /*params*/,int /*count*/)
{
   return x+y;
}

float
GFunction::_punderateSum(float x,float y,float* params,int /*count*/)
{
   return params[0]*x+params[1]*y;
}

float
GFunction::_multiplication(float x,float y,float* params,int count)
{
   if (count == 1)
      cout << "[" << params[0] << "] " << x << " * " << y << " = " << x*y << endl;
   return x*y;
}
//---------------------------------------------------------------------------
 
