// @(#) ENIB/LI2 (c) - oRis - Version 2.3
//--------------------------------------------------------------------------//
// file        : filesPkg.h                                                 //
// description : header file for the ENIB/LI2 oRis package files.pkg        //
// author(s)   : F. Harrouet                                                //
// copyright   : Laboratoire d'Informatique Industrielle (LI2)              //
//               Ecole Nationale d'Ingenieurs de Brest (ENIB)               //
//               CP 30815 - 29608 Brest Cedex - FRANCE                      //
//               Tel: (33) 02.98.05.66.31 , Fax: (33) 02.98.05.66.29        //
//               e-mail: li2@enib.fr                                        //
//--------------------------------------------------------------------------//
// Copyright (c) 1996-2000                                                  //
// Laboratoire d'Informatique Industrielle (LI2)                            //
// Ecole Nationale d'Ingenieurs de Brest (ENIB)                             //
//                                                                          //
// Permission to use, copy, modify, distribute and sell this software       //
// and its documentation for any purpose is hereby granted without fee,     //
// provided that the above copyright notice appear in all copies and        //
// that both that copyright notice and this permission notice appear        //
// in supporting documentation.                                             //
// ENIB/LI2 makes no representations about the suitability of this          //
// software for any purpose.                                                //
// It is provided "as is" without express or implied warranty.              //
//--------------------------------------------------------------------------//

#ifndef FILESPKG_H
#define FILESPKG_H

#include "oRis/Main/agent.h"

#include <stdio.h>

//--------------------------------------------------------------------------//
// class InputFile                                                          //
//--------------------------------------------------------------------------//
 
class InputFile : public Agent
{
public:
  InputFile(Model * model,int instNum);
  virtual ~InputFile(void);
  virtual Strings userSave(int column=0) const;
 
  // native void InputFile::close(void);
  //
  static void close(Context * context);
 
  // native strings InputFile::fileName(void);
  //
  static void fileName(Context * context);
 
  // native bool InputFile::fail(void);
  //
  static void fail(Context * context);
 
  // native bool InputFile::eof(void);
  //
  static void eof(Context * context);

//---- Attributes ----
protected:
  Strings _fileName;
  FILE * _file;
};


//--------------------------------------------------------------------------//
// class TextInputFile                                                      //
//--------------------------------------------------------------------------//
 
class TextInputFile : public InputFile
{
public:
  TextInputFile(Model * model,int instNum);
  virtual ~TextInputFile(void);
 
  // native bool TextInputFile::open(string fileName);
  //
  static void open(Context * context);
 
  // native void TextInputFile::eatBlanks(void);
  //
  static void eatBlanks(Context * context);
 
  // native string TextInputFile::getContent(void);
  //
  static void getContent(Context * context);
 
  // native int TextInputFile::readInt(void);
  //
  static void readInt(Context * context);

  // native float TextInputFile::readFloat(void);
  //
  static void readFloat(Context * context);
 
  // native string TextInputFile::readString(int stringLength);
  //
  static void readString(Context * context);
 
  // native string TextInputFile::readLine(void);
  //
  static void readLine(Context * context);
 
  // native string TextInputFile::readWord(void);
  //
  static void readWord(Context * context);
 
  // native Object TextInputFile::readObject(void);
  //
  static void readObject(Context * context);
};


//--------------------------------------------------------------------------//
// class BinaryInputFile                                                    //
//--------------------------------------------------------------------------//
 
class BinaryInputFile : public InputFile
{
public:
  BinaryInputFile(Model * model,int instNum);
  virtual ~BinaryInputFile(void);
 
  // native bool BinaryInputFile::open(string fileName);
  //
  static void open(Context * context);
 
  // native int BinaryInputFile::readByte(void);
  //
  static void readByte(Context * context);
 
  // native int BinaryInputFile::readInt16(void);
  //
  static void readInt16(Context * context);
 
  // native int BinaryInputFile::readInt32(void);
  //
  static void readInt32(Context * context);
 
  // native float BinaryInputFile::readFloat32(void);
  //
  static void readFloat32(Context * context);
 
  // native float BinaryInputFile::readFloat64(void);
  //
  static void readFloat64(Context * context);
 
  // native string BinaryInputFile::readString(int stringLength);
  //
  static void readString(Context * context);
 
  // native int BinaryInputFile::readBlock(Memory dest,int offset,int length);
  //
  static void readBlock(Context * context);
};


//--------------------------------------------------------------------------//
// class OutputFile                                                         //
//--------------------------------------------------------------------------//
 
class OutputFile : public Agent
{
public:
  OutputFile(Model * model,int instNum);
  virtual ~OutputFile(void);
  virtual Strings userSave(int column=0) const;
 
  // native void OutputFile::close(void);
  //
  static void close(Context * context);
 
  // native strings OutputFile::fileName(void);
  //
  static void fileName(Context * context);
 
  // native bool OutputFile::fail(void);
  //
  static void fail(Context * context);
 
//---- Attributes ----
protected:
  Strings _fileName;
  FILE * _file;
  bool _append;
};


//--------------------------------------------------------------------------//
// class TextOutputFile                                                     //
//--------------------------------------------------------------------------//

class TextOutputFile : public OutputFile
{
public:
  TextOutputFile(Model * model,int instNum);
  virtual ~TextOutputFile(void);

  // native bool TextOutputFile::open(string fileName);
  //
  static void open(Context * context);

  // native bool TextOutputFile::openAppend(string fileName);
  //
  static void openAppend(Context * context);

  // native void TextOutputFile::writeInt(int intValue);
  //
  // native void TextOutputFile::writeFloat(float floatValue);
  //
  // native void TextOutputFile::writeString(string stringValue);
  //
  // native void TextOutputFile::writeObject(Object objectValue);
  //
  static void writeValue(Context * context);
};


//--------------------------------------------------------------------------//
// class BinaryOutputFile                                                   //
//--------------------------------------------------------------------------//
 
class BinaryOutputFile : public OutputFile
{
public:
  BinaryOutputFile(Model * model,int instNum);
  virtual ~BinaryOutputFile(void);
 
  // native bool BinaryOutputFile::open(string fileName);
  //
  static void open(Context * context);
 
  // native bool BinaryOutputFile::openAppend(string fileName);
  //
  static void openAppend(Context * context);
 
  // native void BinaryOutputFile::writeByte(int byteValue);
  //
  static void writeByte(Context * context);

  // native void BinaryOutputFile::writeInt16(int int16Value);
  //
  static void writeInt16(Context * context);

  // native void BinaryOutputFile::writeInt32(int int32Value);
  //
  static void writeInt32(Context * context);

  // native void BinaryOutputFile::writeFloat32(float float32Value);
  //
  static void writeFloat32(Context * context);

  // native void BinaryOutputFile::writeFloat64(float float64Value);
  //
  static void writeFloat64(Context * context);

  // native void BinaryOutputFile::writeString(string stringValue);
  //
  static void writeString(Context * context);

  // native void BinaryOutputFile::writeBlock(Memory src,
  //                                          int offset,int length);
  //
  static void writeBlock(Context * context);
};

#endif // FILESPKG_H

