// @(#) ENIB/LI2 (c) - oRis - Version 2.3
//--------------------------------------------------------------------------//
// file        : guiPkg.h                                                   //
// description : header file for the ENIB/LI2 oRis package gui.pkg          //
// author(s)   : F. Harrouet                                                //
// copyright   : Laboratoire d'Informatique Industrielle (LI2)              //
//               Ecole Nationale d'Ingenieurs de Brest (ENIB)               //
//               CP 30815 - 29608 Brest Cedex - FRANCE                      //
//               Tel: (33) 02.98.05.66.31 , Fax: (33) 02.98.05.66.29        //
//               e-mail: li2@enib.fr                                        //
//--------------------------------------------------------------------------//
// Copyright (c) 1996-2000                                                  //
// Laboratoire d'Informatique Industrielle (LI2)                            //
// Ecole Nationale d'Ingenieurs de Brest (ENIB)                             //
//                                                                          //
// Permission to use, copy, modify, distribute and sell this software       //
// and its documentation for any purpose is hereby granted without fee,     //
// provided that the above copyright notice appear in all copies and        //
// that both that copyright notice and this permission notice appear        //
// in supporting documentation.                                             //
// ENIB/LI2 makes no representations about the suitability of this          //
// software for any purpose.                                                //
// It is provided "as is" without express or implied warranty.              //
//--------------------------------------------------------------------------//

#ifndef GUIPKG_H
#define GUIPKG_H

#include "oRis/Main/agent.h"
#include "guiTrans.h"

//--------------------------------------------------------------------------//
// Gui functions                                                            //
//--------------------------------------------------------------------------//

// native int guiScreenWidth(void);
//
void guiScreenWidth(Context * context);

// native int guiScreenHeight(void);
//
void guiScreenHeight(Context * context);

// native int guiTextWidth(string text);
//
void guiTextWidth(Context * context);

// native int guiTextHeight(void);
//
void guiTextHeight(Context * context);

// native int guiMargin(void);
//
void guiMargin(Context * context);

// native void guiBell(void);
//
void guiBell(Context * context);

//--------------------------------------------------------------------------//
// class OGuiElem                                                           //
//--------------------------------------------------------------------------//

class OGuiElem : public Agent
{
public:
  OGuiElem(Model * model,int instNum);
  virtual ~OGuiElem(void);
  virtual Strings userSave(int column=0) const;
  void setParent(OGuiElem * parent);
  static void callOrisConstructor(Context * context);
  static void destroyCB(GuiElement * calling,void * data);
  static void orisCB(GuiElement * calling,void * data);
  inline const GuiElement * getElem(void) const;
  inline GuiElement * accessElem(void);

  // native void GuiElem::_delete(void);
  //
  static void destruct(Context * context);

  // native int GuiElem::getX(void);
  //
  static void getX(Context * context);

  // native int GuiElem::getY(void);
  //
  static void getY(Context * context);

  // native int GuiElem::getWidth(void);
  //
  static void getWidth(Context * context);

  // native int GuiElem::getHeight(void);
  //
  static void getHeight(Context * context);

  // native string GuiElem::getTitle(void);
  //
  static void getTitle(Context * context);

  // native void GuiElem::setX(int x);
  //
  static void setX(Context * context);

  // native void GuiElem::setY(int y);
  //
  static void setY(Context * context);

  // native void GuiElem::setWidth(int width);
  //
  static void setWidth(Context * context);

  // native void GuiElem::setHeight(int height);
  //
  static void setHeight(Context * context);

  // native void GuiElem::setTitle(string title);
  //
  static void setTitle(Context * context);

  // native bool GuiElem::dump(string fileName);
  //
  static void dump(Context * context);

//---- Attributes ----
protected:
  GuiElement * _elem;
  static Word _childrenWord;
  static Word _parentWord;
  static Word _object3dWord;
  static Word _object2dWord;
  static Word _callbackWord;
  static Word _closeCallbackWord;
  static Word _keyPressWord;
};

inline const GuiElement * OGuiElem::getElem(void) const
{
return(_elem);
}

inline GuiElement * OGuiElem::accessElem(void)
{
return(_elem);
}

//--------------------------------------------------------------------------//
// class OGuiArea                                                           //
//--------------------------------------------------------------------------//

class OGuiArea : public OGuiElem
{
public:
  OGuiArea(Model * model,int instNum);
  virtual ~OGuiArea(void);
  virtual Strings userSave(int column=0) const;

  // native void GuiArea::new(GuiArea parent);
  //
  static void construct(Context * context);

  // native int GuiArea::getWindowID(void);
  //
  static void getWindowID(Context * context);

  // native int GuiArea::getInWidth(void);
  //
  static void getInWidth(Context * context);

  // native int GuiArea::getInHeight(void);
  //
  static void getInHeight(Context * context);

  // native void GuiArea::setInWidth(int inWidth);
  //
  static void setInWidth(Context * context);

  // native void GuiArea::setInHeight(int inHeight);
  //
  static void setInHeight(Context * context);
};

//--------------------------------------------------------------------------//
// class OGuiScrolledArea                                                   //
//--------------------------------------------------------------------------//

class OGuiScrolledArea : public OGuiArea
{
public:
  OGuiScrolledArea(Model * model,int instNum);
  virtual ~OGuiScrolledArea(void);
  virtual Strings userSave(int column=0) const;

  // native void GuiScrolledArea::new(GuiArea parent);
  //
  static void construct(Context * context);
};

//--------------------------------------------------------------------------//
// class OGuiHScrolledArea                                                  //
//--------------------------------------------------------------------------//

class OGuiHScrolledArea : public OGuiScrolledArea
{
public:
  OGuiHScrolledArea(Model * model,int instNum);
  virtual ~OGuiHScrolledArea(void);
  virtual Strings userSave(int column=0) const;

  // native void GuiHScrolledArea::new(GuiArea parent);
  //
  static void construct(Context * context);
};

//--------------------------------------------------------------------------//
// class OGuiVScrolledArea                                                  //
//--------------------------------------------------------------------------//

class OGuiVScrolledArea : public OGuiScrolledArea
{
public:
  OGuiVScrolledArea(Model * model,int instNum);
  virtual ~OGuiVScrolledArea(void);
  virtual Strings userSave(int column=0) const;

  // native void GuiVScrolledArea::new(GuiArea parent);
  //
  static void construct(Context * context);
};

//--------------------------------------------------------------------------//
// class OGuiColorArea                                                      //
//--------------------------------------------------------------------------//
 
class OGuiColorArea : public OGuiArea
{
public:
  OGuiColorArea(Model * model,int instNum);
  virtual ~OGuiColorArea(void);
  virtual Strings userSave(int column=0) const;
 
  // native void GuiColorArea::new(GuiArea parent);
  //
  static void construct(Context * context);
 
  // native string GuiColorArea::getColor(void);
  //
  static void getColor(Context * context);
 
  // native void GuiColorArea::setColor(string color);
  //
  static void setColor(Context * context);
};

//--------------------------------------------------------------------------//
// class OGuiWindow                                                         //
//--------------------------------------------------------------------------//

class OGuiWindow : public OGuiArea
{
public:
  OGuiWindow(Model * model,int instNum);
  virtual ~OGuiWindow(void);
  virtual Strings userSave(int column=0) const;
  static void orisCloseCB(GuiElement * calling,void * data);

  // native void GuiWindow::new(string title);
  //
  static void construct(Context * context);

  // native bool GuiWindow::isVisible(void);
  //
  static void isVisible(Context * context);

  // native void GuiWindow::show(void);
  //
  static void show(Context * context);

  // native void GuiWindow::hide(void);
  //
  static void hide(Context * context);
};

//--------------------------------------------------------------------------//
// class OGuiFrame                                                          //
//--------------------------------------------------------------------------//

class OGuiFrame : public OGuiArea
{
public:
  OGuiFrame(Model * model,int instNum);
  virtual ~OGuiFrame(void);
  virtual Strings userSave(int column=0) const;

  // native void GuiFrame::new(GuiArea parent,string title);
  //
  static void construct(Context * context);
};

//--------------------------------------------------------------------------//
// class OGuiTabGroup                                                       //
//--------------------------------------------------------------------------//

class OGuiTabGroup : public OGuiElem
{
public:
  OGuiTabGroup(Model * model,int instNum);
  virtual ~OGuiTabGroup(void);
  virtual Strings userSave(int column=0) const;

  // native void GuiTabGroup::new(GuiArea parent);
  //
  static void construct(Context * context);

  // native int GuiTabGroup::getCurrentTab(void);
  //
  static void getCurrentTab(Context * context);

  // native int GuiTabGroup::getNbTabs(void);
  //
  static void getNbTabs(Context * context);

  // native string GuiTabGroup::getTab(int pos);
  //
  static void getTab(Context * context);

  // native void GuiTabGroup::setCurrentTab(int pos,bool propagate);
  //
  static void setCurrentTab(Context * context);
};

//--------------------------------------------------------------------------//
// class OGuiTab                                                            //
//--------------------------------------------------------------------------//

class OGuiTab : public OGuiArea
{
public:
  OGuiTab(Model * model,int instNum);
  virtual ~OGuiTab(void);
  virtual Strings userSave(int column=0) const;

  // native void GuiTab::new(GuiTabGroup parent,string title);
  //
  static void construct(Context * context);
};

//--------------------------------------------------------------------------//
// class OGuiMenu                                                           //
//--------------------------------------------------------------------------//

class OGuiMenu : public OGuiElem
{
public:
  OGuiMenu(Model * model,int instNum);
  virtual ~OGuiMenu(void);
  virtual Strings userSave(int column=0) const;

  // native void GuiMenu::new(GuiMenuItem parent,bool horizontal);
  //
  static void construct(Context * context);
};

//--------------------------------------------------------------------------//
// class OGuiMenuBar                                                        //
//--------------------------------------------------------------------------//

class OGuiMenuBar : public OGuiMenu
{
public:
  OGuiMenuBar(Model * model,int instNum);
  virtual ~OGuiMenuBar(void);
  virtual Strings userSave(int column=0) const;

  // native void GuiMenuBar::new(GuiArea parent,
  //                             bool horizontal,bool pushLast);
  //
  static void construct(Context * context);
};

//--------------------------------------------------------------------------//
// class OGuiMenuItem                                                       //
//--------------------------------------------------------------------------//

class OGuiMenuItem : public OGuiElem
{
public:
  OGuiMenuItem(Model * model,int instNum);
  virtual ~OGuiMenuItem(void);
  virtual Strings userSave(int column=0) const;

  // native void GuiMenuItem::new(GuiMenu parent,string title);
  //
  static void construct(Context * context);
};

//--------------------------------------------------------------------------//
// class OGuiMenuSepar                                                      //
//--------------------------------------------------------------------------//

class OGuiMenuSepar : public OGuiElem
{
public:
  OGuiMenuSepar(Model * model,int instNum);
  virtual ~OGuiMenuSepar(void);
  virtual Strings userSave(int column=0) const;

  // native void GuiMenuSepar::new(GuiMenu parent);
  //
  static void construct(Context * context);
};

//--------------------------------------------------------------------------//
// class OGuiLabel                                                          //
//--------------------------------------------------------------------------//

class OGuiLabel : public OGuiElem
{
public:
  OGuiLabel(Model * model,int instNum);
  virtual ~OGuiLabel(void);
  virtual Strings userSave(int column=0) const;

  // native void GuiLabel::new(GuiArea parent,string title,bool deep);
  //
  static void construct(Context * context);
};

//--------------------------------------------------------------------------//
// class OGuiEditLabel                                                      //
//--------------------------------------------------------------------------//

class OGuiEditLabel : public OGuiLabel
{
public:
  OGuiEditLabel(Model * model,int instNum);
  virtual ~OGuiEditLabel(void);
  virtual Strings userSave(int column=0) const;

  // native void GuiEditLabel::new(GuiArea parent,string title,bool deep);
  //
  static void construct(Context * context);
};

//--------------------------------------------------------------------------//
// class OGuiButton                                                         //
//--------------------------------------------------------------------------//

class OGuiButton : public OGuiElem
{
public:
  OGuiButton(Model * model,int instNum);
  virtual ~OGuiButton(void);
  virtual Strings userSave(int column=0) const;

  // native void GuiButton::new(GuiArea parent,string title);
  //
  static void construct(Context * context);
};

//--------------------------------------------------------------------------//
// class OGuiToggle                                                         //
//--------------------------------------------------------------------------//

class OGuiToggle : public OGuiElem
{
public:
  OGuiToggle(Model * model,int instNum);
  virtual ~OGuiToggle(void);
  virtual Strings userSave(int column=0) const;

  // native void GuiToggle::new(GuiArea parent,string title,bool big);
  //
  static void construct(Context * context);

  // native bool GuiToggle::isPressed(void);
  //
  static void isPressed(Context * context);

  // native void GuiToggle::setPressed(bool pressed,bool propagate);
  //
  static void setPressed(Context * context);
};

//--------------------------------------------------------------------------//
// class OGuiToggleGroup                                                    //
//--------------------------------------------------------------------------//

class OGuiToggleGroup : public OGuiElem
{
public:
  OGuiToggleGroup(Model * model,int instNum);
  virtual ~OGuiToggleGroup(void);
  virtual Strings userSave(int column=0) const;

  // native void GuiToggleGroup::new(GuiArea parent,string[] entries,
  //                                 bool horizontal,bool big);
  //
  static void construct(Context * context);

  // native int GuiToggleGroup::getResult(void);
  //
  static void getResult(Context * context);

  // native int GuiToggleGroup::getNbEntries(void);
  //
  static void getNbEntries(Context * context);

  // native string GuiToggleGroup::getEntry(int pos);
  //
  static void getEntry(Context * context);

  // native void GuiToggleGroup::setActiveEntry(int pos,bool propagate);
  //
  static void setActiveEntry(Context * context);
};

//--------------------------------------------------------------------------//
// class OGuiOptionMenu                                                     //
//--------------------------------------------------------------------------//

class OGuiOptionMenu : public OGuiElem
{
public:
  OGuiOptionMenu(Model * model,int instNum);
  virtual ~OGuiOptionMenu(void);
  virtual Strings userSave(int column=0) const;

  // native void GuiOptionMenu::new(GuiArea parent,string[] entries);
  //
  static void construct(Context * context);

  // native int GuiOptionMenu::getResult(void);
  //
  static void getResult(Context * context);

  // native int GuiOptionMenu::getNbEntries(void);
  //
  static void getNbEntries(Context * context);

  // native string GuiOptionMenu::getEntry(int pos);
  //
  static void getEntry(Context * context);

  // native void GuiOptionMenu::setActiveEntry(int pos,bool propagate);
  //
  static void setActiveEntry(Context * context);
};

//--------------------------------------------------------------------------//
// class OGuiCursor                                                         //
//--------------------------------------------------------------------------//

class OGuiCursor : public OGuiElem
{
public:
  OGuiCursor(Model * model,int instNum);
  virtual ~OGuiCursor(void);
  virtual Strings userSave(int column=0) const;

  // native void GuiCursor::new(GuiArea parent,bool horizontal);
  //
  static void construct(Context * context);

  // native float GuiCursor::getValue(void);
  //
  static void getValue(Context * context);

  // native void GuiCursor::setValue(float value,bool propagate);
  //
  static void setValue(Context * context);
};

//--------------------------------------------------------------------------//
// class OGuiScroller                                                       //
//--------------------------------------------------------------------------//

class OGuiScroller : public OGuiElem
{
public:
  OGuiScroller(Model * model,int instNum);
  virtual ~OGuiScroller(void);
  virtual Strings userSave(int column=0) const;

  // native void GuiScroller::new(GuiArea parent,bool horizontal);
  //
  static void construct(Context * context);

  // native float GuiScroller::getValue(void);
  //
  static void getValue(Context * context);

  // native float GuiScroller::getRatio(void);
  //
  static void getRatio(Context * context);

  // native void GuiScroller::setValue(float value,bool propagate);
  //
  static void setValue(Context * context);

  // native void GuiScroller::setRatio(float ratio,bool propagate);
  //
  static void setRatio(Context * context);
};

//--------------------------------------------------------------------------//
// class OGuiList                                                           //
//--------------------------------------------------------------------------//

class OGuiList : public OGuiElem
{
public:
  OGuiList(Model * model,int instNum);
  virtual ~OGuiList(void);
  virtual Strings userSave(int column=0) const;

  // native void GuiList::new(GuiArea parent,string[] entries);
  //
  static void construct(Context * context);

  // native int GuiList::getResult(void);
  //
  static void getResult(Context * context);

  // native bool GuiList::isEntered(void);
  //
  static void isEntered(Context * context);

  // native int GuiList::getNbEntries(void);
  //
  static void getNbEntries(Context * context);

  // native string GuiList::getEntry(int pos);
  //
  static void getEntry(Context * context);

  // native void GuiList::setEntries(string[] entries);
  //
  static void setEntries(Context * context);
};

//--------------------------------------------------------------------------//
// class OGuiEditor                                                         //
//--------------------------------------------------------------------------//

class OGuiEditor : public OGuiElem
{
public:
  OGuiEditor(Model * model,int instNum);
  virtual ~OGuiEditor(void);
  virtual Strings userSave(int column=0) const;

  // native void GuiEditor::new(GuiArea parent,string content,bool readOnly);
  //
  static void construct(Context * context);

  // native string GuiEditor::getContent(void);
  //
  static void getContent(Context * context);

  // native void GuiEditor::setContent(string content);
  //
  static void setContent(Context * context);
};

//--------------------------------------------------------------------------//
// class OGuiViewer3d                                                       //
//--------------------------------------------------------------------------//

#ifdef OPEN_GL

class OGuiViewer3d : public OGuiElem
{
public:
  OGuiViewer3d(Model * model,int instNum);
  virtual ~OGuiViewer3d(void);
  virtual Strings userSave(int column=0) const;
  static void orisKeyCB(GuiViewer3D * calling,
                        const Strings & key,void * data);

  // native void GuiViewer3d::new(GuiArea parent)
  //
  static void construct(Context * context);

  // native bool isSelected(Object3d obj);
  //
  static void isSelected(Context * context);

  // native Object3d[] getSelection(void);
  //
  static void getSelection(Context * context);

  // native void select(Object3d obj,bool propagate);
  //
  static void select(Context * context);

  // native void unselect(Object3d obj,bool propagate);
  //
  static void unselect(Context * context);

  // native string GuiViewer3d::getBackground(void);
  //
  static void getBackground(Context * context);

  // native void GuiViewer3d::setBackground(string color);
  //
  static void setBackground(Context * context);

  // native void GuiViewer3d::getPosition(float & x,float & y,float & z);
  //
  static void getPosition(Context * context);

  // native void GuiViewer3d::globalToLocalPosition(float & x,float & y,
  //                                                float & z);
  //
  static void globalToLocalPosition(Context * context);

  // native void GuiViewer3d::localToGlobalPosition(float & x,float & y,
  //                                                float & z);
  //
  static void localToGlobalPosition(Context * context);

  // native void GuiViewer3d::setPosition(float x,float y,float z);
  //
  static void setPosition(Context * context);

  // native void GuiViewer3d::translate(float dx,float dy,float dz);
  //
  static void translate(Context * context);

  // native void GuiViewer3d::getOrientation(float & roll,
  //                                         float & pitch,float & yaw);
  //
  static void getOrientation(Context * context);

  // native void GuiViewer3d::globalToLocalOrientation(float & roll,
  //                                                   float & pitch,
  //                                                   float & yaw);
  //
  static void globalToLocalOrientation(Context * context);

  // native void GuiViewer3d::localToGlobalOrientation(float & roll,
  //                                                   float & pitch,
  //                                                   float & yaw);
  //
  static void localToGlobalOrientation(Context * context);

  // native void GuiViewer3d::cancelRotation(void);
  //
  static void cancelRotation(Context * context);

  // native void GuiViewer3d::setOrientation(float roll,float pitch,float yaw);
  //
  static void setOrientation(Context * context);

  // native void GuiViewer3d::getLocation(float & x,float & y,float & z,
  //                                      float & roll,float & pitch,
  //                                      float & yaw);
  //
  static void getLocation(Context * context);

  // native void GuiViewer3d::setLocation(float x,float y,float z,
  //                                      float roll,float pitch,float yaw);
  //
  static void setLocation(Context * context);

  // native void GuiViewer3d::roll(float dRoll);
  //
  static void roll(Context * context);

  // native void GuiViewer3d::pitch(float dPitch);
  //
  static void pitch(Context * context);

  // native void GuiViewer3d::yaw(float dYaw);
  //
  static void yaw(Context * context);
};

#endif // OPEN_GL

//--------------------------------------------------------------------------//
// class OGuiViewer2d                                                       //
//--------------------------------------------------------------------------//

class OGuiViewer2d : public OGuiElem
{
public:
  OGuiViewer2d(Model * model,int instNum);
  virtual ~OGuiViewer2d(void);
  virtual Strings userSave(int column=0) const;
  static void orisKeyCB(GuiViewer2D * calling,
                        const Strings & key,void * data);

  // native void GuiViewer2d::new(GuiArea parent)
  //
  static void construct(Context * context);

  // native bool isSelected(Object2d obj);
  //
  static void isSelected(Context * context);
 
  // native Object2d[] getSelection(void);
  //
  static void getSelection(Context * context);
 
  // native void select(Object2d obj,bool propagate);
  //
  static void select(Context * context);
 
  // native void unselect(Object2d obj,bool propagate);
  //
  static void unselect(Context * context);

  // native string GuiViewer2d::getBackground(void);
  //
  static void getBackground(Context * context);

  // native void GuiViewer2d::setBackground(string color);
  //
  static void setBackground(Context * context);

  // native void GuiViewer2d::getViewPoint(float & x,float & y,float & scale);
  //
  static void getViewPoint(Context * context);

  // native void GuiViewer2d::setViewPoint(float x,float y,float scale);
  //
  static void setViewPoint(Context * context);

  // native void GuiViewer2d::autoScale(void);
  //
  static void autoScale(Context * context);
};

//--------------------------------------------------------------------------//
// class OGuiGrapher                                                        //
//--------------------------------------------------------------------------//

class OGuiGrapher : public OGuiElem
{
public:
  OGuiGrapher(Model * model,int instNum);
  virtual ~OGuiGrapher(void);
  virtual Strings userSave(int column=0) const;

  // native void GuiGrapher::new(GuiArea parent)
  //
  static void construct(Context * context);

  // native string GuiGrapher::getBackground(void);
  //
  static void getBackground(Context * context);

  // native void GuiGrapher::setBackground(string color);
  //
  static void setBackground(Context * context);

  // native void GuiGrapher::getViewPoint(float & xmin,float & ymin,
  //                                      float & ymin,float & ymax);
  //
  static void getViewPoint(Context * context);

  // native void GuiGrapher::setViewPoint(float xmin,float ymin,
  //                                      float xmax,float ymax);
  //
  static void setViewPoint(Context * context);

  // native void GuiGrapher::autoScale(void);
  //
  static void autoScale(Context * context);

  // native void GuiGrapher::autoScaleX(void);
  //
  static void autoScaleX(Context * context);

  // native void GuiGrapher::autoScaleY(void);
  //
  static void autoScaleY(Context * context);

  // native void GuiGrapher::addPoint(string color,float x,float y);
  //
  static void addPoint(Context * context);

  // native void GuiGrapher::accumulatePoint(string color,float x,float y);
  //
  static void accumulatePoint(Context * context);

  // native void GuiGrapher::clear(string color);
  //
  static void clear(Context * context);
};

//--------------------------------------------------------------------------//
// class OGuiTextDialog                                                     //
//--------------------------------------------------------------------------//

class OGuiTextDialog : public OGuiWindow
{
public:
  OGuiTextDialog(Model * model,int instNum);
  virtual ~OGuiTextDialog(void);
  virtual Strings userSave(int column=0) const;

  // native void GuiTextDialog::new(string title,string[] message);
  //
  static void construct(Context * context);

  // native bool GuiTextDialog::isEntered(void);
  //
  static void isEntered(Context * context);

  // native bool GuiTextDialog::isCanceled(void);
  //
  static void isCanceled(Context * context);

  // native string GuiTextDialog::getText(void);
  //
  static void getText(Context * context);

  // native void GuiTextDialog::setText(string text);
  //
  static void setText(Context * context);

  // native bool GuiTextDialog::modalAsk(string & answer);
  //
  static void modalAsk(Context * context);
};

//--------------------------------------------------------------------------//
// class OGuiButtonDialog                                                   //
//--------------------------------------------------------------------------//

class OGuiButtonDialog : public OGuiWindow
{
public:
  OGuiButtonDialog(Model * model,int instNum);
  virtual ~OGuiButtonDialog(void);
  virtual Strings userSave(int column=0) const;

  // native void GuiButtonDialog::new(string title,string[] message,
  //                                  string[] buttons);
  //
  static void construct(Context * context);

  // native int GuiButtonDialog::getAnswer(void);
  //
  static void getAnswer(Context * context);

  // native int GuiButtonDialog::getNbButtons(void);
  //
  static void getNbButtons(Context * context);

  // native string GuiButtonDialog::getButton(int pos);
  //
  static void getButton(Context * context);

  // native int GuiButtonDialog::modalAsk(void);
  //
  static void modalAsk(Context * context);
};

//--------------------------------------------------------------------------//
// class OGuiFileChooser                                                    //
//--------------------------------------------------------------------------//

class OGuiFileChooser : public OGuiWindow
{
public:
  OGuiFileChooser(Model * model,int instNum);
  virtual ~OGuiFileChooser(void);
  virtual Strings userSave(int column=0) const;

  // native void GuiFileChooser::new(string title,string message);
  //
  static void construct(Context * context);

  // native bool GuiFileChooser::isEntered(void);
  //
  static void isEntered(Context * context);

  // native bool GuiFileChooser::isCanceled(void);
  //
  static void isCanceled(Context * context);

  // native string GuiFileChooser::getFileName(void);
  //
  static void getFileName(Context * context);

  // native string GuiFileChooser::getPath(void);
  //
  static void getPath(Context * context);

  // native void GuiFileChooser::setPath(string path);
  //
  static void setPath(Context * context);

  // native bool GuiFileChooser::modalAsk(string & answer);
  //
  static void modalAsk(Context * context);
};

//--------------------------------------------------------------------------//
// class OGuiColorChooser                                                   //
//--------------------------------------------------------------------------//

class OGuiColorChooser : public OGuiWindow
{
public:
  OGuiColorChooser(Model * model,int instNum);
  virtual ~OGuiColorChooser(void);
  virtual Strings userSave(int column=0) const;

  // native void GuiColorChooser::new(string title,string message);
  //
  static void construct(Context * context);

  // native bool GuiColorChooser::isEntered(void);
  //
  static void isEntered(Context * context);

  // native bool GuiColorChooser::isCanceled(void);
  //
  static void isCanceled(Context * context);

  // native string GuiColorChooser::getColor(void);
  //
  static void getColor(Context * context);

  // native void GuiColorChooser::setColor(string color);
  //
  static void setColor(Context * context);

  // native bool GuiColorChooser::modalAsk(string & answer);
  //
  static void modalAsk(Context * context);
};

#endif // GUIPKG_H

