// @(#) ENIB/LI2 (c) - oRis - Version 2.3
//--------------------------------------------------------------------------//
// file        : ipcPkg.cpp                                                 //
// description : source file for the ENIB/LI2 oRis package ipc.pkg          //
// author(s)   : F. Harrouet                                                //
// copyright   : Laboratoire d'Informatique Industrielle (LI2)              //
//               Ecole Nationale d'Ingenieurs de Brest (ENIB)               //
//               CP 30815 - 29608 Brest Cedex - FRANCE                      //
//               Tel: (33) 02.98.05.66.31 , Fax: (33) 02.98.05.66.29        //
//               e-mail: li2@enib.fr                                        //
//--------------------------------------------------------------------------//
// Copyright (c) 1996-2000                                                  //
// Laboratoire d'Informatique Industrielle (LI2)                            //
// Ecole Nationale d'Ingenieurs de Brest (ENIB)                             //
//                                                                          //
// Permission to use, copy, modify, distribute and sell this software       //
// and its documentation for any purpose is hereby granted without fee,     //
// provided that the above copyright notice appear in all copies and        //
// that both that copyright notice and this permission notice appear        //
// in supporting documentation.                                             //
// ENIB/LI2 makes no representations about the suitability of this          //
// software for any purpose.                                                //
// It is provided "as is" without express or implied warranty.              //
//--------------------------------------------------------------------------//

#include "Ipc/ipcPkg.h"
#include "oRis/Main/simul.h"
#include "oRis/Main/context.h"
#include "oRis/Main/errors.h"

#ifndef __MSDOS__
#define EXPORT_PREFIX extern "C"
#else
#include <windows.h>
#define EXPORT_PREFIX extern "C" __declspec(dllexport)
#define openingLibrary    openingIpcLibrary
#define closingLibrary    closingIpcLibrary
#define getExternFunction getExternIpcFunction
#define getNewAgent       getNewIpcAgent
#endif

//--------------------------------------------------------------------------//
// Opening and closing the library                                          //
//--------------------------------------------------------------------------//
 
EXPORT_PREFIX void openingLibrary(void)
{
  // nothing to do
}
 
EXPORT_PREFIX void closingLibrary(void)
{
  // nothing to do
}

//--------------------------------------------------------------------------//
// LookUp table for functions                                               //
//--------------------------------------------------------------------------//

EXPORT_PREFIX ExternFunction getExternFunction(const Strings & name)
{
if(name==Strings("IPCMemory::_new()"))       return(OIPCMemory::construct);
if(name==Strings("IPCMemory::_delete()"))    return(OIPCMemory::destruct);
if(name==Strings("IPCMemory::fail()"))       return(OIPCMemory::fail);
if(name==Strings("IPCMemory::getId()"))      return(OIPCMemory::getId);
if(name==Strings("IPCMemory::getSize()"))    return(OIPCMemory::getSize);
if(name==Strings("IPCMemory::getContent()")) return(OIPCMemory::getContent);
if(name==Strings("IPCMemory::setContent()")) return(OIPCMemory::setContent);
if(name==Strings("IPCMutex::_new()"))        return(OIPCMutex::construct);
if(name==Strings("IPCMutex::_delete()"))     return(OIPCMutex::destruct);
if(name==Strings("IPCMutex::fail()"))        return(OIPCMutex::fail);
if(name==Strings("IPCMutex::getId()"))       return(OIPCMutex::getId);
if(name==Strings("IPCMutex::lock()"))        return(OIPCMutex::lock);
if(name==Strings("IPCMutex::tryLock()"))     return(OIPCMutex::tryLock);
if(name==Strings("IPCMutex::unlock()"))      return(OIPCMutex::unlock);
return((ExternFunction)0); // not found
}

//--------------------------------------------------------------------------//
// LookUp table for agents                                                  //
//--------------------------------------------------------------------------//

static Word ipcMemoryWord(Word::create("IPCMemory"));
static Word ipcMutexWord(Word::create("IPCMutex"));

EXPORT_PREFIX Agent * getNewAgent(Model * model,long instNum)
{
if(model->isA(::ipcMemoryWord))  return(new OIPCMemory(model,instNum));
if(model->isA(::ipcMutexWord))   return(new OIPCMutex(model,instNum));
return((Agent *)0); // not found
}

//--------------------------------------------------------------------------//
// IPCMemory functions                                                      //
//--------------------------------------------------------------------------//

// native void IPCMemory::_new(bool create,int sizeOrId);
//
void OIPCMemory::construct(Context * context) // static
{
OIPCMemory * mem=(OIPCMemory *)context->accessOwner();
if(mem->_mem) delete mem->_mem;
if(context->getInteger(0)) // create
  {
  mem->_mem=new IPCSharedMemory((size_t)context->getInteger(1));
  if(!mem->_mem->fail())
    *((char *)mem->_mem->getAddress())='\0';
  }
else // reuse
  {
  mem->_mem=new IPCSharedMemory((int)context->getInteger(1));
  }
}

// native void IPCMemory::_delete(void);
//
void OIPCMemory::destruct(Context * context) // static
{
OIPCMemory * mem=(OIPCMemory *)context->accessOwner();
if(mem->_mem)
  {
  delete mem->_mem;
  mem->_mem=(IPCSharedMemory *)0;
  }
}

// native bool IPCMemory::fail(void);
//
void OIPCMemory::fail(Context * context) // static
{
const OIPCMemory * mem=(const OIPCMemory *)context->getOwner();
context->returnInteger(!mem->_mem||mem->_mem->fail());
}

// native int IPCMemory::getId(void);
//
void OIPCMemory::getId(Context * context) // static
{
const OIPCMemory * mem=(const OIPCMemory *)context->getOwner();
if(mem->_mem)
  context->returnInteger(mem->_mem->getID());
else
  context->returnInteger(-1);
}

// native int IPCMemory::getSize(void);
//
void OIPCMemory::getSize(Context * context) // static
{
const OIPCMemory * mem=(const OIPCMemory *)context->getOwner();
if(mem->_mem)
  context->returnInteger(mem->_mem->getSize());
else
  context->returnInteger(0);
}

// native string IPCMemory::getContent(void);
//
void OIPCMemory::getContent(Context * context) // static
{
const OIPCMemory * mem=(const OIPCMemory *)context->getOwner();
if(mem->_mem&&!mem->_mem->fail())
  {
  char * address=(char *)mem->_mem->getAddress();
  address[mem->_mem->getSize()-1]='\0'; // make sure it ends
  context->returnStrings(Strings(address));
  }
else
  {
  Strings message="Can't read from '"+
                  mem->getName().print()+
                  "' (bad state)";
  getSimul()->manageError(new Error(Error::RUNTIME,message));
  context->returnStrings(Strings(""));
  }
}

// native void IPCMemory::setContent(string content);
//
void OIPCMemory::setContent(Context * context) // static
{
OIPCMemory * mem=(OIPCMemory *)context->accessOwner();
if(mem->_mem&&!mem->_mem->fail())
  {
  size_t nb=context->getStrings(0).len()+1;
  if(nb>mem->_mem->getSize())
    {
    Strings message="Content of '"+
                    mem->getName().print()+
                    "' cannot exceed "+
                    Strings(mem->_mem->getSize()-1)+
                    " chars";
    getSimul()->manageError(new Error(Error::RUNTIME,message));
    }
  else
    {
    ::memcpy((char *)mem->_mem->getAddress(),
             (const char *)context->getStrings(0),nb);
    }
  }
else
  {
  Strings message="Can't write into '"+
                  mem->getName().print()+
                  "' (bad state)";
  getSimul()->manageError(new Error(Error::RUNTIME,message));
  }
}

//--------------------------------------------------------------------------//
// IPCMutex functions                                                       //
//--------------------------------------------------------------------------//
 
// native void IPCMutex::_new(int id);
//
void OIPCMutex::construct(Context * context) // static
{
OIPCMutex * mutex=(OIPCMutex *)context->accessOwner();
if(mutex->_mutex) delete mutex->_mutex;
if(context->getInteger(0)<0) // create
  {
  mutex->_mutex=new IPCMutex;
  }
else // reuse
  {
  mutex->_mutex=new IPCMutex(context->getInteger(0));
  }
}
 
// native void IPCMutex::_delete(void);
//
void OIPCMutex::destruct(Context * context) // static
{
OIPCMutex * mutex=(OIPCMutex *)context->accessOwner();
if(mutex->_mutex)
  {
  delete mutex->_mutex;
  mutex->_mutex=(IPCMutex *)0;
  }
}

// native bool IPCMutex::fail(void);
//
void OIPCMutex::fail(Context * context) // static
{
const OIPCMutex * mutex=(const OIPCMutex *)context->getOwner();
context->returnInteger(!mutex->_mutex||mutex->_mutex->fail());
}
 
// native int IPCMutex::getId(void);
//
void OIPCMutex::getId(Context * context) // static
{
const OIPCMutex * mutex=(const OIPCMutex *)context->getOwner();
if(mutex->_mutex)
  context->returnInteger(mutex->_mutex->getID());
else
  context->returnInteger(-1);
}

// native void IPCMutex::lock(void);
//
void OIPCMutex::lock(Context * context) // static
{
Simulator::embedThread(OIPCMutex::_lockTask,context);
}
 
void OIPCMutex::_lockTask(Context * context) // static
{
OIPCMutex * mutex=(OIPCMutex *)context->accessOwner();
if(mutex->_mutex&&!mutex->_mutex->fail())
  {
  mutex->_mutex->lock();
  }
else
  {
  Strings message="Can't lock '"+
                  mutex->getName().print()+
                  "' (bad state)";
  getSimul()->manageError(new Error(Error::RUNTIME,message));
  }
}

// native bool IPCMutex::tryLock(void);
//
void OIPCMutex::tryLock(Context * context) // static
{
OIPCMutex * mutex=(OIPCMutex *)context->accessOwner();
if(mutex->_mutex&&!mutex->_mutex->fail())
  context->returnInteger(mutex->_mutex->tryLock());
else
  {
  Strings message="Can't try to lock '"+
                  mutex->getName().print()+
                  "' (bad state)";
  getSimul()->manageError(new Error(Error::RUNTIME,message));
  context->returnInteger(0);
  }
}
 
 
// native void IPCMutex::unlock(void);
//
void OIPCMutex::unlock(Context * context) // static
{
OIPCMutex * mutex=(OIPCMutex *)context->accessOwner();
if(mutex->_mutex&&!mutex->_mutex->fail())
  {
  if(mutex->_mutex->isLocked()) mutex->_mutex->unlock();
  else
    {
    Strings message="Can't unlock '"+
                    mutex->getName().print()+
                    "' (already unlocked)";
    getSimul()->manageError(new Error(Error::RUNTIME,message));
    }
  }
else
  {
  Strings message="Can't unlock '"+
                  mutex->getName().print()+
                  "' (bad state)";
  getSimul()->manageError(new Error(Error::RUNTIME,message));
  }
}

//--------------------------------------------------------------------------//
// class OIPCMemory                                                         //
//--------------------------------------------------------------------------//

OIPCMemory::OIPCMemory(Model * model,int instNum)
 : Agent(model,instNum),
   _mem((IPCSharedMemory *)0)
{}

OIPCMemory::~OIPCMemory(void)
{
if(_mem) delete _mem;
}

Strings OIPCMemory::userSave(int /* column */ ) const // virtual
{
Strings str;
return(str);
}

//--------------------------------------------------------------------------//
// class OIPCMutex                                                          //
//--------------------------------------------------------------------------//
 
OIPCMutex::OIPCMutex(Model * model,int instNum)
 : Agent(model,instNum),
   _mutex((IPCMutex *)0)
{}
 
OIPCMutex::~OIPCMutex(void)
{
if(_mutex) delete _mutex;
}
 
Strings OIPCMutex::userSave(int /* column */ ) const // virtual
{
Strings str;
return(str);
}

