// @(#) ENIB/LI2 (c) - oRis - Version 2.3
//--------------------------------------------------------------------------//
// file        : mathsPkg.cpp                                               //
// description : source file for the ENIB/LI2 oRis package maths.pkg        //
// author(s)   : F. Harrouet                                                //
// copyright   : Laboratoire d'Informatique Industrielle (LI2)              //
//               Ecole Nationale d'Ingenieurs de Brest (ENIB)               //
//               CP 30815 - 29608 Brest Cedex - FRANCE                      //
//               Tel: (33) 02.98.05.66.31 , Fax: (33) 02.98.05.66.29        //
//               e-mail: li2@enib.fr                                        //
//--------------------------------------------------------------------------//
// Copyright (c) 1996-2000                                                  //
// Laboratoire d'Informatique Industrielle (LI2)                            //
// Ecole Nationale d'Ingenieurs de Brest (ENIB)                             //
//                                                                          //
// Permission to use, copy, modify, distribute and sell this software       //
// and its documentation for any purpose is hereby granted without fee,     //
// provided that the above copyright notice appear in all copies and        //
// that both that copyright notice and this permission notice appear        //
// in supporting documentation.                                             //
// ENIB/LI2 makes no representations about the suitability of this          //
// software for any purpose.                                                //
// It is provided "as is" without express or implied warranty.              //
//--------------------------------------------------------------------------//

#include "Maths/mathsPkg.h"
#include "oRis/Main/simul.h"
#include "oRis/Main/errors.h"

#include <math.h>

#ifndef __MSDOS__
#define EXPORT_PREFIX extern "C"
static inline bool badResult(double result)
{
return(!finite(result));
}
#else
#include <windows.h>
#define EXPORT_PREFIX extern "C" __declspec(dllexport)
#define openingLibrary    openingMathsLibrary
#define closingLibrary    closingMathsLibrary
#define getExternFunction getExternMathsFunction
#define getNewAgent       getNewMathsAgent
#include <float.h>
static inline bool badResult(double result)
{
return(!_finite(result));
}
#endif

// mathError1 is only accessible in this file
static void mathError1(Context * context,double arg)
{
Strings message="Can't evaluate '"+
                context->getModule()->getName().getStrings()+
                "("+Strings(arg)+")'";
getSimul()->manageError(new Error(Error::RUNTIME,message));
}
 
// mathError2 is only accessible in this file
static void mathError2(Context * context,double arg1,double arg2)
{
Strings message="Can't evaluate '"+
                context->getModule()->getName().getStrings()+
                "("+Strings(arg1)+","+Strings(arg2)+")'";
getSimul()->manageError(new Error(Error::RUNTIME,message));
}

//--------------------------------------------------------------------------//
// Opening and closing the library                                          //
//--------------------------------------------------------------------------//
 
EXPORT_PREFIX void openingLibrary(void)
{
  // nothing to do
}
 
EXPORT_PREFIX void closingLibrary(void)
{
  // nothing to do
}

//--------------------------------------------------------------------------//
// LookUp table for functions                                               //
//--------------------------------------------------------------------------//

EXPORT_PREFIX ExternFunction getExternFunction(const Strings & name)
{
// Maths functions
if(name==Strings("::acos()"))         return(::acos);
if(name==Strings("::asin()"))         return(::asin);
if(name==Strings("::atan()"))         return(::atan);
if(name==Strings("::acos()"))         return(::acos);
if(name==Strings("::cosh()"))         return(::cosh);
if(name==Strings("::cos()"))          return(::cos);
if(name==Strings("::sinh()"))         return(::sinh);
if(name==Strings("::sin()"))          return(::sin);
if(name==Strings("::tanh()"))         return(::tanh);
if(name==Strings("::tan()"))          return(::tan);
if(name==Strings("::exp()"))          return(::exp);
if(name==Strings("::abs()"))          return(::abs);
if(name==Strings("::ln()"))           return(::ln);
if(name==Strings("::log()"))          return(::log);
if(name==Strings("::pow()"))          return(::pow);
if(name==Strings("::square()"))       return(::square);
if(name==Strings("::signedSquare()")) return(::signedSquare);
if(name==Strings("::sqrt()"))         return(::sqrt);
if(name==Strings("::signedSqrt()"))   return(::signedSqrt);
if(name==Strings("::floor()"))        return(::floor);
if(name==Strings("::ceil()"))         return(::ceil);
// Cartesian <===> Polar
if(name==Strings("::cartToAngle()"))  return(::cartToAngle);
if(name==Strings("::cartToDist()"))   return(::cartToDist);
if(name==Strings("::polToX()"))       return(::polToX);
if(name==Strings("::polToY()"))       return(::polToY);
if(name==Strings("::cartToPol()"))    return(::cartToPol);
if(name==Strings("::polToCart()"))    return(::polToCart);
if(name==Strings("::cartToSphere()")) return(::cartToSphere);
if(name==Strings("::sphereToCart()")) return(::sphereToCart);
if(name==Strings("::betweenPi()"))    return(::betweenPi);
return((ExternFunction)0); // not found
}

//--------------------------------------------------------------------------//
// LookUp table for agents                                                  //
//--------------------------------------------------------------------------//
 
EXPORT_PREFIX Agent * getNewAgent(Model * /* model */ ,long /* instNum */ )
{
return((Agent *)0); // not found
}

//--------------------------------------------------------------------------//
// Maths functions                                                          //
//--------------------------------------------------------------------------//

// native float acos(float x);
//
void acos(Context * context)
{
double arg=context->getReal(0);
double result=::acos(arg);
context->returnReal(result);
if(badResult(result)) mathError1(context,arg);
}

// native float asin(float x);
//
void asin(Context * context)
{
double arg=context->getReal(0);
double result=::asin(arg);
context->returnReal(result);
if(badResult(result)) mathError1(context,arg);
}

// native float atan(float x);
//
void atan(Context * context)
{
double arg=context->getReal(0);
double result=::atan(arg);
context->returnReal(result);
if(badResult(result)) mathError1(context,arg);
}

// native float cosh(float x);
//
void cosh(Context * context)
{
double arg=context->getReal(0);
double result=::cosh(arg);
context->returnReal(result);
if(badResult(result)) mathError1(context,arg);
}

// native float cos(float x);
//
void cos(Context * context)
{
double arg=context->getReal(0);
context->returnReal(::cos(arg));
}

// native float sinh(float x);
//
void sinh(Context * context)
{
double arg=context->getReal(0);
double result=::sinh(arg);
context->returnReal(result);
if(badResult(result)) mathError1(context,arg);
}

// native float sin(float x);
//
void sin(Context * context)
{
double arg=context->getReal(0);
context->returnReal(::sin(arg));
}

// native float tanh(float x);
//
void tanh(Context * context)
{
double arg=context->getReal(0);
double result=::tanh(arg);
context->returnReal(result);
if(badResult(result)) mathError1(context,arg);
}

// native float tan(float x);
//
void tan(Context * context)
{
double arg=context->getReal(0);
double result=::tan(arg);
context->returnReal(result);
if(badResult(result)) mathError1(context,arg);
}

// native float exp(float x);
//
void exp(Context * context)
{
double arg=context->getReal(0);
context->returnReal(::exp(arg));
}

// native float abs(float x);
//
void abs(Context * context)
{
double arg=context->getReal(0);
context->returnReal(::fabs(arg));
}

// native float ln(float x);
//
void ln(Context * context)
{
double arg=context->getReal(0);
double result=::log(arg);
context->returnReal(result);
if(badResult(result)) mathError1(context,arg);
}

// native float log(float x);
//
void log(Context * context)
{
double arg=context->getReal(0);
double result=::log10(arg);
context->returnReal(result);
if(badResult(result)) mathError1(context,arg);
}

// native float pow(float x,float y);
//
void pow(Context * context)
{
double x=context->getReal(0);
double y=context->getReal(1);
double result=::pow(x,y);
context->returnReal(result);
if(badResult(result)) mathError2(context,x,y);
}

// native float square(float x);
//
void square(Context * context)
{
double arg=context->getReal(0);
context->returnReal(arg*arg);
}
 
// native float signedSquare(float x);
//
void signedSquare(Context * context)
{
double arg=context->getReal(0);
context->returnReal(arg>=0.0 ? arg*arg : -arg*arg);
}

// native float sqrt(float x);
//
void sqrt(Context * context)
{
double arg=context->getReal(0);
double result=::sqrt(arg);
context->returnReal(result);
if(badResult(result)) mathError1(context,arg);
}

// native float signedSqrt(float x);
//
void signedSqrt(Context * context)
{
double arg=context->getReal(0);
context->returnReal(arg>=0.0 ? ::sqrt(arg) : -::sqrt(-arg));
}

// native float floor(float x);
//
void floor(Context * context)
{
double arg=context->getReal(0);
context->returnReal(::floor(arg));
}

// native float ceil(float x);
//
void ceil(Context * context)
{
double arg=context->getReal(0);
context->returnReal(::ceil(arg));
}

//--------------------------------------------------------------------------//
// Cartesian <---> Polar                                                    //
//--------------------------------------------------------------------------//

// native float cartToAngle(float x,float y);
//
void cartToAngle(Context * context)
{
double x=context->getReal(0);
double y=context->getReal(1);
context->returnReal(x||y ? ::atan2(y,x) : 0.0);
}

// native float cartToDist(float x,float y);
//
void cartToDist(Context * context)
{
double x=context->getReal(0);
double y=context->getReal(1);
context->returnReal(::sqrt(x*x+y*y));
}

// native float polToX(float d,float a);
//
void polToX(Context * context)
{
double d=context->getReal(0);
double a=context->getReal(1);
context->returnReal(d*::cos(a));
}

// native float polToY(float d,float a);
//
void polToY(Context * context)
{
double d=context->getReal(0);
double a=context->getReal(1);
context->returnReal(d*::sin(a));
}

// native void cartToPol(float & d,float & a,float x,float y);
//
void cartToPol(Context * context)
{
double x=context->getReal(2);
double y=context->getReal(3);
context->setReal(0,::sqrt(x*x+y*y));
context->setReal(1,(x||y ? ::atan2(y,x) : 0.0));
}

// native void polToCart(float & x,float & y,float d,float a);
//
void polToCart(Context * context)
{
double d=context->getReal(2);
double a=context->getReal(3);
context->setReal(0,d*::cos(a));
context->setReal(1,d*::sin(a));
}

// native void cartToSphere(float & d,float & xyAngle,float & zAngle,
//                          float x,float y,float z);
//
void cartToSphere(Context * context)
{
double x=context->getReal(3);
double y=context->getReal(4);
double z=context->getReal(5);
double dxy=::sqrt(x*x+y*y);
context->setReal(0,::sqrt(dxy*dxy+z*z));
context->setReal(1,(x||y ? ::atan2(y,x) : 0.0));
context->setReal(2,(dxy||z ? ::atan2(z,dxy) : 0.0));
}

// native void sphereToCart(float & x,float & y,float & z,
//                          float d,float xyAngle,float zAngle);
//
void sphereToCart(Context * context)
{
double d=context->getReal(3);
double xyAngle=context->getReal(4);
double zAngle=context->getReal(5);
double dxy=d*::cos(zAngle);
context->setReal(0,dxy*::cos(xyAngle));
context->setReal(1,dxy*::sin(xyAngle));
context->setReal(2,d*::sin(zAngle));
}

// native float betweenPi(float a);
//
void betweenPi(Context * context)
{
double a=context->getReal(0);
a=::fmod(a,Constant_2PI);
if(a>Constant_PI) a-=Constant_2PI;
else if(a<-Constant_PI) a+=Constant_2PI;
context->returnReal(a);
}

