// @(#) ENIB/LI2 (c) - oRis - Version 2.3
//--------------------------------------------------------------------------//
// file        : memoryPkg.cpp                                              //
// description : source file for the ENIB/LI2 oRis package memory.pkg       //
// author(s)   : F. Harrouet                                                //
// copyright   : Laboratoire d'Informatique Industrielle (LI2)              //
//               Ecole Nationale d'Ingenieurs de Brest (ENIB)               //
//               CP 30815 - 29608 Brest Cedex - FRANCE                      //
//               Tel: (33) 02.98.05.66.31 , Fax: (33) 02.98.05.66.29        //
//               e-mail: li2@enib.fr                                        //
//--------------------------------------------------------------------------//
// Copyright (c) 1996-2000                                                  //
// Laboratoire d'Informatique Industrielle (LI2)                            //
// Ecole Nationale d'Ingenieurs de Brest (ENIB)                             //
//                                                                          //
// Permission to use, copy, modify, distribute and sell this software       //
// and its documentation for any purpose is hereby granted without fee,     //
// provided that the above copyright notice appear in all copies and        //
// that both that copyright notice and this permission notice appear        //
// in supporting documentation.                                             //
// ENIB/LI2 makes no representations about the suitability of this          //
// software for any purpose.                                                //
// It is provided "as is" without express or implied warranty.              //
//--------------------------------------------------------------------------//

#include "Memory/memoryPkg.h"
#include "oRis/Main/simul.h"
#include "oRis/Main/context.h"
#include "oRis/Main/errors.h"

#ifndef __MSDOS__
#define EXPORT_PREFIX extern "C"
#else
#include <windows.h>
#define EXPORT_PREFIX extern "C" __declspec(dllexport)
#define openingLibrary    openingMemoryLibrary
#define closingLibrary    closingMemoryLibrary
#define getExternFunction getExternMemoryFunction
#define getNewAgent       getNewMemoryAgent
#endif

#if defined __i386__ || defined __MSDOS__
// invert16 is only accessible in this file
inline static void invert16(unsigned char * ptr)
{
unsigned char tmp=ptr[0];
ptr[0]=ptr[1];
ptr[1]=tmp;
}

// invert32 is only accessible in this file
inline static void invert32(unsigned char * ptr)
{
unsigned char tmp=ptr[0];
ptr[0]=ptr[3];
ptr[3]=tmp;
tmp=ptr[1];
ptr[1]=ptr[2];
ptr[2]=tmp;
}

// invert64 is only accessible in this file
inline static void invert64(unsigned char * ptr)
{
unsigned char tmp=ptr[0];
ptr[0]=ptr[7];
ptr[7]=tmp;
tmp=ptr[1];
ptr[1]=ptr[6];
ptr[6]=tmp;
tmp=ptr[2];
ptr[2]=ptr[5];
ptr[5]=tmp;
tmp=ptr[3];
ptr[3]=ptr[4];
ptr[4]=tmp;
}
#endif // __i386__ || __MSDOS__

//--------------------------------------------------------------------------//
// Opening and closing the library                                          //
//--------------------------------------------------------------------------//
 
EXPORT_PREFIX void openingLibrary(void)
{
  // nothing to do
}
 
EXPORT_PREFIX void closingLibrary(void)
{
  // nothing to do
}

//--------------------------------------------------------------------------//
// LookUp table for agents                                                  //
//--------------------------------------------------------------------------//

static Word memoryWord(Word::create("Memory"));

EXPORT_PREFIX Agent * getNewAgent(Model * model,long instNum)
{
if(model->isA(::memoryWord)) return(new OMemory(model,instNum));
return((Agent *)0); // not found
}

//--------------------------------------------------------------------------//
// Memory functions                                                         //
//--------------------------------------------------------------------------//

// native bool Memory::fail(void);
//
EXPORT_PREFIX void wrap_Memory_fail(Context * context)
{
const OMemory * memory=(const OMemory *)context->getOwner();
context->returnInteger(memory->getBlock() ? 0 : 1);
}

// native int Memory::getSize(void);
//
EXPORT_PREFIX void wrap_Memory_getSize(Context * context)
{
const OMemory * memory=(const OMemory *)context->getOwner();
context->returnInteger(memory->getSize());
}

// native void Memory::resize(int newSize);
//
EXPORT_PREFIX void wrap_Memory_resize(Context * context)
{
OMemory * memory=(OMemory *)context->accessOwner();
int newSize=context->getInteger(0);
if(newSize<=0)
  {
  void * block=memory->getBlock();
  if(block)
    {
    BlockAllocator::free(block);
    memory->setBlock((void *)0);
    }
  memory->setSize(0);
  }
else
  {
  void * block=memory->getBlock();
  if(block) block=BlockAllocator::realloc(block,newSize);
  else block=BlockAllocator::allocate(newSize);
  if(!block) newSize=0;
  memory->setBlock(block);
  memory->setSize(newSize);
  }
}

// native void Memory::swap(Memory mem);
//
EXPORT_PREFIX void wrap_Memory_swap(Context * context)
{
OMemory * memory=(OMemory *)context->accessOwner();
OMemory * mem=(OMemory *)context->getAgent(0);
if(mem&&mem->isAlive())
  {
  void * tmpBlock=memory->getBlock();
  size_t tmpSize=memory->getSize();
  memory->setBlock(mem->getBlock());
  memory->setSize(mem->getSize());
  mem->setBlock(tmpBlock);
  mem->setSize(tmpSize);
  }
else
  {
  Strings str;
  if(mem) str="deleted object '"+mem->getName().print()+"'";
  else str="'NONE'";
  Strings message="Can't swap memory content with "+str;
  getSimul()->manageError(new Error(Error::RUNTIME,message));
  }
}

// native void Memory::copyBlock(int destOffset,Memory src,
//                               int srcOffset,int length);
//
EXPORT_PREFIX void wrap_Memory_copyBlock(Context * context)
{
OMemory * memory=(OMemory *)context->accessOwner();
int destPos=context->getInteger(0);
OMemory * mem=(OMemory *)context->getAgent(1);
int srcPos=context->getInteger(2);
int length=context->getInteger(3);
if(mem&&mem->isAlive())
  {
  if((srcPos>=0)&&(destPos>=0)&&(length>=0)&&
     (srcPos+length<=(int)mem->getSize())&&
     (destPos+length<=(int)memory->getSize()))
    {
    const char * src=(const char *)mem->getBlock();
    char * dest=(char *)memory->getBlock();
    ::memcpy(dest+destPos,src+srcPos,length);
    }
  else
    {
    Strings message="Bad block limits";
    getSimul()->manageError(new Error(Error::RUNTIME,message));
    }
  }
else
  {
  Strings str;
  if(mem) str="deleted object '"+mem->getName().print()+"'";
  else str="'NONE'";
  Strings message="Can't copy a memory block from "+str;
  getSimul()->manageError(new Error(Error::RUNTIME,message));
  }
}

// native int Memory::readByte(int offset);
//
EXPORT_PREFIX void wrap_Memory_readByte(Context * context)
{
const OMemory * memory=(const OMemory *)context->getOwner();
int offset=context->getInteger(0);
if((offset>=0)&&(offset+sizeof(char)<=memory->getSize()))
  {
  char c;
  const char * block=(const char *)memory->getBlock();
  ::memcpy(&c,block+offset,sizeof(char));
  context->returnInteger((long)c);
  }
else
  {
  context->returnInteger(0);
  Strings message="Can't read a 'byte' at offset '"+Strings(offset)+"'";
  getSimul()->manageError(new Error(Error::RUNTIME,message));
  }
}

// native void Memory::writeByte(int offset,int byteValue);
//
EXPORT_PREFIX void wrap_Memory_writeByte(Context * context)
{
OMemory * memory=(OMemory *)context->accessOwner();
int offset=context->getInteger(0);
if((offset>=0)&&(offset+sizeof(char)<=memory->getSize()))
  {
  char c=(char)context->getInteger(1);
  char * block=(char *)memory->getBlock();
  ::memcpy(block+offset,&c,sizeof(char));
  }
else
  {
  Strings message="Can't write a 'byte' at offset '"+Strings(offset)+"'";
  getSimul()->manageError(new Error(Error::RUNTIME,message));
  }
}

// native int Memory::readInt16(int offset);
//
EXPORT_PREFIX void wrap_Memory_readInt16(Context * context)
{
const OMemory * memory=(const OMemory *)context->getOwner();
int offset=context->getInteger(0);
if((offset>=0)&&(offset+sizeof(short)<=memory->getSize()))
  {
  short s;
  const char * block=(const char *)memory->getBlock();
  ::memcpy(&s,block+offset,sizeof(short));
#if defined __i386__ || defined __MSDOS__
  ::invert16((unsigned char *)&s);
#endif
  context->returnInteger((long)s);
  }
else
  {
  context->returnInteger(0);
  Strings message="Can't read a 'int16' at offset '"+Strings(offset)+"'";
  getSimul()->manageError(new Error(Error::RUNTIME,message));
  }
}

// native void Memory::writeInt16(int offset,int int16Value);
//
EXPORT_PREFIX void wrap_Memory_writeInt16(Context * context)
{
OMemory * memory=(OMemory *)context->accessOwner();
int offset=context->getInteger(0);
if((offset>=0)&&(offset+sizeof(short)<=memory->getSize()))
  {
  short s=(short)context->getInteger(1);
#if defined __i386__ || defined __MSDOS__
  ::invert16((unsigned char *)&s);
#endif
  char * block=(char *)memory->getBlock();
  ::memcpy(block+offset,&s,sizeof(short));
  }
else
  {
  Strings message="Can't write a 'int16' at offset '"+Strings(offset)+"'";
  getSimul()->manageError(new Error(Error::RUNTIME,message));
  }
}

// native int Memory::readInt32(int offset);
//
EXPORT_PREFIX void wrap_Memory_readInt32(Context * context)
{
const OMemory * memory=(const OMemory *)context->getOwner();
int offset=context->getInteger(0);
if((offset>=0)&&(offset+sizeof(long)<=memory->getSize()))
  {
  long l;
  const char * block=(const char *)memory->getBlock();
  ::memcpy(&l,block+offset,sizeof(long));
#if defined __i386__ || defined __MSDOS__
  ::invert32((unsigned char *)&l);
#endif
  context->returnInteger(l);
  }
else
  {
  context->returnInteger(0);
  Strings message="Can't read a 'int32' at offset '"+Strings(offset)+"'";
  getSimul()->manageError(new Error(Error::RUNTIME,message));
  }
}

// native void Memory::writeInt32(int offset,int int32Value);
//
EXPORT_PREFIX void wrap_Memory_writeInt32(Context * context)
{
OMemory * memory=(OMemory *)context->accessOwner();
int offset=context->getInteger(0);
if((offset>=0)&&(offset+sizeof(long)<=memory->getSize()))
  {
  long l=context->getInteger(1);
#if defined __i386__ || defined __MSDOS__
  ::invert32((unsigned char *)&l);
#endif
  char * block=(char *)memory->getBlock();
  ::memcpy(block+offset,&l,sizeof(long));
  }
else
  {
  Strings message="Can't write a 'int32' at offset '"+Strings(offset)+"'";
  getSimul()->manageError(new Error(Error::RUNTIME,message));
  }
}

// native float Memory::readFloat32(int offset);
//
EXPORT_PREFIX void wrap_Memory_readFloat32(Context * context)
{
const OMemory * memory=(const OMemory *)context->getOwner();
int offset=context->getInteger(0);
if((offset>=0)&&(offset+sizeof(float)<=memory->getSize()))
  {
  float f;
  const char * block=(const char *)memory->getBlock();
  ::memcpy(&f,block+offset,sizeof(float));
#if defined __i386__ || defined __MSDOS__
  ::invert32((unsigned char *)&f);
#endif
  context->returnReal((double)f);
  }
else
  {
  context->returnReal(0.0);
  Strings message="Can't read a 'float32' at offset '"+Strings(offset)+"'";
  getSimul()->manageError(new Error(Error::RUNTIME,message));
  }
}

// native void Memory::writeFloat32(int offset,float float32Value);
//
EXPORT_PREFIX void wrap_Memory_writeFloat32(Context * context)
{
OMemory * memory=(OMemory *)context->accessOwner();
int offset=context->getInteger(0);
if((offset>=0)&&(offset+sizeof(long)<=memory->getSize()))
  {
  float f=(float)context->getReal(1);
#if defined __i386__ || defined __MSDOS__
  ::invert32((unsigned char *)&f);
#endif
  char * block=(char *)memory->getBlock();
  ::memcpy(block+offset,&f,sizeof(float));
  }
else
  {
  Strings message="Can't write a 'float32' at offset '"+Strings(offset)+"'";
  getSimul()->manageError(new Error(Error::RUNTIME,message));
  }
}

// native float Memory::readFloat64(int offset);
//
EXPORT_PREFIX void wrap_Memory_readFloat64(Context * context)
{
const OMemory * memory=(const OMemory *)context->getOwner();
int offset=context->getInteger(0);
if((offset>=0)&&(offset+sizeof(double)<=memory->getSize()))
  {
  double d;
  const char * block=(const char *)memory->getBlock();
  ::memcpy(&d,block+offset,sizeof(double));
#if defined __i386__ || defined __MSDOS__
  ::invert64((unsigned char *)&d);
#endif
  context->returnReal(d);
  }
else
  {
  context->returnReal(0.0);
  Strings message="Can't read a 'float64' at offset '"+Strings(offset)+"'";
  getSimul()->manageError(new Error(Error::RUNTIME,message));
  }
}

// native void Memory::writeFloat64(int offset,float float64Value);
//
EXPORT_PREFIX void wrap_Memory_writeFloat64(Context * context)
{
OMemory * memory=(OMemory *)context->accessOwner();
int offset=context->getInteger(0);
if((offset>=0)&&(offset+sizeof(long)<=memory->getSize()))
  {
  double d=context->getReal(1);
#if defined __i386__ || defined __MSDOS__
  ::invert64((unsigned char *)&d);
#endif
  char * block=(char *)memory->getBlock();
  ::memcpy(block+offset,&d,sizeof(double));
  }
else
  {
  Strings message="Can't write a 'float64' at offset '"+Strings(offset)+"'";
  getSimul()->manageError(new Error(Error::RUNTIME,message));
  }
}

// native string Memory::readString(int offset,int stringLength);
//
EXPORT_PREFIX void wrap_Memory_readString(Context * context)
{
const OMemory * memory=(const OMemory *)context->getOwner();
int offset=context->getInteger(0);
int length=context->getInteger(1);
if((offset>=0)&&(length>=0)&&(offset+length<=(int)memory->getSize()))
  {
  char * str=(char *)BlockAllocator::allocate(length+1);
  const char * block=(const char *)memory->getBlock();
  ::memcpy(str,block+offset,length);
  str[length]='\0';
  context->returnStrings(str);
  BlockAllocator::free(str);
  }
else
  {
  context->returnStrings("");
  Strings message="Can't read a '"+Strings(length)+
                  " bytes string' at offset '"+Strings(offset)+"'";
  getSimul()->manageError(new Error(Error::RUNTIME,message));
  }
}

// native void Memory::writeString(int offset,string stringValue);
//
EXPORT_PREFIX void wrap_Memory_writeString(Context * context)
{
OMemory * memory=(OMemory *)context->accessOwner();
int offset=context->getInteger(0);
int length=context->getStrings(1).len();
if((offset>=0)&&(offset+length<=(int)memory->getSize()))
  {
  const char * str=(const char *)context->getStrings(1);
  char * block=(char *)memory->getBlock();
  ::memcpy(block+offset,str,length);
  }
else
  {
  Strings message="Can't write a '"+Strings(length)+
                  " bytes string' at offset '"+Strings(offset)+"'";
  getSimul()->manageError(new Error(Error::RUNTIME,message));
  }
}

//--------------------------------------------------------------------------//
// LookUp table for functions                                               //
//--------------------------------------------------------------------------//
 
EXPORT_PREFIX ExternFunction getExternFunction(const Strings & name)
{
if(name==Strings("Memory::fail()"))
      return(wrap_Memory_fail);
if(name==Strings("Memory::getSize()"))
      return(wrap_Memory_getSize);
if(name==Strings("Memory::resize()"))
      return(wrap_Memory_resize);
if(name==Strings("Memory::swap()"))
      return(wrap_Memory_swap);
if(name==Strings("Memory::copyBlock()"))
      return(wrap_Memory_copyBlock);
if(name==Strings("Memory::readByte()"))
      return(wrap_Memory_readByte);
if(name==Strings("Memory::writeByte()"))
      return(wrap_Memory_writeByte);
if(name==Strings("Memory::readInt16()"))
      return(wrap_Memory_readInt16);
if(name==Strings("Memory::writeInt16()"))
      return(wrap_Memory_writeInt16);
if(name==Strings("Memory::readInt32()"))
      return(wrap_Memory_readInt32);
if(name==Strings("Memory::writeInt32()"))
      return(wrap_Memory_writeInt32);
if(name==Strings("Memory::readFloat32()"))
      return(wrap_Memory_readFloat32);
if(name==Strings("Memory::writeFloat32()"))
      return(wrap_Memory_writeFloat32);
if(name==Strings("Memory::readFloat64()"))
      return(wrap_Memory_readFloat64);
if(name==Strings("Memory::writeFloat64()"))
      return(wrap_Memory_writeFloat64);
if(name==Strings("Memory::readString()"))
      return(wrap_Memory_readString);
if(name==Strings("Memory::writeString()"))
      return(wrap_Memory_writeString);
return((ExternFunction)0); // not found
}

//--------------------------------------------------------------------------//
// class OMemory                                                            //
//--------------------------------------------------------------------------//

OMemory::OMemory(Model * model,int instNum)
 : Agent(model,instNum),
   _size(0), _block((void *)0)
{}

OMemory::~OMemory(void)
{
if(_block) BlockAllocator::free(_block);
}

Strings OMemory::userSave(int /* column */) const // virtual
{
Strings str;
return(str);
}

