// @(#) ENIB/LI2 (c) - oRis - Version 2.3
//--------------------------------------------------------------------------//
// file        : obj2dPkg.h                                                 //
// description : header file for the ENIB/LI2 oRis package object2d.pkg     //
// author(s)   : F. Harrouet                                                //
// copyright   : Laboratoire d'Informatique Industrielle (LI2)              //
//               Ecole Nationale d'Ingenieurs de Brest (ENIB)               //
//               CP 30815 - 29608 Brest Cedex - FRANCE                      //
//               Tel: (33) 02.98.05.66.31 , Fax: (33) 02.98.05.66.29        //
//               e-mail: li2@enib.fr                                        //
//--------------------------------------------------------------------------//
// Copyright (c) 1996-2000                                                  //
// Laboratoire d'Informatique Industrielle (LI2)                            //
// Ecole Nationale d'Ingenieurs de Brest (ENIB)                             //
//                                                                          //
// Permission to use, copy, modify, distribute and sell this software       //
// and its documentation for any purpose is hereby granted without fee,     //
// provided that the above copyright notice appear in all copies and        //
// that both that copyright notice and this permission notice appear        //
// in supporting documentation.                                             //
// ENIB/LI2 makes no representations about the suitability of this          //
// software for any purpose.                                                //
// It is provided "as is" without express or implied warranty.              //
//--------------------------------------------------------------------------//

#ifndef OBJ2DPKG_H
#define OBJ2DPKG_H

#include "oRis/Main/agent.h"
#include "guiTrans.h"

//--------------------------------------------------------------------------//
// class Obj2dCB                                                            //
//--------------------------------------------------------------------------//

class Object2d;

class Obj2dCB : public ValueCallback
{
public:
  typedef void (Object2d::*cbMethod)(void);
  inline Obj2dCB(Object2d * obj,cbMethod beforeMethod,
                                cbMethod afterMethod);
  inline Obj2dCB(const Obj2dCB & cb);
  inline ~Obj2dCB(void);
  inline virtual void before(void);
  inline virtual void after(void);
protected:
  Object2d * _obj;
  Obj2dCB::cbMethod _beforeMethod;
  Obj2dCB::cbMethod _afterMethod;
};

//--------------------------------------------------------------------------//
// class Object2d                                                           //
//--------------------------------------------------------------------------//

class Object2d : public Agent
{
public:
  enum ShapeType {NOSHAPE, POINT, TEXT, LINE, SQUARE, RECTANGLE,
                  POLYLINE, POLYGON, CIRCLE, IMAGE};
  Object2d(Model * model,int instNum);
  virtual ~Object2d(void);
  virtual Strings userSave(int column=0) const;
  void setShape(Object2d::ShapeType shapeType,GuiShape2D * shape);
  void writeLocation(void);
  void beforeLoc(void);
  void afterLoc(void);
  bool isAttachedTo(const Object2d * obj) const;
  void getContent(Value & result,Model * model,bool recursive);
  static void keyNotify(GuiShape2D * calling,const Strings & key,void * data);
  static void motionNotify(GuiShape2D * calling,double dx,double dy,
                           void * data);
  static void selectNotify(GuiShape2D * calling,GuiViewer2D * viewer,
                           bool selected,void * data);
  inline const GuiShape2D * getShape(void) const;
  inline GuiShape2D * accessShape(void);

//---- Selection ----

  // native GuiViewer2d[] Object2d::isSelectedIn(void);
  //
  static void isSelectedIn(Context * context);

//---- Location ----

  // native float Object2d::getX(void);
  //
  static void getX(Context * context);

  // native float Object2d::getY(void);
  //
  static void getY(Context * context);

  // native float Object2d::getTheta(void);
  //
  static void getTheta(Context * context);

  // native void Object2d::getLocation(float & x,float & y,float & theta);
  //
  static void getLocation(Context * context);

  // native bool Object2d::getLocalBoundingBox(float & xmin,float & ymin,
  //                                           float & xmax,float & ymax);
  //
  static void getLocalBoundingBox(Context * context);

  // native bool Object2d::getGlobalBoundingBox(float & xmin,float & ymin,
  //                                            float & xmax,float & ymax);
  //
  static void getGlobalBoundingBox(Context * context);

  // native void Object2d::globalToLocalPosition(float & x,float & y);
  //
  static void globalToLocalPosition(Context * context);

  // native void Object2d::localToGlobalPosition(float & x,float & y);
  //
  static void localToGlobalPosition(Context * context);

  // native float Object2d::globalToLocalAngle(float angle);
  //
  static void globalToLocalAngle(Context * context);

  // native float Object2d::localToGlobalAngle(float angle);
  //
  static void localToGlobalAngle(Context * context);

  // native void Object2d::setX(float x);
  //
  static void setX(Context * context);
  
  // native void Object2d::setY(float y);
  //
  static void setY(Context * context);

  // native void Object2d::setTheta(float theta);
  //
  static void setTheta(Context * context);

  // native void Object2d::setLocation(float x,float y,float theta);
  //
  static void setLocation(Context * context);

//---- Motion ----

  // native void Object2d::translate(float dx,float dy);
  //
  static void translate(Context * context);

  // native void Object2d::rotate(float dtheta);
  //
  static void rotate(Context * context);

//---- Detection ----

  // native bool Object2d::isAttachedTo(Object2d container);
  //
  static void stIsAttachedTo(Context * context);

  // native Object2d[] Object2d::getContent(string className,bool recursive);
  //
  static void stGetContent(Context * context);

  // native Object2d Object2d::viewFirst(float & angle,float & distance,
  //                                     string className,float vision,
  //                                     float range,float turn);
  //
  static void viewFirst(Context * context);

  // native Object2d[] Object2d::view(float[] & angle,float[] & distance,
  //                                  string className,float vision,
  //                                  float range,float turn);
  //
  static void view(Context * context);

  // native Object2d Object2d::throwRay(float & xInter,float & yInter,
  //                                    string className);
  //
  static void throwRay(Context * context);

  // native bool Object2d::intersectRay(float & xInter,float & yInter,
  //                                    float xRay,float yRay,float thetaRay);
  //
  static void intersectRay(Context * context);

  // native bool Object2d::inside(float x,float y)
  //
  static void inside(Context * context);

//---- Representation ----

  // native string Object2d::getColor(void);
  //
  static void getColor(Context * context);

  // native void Object2d::setColor(string color);
  //
  static void setColor(Context * context);

  // native int Object2d::getLayer(void);
  //
  static void getLayer(Context * context);

  // native void Object2d::setLayer(int layer);
  //
  static void setLayer(Context * context);

  // native void Object2d::noShape(void);
  //
  static void noShape(Context * context);

  // native void Object2d::point(void);
  //
  static void point(Context * context);

  // native void Object2d::text(string text);
  //
  static void text(Context * context);

  // native void Object2d::line(float length);
  //
  static void line(Context * context);

  // native void Object2d::square(float side,bool filled);
  //
  static void square(Context * context);

  // native void Object2d::rectangle(float length,float width,bool filled);
  static void rectangle(Context * context);

  // native void Object2d::polyline(float[][] points);
  //
  static void polyline(Context * context);

  // native void Object2d::polygon(float[][] points,bool filled);
  //
  static void polygon(Context * context);

  // native void Object2d::circle(float radius,bool filled);
  //
  static void circle(Context * context);

  // native void Object2d::image(string fileName,float pixelScale);
  //
  static void image(Context * context);

  // native int Object2d::getNbColumns(void);
  //
  static void getNbColumns(Context * context);

  // native int Object2d::getNbLines(void);
  //
  static void getNbLines(Context * context);

  // native string Object2d::getFileName(void);
  //
  static void getFileName(Context * context);

  // native float Object2d::getPixelScale(void);
  //
  static void getPixelScale(Context * context);

  // native int Object2d::getNbColors(void);
  //
  static void getNbColors(Context * context);

  // native void Object2d::sortColors(void);
  //
  static void sortColors(Context * context);

  // native int Object2d::getPixel(int x,int y);
  //
  static void getPixel(Context * context);

  // native void Object2d::setPixel(int x,int y,int color);
  //
  static void setPixel(Context * context);

  // native int Object2d::getPixelAt(float x,float y);
  //
  static void getPixelAt(Context * context);

  // native void Object2d::setPixelAt(float x,float y,int color);
  //
  static void setPixelAt(Context * context);

  // native void Object2d::getRGB(int pixel,int & r,int & g,int & b);
  //
  static void getRGB(Context * context);

//---- Attributes ----
protected:
  Value * _locValue;
  Value * _supValue;
  Value * _subsValue;
  Obj2dCB * _locCB;
  double _oldX,_oldY,_oldT;
  double _x,_y,_theta;
  GuiShape2D * _shape;
  Object2d::ShapeType _shapeType;
  int _layer;
  Strings _color;
  static Word _guiViewer2dWord;
  static Word _locationWord;
  static Word _containerWord;
  static Word _contentWord;
  static Word _keyPressWord;
  static Word _mouseDragWord;
  static Word _onSelectWord;
  static Word _onUnselectWord;
};

inline const GuiShape2D * Object2d::getShape(void) const
{
return(_shape);
}

inline GuiShape2D * Object2d::accessShape(void)
{
return(_shape);
}

//--------------------------------------------------------------------------//
// class Obj2dCB                                                            //
//--------------------------------------------------------------------------//

inline Obj2dCB::Obj2dCB(Object2d * obj,Obj2dCB::cbMethod beforeMethod,
                                       Obj2dCB::cbMethod afterMethod)
 : ValueCallback(), _obj(obj),
   _beforeMethod(beforeMethod), _afterMethod(afterMethod)
{}

inline Obj2dCB::Obj2dCB(const Obj2dCB & cb)
 : ValueCallback(cb), _obj(cb._obj),
   _beforeMethod(cb._beforeMethod), _afterMethod(cb._afterMethod)
{}

inline Obj2dCB::~Obj2dCB(void)
{}

inline void Obj2dCB::before(void)
{
(_obj->*_beforeMethod)();
}

inline void Obj2dCB::after(void)
{
(_obj->*_afterMethod)();
}

#endif // OBJ2DPKG_H

