// @(#) ENIB/LI2 (c) - oRis - Version 2.3
//--------------------------------------------------------------------------//
// file        : obj3dPkg.h                                                 //
// description : header file for the ENIB/LI2 oRis package object3d.pkg     //
// author(s)   : F. Harrouet                                                //
// copyright   : Laboratoire d'Informatique Industrielle (LI2)              //
//               Ecole Nationale d'Ingenieurs de Brest (ENIB)               //
//               CP 30815 - 29608 Brest Cedex - FRANCE                      //
//               Tel: (33) 02.98.05.66.31 , Fax: (33) 02.98.05.66.29        //
//               e-mail: li2@enib.fr                                        //
//--------------------------------------------------------------------------//
// Copyright (c) 1996-2000                                                  //
// Laboratoire d'Informatique Industrielle (LI2)                            //
// Ecole Nationale d'Ingenieurs de Brest (ENIB)                             //
//                                                                          //
// Permission to use, copy, modify, distribute and sell this software       //
// and its documentation for any purpose is hereby granted without fee,     //
// provided that the above copyright notice appear in all copies and        //
// that both that copyright notice and this permission notice appear        //
// in supporting documentation.                                             //
// ENIB/LI2 makes no representations about the suitability of this          //
// software for any purpose.                                                //
// It is provided "as is" without express or implied warranty.              //
//--------------------------------------------------------------------------//

#ifndef OBJ3DPKG_H
#define OBJ3DPKG_H

#include "oRis/Main/agent.h"
#include "guiTrans.h"

//--------------------------------------------------------------------------//
// class Obj3dCB                                                            //
//--------------------------------------------------------------------------//

class Object3d;

class Obj3dCB : public ValueCallback
{
public:
  typedef void (Object3d::*cbMethod)(void);
  inline Obj3dCB(Object3d * obj,cbMethod beforeMethod,
                                cbMethod afterMethod);
  inline Obj3dCB(const Obj3dCB & cb);
  inline ~Obj3dCB(void);
  inline virtual void before(void);
  inline virtual void after(void);
protected:
  Object3d * _obj;
  Obj3dCB::cbMethod _beforeMethod;
  Obj3dCB::cbMethod _afterMethod;
};

//--------------------------------------------------------------------------//
// class Object3d                                                           //
//--------------------------------------------------------------------------//

class Object3d : public Agent
{
public:
  Object3d(Model * model,int instNum);
  virtual ~Object3d(void);
  virtual Strings userSave(int column=0) const;
  void writeLocation(void);
  void beforeLoc(void);
  void afterLoc(void);
  bool isAttachedTo(const Object3d * obj) const;
  void getContent(Value & result,Model * model,bool recursive);
  static void keyNotify(GuiShape3D * calling,const Strings & key,void * data);
  static void motionNotify(GuiShape3D *,double dx,double dy,double dz,
                           void * data);
  static void selectNotify(GuiShape3D * calling,GuiViewer3D * viewer,
                           bool selected,void * data);
  inline const GuiShape3D * getShape(void) const;
  inline GuiShape3D * accessShape(void);


//---- Selection ----
 
  // native GuiViewer3d[] Object3d::isSelectedIn(void);
  //
  static void isSelectedIn(Context * context);

//---- Location ----

  // native void Object3d::getPosition(float & x,float & y,float & z);
  //
  static void getPosition(Context * context);

  // native void Object3d::globalToLocalPosition(float & x,
  //                                             float & y,float & z);
  //
  static void globalToLocalPosition(Context * context);

  // native void Object3d::localToGlobalPosition(float & x,
  //                                             float & y,float & z);
  //
  static void localToGlobalPosition(Context * context);

  // native void Object3d::getOrientation(float & roll,float & pitch,
  //                                      float & yaw);
  //
  static void getOrientation(Context * context);

  // native void Object3d::globalToLocalOrientation(float & roll,
  //                                                float & pitch,float & yaw);
  //
  static void globalToLocalOrientation(Context * context);

  // native void Object3d::localToGlobalOrientation(float & roll,
  //                                                float & pitch,float & yaw);
  //
  static void localToGlobalOrientation(Context * context);

  // native bool Object3d::getLocalBoundingBox(
  //                       float & xmin,float & ymin,float & zmin,
  //                       float & xmax,float & ymax,float & zmax);
  //
  static void getLocalBoundingBox(Context * context);

  // native bool Object3d::getGlobalBoundingBox(
  //                       float & xmin,float & ymin,float & zmin,
  //                       float & xmax,float & ymax,float & zmax);
  //
  static void getGlobalBoundingBox(Context * context);

  // native int Object3d::getNbPoints(void);
  //
  static void getNbPoints(Context * context);

  // native int Object3d::getNbTriangles(void);
  //
  static void getNbTriangles(Context * context);

  // native void Object3d::setPosition(float x,float y,float z);
  //
  static void setPosition(Context * context);

  // native void Object3d::cancelRotation(void);
  //
  static void cancelRotation(Context * context);

  // native void Object3d::setOrientation(float roll,float pitch,float yaw);
  //
  static void setOrientation(Context * context);

//---- Motion ----

  // native void Object3d::translate(float dx,float dy,float dz);
  //
  static void translate(Context * context);

  // native void Object3d::roll(float dRoll);
  //
  static void roll(Context * context);

  // native void Object3d::pitch(float dPitch);
  //
  static void pitch(Context * context);

  // native void Object3d::yaw(float dYaw);
  //
  static void yaw(Context * context);

//---- Detection ----

  // native bool Object3d::isAttachedTo(Object3d container);
  //
  static void stIsAttachedTo(Context * context);

  // native Object3d[] Object3d::getContent(string className,bool recursive);
  //
  static void stGetContent(Context * context);

  // native Object3d Object3d::viewFirst(float & xyAngle,float & zAngle,
  //                                     float & distance,string className,
  //                                     float xyVision,float zVision,
  //                                     float range);
  //
  static void viewFirst(Context * context);

  // native Object3d[] Object3d::view(float[] & xyAngle,float[] & zAngle,
  //                                  float[] & distance,string className,
  //                                  float xyVision,float zVision,
  //                                  float range);
  //
  static void view(Context * context);

  // native Object3d Object3d::throwRay(float & xInter,float & yInter,
  //                                    float & zInter,int & faceNumber,
  //                                    string className);
  //
  static void throwRay(Context * context);

  // native bool Object3d::intersectRay(float & xInter,float & yInter,
  //                                    float & zInter,int & faceNumber,
  //                                    float xRay,float yRay,float zRay,
  //                                    float yawRay,float pitchRay);
  //
  static void intersectRay(Context * context);

  // native bool Object3d::inside(float x,float y,float z)
  //
  static void inside(Context * context);

//---- Representation ----

  // native bool Object3d::readShape(Shape3d shape);
  //
  static void readShape(Context * context);

//---- Attributes ----
protected:
  GuiShape3D * _shape;
  Value * _locValue;
  Value * _supValue;
  Value * _subsValue;
  Obj3dCB * _locCB;
  double _oldX,_oldY,_oldZ;
  double _oldRoll,_oldPitch,_oldYaw;
  static Word _guiViewer3dWord;
  static Word _locationWord;
  static Word _containerWord;
  static Word _contentWord;
  static Word _keyPressWord;
  static Word _mouseDragWord;
  static Word _onSelectWord;
  static Word _onUnselectWord;
};

inline const GuiShape3D * Object3d::getShape(void) const
{
return(_shape);
}

inline GuiShape3D * Object3d::accessShape(void)
{
return(_shape);
}

//--------------------------------------------------------------------------//
// class Shape3d                                                            //
//--------------------------------------------------------------------------//

class Shape3d : public Agent
{
public:
  Shape3d(Model * model,int instNum);
  virtual ~Shape3d(void);
  virtual Strings userSave(int column=0) const;
  inline const Gui3DPointVect & getPoints(void) const;
  inline const Gui3DTriangleVect & getFaces(void) const;
  inline const Gui3DColorVect & getColors(void) const;
  inline const Gui3DTextureVect & getTextures(void) const;
  inline Gui3DPointVect & accessPoints(void);
  inline Gui3DTriangleVect & accessFaces(void);
  inline Gui3DColorVect & accessColors(void);
  inline Gui3DTextureVect & accessTextures(void);
  void insertTexture(const Gui3DTexture & texture);

//-------- Points --------

  // native float[][] Shape3d::getPoints(void);
  //
  static void stGetPoints(Context * context);

  // native void Shape3d::addPoints(float[][] newPoints);
  //
  static void addPoints(Context * context);

  // native void Shape3d::erasePoints(void);
  //
  static void erasePoints(Context * context);

//-------- Triangles --------

  // native int[][] Shape3d::getTriangles(void);
  //
  static void getTriangles(Context * context);

  // native void Shape3d::addTriangles(string color,int[][] newTriangles);
  //
  static void addTriangles(Context * context);

  // native void Shape3d::eraseTriangles(void);
  //
  static void eraseTriangles(Context * context);

//-------- Primitives --------

  // native void Shape3d::addCube(string color);
  //
  static void addCube(Context * context);

  // native void Shape3d::addCone(string color,int precision);
  //
  static void addCone(Context * context);

  // native void Shape3d::addCylinder(string color,int precision);
  //
  static void addCylinder(Context * context);

  // native void Shape3d::addTube(string color,int precision,float rate);
  //
  static void addTube(Context * context);

  // native void Shape3d::addSquareTube(string color,float rate);
  //
  static void addSquareTube(Context * context);

  // native void Shape3d::addSphere(string color,
  //                                int zPrecision,int xyPrecision);
  //
  static void addSphere(Context * context);

  // native void Shape3d::addGeode(string color,int slices,int precision);
  //
  static void addGeode(Context * context);

  // native void Shape3d::addText(string color,string text,float spacing);
  //
  static void addText(Context * context);

//-------- Transformations --------

  // native void Shape3d::erase(void);
  //
  static void erase(Context * context);

  // native void Shape3d::addShape(Shape3d shape);
  //
  static void addShape(Context * context);

  // native void Shape3d::setColor(string color);
  //
  static void setColor(Context * context);

  // native void Shape3d::setTexture(string fileName,int face,
  //                                 float x1,float y1,
  //                                 float x2,float y2,
  //                                 float x3,float y3);
  //
  static void setTexture(Context * context);

  // native void Shape3d::setCubeTexture(string fileName,string where);
  //
  static void setCubeTexture(Context * context);
  
  // native void Shape3d::mapTexture(string fileName,bool halfMap);
  //
  static void mapTexture(Context * context);
  
  // native void Shape3d::reverseTriangles(void);
  //
  static void reverseTriangles(Context * context);

  // native void Shape3d::translatePoints(float dx,float dy,float dz);
  //
  static void translatePoints(Context * context);

  // native void Shape3d::rollPoints(float roll);
  //
  static void rollPoints(Context * context);

  // native void Shape3d::pitchPoints(float pitch);
  //
  static void pitchPoints(Context * context);

  // native void Shape3d::yawPoints(float yaw);
  //
  static void yawPoints(Context * context);

  // native void Shape3d::scalePoints(float sx,float sy,float sz);
  //
  static void scalePoints(Context * context);

//---- Attributes ----
protected:
  Gui3DPointVect _points;
  Gui3DTriangleVect _faces;
  Gui3DColorVect _colors;
  Gui3DTextureVect _textures;
};

//--------------------------------------------------------------------------//
// class Obj3dCB                                                            //
//--------------------------------------------------------------------------//

inline Obj3dCB::Obj3dCB(Object3d * obj,Obj3dCB::cbMethod beforeMethod,
                                       Obj3dCB::cbMethod afterMethod)
 : ValueCallback(), _obj(obj),
   _beforeMethod(beforeMethod), _afterMethod(afterMethod)
{}

inline Obj3dCB::Obj3dCB(const Obj3dCB & cb)
 : ValueCallback(cb), _obj(cb._obj),
   _beforeMethod(cb._beforeMethod), _afterMethod(cb._afterMethod)
{}

inline Obj3dCB::~Obj3dCB(void)
{}

inline void Obj3dCB::before(void)
{
(_obj->*_beforeMethod)();
}

inline void Obj3dCB::after(void)
{
(_obj->*_afterMethod)();
}

//--------------------------------------------------------------------------//
// class Shape3d                                                            //
//--------------------------------------------------------------------------//

inline const Gui3DPointVect & Shape3d::getPoints(void) const
{
return(_points);
}

inline const Gui3DTriangleVect & Shape3d::getFaces(void) const
{
return(_faces);
}

inline const Gui3DColorVect & Shape3d::getColors(void) const
{
return(_colors);
}

inline const Gui3DTextureVect & Shape3d::getTextures(void) const
{
return(_textures);
}

inline Gui3DPointVect & Shape3d::accessPoints(void)
{
return(_points);
}

inline Gui3DTriangleVect & Shape3d::accessFaces(void)
{
return(_faces);
}

inline Gui3DColorVect & Shape3d::accessColors(void)
{
return(_colors);
}

inline Gui3DTextureVect & Shape3d::accessTextures(void)
{
return(_textures);
}

#endif // OBJ3DPKG_H

