// @(#) ENIB/LI2 (c) - oRis - Version 2.3
//--------------------------------------------------------------------------//
// file        : prologPkg.cpp                                              //
// description : source file for the ENIB/LI2 oRis / SWI-Prolog interface   //
// author(s)   : P. Reignier                                                //
// copyright   : Laboratoire d'Informatique Industrielle (LI2)              //
//               Ecole Nationale d'Ingenieurs de Brest (ENIB)               //
//               CP 30815 - 29608 Brest Cedex - FRANCE                      //
//               Tel: (33) 02.98.05.66.31 , Fax: (33) 02.98.05.66.29        //
//               e-mail: li2@enib.fr                                        //
//--------------------------------------------------------------------------//
// Copyright (c) 1996-2000                                                  //
// Laboratoire d'Informatique Industrielle (LI2)                            //
// Ecole Nationale d'Ingenieurs de Brest (ENIB)                             //
//                                                                          //
// Permission to use, copy, modify, distribute and sell this software       //
// and its documentation for any purpose is hereby granted without fee,     //
// provided that the above copyright notice appear in all copies and        //
// that both that copyright notice and this permission notice appear        //
// in supporting documentation.                                             //
// ENIB/LI2 makes no representations about the suitability of this          //
// software for any purpose.                                                //
// It is provided "as is" without express or implied warranty.              //
//--------------------------------------------------------------------------//

extern "C" {
#include <SWI-Prolog.h>
extern bool seeString(char* s) ;
extern bool seenString(void) ;
}

#include "Prolog/prologPkg.h"
#include "oRis/Main/context.h"
#include "oRis/Main/errors.h"

#ifdef FUZZY_PL
#include "softPl/fuzzyProlog.h"
#endif

#ifndef __MSDOS__
#define EXPORT_PREFIX extern "C"
#else
#include <windows.h>
#define EXPORT_PREFIX extern "C" __declspec(dllexport)
#define openingLibrary    openingPrologLibrary
#define closingLibrary    closingPrologLibrary
#define getExternFunction getExternPrologFunction
#define getNewAgent       getNewPrologAgent
#endif

//--------------------------------------------------------------------------//
// Opening and closing the library                                          //
//--------------------------------------------------------------------------//

EXPORT_PREFIX void openingLibrary(void)
{
if(!PrologPkg::instance) (void)new PrologPkg;
}

EXPORT_PREFIX void closingLibrary(void)
{
if(PrologPkg::instance) delete PrologPkg::instance;
}

//--------------------------------------------------------------------------//
// LookUp table for functions                                               //
//--------------------------------------------------------------------------//

EXPORT_PREFIX ExternFunction getExternFunction(const Strings & name)
{
if(name==Strings("::callProlog()"))  return(PrologPkg::staticCallProlog);
return((ExternFunction)0); // not found
}

//--------------------------------------------------------------------------//
// LookUp table for agents                                                  //
//--------------------------------------------------------------------------//

EXPORT_PREFIX Agent * getNewAgent(Model * /* model */ , long /* instNum */ )
{
return((Agent *)0); // not found
}

//----------------------------------------------------------------------//
//  Constructeurs et destructeurs
//----------------------------------------------------------------------//

PrologPkg * PrologPkg::instance=(PrologPkg *)0;

PrologPkg::PrologPkg(void)
 : _insideQuery(false), _queryId((qid_t)0), _goal((term_t)0),
   _oris2Prolog((predicate_t)0), _fail(false)
{
PrologPkg::instance=this;
Strings fileName("Implmnt/init.qlf") ;
Strings name=getSimul()->locateFile(fileName);
if(!name.len())
  {
  Strings message="PROLOG: Fichier <"+
                  fileName+
                  "> introuvable";
  getSimul()->manageError(new Error(Error::RUNTIME,message));
  _fail=true;
  }
else
  {
  char * argv[4];
  Strings load="qload('"+name+"')";
  argv[0]=new char[getSimul()->getCommandLine()[0].len()+1];
  strcpy(argv[0],getSimul()->getCommandLine()[0]);
  argv[1]="-g";
  argv[2]=new char[load.len()+1];
  strcpy(argv[2],load);
  argv[3]=NULL;
  if(!PL_initialise(3,argv)) _fail=true;
  else
    {
    // declaration de la fonction de lien Prolog vers oRis
    PL_register_foreign("prolog2Oris",4, 
  		        (pl_function_t)PrologPkg::staticOrisCall, 
  		        0);
    // on recupere la reference du predicat oris2Prolog
    _oris2Prolog=PL_predicate("oris2Prolog",0,NULL);
#ifdef FUZZY_PL
    (void)new FuzzyProlog;
#endif
    PL_install_readline();
    }
  delete []argv[0];
  delete []argv[2];
  }
}

//----------------------------------------------------------------------//
PrologPkg::~PrologPkg(void)
{
PrologPkg::instance=(PrologPkg *)0;
}

//----------------------------------------------------------------------//
// Interface avec l'interpreteur prolog
//----------------------------------------------------------------------//

void PrologPkg::callProlog(Strings but)
{
char * buffer=new char[but.len()+1];
strcpy(buffer,but);
// conversion de l'ordre prolog exprime sous forme d'une chaine de 
// caractere en un terme prolog. On utilise pour cela le predicat prolog
// read(X) sur le flot d'entree redirige vers la chaine a analyser (ouf !)
seeString(buffer); // le buffer est referme par le predicat oris2Prolog
PL_call_predicate(NULL,FALSE,_oris2Prolog,(long unsigned int)NULL);
delete []buffer ;
}

//----------------------------------------------------------------------//
// Lien avec les fonctions oRis
//----------------------------------------------------------------------//

void PrologPkg::staticCallProlog(Context * context) // static
{
if(PrologPkg::instance->_fail)
  {
  Strings message="PROLOG: interpreteur mal initialise";
  getSimul()->manageError(new Error(Error::RUNTIME,message));
  }
else PrologPkg::instance->callProlog(context->getStrings(0));
}

//----------------------------------------------------------------------//
// Lien avec les fonctions prolog
//----------------------------------------------------------------------//

foreign_t PrologPkg::staticOrisCall(term_t agtName, 
				    term_t methodeName,
				    term_t paramList,
				    term_t result)
{
int i;
bool fail=false;
ValueVect callParams;
const Parameter * param=(const Parameter *)0;
Execution * initExec=(Execution *)0;
Value * orisResult=(Value *)0;
char * tmpStr=(char *)0;
Strings name,methode;
term_t head=(term_t)0;
term_t list=(term_t)0;
term_t head2=(term_t)0;
term_t list2=(term_t)0;
Agent * agt=(Agent *)0;
ExecModule * module=(ExecModule *)0;
int nbParamsRecus=0;
Error * error=(Error *)0;
if(PrologPkg::instance->_fail)
  {
  fail=true;
  Strings message="PROLOG: interpreteur mal initialise";
  getSimul()->manageError(new Error(Error::RUNTIME,message));
  goto clearAndExit;
  }
PL_get_chars(agtName, &tmpStr, CVT_ALL);
name=tmpStr;
PL_get_chars(methodeName, &tmpStr, CVT_ALL);
methode=tmpStr;
head=PL_new_term_ref();           // copie de paramList car elle
list=PL_copy_term_ref(paramList); // va etre modifiee
agt=getSimul()->accessAgent(AgentName(name));
if(!agt)
  {
  fail=true;
  Strings message="PROLOG: L'agent <"+name+"> n'existe pas";
  getSimul()->manageError(new Error(Error::RUNTIME,message));
  goto clearAndExit;
  }
module=agt->accessModule(Word::create(methode));
if(!module)
  {
  fail=true;
  Strings message="PROLOG: La methode <"+methode+
                  "> pour l'agent <"+name+"> n'existe pas";
  getSimul()->manageError(new Error(Error::RUNTIME,message));
  goto clearAndExit;
  }
for(nbParamsRecus=0;nbParamsRecus<module->getNbParams();nbParamsRecus++)
  {
  if(PL_get_nil(list))
    {
    fail=true;
    Strings message="PROLOG: Pas assez de parametres pour la methode <"+
                    methode+"> de l'agent <"+name+">";
    getSimul()->manageError(new Error(Error::RUNTIME,message)) ;
    goto clearAndExit;                                                        
    }
  else PL_get_list(list,head,list);
  param=module->getParam(nbParamsRecus);
  callParams.pushBack(LocalValue::getLocalValue(ValueType(param->getType())));
  if(param->getType().getDepth()) fail=true; // no vector
  if(!fail)
    {
    if(PrologPkg::instance->_buildValue(param,head,callParams.accessBack()))
      {
      if(param->getMode()==Parameter::REFERENCE)
        {
        Strings message="PROLOG: Le parametre <"+
                        param->getName().getStrings()+
                        "de la methode <"+methode+"> de l'agent <"+name+
                        "> devrait etre une variable non instanciee";
        getSimul()->manageError(new Error(Error::WARNING,message));
        }
      }
    else
      {
      if(param->getMode()!=Parameter::REFERENCE) fail=true;
      }
    }
  if(fail)
    {
    Strings message="PROLOG: Le parametre <"+
                    param->getName().getStrings()+
                    "> de la methode <"+methode+"> de l'agent <"+name+
                    "> est de type incorrect";
    getSimul()->manageError(new Error(Error::RUNTIME,message));
    goto clearAndExit;
    }
  }
if(!PL_get_nil(list))
  {
  fail=true;
  Strings message="PROLOG: Trop de parametres pour la methode <"+
                  methode+"> de l'agent <"+name+">";
  getSimul()->manageError(new Error(Error::RUNTIME,message)) ;
  goto clearAndExit;
  }
initExec=Execution::getExecution(agt,ExecModule::NORMAL);
for(i=callParams.getSize();i--;) initExec->pushValue(callParams[i]);
error=module->tryToCall(initExec,module->getNbParams());
if(error)
  {
  fail=true;
  Error * tmp=error; // convert from Semantic to Run-Time
  error=new Error(Error::RUNTIME,"PROLOG: "+tmp->getMessage());
  delete tmp;
  getSimul()->manageError(error);
  goto clearAndExit;
  }  
getSimul()->executeModule(module,agt,initExec) ;
if(initExec->isAborted())
  {
  fail=true;
  goto clearAndExit;
  }
if(module->getResultType()!=ValueType(ValueType::UNKNOWN))
  orisResult=initExec->popValue();
head2=PL_new_term_ref();           // copie de paramList car elle
list2=PL_copy_term_ref(paramList); // va etre modifiee
for(i=0;i<module->getNbParams();i++)
  {
  PL_get_list(list2,head2,list2);
  if(module->getParam(i)->getMode()==Parameter::REFERENCE)
    {
    switch(callParams[i]->getType().getSimpleType())
      {
      case ValueType::INTEGER:
        fail=!PL_unify_integer(head2,callParams[i]->getInteger());
	break ;
      case ValueType::REAL:
        fail=!PL_unify_float(head2,callParams[i]->getReal());
        break ;
      case ValueType::STRING:
        fail=!PL_unify_string_chars(head2,callParams[i]->getStrings());
        break ;
      case ValueType::AGENT:
        fail=!PL_unify_string_chars(head2,callParams[i]->print());
        break;
      default:
        break ;
      }
    if(fail)
      {
      Strings message="PROLOG: Unification de l'argument <"+
                      module->getParam(i)->getName().getStrings()+
                      "> de la methode <"+
                      methode+"> de l'agent <"+name+"> impossible";
      getSimul()->manageError(new Error(Error::RUNTIME,message));
      goto clearAndExit;
      }
    }
  }
if(module->getResultType().getDepth()) fail=true; // no vector
switch(module->getResultType().getSimpleType())
  {
  case ValueType::INTEGER:
    fail=!PL_unify_integer(result,orisResult->getInteger());
    break;
  case ValueType::REAL:
    fail=!PL_unify_float(result,orisResult->getReal());
    break;
  case ValueType::STRING:
    fail=!PL_unify_string_chars(result,orisResult->getStrings());
    break;
  case ValueType::AGENT:
    fail=!PL_unify_string_chars(result,orisResult->print());
    break;
  default:
    break;
  }
if(fail)
  {
  Strings message="PROLOG: Unification du resultat de la methode <"+
                  methode+"> de l'agent <"+name+"> impossible";
  getSimul()->manageError(new Error(Error::RUNTIME,message));
  goto clearAndExit;
  }
clearAndExit:
  // head list head2 list2 are automatically destroyed on exit
  if(initExec) Execution::relaxExecution(initExec);
  while(callParams.getSize())
    LocalValue::relaxLocalValue((LocalValue *) callParams.readPopBack());
  if(orisResult) orisResult->relax();
  if(fail) PL_fail;
  else PL_succeed;
}

//----------------------------------------------------------------------//
//  Fonction utilitaire
//----------------------------------------------------------------------//

bool PrologPkg::_buildValue(const Parameter * param, 
			    term_t term,
			    Value * value)
{
switch(param->getType().getSimpleType())
  {
  case ValueType::INTEGER:
    {
    long a;
    if(!PL_get_long(term,&a)) return(false);
    else value->setInteger(a);
    }
    break;
  case ValueType::REAL:
    {
    int a;
    double b;
    if(!PL_get_float(term,&b))
      {
      if(!PL_get_integer(term, &a)) return(false);
      else value->setReal((double)a);
      }
    else value->setReal(b);
    }
    break;
  case ValueType::STRING:
    {
    char * a;
    int len;
    if(!PL_get_string(term,&a,&len)) return(false);
    else value->setStrings(a);
    }
    break;
  case ValueType::AGENT:
    {
    char * a;
    int len;
    if(!PL_get_string(term,&a,&len)) return(false);
    else
      {
      Agent * agt=getSimul()->accessAgent(AgentName(Strings(a)));
      if(!agt) return(false);
      else
        {
	value->setAgent(agt);
	if(!value->getType().canCastTo(param->getType())) return(false);
        }
      }
    }
    break;
  default :
    return(false);
  }
return(true);
}

