// @(#) ENIB/LI2 (c) - oRis - Version 2.3
//--------------------------------------------------------------------------//
// file        : reflPkg.cpp                                                //
// description : source file for the ENIB/LI2 oRis package reflect.pkg      //
// author(s)   : F. Harrouet                                                //
// copyright   : Laboratoire d'Informatique Industrielle (LI2)              //
//               Ecole Nationale d'Ingenieurs de Brest (ENIB)               //
//               CP 30815 - 29608 Brest Cedex - FRANCE                      //
//               Tel: (33) 02.98.05.66.31 , Fax: (33) 02.98.05.66.29        //
//               e-mail: li2@enib.fr                                        //
//--------------------------------------------------------------------------//
// Copyright (c) 1996-2000                                                  //
// Laboratoire d'Informatique Industrielle (LI2)                            //
// Ecole Nationale d'Ingenieurs de Brest (ENIB)                             //
//                                                                          //
// Permission to use, copy, modify, distribute and sell this software       //
// and its documentation for any purpose is hereby granted without fee,     //
// provided that the above copyright notice appear in all copies and        //
// that both that copyright notice and this permission notice appear        //
// in supporting documentation.                                             //
// ENIB/LI2 makes no representations about the suitability of this          //
// software for any purpose.                                                //
// It is provided "as is" without express or implied warranty.              //
//--------------------------------------------------------------------------//

#include "Reflect/reflPkg.h"
#include "oRis/Main/simul.h"
#include "oRis/Main/extMod.h"
#include "oRis/Main/context.h"

#include <stdio.h>

#ifndef __MSDOS__
#define EXPORT_PREFIX extern "C"
#else
#include <windows.h>
#define EXPORT_PREFIX extern "C" __declspec(dllexport)
#define openingLibrary    openingReflLibrary
#define closingLibrary    closingReflLibrary
#define getExternFunction getExternReflFunction
#define getNewAgent       getNewReflAgent
#endif

//--------------------------------------------------------------------------//
// Opening and closing the library                                          //
//--------------------------------------------------------------------------//
 
EXPORT_PREFIX void openingLibrary(void)
{
  // nothing to do
}
 
EXPORT_PREFIX void closingLibrary(void)
{
  // nothing to do
}

//--------------------------------------------------------------------------//
// LookUp table for functions                                               //
//--------------------------------------------------------------------------//

EXPORT_PREFIX ExternFunction getExternFunction(const Strings & name)
{
if(name==Strings("AttributeDescription::readInClass()"))
  return(AttributeDescription::readInClass);
if(name==Strings("AttributeDescription::readInObject()"))
  return(AttributeDescription::readInObject);

if(name==Strings("ModuleDescription::readGlobal()"))
  return(ModuleDescription::readGlobal);
if(name==Strings("ModuleDescription::readInClass()"))
  return(ModuleDescription::readInClass);
if(name==Strings("ModuleDescription::readInObject()"))
  return(ModuleDescription::readInObject);

if(name==Strings("ObjectDescription::read()"))
  return(ObjectDescription::read);

if(name==Strings("ClassDescription::read()"))
  return(ClassDescription::read);

if(name==Strings("GlobalDescription::read()"))
  return(GlobalDescription::read);

if(name==Strings("TypeDescription::read()"))
  return(TypeDescription::read);
if(name==Strings("TypeDescription::isInteger()"))
  return(TypeDescription::isInteger);
if(name==Strings("TypeDescription::isFloat()"))
  return(TypeDescription::isFloat);
if(name==Strings("TypeDescription::isString()"))
  return(TypeDescription::isString);
if(name==Strings("TypeDescription::isObject()"))
  return(TypeDescription::isObject);
if(name==Strings("TypeDescription::isVoid()"))
  return(TypeDescription::isVoid);
if(name==Strings("TypeDescription::getBaseType()"))
  return(TypeDescription::getBaseType);
if(name==Strings("TypeDescription::getDimension()"))
  return(TypeDescription::getDimension);
if(name==Strings("TypeDescription::isReference()"))
  return(TypeDescription::isReference);

return((ExternFunction)0); // not found
}

//--------------------------------------------------------------------------//
// LookUp table for agents                                                  //
//--------------------------------------------------------------------------//

static Word attributeDescriptionWord(Word::create("AttributeDescription"));
static Word moduleDescriptionWord(Word::create("ModuleDescription"));
static Word objectDescriptionWord(Word::create("ObjectDescription"));
static Word classDescriptionWord(Word::create("ClassDescription"));
static Word globalDescriptionWord(Word::create("GlobalDescription"));
static Word typeDescriptionWord(Word::create("TypeDescription"));

EXPORT_PREFIX Agent * getNewAgent(Model * model,long instNum)
{
if(model->isA(::attributeDescriptionWord))
  return(new AttributeDescription(model,instNum));
if(model->isA(::moduleDescriptionWord))
  return(new ModuleDescription(model,instNum));
if(model->isA(::objectDescriptionWord))
  return(new ObjectDescription(model,instNum));
if(model->isA(::classDescriptionWord))
  return(new ClassDescription(model,instNum));
if(model->isA(::globalDescriptionWord))
  return(new GlobalDescription(model,instNum));
if(model->isA(::typeDescriptionWord))
  return(new TypeDescription(model,instNum));
return((Agent *)0); // not found
}

//--------------------------------------------------------------------------//
// AttributeDescription functions                                           //
//--------------------------------------------------------------------------//

// native int AttributeDescription::readInClass(string className,
//                                              string attrName);
//
void AttributeDescription::readInClass(Context * context) // static
{
AttributeDescription * attrD=(AttributeDescription *)context->accessOwner();
int status=1;
const Model * model=getSimul()->getModel(Word::create(context->getStrings(0)));
if(model)
  {
  const Attribute * attribute=model->getAttribute(
                              Word::create(context->getStrings(1)));
  if(attribute)
    {
    attrD->_attrClass->setStrings(attribute->getModel()->getName()
                                                        .getStrings());
    attrD->_attrOwner->setAgent((Agent *)0);
    attrD->_attrType->setStrings(attribute->getType().print());
    attrD->_attrName->setStrings(attribute->getName().getStrings());
    attrD->_attrValue->setStrings(attribute->getValue()->dump());
    }
  else status=0;
  }
else status=0;
if(!status)
  {
  attrD->_attrType->setStrings(Strings());
  attrD->_attrClass->setStrings(Strings());
  attrD->_attrOwner->setAgent((Agent *)0);
  attrD->_attrName->setStrings(Strings());
  attrD->_attrValue->setStrings(Strings());
  }
context->returnInteger(status);
}

// native int AttributeDescription::readInObject(Object obj,string attrName);
//
void AttributeDescription::readInObject(Context * context) // static
{
AttributeDescription * attrD=(AttributeDescription *)context->accessOwner();
int status=1;
Agent * agent=context->getAgent(0);
if(agent&&agent->isAlive())
  {
  const Attribute * attribute=agent->getAttribute(
                                     Word::create(context->getStrings(1)));
  if(attribute)
    {
    if(attribute->getOwner())
      {
      attrD->_attrClass->setStrings(Strings());
      attrD->_attrOwner->setAgent((Agent *)attribute->getOwner());
      }
    else
      {
      attrD->_attrClass->setStrings(attribute->getModel()->getName()
                                                          .getStrings());
      attrD->_attrOwner->setAgent((Agent *)0);
      }
    attrD->_attrType->setStrings(attribute->getType().print());
    attrD->_attrName->setStrings(attribute->getName().getStrings());
    attrD->_attrValue->setStrings(attribute->getValue()->dump());
    }
  else status=0;
  }
else status=0;
if(!status)
  {
  attrD->_attrType->setStrings(Strings());
  attrD->_attrClass->setStrings(Strings());
  attrD->_attrOwner->setAgent((Agent *)0);
  attrD->_attrName->setStrings(Strings());
  attrD->_attrValue->setStrings(Strings());
  }
context->returnInteger(status);
}

//--------------------------------------------------------------------------//
// ModuleDescription functions                                              //
//--------------------------------------------------------------------------//

// native int ModuleDescription::readGlobal(string moduleName);
//
void ModuleDescription::readGlobal(Context * context) // static
{
ModuleDescription * moduleD=(ModuleDescription *)context->accessOwner();
int status=1;
moduleD->_moduleClass->setStrings(Strings());
moduleD->_moduleOwner->setAgent((Agent *)0);
const ExecModule * module=getSimul()->getModule(
                                      Word::create(context->getStrings(0)));
if(module)
  {
  int i;
  moduleD->_moduleType->setStrings(module->getResultType().print());
  moduleD->_moduleName->setStrings(module->getName().getStrings());
  SubValueVect * nameVect=SubValue::getSubValueVect();
  SubValueVect * typeVect=SubValue::getSubValueVect();
  for(i=0;i<module->getNbParams();i++)
    {
    nameVect->pushBack(SubValue::getSubValue(ValueType(ValueType::STRING),
                                             moduleD->_moduleParamNames));
    nameVect->accessBack()->setStrings(module->getParam(i)->getName()
                                                           .getStrings());
    typeVect->pushBack(SubValue::getSubValue(ValueType(ValueType::STRING),
                                             moduleD->_moduleParamTypes));
    if(module->getParam(i)->getMode()==Parameter::REFERENCE)
      typeVect->accessBack()
              ->setStrings(module->getParam(i)->getType().print()+" &");
    else
      typeVect->accessBack()
              ->setStrings(module->getParam(i)->getType().print());
    }
  moduleD->_moduleParamNames->setValueVector(nameVect);
  moduleD->_moduleParamTypes->setValueVector(typeVect);
  nameVect=SubValue::getSubValueVect();
  typeVect=SubValue::getSubValueVect();
  for(i=0;i<module->getNbLocals();i++)
    {
    nameVect->pushBack(SubValue::getSubValue(ValueType(ValueType::STRING),
                                             moduleD->_moduleLocalNames));
    nameVect->accessBack()->setStrings(module->getLocal(i)->getName()
                                                           .getStrings());
    typeVect->pushBack(SubValue::getSubValue(ValueType(ValueType::STRING),
                                             moduleD->_moduleLocalTypes));
    typeVect->accessBack()->setStrings(module->getLocal(i)->getType().print());
    }
  moduleD->_moduleLocalNames->setValueVector(nameVect);
  moduleD->_moduleLocalTypes->setValueVector(typeVect);
  switch(module->getType())
    {
    case ExecModule::PROCEDURE:
      moduleD->_moduleDefined->setInteger(1);
      moduleD->_moduleNativeCode->setStrings(Strings());
      moduleD->_moduleCode->setStrings(module->printCode());
      break;
    case ExecModule::EXTERN:
      moduleD->_moduleDefined->setInteger(1);
      moduleD->_moduleNativeCode
             ->setStrings(((const ExternModule *)module)->getNativeName());
      moduleD->_moduleCode->setStrings(Strings());
      break;
    case ExecModule::PROTOTYPE:
      moduleD->_moduleDefined->setInteger(0);
      moduleD->_moduleNativeCode->setStrings(Strings());
      moduleD->_moduleCode->setStrings(Strings());
      break;
    }
  }
else status=0;
if(!status)
  {
  moduleD->_moduleType->setStrings(Strings());
  moduleD->_moduleClass->setStrings(Strings());
  moduleD->_moduleOwner->setAgent((Agent *)0);
  moduleD->_moduleName->setStrings(Strings());
  moduleD->_moduleParamNames->setValueVector(SubValue::getSubValueVect());
  moduleD->_moduleParamTypes->setValueVector(SubValue::getSubValueVect());
  moduleD->_moduleDefined->setInteger(0);
  moduleD->_moduleNativeCode->setStrings(Strings());
  moduleD->_moduleLocalNames->setValueVector(SubValue::getSubValueVect());
  moduleD->_moduleLocalTypes->setValueVector(SubValue::getSubValueVect());
  moduleD->_moduleCode->setStrings(Strings());
  }
context->returnInteger(status);
}

// native int ModuleDescription::readInClass(string className,
//                                           string moduleName);
//
void ModuleDescription::readInClass(Context * context) // static
{
ModuleDescription * moduleD=(ModuleDescription *)context->accessOwner();
int status=1;
const Model * model=getSimul()->getModel(
                                Word::create(context->getStrings(0)));
if(model)
  {
  const ExecModule * module=model->getModule(
                                   Word::create(context->getStrings(1)));
  if(module)
    {
    moduleD->_moduleClass->setStrings(module->getModel()->getName()
                                                         .getStrings());
    moduleD->_moduleOwner->setAgent((Agent *)0);
    int i;
    moduleD->_moduleType->setStrings(module->getResultType().print());
    moduleD->_moduleName->setStrings(module->getName().getStrings());
    SubValueVect * nameVect=SubValue::getSubValueVect();
    SubValueVect * typeVect=SubValue::getSubValueVect();
    for(i=0;i<module->getNbParams();i++)
      {
      nameVect->pushBack(SubValue::getSubValue(ValueType(ValueType::STRING),
                                               moduleD->_moduleParamNames));
      nameVect->accessBack()->setStrings(module->getParam(i)->getName()
                                                             .getStrings());
      typeVect->pushBack(SubValue::getSubValue(ValueType(ValueType::STRING),
                                               moduleD->_moduleParamTypes));
      if(module->getParam(i)->getMode()==Parameter::REFERENCE)
        typeVect->accessBack()
                ->setStrings(module->getParam(i)->getType().print()+" &");
      else
        typeVect->accessBack()
                ->setStrings(module->getParam(i)->getType().print());
      }
    moduleD->_moduleParamNames->setValueVector(nameVect);
    moduleD->_moduleParamTypes->setValueVector(typeVect);
    nameVect=SubValue::getSubValueVect();
    typeVect=SubValue::getSubValueVect();
    for(i=0;i<module->getNbLocals();i++)
      {
      nameVect->pushBack(SubValue::getSubValue(ValueType(ValueType::STRING),
                                               moduleD->_moduleLocalNames));
      nameVect->accessBack()->setStrings(module->getLocal(i)->getName()
                                                             .getStrings());
      typeVect->pushBack(SubValue::getSubValue(ValueType(ValueType::STRING),
                                               moduleD->_moduleLocalTypes));
      typeVect->accessBack()
              ->setStrings(module->getLocal(i)->getType().print());
      }
    moduleD->_moduleLocalNames->setValueVector(nameVect);
    moduleD->_moduleLocalTypes->setValueVector(typeVect);
    switch(module->getType())
      {
      case ExecModule::PROCEDURE:
        moduleD->_moduleDefined->setInteger(1);
        moduleD->_moduleNativeCode->setStrings(Strings());
        moduleD->_moduleCode->setStrings(module->printCode());
        break;
      case ExecModule::EXTERN:
        moduleD->_moduleDefined->setInteger(1);
        moduleD->_moduleNativeCode
               ->setStrings(((const ExternModule *)module)->getNativeName());
        moduleD->_moduleCode->setStrings(Strings());
        break;
      case ExecModule::PROTOTYPE:
        moduleD->_moduleDefined->setInteger(0);
        moduleD->_moduleNativeCode->setStrings(Strings());
        moduleD->_moduleCode->setStrings(Strings());
        break;
      }
    }
  else status=0;
  }
else status=0;
if(!status)
  {
  moduleD->_moduleType->setStrings(Strings());
  moduleD->_moduleClass->setStrings(Strings());
  moduleD->_moduleOwner->setAgent((Agent *)0);
  moduleD->_moduleName->setStrings(Strings());
  moduleD->_moduleParamNames->setValueVector(SubValue::getSubValueVect());
  moduleD->_moduleParamTypes->setValueVector(SubValue::getSubValueVect());
  moduleD->_moduleDefined->setInteger(0);
  moduleD->_moduleNativeCode->setStrings(Strings());
  moduleD->_moduleLocalNames->setValueVector(SubValue::getSubValueVect());
  moduleD->_moduleLocalTypes->setValueVector(SubValue::getSubValueVect());
  moduleD->_moduleCode->setStrings(Strings());
  }
context->returnInteger(status);
}

// native int ModuleDescription::readInObject(Object obj,
//                                            string moduleName);
//
void ModuleDescription::readInObject(Context * context) // static
{
ModuleDescription * moduleD=(ModuleDescription *)context->accessOwner();
int status=1;
Agent * agent=context->getAgent(0);
if(agent&&agent->isAlive())
  {
  const ExecModule * module=agent->getModule(
                                   Word::create(context->getStrings(1)));
  if(module)
    {
    if(module->getOwner())
      {
      moduleD->_moduleClass->setStrings(Strings());
      moduleD->_moduleOwner->setAgent((Agent *)module->getOwner());
      }
    else
      {
      moduleD->_moduleClass->setStrings(module->getModel()->getName()
                                                           .getStrings());
      moduleD->_moduleOwner->setAgent((Agent *)0);
      }
    int i;
    moduleD->_moduleType->setStrings(module->getResultType().print());
    moduleD->_moduleName->setStrings(module->getName().getStrings());
    SubValueVect * nameVect=SubValue::getSubValueVect();
    SubValueVect * typeVect=SubValue::getSubValueVect();
    for(i=0;i<module->getNbParams();i++)
      {
      nameVect->pushBack(SubValue::getSubValue(ValueType(ValueType::STRING),
                                               moduleD->_moduleParamNames));
      nameVect->accessBack()->setStrings(module->getParam(i)->getName()
                                                             .getStrings());
      typeVect->pushBack(SubValue::getSubValue(ValueType(ValueType::STRING),
                                               moduleD->_moduleParamTypes));
      if(module->getParam(i)->getMode()==Parameter::REFERENCE)
        typeVect->accessBack()
                ->setStrings(module->getParam(i)->getType().print()+" &");
      else
        typeVect->accessBack()
                ->setStrings(module->getParam(i)->getType().print());
      }
    moduleD->_moduleParamNames->setValueVector(nameVect);
    moduleD->_moduleParamTypes->setValueVector(typeVect);
    nameVect=SubValue::getSubValueVect();
    typeVect=SubValue::getSubValueVect();
    for(i=0;i<module->getNbLocals();i++)
      {
      nameVect->pushBack(SubValue::getSubValue(ValueType(ValueType::STRING),
                                               moduleD->_moduleLocalNames));
      nameVect->accessBack()->setStrings(module->getLocal(i)->getName()
                                                             .getStrings());
      typeVect->pushBack(SubValue::getSubValue(ValueType(ValueType::STRING),
                                               moduleD->_moduleLocalTypes));
      typeVect->accessBack()
              ->setStrings(module->getLocal(i)->getType().print());
      }
    moduleD->_moduleLocalNames->setValueVector(nameVect);
    moduleD->_moduleLocalTypes->setValueVector(typeVect);
    switch(module->getType())
      {
      case ExecModule::PROCEDURE:
        moduleD->_moduleDefined->setInteger(1);
        moduleD->_moduleNativeCode->setStrings(Strings());
        moduleD->_moduleCode->setStrings(module->printCode());
        break;
      case ExecModule::EXTERN:
        moduleD->_moduleDefined->setInteger(1);
        moduleD->_moduleNativeCode
               ->setStrings(((const ExternModule *)module)->getNativeName());
        moduleD->_moduleCode->setStrings(Strings());
        break;
      case ExecModule::PROTOTYPE:
        moduleD->_moduleDefined->setInteger(0);
        moduleD->_moduleNativeCode->setStrings(Strings());
        moduleD->_moduleCode->setStrings(Strings());
        break;
      }
    }
  else status=0;
  }
else status=0;
if(!status)
  {
  moduleD->_moduleType->setStrings(Strings());
  moduleD->_moduleClass->setStrings(Strings());
  moduleD->_moduleOwner->setAgent((Agent *)0);
  moduleD->_moduleName->setStrings(Strings());
  moduleD->_moduleParamNames->setValueVector(SubValue::getSubValueVect());
  moduleD->_moduleParamTypes->setValueVector(SubValue::getSubValueVect());
  moduleD->_moduleDefined->setInteger(0);
  moduleD->_moduleNativeCode->setStrings(Strings());
  moduleD->_moduleLocalNames->setValueVector(SubValue::getSubValueVect());
  moduleD->_moduleLocalTypes->setValueVector(SubValue::getSubValueVect());
  moduleD->_moduleCode->setStrings(Strings());
  }
context->returnInteger(status);
}

//--------------------------------------------------------------------------//
// ObjectDescription functions                                              //
//--------------------------------------------------------------------------//

// native int ObjectDescription::read(Object obj);
//
void ObjectDescription::read(Context * context) // static
{
ObjectDescription * objectD=(ObjectDescription *)context->accessOwner();
int status=1;
const Agent * agent=context->getAgent(0);
if(agent&&agent->isAlive())
  {
  objectD->_objectName->setAgent(context->getAgent(0));
  SubValueVect * attrVect=SubValue::getSubValueVect();
  AttrMap::const_iterator itA=agent->getAttrMap().begin();
  for(;!(itA==agent->getAttrMap().end());itA++)
    {
    attrVect->pushBack(SubValue::getSubValue(ValueType(ValueType::STRING),
                                             objectD->_objectAttributes));
    attrVect->accessBack()->setStrings((*itA).second.getName().getStrings());
    }
  objectD->_objectAttributes->setValueVector(attrVect);
  SubValueVect * modVect=SubValue::getSubValueVect();
  if(agent->getNewModule())
    {
    modVect->pushBack(SubValue::getSubValue(ValueType(ValueType::STRING),
                                            objectD->_objectModules));
    modVect->accessBack()->setStrings(agent->getNewModule()->getName()
                                                            .getStrings());
    }
  else if(agent->getModel()->getNewModule())
    {
    modVect->pushBack(SubValue::getSubValue(ValueType(ValueType::STRING),
                                            objectD->_objectModules));
    modVect->accessBack()
           ->setStrings(agent->getModel()->getNewModule()->getName()
                                                          .getStrings());
    }
  if(agent->getDeleteModule())
    {
    modVect->pushBack(SubValue::getSubValue(ValueType(ValueType::STRING),
                                            objectD->_objectModules));
    modVect->accessBack()->setStrings(agent->getDeleteModule()->getName()
                                                               .getStrings());
    }
  else if(agent->getModel()->getDeleteModule())
    {
    modVect->pushBack(SubValue::getSubValue(ValueType(ValueType::STRING),
                                            objectD->_objectModules));
    modVect->accessBack()
           ->setStrings(agent->getModel()->getDeleteModule()->getName()
                                                             .getStrings());
    }
  ExecModMap::const_iterator itM=agent->getModuleMap().begin();
  for(;!(itM==agent->getModuleMap().end());itM++)
    {
    modVect->pushBack(SubValue::getSubValue(ValueType(ValueType::STRING),
                                            objectD->_objectModules));
    modVect->accessBack()->setStrings((*itM).second->getName().getStrings());
    }
  objectD->_objectModules->setValueVector(modVect);
  }
else status=0;
if(!status)
  {
  objectD->_objectName->setAgent((Agent *)0);
  objectD->_objectAttributes->setValueVector(SubValue::getSubValueVect());
  objectD->_objectModules->setValueVector(SubValue::getSubValueVect());
  }
context->returnInteger(status);
}

//--------------------------------------------------------------------------//
// ClassDescription functions                                               //
//--------------------------------------------------------------------------//

// native int ClassDescription::read(string className);
//
void ClassDescription::read(Context * context) // static
{
ClassDescription * classD=(ClassDescription *)context->accessOwner();
int status=1;
classD->_className->setStrings(context->getStrings(0));
const Model * model=getSimul()->getModel(
                                Word::create(context->getStrings(0)));
if(model)
  {
  classD->_classNative->setInteger(model->isNative());
  unsigned int i;
  SubValueVect * parentVect=SubValue::getSubValueVect();
  for(i=0;i<model->getParents().getSize();i++)
    {
    parentVect->pushBack(SubValue::getSubValue(ValueType(ValueType::STRING),
                                               classD->_classParents));
    parentVect->accessBack()->setStrings(model->getParents()[i]->getName()
                                                                .getStrings());
    }
  classD->_classParents->setValueVector(parentVect);
  SubValueVect * ancestorVect=SubValue::getSubValueVect();
  for(i=0;i<model->getAncestors().getSize();i++)
    {
    ancestorVect->pushBack(SubValue::getSubValue(ValueType(ValueType::STRING),
                                                 classD->_classAncestors));
    ancestorVect->accessBack()
                ->setStrings(model->getAncestors()[i]->getName()
                                                      .getStrings());
    }
  classD->_classAncestors->setValueVector(ancestorVect);
  SubValueVect * childrenVect=SubValue::getSubValueVect();
  for(i=0;i<model->getChildren().getSize();i++)
    {
    childrenVect->pushBack(SubValue::getSubValue(ValueType(ValueType::STRING),
                                                 classD->_classChildren));
    childrenVect->accessBack()
                ->setStrings(model->getChildren()[i]->getName()
                                                     .getStrings());
    }
  classD->_classChildren->setValueVector(childrenVect);
  SubValueVect * descendantVect=SubValue::getSubValueVect();
  for(i=0;i<model->getDescendants().getSize();i++)
    {
    descendantVect->pushBack(SubValue::getSubValue(ValueType(ValueType::STRING),
                                                   classD->_classDescendants));
    descendantVect->accessBack()
                  ->setStrings(model->getDescendants()[i]->getName()
                                                          .getStrings());
    }
  classD->_classDescendants->setValueVector(descendantVect);
  SubValueVect * attrVect=SubValue::getSubValueVect();
  AttrMap::const_iterator itA=model->getAttrMap().begin();
  for(;!(itA==model->getAttrMap().end());itA++)
    {
    attrVect->pushBack(SubValue::getSubValue(ValueType(ValueType::STRING),
                                             classD->_classAttributes));
    attrVect->accessBack()->setStrings((*itA).second.getName().getStrings());
    }
  classD->_classAttributes->setValueVector(attrVect);
  SubValueVect * modVect=SubValue::getSubValueVect();
  if(model->getNewModule())
    {
    modVect->pushBack(SubValue::getSubValue(ValueType(ValueType::STRING),
                                            classD->_classModules));
    modVect->accessBack()->setStrings(model->getNewModule()->getName()
                                                            .getStrings());
    }
  if(model->getDeleteModule())
    {
    modVect->pushBack(SubValue::getSubValue(ValueType(ValueType::STRING),
                                            classD->_classModules));
    modVect->accessBack()->setStrings(model->getDeleteModule()->getName()
                                                               .getStrings());
    }
  ExecModMap::const_iterator itM=model->getModuleMap().begin();
  for(;!(itM==model->getModuleMap().end());itM++)
    {
    modVect->pushBack(SubValue::getSubValue(ValueType(ValueType::STRING),
                                            classD->_classModules));
    modVect->accessBack()->setStrings((*itM).second->getName().getStrings());
    }
  classD->_classModules->setValueVector(modVect);
  }
else status=0;
if(!status)
  {
  classD->_className->setStrings(Strings());
  classD->_classNative->setInteger(0);
  classD->_classParents->setValueVector(SubValue::getSubValueVect());
  classD->_classAncestors->setValueVector(SubValue::getSubValueVect());
  classD->_classChildren->setValueVector(SubValue::getSubValueVect());
  classD->_classDescendants->setValueVector(SubValue::getSubValueVect());
  classD->_classAttributes->setValueVector(SubValue::getSubValueVect());
  classD->_classModules->setValueVector(SubValue::getSubValueVect());
  }
context->returnInteger(status);
}

//--------------------------------------------------------------------------//
// GlobalDescription functions                                              //
//--------------------------------------------------------------------------//

// native bool GlobalDescription::read(void);
//
void GlobalDescription::read(Context * context) // static
{
GlobalDescription * globalD=(GlobalDescription *)context->accessOwner();
SubValueVect * classVect=SubValue::getSubValueVect();
ModelMap::const_iterator itM=getSimul()->getModelMap().begin();
for(;!(itM==getSimul()->getModelMap().end());itM++)
  {
  classVect->pushBack(SubValue::getSubValue(ValueType(ValueType::STRING),
                                            globalD->_globalClasses));
  classVect->accessBack()->setStrings((*itM).second->getName().getStrings());
  }
globalD->_globalClasses->setValueVector(classVect);
SubValueVect * modVect=SubValue::getSubValueVect();
ExecModMap::const_iterator itF=getSimul()->getModuleMap().begin();
for(;!(itF==getSimul()->getModuleMap().end());itF++)
  {
  modVect->pushBack(SubValue::getSubValue(ValueType(ValueType::STRING),
                                          globalD->_globalModules));
  modVect->accessBack()->setStrings((*itF).second->getName().getStrings());
  }
globalD->_globalModules->setValueVector(modVect);
context->returnInteger(1);
}

//--------------------------------------------------------------------------//
// TypeDescription functions                                                //
//--------------------------------------------------------------------------//

// native bool TypeDescription::read(string type);
//
void TypeDescription::read(Context * context) // static
{
TypeDescription * typeD=(TypeDescription *)context->accessOwner();
int status=1;
const char * str=context->getStrings(0);
char c;
//---- eat spaces
do
  {
  c=*(str++);
  } while(c&&((c==' ')||(c<0x20)));
if(c)
  {
  //---- read base type
  if((c=='_')||
     ((c>='A')&&(c<='Z'))||
     ((c>='a')&&(c<='z')))
    {
    const char * start=str;
    do
      {
      c=*(str++);
      } while((c=='_')||
              ((c>='A')&&(c<='Z'))||
              ((c>='a')&&(c<='z'))||
              ((c>='0')&&(c<='9')));
    typeD->_typeBaseName=Strings(start-1);
    typeD->_typeBaseName=typeD->_typeBaseName.subString(0,str-start);
    if(typeD->_typeBaseName==Strings("int"))
      typeD->_typeBase=ValueType::INTEGER;
    else if(typeD->_typeBaseName==Strings("float"))
      typeD->_typeBase=ValueType::REAL;
    else if(typeD->_typeBaseName==Strings("string"))
      typeD->_typeBase=ValueType::STRING;
    else if(typeD->_typeBaseName==Strings("void"))
      typeD->_typeBase=ValueType::UNKNOWN;
    else typeD->_typeBase=ValueType::AGENT;
    //---- read dimension
    typeD->_typeDimension=0;
    bool done=false;
    bool open=false;
    do
      {
      while(c&&((c==' ')||(c<0x20))) c=*(str++); // eat spaces
      if(open)
        {
        if(c==']') {typeD->_typeDimension++; open=false; c=*(str++);}
        else {status=0; done=true;}
        }
      else
        {
        if(c=='[') {open=true; c=*(str++);}
        else done=true;
        }
      } while(!done);
    if(status)
      {
      typeD->_typeMode=Parameter::COPY;
      while(c&&((c==' ')||(c<0x20))) c=*(str++); // eat spaces
      if(c)
        {
        if(c=='&') typeD->_typeMode=Parameter::REFERENCE;
        else status=0;
        }
      }
    }
  else status=0;
  }
else status=0;
if(!status)
  {
  typeD->_typeBase=ValueType::UNKNOWN;
  typeD->_typeBaseName=Strings();
  typeD->_typeDimension=-1;
  typeD->_typeMode=Parameter::COPY;
  }
context->returnInteger(status);
}

// native int TypeDescription::isInteger(void);
//
void TypeDescription::isInteger(Context * context) // static
{
TypeDescription * typeD=(TypeDescription *)context->accessOwner();
context->returnInteger(typeD->_typeBase==ValueType::INTEGER);
}

// native int TypeDescription::isFloat(void);
//
void TypeDescription::isFloat(Context * context) // static
{
TypeDescription * typeD=(TypeDescription *)context->accessOwner();
context->returnInteger(typeD->_typeBase==ValueType::REAL);
}

// native int TypeDescription::isString(void);
//
void TypeDescription::isString(Context * context) // static
{
TypeDescription * typeD=(TypeDescription *)context->accessOwner();
context->returnInteger(typeD->_typeBase==ValueType::STRING);
}

// native int TypeDescription::isObject(void);
//
void TypeDescription::isObject(Context * context) // static
{
TypeDescription * typeD=(TypeDescription *)context->accessOwner();
context->returnInteger(typeD->_typeBase==ValueType::AGENT);
}

// native int TypeDescription::isVoid(void);
//
void TypeDescription::isVoid(Context * context) // static
{
TypeDescription * typeD=(TypeDescription *)context->accessOwner();
context->returnInteger(typeD->_typeBase==ValueType::UNKNOWN);
}

// native string TypeDescription::getBaseType(void);
//
void TypeDescription::getBaseType(Context * context) // static
{
TypeDescription * typeD=(TypeDescription *)context->accessOwner();
context->returnStrings(typeD->_typeBaseName);
}

// native int TypeDescription::getDimension(void);
//
void TypeDescription::getDimension(Context * context) // static
{
TypeDescription * typeD=(TypeDescription *)context->accessOwner();
context->returnInteger(typeD->_typeDimension);
}

// native int TypeDescription::isReference(void);
//
void TypeDescription::isReference(Context * context) // static
{
TypeDescription * typeD=(TypeDescription *)context->accessOwner();
context->returnInteger(typeD->_typeMode==Parameter::REFERENCE);
}

//--------------------------------------------------------------------------//

static Word typeWord(Word::create("type"));
static Word classNameWord(Word::create("className"));
static Word ownerWord(Word::create("owner"));
static Word nameWord(Word::create("name"));
static Word valueWord(Word::create("value"));
static Word paramNamesWord(Word::create("paramNames"));
static Word paramTypesWord(Word::create("paramTypes"));
static Word definedWord(Word::create("defined"));
static Word nativeCodeWord(Word::create("nativeCode"));
static Word localNamesWord(Word::create("localNames"));
static Word localTypesWord(Word::create("localTypes"));
static Word orisCodeWord(Word::create("orisCode"));
static Word objectWord(Word::create("object"));
static Word attributeNamesWord(Word::create("attributeNames"));
static Word moduleNamesWord(Word::create("moduleNames"));
static Word isNativeWord(Word::create("isNative"));
static Word parentsWord(Word::create("parents"));
static Word ancestorsWord(Word::create("ancestors"));
static Word childrenWord(Word::create("children"));
static Word descendantsWord(Word::create("descendants"));
static Word classNamesWord(Word::create("classNames"));

//--------------------------------------------------------------------------//
// class AttributeDescription                                               //
//--------------------------------------------------------------------------//

AttributeDescription::AttributeDescription(Model * model,int instNum)
 : Agent(model,instNum),
   _attrType(accessAttribute(::typeWord)->accessValue()),
   _attrClass(accessAttribute(::classNameWord)->accessValue()),
   _attrOwner(accessAttribute(::ownerWord)->accessValue()),
   _attrName(accessAttribute(::nameWord)->accessValue()),
   _attrValue(accessAttribute(::valueWord)->accessValue())
{}

AttributeDescription::~AttributeDescription(void)
{}

Strings AttributeDescription::userSave(int) const // virtual
{
Strings str;
return(str);
}

//--------------------------------------------------------------------------//
// class ModuleDescription                                                  //
//--------------------------------------------------------------------------//

ModuleDescription::ModuleDescription(Model * model,int instNum)
 : Agent(model,instNum),
   _moduleType(accessAttribute(::typeWord)->accessValue()),
   _moduleClass(accessAttribute(::classNameWord)->accessValue()),
   _moduleOwner(accessAttribute(::ownerWord)->accessValue()),
   _moduleName(accessAttribute(::nameWord)->accessValue()),
   _moduleParamNames(accessAttribute(::paramNamesWord)->accessValue()),
   _moduleParamTypes(accessAttribute(::paramTypesWord)->accessValue()),
   _moduleDefined(accessAttribute(::definedWord)->accessValue()),
   _moduleNativeCode(accessAttribute(::nativeCodeWord)->accessValue()),
   _moduleLocalNames(accessAttribute(::localNamesWord)->accessValue()),
   _moduleLocalTypes(accessAttribute(::localTypesWord)->accessValue()),
   _moduleCode(accessAttribute(::orisCodeWord)->accessValue())
{}

ModuleDescription::~ModuleDescription(void)
{}

Strings ModuleDescription::userSave(int) const // virtual
{
Strings str;
return(str);
}

//--------------------------------------------------------------------------//
// class ObjectDescription                                                   //
//--------------------------------------------------------------------------//

ObjectDescription::ObjectDescription(Model * model,int instNum)
 : Agent(model,instNum),
   _objectName(accessAttribute(::objectWord)->accessValue()),
   _objectAttributes(accessAttribute(::attributeNamesWord)->accessValue()),
   _objectModules(accessAttribute(::moduleNamesWord)->accessValue())
{}

ObjectDescription::~ObjectDescription(void)
{}

Strings ObjectDescription::userSave(int) const // virtual
{
Strings str;
return(str);
}

//--------------------------------------------------------------------------//
// class ClassDescription                                                   //
//--------------------------------------------------------------------------//

ClassDescription::ClassDescription(Model * model,int instNum)
 : Agent(model,instNum),
   _className(accessAttribute(::classNameWord)->accessValue()),
   _classNative(accessAttribute(::isNativeWord)->accessValue()),
   _classParents(accessAttribute(::parentsWord)->accessValue()),
   _classAncestors(accessAttribute(::ancestorsWord)->accessValue()),
   _classChildren(accessAttribute(::childrenWord)->accessValue()),
   _classDescendants(accessAttribute(::descendantsWord)->accessValue()),
   _classAttributes(accessAttribute(::attributeNamesWord)->accessValue()),
   _classModules(accessAttribute(::moduleNamesWord)->accessValue())
{}

ClassDescription::~ClassDescription(void)
{}

Strings ClassDescription::userSave(int) const // virtual
{
Strings str;
return(str);
}

//--------------------------------------------------------------------------//
// class GlobalDescription                                                  //
//--------------------------------------------------------------------------//

GlobalDescription::GlobalDescription(Model * model,int instNum)
 : Agent(model,instNum),
   _globalClasses(accessAttribute(::classNamesWord)->accessValue()),
   _globalModules(accessAttribute(::moduleNamesWord)->accessValue())
{}

GlobalDescription::~GlobalDescription(void)
{}

Strings GlobalDescription::userSave(int) const // virtual
{
Strings str;
return(str);
}

//--------------------------------------------------------------------------//
// class TypeDescription                                                    //
//--------------------------------------------------------------------------//

TypeDescription::TypeDescription(Model * model,int instNum)
 : Agent(model,instNum),
   _typeBase(ValueType::UNKNOWN), _typeBaseName(),
   _typeDimension(-1), _typeMode(Parameter::COPY)
{}

TypeDescription::~TypeDescription(void)
{}

Strings TypeDescription::userSave(int column) const // virtual
{
Strings str;
if(_typeBaseName.len())
  {
  int i;
  for(i=0;i<column;i++) str+=" ";
  Strings typeName=_typeBaseName;
  for(i=0;i<_typeDimension;i++) typeName+="[]";
  if(_typeMode==Parameter::REFERENCE) typeName+="&";
  str+=Agent::getName().print()+"->read("+typeName.dump()+");\n";
  }
return(str);
}

