// @(#) ENIB/LI2 (c) - oRis - Version 2.3
//--------------------------------------------------------------------------//
// file        : reflPkg.h                                                  //
// description : header file for the ENIB/LI2 oRis package reflect.pkg      //
// author(s)   : F. Harrouet                                                //
// copyright   : Laboratoire d'Informatique Industrielle (LI2)              //
//               Ecole Nationale d'Ingenieurs de Brest (ENIB)               //
//               CP 30815 - 29608 Brest Cedex - FRANCE                      //
//               Tel: (33) 02.98.05.66.31 , Fax: (33) 02.98.05.66.29        //
//               e-mail: li2@enib.fr                                        //
//--------------------------------------------------------------------------//
// Copyright (c) 1996-2000                                                  //
// Laboratoire d'Informatique Industrielle (LI2)                            //
// Ecole Nationale d'Ingenieurs de Brest (ENIB)                             //
//                                                                          //
// Permission to use, copy, modify, distribute and sell this software       //
// and its documentation for any purpose is hereby granted without fee,     //
// provided that the above copyright notice appear in all copies and        //
// that both that copyright notice and this permission notice appear        //
// in supporting documentation.                                             //
// ENIB/LI2 makes no representations about the suitability of this          //
// software for any purpose.                                                //
// It is provided "as is" without express or implied warranty.              //
//--------------------------------------------------------------------------//

#ifndef REFLPKG_H
#define REFLPKG_H

#include "oRis/Main/agent.h"

//--------------------------------------------------------------------------//
// class AttributeDescription                                               //
//--------------------------------------------------------------------------//

class AttributeDescription : public Agent
{
public:
  AttributeDescription(Model * model,int instNum);
  virtual ~AttributeDescription(void);
  virtual Strings userSave(int column=0) const;

  // native int AttributeDescription::readInClass(string className,
  //                                              string attrName);
  //
  static void readInClass(Context * context);

  // native int AttributeDescription::readInObject(Object obj,
  //                                               string attrName);
  //
  static void readInObject(Context * context);

//---- Attributes ----
protected:
  Value * _attrType;
  Value * _attrClass;
  Value * _attrOwner;
  Value * _attrName;
  Value * _attrValue;
};

//--------------------------------------------------------------------------//
// class ModuleDescription                                                  //
//--------------------------------------------------------------------------//

class ModuleDescription : public Agent
{
public:
  ModuleDescription(Model * model,int instNum);
  virtual ~ModuleDescription(void);
  virtual Strings userSave(int column=0) const;

  // native int ModuleDescription::readGlobal(string moduleName);
  //
  static void readGlobal(Context * context);

  // native int ModuleDescription::readInClass(string className,
  //                                           string moduleName);
  //
  static void readInClass(Context * context);

  // native int ModuleDescription::readInObject(Object obj,
  //                                            string moduleName);
  //
  static void readInObject(Context * context);

//---- Attributes ----
protected:
  Value * _moduleType;
  Value * _moduleClass;
  Value * _moduleOwner;
  Value * _moduleName;
  Value * _moduleParamNames;
  Value * _moduleParamTypes;
  Value * _moduleDefined;
  Value * _moduleNativeCode;
  Value * _moduleLocalNames;
  Value * _moduleLocalTypes;
  Value * _moduleCode;
};

//--------------------------------------------------------------------------//
// class ObjectDescription                                                  //
//--------------------------------------------------------------------------//

class ObjectDescription : public Agent
{
public:
  ObjectDescription(Model * model,int instNum);
  virtual ~ObjectDescription(void);
  virtual Strings userSave(int column=0) const;

  // native int ObjectDescription::read(Object obj);
  //
  static void read(Context * context);

//---- Attributes ----
protected:
  Value * _objectName;
  Value * _objectAttributes;
  Value * _objectModules;
};

//--------------------------------------------------------------------------//
// class ClassDescription                                                   //
//--------------------------------------------------------------------------//

class ClassDescription : public Agent
{
public:
  ClassDescription(Model * model,int instNum);
  virtual ~ClassDescription(void);
  virtual Strings userSave(int column=0) const;

  // native int ClassDescription::read(string className);
  //
  static void read(Context * context);

//---- Attributes ----
protected:
  Value * _className;
  Value * _classNative;
  Value * _classParents;
  Value * _classAncestors;
  Value * _classChildren;
  Value * _classDescendants;
  Value * _classAttributes;
  Value * _classModules;
};

//--------------------------------------------------------------------------//
// class GlobalDescription                                                  //
//--------------------------------------------------------------------------//

class GlobalDescription : public Agent
{
public:
  GlobalDescription(Model * model,int instNum);
  virtual ~GlobalDescription(void);
  virtual Strings userSave(int column=0) const;

  // native int GlobalDescription::read(void);
  //
  static void read(Context * context);

//---- Attributes ----
protected:
  Value * _globalClasses;
  Value * _globalModules;
};

//--------------------------------------------------------------------------//
// class TypeDescription                                                    //
//--------------------------------------------------------------------------//

class TypeDescription : public Agent
{
public:
  TypeDescription(Model * model,int instNum);
  virtual ~TypeDescription(void);
  virtual Strings userSave(int column=0) const;

  // native int TypeDescription::read(string type);
  //
  static void read(Context * context);

  // native int TypeDescription::isInteger(void);
  //
  static void isInteger(Context * context);

  // native int TypeDescription::isFloat(void);
  //
  static void isFloat(Context * context);

  // native int TypeDescription::isString(void);
  //
  static void isString(Context * context);

  // native int TypeDescription::isObject(void);
  //
  static void isObject(Context * context);

  // native int TypeDescription::isVoid(void);
  //
  static void isVoid(Context * context);

  // native string TypeDescription::getBaseType(void);
  //
  static void getBaseType(Context * context);

  // native int TypeDescription::getDimension(void);
  //
  static void getDimension(Context * context);

  // native int TypeDescription::isReference(void);
  //
  static void isReference(Context * context);

//---- Attributes ----
protected:
  ValueType::SimpleType _typeBase;
  Strings _typeBaseName;
  int _typeDimension;
  Parameter::ParamMode _typeMode;
};

#endif // REFLPKG_H

