// @(#) ENIB/LI2 (c) - oRis - Version 2.3
//--------------------------------------------------------------------------//
// file        : scmPkg.cpp                                                 //
// description : source file for the ENIB/LI2 oRis / SCM-Guile interface    //
// author(s)   : F. Harrouet                                                //
// copyright   : Laboratoire d'Informatique Industrielle (LI2)              //
//               Ecole Nationale d'Ingenieurs de Brest (ENIB)               //
//               CP 30815 - 29608 Brest Cedex - FRANCE                      //
//               Tel: (33) 02.98.05.66.31 , Fax: (33) 02.98.05.66.29        //
//               e-mail: li2@enib.fr                                        //
//--------------------------------------------------------------------------//
// Copyright (c) 1996-2000                                                  //
// Laboratoire d'Informatique Industrielle (LI2)                            //
// Ecole Nationale d'Ingenieurs de Brest (ENIB)                             //
//                                                                          //
// Permission to use, copy, modify, distribute and sell this software       //
// and its documentation for any purpose is hereby granted without fee,     //
// provided that the above copyright notice appear in all copies and        //
// that both that copyright notice and this permission notice appear        //
// in supporting documentation.                                             //
// ENIB/LI2 makes no representations about the suitability of this          //
// software for any purpose.                                                //
// It is provided "as is" without express or implied warranty.              //
//--------------------------------------------------------------------------//

#include "Scheme/scmPkg.h"
#include "oRis/Main/context.h"
#include "oRis/Main/errors.h"

#ifndef __MSDOS__
#define EXPORT_PREFIX extern "C"
#else
#include <windows.h>
#define EXPORT_PREFIX extern "C" __declspec(dllexport)
#define openingLibrary    openingScmLibrary
#define closingLibrary    closingScmLibrary
#define getExternFunction getExternScmFunction
#define getNewAgent       getNewScmAgent
#endif


//--------------------------------------------------------------------------//
// Opening and closing the library                                          //
//--------------------------------------------------------------------------//

EXPORT_PREFIX void openingLibrary(void)
{
if(!EmbedScheme::getInstance()) (void)new EmbedScheme;
}

EXPORT_PREFIX void closingLibrary(void)
{
if(EmbedScheme::getInstance()) delete EmbedScheme::getInstance();
}

//--------------------------------------------------------------------------//
// LookUp table for functions                                               //
//--------------------------------------------------------------------------//

EXPORT_PREFIX ExternFunction getExternFunction(const Strings & name)
{
if(name==Strings("::callScheme()"))  return(EmbedScheme::callScheme);
return((ExternFunction)0); // not found
}

//--------------------------------------------------------------------------//
// LookUp table for agents                                                  //
//--------------------------------------------------------------------------//

EXPORT_PREFIX Agent * getNewAgent(Model * /* model */ , long /* instNum */ )
{
return((Agent *)0); // not found
}

//--------------------------------------------------------------------------//
// EmbedScheme functions                                                    //
//--------------------------------------------------------------------------//

// native void callScheme(string command);
//
void EmbedScheme::callScheme(Context * context) // static
{
EmbedScheme::_instance->_command=context->getStrings(0);
EmbedScheme::_instance->_mutex1.unlock();
EmbedScheme::_instance->_mutex2.lock();
}
  
//--------------------------------------------------------------------------//
// class EmbedScheme                                                        //
//--------------------------------------------------------------------------//

EmbedScheme * EmbedScheme::_instance=(EmbedScheme *)0;

//-------- Static Methods --------------------------------------------------//
EmbedScheme * EmbedScheme::getInstance(void) // static
{
return(EmbedScheme::_instance);
}

//-------- Allocators ------------------------------------------------------//
EmbedScheme::EmbedScheme(void)
 : BlockAllocatedObject(),
   _command(), _thread((SimpleThread *)0), _mutex1(), _mutex2()
{
if(EmbedScheme::_instance) delete EmbedScheme::_instance;
EmbedScheme::_instance=this;
_mutex1.lock();
_mutex2.lock();
_thread=new SimpleThread(EmbedScheme::_threadTask);
_thread->start();
}

EmbedScheme::~EmbedScheme(void)
{
if(_thread) delete _thread;
EmbedScheme::_instance=(EmbedScheme *)0;
}

//-------- Inspectors ------------------------------------------------------//

//-------- Modifiers -------------------------------------------------------//

//-------- Private Methods -------------------------------------------------//
void * EmbedScheme::_threadTask(void * /* data */) // static
{
int argc=1;
char * argv[2];
argv[0]=(char *)(const char *)getSimul()->getCommandLine()[0];
argv[1]=(char *)0;
gh_enter(argc,argv,(void (*)(...))EmbedScheme::_guileTask);
return((void *)0);
}

void EmbedScheme::_guileTask(int /* argc */,char ** /* argv */) // static
{
gh_new_procedure2_0("oRisFunction",
                    (SCM (*)(...))EmbedScheme::_functionCall);
gh_new_procedure3_0("oRisMethod",
                    (SCM (*)(...))EmbedScheme::_methodCall);
bool done=false;
while(!done)
  {
  EmbedScheme::_instance->_mutex1.lock();
  gh_eval_str_with_standard_handler((char *)(const char *)
                                    EmbedScheme::_instance->_command);
  EmbedScheme::_instance->_mutex2.unlock();
  }
}

SCM EmbedScheme::_functionCall(SCM function,SCM /* args */)
{
SCM result;
bool fail=false;
Strings moduleName;
ExecModule * module=(ExecModule *)0;
if(gh_string_p(function))
  {
  int length;
  char * str=gh_scm2newstr(function,&length);
  moduleName=str;
  }
module=getSimul()->accessModule(Word::create(moduleName));
if(!module)
  {
  fail=true;
  Strings message="SCHEME: Module '"+moduleName+"' is unknown";
  getSimul()->manageError(new Error(Error::RUNTIME,message));
  goto clearAndExit;
  }
cerr << "Appel de la fonction : " << moduleName << endl;
clearAndExit:
  if(fail) result=gh_bool2scm(false);
  else result=gh_bool2scm(true);
  return(result);
}

SCM EmbedScheme::_methodCall(SCM instance,SCM method,SCM /* args */ )
{
SCM result;
bool fail=false;
Strings agentName;
Agent * agent=(Agent *)0;
Strings moduleName;
ExecModule * module=(ExecModule *)0;
if(gh_string_p(instance))
  {
  int length;
  char * str=gh_scm2newstr(instance,&length);
  agentName=str;
  }
agent=getSimul()->accessAgent(AgentName(agentName));
if(!agent)
  {
  fail=true;
  Strings message="SCHEME: Instance '"+agentName+"' is unknown";
  getSimul()->manageError(new Error(Error::RUNTIME,message));
  goto clearAndExit;
  }
if(gh_string_p(method))
  {
  int length;
  char * str=gh_scm2newstr(method,&length);
  moduleName=str;
  }
module=agent->accessModule(Word::create(moduleName));
if(!module)
  {
  fail=true;
  Strings message="SCHEME: Module '"+
                  agentName+"::"+moduleName+"()' is unknown";
  getSimul()->manageError(new Error(Error::RUNTIME,message));
  goto clearAndExit;
  }
cerr << "Appel de la methode : " << agentName << "::" << moduleName << endl;
clearAndExit:
  if(fail) result=gh_bool2scm(false);
  else result=gh_bool2scm(true);
  return(result);
}

