// @(#) ENIB/LI2 (c) - oRis - Version 2.3
//--------------------------------------------------------------------------//
// file        : semaPkg.cpp                                                //
// description : source file for the ENIB/LI2 oRis package semaphor.pkg     //
// author(s)   : F. Harrouet                                                //
// copyright   : Laboratoire d'Informatique Industrielle (LI2)              //
//               Ecole Nationale d'Ingenieurs de Brest (ENIB)               //
//               CP 30815 - 29608 Brest Cedex - FRANCE                      //
//               Tel: (33) 02.98.05.66.31 , Fax: (33) 02.98.05.66.29        //
//               e-mail: li2@enib.fr                                        //
//--------------------------------------------------------------------------//
// Copyright (c) 1996-2000                                                  //
// Laboratoire d'Informatique Industrielle (LI2)                            //
// Ecole Nationale d'Ingenieurs de Brest (ENIB)                             //
//                                                                          //
// Permission to use, copy, modify, distribute and sell this software       //
// and its documentation for any purpose is hereby granted without fee,     //
// provided that the above copyright notice appear in all copies and        //
// that both that copyright notice and this permission notice appear        //
// in supporting documentation.                                             //
// ENIB/LI2 makes no representations about the suitability of this          //
// software for any purpose.                                                //
// It is provided "as is" without express or implied warranty.              //
//--------------------------------------------------------------------------//

#include "Semaphor/semaPkg.h"
#include "oRis/Main/simul.h"
#include "oRis/Main/context.h"

#ifndef __MSDOS__
#define EXPORT_PREFIX extern "C"
#else
#include <windows.h>
#define EXPORT_PREFIX extern "C" __declspec(dllexport)
#define openingLibrary    openingSemaLibrary
#define closingLibrary    closingSemaLibrary
#define getExternFunction getExternSemaFunction
#define getNewAgent       getNewSemaAgent
#endif

//--------------------------------------------------------------------------//
// Opening and closing the library                                          //
//--------------------------------------------------------------------------//
 
EXPORT_PREFIX void openingLibrary(void)
{
  // nothing to do
}
 
EXPORT_PREFIX void closingLibrary(void)
{
  // nothing to do
}

//--------------------------------------------------------------------------//
// LookUp table for functions                                               //
//--------------------------------------------------------------------------//

EXPORT_PREFIX ExternFunction getExternFunction(const Strings & name)
{
if(name==Strings("Semaphore::new()"))       return(Semaphore::constructor);
if(name==Strings("Semaphore::isRandom()"))  return(Semaphore::isRandom);
if(name==Strings("Semaphore::setRandom()")) return(Semaphore::setRandom);
if(name==Strings("Semaphore::getValue()"))  return(Semaphore::getValue);
if(name==Strings("Semaphore::unlock()"))    return(Semaphore::unlock);
if(name==Strings("Semaphore::lock()"))      return(Semaphore::lock);
if(name==Strings("Semaphore::tryLock()"))   return(Semaphore::tryLock);
return((ExternFunction)0); // not found
}

//--------------------------------------------------------------------------//
// LookUp table for agents                                                  //
//--------------------------------------------------------------------------//

static Word semaphoreWord(Word::create("Semaphore"));

EXPORT_PREFIX Agent * getNewAgent(Model * model,long instNum)
{
if(model->isA(::semaphoreWord))  return(new Semaphore(model,instNum));
return((Agent *)0); // not found
}

//--------------------------------------------------------------------------//
// Semaphore functions                                                      //
//--------------------------------------------------------------------------//

// native void Semaphore::new(int initValue);
//
void Semaphore::constructor(Context * context) // static
{
context->accessExecution()->enterContext(
                            Model::getBaseModel()->accessNewModule(),
                            context->accessOwner());
((Semaphore *)context->accessOwner())->_value=context->getInteger(0);
}

// native bool Semaphore::isRandom(void);
//
void Semaphore::isRandom(Context * context) // static
{
context->returnInteger(((const Semaphore *)context->getOwner())->_random);
}

// native void Semaphore::setRandom(bool isRandom);
//
void Semaphore::setRandom(Context * context) // static
{
((Semaphore *)context->accessOwner())->_random=context->getInteger(0);
}

// native int Semaphore::getValue(void);
//
void Semaphore::getValue(Context * context) // static
{
context->returnInteger(((const Semaphore *)context->getOwner())->_value);
}

// native void Semaphore::unlock(void);
//
void Semaphore::unlock(Context * context) // static
{
Semaphore * sem=(Semaphore *)context->accessOwner();
++sem->_value;
sem->resume();
}

// native void Semaphore::lock(void);
//
void Semaphore::lock(Context * context) // static
{
Semaphore * sem=(Semaphore *)context->accessOwner();
if(sem->_value<=0) sem->suspend(context);
--sem->_value;
}

// native bool Semaphore::tryLock(void);
//
void Semaphore::tryLock(Context * context) // static
{
Semaphore * sem=(Semaphore *)context->accessOwner();
if(sem->_value>0)
  {
  --sem->_value;
  context->returnInteger(1);
  }
else context->returnInteger(0);
}

//--------------------------------------------------------------------------//
// class Semaphore                                                          //
//--------------------------------------------------------------------------//

Semaphore::Semaphore(Model * model,int instNum)
 : Agent(model,instNum), _value(0), _random(false), _suspendedVect()
{}

Semaphore::~Semaphore(void)
{}

Strings Semaphore::userSave(int column) const // virtual
{
Strings str;
for(int i=0;i<column;i++) str+=" ";
str+=getName().print()+"->init("+Strings(_value)+");\n";
return(str);
}

void Semaphore::suspend(Context * context)
{
_suspendedVect.pushBack(context->accessExecution()->getID());
getSimul()->suspendExecution(context->accessExecution()->getID());
}

void Semaphore::resume(void)
{
while(_suspendedVect.getSize()&&
      !getSimul()->resumeExecution(
                   _suspendedVect.readErase(_random ?
                                            getSimul()->integerRand(
                                            _suspendedVect.getSize()) : 0)));
}

