// @(#) ENIB/LI2 (c) - oRis - Version 2.3
//--------------------------------------------------------------------------//
// file        : tcpUdpPkg.cpp                                              //
// description : source file for the ENIB/LI2 oRis package tcpUdp.pkg       //
// author(s)   : F. Harrouet                                                //
// copyright   : Laboratoire d'Informatique Industrielle (LI2)              //
//               Ecole Nationale d'Ingenieurs de Brest (ENIB)               //
//               CP 30815 - 29608 Brest Cedex - FRANCE                      //
//               Tel: (33) 02.98.05.66.31 , Fax: (33) 02.98.05.66.29        //
//               e-mail: li2@enib.fr                                        //
//--------------------------------------------------------------------------//
// Copyright (c) 1996-2000                                                  //
// Laboratoire d'Informatique Industrielle (LI2)                            //
// Ecole Nationale d'Ingenieurs de Brest (ENIB)                             //
//                                                                          //
// Permission to use, copy, modify, distribute and sell this software       //
// and its documentation for any purpose is hereby granted without fee,     //
// provided that the above copyright notice appear in all copies and        //
// that both that copyright notice and this permission notice appear        //
// in supporting documentation.                                             //
// ENIB/LI2 makes no representations about the suitability of this          //
// software for any purpose.                                                //
// It is provided "as is" without express or implied warranty.              //
//--------------------------------------------------------------------------//

#include "TcpUdp/tcpUdpPkg.h"
#include "Memory/memoryPkg.h"
#include "oRis/Main/simul.h"
#include "oRis/Main/context.h"
#include "oRis/Main/errors.h"
#include "oRis/Main/tmpVal.h"

#ifndef __MSDOS__
#define EXPORT_PREFIX extern "C"
#else
#include <windows.h>
#define EXPORT_PREFIX extern "C" __declspec(dllexport)
#define openingLibrary    openingTcpUdpLibrary
#define closingLibrary    closingTcpUdpLibrary
#define getExternFunction getExternTcpUdpFunction
#define getNewAgent       getNewTcpUdpAgent
#endif
 
//--------------------------------------------------------------------------//
// Opening and closing the library                                          //
//--------------------------------------------------------------------------//
 
EXPORT_PREFIX void openingLibrary(void)
{
  // nothing to do
}
 
EXPORT_PREFIX void closingLibrary(void)
{
  // nothing to do
}

//--------------------------------------------------------------------------//
// LookUp table for functions                                               //
//--------------------------------------------------------------------------//

EXPORT_PREFIX ExternFunction getExternFunction(const Strings & name)
{
if(name==Strings("::getHostName()"))
    return(::getHostName);
if(name==Strings("::getIPAddress()"))
    return(::getIPAddress);

if(name==Strings("TCPListener::_new()"))
    return(OTCPListener::construct);
if(name==Strings("TCPListener::_delete()"))
    return(OTCPListener::destruct);
if(name==Strings("TCPListener::fail()"))
    return(OTCPListener::fail);
if(name==Strings("TCPListener::getPortNumber()"))
    return(OTCPListener::getPortNumber);
if(name==Strings("TCPListener::getIPAddress()"))
    return(OTCPListener::getIPAddress);
if(name==Strings("TCPListener::getNbPending()"))
    return(OTCPListener::getNbPending);
if(name==Strings("TCPListener::nextConnection()"))
    return(OTCPListener::nextConnection);

if(name==Strings("TCPConnection::_new()"))
    return(OTCPConnection::construct);
if(name==Strings("TCPConnection::_delete()"))
    return(OTCPConnection::destruct);
if(name==Strings("TCPConnection::fail()"))
    return(OTCPConnection::fail);
if(name==Strings("TCPConnection::getLocalPortNumber()"))
    return(OTCPConnection::getLocalPortNumber);
if(name==Strings("TCPConnection::getLocalIPAddress()"))
    return(OTCPConnection::getLocalIPAddress);
if(name==Strings("TCPConnection::getDistantPortNumber()"))
    return(OTCPConnection::getDistantPortNumber);
if(name==Strings("TCPConnection::getDistantIPAddress()"))
    return(OTCPConnection::getDistantIPAddress);
if(name==Strings("TCPConnection::read()"))
    return(OTCPConnection::read);
if(name==Strings("TCPConnection::readBlock()"))
    return(OTCPConnection::readBlock);
if(name==Strings("TCPConnection::write()"))
    return(OTCPConnection::write);
if(name==Strings("TCPConnection::writeBlock()"))
    return(OTCPConnection::writeBlock);

if(name==Strings("UDPTransmitter::_new()"))
    return(OUDPTransmitter::construct);
if(name==Strings("UDPTransmitter::_delete()"))
    return(OUDPTransmitter::destruct);
if(name==Strings("UDPTransmitter::fail()"))
    return(OUDPTransmitter::fail);
if(name==Strings("UDPTransmitter::getPortNumber()"))
    return(OUDPTransmitter::getPortNumber);
if(name==Strings("UDPTransmitter::getIPAddress()"))
    return(OUDPTransmitter::getIPAddress);
if(name==Strings("UDPTransmitter::read()"))
    return(OUDPTransmitter::read);
if(name==Strings("UDPTransmitter::readFrom()"))
    return(OUDPTransmitter::readFrom);
if(name==Strings("UDPTransmitter::readBlock()"))
    return(OUDPTransmitter::readBlock);
if(name==Strings("UDPTransmitter::readBlockFrom()"))
    return(OUDPTransmitter::readBlockFrom);
if(name==Strings("UDPTransmitter::writeTo()"))
    return(OUDPTransmitter::writeTo);
if(name==Strings("UDPTransmitter::writeBlockTo()"))
    return(OUDPTransmitter::writeBlockTo);
if(name==Strings("UDPTransmitter::broadcast()"))
    return(OUDPTransmitter::broadcast);
if(name==Strings("UDPTransmitter::broadcastBlock()"))
    return(OUDPTransmitter::broadcastBlock);

return((ExternFunction)0); // not found
}

//--------------------------------------------------------------------------//
// LookUp table for agents                                                  //
//--------------------------------------------------------------------------//

static Word tcpListenerWord(Word::create("TCPListener"));
static Word tcpConnectionWord(Word::create("TCPConnection"));
static Word udpTransmitterWord(Word::create("UDPTransmitter"));

EXPORT_PREFIX Agent * getNewAgent(Model * model,long instNum)
{
if(model->isA(::tcpListenerWord))
        return(new OTCPListener(model,instNum));
if(model->isA(::tcpConnectionWord))
        return(new OTCPConnection(model,instNum));
if(model->isA(::udpTransmitterWord))
        return(new OUDPTransmitter(model,instNum));
return((Agent *)0); // not found
}

//--------------------------------------------------------------------------//
// TCP/UDP functions                                                        //
//--------------------------------------------------------------------------//

// native string getHostName(void);
//
void getHostName(Context * context)
{
context->returnStrings(SocketElem::getHostName());
}

// native string getIPAddress(string hostName);
//
void getIPAddress(Context * context)
{
context->returnStrings(SocketElem::getIPAddress(context->getStrings(0)));
}

//--------------------------------------------------------------------------//
// TCPListener functions                                                    //
//--------------------------------------------------------------------------//

// native void TCPListener::_new(int portNumber);
//
void OTCPListener::construct(Context * context) // static
{
OTCPListener * listener=(OTCPListener *)context->accessOwner();
if(listener->_listener) delete listener->_listener;
listener->_listener=new TCPListener(context->getInteger(0));
}

// native void TCPListener::_delete(void);
//
void OTCPListener::destruct(Context * context) // static
{
OTCPListener * listener=(OTCPListener *)context->accessOwner();
if(listener->_listener)
  {
  delete listener->_listener;
  listener->_listener=(TCPListener *)0;
  }
}

// native bool TCPListener::fail(void);
//
void OTCPListener::fail(Context * context) // static
{
const OTCPListener * listener=(const OTCPListener *)context->getOwner();
context->returnInteger(!listener->_listener||listener->_listener->fail());
}

// native int TCPListener::getPortNumber(void);
//
void OTCPListener::getPortNumber(Context * context) // static
{
const OTCPListener * listener=(const OTCPListener *)context->getOwner();
if(listener->_listener)
  context->returnInteger(listener->_listener->getPortNumber());
else
  context->returnInteger(-1);
}

// native string TCPListener::getIPAddress(void);
//
void OTCPListener::getIPAddress(Context * context) // static
{
const OTCPListener * listener=(const OTCPListener *)context->getOwner();
if(listener->_listener)
  context->returnStrings(listener->_listener->getIPAddress());
else
  context->returnStrings("");
}

// native int TCPListener::getNbPending(void);
//
void OTCPListener::getNbPending(Context * context) // static
{
const OTCPListener * listener=(const OTCPListener *)context->getOwner();
if(listener->_listener)
  context->returnInteger(listener->_listener->getNbPending());
else
  context->returnInteger(0);
}

// native TCPConnection TCPListener::nextConnection(string content);
//
void OTCPListener::nextConnection(Context * context) // static
{
OTCPListener * listener=(OTCPListener *)context->accessOwner();
TCPConnection * connection=(TCPConnection *)0;
Model * model=context->getModule()->getResultType().getModel();
if(listener->_listener) connection=listener->_listener->nextConnection();
if(connection)
  {
  Value * value=TmpValue::getTmpValue(ValueType(model));
  getSimul()->createAgent(value,true,model,0);
  ((OTCPConnection *)value->getAgent())->setConnection(connection);
  // skip constructor of TCPConnection
  getSimul()->startModule(Model::getBaseModel()->accessNewModule(),
                          value->getAgent(),context->accessExecution());
  context->returnValue(value);
  }
else context->returnAgent((Agent *)0,model);
}

//--------------------------------------------------------------------------//
// TCPConnection functions                                                  //
//--------------------------------------------------------------------------//
 
// native void TCPConnection::_new(int portNumber,string host);
//
void OTCPConnection::construct(Context * context) // static
{
OTCPConnection * connection=(OTCPConnection *)context->accessOwner();
if(connection->_connection) delete connection->_connection;
connection->_connection=new TCPConnection(context->getInteger(0),
                                          context->getStrings(1));
}
 
// native void TCPConnection::_delete(void);
//
void OTCPConnection::destruct(Context * context) // static
{
OTCPConnection * connection=(OTCPConnection *)context->accessOwner();
if(connection->_connection)
  {
  delete connection->_connection;
  connection->_connection=(TCPConnection *)0;
  }
}

// native bool TCPConnection::fail(void);
//
void OTCPConnection::fail(Context * context) // static
{
const OTCPConnection * connection=(const OTCPConnection *)context->getOwner();
context->returnInteger(!connection->_connection||
                       connection->_connection->fail());
}
 
// native int TCPConnection::getLocalPortNumber(void);
//
void OTCPConnection::getLocalPortNumber(Context * context) // static
{
const OTCPConnection * connection=(const OTCPConnection *)context->getOwner();
if(connection->_connection)
  context->returnInteger(connection->_connection->getLocalPortNumber());
else
  context->returnInteger(-1);
}

// native string TCPConnection::getLocalIPAddress(void);
//
void OTCPConnection::getLocalIPAddress(Context * context) // static
{
const OTCPConnection * connection=(const OTCPConnection *)context->getOwner();
if(connection->_connection)
  context->returnStrings(connection->_connection->getLocalIPAddress());
else
  context->returnStrings("");
}
 
// native int TCPConnection::getDistantPortNumber(void);
//
void OTCPConnection::getDistantPortNumber(Context * context) // static
{
const OTCPConnection * connection=(const OTCPConnection *)context->getOwner();
if(connection->_connection)
  context->returnInteger(connection->_connection->getDistantPortNumber());
else
  context->returnInteger(-1);
}
 
// native string TCPConnection::getDistantIPAddress(void);
//
void OTCPConnection::getDistantIPAddress(Context * context) // static
{
const OTCPConnection * connection=(const OTCPConnection *)context->getOwner();
if(connection->_connection)
  context->returnStrings(connection->_connection->getDistantIPAddress());
else
  context->returnStrings("");
}

// native bool TCPConnection::read(string & result);
//
void OTCPConnection::read(Context * context) // static
{
OTCPConnection * connection=(OTCPConnection *)context->accessOwner();
if(connection->_connection)
  {
  Strings result;
  if(connection->_connection->read(result))
    {
    context->setStrings(0,result);
    context->returnInteger(1);
    }
  else context->returnInteger(0);
  }
else context->returnInteger(0);
}
 
// native bool TCPConnection::readBlock(Memory result);
//
void OTCPConnection::readBlock(Context * context) // static
{
OTCPConnection * connection=(OTCPConnection *)context->accessOwner();
OMemory * memory=(OMemory *)context->getAgent(0);
if(memory&&memory->isAlive())
  {
  if(connection->_connection)
    {
    void * block;
    unsigned long nBytes;
    if(connection->_connection->readBytes(block,nBytes))
      {
      if(memory->getBlock()) BlockAllocator::free(memory->getBlock());
      memory->setBlock(block);
      memory->setSize(nBytes);
      context->returnInteger(1);
      }
    else context->returnInteger(0);
    }
  else context->returnInteger(0);
  }
else
  {
  context->returnInteger(0);
  Strings str;
  if(memory) str="deleted object '"+memory->getName().print()+"'";
  else str="'NONE'";
  Strings message="Can't store result in "+str;
  getSimul()->manageError(new Error(Error::RUNTIME,message));
  }
}

// native void TCPConnection::write(void);
//
void OTCPConnection::write(Context * context) // static
{
OTCPConnection * connection=(OTCPConnection *)context->accessOwner();
if(connection->_connection)
  {
  connection->_connection->write(context->getStrings(0));
  }
}

// native void TCPConnection::writeBlock(Memory mem,int offset,int length);
//
void OTCPConnection::writeBlock(Context * context) // static
{
OTCPConnection * connection=(OTCPConnection *)context->accessOwner();
if(connection->_connection)
  {
  const OMemory * memory=(const OMemory *)context->getAgent(0);
  int offset=context->getInteger(1);
  int length=context->getInteger(2);
  if(memory&&memory->isAlive()&&
     (offset>=0)&&(length>=0)&&(offset+length<=(int)memory->getSize()))
    {
    const char * block=(const char *)memory->getBlock();
    connection->_connection->writeBytes(block+offset,length);
    }
  }
}

//--------------------------------------------------------------------------//
// UDPTransmitter functions                                                 //
//--------------------------------------------------------------------------//
 
// native void UDPTransmitter::_new(int portNumber);
//
void OUDPTransmitter::construct(Context * context) // static
{
OUDPTransmitter * transmitter=(OUDPTransmitter *)context->accessOwner();
if(transmitter->_transmitter) delete transmitter->_transmitter;
transmitter->_transmitter=new UDPTransmitter(context->getInteger(0));
}
 
// native void UDPTransmitter::_delete(void);
//
void OUDPTransmitter::destruct(Context * context) // static
{
OUDPTransmitter * transmitter=(OUDPTransmitter *)context->accessOwner();
if(transmitter->_transmitter)
  {
  delete transmitter->_transmitter;
  transmitter->_transmitter=(UDPTransmitter *)0;
  }
}

// native bool UDPTransmitter::fail(void);
//
void OUDPTransmitter::fail(Context * context) // static
{
const OUDPTransmitter * transmitter=(const OUDPTransmitter *)
                                    context->getOwner();
context->returnInteger(!transmitter->_transmitter||
                       transmitter->_transmitter->fail());
}
 
// native int UDPTransmitter::getPortNumber(void);
//
void OUDPTransmitter::getPortNumber(Context * context) // static
{
const OUDPTransmitter * transmitter=(const OUDPTransmitter *)
                                    context->getOwner();
if(transmitter->_transmitter)
  context->returnInteger(transmitter->_transmitter->getPortNumber());
else
  context->returnInteger(-1);
}
 
// native string UDPTransmitter::getIPAddress(void);
//
void OUDPTransmitter::getIPAddress(Context * context) // static
{
const OUDPTransmitter * transmitter=(const OUDPTransmitter *)
                                    context->getOwner();
if(transmitter->_transmitter)
  context->returnStrings(transmitter->_transmitter->getIPAddress());
else
  context->returnStrings("");
}

// native bool UDPTransmitter::read(string & result);
//
void OUDPTransmitter::read(Context * context) // static
{
OUDPTransmitter * transmitter=(OUDPTransmitter *)context->accessOwner();
if(transmitter->_transmitter)
  {
  Strings result;
  if(transmitter->_transmitter->read(result))
    {
    context->setStrings(0,result);
    context->returnInteger(1);
    }
  else context->returnInteger(0);
  }
else context->returnInteger(0);
}
 
// native bool UDPTransmitter::readBlock(Memory result);
//
void OUDPTransmitter::readBlock(Context * context) // static
{
OUDPTransmitter * transmitter=(OUDPTransmitter *)context->accessOwner();
OMemory * memory=(OMemory *)context->getAgent(0);
if(memory&&memory->isAlive())
  {
  if(transmitter->_transmitter)
    {
    void * block;
    unsigned long nBytes;
    if(transmitter->_transmitter->readBytes(block,nBytes))
      {
      if(memory->getBlock()) BlockAllocator::free(memory->getBlock());
      memory->setBlock(block);
      memory->setSize(nBytes);
      context->returnInteger(1);
      }
    else context->returnInteger(0);
    }
  else context->returnInteger(0);
  }
else
  {
  context->returnInteger(0);
  Strings str;
  if(memory) str="deleted object '"+memory->getName().print()+"'";
  else str="'NONE'";
  Strings message="Can't store result in "+str;
  getSimul()->manageError(new Error(Error::RUNTIME,message));
  }
}

// native bool UDPTransmitter::readFrom(string & result,
//                                      int & portNumber,
//                                      string & ipAddress);
//
void OUDPTransmitter::readFrom(Context * context) // static
{
OUDPTransmitter * transmitter=(OUDPTransmitter *)context->accessOwner();
if(transmitter->_transmitter)
  {
  Strings result;
  int portNumber;
  Strings ipAddress;
  if(transmitter->_transmitter->readFrom(result,portNumber,ipAddress))
    {
    context->setStrings(0,result);
    context->setInteger(1,portNumber);
    context->setStrings(2,ipAddress);
    context->returnInteger(1);
    }
  else context->returnInteger(0);
  }
else context->returnInteger(0);
}

// native bool UDPTransmitter::readBlockFrom(Memory result,
//                                           int & portNumber,
//                                           string & ipAddress);
//
void OUDPTransmitter::readBlockFrom(Context * context) // static
{
OUDPTransmitter * transmitter=(OUDPTransmitter *)context->accessOwner();
OMemory * memory=(OMemory *)context->getAgent(0);
if(memory&&memory->isAlive())
  {
  if(transmitter->_transmitter)
    {
    void * block;
    unsigned long nBytes;
    int portNumber;
    Strings ipAddress;
    if(transmitter->_transmitter
                  ->readBytesFrom(block,nBytes,portNumber,ipAddress))
      {
      context->setInteger(1,portNumber);
      context->setStrings(2,ipAddress);
      if(memory->getBlock()) BlockAllocator::free(memory->getBlock());
      memory->setBlock(block);
      memory->setSize(nBytes);
      context->returnInteger(1);
      }
    else context->returnInteger(0);
    }
  else context->returnInteger(0);
  }
else
  {
  context->returnInteger(0);
  Strings str;
  if(memory) str="deleted object '"+memory->getName().print()+"'";
  else str="'NONE'";
  Strings message="Can't store result in "+str;
  getSimul()->manageError(new Error(Error::RUNTIME,message));
  }
}

// native void UDPTransmitter::writeTo(string str,
//                                     int portNumber,string host);
//
void OUDPTransmitter::writeTo(Context * context) // static
{
OUDPTransmitter * transmitter=(OUDPTransmitter *)context->accessOwner();
if(transmitter->_transmitter)
  {
  transmitter->_transmitter->writeTo(context->getStrings(0),
                                     context->getInteger(1),
                                     context->getStrings(2));
  }
}

// native void UDPTransmitter::writeBlockTo(Memory mem,int offset,int length,
//                                          int portNumber,string host);
//
void OUDPTransmitter::writeBlockTo(Context * context) // static
{
OUDPTransmitter * transmitter=(OUDPTransmitter *)context->accessOwner();
if(transmitter->_transmitter)
  {
  const OMemory * memory=(const OMemory *)context->getAgent(0);
  int offset=context->getInteger(1);
  int length=context->getInteger(2);
  if(memory&&memory->isAlive()&&
     (offset>=0)&&(length>=0)&&(offset+length<=(int)memory->getSize()))
    {
    const char * block=(const char *)memory->getBlock();
    transmitter->_transmitter->writeBytesTo(block+offset,length,
                                            context->getInteger(3),
                                            context->getStrings(4));
    }
  }
}

// native void UDPTransmitter::broadcast(string str);
//
void OUDPTransmitter::broadcast(Context * context) // static
{
OUDPTransmitter * transmitter=(OUDPTransmitter *)context->accessOwner();
if(transmitter->_transmitter)
  {
  transmitter->_transmitter->broadcast(context->getStrings(0));
  }
}

// native void UDPTransmitter::broadcastBlock(Memory mem,
//                                            int offset,int length);
//
void OUDPTransmitter::broadcastBlock(Context * context) // static
{
OUDPTransmitter * transmitter=(OUDPTransmitter *)context->accessOwner();
if(transmitter->_transmitter)
  {
  const OMemory * memory=(const OMemory *)context->getAgent(0);
  int offset=context->getInteger(1);
  int length=context->getInteger(2);
  if(memory&&memory->isAlive()&&
     (offset>=0)&&(length>=0)&&(offset+length<=(int)memory->getSize()))
    {
    const char * block=(const char *)memory->getBlock();
    transmitter->_transmitter->broadcastBytes(block+offset,length);
    }
  }
}

//--------------------------------------------------------------------------//
// class OTCPListener                                                       //
//--------------------------------------------------------------------------//

OTCPListener::OTCPListener(Model * model,int instNum)
 : Agent(model,instNum),
   _listener((TCPListener *)0)
{}

OTCPListener::~OTCPListener(void)
{
if(_listener) delete _listener;
}

Strings OTCPListener::userSave(int /* column */ ) const // virtual
{
Strings str;
return(str);
}

//--------------------------------------------------------------------------//
// class OTCPConnection                                                     //
//--------------------------------------------------------------------------//
 
OTCPConnection::OTCPConnection(Model * model,int instNum)
 : Agent(model,instNum),
   _connection((TCPConnection *)0)
{}
 
OTCPConnection::~OTCPConnection(void)
{
if(_connection) delete _connection;
}
 
Strings OTCPConnection::userSave(int /* column */ ) const // virtual
{
Strings str;
return(str);
}

void OTCPConnection::setConnection(TCPConnection * connection)
{
if(_connection) delete _connection;
_connection=connection;
}

//--------------------------------------------------------------------------//
// class OUDPTransmitter                                                    //
//--------------------------------------------------------------------------//
 
OUDPTransmitter::OUDPTransmitter(Model * model,int instNum)
 : Agent(model,instNum),
   _transmitter((UDPTransmitter *)0)
{}
 
OUDPTransmitter::~OUDPTransmitter(void)
{
if(_transmitter) delete _transmitter;
}
 
Strings OUDPTransmitter::userSave(int /* column */ ) const // virtual
{
Strings str;
return(str);
}

