// @(#) ENIB/LI2 (c) - oRis - Version 2.3
//--------------------------------------------------------------------------//
// file        : tcpUdpPkg.h                                                //
// description : header file for the ENIB/LI2 oRis package tcpUdp.pkg       //
// author(s)   : F. Harrouet                                                //
// copyright   : Laboratoire d'Informatique Industrielle (LI2)              //
//               Ecole Nationale d'Ingenieurs de Brest (ENIB)               //
//               CP 30815 - 29608 Brest Cedex - FRANCE                      //
//               Tel: (33) 02.98.05.66.31 , Fax: (33) 02.98.05.66.29        //
//               e-mail: li2@enib.fr                                        //
//--------------------------------------------------------------------------//
// Copyright (c) 1996-2000                                                  //
// Laboratoire d'Informatique Industrielle (LI2)                            //
// Ecole Nationale d'Ingenieurs de Brest (ENIB)                             //
//                                                                          //
// Permission to use, copy, modify, distribute and sell this software       //
// and its documentation for any purpose is hereby granted without fee,     //
// provided that the above copyright notice appear in all copies and        //
// that both that copyright notice and this permission notice appear        //
// in supporting documentation.                                             //
// ENIB/LI2 makes no representations about the suitability of this          //
// software for any purpose.                                                //
// It is provided "as is" without express or implied warranty.              //
//--------------------------------------------------------------------------//

#ifndef TCPUDPPKG_H
#define TCPUDPPKG_H

#include "oRis/Main/agent.h"

#include "Utility/tcpUdp.h"

//--------------------------------------------------------------------------//
// TCP/UDP functions                                                        //
//--------------------------------------------------------------------------//
 
// native string getHostName(void);
//
void getHostName(Context * context);
 
// native string getIPAddress(string hostName);
//
void getIPAddress(Context * context);

//--------------------------------------------------------------------------//
// class OTCPListener                                                       //
//--------------------------------------------------------------------------//

class OTCPListener : public Agent
{
public:
  OTCPListener(Model * model,int instNum);
  virtual ~OTCPListener(void);
  virtual Strings userSave(int column=0) const;

  // native void TCPListener::_new(int portNumber);
  //
  static void construct(Context * context);

  // native void TCPListener::_delete(void);
  //
  static void destruct(Context * context);

  // native bool TCPListener::fail(void);
  //
  static void fail(Context * context);

  // native int TCPListener::getPortNumber(void);
  //
  static void getPortNumber(Context * context);

  // native string TCPListener::getIPAddress(void);
  //
  static void getIPAddress(Context * context);

  // native int TCPListener::getNbPending(void);
  //
  static void getNbPending(Context * context);

  // native TCPConnection TCPListener::nextConnection(void);
  //
  static void nextConnection(Context * context);

//---- Attributes ----
protected:
  TCPListener * _listener;
};

//--------------------------------------------------------------------------//
// class OTCPConnection                                                     //
//--------------------------------------------------------------------------//

class OTCPConnection : public Agent
{
public:
  OTCPConnection(Model * model,int instNum);
  virtual ~OTCPConnection(void);
  virtual Strings userSave(int column=0) const;
  void setConnection(TCPConnection * connection);
 
  // native void TCPConnection::_new(int portNumber,string host);
  //
  static void construct(Context * context);                                     

  // native void TCPConnection::_delete(void);
  //
  static void destruct(Context * context);

  // native bool TCPConnection::fail(void);
  //
  static void fail(Context * context);
 
  // native int TCPConnection::getLocalPortNumber(void);
  //
  static void getLocalPortNumber(Context * context);

  // native string TCPConnection::getLocalIPAddress(void);
  //
  static void getLocalIPAddress(Context * context);

  // native int TCPConnection::getDistantPortNumber(void);
  //
  static void getDistantPortNumber(Context * context);

  // native string TCPConnection::getDistantIPAddress(void);
  //
  static void getDistantIPAddress(Context * context);

  // native bool TCPConnection::read(string & result);
  //
  static void read(Context * context);

  // native bool TCPConnection::readBlock(Memory result);
  //
  static void readBlock(Context * context);

  // native void TCPConnection::write(string str);
  //
  static void write(Context * context);

  // native void TCPConnection::writeBlock(Memory mem,int offset,int length);
  //
  static void writeBlock(Context * context);

//---- Attributes ----
protected:
  TCPConnection * _connection;
};

//--------------------------------------------------------------------------//
// class OUDPTransmitter                                                    //
//--------------------------------------------------------------------------//
 
class OUDPTransmitter : public Agent
{
public:
  OUDPTransmitter(Model * model,int instNum);
  virtual ~OUDPTransmitter(void);
  virtual Strings userSave(int column=0) const;
 
  // native void UDPTransmitter::_new(int portNumber);
  //
  static void construct(Context * context);
 
  // native void UDPTransmitter::_delete(void);
  //
  static void destruct(Context * context);
 
  // native bool UDPTransmitter::fail(void);
  //
  static void fail(Context * context);
 
  // native int UDPTransmitter::getPortNumber(void);
  //
  static void getPortNumber(Context * context);
 
  // native string UDPTransmitter::getIPAddress(void);
  //
  static void getIPAddress(Context * context);
 
  // native bool UDPTransmitter::read(string & result);
  //
  static void read(Context * context);

  // native bool UDPTransmitter::readBlock(Memory result);
  //
  static void readBlock(Context * context);

  // native bool UDPTransmitter::readFrom(string & result,
  //                                      int & portNumber,
  //                                      string & ipAddress);
  //
  static void readFrom(Context * context);

  // native bool UDPTransmitter::readBlockFrom(Memory result,
  //                                           int & portNumber,
  //                                           string & ipAddress);
  //
  static void readBlockFrom(Context * context);

  // native void UDPTransmitter::writeTo(string str,
  //                                     int portNumber,string host);
  //
  static void writeTo(Context * context);
 
  // native void UDPTransmitter::writeBlockTo(Memory mem,
  //                                          int offset,int length,
  //                                          int portNumber,string host);
  //
  static void writeBlockTo(Context * context);
 
  // native void UDPTransmitter::broadcast(string str);
  //
  static void broadcast(Context * context);
 
  // native void UDPTransmitter::broadcastBlock(Memory mem,
  //                                            int offset,int length);
  //
  static void broadcastBlock(Context * context);
 
//---- Attributes ----
protected:
  UDPTransmitter * _transmitter;
};

#endif // TCPUDPPKG_H

