// @(#) ENIB/LI2 (c) - oRis - Version 2.3
//--------------------------------------------------------------------------//
// file        : oRisBase.cpp                                               //
// description : source file for the ENIB/LI2 oRis base classes library     //
// author(s)   : F. Harrouet                                                //
// copyright   : Laboratoire d'Informatique Industrielle (LI2)              //
//               Ecole Nationale d'Ingenieurs de Brest (ENIB)               //
//               CP 30815 - 29608 Brest Cedex - FRANCE                      //
//               Tel: (33) 02.98.05.66.31 , Fax: (33) 02.98.05.66.29        //
//               e-mail: li2@enib.fr                                        //
//--------------------------------------------------------------------------//
// Copyright (c) 1996-2000                                                  //
// Laboratoire d'Informatique Industrielle (LI2)                            //
// Ecole Nationale d'Ingenieurs de Brest (ENIB)                             //
//                                                                          //
// Permission to use, copy, modify, distribute and sell this software       //
// and its documentation for any purpose is hereby granted without fee,     //
// provided that the above copyright notice appear in all copies and        //
// that both that copyright notice and this permission notice appear        //
// in supporting documentation.                                             //
// ENIB/LI2 makes no representations about the suitability of this          //
// software for any purpose.                                                //
// It is provided "as is" without express or implied warranty.              //
//--------------------------------------------------------------------------//

#include "oRis/Main/simul.h"
#include "oRis/Main/context.h"

#ifndef __MSDOS__
#define EXPORT_PREFIX extern "C"
#else
#include <windows.h>
#define EXPORT_PREFIX extern "C" __declspec(dllexport)
#define openingLibrary    openingORisBaseLibrary
#define closingLibrary    closingORisBaseLibrary
#define getExternFunction getExternORisBaseFunction
#define getNewAgent       getNewORisBaseAgent
#endif

//--------------------------------------------------------------------------//
// Opening and closing the library                                          //
//--------------------------------------------------------------------------//

EXPORT_PREFIX void openingLibrary(void)
{
  // nothing to do
}

EXPORT_PREFIX void closingLibrary(void)
{
  // nothing to do
}

//--------------------------------------------------------------------------//
// LookUp table for functions                                               //
//--------------------------------------------------------------------------//

// native "::nop()" void a_module_name(void); // modif Vincent Rodin
//
void stNop(Context * /* context */)
{
  // No operation !
}

void stCallAnotherDelete(Context * context) // modif Vincent Rodin
{
 Agent *agt = context->accessOwner();

 if (agt!=NULL) {

    agt->newBirth();

    Strings agtName =  context->accessOwner()->getName().print();
    Strings aString = "execute{ "+agtName+"->anotherDelete(); }";
    getSimul()->parseStrings(aString);

    agt->kill();
 }
 else { cerr << "Thank you for calling `callAnotherDelete' ";
        cerr << "function" << endl;
 }
}

EXPORT_PREFIX ExternFunction getExternFunction(const Strings & name)
{
// No operation !
if(name==Strings("::nop()"))               // modif Vincent Rodin
    return(::stNop);
// To Call an other delete method
if(name==Strings("::callAnotherDelete()")) // modif Vincent Rodin
    return(::stCallAnotherDelete);

// Default functions
if(name==Strings("::onExit()"))
    return(Simulator::stOnExit);
if(name==Strings("::loadLibrary()"))
    return(Simulator::stLoadLibrary);
if(name==Strings("::parse()"))
    return(Simulator::stParse);
if(name==Strings("::load()"))
    return(Simulator::stLoad);
if(name==Strings("::parseEval()"))
    return(Simulator::stParseEval);
if(name==Strings("::eval()"))
    return(Simulator::stEval);
if(name==Strings("::save()"))
    return(Simulator::stSave);
if(name==Strings("::getPickingMode()"))
    return(Simulator::stGetPickingMode);
if(name==Strings("::setPickingMode()"))
    return(Simulator::stSetPickingMode);
if(name==Strings("::getInterval()"))
    return(Simulator::stGetInterval);
if(name==Strings("::setInterval()"))
    return(Simulator::stSetInterval);
if(name==Strings("::yield()"))
    return(Simulator::stYield);
if(name==Strings("::softYield()"))
    return(Simulator::stSoftYield);
if(name==Strings("::getExecutionStack()"))
    return(Simulator::stGetExecutionStack);
if(name==Strings("::printStack()"))
    return(Simulator::stPrintStack);
if(name==Strings("::currentExecution()"))
    return(Simulator::stCurrentExecution);
if(name==Strings("::abort()"))
    return(Simulator::stAbort);
if(name==Strings("::suspend()"))
    return(Simulator::stSuspend);
if(name==Strings("::resume()"))
    return(Simulator::stResume);
if(name==Strings("::isSuspended()"))
    return(Simulator::stIsSuspended);
if(name==Strings("::getMaxDepth()"))
    return(Simulator::stGetMaxDepth);
if(name==Strings("::setMaxDepth()"))
    return(Simulator::stSetMaxDepth);
if(name==Strings("::getClock()"))
    return(Simulator::stGetClock);
if(name==Strings("::setClock()"))
    return(Simulator::stSetClock);
if(name==Strings("::create()"))
    return(Simulator::stCreate);
if(name==Strings("::getObject()"))
    return(Simulator::stGetObject);
if(name==Strings("::exist()"))
    return(Simulator::stExist);
if(name==Strings("::numberOf()"))
    return(Simulator::stNumberOf);
if(name==Strings("::getAll()"))
    return(Simulator::stGetAll);
if(name==Strings("::getClasses()"))
    return(Simulator::stGetClasses);
if(name==Strings("::getTime()"))
    return(Simulator::stGetTime);
if(name==Strings("::getenv()"))
    return(Simulator::stGetenv);
if(name==Strings("::putenv()"))
    return(Simulator::stPutenv);
if(name==Strings("::systemName()"))
    return(Simulator::stSystemName);
if(name==Strings("::systemYield()"))
    return(Simulator::stSystemYield);
if(name==Strings("::system()"))
    return(Simulator::stSystem);
if(name==Strings("::systemI()"))
    return(Simulator::stSystemI);
if(name==Strings("::systemO()"))
    return(Simulator::stSystemO);
if(name==Strings("::systemE()"))
    return(Simulator::stSystemE);
if(name==Strings("::systemIO()"))
    return(Simulator::stSystemIO);
if(name==Strings("::systemIE()"))
    return(Simulator::stSystemIE);
if(name==Strings("::systemOE()"))
    return(Simulator::stSystemOE);
if(name==Strings("::systemIOE()"))
    return(Simulator::stSystemIOE);
if(name==Strings("::control()"))
    return(Simulator::stControl);
if(name==Strings("::commandLine()"))
    return(Simulator::stCommandLine);
if(name==Strings("::asciiCode()"))
    return(Simulator::stAsciiCode);
if(name==Strings("::asciiChar()"))
    return(Simulator::stAsciiChar);
if(name==Strings("::hexaString()"))
    return(Simulator::stHexaString);
if(name==Strings("::octalString()"))
    return(Simulator::stOctalString);
if(name==Strings("::binaryString()"))
    return(Simulator::stBinaryString);
if(name==Strings("::locateFile()"))
    return(Simulator::stLocateFile);
if(name==Strings("::showURL()"))
    return(Simulator::stShowURL);
if(name==Strings("::readln()"))
    return(Simulator::stReadln);

// class Message
if(name==Strings("Message::sendTo()"))
    return(Agent::stMsgSendTo);
if(name==Strings("Message::broadcast()"))
    return(Agent::stMsgBroadcast);

// class Object
if(name==Strings("Object::name()"))
    return(Agent::stName);
if(name==Strings("Object::isA()"))
    return(Agent::stIsA);
if(name==Strings("Object::className()"))
    return(Agent::stClassName);
if(name==Strings("Object::suffix()"))
    return(Agent::stSuffix);
if(name==Strings("Object::suspend()"))
    return(Agent::stSuspend);
if(name==Strings("Object::resume()"))
    return(Agent::stResume);
if(name==Strings("Object::isSuspended()"))
    return(Agent::stIsSuspended);
if(name==Strings("Object::setTransient()"))
    return(Agent::stSetTransient);
if(name==Strings("Object::isTransient()"))
    return(Agent::stIsTransient);
if(name==Strings("Object::getSensitivity()"))
    return(Agent::stGetSensitivity);
if(name==Strings("Object::setSensitivity()"))
    return(Agent::stSetSensitivity);
if(name==Strings("Object::cancelSensitivity()"))
    return(Agent::stCancelSensitivity);
if(name==Strings("Object::getNbMessages()"))
    return(Agent::stGetNbMessages);
if(name==Strings("Object::peekMessage()"))
    return(Agent::stPeekMessage);
if(name==Strings("Object::getNextMessage()"))
    return(Agent::stGetNextMessage);
if(name==Strings("Object::putBackMessage()"))
    return(Agent::stPutBackMessage);
if(name==Strings("Object::asynchronousReception()"))
    return(Agent::stAsynchronousReception);

return((ExternFunction)0); // not found
}

//--------------------------------------------------------------------------//
// LookUp table for agents                                                  //
//--------------------------------------------------------------------------//

EXPORT_PREFIX Agent * getNewAgent(Model * model,long instNum)
{
return(new Agent(model,instNum));
}

