// @(#) ENIB/LI2 (c) - oRis - Version 2.3
//--------------------------------------------------------------------------//
// file        : main.cpp                                                   //
// description : source file for the ENIB/LI2 oRis program entry point      //
// author(s)   : F. Harrouet                                                //
// copyright   : Laboratoire d'Informatique Industrielle (LI2)              //
//               Ecole Nationale d'Ingenieurs de Brest (ENIB)               //
//               CP 30815 - 29608 Brest Cedex - FRANCE                      //
//               Tel: (33) 02.98.05.66.31 , Fax: (33) 02.98.05.66.29        //
//               e-mail: li2@enib.fr                                        //
//--------------------------------------------------------------------------//
// Copyright (c) 1996-2000                                                  //
// Laboratoire d'Informatique Industrielle (LI2)                            //
// Ecole Nationale d'Ingenieurs de Brest (ENIB)                             //
//                                                                          //
// Permission to use, copy, modify, distribute and sell this software       //
// and its documentation for any purpose is hereby granted without fee,     //
// provided that the above copyright notice appear in all copies and        //
// that both that copyright notice and this permission notice appear        //
// in supporting documentation.                                             //
// ENIB/LI2 makes no representations about the suitability of this          //
// software for any purpose.                                                //
// It is provided "as is" without express or implied warranty.              //
//--------------------------------------------------------------------------//

#include "Utility/tcpUdp.h"
#include "Utility/ipcMutex.h"
#include "Utility/ipcShMem.h"
#include "Utility/smplThrd.h"
#include "Utility/system.h"

bool exitHandler(int /* signum */ )   // relax IPCs, sockets and threads
{                                     // when the process is abnormally
SimpleThread::relaxThreads();         // terminated
SocketElem::relaxSockets();
IPCMutex::relaxIPCs();
IPCSharedMemory::relaxIPCs();
return(true);
}

#if defined(X_GUI) || defined(W_GUI)

#include "orisCtrl.h"

int main(int argc,char ** argv)
{
int result=0;
System::installExitHandler(exitHandler);
GuiManager * manager=new GuiManager;
if(manager->fail())
  {
  cerr << "Can't connect to X Server" << endl;
  result=1;
  }
else
  {
  StringsVect commandLine;
  for(int i=0;i<argc;i++)
    commandLine.push_back(argv[i]);
  OrisControl * control=new OrisControl(commandLine);
  manager->mainLoop();
  delete control;
  }
delete manager;
::exitHandler(0);
return(result);
}

#else // X_GUI || W_GUI

#include "oRis/Main/simul.h"
#include "oRis/Main/textOut.h"

#include <stdio.h>
#include <signal.h>

bool quit=false;

Strings controlCB(Simulator *,void *,
                  const Strings & command,const Strings & arg)
{
if(command==Strings("quit"))
  {
  quit=true;
  return("yes");
  }
else if(command==Strings("state"))
  {
  return("run");
  }
else if(command==Strings("run"))
  {
  return("");
  }
else if(command==Strings("stop"))
  {
  return("command '"+command+"' not supported in text mode");
  }
else if(command==Strings("one step"))
  {
  return("command '"+command+"' not supported in text mode");
  }
else if(command==Strings("get step"))
  {
  return("1");
  }
else if(command==Strings("set step"))
  {
  return("command '"+command+"' not supported in text mode");
  }
else if(command==Strings("get shape"))
  {
  return("command '"+command+"' not supported in text mode");
  }
else if(command==Strings("set shape"))
  {
  return("command '"+command+"' not supported in text mode");
  }
else if(command==Strings("about"))
  {
  cout << "o R i s : Active Object Simulator"             << endl;
  cout                                                    << endl;
  cout << "  - Version 2.3 (text only)"                   << endl;
  cout << "  - Build date : May, 16th, 2001"              << endl;
  cout << "  - Copyright (c) 1996-2001"                   << endl;
  cout << "_____________________________________________" << endl;
  cout << ""                                              << endl;
  cout << "Laboratoire d'Informatique Industrielle (LI2)" << endl;
  cout << "Ecole Nationale d'Ingenieurs de Brest (ENIB)"  << endl;
  cout << ""                                              << endl;
  cout << "  - CP 30815 - 29608 Brest Cedex - FRANCE"     << endl;
  cout << "  - Tel: +33 (0)298 05 66 31"                  << endl;
  cout << "  - Fax: +33 (0)298 05 66 29"                  << endl;
  cout << "_____________________________________________" << endl;
  cout << ""                                              << endl;
  cout << "Contact: Fabrice HARROUET"                     << endl;
  cout << "  - e-mail: harrouet@enib.fr"                  << endl;
  cout << "  - url: http://www.enib.fr/~harrouet/"        << endl;
  return("");
  }
else if(command==Strings("manual"))
  {
  Strings url="file:"+getSimul()->getInstallDir()+"Doc/index.html";
  bool newWindow=true;
  if((arg==Strings("false"))||(arg==Strings("0"))) newWindow=false;
  getSimul()->showURL(url,newWindow);
  return("");
  }
return("unknown control");
}

void abortSignal(int)
{
getSimul()->abortSignal();
quit=true;
}

int main(int argc,char ** argv)
{
System::installExitHandler(exitHandler);
struct sigaction act;
act.sa_handler=abortSignal;
sigemptyset(&act.sa_mask);
act.sa_flags=0;
sigaction(SIGINT,&act,(struct sigaction *)0);
StringsVect commandLine;
for(int i=0;i<argc;i++)
  commandLine.push_back(argv[i]);
bool mustInsert=true;
if(commandLine.size()>1)
  {
  if((commandLine[1]==Strings("-gui"))||
     (commandLine[1]==Strings("-text")))
    {
    commandLine[1]=Strings("-text");
    mustInsert=false;
    }
  }
if(mustInsert) commandLine.insert(commandLine.begin()+1,Strings("-text"));
if(commandLine.size()<3)
  {
  cerr << "usage : " << argv[0] << " [-text] file.ors" << endl;
  return(1);
  }
Strings fileName(commandLine[2]);
TextOutput out(&cout);
TextOutput err(&cerr);
Simulator * simul=new Simulator(commandLine,&out,&err);
simul->setControlCB(controlCB,(void *)0);
simul->parseFile(fileName);
while(!simul->isBad()&&!quit) simul->run(1);
delete simul;
::exitHandler(0);
return(0);
}

#endif // X_GUI || W_GUI

