// @(#) ENIB/LI2 (c) - oRis - Version 2.3
//--------------------------------------------------------------------------//
// file        : affectCd.cpp                                               //
// description : source file for the ENIB/LI2 Affectation codes             //
// author(s)   : F. Harrouet                                                //
// copyright   : Laboratoire d'Informatique Industrielle (LI2)              //
//               Ecole Nationale d'Ingenieurs de Brest (ENIB)               //
//               CP 30815 - 29608 Brest Cedex - FRANCE                      //
//               Tel: (33) 02.98.05.66.31 , Fax: (33) 02.98.05.66.29        //
//               e-mail: li2@enib.fr                                        //
//--------------------------------------------------------------------------//
// Copyright (c) 1996-2000                                                  //
// Laboratoire d'Informatique Industrielle (LI2)                            //
// Ecole Nationale d'Ingenieurs de Brest (ENIB)                             //
//                                                                          //
// Permission to use, copy, modify, distribute and sell this software       //
// and its documentation for any purpose is hereby granted without fee,     //
// provided that the above copyright notice appear in all copies and        //
// that both that copyright notice and this permission notice appear        //
// in supporting documentation.                                             //
// ENIB/LI2 makes no representations about the suitability of this          //
// software for any purpose.                                                //
// It is provided "as is" without express or implied warranty.              //
//--------------------------------------------------------------------------//

#include "oRis/Codes/affectCd.h"
#include "oRis/Main/context.h"
#include "oRis/Main/simul.h"
#include "oRis/Main/exec.h"
#include "oRis/Main/tmpVal.h"
#include "oRis/Main/errors.h"

#include <math.h>

//--------------------------------------------------------------------------//
// class Affect_I_I_Code                                                    //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
Affect_I_I_Code::Affect_I_I_Code(const SmaNode * node)
 : SmaCode(node)
{}

Affect_I_I_Code::~Affect_I_I_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void Affect_I_I_Code::eval(Context * context) const // virtual
{
Value * value=context->accessExecution()->popValue();
Value * result=context->accessExecution()->lastValue();
value->setInteger(result->getInteger());
value->relax();
}


//--------------------------------------------------------------------------//
// class Affect_I_R_Code                                                    //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
Affect_I_R_Code::Affect_I_R_Code(const SmaNode * node)
 : SmaCode(node)
{}

Affect_I_R_Code::~Affect_I_R_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void Affect_I_R_Code::eval(Context * context) const // virtual
{
Value * value=context->accessExecution()->popValue();
Value * result=context->accessExecution()->lastValue();
value->setInteger((long)result->getReal());
value->relax();
}


//--------------------------------------------------------------------------//
// class Affect_R_R_Code                                                    //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
Affect_R_R_Code::Affect_R_R_Code(const SmaNode * node)
 : SmaCode(node)
{}

Affect_R_R_Code::~Affect_R_R_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void Affect_R_R_Code::eval(Context * context) const // virtual
{
Value * value=context->accessExecution()->popValue();
Value * result=context->accessExecution()->lastValue();
value->setReal(result->getReal());
value->relax();
}


//--------------------------------------------------------------------------//
// class Affect_R_I_Code                                                    //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
Affect_R_I_Code::Affect_R_I_Code(const SmaNode * node)
 : SmaCode(node)
{}

Affect_R_I_Code::~Affect_R_I_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void Affect_R_I_Code::eval(Context * context) const // virtual
{
Value * value=context->accessExecution()->popValue();
Value * result=context->accessExecution()->lastValue();
value->setReal((double)result->getInteger());
value->relax();
}


//--------------------------------------------------------------------------//
// class Affect_S_S_Code                                                    //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
Affect_S_S_Code::Affect_S_S_Code(const SmaNode * node)
 : SmaCode(node)
{}

Affect_S_S_Code::~Affect_S_S_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void Affect_S_S_Code::eval(Context * context) const // virtual
{
Value * value=context->accessExecution()->popValue();
Value * result=context->accessExecution()->lastValue();
value->setStrings(result->getStrings());
value->relax();
}


//--------------------------------------------------------------------------//
// class Affect_A_A_Code                                                    //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
Affect_A_A_Code::Affect_A_A_Code(const SmaNode * node)
 : SmaCode(node)
{}

Affect_A_A_Code::~Affect_A_A_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void Affect_A_A_Code::eval(Context * context) const // virtual
{
Value * value=context->accessExecution()->popValue();
Value * result=context->accessExecution()->lastValue();
value->setAgent(result->getAgent());
value->accessType().setModel(result->getType().getModel());
value->relax();
}


//--------------------------------------------------------------------------//
// class Affect_Deep_Code                                                   //
//--------------------------------------------------------------------------//
 
Word Affect_Deep_Code::_affectWord(Word::create("affect")); // static

//-------- Allocators ------------------------------------------------------//
Affect_Deep_Code::Affect_Deep_Code(const SmaNode * node)
 : SmaCode(node)
{}
 
Affect_Deep_Code::~Affect_Deep_Code(void)
{}
 
//-------- Evaluation ------------------------------------------------------//
void Affect_Deep_Code::eval(Context * context) const // virtual
{
Value * value=context->accessExecution()->popValue();
Value * result=context->accessExecution()->lastValue();
Agent * agent=value->getAgent();
if(agent&&agent->isAlive())
  {
  ExecModule * module=agent->accessModule(_affectWord);
  Execution * initExec=Execution::getExecution(agent,ExecModule::NORMAL);
  Value * val=TmpValue::getTmpValue(result->getType());
  val->setAgent(result->getAgent());
  initExec->pushValue(val);
  getSimul()->executeModule(module,agent,initExec);
  Execution::relaxExecution(initExec);
  value->relax();
  }
else
  {
  Strings str;
  if(agent) str="deleted object '"+value->print()+"'";
  else str="'NONE'";
  value->relax();
  Strings message="Can't use module 'affect()' on "+str;
  getSimul()->manageError(new Error(Error::RUNTIME,message));
  }
}

//--------------------------------------------------------------------------//
// class Affect_V_V_Code                                                    //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
Affect_V_V_Code::Affect_V_V_Code(const SmaNode * node)
 : SmaCode(node)
{}

Affect_V_V_Code::~Affect_V_V_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void Affect_V_V_Code::eval(Context * context) const // virtual
{
Value * value=context->accessExecution()->popValue();
Value * result=context->accessExecution()->lastValue();
value->setValueVectorCopy(*result);
value->relax();
}


//--------------------------------------------------------------------------//
// class Affect_IV_RV_Code                                                  //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
Affect_IV_RV_Code::Affect_IV_RV_Code(const SmaNode * node)
 : SmaCode(node)
{}

Affect_IV_RV_Code::~Affect_IV_RV_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void Affect_IV_RV_Code::eval(Context * context) const // virtual
{
Value * value=context->accessExecution()->popValue();
Value * result=context->accessExecution()->lastValue();
value->setValueVectorCopyR2I(*result->getValueVector());
value->relax();
}


//--------------------------------------------------------------------------//
// class Affect_RV_IV_Code                                                  //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
Affect_RV_IV_Code::Affect_RV_IV_Code(const SmaNode * node)
 : SmaCode(node)
{}

Affect_RV_IV_Code::~Affect_RV_IV_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void Affect_RV_IV_Code::eval(Context * context) const // virtual
{
Value * value=context->accessExecution()->popValue();
Value * result=context->accessExecution()->lastValue();
value->setValueVectorCopyI2R(*result->getValueVector());
value->relax();
}


//--------------------------------------------------------------------------//
// class AffectCastCode                                                     //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
AffectCastCode::AffectCastCode(const SmaNode * node)
 : SmaCode(node)
{}

AffectCastCode::~AffectCastCode(void)
{}

//-------- Evaluation ------------------------------------------------------//
void AffectCastCode::eval(Context * context) const // virtual
{
Value * value=context->accessExecution()->popValue();
Value * result=context->accessExecution()->lastValue();
value->affectCast(*result);
value->relax();
}


//--------------------------------------------------------------------------//
// class PlusEq_V_V_Code                                                    //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
PlusEq_V_V_Code::PlusEq_V_V_Code(const SmaNode * node)
 : SmaCode(node)
{}

PlusEq_V_V_Code::~PlusEq_V_V_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void PlusEq_V_V_Code::eval(Context * context) const // virtual
{
Value * value=context->accessExecution()->popValue();
Value * result=context->accessExecution()->lastValue();
result->appendVector(*value->getValueVector());
value->relax();
}


//--------------------------------------------------------------------------//
// class PlusEq_IV_RV_Code                                                  //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
PlusEq_IV_RV_Code::PlusEq_IV_RV_Code(const SmaNode * node)
 : SmaCode(node)
{}

PlusEq_IV_RV_Code::~PlusEq_IV_RV_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void PlusEq_IV_RV_Code::eval(Context * context) const // virtual
{
Value * value=context->accessExecution()->popValue();
Value * result=context->accessExecution()->lastValue();
Value added(result->getType());
added.setValueVectorCopyR2I(*value->getValueVector());
result->appendVector(*added.getValueVector());
value->relax();
}


//--------------------------------------------------------------------------//
// class PlusEq_RV_IV_Code                                                  //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
PlusEq_RV_IV_Code::PlusEq_RV_IV_Code(const SmaNode * node)
 : SmaCode(node)
{}

PlusEq_RV_IV_Code::~PlusEq_RV_IV_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void PlusEq_RV_IV_Code::eval(Context * context) const // virtual
{
Value * value=context->accessExecution()->popValue();
Value * result=context->accessExecution()->lastValue();
Value added(result->getType());
added.setValueVectorCopyI2R(*value->getValueVector());
result->appendVector(*added.getValueVector());
value->relax();
}


//--------------------------------------------------------------------------//
// class PlusEq_I_I_Code                                                    //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
PlusEq_I_I_Code::PlusEq_I_I_Code(const SmaNode * node)
 : SmaCode(node)
{}

PlusEq_I_I_Code::~PlusEq_I_I_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void PlusEq_I_I_Code::eval(Context * context) const // virtual
{
Value * value=context->accessExecution()->popValue();
Value * result=context->accessExecution()->lastValue();
result->setInteger(result->getInteger()+value->getInteger());
value->relax();
}


//--------------------------------------------------------------------------//
// class PlusEq_I_R_Code                                                    //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
PlusEq_I_R_Code::PlusEq_I_R_Code(const SmaNode * node)
 : SmaCode(node)
{}

PlusEq_I_R_Code::~PlusEq_I_R_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void PlusEq_I_R_Code::eval(Context * context) const // virtual
{
Value * value=context->accessExecution()->popValue();
Value * result=context->accessExecution()->lastValue();
result->setInteger(result->getInteger()+(long)value->getReal());
value->relax();
}


//--------------------------------------------------------------------------//
// class PlusEq_R_R_Code                                                    //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
PlusEq_R_R_Code::PlusEq_R_R_Code(const SmaNode * node)
 : SmaCode(node)
{}

PlusEq_R_R_Code::~PlusEq_R_R_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void PlusEq_R_R_Code::eval(Context * context) const // virtual
{
Value * value=context->accessExecution()->popValue();
Value * result=context->accessExecution()->lastValue();
result->setReal(result->getReal()+value->getReal());
value->relax();
}


//--------------------------------------------------------------------------//
// class PlusEq_R_I_Code                                                    //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
PlusEq_R_I_Code::PlusEq_R_I_Code(const SmaNode * node)
 : SmaCode(node)
{}

PlusEq_R_I_Code::~PlusEq_R_I_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void PlusEq_R_I_Code::eval(Context * context) const // virtual
{
Value * value=context->accessExecution()->popValue();
Value * result=context->accessExecution()->lastValue();
result->setReal(result->getReal()+(double)value->getInteger());
value->relax();
}


//--------------------------------------------------------------------------//
// class PlusEq_S_S_Code                                                    //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
PlusEq_S_S_Code::PlusEq_S_S_Code(const SmaNode * node)
 : SmaCode(node)
{}

PlusEq_S_S_Code::~PlusEq_S_S_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void PlusEq_S_S_Code::eval(Context * context) const // virtual
{
Value * value=context->accessExecution()->popValue();
Value * result=context->accessExecution()->lastValue();
result->setStrings(result->getStrings()+value->getStrings());
value->relax();
}


//--------------------------------------------------------------------------//
// class MinusEq_I_I_Code                                                   //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
MinusEq_I_I_Code::MinusEq_I_I_Code(const SmaNode * node)
 : SmaCode(node)
{}

MinusEq_I_I_Code::~MinusEq_I_I_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void MinusEq_I_I_Code::eval(Context * context) const // virtual
{
Value * value=context->accessExecution()->popValue();
Value * result=context->accessExecution()->lastValue();
result->setInteger(result->getInteger()-value->getInteger());
value->relax();
}


//--------------------------------------------------------------------------//
// class MinusEq_I_R_Code                                                   //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
MinusEq_I_R_Code::MinusEq_I_R_Code(const SmaNode * node)
 : SmaCode(node)
{}

MinusEq_I_R_Code::~MinusEq_I_R_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void MinusEq_I_R_Code::eval(Context * context) const // virtual
{
Value * value=context->accessExecution()->popValue();
Value * result=context->accessExecution()->lastValue();
result->setInteger(result->getInteger()-(long)value->getReal());
value->relax();
}


//--------------------------------------------------------------------------//
// class MinusEq_R_R_Code                                                   //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
MinusEq_R_R_Code::MinusEq_R_R_Code(const SmaNode * node)
 : SmaCode(node)
{}

MinusEq_R_R_Code::~MinusEq_R_R_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void MinusEq_R_R_Code::eval(Context * context) const // virtual
{
Value * value=context->accessExecution()->popValue();
Value * result=context->accessExecution()->lastValue();
result->setReal(result->getReal()-value->getReal());
value->relax();
}


//--------------------------------------------------------------------------//
// class MinusEq_R_I_Code                                                   //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
MinusEq_R_I_Code::MinusEq_R_I_Code(const SmaNode * node)
 : SmaCode(node)
{}

MinusEq_R_I_Code::~MinusEq_R_I_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void MinusEq_R_I_Code::eval(Context * context) const // virtual
{
Value * value=context->accessExecution()->popValue();
Value * result=context->accessExecution()->lastValue();
result->setReal(result->getReal()-(double)value->getInteger());
value->relax();
}


//--------------------------------------------------------------------------//
// class MultEq_I_I_Code                                                    //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
MultEq_I_I_Code::MultEq_I_I_Code(const SmaNode * node)
 : SmaCode(node)
{}

MultEq_I_I_Code::~MultEq_I_I_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void MultEq_I_I_Code::eval(Context * context) const // virtual
{
Value * value=context->accessExecution()->popValue();
Value * result=context->accessExecution()->lastValue();
result->setInteger(result->getInteger()*value->getInteger());
value->relax();
}


//--------------------------------------------------------------------------//
// class MultEq_I_R_Code                                                    //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
MultEq_I_R_Code::MultEq_I_R_Code(const SmaNode * node)
 : SmaCode(node)
{}

MultEq_I_R_Code::~MultEq_I_R_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void MultEq_I_R_Code::eval(Context * context) const // virtual
{
Value * value=context->accessExecution()->popValue();
Value * result=context->accessExecution()->lastValue();
result->setInteger(result->getInteger()*(long)value->getReal());
value->relax();
}


//--------------------------------------------------------------------------//
// class MultEq_R_R_Code                                                    //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
MultEq_R_R_Code::MultEq_R_R_Code(const SmaNode * node)
 : SmaCode(node)
{}

MultEq_R_R_Code::~MultEq_R_R_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void MultEq_R_R_Code::eval(Context * context) const // virtual
{
Value * value=context->accessExecution()->popValue();
Value * result=context->accessExecution()->lastValue();
result->setReal(result->getReal()*value->getReal());
value->relax();
}


//--------------------------------------------------------------------------//
// class MultEq_R_I_Code                                                    //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
MultEq_R_I_Code::MultEq_R_I_Code(const SmaNode * node)
 : SmaCode(node)
{}

MultEq_R_I_Code::~MultEq_R_I_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void MultEq_R_I_Code::eval(Context * context) const // virtual
{
Value * value=context->accessExecution()->popValue();
Value * result=context->accessExecution()->lastValue();
result->setReal(result->getReal()*(double)value->getInteger());
value->relax();
}


//--------------------------------------------------------------------------//
// class DivEq_I_I_Code                                                     //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
DivEq_I_I_Code::DivEq_I_I_Code(const SmaNode * node)
 : SmaCode(node)
{}

DivEq_I_I_Code::~DivEq_I_I_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void DivEq_I_I_Code::eval(Context * context) const // virtual
{
Value * value=context->accessExecution()->popValue();
Value * result=context->accessExecution()->lastValue();
long denominator=value->getInteger();
value->relax();
if(denominator) result->setInteger(result->getInteger()/denominator);
else
  {
  Strings message="Can't divide by zero";
  manageError(new Error(Error::RUNTIME,message));
  }
}


//--------------------------------------------------------------------------//
// class DivEq_I_R_Code                                                     //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
DivEq_I_R_Code::DivEq_I_R_Code(const SmaNode * node)
 : SmaCode(node)
{}

DivEq_I_R_Code::~DivEq_I_R_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void DivEq_I_R_Code::eval(Context * context) const // virtual
{
Value * value=context->accessExecution()->popValue();
Value * result=context->accessExecution()->lastValue();
long denominator=(long)value->getReal();
value->relax();
if(denominator) result->setInteger(result->getInteger()/denominator);
else
  {
  Strings message="Can't divide by zero";
  manageError(new Error(Error::RUNTIME,message));
  }
}


//--------------------------------------------------------------------------//
// class DivEq_R_R_Code                                                     //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
DivEq_R_R_Code::DivEq_R_R_Code(const SmaNode * node)
 : SmaCode(node)
{}

DivEq_R_R_Code::~DivEq_R_R_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void DivEq_R_R_Code::eval(Context * context) const // virtual
{
Value * value=context->accessExecution()->popValue();
Value * result=context->accessExecution()->lastValue();
double denominator=value->getReal();
value->relax();
if(denominator) result->setReal(result->getReal()/denominator);
else
  {
  Strings message="Can't divide by zero";
  manageError(new Error(Error::RUNTIME,message));
  }
}


//--------------------------------------------------------------------------//
// class DivEq_R_I_Code                                                     //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
DivEq_R_I_Code::DivEq_R_I_Code(const SmaNode * node)
 : SmaCode(node)
{}

DivEq_R_I_Code::~DivEq_R_I_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void DivEq_R_I_Code::eval(Context * context) const // virtual
{
Value * value=context->accessExecution()->popValue();
Value * result=context->accessExecution()->lastValue();
double denominator=(double)value->getInteger();
value->relax();
if(denominator) result->setReal(result->getReal()/denominator);
else
  {
  Strings message="Can't divide by zero";
  manageError(new Error(Error::RUNTIME,message));
  }
}


//--------------------------------------------------------------------------//
// class ModEq_I_I_Code                                                     //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
ModEq_I_I_Code::ModEq_I_I_Code(const SmaNode * node)
 : SmaCode(node)
{}

ModEq_I_I_Code::~ModEq_I_I_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void ModEq_I_I_Code::eval(Context * context) const // virtual
{
Value * value=context->accessExecution()->popValue();
Value * result=context->accessExecution()->lastValue();
long denominator=value->getInteger();
value->relax();
if(denominator) result->setInteger(result->getInteger()%denominator);
else
  {
  Strings message="Can't divide by zero";
  manageError(new Error(Error::RUNTIME,message));
  }
}


//--------------------------------------------------------------------------//
// class ModEq_I_R_Code                                                     //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
ModEq_I_R_Code::ModEq_I_R_Code(const SmaNode * node)
 : SmaCode(node)
{}

ModEq_I_R_Code::~ModEq_I_R_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void ModEq_I_R_Code::eval(Context * context) const // virtual
{
Value * value=context->accessExecution()->popValue();
Value * result=context->accessExecution()->lastValue();
long denominator=(long)value->getReal();
value->relax();
if(denominator) result->setInteger(result->getInteger()%denominator);
else
  {
  Strings message="Can't divide by zero";
  manageError(new Error(Error::RUNTIME,message));
  }
}


//--------------------------------------------------------------------------//
// class ModEq_R_R_Code                                                     //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
ModEq_R_R_Code::ModEq_R_R_Code(const SmaNode * node)
 : SmaCode(node)
{}

ModEq_R_R_Code::~ModEq_R_R_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void ModEq_R_R_Code::eval(Context * context) const // virtual
{
Value * value=context->accessExecution()->popValue();
Value * result=context->accessExecution()->lastValue();
double denominator=value->getReal();
value->relax();
if(denominator) result->setReal(::fmod(result->getReal(),denominator));
else
  {
  Strings message="Can't divide by zero";
  manageError(new Error(Error::RUNTIME,message));
  }
}


//--------------------------------------------------------------------------//
// class ModEq_R_I_Code                                                     //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
ModEq_R_I_Code::ModEq_R_I_Code(const SmaNode * node)
 : SmaCode(node)
{}

ModEq_R_I_Code::~ModEq_R_I_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void ModEq_R_I_Code::eval(Context * context) const // virtual
{
Value * value=context->accessExecution()->popValue();
Value * result=context->accessExecution()->lastValue();
double denominator=(double)value->getInteger();
value->relax();
if(denominator) result->setReal(::fmod(result->getReal(),denominator));
else
  {
  Strings message="Can't divide by zero";
  manageError(new Error(Error::RUNTIME,message));
  }
}


//--------------------------------------------------------------------------//
// class BitAndEqCode                                                       //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
BitAndEqCode::BitAndEqCode(const SmaNode * node)
 : SmaCode(node)
{}

BitAndEqCode::~BitAndEqCode(void)
{}

//-------- Evaluation ------------------------------------------------------//
void BitAndEqCode::eval(Context * context) const // virtual
{
Value * value=context->accessExecution()->popValue();
Value * result=context->accessExecution()->lastValue();
result->setInteger(result->getInteger()&value->getInteger());
value->relax();
}


//--------------------------------------------------------------------------//
// class BitOrEqCode                                                        //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
BitOrEqCode::BitOrEqCode(const SmaNode * node)
 : SmaCode(node)
{}

BitOrEqCode::~BitOrEqCode(void)
{}

//-------- Evaluation ------------------------------------------------------//
void BitOrEqCode::eval(Context * context) const // virtual
{
Value * value=context->accessExecution()->popValue();
Value * result=context->accessExecution()->lastValue();
result->setInteger(result->getInteger()|value->getInteger());
value->relax();
}


//--------------------------------------------------------------------------//
// class BitXorEqCode                                                       //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
BitXorEqCode::BitXorEqCode(const SmaNode * node)
 : SmaCode(node)
{}

BitXorEqCode::~BitXorEqCode(void)
{}

//-------- Evaluation ------------------------------------------------------//
void BitXorEqCode::eval(Context * context) const // virtual
{
Value * value=context->accessExecution()->popValue();
Value * result=context->accessExecution()->lastValue();
result->setInteger(result->getInteger()^value->getInteger());
value->relax();
}


//--------------------------------------------------------------------------//
// class LShiftEqCode                                                       //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
LShiftEqCode::LShiftEqCode(const SmaNode * node)
 : SmaCode(node)
{}

LShiftEqCode::~LShiftEqCode(void)
{}

//-------- Evaluation ------------------------------------------------------//
void LShiftEqCode::eval(Context * context) const // virtual
{
Value * value=context->accessExecution()->popValue();
Value * result=context->accessExecution()->lastValue();
result->setInteger(result->getInteger()<<value->getInteger());
value->relax();
}


//--------------------------------------------------------------------------//
// class RShiftEqCode                                                       //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
RShiftEqCode::RShiftEqCode(const SmaNode * node)
 : SmaCode(node)
{}

RShiftEqCode::~RShiftEqCode(void)
{}

//-------- Evaluation ------------------------------------------------------//
void RShiftEqCode::eval(Context * context) const // virtual
{
Value * value=context->accessExecution()->popValue();
Value * result=context->accessExecution()->lastValue();
result->setInteger(result->getInteger()>>value->getInteger());
value->relax();
}

