// @(#) ENIB/LI2 (c) - oRis - Version 2.3
//--------------------------------------------------------------------------//
// file        : calculCd.cpp                                               //
// description : source file for the ENIB/LI2 Calcul codes                  //
// author(s)   : F. Harrouet                                                //
// copyright   : Laboratoire d'Informatique Industrielle (LI2)              //
//               Ecole Nationale d'Ingenieurs de Brest (ENIB)               //
//               CP 30815 - 29608 Brest Cedex - FRANCE                      //
//               Tel: (33) 02.98.05.66.31 , Fax: (33) 02.98.05.66.29        //
//               e-mail: li2@enib.fr                                        //
//--------------------------------------------------------------------------//
// Copyright (c) 1996-2000                                                  //
// Laboratoire d'Informatique Industrielle (LI2)                            //
// Ecole Nationale d'Ingenieurs de Brest (ENIB)                             //
//                                                                          //
// Permission to use, copy, modify, distribute and sell this software       //
// and its documentation for any purpose is hereby granted without fee,     //
// provided that the above copyright notice appear in all copies and        //
// that both that copyright notice and this permission notice appear        //
// in supporting documentation.                                             //
// ENIB/LI2 makes no representations about the suitability of this          //
// software for any purpose.                                                //
// It is provided "as is" without express or implied warranty.              //
//--------------------------------------------------------------------------//

#include "oRis/Codes/calculCd.h"
#include "oRis/Main/context.h"
#include "oRis/Main/simul.h"
#include "oRis/Main/exec.h"
#include "oRis/Main/tmpVal.h"
#include "oRis/Main/errors.h"
#include "oRis/Nodes/smaNd.h"

#include <math.h>

//--------------------------------------------------------------------------//
// class Decr_I_Code                                                        //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
Decr_I_Code::Decr_I_Code(const SmaNode * node)
 : SmaCode(node)
{}

Decr_I_Code::~Decr_I_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void Decr_I_Code::eval(Context * context) const // virtual
{
Value * value=context->accessExecution()->popValue();
value->setInteger(value->getInteger()-1);
value->relax();
}


//--------------------------------------------------------------------------//
// class Decr_R_Code                                                        //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
Decr_R_Code::Decr_R_Code(const SmaNode * node)
 : SmaCode(node)
{}

Decr_R_Code::~Decr_R_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void Decr_R_Code::eval(Context * context) const // virtual
{
Value * value=context->accessExecution()->popValue();
value->setReal(value->getReal()-1.0);
value->relax();
}


//--------------------------------------------------------------------------//
// class PreDecr_I_Code                                                     //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
PreDecr_I_Code::PreDecr_I_Code(const SmaNode * node)
 : SmaCode(node)
{}

PreDecr_I_Code::~PreDecr_I_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void PreDecr_I_Code::eval(Context * context) const // virtual
{
Value * value=context->accessExecution()->lastValue();
value->setInteger(value->getInteger()-1);
}


//--------------------------------------------------------------------------//
// class PreDecr_R_Code                                                     //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
PreDecr_R_Code::PreDecr_R_Code(const SmaNode * node)
 : SmaCode(node)
{}

PreDecr_R_Code::~PreDecr_R_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void PreDecr_R_Code::eval(Context * context) const // virtual
{
Value * value=context->accessExecution()->lastValue();
value->setReal(value->getReal()-1.0);
}


//--------------------------------------------------------------------------//
// class PostDecr_I_Code                                                    //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
PostDecr_I_Code::PostDecr_I_Code(const SmaNode * node)
 : SmaCode(node)
{}

PostDecr_I_Code::~PostDecr_I_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void PostDecr_I_Code::eval(Context * context) const // virtual
{
Value * value=context->accessExecution()->popValue();
Value * result=TmpValue::getTmpValue(value->getType());
result->setInteger(value->getInteger());
context->accessExecution()->pushValue(result);
value->setInteger(value->getInteger()-1);
value->relax();
}


//--------------------------------------------------------------------------//
// class PostDecr_R_Code                                                    //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
PostDecr_R_Code::PostDecr_R_Code(const SmaNode * node)
 : SmaCode(node)
{}

PostDecr_R_Code::~PostDecr_R_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void PostDecr_R_Code::eval(Context * context) const // virtual
{
Value * value=context->accessExecution()->popValue();
Value * result=TmpValue::getTmpValue(value->getType());
result->setReal(value->getReal());
context->accessExecution()->pushValue(result);
value->setReal(value->getReal()-1.0);
value->relax();
}


//--------------------------------------------------------------------------//
// class Incr_I_Code                                                        //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
Incr_I_Code::Incr_I_Code(const SmaNode * node)
 : SmaCode(node)
{}

Incr_I_Code::~Incr_I_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void Incr_I_Code::eval(Context * context) const // virtual
{
Value * value=context->accessExecution()->popValue();
value->setInteger(value->getInteger()+1);
value->relax();
}


//--------------------------------------------------------------------------//
// class Incr_R_Code                                                        //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
Incr_R_Code::Incr_R_Code(const SmaNode * node)
 : SmaCode(node)
{}

Incr_R_Code::~Incr_R_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void Incr_R_Code::eval(Context * context) const // virtual
{
Value * value=context->accessExecution()->popValue();
value->setReal(value->getReal()+1.0);
value->relax();
}


//--------------------------------------------------------------------------//
// class PreIncr_I_Code                                                     //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
PreIncr_I_Code::PreIncr_I_Code(const SmaNode * node)
 : SmaCode(node)
{}

PreIncr_I_Code::~PreIncr_I_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void PreIncr_I_Code::eval(Context * context) const // virtual
{
Value * value=context->accessExecution()->lastValue();
value->setInteger(value->getInteger()+1);
}


//--------------------------------------------------------------------------//
// class PreIncr_R_Code                                                     //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
PreIncr_R_Code::PreIncr_R_Code(const SmaNode * node)
 : SmaCode(node)
{}

PreIncr_R_Code::~PreIncr_R_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void PreIncr_R_Code::eval(Context * context) const // virtual
{
Value * value=context->accessExecution()->lastValue();
value->setReal(value->getReal()+1.0);
}


//--------------------------------------------------------------------------//
// class PostIncr_I_Code                                                    //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
PostIncr_I_Code::PostIncr_I_Code(const SmaNode * node)
 : SmaCode(node)
{}

PostIncr_I_Code::~PostIncr_I_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void PostIncr_I_Code::eval(Context * context) const // virtual
{
Value * value=context->accessExecution()->popValue();
Value * result=TmpValue::getTmpValue(value->getType());
result->setInteger(value->getInteger());
context->accessExecution()->pushValue(result);
value->setInteger(value->getInteger()+1);
value->relax();
}


//--------------------------------------------------------------------------//
// class PostIncr_R_Code                                                    //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
PostIncr_R_Code::PostIncr_R_Code(const SmaNode * node)
 : SmaCode(node)
{}

PostIncr_R_Code::~PostIncr_R_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void PostIncr_R_Code::eval(Context * context) const // virtual
{
Value * value=context->accessExecution()->popValue();
Value * result=TmpValue::getTmpValue(value->getType());
result->setReal(value->getReal());
context->accessExecution()->pushValue(result);
value->setReal(value->getReal()+1.0);
value->relax();
}


//--------------------------------------------------------------------------//
// class Plus_V_V_Code                                                      //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
Plus_V_V_Code::Plus_V_V_Code(const SmaNode * node)
 : SmaCode(node)
{}

Plus_V_V_Code::~Plus_V_V_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void Plus_V_V_Code::eval(Context * context) const // virtual
{
Value * value=context->accessExecution()->popValue();
Value * result=TmpValue::getTmpValue(value->getType());
result->setValueVectorCopy(*value);
value->relax();
value=context->accessExecution()->popValue();
result->appendVector(*value->getValueVector());
value->relax();
context->accessExecution()->pushValue(result);
}


//--------------------------------------------------------------------------//
// class Plus_IV_RV_Code                                                    //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
Plus_IV_RV_Code::Plus_IV_RV_Code(const SmaNode * node)
 : SmaCode(node)
{}

Plus_IV_RV_Code::~Plus_IV_RV_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void Plus_IV_RV_Code::eval(Context * context) const // virtual
{
Value * value1=context->accessExecution()->popValue();
Value * value2=context->accessExecution()->popValue();
Value * result=TmpValue::getTmpValue(value2->getType()); // REAL
result->setValueVectorCopyI2R(*value1->getValueVector());
value1->relax();
result->appendVector(*value2->getValueVector());
value2->relax();
context->accessExecution()->pushValue(result);
}


//--------------------------------------------------------------------------//
// class Plus_RV_IV_Code                                                    //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
Plus_RV_IV_Code::Plus_RV_IV_Code(const SmaNode * node)
 : SmaCode(node)
{}

Plus_RV_IV_Code::~Plus_RV_IV_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void Plus_RV_IV_Code::eval(Context * context) const // virtual
{
Value * value1=context->accessExecution()->popValue();
Value * value2=context->accessExecution()->popValue();
Value * result=TmpValue::getTmpValue(value1->getType()); // REAL
result->setValueVectorCopy(*value1);
value1->relax();
Value added(result->getType());
added.setValueVectorCopyI2R(*value2->getValueVector());
value2->relax();
result->appendVector(*added.getValueVector());
context->accessExecution()->pushValue(result);
}


//--------------------------------------------------------------------------//
// class Plus_I_I_Code                                                      //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
Plus_I_I_Code::Plus_I_I_Code(const SmaNode * node)
 : SmaCode(node)
{}

Plus_I_I_Code::~Plus_I_I_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void Plus_I_I_Code::eval(Context * context) const // virtual
{
Value * value=context->accessExecution()->popValue();
long x=value->getInteger();
value->relax();
value=context->accessExecution()->popValue();
long y=value->getInteger();
value->relax();
value=TmpValue::getTmpValue(_node->getType());
value->setInteger(x+y);
context->accessExecution()->pushValue(value);
}


//--------------------------------------------------------------------------//
// class Plus_I_R_Code                                                      //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
Plus_I_R_Code::Plus_I_R_Code(const SmaNode * node)
 : SmaCode(node)
{}

Plus_I_R_Code::~Plus_I_R_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void Plus_I_R_Code::eval(Context * context) const // virtual
{
Value * value=context->accessExecution()->popValue();
double x=(double)value->getInteger();
value->relax();
value=context->accessExecution()->popValue();
double y=value->getReal();
value->relax();
value=TmpValue::getTmpValue(_node->getType());
value->setReal(x+y);
context->accessExecution()->pushValue(value);
}


//--------------------------------------------------------------------------//
// class Plus_R_R_Code                                                      //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
Plus_R_R_Code::Plus_R_R_Code(const SmaNode * node)
 : SmaCode(node)
{}

Plus_R_R_Code::~Plus_R_R_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void Plus_R_R_Code::eval(Context * context) const // virtual
{
Value * value=context->accessExecution()->popValue();
double x=value->getReal();
value->relax();
value=context->accessExecution()->popValue();
double y=value->getReal();
value->relax();
value=TmpValue::getTmpValue(_node->getType());
value->setReal(x+y);
context->accessExecution()->pushValue(value);
}


//--------------------------------------------------------------------------//
// class Plus_R_I_Code                                                      //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
Plus_R_I_Code::Plus_R_I_Code(const SmaNode * node)
 : SmaCode(node)
{}

Plus_R_I_Code::~Plus_R_I_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void Plus_R_I_Code::eval(Context * context) const // virtual
{
Value * value=context->accessExecution()->popValue();
double x=value->getReal();
value->relax();
value=context->accessExecution()->popValue();
double y=(double)value->getInteger();
value->relax();
value=TmpValue::getTmpValue(_node->getType());
value->setReal(x+y);
context->accessExecution()->pushValue(value);
}


//--------------------------------------------------------------------------//
// class Plus_S_S_Code                                                      //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
Plus_S_S_Code::Plus_S_S_Code(const SmaNode * node)
 : SmaCode(node)
{}

Plus_S_S_Code::~Plus_S_S_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void Plus_S_S_Code::eval(Context * context) const // virtual
{
Value * value=context->accessExecution()->popValue();
Strings x=value->getStrings();
value->relax();
value=context->accessExecution()->popValue();
Strings y=value->getStrings();
value->relax();
value=TmpValue::getTmpValue(_node->getType());
value->setStrings(x+y);
context->accessExecution()->pushValue(value);
}


//--------------------------------------------------------------------------//
// class Minus_I_I_Code                                                     //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
Minus_I_I_Code::Minus_I_I_Code(const SmaNode * node)
 : SmaCode(node)
{}

Minus_I_I_Code::~Minus_I_I_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void Minus_I_I_Code::eval(Context * context) const // virtual
{
Value * value=context->accessExecution()->popValue();
long x=value->getInteger();
value->relax();
value=context->accessExecution()->popValue();
long y=value->getInteger();
value->relax();
value=TmpValue::getTmpValue(_node->getType());
value->setInteger(x-y);
context->accessExecution()->pushValue(value);
}


//--------------------------------------------------------------------------//
// class Minus_I_R_Code                                                     //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
Minus_I_R_Code::Minus_I_R_Code(const SmaNode * node)
 : SmaCode(node)
{}

Minus_I_R_Code::~Minus_I_R_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void Minus_I_R_Code::eval(Context * context) const // virtual
{
Value * value=context->accessExecution()->popValue();
double x=(double)value->getInteger();
value->relax();
value=context->accessExecution()->popValue();
double y=value->getReal();
value->relax();
value=TmpValue::getTmpValue(_node->getType());
value->setReal(x-y);
context->accessExecution()->pushValue(value);
}


//--------------------------------------------------------------------------//
// class Minus_R_R_Code                                                     //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
Minus_R_R_Code::Minus_R_R_Code(const SmaNode * node)
 : SmaCode(node)
{}

Minus_R_R_Code::~Minus_R_R_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void Minus_R_R_Code::eval(Context * context) const // virtual
{
Value * value=context->accessExecution()->popValue();
double x=value->getReal();
value->relax();
value=context->accessExecution()->popValue();
double y=value->getReal();
value->relax();
value=TmpValue::getTmpValue(_node->getType());
value->setReal(x-y);
context->accessExecution()->pushValue(value);
}


//--------------------------------------------------------------------------//
// class Minus_R_I_Code                                                     //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
Minus_R_I_Code::Minus_R_I_Code(const SmaNode * node)
 : SmaCode(node)
{}

Minus_R_I_Code::~Minus_R_I_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void Minus_R_I_Code::eval(Context * context) const // virtual
{
Value * value=context->accessExecution()->popValue();
double x=value->getReal();
value->relax();
value=context->accessExecution()->popValue();
double y=(double)value->getInteger();
value->relax();
value=TmpValue::getTmpValue(_node->getType());
value->setReal(x-y);
context->accessExecution()->pushValue(value);
}


//--------------------------------------------------------------------------//
// class Mult_I_I_Code                                                      //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
Mult_I_I_Code::Mult_I_I_Code(const SmaNode * node)
 : SmaCode(node)
{}

Mult_I_I_Code::~Mult_I_I_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void Mult_I_I_Code::eval(Context * context) const // virtual
{
Value * value=context->accessExecution()->popValue();
long x=value->getInteger();
value->relax();
value=context->accessExecution()->popValue();
long y=value->getInteger();
value->relax();
value=TmpValue::getTmpValue(_node->getType());
value->setInteger(x*y);
context->accessExecution()->pushValue(value);
}


//--------------------------------------------------------------------------//
// class Mult_I_R_Code                                                      //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
Mult_I_R_Code::Mult_I_R_Code(const SmaNode * node)
 : SmaCode(node)
{}

Mult_I_R_Code::~Mult_I_R_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void Mult_I_R_Code::eval(Context * context) const // virtual
{
Value * value=context->accessExecution()->popValue();
double x=(double)value->getInteger();
value->relax();
value=context->accessExecution()->popValue();
double y=value->getReal();
value->relax();
value=TmpValue::getTmpValue(_node->getType());
value->setReal(x*y);
context->accessExecution()->pushValue(value);
}


//--------------------------------------------------------------------------//
// class Mult_R_R_Code                                                      //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
Mult_R_R_Code::Mult_R_R_Code(const SmaNode * node)
 : SmaCode(node)
{}

Mult_R_R_Code::~Mult_R_R_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void Mult_R_R_Code::eval(Context * context) const // virtual
{
Value * value=context->accessExecution()->popValue();
double x=value->getReal();
value->relax();
value=context->accessExecution()->popValue();
double y=value->getReal();
value->relax();
value=TmpValue::getTmpValue(_node->getType());
value->setReal(x*y);
context->accessExecution()->pushValue(value);
}


//--------------------------------------------------------------------------//
// class Mult_R_I_Code                                                      //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
Mult_R_I_Code::Mult_R_I_Code(const SmaNode * node)
 : SmaCode(node)
{}

Mult_R_I_Code::~Mult_R_I_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void Mult_R_I_Code::eval(Context * context) const // virtual
{
Value * value=context->accessExecution()->popValue();
double x=value->getReal();
value->relax();
value=context->accessExecution()->popValue();
double y=(double)value->getInteger();
value->relax();
value=TmpValue::getTmpValue(_node->getType());
value->setReal(x*y);
context->accessExecution()->pushValue(value);
}


//--------------------------------------------------------------------------//
// class Div_I_I_Code                                                       //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
Div_I_I_Code::Div_I_I_Code(const SmaNode * node)
 : SmaCode(node)
{}

Div_I_I_Code::~Div_I_I_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void Div_I_I_Code::eval(Context * context) const // virtual
{
Value * value=context->accessExecution()->popValue();
long x=value->getInteger();
value->relax();
value=context->accessExecution()->popValue();
long y=value->getInteger();
value->relax();
value=TmpValue::getTmpValue(_node->getType());
if(y)
  {
  value->setInteger(x/y);
  context->accessExecution()->pushValue(value);
  }
else
  {
  context->accessExecution()->pushValue(value); // not used
  Strings message="Can't divide by zero";
  manageError(new Error(Error::RUNTIME,message));
  }
}


//--------------------------------------------------------------------------//
// class Div_I_R_Code                                                       //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
Div_I_R_Code::Div_I_R_Code(const SmaNode * node)
 : SmaCode(node)
{}

Div_I_R_Code::~Div_I_R_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void Div_I_R_Code::eval(Context * context) const // virtual
{
Value * value=context->accessExecution()->popValue();
double x=(double)value->getInteger();
value->relax();
value=context->accessExecution()->popValue();
double y=value->getReal();
value->relax();
value=TmpValue::getTmpValue(_node->getType());
if(y)
  {
  value->setReal(x/y);
  context->accessExecution()->pushValue(value);
  }
else
  {
  context->accessExecution()->pushValue(value); // not used
  Strings message="Can't divide by zero";
  manageError(new Error(Error::RUNTIME,message));
  }
}


//--------------------------------------------------------------------------//
// class Div_R_R_Code                                                       //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
Div_R_R_Code::Div_R_R_Code(const SmaNode * node)
 : SmaCode(node)
{}

Div_R_R_Code::~Div_R_R_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void Div_R_R_Code::eval(Context * context) const // virtual
{
Value * value=context->accessExecution()->popValue();
double x=value->getReal();
value->relax();
value=context->accessExecution()->popValue();
double y=value->getReal();
value->relax();
value=TmpValue::getTmpValue(_node->getType());
if(y)
  {
  value->setReal(x/y);
  context->accessExecution()->pushValue(value);
  }
else
  {
  context->accessExecution()->pushValue(value); // not used
  Strings message="Can't divide by zero";
  manageError(new Error(Error::RUNTIME,message));
  }
}


//--------------------------------------------------------------------------//
// class Div_R_I_Code                                                       //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
Div_R_I_Code::Div_R_I_Code(const SmaNode * node)
 : SmaCode(node)
{}

Div_R_I_Code::~Div_R_I_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void Div_R_I_Code::eval(Context * context) const // virtual
{
Value * value=context->accessExecution()->popValue();
double x=value->getReal();
value->relax();
value=context->accessExecution()->popValue();
double y=(double)value->getInteger();
value->relax();
value=TmpValue::getTmpValue(_node->getType());
if(y)
  {
  value->setReal(x/y);
  context->accessExecution()->pushValue(value);
  }
else
  {
  context->accessExecution()->pushValue(value); // not used
  Strings message="Can't divide by zero";
  manageError(new Error(Error::RUNTIME,message));
  }
}


//--------------------------------------------------------------------------//
// class Mod_I_I_Code                                                       //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
Mod_I_I_Code::Mod_I_I_Code(const SmaNode * node)
 : SmaCode(node)
{}

Mod_I_I_Code::~Mod_I_I_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void Mod_I_I_Code::eval(Context * context) const // virtual
{
Value * value=context->accessExecution()->popValue();
long x=value->getInteger();
value->relax();
value=context->accessExecution()->popValue();
long y=value->getInteger();
value->relax();
value=TmpValue::getTmpValue(_node->getType());
if(y)
  {
  value->setInteger(x%y);
  context->accessExecution()->pushValue(value);
  }
else
  {
  context->accessExecution()->pushValue(value); // not used
  Strings message="Can't divide by zero";
  manageError(new Error(Error::RUNTIME,message));
  }
}


//--------------------------------------------------------------------------//
// class Mod_I_R_Code                                                       //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
Mod_I_R_Code::Mod_I_R_Code(const SmaNode * node)
 : SmaCode(node)
{}

Mod_I_R_Code::~Mod_I_R_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void Mod_I_R_Code::eval(Context * context) const // virtual
{
Value * value=context->accessExecution()->popValue();
double x=(double)value->getInteger();
value->relax();
value=context->accessExecution()->popValue();
double y=value->getReal();
value->relax();
value=TmpValue::getTmpValue(_node->getType());
if(y)
  {
  value->setReal(::fmod(x,y));
  context->accessExecution()->pushValue(value);
  }
else
  {
  context->accessExecution()->pushValue(value); // not used
  Strings message="Can't divide by zero";
  manageError(new Error(Error::RUNTIME,message));
  }
}


//--------------------------------------------------------------------------//
// class Mod_R_R_Code                                                       //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
Mod_R_R_Code::Mod_R_R_Code(const SmaNode * node)
 : SmaCode(node)
{}

Mod_R_R_Code::~Mod_R_R_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void Mod_R_R_Code::eval(Context * context) const // virtual
{
Value * value=context->accessExecution()->popValue();
double x=value->getReal();
value->relax();
value=context->accessExecution()->popValue();
double y=value->getReal();
value->relax();
value=TmpValue::getTmpValue(_node->getType());
if(y)
  {
  value->setReal(::fmod(x,y));
  context->accessExecution()->pushValue(value);
  }
else
  {
  context->accessExecution()->pushValue(value); // not used
  Strings message="Can't divide by zero";
  manageError(new Error(Error::RUNTIME,message));
  }
}


//--------------------------------------------------------------------------//
// class Mod_R_I_Code                                                       //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
Mod_R_I_Code::Mod_R_I_Code(const SmaNode * node)
 : SmaCode(node)
{}

Mod_R_I_Code::~Mod_R_I_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void Mod_R_I_Code::eval(Context * context) const // virtual
{
Value * value=context->accessExecution()->popValue();
double x=value->getReal();
value->relax();
value=context->accessExecution()->popValue();
double y=(double)value->getInteger();
value->relax();
value=TmpValue::getTmpValue(_node->getType());
if(y)
  {
  value->setReal(::fmod(x,y));
  context->accessExecution()->pushValue(value);
  }
else
  {
  context->accessExecution()->pushValue(value); // not used
  Strings message="Can't divide by zero";
  manageError(new Error(Error::RUNTIME,message));
  }
}


//--------------------------------------------------------------------------//
// class UMinus_I_Code                                                      //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
UMinus_I_Code::UMinus_I_Code(const SmaNode * node)
 : SmaCode(node)
{}

UMinus_I_Code::~UMinus_I_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void UMinus_I_Code::eval(Context * context) const // virtual
{
Value * value=context->accessExecution()->popValue();
long x=value->getInteger();
value->relax();
value=TmpValue::getTmpValue(_node->getType());
value->setInteger(-x);
context->accessExecution()->pushValue(value);
}


//--------------------------------------------------------------------------//
// class UMinus_R_Code                                                      //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
UMinus_R_Code::UMinus_R_Code(const SmaNode * node)
 : SmaCode(node)
{}

UMinus_R_Code::~UMinus_R_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void UMinus_R_Code::eval(Context * context) const // virtual
{
Value * value=context->accessExecution()->popValue();
double x=value->getReal();
value->relax();
value=TmpValue::getTmpValue(_node->getType());
value->setReal(-x);
context->accessExecution()->pushValue(value);
}


//--------------------------------------------------------------------------//
// class BitNotCode                                                         //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
BitNotCode::BitNotCode(const SmaNode * node)
 : SmaCode(node)
{}

BitNotCode::~BitNotCode(void)
{}

//-------- Evaluation ------------------------------------------------------//
void BitNotCode::eval(Context * context) const // virtual
{
Value * value=context->accessExecution()->popValue();
long x=value->getInteger();
value->relax();
value=TmpValue::getTmpValue(_node->getType());
value->setInteger(~x);
context->accessExecution()->pushValue(value);
}


//--------------------------------------------------------------------------//
// class BitAndCode                                                         //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
BitAndCode::BitAndCode(const SmaNode * node)
 : SmaCode(node)
{}

BitAndCode::~BitAndCode(void)
{}

//-------- Evaluation ------------------------------------------------------//
void BitAndCode::eval(Context * context) const // virtual
{
Value * value=context->accessExecution()->popValue();
long x=value->getInteger();
value->relax();
value=context->accessExecution()->popValue();
long y=value->getInteger();
value->relax();
value=TmpValue::getTmpValue(_node->getType());
value->setInteger(x&y);
context->accessExecution()->pushValue(value);
}


//--------------------------------------------------------------------------//
// class BitOrCode                                                          //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
BitOrCode::BitOrCode(const SmaNode * node)
 : SmaCode(node)
{}

BitOrCode::~BitOrCode(void)
{}

//-------- Evaluation ------------------------------------------------------//
void BitOrCode::eval(Context * context) const // virtual
{
Value * value=context->accessExecution()->popValue();
long x=value->getInteger();
value->relax();
value=context->accessExecution()->popValue();
long y=value->getInteger();
value->relax();
value=TmpValue::getTmpValue(_node->getType());
value->setInteger(x|y);
context->accessExecution()->pushValue(value);
}


//--------------------------------------------------------------------------//
// class BitXorCode                                                         //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
BitXorCode::BitXorCode(const SmaNode * node)
 : SmaCode(node)
{}

BitXorCode::~BitXorCode(void)
{}

//-------- Evaluation ------------------------------------------------------//
void BitXorCode::eval(Context * context) const // virtual
{
Value * value=context->accessExecution()->popValue();
long x=value->getInteger();
value->relax();
value=context->accessExecution()->popValue();
long y=value->getInteger();
value->relax();
value=TmpValue::getTmpValue(_node->getType());
value->setInteger(x^y);
context->accessExecution()->pushValue(value);
}


//--------------------------------------------------------------------------//
// class LShiftCode                                                         //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
LShiftCode::LShiftCode(const SmaNode * node)
 : SmaCode(node)
{}

LShiftCode::~LShiftCode(void)
{}

//-------- Evaluation ------------------------------------------------------//
void LShiftCode::eval(Context * context) const // virtual
{
Value * value=context->accessExecution()->popValue();
long x=value->getInteger();
value->relax();
value=context->accessExecution()->popValue();
long y=value->getInteger();
value->relax();
value=TmpValue::getTmpValue(_node->getType());
value->setInteger(x<<y);
context->accessExecution()->pushValue(value);
}


//--------------------------------------------------------------------------//
// class RShiftCode                                                         //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
RShiftCode::RShiftCode(const SmaNode * node)
 : SmaCode(node)
{}

RShiftCode::~RShiftCode(void)
{}

//-------- Evaluation ------------------------------------------------------//
void RShiftCode::eval(Context * context) const // virtual
{
Value * value=context->accessExecution()->popValue();
long x=value->getInteger();
value->relax();
value=context->accessExecution()->popValue();
long y=value->getInteger();
value->relax();
value=TmpValue::getTmpValue(_node->getType());
value->setInteger(x>>y);
context->accessExecution()->pushValue(value);
}

