// @(#) ENIB/LI2 (c) - oRis - Version 2.3
//--------------------------------------------------------------------------//
// file        : comparCd.cpp                                               //
// description : source file for the ENIB/LI2 Comparaison codes             //
// author(s)   : F. Harrouet                                                //
// copyright   : Laboratoire d'Informatique Industrielle (LI2)              //
//               Ecole Nationale d'Ingenieurs de Brest (ENIB)               //
//               CP 30815 - 29608 Brest Cedex - FRANCE                      //
//               Tel: (33) 02.98.05.66.31 , Fax: (33) 02.98.05.66.29        //
//               e-mail: li2@enib.fr                                        //
//--------------------------------------------------------------------------//
// Copyright (c) 1996-2000                                                  //
// Laboratoire d'Informatique Industrielle (LI2)                            //
// Ecole Nationale d'Ingenieurs de Brest (ENIB)                             //
//                                                                          //
// Permission to use, copy, modify, distribute and sell this software       //
// and its documentation for any purpose is hereby granted without fee,     //
// provided that the above copyright notice appear in all copies and        //
// that both that copyright notice and this permission notice appear        //
// in supporting documentation.                                             //
// ENIB/LI2 makes no representations about the suitability of this          //
// software for any purpose.                                                //
// It is provided "as is" without express or implied warranty.              //
//--------------------------------------------------------------------------//

#include "oRis/Codes/comparCd.h"
#include "oRis/Main/context.h"
#include "oRis/Main/exec.h"
#include "oRis/Main/tmpVal.h"
#include "oRis/Nodes/smaNd.h"

//--------------------------------------------------------------------------//
// class Equal_Deep_Code                                                    //
//--------------------------------------------------------------------------//
 
//-------- Allocators ------------------------------------------------------//
Equal_Deep_Code::Equal_Deep_Code(const SmaNode * node)
 : SmaCode(node)
{}
 
Equal_Deep_Code::~Equal_Deep_Code(void)
{}
 
//-------- Evaluation ------------------------------------------------------//
void Equal_Deep_Code::eval(Context * context) const // virtual
{
Value * val1=context->accessExecution()->popValue();
Value * val2=context->accessExecution()->popValue();
Value * result=TmpValue::getTmpValue(_node->getType());
result->setInteger(val1->deepEquals(*val2));
context->accessExecution()->pushValue(result);
val1->relax();
val2->relax();
}


//--------------------------------------------------------------------------//
// class Equal_V_V_Code                                                     //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
Equal_V_V_Code::Equal_V_V_Code(const SmaNode * node)
 : SmaCode(node)
{}

Equal_V_V_Code::~Equal_V_V_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void Equal_V_V_Code::eval(Context * context) const // virtual
{
Value * val1=context->accessExecution()->popValue();
Value * val2=context->accessExecution()->popValue();
Value * result=TmpValue::getTmpValue(_node->getType());
result->setInteger(*val1==*val2);
context->accessExecution()->pushValue(result);
val1->relax();
val2->relax();
}


//--------------------------------------------------------------------------//
// class Equal_I_I_Code                                                     //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
Equal_I_I_Code::Equal_I_I_Code(const SmaNode * node)
 : SmaCode(node)
{}

Equal_I_I_Code::~Equal_I_I_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void Equal_I_I_Code::eval(Context * context) const // virtual
{
Value * val1=context->accessExecution()->popValue();
Value * val2=context->accessExecution()->popValue();
Value * result=TmpValue::getTmpValue(_node->getType());
result->setInteger(val1->getInteger()==val2->getInteger());
context->accessExecution()->pushValue(result);
val1->relax();
val2->relax();
}


//--------------------------------------------------------------------------//
// class Equal_I_R_Code                                                     //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
Equal_I_R_Code::Equal_I_R_Code(const SmaNode * node)
 : SmaCode(node)
{}

Equal_I_R_Code::~Equal_I_R_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void Equal_I_R_Code::eval(Context * context) const // virtual
{
Value * val1=context->accessExecution()->popValue();
Value * val2=context->accessExecution()->popValue();
Value * result=TmpValue::getTmpValue(_node->getType());
result->setInteger((double)(val1->getInteger())==val2->getReal());
context->accessExecution()->pushValue(result);
val1->relax();
val2->relax();
}


//--------------------------------------------------------------------------//
// class Equal_R_R_Code                                                     //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
Equal_R_R_Code::Equal_R_R_Code(const SmaNode * node)
 : SmaCode(node)
{}

Equal_R_R_Code::~Equal_R_R_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void Equal_R_R_Code::eval(Context * context) const // virtual
{
Value * val1=context->accessExecution()->popValue();
Value * val2=context->accessExecution()->popValue();
Value * result=TmpValue::getTmpValue(_node->getType());
result->setInteger(val1->getReal()==val2->getReal());
context->accessExecution()->pushValue(result);
val1->relax();
val2->relax();
}


//--------------------------------------------------------------------------//
// class Equal_R_I_Code                                                     //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
Equal_R_I_Code::Equal_R_I_Code(const SmaNode * node)
 : SmaCode(node)
{}

Equal_R_I_Code::~Equal_R_I_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void Equal_R_I_Code::eval(Context * context) const // virtual
{
Value * val1=context->accessExecution()->popValue();
Value * val2=context->accessExecution()->popValue();
Value * result=TmpValue::getTmpValue(_node->getType());
result->setInteger(val1->getReal()==(double)(val2->getInteger()));
context->accessExecution()->pushValue(result);
val1->relax();
val2->relax();
}


//--------------------------------------------------------------------------//
// class Equal_S_S_Code                                                     //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
Equal_S_S_Code::Equal_S_S_Code(const SmaNode * node)
 : SmaCode(node)
{}

Equal_S_S_Code::~Equal_S_S_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void Equal_S_S_Code::eval(Context * context) const // virtual
{
Value * val1=context->accessExecution()->popValue();
Value * val2=context->accessExecution()->popValue();
Value * result=TmpValue::getTmpValue(_node->getType());
result->setInteger(val1->getStrings()==val2->getStrings());
context->accessExecution()->pushValue(result);
val1->relax();
val2->relax();
}


//--------------------------------------------------------------------------//
// class Equal_A_A_Code                                                     //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
Equal_A_A_Code::Equal_A_A_Code(const SmaNode * node)
 : SmaCode(node)
{}

Equal_A_A_Code::~Equal_A_A_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void Equal_A_A_Code::eval(Context * context) const // virtual
{
Value * val1=context->accessExecution()->popValue();
Value * val2=context->accessExecution()->popValue();
Value * result=TmpValue::getTmpValue(_node->getType());
result->setInteger(val1->getAgent()==val2->getAgent());
context->accessExecution()->pushValue(result);
val1->relax();
val2->relax();
}


//--------------------------------------------------------------------------//
// class Diff_Deep_Code                                                     //
//--------------------------------------------------------------------------//
 
//-------- Allocators ------------------------------------------------------//
Diff_Deep_Code::Diff_Deep_Code(const SmaNode * node)
 : SmaCode(node)
{}
 
Diff_Deep_Code::~Diff_Deep_Code(void)
{}
 
//-------- Evaluation ------------------------------------------------------//
void Diff_Deep_Code::eval(Context * context) const // virtual
{
Value * val1=context->accessExecution()->popValue();
Value * val2=context->accessExecution()->popValue();
Value * result=TmpValue::getTmpValue(_node->getType());
result->setInteger(!(val1->deepEquals(*val2)));
context->accessExecution()->pushValue(result);
val1->relax();
val2->relax();
}


//--------------------------------------------------------------------------//
// class Diff_V_V_Code                                                      //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
Diff_V_V_Code::Diff_V_V_Code(const SmaNode * node)
 : SmaCode(node)
{}

Diff_V_V_Code::~Diff_V_V_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void Diff_V_V_Code::eval(Context * context) const // virtual
{
Value * val1=context->accessExecution()->popValue();
Value * val2=context->accessExecution()->popValue();
Value * result=TmpValue::getTmpValue(_node->getType());
result->setInteger(*val1!=*val2);
context->accessExecution()->pushValue(result);
val1->relax();
val2->relax();
}


//--------------------------------------------------------------------------//
// class Diff_I_I_Code                                                      //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
Diff_I_I_Code::Diff_I_I_Code(const SmaNode * node)
 : SmaCode(node)
{}

Diff_I_I_Code::~Diff_I_I_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void Diff_I_I_Code::eval(Context * context) const // virtual
{
Value * val1=context->accessExecution()->popValue();
Value * val2=context->accessExecution()->popValue();
Value * result=TmpValue::getTmpValue(_node->getType());
result->setInteger(val1->getInteger()!=val2->getInteger());
context->accessExecution()->pushValue(result);
val1->relax();
val2->relax();
}


//--------------------------------------------------------------------------//
// class Diff_I_R_Code                                                      //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
Diff_I_R_Code::Diff_I_R_Code(const SmaNode * node)
 : SmaCode(node)
{}

Diff_I_R_Code::~Diff_I_R_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void Diff_I_R_Code::eval(Context * context) const // virtual
{
Value * val1=context->accessExecution()->popValue();
Value * val2=context->accessExecution()->popValue();
Value * result=TmpValue::getTmpValue(_node->getType());
result->setInteger((double)(val1->getInteger())!=val2->getReal());
context->accessExecution()->pushValue(result);
val1->relax();
val2->relax();
}


//--------------------------------------------------------------------------//
// class Diff_R_R_Code                                                      //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
Diff_R_R_Code::Diff_R_R_Code(const SmaNode * node)
 : SmaCode(node)
{}

Diff_R_R_Code::~Diff_R_R_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void Diff_R_R_Code::eval(Context * context) const // virtual
{
Value * val1=context->accessExecution()->popValue();
Value * val2=context->accessExecution()->popValue();
Value * result=TmpValue::getTmpValue(_node->getType());
result->setInteger(val1->getReal()!=val2->getReal());
context->accessExecution()->pushValue(result);
val1->relax();
val2->relax();
}


//--------------------------------------------------------------------------//
// class Diff_R_I_Code                                                      //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
Diff_R_I_Code::Diff_R_I_Code(const SmaNode * node)
 : SmaCode(node)
{}

Diff_R_I_Code::~Diff_R_I_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void Diff_R_I_Code::eval(Context * context) const // virtual
{
Value * val1=context->accessExecution()->popValue();
Value * val2=context->accessExecution()->popValue();
Value * result=TmpValue::getTmpValue(_node->getType());
result->setInteger(val1->getReal()!=(double)(val2->getInteger()));
context->accessExecution()->pushValue(result);
val1->relax();
val2->relax();
}


//--------------------------------------------------------------------------//
// class Diff_S_S_Code                                                      //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
Diff_S_S_Code::Diff_S_S_Code(const SmaNode * node)
 : SmaCode(node)
{}

Diff_S_S_Code::~Diff_S_S_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void Diff_S_S_Code::eval(Context * context) const // virtual
{
Value * val1=context->accessExecution()->popValue();
Value * val2=context->accessExecution()->popValue();
Value * result=TmpValue::getTmpValue(_node->getType());
result->setInteger(val1->getStrings()!=val2->getStrings());
context->accessExecution()->pushValue(result);
val1->relax();
val2->relax();
}


//--------------------------------------------------------------------------//
// class Diff_A_A_Code                                                      //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
Diff_A_A_Code::Diff_A_A_Code(const SmaNode * node)
 : SmaCode(node)
{}

Diff_A_A_Code::~Diff_A_A_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void Diff_A_A_Code::eval(Context * context) const // virtual
{
Value * val1=context->accessExecution()->popValue();
Value * val2=context->accessExecution()->popValue();
Value * result=TmpValue::getTmpValue(_node->getType());
result->setInteger(val1->getAgent()!=val2->getAgent());
context->accessExecution()->pushValue(result);
val1->relax();
val2->relax();
}


//--------------------------------------------------------------------------//
// class Inf_Deep_Code                                                      //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
Inf_Deep_Code::Inf_Deep_Code(const SmaNode * node)
 : SmaCode(node)
{}

Inf_Deep_Code::~Inf_Deep_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void Inf_Deep_Code::eval(Context * context) const // virtual
{
Value * val1=context->accessExecution()->popValue();
Value * val2=context->accessExecution()->popValue();
Value * result=TmpValue::getTmpValue(_node->getType());
result->setInteger(*val1<*val2);
context->accessExecution()->pushValue(result);
val1->relax();
val2->relax();
}


//--------------------------------------------------------------------------//
// class Inf_I_I_Code                                                       //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
Inf_I_I_Code::Inf_I_I_Code(const SmaNode * node)
 : SmaCode(node)
{}

Inf_I_I_Code::~Inf_I_I_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void Inf_I_I_Code::eval(Context * context) const // virtual
{
Value * val1=context->accessExecution()->popValue();
Value * val2=context->accessExecution()->popValue();
Value * result=TmpValue::getTmpValue(_node->getType());
result->setInteger(val1->getInteger()<val2->getInteger());
context->accessExecution()->pushValue(result);
val1->relax();
val2->relax();
}


//--------------------------------------------------------------------------//
// class Inf_I_R_Code                                                       //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
Inf_I_R_Code::Inf_I_R_Code(const SmaNode * node)
 : SmaCode(node)
{}

Inf_I_R_Code::~Inf_I_R_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void Inf_I_R_Code::eval(Context * context) const // virtual
{
Value * val1=context->accessExecution()->popValue();
Value * val2=context->accessExecution()->popValue();
Value * result=TmpValue::getTmpValue(_node->getType());
result->setInteger((double)(val1->getInteger())<val2->getReal());
context->accessExecution()->pushValue(result);
val1->relax();
val2->relax();
}


//--------------------------------------------------------------------------//
// class Inf_R_R_Code                                                       //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
Inf_R_R_Code::Inf_R_R_Code(const SmaNode * node)
 : SmaCode(node)
{}

Inf_R_R_Code::~Inf_R_R_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void Inf_R_R_Code::eval(Context * context) const // virtual
{
Value * val1=context->accessExecution()->popValue();
Value * val2=context->accessExecution()->popValue();
Value * result=TmpValue::getTmpValue(_node->getType());
result->setInteger(val1->getReal()<val2->getReal());
context->accessExecution()->pushValue(result);
val1->relax();
val2->relax();
}


//--------------------------------------------------------------------------//
// class Inf_R_I_Code                                                       //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
Inf_R_I_Code::Inf_R_I_Code(const SmaNode * node)
 : SmaCode(node)
{}

Inf_R_I_Code::~Inf_R_I_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void Inf_R_I_Code::eval(Context * context) const // virtual
{
Value * val1=context->accessExecution()->popValue();
Value * val2=context->accessExecution()->popValue();
Value * result=TmpValue::getTmpValue(_node->getType());
result->setInteger(val1->getReal()<(double)(val2->getInteger()));
context->accessExecution()->pushValue(result);
val1->relax();
val2->relax();
}


//--------------------------------------------------------------------------//
// class Inf_S_S_Code                                                       //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
Inf_S_S_Code::Inf_S_S_Code(const SmaNode * node)
 : SmaCode(node)
{}

Inf_S_S_Code::~Inf_S_S_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void Inf_S_S_Code::eval(Context * context) const // virtual
{
Value * val1=context->accessExecution()->popValue();
Value * val2=context->accessExecution()->popValue();
Value * result=TmpValue::getTmpValue(_node->getType());
result->setInteger(val1->getStrings()<val2->getStrings());
context->accessExecution()->pushValue(result);
val1->relax();
val2->relax();
}


//--------------------------------------------------------------------------//
// class InfEq_Deep_Code                                                    //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
InfEq_Deep_Code::InfEq_Deep_Code(const SmaNode * node)
 : SmaCode(node)
{}

InfEq_Deep_Code::~InfEq_Deep_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void InfEq_Deep_Code::eval(Context * context) const // virtual
{
Value * val1=context->accessExecution()->popValue();
Value * val2=context->accessExecution()->popValue();
Value * result=TmpValue::getTmpValue(_node->getType());
result->setInteger(!(*val2<*val1));
context->accessExecution()->pushValue(result);
val1->relax();
val2->relax();
}


//--------------------------------------------------------------------------//
// class InfEq_I_I_Code                                                     //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
InfEq_I_I_Code::InfEq_I_I_Code(const SmaNode * node)
 : SmaCode(node)
{}

InfEq_I_I_Code::~InfEq_I_I_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void InfEq_I_I_Code::eval(Context * context) const // virtual
{
Value * val1=context->accessExecution()->popValue();
Value * val2=context->accessExecution()->popValue();
Value * result=TmpValue::getTmpValue(_node->getType());
result->setInteger(val1->getInteger()<=val2->getInteger());
context->accessExecution()->pushValue(result);
val1->relax();
val2->relax();
}


//--------------------------------------------------------------------------//
// class InfEq_I_R_Code                                                     //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
InfEq_I_R_Code::InfEq_I_R_Code(const SmaNode * node)
 : SmaCode(node)
{}

InfEq_I_R_Code::~InfEq_I_R_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void InfEq_I_R_Code::eval(Context * context) const // virtual
{
Value * val1=context->accessExecution()->popValue();
Value * val2=context->accessExecution()->popValue();
Value * result=TmpValue::getTmpValue(_node->getType());
result->setInteger((double)(val1->getInteger())<=val2->getReal());
context->accessExecution()->pushValue(result);
val1->relax();
val2->relax();
}


//--------------------------------------------------------------------------//
// class InfEq_R_R_Code                                                     //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
InfEq_R_R_Code::InfEq_R_R_Code(const SmaNode * node)
 : SmaCode(node)
{}

InfEq_R_R_Code::~InfEq_R_R_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void InfEq_R_R_Code::eval(Context * context) const // virtual
{
Value * val1=context->accessExecution()->popValue();
Value * val2=context->accessExecution()->popValue();
Value * result=TmpValue::getTmpValue(_node->getType());
result->setInteger(val1->getReal()<=val2->getReal());
context->accessExecution()->pushValue(result);
val1->relax();
val2->relax();
}


//--------------------------------------------------------------------------//
// class InfEq_R_I_Code                                                     //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
InfEq_R_I_Code::InfEq_R_I_Code(const SmaNode * node)
 : SmaCode(node)
{}

InfEq_R_I_Code::~InfEq_R_I_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void InfEq_R_I_Code::eval(Context * context) const // virtual
{
Value * val1=context->accessExecution()->popValue();
Value * val2=context->accessExecution()->popValue();
Value * result=TmpValue::getTmpValue(_node->getType());
result->setInteger(val1->getReal()<=(double)(val2->getInteger()));
context->accessExecution()->pushValue(result);
val1->relax();
val2->relax();
}


//--------------------------------------------------------------------------//
// class InfEq_S_S_Code                                                     //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
InfEq_S_S_Code::InfEq_S_S_Code(const SmaNode * node)
 : SmaCode(node)
{}

InfEq_S_S_Code::~InfEq_S_S_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void InfEq_S_S_Code::eval(Context * context) const // virtual
{
Value * val1=context->accessExecution()->popValue();
Value * val2=context->accessExecution()->popValue();
Value * result=TmpValue::getTmpValue(_node->getType());
result->setInteger(val1->getStrings()<=val2->getStrings());
context->accessExecution()->pushValue(result);
val1->relax();
val2->relax();
}


//--------------------------------------------------------------------------//
// class Sup_Deep_Code                                                      //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
Sup_Deep_Code::Sup_Deep_Code(const SmaNode * node)
 : SmaCode(node)
{}

Sup_Deep_Code::~Sup_Deep_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void Sup_Deep_Code::eval(Context * context) const // virtual
{
Value * val1=context->accessExecution()->popValue();
Value * val2=context->accessExecution()->popValue();
Value * result=TmpValue::getTmpValue(_node->getType());
result->setInteger(*val2<*val1);
context->accessExecution()->pushValue(result);
val1->relax();
val2->relax();
}


//--------------------------------------------------------------------------//
// class Sup_I_I_Code                                                       //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
Sup_I_I_Code::Sup_I_I_Code(const SmaNode * node)
 : SmaCode(node)
{}

Sup_I_I_Code::~Sup_I_I_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void Sup_I_I_Code::eval(Context * context) const // virtual
{
Value * val1=context->accessExecution()->popValue();
Value * val2=context->accessExecution()->popValue();
Value * result=TmpValue::getTmpValue(_node->getType());
result->setInteger(val1->getInteger()>val2->getInteger());
context->accessExecution()->pushValue(result);
val1->relax();
val2->relax();
}


//--------------------------------------------------------------------------//
// class Sup_I_R_Code                                                       //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
Sup_I_R_Code::Sup_I_R_Code(const SmaNode * node)
 : SmaCode(node)
{}

Sup_I_R_Code::~Sup_I_R_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void Sup_I_R_Code::eval(Context * context) const // virtual
{
Value * val1=context->accessExecution()->popValue();
Value * val2=context->accessExecution()->popValue();
Value * result=TmpValue::getTmpValue(_node->getType());
result->setInteger((double)(val1->getInteger())>val2->getReal());
context->accessExecution()->pushValue(result);
val1->relax();
val2->relax();
}


//--------------------------------------------------------------------------//
// class Sup_R_R_Code                                                       //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
Sup_R_R_Code::Sup_R_R_Code(const SmaNode * node)
 : SmaCode(node)
{}

Sup_R_R_Code::~Sup_R_R_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void Sup_R_R_Code::eval(Context * context) const // virtual
{
Value * val1=context->accessExecution()->popValue();
Value * val2=context->accessExecution()->popValue();
Value * result=TmpValue::getTmpValue(_node->getType());
result->setInteger(val1->getReal()>val2->getReal());
context->accessExecution()->pushValue(result);
val1->relax();
val2->relax();
}


//--------------------------------------------------------------------------//
// class Sup_R_I_Code                                                       //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
Sup_R_I_Code::Sup_R_I_Code(const SmaNode * node)
 : SmaCode(node)
{}

Sup_R_I_Code::~Sup_R_I_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void Sup_R_I_Code::eval(Context * context) const // virtual
{
Value * val1=context->accessExecution()->popValue();
Value * val2=context->accessExecution()->popValue();
Value * result=TmpValue::getTmpValue(_node->getType());
result->setInteger(val1->getReal()>(double)(val2->getInteger()));
context->accessExecution()->pushValue(result);
val1->relax();
val2->relax();
}


//--------------------------------------------------------------------------//
// class Sup_S_S_Code                                                       //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
Sup_S_S_Code::Sup_S_S_Code(const SmaNode * node)
 : SmaCode(node)
{}

Sup_S_S_Code::~Sup_S_S_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void Sup_S_S_Code::eval(Context * context) const // virtual
{
Value * val1=context->accessExecution()->popValue();
Value * val2=context->accessExecution()->popValue();
Value * result=TmpValue::getTmpValue(_node->getType());
result->setInteger(val1->getStrings()>val2->getStrings());
context->accessExecution()->pushValue(result);
val1->relax();
val2->relax();
}


//--------------------------------------------------------------------------//
// class SupEq_Deep_Code                                                    //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
SupEq_Deep_Code::SupEq_Deep_Code(const SmaNode * node)
 : SmaCode(node)
{}

SupEq_Deep_Code::~SupEq_Deep_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void SupEq_Deep_Code::eval(Context * context) const // virtual
{
Value * val1=context->accessExecution()->popValue();
Value * val2=context->accessExecution()->popValue();
Value * result=TmpValue::getTmpValue(_node->getType());
result->setInteger(!(*val1<*val2));
context->accessExecution()->pushValue(result);
val1->relax();
val2->relax();
}


//--------------------------------------------------------------------------//
// class SupEq_I_I_Code                                                     //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
SupEq_I_I_Code::SupEq_I_I_Code(const SmaNode * node)
 : SmaCode(node)
{}

SupEq_I_I_Code::~SupEq_I_I_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void SupEq_I_I_Code::eval(Context * context) const // virtual
{
Value * val1=context->accessExecution()->popValue();
Value * val2=context->accessExecution()->popValue();
Value * result=TmpValue::getTmpValue(_node->getType());
result->setInteger(val1->getInteger()>=val2->getInteger());
context->accessExecution()->pushValue(result);
val1->relax();
val2->relax();
}


//--------------------------------------------------------------------------//
// class SupEq_I_R_Code                                                     //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
SupEq_I_R_Code::SupEq_I_R_Code(const SmaNode * node)
 : SmaCode(node)
{}

SupEq_I_R_Code::~SupEq_I_R_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void SupEq_I_R_Code::eval(Context * context) const // virtual
{
Value * val1=context->accessExecution()->popValue();
Value * val2=context->accessExecution()->popValue();
Value * result=TmpValue::getTmpValue(_node->getType());
result->setInteger((double)(val1->getInteger())>=val2->getReal());
context->accessExecution()->pushValue(result);
val1->relax();
val2->relax();
}


//--------------------------------------------------------------------------//
// class SupEq_R_R_Code                                                     //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
SupEq_R_R_Code::SupEq_R_R_Code(const SmaNode * node)
 : SmaCode(node)
{}

SupEq_R_R_Code::~SupEq_R_R_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void SupEq_R_R_Code::eval(Context * context) const // virtual
{
Value * val1=context->accessExecution()->popValue();
Value * val2=context->accessExecution()->popValue();
Value * result=TmpValue::getTmpValue(_node->getType());
result->setInteger(val1->getReal()>=val2->getReal());
context->accessExecution()->pushValue(result);
val1->relax();
val2->relax();
}


//--------------------------------------------------------------------------//
// class SupEq_R_I_Code                                                     //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
SupEq_R_I_Code::SupEq_R_I_Code(const SmaNode * node)
 : SmaCode(node)
{}

SupEq_R_I_Code::~SupEq_R_I_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void SupEq_R_I_Code::eval(Context * context) const // virtual
{
Value * val1=context->accessExecution()->popValue();
Value * val2=context->accessExecution()->popValue();
Value * result=TmpValue::getTmpValue(_node->getType());
result->setInteger(val1->getReal()>=(double)(val2->getInteger()));
context->accessExecution()->pushValue(result);
val1->relax();
val2->relax();
}


//--------------------------------------------------------------------------//
// class SupEq_S_S_Code                                                     //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
SupEq_S_S_Code::SupEq_S_S_Code(const SmaNode * node)
 : SmaCode(node)
{}

SupEq_S_S_Code::~SupEq_S_S_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void SupEq_S_S_Code::eval(Context * context) const // virtual
{
Value * val1=context->accessExecution()->popValue();
Value * val2=context->accessExecution()->popValue();
Value * result=TmpValue::getTmpValue(_node->getType());
result->setInteger(val1->getStrings()>=val2->getStrings());
context->accessExecution()->pushValue(result);
val1->relax();
val2->relax();
}


//--------------------------------------------------------------------------//
// class Min_Deep_Code                                                      //
//--------------------------------------------------------------------------//
 
//-------- Allocators ------------------------------------------------------//
Min_Deep_Code::Min_Deep_Code(const SmaNode * node)
 : SmaCode(node)
{}
 
Min_Deep_Code::~Min_Deep_Code(void)
{}
 
//-------- Evaluation ------------------------------------------------------//
void Min_Deep_Code::eval(Context * context) const // virtual
{
Value * val1=context->accessExecution()->popValue();
Value * val2=context->accessExecution()->popValue();
if(*val1<*val2)
  {
  context->accessExecution()->pushValue(val1);
  val2->relax();
  }
else
  {
  context->accessExecution()->pushValue(val2);
  val1->relax();
  }
}


//--------------------------------------------------------------------------//
// class Min_I_Code                                                         //
//--------------------------------------------------------------------------//
 
//-------- Allocators ------------------------------------------------------//
Min_I_Code::Min_I_Code(const SmaNode * node)
 : SmaCode(node)
{}
 
Min_I_Code::~Min_I_Code(void)
{}
 
//-------- Evaluation ------------------------------------------------------//
void Min_I_Code::eval(Context * context) const // virtual
{
Value * val1=context->accessExecution()->popValue();
Value * val2=context->accessExecution()->popValue();
if(val1->getInteger()<val2->getInteger())
  {
  context->accessExecution()->pushValue(val1);
  val2->relax();
  }
else
  {
  context->accessExecution()->pushValue(val2);
  val1->relax();
  }
}


//--------------------------------------------------------------------------//
// class Min_R_Code                                                         //
//--------------------------------------------------------------------------//
 
//-------- Allocators ------------------------------------------------------//
Min_R_Code::Min_R_Code(const SmaNode * node)
 : SmaCode(node)
{}
 
Min_R_Code::~Min_R_Code(void)
{}
 
//-------- Evaluation ------------------------------------------------------//
void Min_R_Code::eval(Context * context) const // virtual
{
Value * val1=context->accessExecution()->popValue();
Value * val2=context->accessExecution()->popValue();
if(val1->getReal()<val2->getReal())
  {
  context->accessExecution()->pushValue(val1);
  val2->relax();
  }
else
  {
  context->accessExecution()->pushValue(val2);
  val1->relax();
  }
}


//--------------------------------------------------------------------------//
// class Min_S_Code                                                         //
//--------------------------------------------------------------------------//
 
//-------- Allocators ------------------------------------------------------//
Min_S_Code::Min_S_Code(const SmaNode * node)
 : SmaCode(node)
{}
 
Min_S_Code::~Min_S_Code(void)
{}
 
//-------- Evaluation ------------------------------------------------------//
void Min_S_Code::eval(Context * context) const // virtual
{
Value * val1=context->accessExecution()->popValue();
Value * val2=context->accessExecution()->popValue();
if(val1->getStrings()<val2->getStrings())
  {
  context->accessExecution()->pushValue(val1);
  val2->relax();
  }
else
  {
  context->accessExecution()->pushValue(val2);
  val1->relax();
  }
}


//--------------------------------------------------------------------------//
// class Max_Deep_Code                                                      //
//--------------------------------------------------------------------------//
 
//-------- Allocators ------------------------------------------------------//
Max_Deep_Code::Max_Deep_Code(const SmaNode * node)
 : SmaCode(node)
{}
 
Max_Deep_Code::~Max_Deep_Code(void)
{}
 
//-------- Evaluation ------------------------------------------------------//
void Max_Deep_Code::eval(Context * context) const // virtual
{
Value * val1=context->accessExecution()->popValue();
Value * val2=context->accessExecution()->popValue();
if(*val1<*val2)
  {
  context->accessExecution()->pushValue(val2);
  val1->relax();
  }
else
  {
  context->accessExecution()->pushValue(val1);
  val2->relax();
  }
}


//--------------------------------------------------------------------------//
// class Max_I_Code                                                         //
//--------------------------------------------------------------------------//
 
//-------- Allocators ------------------------------------------------------//
Max_I_Code::Max_I_Code(const SmaNode * node)
 : SmaCode(node)
{}
 
Max_I_Code::~Max_I_Code(void)
{}
 
//-------- Evaluation ------------------------------------------------------//
void Max_I_Code::eval(Context * context) const // virtual
{
Value * val1=context->accessExecution()->popValue();
Value * val2=context->accessExecution()->popValue();
if(val1->getInteger()<val2->getInteger())
  {
  context->accessExecution()->pushValue(val2);
  val1->relax();
  }
else
  {
  context->accessExecution()->pushValue(val1);
  val2->relax();
  }
}


//--------------------------------------------------------------------------//
// class Max_R_Code                                                         //
//--------------------------------------------------------------------------//
 
//-------- Allocators ------------------------------------------------------//
Max_R_Code::Max_R_Code(const SmaNode * node)
 : SmaCode(node)
{}
 
Max_R_Code::~Max_R_Code(void)
{}
 
//-------- Evaluation ------------------------------------------------------//
void Max_R_Code::eval(Context * context) const // virtual
{
Value * val1=context->accessExecution()->popValue();
Value * val2=context->accessExecution()->popValue();
if(val1->getReal()<val2->getReal())
  {
  context->accessExecution()->pushValue(val2);
  val1->relax();
  }
else
  {
  context->accessExecution()->pushValue(val1);
  val2->relax();
  }
}


//--------------------------------------------------------------------------//
// class Max_S_Code                                                         //
//--------------------------------------------------------------------------//
 
//-------- Allocators ------------------------------------------------------//
Max_S_Code::Max_S_Code(const SmaNode * node)
 : SmaCode(node)
{}
 
Max_S_Code::~Max_S_Code(void)
{}
 
//-------- Evaluation ------------------------------------------------------//
void Max_S_Code::eval(Context * context) const // virtual
{
Value * val1=context->accessExecution()->popValue();
Value * val2=context->accessExecution()->popValue();
if(val1->getStrings()<val2->getStrings())
  {
  context->accessExecution()->pushValue(val2);
  val1->relax();
  }
else
  {
  context->accessExecution()->pushValue(val1);
  val2->relax();
  }
}

