// @(#) ENIB/LI2 (c) - oRis - Version 2.3
//--------------------------------------------------------------------------//
// file        : listCd.cpp                                                 //
// description : source file for the ENIB/LI2 List codes                    //
// author(s)   : F. Harrouet                                                //
// copyright   : Laboratoire d'Informatique Industrielle (LI2)              //
//               Ecole Nationale d'Ingenieurs de Brest (ENIB)               //
//               CP 30815 - 29608 Brest Cedex - FRANCE                      //
//               Tel: (33) 02.98.05.66.31 , Fax: (33) 02.98.05.66.29        //
//               e-mail: li2@enib.fr                                        //
//--------------------------------------------------------------------------//
// Copyright (c) 1996-2000                                                  //
// Laboratoire d'Informatique Industrielle (LI2)                            //
// Ecole Nationale d'Ingenieurs de Brest (ENIB)                             //
//                                                                          //
// Permission to use, copy, modify, distribute and sell this software       //
// and its documentation for any purpose is hereby granted without fee,     //
// provided that the above copyright notice appear in all copies and        //
// that both that copyright notice and this permission notice appear        //
// in supporting documentation.                                             //
// ENIB/LI2 makes no representations about the suitability of this          //
// software for any purpose.                                                //
// It is provided "as is" without express or implied warranty.              //
//--------------------------------------------------------------------------//

#include "oRis/Codes/listCd.h"
#include "oRis/Main/context.h"
#include "oRis/Main/simul.h"
#include "oRis/Main/exec.h"
#include "oRis/Main/errors.h"
#include "oRis/Nodes/smaNd.h"

//--------------------------------------------------------------------------//
// class BackRefCode                                                        //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
BackRefCode::BackRefCode(const SmaNode * node)
 : SmaCode(node)
{}

BackRefCode::~BackRefCode(void)
{}

//-------- Evaluation ------------------------------------------------------//
void BackRefCode::eval(Context * context) const // virtual
{
Value * value=context->accessExecution()->popValue();
if(value->getLen())
  context->accessExecution()->pushValue(value->accessBack()); // NO RELAX !
else
  {
  value->relax();
  context->accessExecution()
         ->pushValue(TmpValue::getTmpValue(_node->getType())); // not used
  Strings message="Can't use '"+_node->getName()+"' on an empty list";
  manageError(new Error(Error::RUNTIME,message));
  }
}


//--------------------------------------------------------------------------//
// class BackCopyCode                                                       //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
BackCopyCode::BackCopyCode(const SmaNode * node)
 : SmaCode(node)
{}

BackCopyCode::~BackCopyCode(void)
{}

//-------- Evaluation ------------------------------------------------------//
void BackCopyCode::eval(Context * context) const // virtual
{
Value * value=context->accessExecution()->popValue();
if(value->getLen())
  {
  Value * result=TmpValue::getTmpValue(_node->getType());
  *result=*value->accessBack();
  value->relax();
  context->accessExecution()->pushValue(result);
  }
else
  {
  value->relax();
  context->accessExecution()
         ->pushValue(TmpValue::getTmpValue(_node->getType())); // not used
  Strings message="Can't use '"+_node->getName()+"' on an empty list";
  manageError(new Error(Error::RUNTIME,message));
  }
}


//--------------------------------------------------------------------------//
// class FrontRefCode                                                       //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
FrontRefCode::FrontRefCode(const SmaNode * node)
 : SmaCode(node)
{}

FrontRefCode::~FrontRefCode(void)
{}

//-------- Evaluation ------------------------------------------------------//
void FrontRefCode::eval(Context * context) const // virtual
{
Value * value=context->accessExecution()->popValue();
if(value->getLen())
  context->accessExecution()->pushValue(value->accessFront()); // NO RELAX !
else
  {
  value->relax();
  context->accessExecution()
         ->pushValue(TmpValue::getTmpValue(_node->getType())); // not used
  Strings message="Can't use '"+_node->getName()+"' on an empty list";
  manageError(new Error(Error::RUNTIME,message));
  }
}


//--------------------------------------------------------------------------//
// class FrontCopyCode                                                      //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
FrontCopyCode::FrontCopyCode(const SmaNode * node)
 : SmaCode(node)
{}

FrontCopyCode::~FrontCopyCode(void)
{}

//-------- Evaluation ------------------------------------------------------//
void FrontCopyCode::eval(Context * context) const // virtual
{
Value * value=context->accessExecution()->popValue();
if(value->getLen())
  {
  Value * result=TmpValue::getTmpValue(_node->getType());
  *result=*value->accessFront();
  value->relax();
  context->accessExecution()->pushValue(result);
  }
else
  {
  value->relax();
  context->accessExecution()
         ->pushValue(TmpValue::getTmpValue(_node->getType())); // not used
  Strings message="Can't use '"+_node->getName()+"' on an empty list";
  manageError(new Error(Error::RUNTIME,message));
  }
}


//--------------------------------------------------------------------------//
// class Index_S_I_Code                                                     //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
Index_S_I_Code::Index_S_I_Code(const SmaNode * node)
 : SmaCode(node)
{}

Index_S_I_Code::~Index_S_I_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void Index_S_I_Code::eval(Context * context) const // virtual
{
Value * value=context->accessExecution()->popValue();
int index=value->getInteger();
value->relax();
value=context->accessExecution()->popValue();
Value * result=TmpValue::getTmpValue(_node->getType());
if((index>=0)&&(index<(int)value->getStrings().len()))
  result->setStrings(value->getStrings()[index]);
else
  result->setStrings(Strings());
value->relax();
context->accessExecution()->pushValue(result);
}


//--------------------------------------------------------------------------//
// class Index_S_R_Code                                                     //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
Index_S_R_Code::Index_S_R_Code(const SmaNode * node)
 : SmaCode(node)
{}

Index_S_R_Code::~Index_S_R_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void Index_S_R_Code::eval(Context * context) const // virtual
{
Value * value=context->accessExecution()->popValue();
int index=(int)value->getReal();
value->relax();
value=context->accessExecution()->popValue();
Value * result=TmpValue::getTmpValue(_node->getType());
if((index>=0)&&(index<(int)value->getStrings().len()))
  result->setStrings(value->getStrings()[index]);
else
  result->setStrings(Strings());
value->relax();
context->accessExecution()->pushValue(result);
}


//--------------------------------------------------------------------------//
// class IndexRef_I_Code                                                    //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
IndexRef_I_Code::IndexRef_I_Code(const SmaNode * node)
 : SmaCode(node)
{}

IndexRef_I_Code::~IndexRef_I_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void IndexRef_I_Code::eval(Context * context) const // virtual
{
Value * value=context->accessExecution()->popValue();
int index=value->getInteger();
value->relax();
value=context->accessExecution()->popValue();
if((index>=0)&&(index<value->getLen()))
  context->accessExecution()
         ->pushValue(value->accessElem(index)); // NO RELAX !
else
  {
  int len=value->getLen();
  value->relax();
  ValueType resultType(_node->getType());
  context->accessExecution()
         ->pushValue(TmpValue::getTmpValue(resultType)); // not used
  if(len)
    {
    Strings message="Can use '[]' only from "+Strings(0)+
                    " to "+Strings(len-1)+
                    " (called with "+Strings(index)+")";
    manageError(new Error(Error::RUNTIME,message));
    }
  else
    {
    Strings message="Can't use '[]' on an empty list";
    manageError(new Error(Error::RUNTIME,message));
    }
  }
}


//--------------------------------------------------------------------------//
// class IndexRef_R_Code                                                    //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
IndexRef_R_Code::IndexRef_R_Code(const SmaNode * node)
 : SmaCode(node)
{}

IndexRef_R_Code::~IndexRef_R_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void IndexRef_R_Code::eval(Context * context) const // virtual
{
Value * value=context->accessExecution()->popValue();
int index=(int)value->getReal();
value->relax();
value=context->accessExecution()->popValue();
if((index>=0)&&(index<value->getLen()))
  context->accessExecution()
         ->pushValue(value->accessElem(index)); // NO RELAX !
else
  {
  int len=value->getLen();
  value->relax();
  ValueType resultType(_node->getType());
  context->accessExecution()
         ->pushValue(TmpValue::getTmpValue(resultType)); // not used
  if(len)
    {
    Strings message="Can use '[]' only from "+Strings(0)+
                    " to "+Strings(len-1)+
                    " (called with "+Strings(index)+")";
    manageError(new Error(Error::RUNTIME,message));
    }
  else
    {
    Strings message="Can't use '[]' on an empty list";
    manageError(new Error(Error::RUNTIME,message));
    }
  }
}


//--------------------------------------------------------------------------//
// class IndexCopy_I_Code                                                   //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
IndexCopy_I_Code::IndexCopy_I_Code(const SmaNode * node)
 : SmaCode(node)
{}

IndexCopy_I_Code::~IndexCopy_I_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void IndexCopy_I_Code::eval(Context * context) const // virtual
{
Value * value=context->accessExecution()->popValue();
int index=value->getInteger();
value->relax();
value=context->accessExecution()->popValue();
if((index>=0)&&(index<value->getLen()))
  {
  ValueType resultType(_node->getType());
  Value * result=TmpValue::getTmpValue(resultType);
  *result=*value->accessElem(index);
  value->relax();
  context->accessExecution()->pushValue(result);
  }
else
  {
  int len=value->getLen();
  value->relax();
  ValueType resultType(_node->getType());
  context->accessExecution()
         ->pushValue(TmpValue::getTmpValue(resultType)); // not used
  if(len)
    {
    Strings message="Can use '[]' only from "+Strings(0)+
                    " to "+Strings(len-1)+
                    " (called with "+Strings(index)+")";
    manageError(new Error(Error::RUNTIME,message));
    }
  else
    {
    Strings message="Can't use '[]' on an empty list";
    manageError(new Error(Error::RUNTIME,message));
    }
  }
}


//--------------------------------------------------------------------------//
// class IndexCopy_R_Code                                                   //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
IndexCopy_R_Code::IndexCopy_R_Code(const SmaNode * node)
 : SmaCode(node)
{}

IndexCopy_R_Code::~IndexCopy_R_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void IndexCopy_R_Code::eval(Context * context) const // virtual
{
Value * value=context->accessExecution()->popValue();
int index=(int)value->getReal();
value->relax();
value=context->accessExecution()->popValue();
if((index>=0)&&(index<value->getLen()))
  {
  ValueType resultType(_node->getType());
  Value * result=TmpValue::getTmpValue(resultType);
  *result=*value->accessElem(index);
  value->relax();
  context->accessExecution()->pushValue(result);
  }
else
  {
  int len=value->getLen();
  value->relax();
  ValueType resultType(_node->getType());
  context->accessExecution()
         ->pushValue(TmpValue::getTmpValue(resultType)); // not used
  if(len)
    {
    Strings message="Can use '[]' only from "+Strings(0)+
                    " to "+Strings(len-1)+
                    " (called with "+Strings(index)+")";
    manageError(new Error(Error::RUNTIME,message));
    }
  else
    {
    Strings message="Can't use '[]' on an empty list";
    manageError(new Error(Error::RUNTIME,message));
    }
  }
}


//--------------------------------------------------------------------------//
// class EraseCode                                                          //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
EraseCode::EraseCode(const SmaNode * node)
 : SmaCode(node)
{}

EraseCode::~EraseCode(void)
{}

//-------- Evaluation ------------------------------------------------------//
void EraseCode::eval(Context * context) const // virtual
{
Value * value=context->accessExecution()->popValue();
value->setValueVector(SubValue::getSubValueVect());
value->relax();
}

                                                                  
//--------------------------------------------------------------------------//
// class SortCode                                                           //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
SortCode::SortCode(const SmaNode * node)
 : SmaCode(node)
{}

SortCode::~SortCode(void)
{}

//-------- Evaluation ------------------------------------------------------//
void SortCode::eval(Context * context) const // virtual
{
Value * value=context->accessExecution()->popValue();
value->sortVector();
value->relax();
}


//--------------------------------------------------------------------------//
// class MinPosCode                                                         //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
MinPosCode::MinPosCode(const SmaNode * node)
 : SmaCode(node)
{}

MinPosCode::~MinPosCode(void)
{}

//-------- Evaluation ------------------------------------------------------//
void MinPosCode::eval(Context * context) const // virtual
{
Value * value=context->accessExecution()->popValue();
int min=value->minPosition();
value->relax();
value=TmpValue::getTmpValue(_node->getType());
value->setInteger(min);
context->accessExecution()->pushValue(value);
}


//--------------------------------------------------------------------------//
// class MaxPosCode                                                         //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
MaxPosCode::MaxPosCode(const SmaNode * node)
 : SmaCode(node)
{}

MaxPosCode::~MaxPosCode(void)
{}

//-------- Evaluation ------------------------------------------------------//
void MaxPosCode::eval(Context * context) const // virtual
{
Value * value=context->accessExecution()->popValue();
int max=value->maxPosition();
value->relax();
value=TmpValue::getTmpValue(_node->getType());
value->setInteger(max);
context->accessExecution()->pushValue(value);
}


//--------------------------------------------------------------------------//
// class Insert_same_I_Code                                                 //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
Insert_same_I_Code::Insert_same_I_Code(const SmaNode * node)
 : SmaCode(node)
{}

Insert_same_I_Code::~Insert_same_I_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void Insert_same_I_Code::eval(Context * context) const // virtual
{
Value * value=context->accessExecution()->popValue();
int index=value->getInteger();
value->relax();
value=context->accessExecution()->popValue();
if((index>=0)&&(index<=value->getLen()))
  {
  Value * elem=context->accessExecution()->popValue();
  value->insert(*elem,index);
  elem->relax();
  value->relax();
  }
else
  {
  int len=value->getLen();
  value->relax();
  context->accessExecution()->popValue()->relax(); // elem
  Strings message="Can use '"+
                  _node->getName()+
                  "' only from "+Strings(0)+
                  " to "+Strings(len)+
                  " (called with "+Strings(index)+")";
  manageError(new Error(Error::RUNTIME,message));
  }
}

                                                                  
//--------------------------------------------------------------------------//
// class Insert_same_R_Code                                                 //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
Insert_same_R_Code::Insert_same_R_Code(const SmaNode * node)
 : SmaCode(node)
{}

Insert_same_R_Code::~Insert_same_R_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void Insert_same_R_Code::eval(Context * context) const // virtual
{
Value * value=context->accessExecution()->popValue();
int index=(int)value->getReal();
value->relax();
value=context->accessExecution()->popValue();
if((index>=0)&&(index<=value->getLen()))
  {
  Value * elem=context->accessExecution()->popValue();
  value->insert(*elem,index);
  elem->relax();
  value->relax();
  }
else
  {
  int len=value->getLen();
  value->relax();
  context->accessExecution()->popValue()->relax(); // elem
  Strings message="Can use '"+
                  _node->getName()+
                  "' only from "+Strings(0)+
                  " to "+Strings(len)+
                  " (called with "+Strings(index)+")";
  manageError(new Error(Error::RUNTIME,message));
  }
}

                                                                  
//--------------------------------------------------------------------------//
// class Insert_diff_I_Code                                                 //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
Insert_diff_I_Code::Insert_diff_I_Code(const SmaNode * node,
                                       const ValueType & listElemType)
 : SmaCode(node), _listElemType(listElemType)
{}

Insert_diff_I_Code::~Insert_diff_I_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void Insert_diff_I_Code::eval(Context * context) const // virtual
{
Value * value=context->accessExecution()->popValue();
int index=value->getInteger();
value->relax();
value=context->accessExecution()->popValue();
if((index>=0)&&(index<=value->getLen()))
  {
  ValueType listElemType(_listElemType);
  Value * tmp=TmpValue::getTmpValue(listElemType);
  Value * elem=context->accessExecution()->popValue();
  tmp->affectCast(*elem);
  elem->relax();
  value->insert(*tmp,index);
  tmp->relax();
  value->relax();
  }
else
  {
  int len=value->getLen();
  value->relax();
  context->accessExecution()->popValue()->relax(); // elem
  Strings message="Can use '"+
                  _node->getName()+
                  "' only from "+Strings(0)+
                  " to "+Strings(len)+
                  " (called with "+Strings(index)+")";
  manageError(new Error(Error::RUNTIME,message));
  }
}


//--------------------------------------------------------------------------//
// class Insert_diff_R_Code                                                 //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
Insert_diff_R_Code::Insert_diff_R_Code(const SmaNode * node,
                                       const ValueType & listElemType)
 : SmaCode(node), _listElemType(listElemType)
{}

Insert_diff_R_Code::~Insert_diff_R_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void Insert_diff_R_Code::eval(Context * context) const // virtual
{
Value * value=context->accessExecution()->popValue();
int index=(int)value->getReal();
value->relax();
value=context->accessExecution()->popValue();
if((index>=0)&&(index<=value->getLen()))
  {
  ValueType listElemType(_listElemType);
  Value * tmp=TmpValue::getTmpValue(listElemType);
  Value * elem=context->accessExecution()->popValue();
  tmp->affectCast(*elem);
  elem->relax();
  value->insert(*tmp,index);
  tmp->relax();
  value->relax();
  }
else
  {
  int len=value->getLen();
  value->relax();
  context->accessExecution()->popValue()->relax(); // elem
  Strings message="Can use '"+
                  _node->getName()+
                  "' only from "+Strings(0)+
                  " to "+Strings(len)+
                  " (called with "+Strings(index)+")";
  manageError(new Error(Error::RUNTIME,message));
  }
}


//--------------------------------------------------------------------------//
// class PushBack_same_Code                                                 //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
PushBack_same_Code::PushBack_same_Code(const SmaNode * node)
 : SmaCode(node)
{}

PushBack_same_Code::~PushBack_same_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void PushBack_same_Code::eval(Context * context) const // virtual
{
Value * value=context->accessExecution()->popValue();
Value * elem=context->accessExecution()->popValue();
value->pushBack(*elem);
elem->relax();
value->relax();
}


//--------------------------------------------------------------------------//
// class PushBack_diff_Code                                                 //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
PushBack_diff_Code::PushBack_diff_Code(const SmaNode * node,
                                       const ValueType & listElemType)
 : SmaCode(node), _listElemType(listElemType)
{}

PushBack_diff_Code::~PushBack_diff_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void PushBack_diff_Code::eval(Context * context) const // virtual
{
Value * value=context->accessExecution()->popValue();
Value * tmp=TmpValue::getTmpValue(_listElemType);
Value * elem=context->accessExecution()->popValue();
tmp->affectCast(*elem);
elem->relax();
value->pushBack(*tmp);
tmp->relax();
value->relax();
}


//--------------------------------------------------------------------------//
// class PushFront_same_Code                                                //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
PushFront_same_Code::PushFront_same_Code(const SmaNode * node)
 : SmaCode(node)
{}

PushFront_same_Code::~PushFront_same_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void PushFront_same_Code::eval(Context * context) const // virtual
{
Value * value=context->accessExecution()->popValue();
Value * elem=context->accessExecution()->popValue();
value->pushFront(*elem);
elem->relax();
value->relax();
}


//--------------------------------------------------------------------------//
// class PushFront_diff_Code                                                //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
PushFront_diff_Code::PushFront_diff_Code(const SmaNode * node,
                                         const ValueType & listElemType)
 : SmaCode(node), _listElemType(listElemType)
{}

PushFront_diff_Code::~PushFront_diff_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void PushFront_diff_Code::eval(Context * context) const // virtual
{
Value * value=context->accessExecution()->popValue();
Value * tmp=TmpValue::getTmpValue(_listElemType);
Value * elem=context->accessExecution()->popValue();
tmp->affectCast(*elem);
elem->relax();
value->pushFront(*tmp);
tmp->relax();
value->relax();
}


//--------------------------------------------------------------------------//
// class Remove_I_Code                                                      //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
Remove_I_Code::Remove_I_Code(const SmaNode * node)
 : SmaCode(node)
{}

Remove_I_Code::~Remove_I_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void Remove_I_Code::eval(Context * context) const // virtual
{
Value * value=context->accessExecution()->popValue();
int index=value->getInteger();
value->relax();
value=context->accessExecution()->popValue();
if((index>=0)&&(index<value->getLen()))
  {
  Value * tmp=TmpValue::getTmpValue(_node->getType());
  value->readErase(*tmp,index);
  value->relax();
  context->accessExecution()->pushValue(tmp);
  }
else
  {
  int len=value->getLen();
  value->relax();
  context->accessExecution()
         ->pushValue(TmpValue::getTmpValue(_node->getType())); // not used
  if(len)
    {
    Strings message="Can use '"+
                    _node->getName()+
                    "' only from "+Strings(0)+
                    " to "+Strings(len-1)+
                    " (called with "+Strings(index)+")";
    manageError(new Error(Error::RUNTIME,message));
    }
  else
    {
    Strings message="Can't use '"+_node->getName()+"' on an empty list";
    manageError(new Error(Error::RUNTIME,message));
    }
  }
}


//--------------------------------------------------------------------------//
// class Remove_R_Code                                                      //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
Remove_R_Code::Remove_R_Code(const SmaNode * node)
 : SmaCode(node)
{}

Remove_R_Code::~Remove_R_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void Remove_R_Code::eval(Context * context) const // virtual
{
Value * value=context->accessExecution()->popValue();
int index=(int)value->getReal();
value->relax();
value=context->accessExecution()->popValue();
if((index>=0)&&(index<value->getLen()))
  {
  Value * tmp=TmpValue::getTmpValue(_node->getType());
  value->readErase(*tmp,index);
  value->relax();
  context->accessExecution()->pushValue(tmp);
  }
else
  {
  int len=value->getLen();
  value->relax();
  context->accessExecution()
         ->pushValue(TmpValue::getTmpValue(_node->getType())); // not used
  if(len)
    {
    Strings message="Can use '"+
                    _node->getName()+
                    "' only from "+Strings(0)+
                    " to "+Strings(len-1)+
                    " (called with "+Strings(index)+")";
    manageError(new Error(Error::RUNTIME,message));
    }
  else
    {
    Strings message="Can't use '"+_node->getName()+"' on an empty list";
    manageError(new Error(Error::RUNTIME,message));
    }
  }
}


//--------------------------------------------------------------------------//
// class RemoveVoid_I_Code                                                  //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
RemoveVoid_I_Code::RemoveVoid_I_Code(const SmaNode * node)
 : SmaCode(node)
{}

RemoveVoid_I_Code::~RemoveVoid_I_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void RemoveVoid_I_Code::eval(Context * context) const // virtual
{
Value * value=context->accessExecution()->popValue();
int index=value->getInteger();
value->relax();
value=context->accessExecution()->popValue();
if((index>=0)&&(index<value->getLen()))
  {
  value->erase(index);
  value->relax();
  }
else
  {
  int len=value->getLen();
  value->relax();
  if(len)
    {
    Strings message="Can use '"+
                    _node->getName()+
                    "' only from "+Strings(0)+
                    " to "+Strings(len-1)+
                    " (called with "+Strings(index)+")";
    manageError(new Error(Error::RUNTIME,message));
    }
  else
    {
    Strings message="Can't use '"+_node->getName()+"' on an empty list";
    manageError(new Error(Error::RUNTIME,message));
    }
  }
}


//--------------------------------------------------------------------------//
// class RemoveVoid_R_Code                                                  //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
RemoveVoid_R_Code::RemoveVoid_R_Code(const SmaNode * node)
 : SmaCode(node)
{}

RemoveVoid_R_Code::~RemoveVoid_R_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void RemoveVoid_R_Code::eval(Context * context) const // virtual
{
Value * value=context->accessExecution()->popValue();
int index=(int)value->getReal();
value->relax();
value=context->accessExecution()->popValue();
if((index>=0)&&(index<value->getLen()))
  {
  value->erase(index);
  value->relax();
  }
else
  {
  int len=value->getLen();
  value->relax();
  if(len)
    {
    Strings message="Can use '"+
                    _node->getName()+
                    "' only from "+Strings(0)+
                    " to "+Strings(len-1)+
                    " (called with "+Strings(index)+")";
    manageError(new Error(Error::RUNTIME,message));
    }
  else
    {
    Strings message="Can't use '"+_node->getName()+"' on an empty list";
    manageError(new Error(Error::RUNTIME,message));
    }
  }
}


//--------------------------------------------------------------------------//
// class PopBackCode                                                        //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
PopBackCode::PopBackCode(const SmaNode * node)
 : SmaCode(node)
{}

PopBackCode::~PopBackCode(void)
{}

//-------- Evaluation ------------------------------------------------------//
void PopBackCode::eval(Context * context) const // virtual
{
Value * value=context->accessExecution()->popValue();
if(value->getLen())
  {
  Value * tmp=TmpValue::getTmpValue(_node->getType());
  value->readPopBack(*tmp);
  value->relax();
  context->accessExecution()->pushValue(tmp);
  }
else
  {
  value->relax();
  context->accessExecution()
         ->pushValue(TmpValue::getTmpValue(_node->getType())); // not used
  Strings message="Can't use '"+_node->getName()+"' on an empty list";
  manageError(new Error(Error::RUNTIME,message));
  }
}


//--------------------------------------------------------------------------//
// class PopBackVoidCode                                                    //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
PopBackVoidCode::PopBackVoidCode(const SmaNode * node)
 : SmaCode(node)
{}

PopBackVoidCode::~PopBackVoidCode(void)
{}

//-------- Evaluation ------------------------------------------------------//
void PopBackVoidCode::eval(Context * context) const // virtual
{
Value * value=context->accessExecution()->popValue();
if(value->getLen())
  {
  value->popBack();
  value->relax();
  }
else
  {
  value->relax();
  Strings message="Can't use '"+_node->getName()+"' on an empty list";
  manageError(new Error(Error::RUNTIME,message));
  }
}


//--------------------------------------------------------------------------//
// class PopFrontCode                                                       //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
PopFrontCode::PopFrontCode(const SmaNode * node)
 : SmaCode(node)
{}

PopFrontCode::~PopFrontCode(void)
{}

//-------- Evaluation ------------------------------------------------------//
void PopFrontCode::eval(Context * context) const // virtual
{
Value * value=context->accessExecution()->popValue();
if(value->getLen())
  {
  Value * tmp=TmpValue::getTmpValue(_node->getType());
  value->readPopFront(*tmp);
  value->relax();
  context->accessExecution()->pushValue(tmp);
  }
else
  {
  value->relax();
  context->accessExecution()
         ->pushValue(TmpValue::getTmpValue(_node->getType())); // not used
  Strings message="Can't use '"+_node->getName()+"' on an empty list";
  manageError(new Error(Error::RUNTIME,message));
  }
}


//--------------------------------------------------------------------------//
// class PopFrontVoidCode                                                   //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
PopFrontVoidCode::PopFrontVoidCode(const SmaNode * node)
 : SmaCode(node)
{}

PopFrontVoidCode::~PopFrontVoidCode(void)
{}

//-------- Evaluation ------------------------------------------------------//
void PopFrontVoidCode::eval(Context * context) const // virtual
{
Value * value=context->accessExecution()->popValue();
if(value->getLen())
  {
  value->popFront();
  value->relax();
  }
else
  {
  value->relax();
  Strings message="Can't use '"+_node->getName()+"' on an empty list";
  manageError(new Error(Error::RUNTIME,message));
  }
}


//--------------------------------------------------------------------------//
// class Length_V_Code                                                      //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
Length_V_Code::Length_V_Code(const SmaNode * node)
 : SmaCode(node)
{}

Length_V_Code::~Length_V_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void Length_V_Code::eval(Context * context) const // virtual
{
Value * value=context->accessExecution()->popValue();
int len=value->getLen();
value->relax();
value=TmpValue::getTmpValue(_node->getType());
value->setInteger(len);
context->accessExecution()->pushValue(value);
}


//--------------------------------------------------------------------------//
// class Length_S_Code                                                      //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
Length_S_Code::Length_S_Code(const SmaNode * node)
 : SmaCode(node)
{}

Length_S_Code::~Length_S_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void Length_S_Code::eval(Context * context) const // virtual
{
Value * value=context->accessExecution()->popValue();
int len=value->getStrings().len();
value->relax();
value=TmpValue::getTmpValue(_node->getType());
value->setInteger(len);
context->accessExecution()->pushValue(value);
}


//--------------------------------------------------------------------------//
// class Seek_same_V_Code                                                   //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
Seek_same_V_Code::Seek_same_V_Code(const SmaNode * node,bool deep)
 : SmaCode(node), _deep(deep)
{}

Seek_same_V_Code::~Seek_same_V_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void Seek_same_V_Code::eval(Context * context) const // virtual
{
Value * value=context->accessExecution()->popValue();
Value * elem=context->accessExecution()->popValue();
int rank=value->findElem(*elem,_deep);
elem->relax();
value->relax();
elem=TmpValue::getTmpValue(_node->getType());
elem->setInteger(rank);
context->accessExecution()->pushValue(elem);
}


//--------------------------------------------------------------------------//
// class Seek_diff_V_Code                                                   //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
Seek_diff_V_Code::Seek_diff_V_Code(const SmaNode * node,
                                   const ValueType & listElemType)
 : SmaCode(node), _listElemType(listElemType)
{}

Seek_diff_V_Code::~Seek_diff_V_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void Seek_diff_V_Code::eval(Context * context) const // virtual
{
Value * value=context->accessExecution()->popValue();
Value * elem=context->accessExecution()->popValue();
Value * tmp=TmpValue::getTmpValue(_listElemType);
tmp->affectCast(*elem);
elem->relax();
int rank=value->findElem(*tmp,false);
tmp->relax();
value->relax();
elem=TmpValue::getTmpValue(_node->getType());
elem->setInteger(rank);
context->accessExecution()->pushValue(elem);
}


//--------------------------------------------------------------------------//
// class Seek_S_Code                                                        //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
Seek_S_Code::Seek_S_Code(const SmaNode * node)
 : SmaCode(node)
{}

Seek_S_Code::~Seek_S_Code(void)
{}

//-------- Evaluation ------------------------------------------------------//
void Seek_S_Code::eval(Context * context) const // virtual
{
Value * str=context->accessExecution()->popValue();
Value * elem=context->accessExecution()->popValue();
int rank=str->getStrings().lookUp(elem->getStrings());
elem->relax();
str->relax();
elem=TmpValue::getTmpValue(_node->getType());
elem->setInteger(rank);
context->accessExecution()->pushValue(elem);
}


