// @(#) ENIB/LI2 (c) - oRis - Version 2.3
//--------------------------------------------------------------------------//
// file        : moduleCd.cpp                                               //
// description : source file for the ENIB/LI2 class ...ModuleCode           //
// author(s)   : F. Harrouet                                                //
// copyright   : Laboratoire d'Informatique Industrielle (LI2)              //
//               Ecole Nationale d'Ingenieurs de Brest (ENIB)               //
//               CP 30815 - 29608 Brest Cedex - FRANCE                      //
//               Tel: (33) 02.98.05.66.31 , Fax: (33) 02.98.05.66.29        //
//               e-mail: li2@enib.fr                                        //
//--------------------------------------------------------------------------//
// Copyright (c) 1996-2000                                                  //
// Laboratoire d'Informatique Industrielle (LI2)                            //
// Ecole Nationale d'Ingenieurs de Brest (ENIB)                             //
//                                                                          //
// Permission to use, copy, modify, distribute and sell this software       //
// and its documentation for any purpose is hereby granted without fee,     //
// provided that the above copyright notice appear in all copies and        //
// that both that copyright notice and this permission notice appear        //
// in supporting documentation.                                             //
// ENIB/LI2 makes no representations about the suitability of this          //
// software for any purpose.                                                //
// It is provided "as is" without express or implied warranty.              //
//--------------------------------------------------------------------------//

#include "oRis/Codes/moduleCd.h"
#include "oRis/Main/context.h"
#include "oRis/Main/simul.h"
#include "oRis/Main/errors.h"
#include "oRis/Nodes/smaNd.h"

//--------------------------------------------------------------------------//
// class GlobalModuleCode                                                   //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
GlobalModuleCode::GlobalModuleCode(const SmaNode * node,
                                   const Word & moduleName)
 : SmaCode(node), _moduleName(moduleName)
{}

GlobalModuleCode::~GlobalModuleCode(void)
{}

//-------- Evaluation ------------------------------------------------------//
void GlobalModuleCode::eval(Context * context) const // virtual
{
context->accessExecution()
       ->enterContext(getSimul()->accessModule(_moduleName),
                      context->accessOwner());
}


//--------------------------------------------------------------------------//
// class StartCode                                                          //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
StartCode::StartCode(const SmaNode * node,ExecModule * module,bool returnID)
 : SmaCode(node), _module(module), _returnID(returnID)
{}

StartCode::~StartCode(void)
{}

//-------- Evaluation ------------------------------------------------------//
void StartCode::eval(Context * context) const // virtual
{
long id=getSimul()->startModule(_module,context->accessOwner(),
                                context->accessExecution());
if(_returnID)
  {
  Value * value=TmpValue::getTmpValue(_node->getType());
  value->setInteger(id);
  context->accessExecution()->pushValue(value);
  }
}


//--------------------------------------------------------------------------//
// class ModelModuleCode                                                    //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
ModelModuleCode::ModelModuleCode(const SmaNode * node,
                                 const Word & moduleName,Model * model)
 : SmaCode(node), _moduleName(moduleName), _model(model)
{}

ModelModuleCode::~ModelModuleCode(void)
{}

//-------- Evaluation ------------------------------------------------------//
void ModelModuleCode::eval(Context * context) const // virtual
{
context->accessExecution()
       ->enterContext(_model->accessModule(_moduleName),
                      context->accessOwner());
}


//--------------------------------------------------------------------------//
// class AgentModuleCode                                                    //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
AgentModuleCode::AgentModuleCode(const SmaNode * node,
                                 const Word & moduleName)
 : SmaCode(node), _moduleName(moduleName)
{}

AgentModuleCode::~AgentModuleCode(void)
{}

//-------- Evaluation ------------------------------------------------------//
void AgentModuleCode::eval(Context * context) const // virtual
{
context->accessExecution()
       ->enterContext(context->accessOwner()->accessModule(_moduleName),
                      context->accessOwner());
}


//--------------------------------------------------------------------------//
// class ConstrModuleCode                                                   //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
ConstrModuleCode::ConstrModuleCode(const SmaNode * node,Model * model)
 : SmaCode(node), _model(model)
{}

ConstrModuleCode::~ConstrModuleCode(void)
{}

//-------- Evaluation ------------------------------------------------------//
void ConstrModuleCode::eval(Context * context) const // virtual
{
context->accessExecution()
       ->enterContext(_model->accessNewModule(),
                      context->accessOwner());
}

//--------------------------------------------------------------------------//
// class DestrModuleCode                                                    //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
DestrModuleCode::DestrModuleCode(const SmaNode * node,Model * model)
 : SmaCode(node), _model(model)
{}

DestrModuleCode::~DestrModuleCode(void)
{}

//-------- Evaluation ------------------------------------------------------//
void DestrModuleCode::eval(Context * context) const // virtual
{
context->accessExecution()
       ->enterContext(_model->accessDeleteModule(),
                      context->accessOwner());
}


//--------------------------------------------------------------------------//
// class FarModelModuleCode                                                 //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
FarModelModuleCode::FarModelModuleCode(const SmaNode * node,
                                       const Word & moduleName,
                                       Model * model)
 : SmaCode(node), _moduleName(moduleName), _model(model)
{}

FarModelModuleCode::~FarModelModuleCode(void)
{}

//-------- Evaluation ------------------------------------------------------//
void FarModelModuleCode::eval(Context * context) const // virtual
{
Value * value=context->accessExecution()->popValue();
Agent * agent=value->getAgent();
if(agent&&agent->isAlive())
  {
  context->accessExecution()
         ->enterContext(_model->accessModule(_moduleName),agent);
  value->relax();
  }
else
  {
  Strings str;
  if(agent) str="deleted object '"+value->print()+"'";
  else str="'NONE'";
  value->relax();
  Strings message="Can't use module '"+
                  _model->getName().getStrings()+
                  "::"+
                  _moduleName.getStrings()+
                  "()' on "+
                  str;
  manageError(new Error(Error::RUNTIME,message));
  }
}


//--------------------------------------------------------------------------//
// class FarAgentModuleCode                                                 //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
FarAgentModuleCode::FarAgentModuleCode(const SmaNode * node,
                                       const Word & moduleName)
 : SmaCode(node), _moduleName(moduleName)
{}

FarAgentModuleCode::~FarAgentModuleCode(void)
{}

//-------- Evaluation ------------------------------------------------------//
void FarAgentModuleCode::eval(Context * context) const // virtual
{
Value * value=context->accessExecution()->popValue();
Agent * agent=value->getAgent();
if(agent&&agent->isAlive())
  {
  context->accessExecution()
         ->enterContext(agent->accessModule(_moduleName),agent);
  value->relax();
  }
else
  {
  Strings str;
  if(agent) str="deleted object '"+value->print()+"'";
  else str="'NONE'";
  value->relax();
  Strings message="Can't use module '"+
                  _moduleName.getStrings()+
                  "()' on "+
                  str;
  manageError(new Error(Error::RUNTIME,message));
  }
}


//--------------------------------------------------------------------------//
// class NewCode                                                            //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
NewCode::NewCode(const SmaNode * node,Model * model,
                 bool isTransient,bool suffixGiven)
 : SmaCode(node), _model(model),
   _isTransient(isTransient), _suffixGiven(suffixGiven)
{}

NewCode::~NewCode(void)
{}

//-------- Evaluation ------------------------------------------------------//
void NewCode::eval(Context * context) const // virtual
{
long suffix=0; // automatic number
if(_suffixGiven)
  {
  Value * value=context->accessExecution()->popValue();
  if(value->getType()==ValueType(ValueType::INTEGER))
    suffix=value->getInteger();
  else // REAL
    suffix=(long)value->getReal();
  value->relax();
  if(suffix<0) suffix=0;
  }
ExecModule * module=_model->accessNewModule();
Value * created=context->accessExecution()
                       ->accessValue(module->getNbParams());
getSimul()->createAgent(created,_isTransient,_model,suffix);
if(created->getAgent())
  {
  getSimul()->startModule(module,created->getAgent(),
                          context->accessExecution());
  }
else
  {
  Strings message="Can't create '"+
                  AgentName(_model->getName(),suffix).print()+
                  "' (object already exists)";
  manageError(new Error(Error::RUNTIME,message));
  }
}


//--------------------------------------------------------------------------//
// class DeleteCode                                                         //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
DeleteCode::DeleteCode(const SmaNode * node)
 : SmaCode(node)
{}

DeleteCode::~DeleteCode(void)
{}

//-------- Evaluation ------------------------------------------------------//
void DeleteCode::eval(Context * context) const // virtual
{
Value * value=context->accessExecution()->popValue();
Agent * agent=value->getAgent();
if(agent&&agent->isAlive())
  {
  getSimul()->deleteAgent(agent);
  value->relax();
  }
else
  {
  Strings agentName=value->print();
  value->relax();
  if(agent)
    {
    Strings message="Can't delete '"+
                    agentName+
                    "' (already deleted)";
    manageError(new Error(Error::WARNING,message));
    }
  else
    {
    Strings message="Can't delete 'NONE'";
    manageError(new Error(Error::WARNING,message));
    }
  }
}
                                                                  

//--------------------------------------------------------------------------//
// class NativeCode                                                         //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
NativeCode::NativeCode(const SmaNode * node,ExternFunction function)
 : SmaCode(node), _function(function)
{}

NativeCode::~NativeCode(void)
{}

//-------- Evaluation ------------------------------------------------------//
void NativeCode::eval(Context * context) const // virtual
{
(*_function)(context);
}


//--------------------------------------------------------------------------//
// class ProtoCode                                                          //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
ProtoCode::ProtoCode(const SmaNode * node)
 : SmaCode(node)
{}

ProtoCode::~ProtoCode(void)
{}

//-------- Evaluation ------------------------------------------------------//
void ProtoCode::eval(Context * context) const // virtual
{
if(context->getModule()->getResultType().getSimpleType()!=ValueType::UNKNOWN)
  {
  context->returnValue(TmpValue::getTmpValue(context->getModule()
                                                    ->getResultType()));
  }
Strings message="Module is declared but not defined";
manageError(new Error(Error::RUNTIME,message));
}

