// @(#) ENIB/LI2 (c) - oRis - Version 2.3
//--------------------------------------------------------------------------//
// file        : varCd.cpp                                                  //
// description : source file for the ENIB/LI2 Variables access codes        //
// author(s)   : F. Harrouet                                                //
// copyright   : Laboratoire d'Informatique Industrielle (LI2)              //
//               Ecole Nationale d'Ingenieurs de Brest (ENIB)               //
//               CP 30815 - 29608 Brest Cedex - FRANCE                      //
//               Tel: (33) 02.98.05.66.31 , Fax: (33) 02.98.05.66.29        //
//               e-mail: li2@enib.fr                                        //
//--------------------------------------------------------------------------//
// Copyright (c) 1996-2000                                                  //
// Laboratoire d'Informatique Industrielle (LI2)                            //
// Ecole Nationale d'Ingenieurs de Brest (ENIB)                             //
//                                                                          //
// Permission to use, copy, modify, distribute and sell this software       //
// and its documentation for any purpose is hereby granted without fee,     //
// provided that the above copyright notice appear in all copies and        //
// that both that copyright notice and this permission notice appear        //
// in supporting documentation.                                             //
// ENIB/LI2 makes no representations about the suitability of this          //
// software for any purpose.                                                //
// It is provided "as is" without express or implied warranty.              //
//--------------------------------------------------------------------------//

#include "oRis/Codes/varCd.h"
#include "oRis/Main/context.h"
#include "oRis/Main/simul.h"
#include "oRis/Main/exec.h"
#include "oRis/Main/agent.h"
#include "oRis/Main/tmpVal.h"
#include "oRis/Main/errors.h"
#include "oRis/Nodes/smaNd.h"

//--------------------------------------------------------------------------//
// class VarCode                                                            //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
VarCode::VarCode(const SmaNode * node,const Word & name)
 : SmaCode(node), _name(name)
{}

VarCode::~VarCode(void)
{}

//-------- Evaluation ------------------------------------------------------//
void VarCode::eval(Context * context) const // virtual
{
Value * result=context->accessOwner()->accessAttribute(_name)
                                     ->accessValue();
result->reference();
context->accessExecution()->pushValue(result);
}

//--------------------------------------------------------------------------//
// class LocalVarCode                                                       //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
LocalVarCode::LocalVarCode(const SmaNode * node,int rank)
 : SmaCode(node), _rank(rank)
{}

LocalVarCode::~LocalVarCode(void)
{}

//-------- Evaluation ------------------------------------------------------//
void LocalVarCode::eval(Context * context) const // virtual
{
context->accessExecution()->pushValue(context->accessLocal(_rank));
context->accessLocal(_rank)->reference();
}

//--------------------------------------------------------------------------//
// class FarVarCode                                                         //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
FarVarCode::FarVarCode(const SmaNode * node,const Word & name)
 : SmaCode(node), _name(name)
{}

FarVarCode::~FarVarCode(void)
{}

//-------- Evaluation ------------------------------------------------------//
void FarVarCode::eval(Context * context) const // virtual
{
Value * value=context->accessExecution()->popValue();
Agent * agent=value->getAgent();
if(agent&&agent->isAlive())
  {
  Value * result=agent->accessAttribute(_name)->accessValue();
  result->reference();
  value->relax();
  context->accessExecution()->pushValue(result);
  }
else
  {
  Strings str;
  if(agent) str="deleted object '"+value->print()+"'";
  else str="'NONE'";
  value->relax();
  context->accessExecution()
         ->pushValue(TmpValue::getTmpValue(_node->getType())); // not used
  Strings message="Can't use attribute '"+
                  _name.getStrings()+
                  "' of "+
                  str;
  manageError(new Error(Error::RUNTIME,message));
  }
}

