// @(#) ENIB/LI2 (c) - oRis - Version 2.3
//--------------------------------------------------------------------------//
// file        : agent.cpp                                                  //
// description : source file for the ENIB/LI2 class Agent                   //
// author(s)   : F. Harrouet                                                //
// copyright   : Laboratoire d'Informatique Industrielle (LI2)              //
//               Ecole Nationale d'Ingenieurs de Brest (ENIB)               //
//               CP 30815 - 29608 Brest Cedex - FRANCE                      //
//               Tel: (33) 02.98.05.66.31 , Fax: (33) 02.98.05.66.29        //
//               e-mail: li2@enib.fr                                        //
//--------------------------------------------------------------------------//
// Copyright (c) 1996-2000                                                  //
// Laboratoire d'Informatique Industrielle (LI2)                            //
// Ecole Nationale d'Ingenieurs de Brest (ENIB)                             //
//                                                                          //
// Permission to use, copy, modify, distribute and sell this software       //
// and its documentation for any purpose is hereby granted without fee,     //
// provided that the above copyright notice appear in all copies and        //
// that both that copyright notice and this permission notice appear        //
// in supporting documentation.                                             //
// ENIB/LI2 makes no representations about the suitability of this          //
// software for any purpose.                                                //
// It is provided "as is" without express or implied warranty.              //
//------------------------------------------------------------------------- //

#include "oRis/Main/agent.h"
#include "oRis/Main/simul.h"
#include "oRis/Main/errors.h"
#include "oRis/Main/tmpVal.h"
#include "oRis/Main/context.h"

//---------------------------------------------------------------------------//
// class Agent                                                               //
//---------------------------------------------------------------------------//

Word Agent::_notifyAttributeWord(Word::create("notifyAttribute")); // static
Word Agent::_notifyModuleWord(Word::create("notifyModule"));       // static
Word Agent::_messageWord(Word::create("Message"));                 // static
Word Agent::_onMessageWord(Word::create("onMessage"));             // static
Word Agent::_objectDescriptionWord(Word::create("ObjectDescription"));// static
Word Agent::_objectWord(Word::create("object"));                   // static

//-------- Allocators -------------------------------------------------------//
Agent::Agent(Model * model,long instNum)
 : BlockAllocatedObject(),
   _model(model), _name(model->getName(),instNum),
   _attrMap(model->getAttrMap()), _moduleMap(model->getModuleMap()),
   _mainModule(model->accessMainModule()),
   _newModule((ExecModule *)0),
   _deleteModule((ExecModule *)0),
   _isTransient(true), _isAlive(true), _nbRefs(0),
   _isSuspended(false), _msgList()
{
ExecModMap::iterator it=_moduleMap.begin();
for(;!(it==_moduleMap.end());it++)
  (*it).second->reference();
if(_mainModule&&(_mainModule->getType()!=ExecModule::PROTOTYPE))
  getSimul()->manageAgent(this);
}

Agent::~Agent(void)
{
// cerr << "DESTRUCTION DE " << _name << " (" << _nbRefs << " refs)" << endl;
ExecModMap::iterator it=_moduleMap.begin();
for(;!(it==_moduleMap.end());it++)
  (*it).second->unReference();
if(_newModule) _newModule->unReference();
if(_deleteModule) _deleteModule->unReference();
getSimul()->removeDeletedAgent(this);
}

//-------- Inspectors ------------------------------------------------------//
// inline const Model * Agent::getModel(void) const

// inline const AgentName & Agent::getName(void) const

// inline bool Agent::isSuspended(void) const

// inline bool Agent::isTransient(void) const

// inline bool Agent::isAlive(void) const

// inline int Agent::getNbAttributes(void) const

// inline const AttrMap & Agent::getAttrMap(void) const

// inline const Attribute * Agent::getAttribute(const Word & name) const

Error * Agent::tryToSetAttribute(const Attribute & attr) const
{
const Attribute * oldAttr=getAttribute(attr.getName());
if(oldAttr&&oldAttr->getType()!=attr.getType())
  {
  Strings message="Attribute '"+
                  oldAttr->getName().getStrings()+
                  "' has type '"+
                  oldAttr->getType().print()+
                  "' in '"+
                  _name.print()+
                  "'";
  return(new Error(Error::SEMANTICS,message));
  }
return((Error *)0);
}

// inline int Agent::getNbModules(void) const

// inline const ExecModMap & Agent::getModuleMap(void) const

// inline const ExecModule * Agent::getMainModule(void) const

// inline const ExecModule * Agent::getNewModule(void) const

// inline const ExecModule * Agent::getDeleteModule(void) const

const ExecModule * Agent::getModule(const Word & name) const
{
ExecModMap::const_iterator it=_moduleMap.find(name);
if(it==_moduleMap.end())
  {
  if(_newModule&&(_newModule->getName()==name)) return(_newModule);
  if(_model->getNewModule()&&(_model->getNewModule()->getName()==name))
    return(_model->getNewModule());
  if(_deleteModule&&(_deleteModule->getName()==name)) return(_deleteModule);
  if(_model->getDeleteModule()&&(_model->getDeleteModule()->getName()==name))
    return(_model->getDeleteModule());
  return((const ExecModule *)0);
  }
return((*it).second);
}

Error * Agent::tryToSetModule(const ExecModule * newModule) const
{
switch(newModule->getSpecialType())
  {
  case ExecModule::NORMAL:
    {
    const ExecModule * oldModule=getModule(newModule->getName());
    if(oldModule&&!oldModule->isCompatible(*newModule))
      {
      Strings message="Module '"+
                      newModule->getCompleteName()+
                      "' is incompatible with '"+
                      oldModule->getCompleteName()+
                      "'";
      return(new Error(Error::SEMANTICS,message));
      }
    }
    break;
  case ExecModule::BEHAVIOR:
    if((newModule->getResultType()!=ValueType())||
       newModule->getNbParams())
      {
      Strings message="Prototype of module '"+
                      newModule->getCompleteName()+
                      "' should be 'void "+
                      newModule->getName().getStrings()+
                      "(void)'";
      return(new Error(Error::SEMANTICS,message));
      }
    if(_mainModule&&!_mainModule->isCompatible(*newModule))
      {
      Strings message="Module '"+
                      newModule->getCompleteName()+
                      "' is incompatible with '"+
                      _mainModule->getCompleteName()+
                      "'";
      return(new Error(Error::SEMANTICS,message));
      }
    break;
  case ExecModule::CONSTRUCTOR:
    if(newModule->getOwner()==this) // own module
      {
      if(newModule->getResultType()!=ValueType())
        {
        Strings message="Prototype of module '"+
                        newModule->getCompleteName()+
                        "' should be 'void "+
                        newModule->getName().getStrings()+
                        "(...)'";
        return(new Error(Error::SEMANTICS,message));
        }
      if(_newModule&&!_newModule->isCompatible(*newModule))
        {
        Strings message="Module '"+
                        newModule->getCompleteName()+
                        "' is incompatible with '"+
                        _newModule->getCompleteName()+
                        "'";
        return(new Error(Error::SEMANTICS,message));
        }
      }
    break;
  case ExecModule::DESTRUCTOR:
    if(newModule->getOwner()==this) // own module
      {
      if((newModule->getResultType()!=ValueType())||
         newModule->getNbParams())
        {
        Strings message="Prototype of module '"+
                        newModule->getCompleteName()+
                        "' should be 'void "+
                        newModule->getName().getStrings()+
                        "(void)'";
        return(new Error(Error::SEMANTICS,message));
        }
      if(_deleteModule&&!_deleteModule->isCompatible(*newModule))
        {
        Strings message="Module '"+
                        newModule->getCompleteName()+
                        "' is incompatible with '"+
                        _deleteModule->getCompleteName()+
                        "'";
        return(new Error(Error::SEMANTICS,message));
        }
      }
    break;
  default:
    break;
  }
return((Error *)0);
}


//-------- Comparisons -----------------------------------------------------//
// inline bool Agent::operator==(const Agent & agent) const

// inline bool Agent::operator!=(const Agent & agent) const

// inline bool Agent::operator<(const Agent & agent) const

// inline bool Agent::operator>(const Agent & agent) const

// inline bool Agent::operator<=(const Agent & agent) const

// inline bool Agent::operator>=(const Agent & agent) const

//-------- Input/Output ----------------------------------------------------//
ostream & operator<<(ostream & stream, const Agent & agent)
{
return(stream << agent.print());
}

Strings Agent::print(int column) const
{
Strings str;
str+="execute\n{\n"+printInstanciation(column)+"}\n\n";
str+=printDeclarations(column);
str+=printDefinitions(column);
str+="execute\n{\nforce\n  {\n"+printInitialisations(column+2)+"  }\n}\n\n";
return(str);
}

Strings Agent::printInstanciation(int column) const
{
Strings str;
int i;
for(i=0;i<column;i++) str+=" ";
str+="create("+_name.print().dump()+");\n";
return(str);
}

Strings Agent::printDeclarations(int column) const
{
Strings str;
int i;
AttrMap::const_iterator itA=_attrMap.begin();
for(;!(itA==_attrMap.end());itA++)
  {
  if(!_model->getAttribute((*itA).second.getName()))
    {
    for(i=0;i<column;i++) str+=" ";
    str+=(*itA).second.getType().print()+" "+           // declare own
         _name.print()+"::"+                            // attributes
         (*itA).second.getName().getStrings()+";\n\n";
    }
  }
if(_newModule)
  {
  for(i=0;i<column;i++) str+=" ";     // declare own
  str+=_newModule->printProto()+"\n"; // constructor
  }
if(_deleteModule)
  {
  for(i=0;i<column;i++) str+=" ";        // declare own
  str+=_deleteModule->printProto()+"\n"; // destructor
  }
ExecModMap::const_iterator itM=_moduleMap.begin();
for(;!(itM==_moduleMap.end());itM++)
  {
  if((*itM).second->getOwner()==this)
    {
    for(i=0;i<column;i++) str+=" ";        // declare own
    str+=(*itM).second->printProto()+"\n"; // modules
    }
  }
return(str);
}

Strings Agent::printDefinitions(int column) const
{
Strings str;
int i;
if(_newModule)
  {
  for(i=0;i<column;i++) str+=" "; // define own
  str+=_newModule->print()+"\n";  // constructor
  }
if(_deleteModule)
  {
  for(i=0;i<column;i++) str+=" ";   // define own
  str+=_deleteModule->print()+"\n"; // destructor
  }
ExecModMap::const_iterator itM=_moduleMap.begin();
for(;!(itM==_moduleMap.end());itM++)
  {
  if((*itM).second->getOwner()==this)
    {
    for(i=0;i<column;i++) str+=" ";   // define own
    str+=(*itM).second->print()+"\n"; // modules
    }
  }
return(str);
}

Strings Agent::printInitialisations(int column) const
{
Strings str;
int i;
AttrMap::const_iterator itA=_attrMap.begin();
for(;!(itA==_attrMap.end());itA++)
  {
  for(i=0;i<column;i++) str+=" ";                 // init attributes
  str+="valueParse("+_name.print()+"->"+
       (*itA).second.getName().getStrings()+","+
       (*itA).second.getValue()->dump().dump()+");\n";
  }
str+=userSave(column);
for(i=0;i<column;i++) str+=" ";
str+=_name.print()+"->setTransient("+             // transient ?
     (_isTransient ? "true" : "false")+");\n";
for(i=0;i<column;i++) str+=" ";
if(isSuspended()) str+=_name.print()+"->suspended();\n"; // suspended ?
else str+=_name.print()+"->resume();\n";
return(str);
}

Strings Agent::userSave(int) const // virtual
{
return(Strings(""));
}

//-------- Modifiers -------------------------------------------------------//
// inline Model * Agent::accessModel(void)

// inline void Agent::setName(const AgentName & name)

// inline void Agent::reference(void)

void Agent::unReference(void)
{
if(!--_nbRefs)
  {
  if(_isAlive)
    {
    if(_isTransient) getSimul()->deleteAgent(this);
    }
  else delete this;
  }
}

void Agent::clearReferences(bool strongClear) // virtual
{
getSimul()->accessModel(_messageWord)->removeSensitive(this);
AttrMap::iterator itA=_attrMap.begin();
for(;!(itA==_attrMap.end());itA++)
  {
  (*itA).second.clearReferences(strongClear);
  }
while(_msgList.size())
  {
  if(!strongClear) _msgList.back()->unReference();
  _msgList.pop_back();
  }
}

// inline void Agent::setTransient(bool isTransient)

void Agent::kill(void)
{
_isAlive=false;
while(_msgList.size())
  {
  _msgList.back()->unReference();
  _msgList.pop_back();
  }
}

// inline void Agent::newBirth(void)

void Agent::suspend(void)
{
if(!_isSuspended)
  {
  _isSuspended=true;
  getSimul()->suspendAgent(this);
  }
}

void Agent::resume(void)
{
if(_isSuspended)
  {
  getSimul()->resumeAgent(this);
  _isSuspended=false;
  }
}

void Agent::setAttribute(const Attribute & attr,bool notify)
{
if(!getAttribute(attr.getName())) // not found --> insert
  {
  _attrMap.insert(AttrMap::value_type(attr.getName(),attr));
  if(notify) _notifyChange(_notifyAttributeWord,attr.getName());
  }
}

// inline AttrMap & Agent::accessAttrMap(void)

// inline Attribute * Agent::accessAttribute(const Word & name)

void Agent::setModule(ExecModule * newModule,bool notify)
{
bool used=false;
switch(newModule->getSpecialType())
  {
  case ExecModule::NORMAL:
  case ExecModule::BEHAVIOR:
    {
    ExecModMap::iterator it;
    it=_moduleMap.find(newModule->getName());
    if(it==_moduleMap.end())                 // not found --> insert
      {
      _moduleMap.insert(ExecModMap::value_type(newModule->getName(),
                                               newModule));
      used=true;
      newModule->reference();
      if(newModule->getSpecialType()==ExecModule::BEHAVIOR)
        {
        _mainModule=newModule;
        if(_mainModule->getType()!=ExecModule::PROTOTYPE)
          if(!_isSuspended) getSimul()->manageAgent(this);
        }
      }
    else                                     // found --> replace if needed
      {
      ExecModule * oldModule=(*it).second;
      if((oldModule!=newModule)&&                             // not the same
         ((newModule->getOwner()==this)||                     // and own module
          newModule->getModel()->isA(oldModule->getModel()))) // or more recent
        {
        used=true;
        (*it).second=newModule;
        newModule->reference();
        if(newModule->getSpecialType()==ExecModule::BEHAVIOR)
          {
          ExecModule * oldMain=_mainModule;
          _mainModule=newModule;
          if(_mainModule->getType()==ExecModule::PROTOTYPE)
            {
            if(oldMain->getType()!=ExecModule::PROTOTYPE)
              getSimul()->unManageAgent(this);
            }
          else
            {
            if(!_isSuspended&&(oldMain->getType()==ExecModule::PROTOTYPE))
              getSimul()->manageAgent(this);
            }
          }
        oldModule->unReference();
        }
      }
    }
    break;
  case ExecModule::CONSTRUCTOR:
    if((newModule!=_newModule)&&      // not the same
       (newModule->getOwner()==this)) // own module
      {
      if(_newModule) _newModule->unReference();
      _newModule=newModule;
      used=true;
      _newModule->reference();
      }
    break;
  case ExecModule::DESTRUCTOR:
    if((newModule!=_deleteModule)&&   // not the same
       (newModule->getOwner()==this)) // own module
      {
      if(_deleteModule) _deleteModule->unReference();
      _deleteModule=newModule;
      used=true;
      _deleteModule->reference();
      }
    break;
  default:
    break;
  }
if(notify&&used) _notifyChange(_notifyModuleWord,newModule->getName());
}

// inline ExecModMap & Agent::accessModuleMap(void)

// inline ExecModule * Agent::accessMainModule(void)

// inline ExecModule * Agent::accessNewModule(void)

// inline ExecModule * Agent::accessDeleteModule(void)

ExecModule * Agent::accessModule(const Word & name)
{
ExecModMap::iterator it=_moduleMap.find(name);
if(it==_moduleMap.end())
  {
  if(_newModule&&(_newModule->getName()==name)) return(_newModule);
  if(_model->getNewModule()&&(_model->getNewModule()->getName()==name))
    return(_model->accessNewModule());
  if(_deleteModule&&(_deleteModule->getName()==name)) return(_deleteModule);
  if(_model->getDeleteModule()&&(_model->getDeleteModule()->getName()==name))
    return(_model->accessDeleteModule());
  return((ExecModule *)0);
  }
return((*it).second);
}

//-------- Native Code -----------------------------------------------------//

// native string Object::name(void);
//
void Agent::stName(Context * context) // static
{
context->returnStrings(context->getOwner()->getName().print());
}

// native bool Object::isA(string className);
//
void Agent::stIsA(Context * context) // static
{
context->returnInteger(context->getOwner()->getModel()
                              ->isA(Word::create(context->getStrings(0))));
}

// native string Object::className(void);
//
void Agent::stClassName(Context * context) // static
{
context->returnStrings(context->getOwner()->getModel()
                              ->getName().getStrings());
}

// native int Object::suffix(void);
//
void Agent::stSuffix(Context * context) // static
{
context->returnInteger(context->getOwner()->getName().getInstNum());
}

// native void Object::suspend(void);
//
void Agent::stSuspend(Context * context) // static
{
context->accessOwner()->suspend();
}

// native void Object::resume(void);
//
void Agent::stResume(Context * context) // static
{
context->accessOwner()->resume();
}

// native bool Object::isSuspended(void);
//
void Agent::stIsSuspended(Context * context) // static
{
context->returnInteger(context->getOwner()->isSuspended());
}

// native void Object::setTransient(bool isTransient);
//
void Agent::stSetTransient(Context * context) // static
{
context->accessOwner()->setTransient((bool)context->getInteger(0));
}

// native bool Object::isTransient(void);
//
void Agent::stIsTransient(Context * context) // static
{
context->returnInteger(context->getOwner()->isTransient());
}

//-------- Communication ---------------------------------------------------//

// native void Agent::getSensitivity(string[] & msgTypes,
//                                   string[] & callbacks);
//
void Agent::stGetSensitivity(Context * context) // static
{
ModelVect msgTypes;
StringsVect callbacks;
Model * model=getSimul()->accessModel(_messageWord);
model->getSensitivity(context->accessOwner(),msgTypes,callbacks);
SubValueVect * vect=SubValue::getSubValueVect();
unsigned int i;
ValueType type(ValueType::STRING);
for(i=0;i<msgTypes.getSize();i++)
  {
  vect->pushBack(SubValue::getSubValue(type,context->accessLocal(0)));
  vect->accessBack()->setStrings(msgTypes[i]->getName().getStrings());
  }
context->accessLocal(0)->setValueVector(vect);
vect=SubValue::getSubValueVect();
for(i=0;i<callbacks.size();i++)
  {
  vect->pushBack(SubValue::getSubValue(type,context->accessLocal(1)));
  vect->accessBack()->setStrings(callbacks[i]);
  }
context->accessLocal(1)->setValueVector(vect);
}

// native void Agent::setSensitivity(string msgType,string callback);
//
void Agent::stSetSensitivity(Context * context) // static
{
Model * msg=getSimul()->accessModel(Word::create(context->getStrings(0)));
const ExecModule * module=context->accessOwner()
                          ->accessModule(Word::create(context->getStrings(1)));
if(msg&&module)
  {
  if(msg->isA(_messageWord))
    {
    if((module->getSpecialType()==ExecModule::NORMAL)&&
       (module->getNbParams()==1)&&
       module->getParam(0)->getType().canCastFrom(ValueType(msg)))
      {
      msg->setSensitivity(context->accessOwner(),module->getName());
      }
    else
      {
      Strings message="Module '"+module->getCompleteName()+
                      "' can't be used to react to '"+
                      msg->getName().getStrings()+"'";
      getSimul()->manageError(new Error(Error::WARNING,message));
      }
    }
  else
    {
    Strings message="Class '"+msg->getName().getStrings()+
                    "' can't be used as a 'Message'";
    getSimul()->manageError(new Error(Error::WARNING,message));
    }
  }
else
  {
  Strings message;
  if(module) 
    {
    message+="Class '"+context->getStrings(0)+"' is unknown";
    }
  else if(msg) 
    {
    message+="Module '"+context->getOwner()->getName().print()+
             "::"+context->getStrings(1)+"()' is unknown";
    }
  else
    {
    message+="Class '"+context->getStrings(0)+"' and module '"+
             context->getOwner()->getName().print()+
             "::"+context->getStrings(1)+"()' are unknown";
    }
  getSimul()->manageError(new Error(Error::WARNING,message));
  }
}

// native void Agent::cancelSensitivity(string msgType);
//
void Agent::stCancelSensitivity(Context * context) // static
{
Model * msg=getSimul()->accessModel(Word::create(context->getStrings(0)));
if(msg)
  {
  msg->cancelSensitivity(context->accessOwner());
  }
else
  {
  Strings message="Class '"+context->getStrings(0)+"' is unknown";
  getSimul()->manageError(new Error(Error::WARNING,message));
  }
}

// native int Agent::getNbMessages(void);
//
void Agent::stGetNbMessages(Context * context) // static
{
context->returnInteger(context->getOwner()->_msgList.size());
}

// native Message Agent::peekMessage(int pos);
//
void Agent::stPeekMessage(Context * context) // static
{
Agent * agt=context->accessOwner();
int pos=context->getInteger(0);
if((pos>=0)&&(pos<(int)agt->_msgList.size()))
  {
  AgentList::iterator it=agt->_msgList.begin();
  while(pos--) it++;
  context->returnAgent(*it,(*it)->accessModel());
  }
else
  {
  context->returnAgent((Agent *)0,getSimul()->accessModel(_messageWord));
  }
}

// native Message Agent::getNextMessage(void);
//
void Agent::stGetNextMessage(Context * context) // static
{
Agent * agt=context->accessOwner();
if(agt->_msgList.size())
  {
  Agent * message=agt->_msgList.front();
  context->returnAgent(message,message->accessModel());
  message->unReference();
  agt->_msgList.pop_front();
  }
else
  {
  context->returnAgent((Agent *)0,getSimul()->accessModel(_messageWord));
  }
}

// native void Agent::putBackMessage(Message message);
//
void Agent::stPutBackMessage(Context * context) // static
{
Agent * message=(Agent *)context->getAgent(0);
if(message)
  {
  message->reference();
  context->accessOwner()->_msgList.push_front(message);
  }
}

// native void Agent::asynchronousReception(Message message);
//
void Agent::stAsynchronousReception(Context * context) // static
{
Agent * message=(Agent *)context->getAgent(0);
Agent * receiver=context->accessOwner();
if(message)
  {
  message->reference();
  receiver->_msgList.push_back(message);
  ExecModule * module=receiver->accessModule(_onMessageWord);
  if(module->getType()!=ExecModule::PROTOTYPE)
    {
    getSimul()->executeModule(module,receiver);
    }
  }
}

// native void Message::sendTo(Agent receiver);
//
void Agent::stMsgSendTo(Context * context) // static
{
Agent * receiver=(Agent *)context->getAgent(0);
if(receiver&&receiver->isAlive())
  {
  context->accessOwner()->reference();
  receiver->_msgList.push_back(context->accessOwner());
  ExecModule * module=receiver->accessModule(_onMessageWord);
  if(module->getType()!=ExecModule::PROTOTYPE)
    {
    getSimul()->executeModule(module,receiver);
    }
  }
else
  {
  Strings message;
  if(receiver)
    message="Can't send to deleted Agent '"+receiver->getName().print()+"'";
  else
    message="Can't send to 'NONE'";
  getSimul()->manageError(new Error(Error::WARNING,message));
  }
}

// native void Message::broadcast(void);
//
void Agent::stMsgBroadcast(Context * context) // static
{
context->accessOwner()->accessModel()->broadcast(context->accessOwner());
}

void Agent::_notifyChange(const Word & method,const Word & param)
{
Model * model=getSimul()->accessModel(_objectDescriptionWord);
if(model)
  {
  Agent * descr;
  Execution * initExec;
  ExecModule * module;
  Value * value;
  StlVector<AgentName> agtVect;
  AgentMap::const_iterator it=model->getAgentMap().begin();
  for(;it!=model->getAgentMap().end();it++)
    agtVect.push_back((*it).first);
  while(agtVect.size())
    {
    descr=getSimul()->accessAgent(agtVect.back());
    agtVect.pop_back();
    if(descr&&
       (descr->getAttribute(_objectWord)->getValue()->getAgent()==this))
      {
      module=descr->accessModule(method);
      if(module->getType()!=ExecModule::PROTOTYPE)
        {
        initExec=Execution::getExecution(descr,ExecModule::NORMAL);
        value=TmpValue::getTmpValue(ValueType(ValueType::STRING));
        value->setStrings(param.getStrings());
        initExec->pushValue(value);
        getSimul()->executeModule(module,descr,initExec);
        Execution::relaxExecution(initExec);
        }
      }
    }
  }
}

