// @(#) ENIB/LI2 (c) - oRis - Version 2.3
//--------------------------------------------------------------------------//
// file        : attrib.cpp                                                 //
// description : source file for the ENIB/LI2 class Attribute               //
// author(s)   : F. Harrouet                                                //
// copyright   : Laboratoire d'Informatique Industrielle (LI2)              //
//               Ecole Nationale d'Ingenieurs de Brest (ENIB)               //
//               CP 30815 - 29608 Brest Cedex - FRANCE                      //
//               Tel: (33) 02.98.05.66.31 , Fax: (33) 02.98.05.66.29        //
//               e-mail: li2@enib.fr                                        //
//--------------------------------------------------------------------------//
// Copyright (c) 1996-2000                                                  //
// Laboratoire d'Informatique Industrielle (LI2)                            //
// Ecole Nationale d'Ingenieurs de Brest (ENIB)                             //
//                                                                          //
// Permission to use, copy, modify, distribute and sell this software       //
// and its documentation for any purpose is hereby granted without fee,     //
// provided that the above copyright notice appear in all copies and        //
// that both that copyright notice and this permission notice appear        //
// in supporting documentation.                                             //
// ENIB/LI2 makes no representations about the suitability of this          //
// software for any purpose.                                                //
// It is provided "as is" without express or implied warranty.              //
//--------------------------------------------------------------------------//

#include "oRis/Main/attrib.h"

//--------------------------------------------------------------------------//
// class Attribute                                                          //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
Attribute::Attribute(void)
 : BlockAllocatedObject(),
   _type(), _name(Word::create("")), _value(new AttrValue(ValueType())),
   _model((Model *)0), _owner((Agent *)0)
{
_value->reference();
}

Attribute::Attribute(const Attribute & attr)
 : BlockAllocatedObject(),
   _type(attr._type), _name(attr._name), _value(new AttrValue(attr._type)),
   _model(attr._model), _owner(attr._owner)
{
_value->reference();
*_value=*attr._value;
}

Attribute::Attribute(const Word & name,const ValueType & type,
                     Model * model,Agent * owner)
 : BlockAllocatedObject(),
   _type(type), _name(name), _value(new AttrValue(type)),
   _model(model), _owner(owner)
{
_value->reference();
}

Attribute::~Attribute(void)
{
_value->relax();
}

//-------- Inspectors ------------------------------------------------------//
// inline const Value * Attribute::getValue(void) const

// inline const Model * Attribute::getModel(void) const

// inline const Agent * Attribute::getOwner(void) const

// inline const Word & Attribute::getName(void) const

// inline const ValueType & Attribute::getType(void) const

//-------- Comparisons -----------------------------------------------------//

//-------- Input/Output ----------------------------------------------------//
ostream & operator<<(ostream & stream, const Attribute & attr)
{
return(stream << attr.print());
}

Strings Attribute::print(int column) const
{
Strings str;
for(int i=0;i<column;i++) str+=" ";
str+=_type.print()+" "+_name.getStrings()+" = "+_value->dump()+";\n";
return(str);
}

//-------- Modifiers -------------------------------------------------------//

void Attribute::clearReferences(bool strongClear)
{
_value->clearReferences(strongClear);
}

// inline Value * Attribute::accessValue(void)

// inline Model * Attribute::accessModel(void)

// inline Agent * Attribute::accessOwner(void)

// inline void setOwner(Agent * owner);

Attribute & Attribute::operator=(const Attribute & attr)
{
if(&attr!=this)
  {
  _name=attr._name;
  _type=attr._type;
  _value->relax();
  _value=new AttrValue(_type);
  _value->reference();
  *_value=*attr._value;
  _model=attr._model;
  _owner=attr._owner;
  }
return(*this);
}

