// @(#) ENIB/LI2 (c) - oRis - Version 2.3
//--------------------------------------------------------------------------//
// file        : cstLex.l                                                   //
// description : source file for the ENIB/LI2 class ConstantLexer           //
// author(s)   : F. Harrouet                                                //
// copyright   : Laboratoire d'Informatique Industrielle (LI2)              //
//               Ecole Nationale d'Ingenieurs de Brest (ENIB)               //
//               CP 30815 - 29608 Brest Cedex - FRANCE                      //
//               Tel: (33) 02.98.05.66.31 , Fax: (33) 02.98.05.66.29        //
//               e-mail: li2@enib.fr                                        //
//--------------------------------------------------------------------------//
// Copyright (c) 1996-2000                                                  //
// Laboratoire d'Informatique Industrielle (LI2)                            //
// Ecole Nationale d'Ingenieurs de Brest (ENIB)                             //
//                                                                          //
// Permission to use, copy, modify, distribute and sell this software       //
// and its documentation for any purpose is hereby granted without fee,     //
// provided that the above copyright notice appear in all copies and        //
// that both that copyright notice and this permission notice appear        //
// in supporting documentation.                                             //
// ENIB/LI2 makes no representations about the suitability of this          //
// software for any purpose.                                                //
// It is provided "as is" without express or implied warranty.              //
//--------------------------------------------------------------------------//

%{
#include <iostream>
#include <stdlib.h>
#include <string.h>

class ConstantParser;

#include "oRis/Main/simul.h"
#include "oRis/Main/errors.h"

#include "oRis/Main/cstLex.h"
#include "oRis/Main/cstParse.h"

#ifndef __MSDOS__
#define malloc(size) BlockAllocator::allocate(size);
#define free(ptr)    BlockAllocator::free(ptr);
#endif
%}

%name ConstantLexer
%define INHERIT : public BlockAllocatedObject

%define IOSTREAM
%define MEMBERS \
        Strings _curStrings; \
        int keyWord(ConstantParser * parser, \
                    const unsigned char * text,int code); \
        int integer(ConstantParser * parser, \
                    const unsigned char * text,int code); \
        int octal(ConstantParser * parser, \
                  const unsigned char * text,int code); \
        int hexa(ConstantParser * parser, \
                 const unsigned char * text,int code); \
        int real(ConstantParser * parser, \
                 const unsigned char * text,int code); \
        int string(ConstantParser * parser, \
                   const char * text,int code); \
        int agent(ConstantParser * parser, \
                  const unsigned char * text,int code); \
        int identifier(ConstantParser * parser, \
                       const unsigned char * text,int code); \
        void error(ConstantParser * parser,const unsigned char * text);

%define CONSTRUCTOR_PARAM istream * stream
%define CONSTRUCTOR_CODE yyin = stream;
%define LEX_PARAM ConstantParser * parser

%x strEnv

space		[ \t\r]+
nl		[\n]
anInteger	[1-9][0-9]*|0
octalInteger	0[0-7]+
hexaInteger	0[xX][0-9A-Fa-f]+
r1		[0-9]*\.[0-9]+
r2		[0-9]+\.[0-9]*
r_exp		[eE][-+]?[0-9]+
aReal		{r1}|{r2}|{r1}{r_exp}|{r2}{r_exp}|{anInteger}{r_exp}
anAgent		[A-Za-z_][0-9a-zA-Z_]*\.[0-9]+
anIdent		[A-Za-z_][0-9a-zA-Z_]*

%%

 //----------------------------------------------------------------------//

"("		{return(keyWord(parser,yytext,ConstantParser::LP));}
")"		{return(keyWord(parser,yytext,ConstantParser::RP));}
"["		{return(keyWord(parser,yytext,ConstantParser::LH));}
"]"		{return(keyWord(parser,yytext,ConstantParser::RH));}
","		{return(keyWord(parser,yytext,ConstantParser::COMMA));}
"+"       	{return(keyWord(parser,yytext,ConstantParser::PLUS));}
"-"       	{return(keyWord(parser,yytext,ConstantParser::MINUS));}

"int"		{return(keyWord(parser,yytext,ConstantParser::INTEGER));}
"float"		{return(keyWord(parser,yytext,ConstantParser::REAL));}
"string"	{return(keyWord(parser,yytext,ConstantParser::STRING));}
"NONE"       	{return(keyWord(parser,yytext,ConstantParser::NONE));}

{space}		{parser->store((const char *)yytext);}
{nl}		{parser->nextLine();}
{anIdent}	{return(identifier(parser,yytext,ConstantParser::IDENT));}

"\""		{_curStrings=""; BEGIN(strEnv);}
<strEnv>"\""	{
                BEGIN(INITIAL);
                return(string(parser,_curStrings,ConstantParser::STRING_VAL));
                }
<strEnv>"\\a"	{_curStrings+="\a";}
<strEnv>"\\b"	{_curStrings+="\b";}
<strEnv>"\\f"	{_curStrings+="\f";}
<strEnv>"\\n"	{_curStrings+="\n";}
<strEnv>"\\r"	{_curStrings+="\r";}
<strEnv>"\\t"	{_curStrings+="\t";}
<strEnv>"\\v"	{_curStrings+="\v";}
<strEnv>"\\\\"	{_curStrings+="\\";}
<strEnv>"\\\""	{_curStrings+="\"";}
<strEnv>"\n"	{_curStrings+="\n";parser->nextLine();}
<strEnv>"\\\n"	{_curStrings+="\n";parser->nextLine();}
<strEnv>"\\".	{_curStrings+=yytext[1];}
<strEnv><<EOF>>	{
                BEGIN(INITIAL);
                return(0);
                }
<strEnv>.	{_curStrings+=(const char *)yytext;}

{aReal}  	{return(real(parser,yytext,ConstantParser::REAL_VAL));}
{anInteger}	{return(integer(parser,yytext,ConstantParser::INTEGER_VAL));}
{octalInteger}	{return(octal(parser,yytext,ConstantParser::INTEGER_VAL));}
{hexaInteger}	{return(hexa(parser,yytext,ConstantParser::INTEGER_VAL));}
{anAgent}  	{return(agent(parser,yytext,ConstantParser::AGENT_VAL));}

.		{error(parser,yytext);}

 //----------------------------------------------------------------------//

%%

int ConstantLexer::keyWord(ConstantParser * parser,
                           const unsigned char * text,int code)
{
parser->store((const char *)text);
parser->yylval.setType(ValueType(ValueType::STRING));
parser->yylval.setStrings((const char *)text);
return(code);
}

int ConstantLexer::integer(ConstantParser * parser,
                           const unsigned char * text,int code)
{
parser->store((const char *)text);
parser->yylval.setType(ValueType(ValueType::INTEGER));
parser->yylval.setInteger(atol((const char *)text));
return(code);
}

int ConstantLexer::octal(ConstantParser * parser,
                         const unsigned char * text,int code)
{
parser->store((const char *)text);
parser->yylval.setType(ValueType(ValueType::INTEGER));
long nb;
sscanf((const char *)text,"%lo",&nb);
parser->yylval.setInteger(nb);
return(code);
}

int ConstantLexer::hexa(ConstantParser * parser,
                        const unsigned char * text,int code)
{
parser->store((const char *)text);
parser->yylval.setType(ValueType(ValueType::INTEGER));
long nb;
sscanf((const char *)text,"%lx",&nb);
parser->yylval.setInteger(nb);
return(code);
}

int ConstantLexer::real(ConstantParser * parser,
                        const unsigned char * text,int code)
{
parser->store((const char *)text);
parser->yylval.setType(ValueType(ValueType::REAL));
parser->yylval.setReal(atof((const char *)text));
return(code);
}

int ConstantLexer::string(ConstantParser * parser,
                          const char * text,int code)
{
parser->store(Strings(text).dump());
parser->yylval.setType(ValueType(ValueType::STRING));
parser->yylval.setStrings(text);
return(code);
}

int ConstantLexer::agent(ConstantParser * parser,
                         const unsigned char * text,int code)
{
parser->store((const char *)text);
AgentName agtName((const char *)text);
Agent * agent=getSimul()->accessAgent(agtName);
if(agent&&agent->isAlive())
  {
  parser->yylval.setType(ValueType(agent->accessModel()));
  parser->yylval.setAgent(agent);
  return(code);
  }
else
  {
  Strings message="Object '"+
                  agtName.print()+
                  "' is unknown";
  parser->manageError(new Error(Error::SEMANTICS,message));
  return(0);
  }
}

int ConstantLexer::identifier(ConstantParser * parser,
                              const unsigned char * text,int code)
{
parser->store((const char *)text);
parser->yylval.setType(ValueType(ValueType::STRING));
parser->yylval.setStrings((const char *)text);
if(getSimul()->getModel(Word::create(parser->yylval.getStrings())))
  return(ConstantParser::MODELTYPE);
else return(code);
}

void ConstantLexer::error(ConstantParser * parser,const unsigned char * text)
{
parser->store((const char *)text);
parser->yyerror("lexical error");
}

