// @(#) ENIB/LI2 (c) - oRis - Version 2.3
//--------------------------------------------------------------------------//
// file        : cstParse.y                                                 //
// description : source file for the ENIB/LI2 class ConstantParser          //
// author(s)   : F. Harrouet                                                //
// copyright   : Laboratoire d'Informatique Industrielle (LI2)              //
//               Ecole Nationale d'Ingenieurs de Brest (ENIB)               //
//               CP 30815 - 29608 Brest Cedex - FRANCE                      //
//               Tel: (33) 02.98.05.66.31 , Fax: (33) 02.98.05.66.29        //
//               e-mail: li2@enib.fr                                        //
//--------------------------------------------------------------------------//
// Copyright (c) 1996-2000                                                  //
// Laboratoire d'Informatique Industrielle (LI2)                            //
// Ecole Nationale d'Ingenieurs de Brest (ENIB)                             //
//                                                                          //
// Permission to use, copy, modify, distribute and sell this software       //
// and its documentation for any purpose is hereby granted without fee,     //
// provided that the above copyright notice appear in all copies and        //
// that both that copyright notice and this permission notice appear        //
// in supporting documentation.                                             //
// ENIB/LI2 makes no representations about the suitability of this          //
// software for any purpose.                                                //
// It is provided "as is" without express or implied warranty.              //
//--------------------------------------------------------------------------//

%{
#include <fstream>
#include <string.h>

class ConstantParser;

#include "oRis/Main/simul.h"
#include "oRis/Main/errors.h"

#include "oRis/Main/cstLex.h"

#ifndef __MSDOS__
#define malloc(size) BlockAllocator::allocate(size);
#define free(ptr)    BlockAllocator::free(ptr);
#endif
%}

%name ConstantParser
%define INHERIT : public BlockAllocatedObject

%define MEMBERS \
        ConstantLexer * _scanner; \
        istream * _stream; \
        int _lineNum; \
        Strings _line; \
        ValueType _curType; \
        int _curDepth; \
        bool _fail; \
        Value * _result; \
        void init(istream * stream,Value * result); \
        virtual ~ConstantParser(void); \
        bool fail(void) const; \
        void manageError(Error * error); \
        void nextLine(void); \
        void store(const char * text);

%define CONSTRUCTOR_PARAM \
        istream * stream, \
        Value * result

%define CONSTRUCTOR_CODE init(stream,result);

%define LEX_BODY {return(_scanner->yylex(this));}

//-------- Tokens ----------------------------------------------------------//

%token LP RP LH RH COMMA PLUS MINUS INTEGER REAL STRING NONE IDENT MODELTYPE
%token STRING_VAL REAL_VAL INTEGER_VAL AGENT_VAL

//-------- Token types -----------------------------------------------------//

%define STYPE Value

%start constant

%%

//--------------------------------------------------------------------------//

constant	: constantValue
                    {
                    if(!_fail) _result->affectCast($1);
                    }
;

cast		: LP simpleType hooks RP
;

hooks           : hooks LH RH
                    {
                    _curType.setDepth(_curType.getDepth()+1);
                    }
                | // empty
                    {
                    _curType.setDepth(0);
                    }
;

simpleType      : INTEGER
                    {
                    _curType=ValueType(ValueType::INTEGER);
                    }
                | REAL
                    {
                    _curType=ValueType(ValueType::REAL);
                    }
                | STRING
                    {
                    _curType=ValueType(ValueType::STRING);
                    }
                | MODELTYPE
                    {
                    Model * model=getSimul()->accessModel(
                                  Word::create($1.getStrings()));
                    _curType=ValueType(model);
                    }
;

constantValue	: constantElem
                    {
                    ValueType type($1.getType());
                    type.setDepth(type.getDepth()+_curDepth);
                    if(type.canCastTo(_result->getType()))
                      {
                      if(type!=_result->getType())
                        {
                        type=_result->getType();
                        type.setDepth(type.getDepth()-_curDepth);
                        $1.cast(type);
                        }
                      $$=$1;
                      }
                    else
                      {
                      Strings message="Type '"+
                                      type.print()+
                                      "' found in a '"+
                                      _result->getType().print()+
                                      "'";
                      manageError(new Error(Error::SEMANTICS,message));
                      type.setDepth(_result->getType().getDepth()-_curDepth);
                      if(type.getDepth()<0) type.setDepth(0);
                      Value value(type);
                      $$=value;
                      }
                    }
;

constantElem	: INTEGER_VAL
                    {
                    $$.setType(ValueType(ValueType::INTEGER));
                    $$.setInteger($1.getInteger());
                    }
		| PLUS INTEGER_VAL
                    {
                    $$.setType(ValueType(ValueType::INTEGER));
                    $$.setInteger($2.getInteger());
                    }
		| MINUS INTEGER_VAL
                    {
                    $$.setType(ValueType(ValueType::INTEGER));
                    $$.setInteger(-$2.getInteger());
                    }
		| REAL_VAL
                    {
                    $$.setType(ValueType(ValueType::REAL));
                    $$.setReal($1.getReal());
                    }
		| PLUS REAL_VAL
                    {
                    $$.setType(ValueType(ValueType::REAL));
                    $$.setReal($2.getReal());
                    }
		| MINUS REAL_VAL
                    {
                    $$.setType(ValueType(ValueType::REAL));
                    $$.setReal(-$2.getReal());
                    }
		| STRING_VAL
                    {
                    $$.setType(ValueType(ValueType::STRING));
                    $$.setStrings($1.getStrings());
                    }
		| AGENT_VAL
                    {
                    $$.setType($1.getType());
                    $$.setAgent($1.getAgent());
                    }
		| NONE
                    {
                    $$.setType(Model::getBaseModel());
                    $$.setAgent((Agent *)0);
                    }
		| cast NONE
                    {
                    if(_curType.canCastTo(ValueType::AGENT))
                      {
                      Value value(_curType);
                      $$=value;
                      }
                    else
                      {
                      Strings message="Type '"+
                                      _curType.print()+
                                      "' can't be given to '"+
                                      $2.getStrings()+
                                      "'";
                      manageError(new Error(Error::SEMANTICS,message));
                      Value value(Model::getBaseModel());
                      $$=value;
                      }
                    }
		| cast LH RH
                    {
                    if(_curType.getDepth())
                      {
                      Value value(_curType);
                      $$=value;
                      }
                    else
                      {
                      Strings message="Type '"+
                                      _curType.print()+
                                      "' can't be given to '"+
                                      $2.getStrings()+
                                      $3.getStrings()+
                                      "'";
                      manageError(new Error(Error::SEMANTICS,message));
                      _curType.setDepth(1);
                      Value value(_curType);
                      $$=value;
                      }
                    }
		| LH
                    {
                    _curDepth++;
                    }
                  valueList RH
                    {
                    _curDepth--;
                    $$=$3;
                    }
; 

valueList	: constantValue
                    {
                    ValueType type(_result->getType());
                    type.setDepth($1.getType().getDepth()+1);
                    $$.setType(type);
                    $$.pushBack($1);
                    }
		| valueList COMMA constantValue
                    {
                    $$=$1;
                    $$.pushBack($3);
                    }
;

%%

//----------------------------------------------------------------------//

void ConstantParser::init(istream * stream,Value * result)
{
_stream=stream;
_result=result;
_lineNum=1;
_scanner=new ConstantLexer(_stream);
_curDepth=0;
_fail=false;
}

ConstantParser::~ConstantParser(void)
{
if(_scanner) delete _scanner;
}

bool ConstantParser::fail(void) const
{
return(_fail);
}

void ConstantParser::yyerror(char * text)
{
Strings message;
if(!::strcmp(text,"parse error")) message="Syntax error";
else message=text;
if(_line.len())
  message+="\n"+_line+"<---";
manageError(new Error(Error::SYNTAX,message));
}

void ConstantParser::manageError(Error * error)
{
if(error)
  {
  _fail=true;
  error->setLineNumber(_lineNum);
  // getSimul()->manageError(error);
  delete error;
  }
}

void ConstantParser::nextLine(void)
{
_lineNum++;
_line=Strings("");
}

void ConstantParser::store(const char * text)
{
_line+=Strings(text);
}

