// @(#) ENIB/LI2 (c) - oRis - Version 2.3
//--------------------------------------------------------------------------//
// file        : dLibInfo.cpp                                               //
// description : source file for the ENIB/LI2 class DynLibInfo              //
// author(s)   : F. Harrouet                                                //
// copyright   : Laboratoire d'Informatique Industrielle (LI2)              //
//               Ecole Nationale d'Ingenieurs de Brest (ENIB)               //
//               CP 30815 - 29608 Brest Cedex - FRANCE                      //
//               Tel: (33) 02.98.05.66.31 , Fax: (33) 02.98.05.66.29        //
//               e-mail: li2@enib.fr                                        //
//--------------------------------------------------------------------------//
// Copyright (c) 1996-2000                                                  //
// Laboratoire d'Informatique Industrielle (LI2)                            //
// Ecole Nationale d'Ingenieurs de Brest (ENIB)                             //
//                                                                          //
// Permission to use, copy, modify, distribute and sell this software       //
// and its documentation for any purpose is hereby granted without fee,     //
// provided that the above copyright notice appear in all copies and        //
// that both that copyright notice and this permission notice appear        //
// in supporting documentation.                                             //
// ENIB/LI2 makes no representations about the suitability of this          //
// software for any purpose.                                                //
// It is provided "as is" without express or implied warranty.              //
//--------------------------------------------------------------------------//


#include "oRis/Main/dLibInfo.h"

#include "Utility/system.h"

//--------------------------------------------------------------------------//
// class DynLibInfo                                                         //
//--------------------------------------------------------------------------//

//-------- Static Methods --------------------------------------------------//
Strings DynLibInfo::buildWrapName(const Strings & completeName) // static
{
Strings wrapName;
int lim1=completeName.lookUp("::");
if(lim1<0) return(wrapName);
int lim2=completeName.lookUp("()");
if((lim2<0)||(lim2<lim1)||(lim2<(int)completeName.len()-2)) return(wrapName);
wrapName+="wrap_";
wrapName+=completeName.subString(0,lim1);
wrapName+="_";
wrapName+=completeName.subString(lim1+2,lim2-(lim1+2));
return(wrapName);
}

//-------- Allocators ------------------------------------------------------//
DynLibInfo::DynLibInfo(const Strings & libName)
 : BlockAllocatedObject(),
   _libName(libName), _handle((void *)0),
   _openFunction((void (*)(void))0),
   _closeFunction((void (*)(void))0),
   _agtFunction((Agent * (*)(Model *,long))0),
   _modFunction((ExternFunction (*)(const Strings &))0)
{
#ifdef __MSDOS__
  _handle=System::openDynLib(_libName+".dll");
#else
# ifdef __APPLE__
    _handle=System::openDynLib("lib"+_libName+".dylib");
# else
    _handle=System::openDynLib("lib"+_libName+".so");
# endif
#endif
if(_handle)
  {
  _openFunction=(void (*)(void))
                System::getDynLibSymbol(_handle,"openingLibrary");
  _closeFunction=(void (*)(void))
                  System::getDynLibSymbol(_handle,"closingLibrary");
  _agtFunction=(Agent * (*)(Model *,long))
               System::getDynLibSymbol(_handle,"getNewAgent");
  _modFunction=(ExternFunction (*)(const Strings &))
                System::getDynLibSymbol(_handle,"getExternFunction");
  }
if(_openFunction) (*_openFunction)();
}

DynLibInfo::~DynLibInfo(void)
{
if(_closeFunction) (*_closeFunction)();
#ifndef __sgi                             // sometimes there are problems
if(_handle) System::closeDynLib(_handle); // after reloading libraries on SGI
#endif
}

//-------- Inspectors ------------------------------------------------------//
// inline const Strings & DynLibInfo::getLibName(void) const

// inline bool DynLibInfo::fail(void) const

// inline Agent * DynLibInfo::getNewAgent(Model * model,long instNum) const

ExternFunction DynLibInfo::getExternFunction(const Strings & completeName,
                                             const Strings & wrapName) const
{
ExternFunction fnct=(ExternFunction)0;
if(wrapName.len())
  fnct=(ExternFunction)System::getDynLibSymbol(_handle,wrapName);
if(!fnct&&_modFunction) fnct=(*_modFunction)(completeName);
return(fnct);
}

