// @(#) ENIB/LI2 (c) - oRis - Version 2.3
//--------------------------------------------------------------------------//
// file        : exec.cpp                                                   //
// description : source file for the ENIB/LI2 class Execution               //
// author(s)   : F. Harrouet                                                //
// copyright   : Laboratoire d'Informatique Industrielle (LI2)              //
//               Ecole Nationale d'Ingenieurs de Brest (ENIB)               //
//               CP 30815 - 29608 Brest Cedex - FRANCE                      //
//               Tel: (33) 02.98.05.66.31 , Fax: (33) 02.98.05.66.29        //
//               e-mail: li2@enib.fr                                        //
//--------------------------------------------------------------------------//
// Copyright (c) 1996-2000                                                  //
// Laboratoire d'Informatique Industrielle (LI2)                            //
// Ecole Nationale d'Ingenieurs de Brest (ENIB)                             //
//                                                                          //
// Permission to use, copy, modify, distribute and sell this software       //
// and its documentation for any purpose is hereby granted without fee,     //
// provided that the above copyright notice appear in all copies and        //
// that both that copyright notice and this permission notice appear        //
// in supporting documentation.                                             //
// ENIB/LI2 makes no representations about the suitability of this          //
// software for any purpose.                                                //
// It is provided "as is" without express or implied warranty.              //
//------------------------------------------------------------------------- //

#include "oRis/Main/exec.h"
#include "oRis/Main/context.h"
#include "oRis/Main/simul.h"
#include "oRis/Main/errors.h"
#include "oRis/Main/thrdNtv.h"

//--------------------------------------------------------------------------//
// class Execution                                                          //
//--------------------------------------------------------------------------//

ExecutionVect Execution::_execStock;
long Execution::_lastID=0L;

//-------- Static methods --------------------------------------------------//
Execution * Execution::getExecution(Agent * owner,
                                    ExecModule::SpecialType type) // static
{
Execution * execution;
if(Execution::_execStock.getSize())
  execution=Execution::_execStock.readPopBack();
else
  execution=new Execution;
execution->_id=++_lastID;
execution->_owner=owner;
if(execution->_owner) execution->_owner->reference();
execution->_type=type;
execution->_canRestart=(type==ExecModule::BEHAVIOR);
execution->_criticalLevel=0;
execution->_goOn=false;
execution->_isFinished=false;
execution->_isSuspended=false;
execution->_isAborted=false;
execution->_depth=0;
return(execution);
}

void Execution::relaxExecution(Execution * execution) // static
{
if(execution->_type==ExecModule::DESTRUCTOR)
  {
  getSimul()->removeAgent(execution->_owner);
  }
if(execution->_owner)
  {
  execution->_owner->unReference();
  }
execution->_canRestart=false;
execution->abort(true);
Execution::_execStock.pushBack(execution);
}

void Execution::clearExecutionStock(void) // static
{
while(Execution::_execStock.getSize())
  {
  delete Execution::_execStock.readPopBack();
  }
Execution::_lastID=0L;
}

//-------- Allocators ------------------------------------------------------//
Execution::Execution(void)
 : BlockAllocatedObject(),
   _owner((Agent *)0), _type(ExecModule::NORMAL), _canRestart(false),
   _values(), _lastContext((Context *)0), _criticalLevel(0), _goOn(false),
   _isFinished(false), _isSuspended(false), _isAborted(false), _depth(0),
   _thNative((ThreadedNative *)0)
{}

Execution::~Execution(void)
{}

//-------- Inspectors ------------------------------------------------------//
// inline long Execution::getID() const

// inline const Agent * Execution::getOwner(void) const

// inline const Context * Execution::getContext(void) const

// inline bool Execution::goOn(void) const

// inline bool Execution::isFinished(void) const

// inline bool Execution::isAborted(void) const

// inline ExecModule::SpecialType Execution::getType(void) const

// inline bool Execution::canRestart(void) const

// inline bool Execution::isSuspended(void) const

// inline int Execution::getCriticalLevel(void) const

// inline int Execution::getStackSize(void) const

// inline const Value * Execution::getValue(int rank) const

// inline int Execution::getDepth(void) const

void Execution::getStack(AgentVect & callers,StringsVect & modules,
                         int depth) const
{
if(_lastContext) _lastContext->getStack(callers,modules,depth);
}

bool Execution::usesAgent(const Agent * agent) const
{
if((_owner==agent)&&(_type==ExecModule::BEHAVIOR)&&!started())
  {
  return(false); // main has not started yet
  }
bool endReached=true;
const Context * context=_lastContext;
while(context)
  {
  if(!context->endReached()) endReached=false;
  if((context->getOwner()==agent)&&!endReached) return(true);
  context=context->getPrevious();
  }
return(false);
}

bool Execution::started(void) const
{
const Context * context=_lastContext;
while(context)
  {
  if(context->started()) return(true);
  context=context->getPrevious();
  }
return(false);
}

bool Execution::endReached(void) const
{
const Context * context=_lastContext;
while(context)
  {
  if(!context->endReached()) return(false);
  context=context->getPrevious();
  }
return(true);
}

// inline const ThreadedNative * Execution::getThreadedNative(void) const

//-------- Comparisons -----------------------------------------------------//

//-------- Input/output ----------------------------------------------------//
Strings Execution::print(void) const
{
Strings str;
if(_owner) str+=_owner->getName().print();
str+="::";
const Context * context=_lastContext;
while(context)
  {
  if(!context->getPrevious())
    {
    if(context->getModule())
      str+=context->getModule()->getName().getStrings();
    }
  context=context->getPrevious();
  }
str+="("+Strings(_id)+")";
return(str);
}

Strings Execution::printStack(void) const
{
if(_lastContext) return(_lastContext->printStack());
else return(Strings(""));
}

//-------- Modifiers -------------------------------------------------------//
// inline Agent * Execution::accessOwner(void)

// inline Context * Execution::accessContext(void)

// inline void Execution::setGoOn(bool goOn)

// inline void Execution::mustDisappear(void)

// inline void Execution::avoidRestart(void)

// inline void Execution::allowRestart(void)

// inline void Execution::setSuspended(bool isSuspended)

// inline void Execution::beginCritical(void)

void Execution::endCritical(int level)
{
_criticalLevel-=level;
}

void Execution::enterContext(ExecModule * module,Agent * contextOwner)
{
_lastContext=Context::getContext(module,contextOwner,_lastContext);
if(module->isCritical()) beginCritical();
_lastContext->enterExecution(this);
if(++_depth>getSimul()->getMaxDepth())
  {
  Strings message="Recursion too deep ("+
                  Strings(getSimul()->getMaxDepth())+
                  " reached)";
  getSimul()->manageError(new Error(Error::RUNTIME,message));
  }
}

void Execution::leaveContext(void)
{
--_depth;
Context * lastContext=_lastContext;
_lastContext=lastContext->accessPrevious();
if(lastContext->getModule()->isCritical()) endCritical();
if(!_lastContext)
  {
  if(_canRestart)
    {
    enterContext(_owner->accessMainModule(),_owner);
    _goOn=false;
    }
  else mustDisappear();
  }
lastContext->leaveExecution();
Context::relaxContext(lastContext);
}

void Execution::abort(bool canDestroy)
{
if(_thNative)
  {
  if(_thNative->taskEnded()) ThreadedNative::relaxThreadedNative(_thNative);
  else delete _thNative;
  _thNative=(ThreadedNative *)0;
  }
if(canDestroy)
  {
  if(_canRestart)
    {
    _canRestart=false;
    while(_lastContext) leaveContext();
    while(_values.getSize())
      {
      _values.readPopBack()->relax();
      }
    _isFinished=false;
    _canRestart=true;
    enterContext(_owner->accessMainModule(),_owner);
    _lastContext->moveCounterToEnd(); // avoid re-execution of same code
    }
  else
    {
    while(_lastContext) leaveContext();
    while(_values.getSize())
      {
      _values.readPopBack()->relax();
      }
    _isFinished=true;
    }
  }
else _isFinished=true;
_isAborted=true;
_goOn=false;
}

// inline void Execution::cancelAbort(void)

// inline void Execution::pushValue(Value * value)

// inline Value * Execution::popValue(void)

// inline Value * Execution::lastValue(void)

// inline Value * Execution::accessValue(int rank)

// inline void Execution::setThreadedNative(ThreadedNative * thNative)

// inline ThreadedNative * Execution::accessThreadedNative(void)

