// @(#) ENIB/LI2 (c) - oRis - Version 2.3
//--------------------------------------------------------------------------//
// file        : extMod.cpp                                                 //
// description : source file for the ENIB/LI2 class ExternModule            //
// author(s)   : F. Harrouet                                                //
// copyright   : Laboratoire d'Informatique Industrielle (LI2)              //
//               Ecole Nationale d'Ingenieurs de Brest (ENIB)               //
//               CP 30815 - 29608 Brest Cedex - FRANCE                      //
//               Tel: (33) 02.98.05.66.31 , Fax: (33) 02.98.05.66.29        //
//               e-mail: li2@enib.fr                                        //
//--------------------------------------------------------------------------//
// Copyright (c) 1996-2000                                                  //
// Laboratoire d'Informatique Industrielle (LI2)                            //
// Ecole Nationale d'Ingenieurs de Brest (ENIB)                             //
//                                                                          //
// Permission to use, copy, modify, distribute and sell this software       //
// and its documentation for any purpose is hereby granted without fee,     //
// provided that the above copyright notice appear in all copies and        //
// that both that copyright notice and this permission notice appear        //
// in supporting documentation.                                             //
// ENIB/LI2 makes no representations about the suitability of this          //
// software for any purpose.                                                //
// It is provided "as is" without express or implied warranty.              //
//------------------------------------------------------------------------- //

#include "oRis/Main/extMod.h"
#include "oRis/Main/simul.h"
#include "oRis/Codes/moduleCd.h"
#include "oRis/Codes/utilCd.h"

//---------------------------------------------------------------------------//
// class ExternModule                                                        //
//---------------------------------------------------------------------------//

//-------- Allocators -------------------------------------------------------//
ExternModule::ExternModule(const Word & name,
                           ExecModule::SpecialType specialType,
                           const ValueType & valueType,
                           Model * model,Agent * owner,
                           const Strings & nativeName)
 : ExecModule(name,specialType,valueType,model,owner),
   _nativeName(nativeName)
{
if(!_nativeName.len()) _nativeName=getCompleteName();
_function=getSimul()->getExternFunction(_nativeName);
if(_function)
  {
  _codeVect=new CodeVect;
  _codeVect->pushBack(new NativeCode((SmaNode *)0,_function));
  _codeVect->pushBack(new ReturnCode((SmaNode *)0,0,0));
  }
else mustDisappear()=true;
}

ExternModule::ExternModule(const ExternModule & module)
 : ExecModule(module), _function(module._function),
   _nativeName(module._nativeName)
{
if(_function)
  {
  _codeVect=new CodeVect;
  _codeVect->pushBack(new NativeCode((SmaNode *)0,_function));
  _codeVect->pushBack(new ReturnCode((SmaNode *)0,0,0));
  }
else mustDisappear()=true;
}

ExecModule * ExternModule::duplicate(void) const // virtual
{
return(new ExternModule(*this));
}

ExternModule::~ExternModule(void)
{}

//-------- Inspectors ------------------------------------------------------//
// inline ExecModule::ModuleType ExternModule::getType(void) const // virtual

const Strings & ExternModule::getNativeName(void) const
{
return(_nativeName);
}

//-------- Comparisons -----------------------------------------------------//

//-------- Input/Output ----------------------------------------------------//
Strings ExternModule::print(int column) const // virtual
{
Strings str;
for(int j=0;j<column;j++) str+=" ";
str+="native ";
if(_nativeName!=getCompleteName())
  str+=_nativeName.dump()+" ";
str+=getResultType().print()+" ";
if(getOwner()) str+=getOwner()->getName().print()+"::";
else if(getModel()) str+=getModel()->getName().getStrings()+"::";
str+=getName().getStrings()+_printParams()+";\n";
return(str);
}

//-------- Modifiers -------------------------------------------------------//

