// @(#) ENIB/LI2 (c) - oRis - Version 2.3
//--------------------------------------------------------------------------//
// file        : simul.cpp                                                  //
// description : source file for the ENIB/LI2 class Simulator               //
// author(s)   : F. Harrouet                                                //
// copyright   : Laboratoire d'Informatique Industrielle (LI2)              //
//               Ecole Nationale d'Ingenieurs de Brest (ENIB)               //
//               CP 30815 - 29608 Brest Cedex - FRANCE                      //
//               Tel: (33) 02.98.05.66.31 , Fax: (33) 02.98.05.66.29        //
//               e-mail: li2@enib.fr                                        //
//--------------------------------------------------------------------------//
// Copyright (c) 1996-2000                                                  //
// Laboratoire d'Informatique Industrielle (LI2)                            //
// Ecole Nationale d'Ingenieurs de Brest (ENIB)                             //
//                                                                          //
// Permission to use, copy, modify, distribute and sell this software       //
// and its documentation for any purpose is hereby granted without fee,     //
// provided that the above copyright notice appear in all copies and        //
// that both that copyright notice and this permission notice appear        //
// in supporting documentation.                                             //
// ENIB/LI2 makes no representations about the suitability of this          //
// software for any purpose.                                                //
// It is provided "as is" without express or implied warranty.              //
//--------------------------------------------------------------------------//

#include "oRis/Main/simul.h"
#include "oRis/Main/errors.h"
#include "oRis/Main/textOut.h"
#include "oRis/Main/context.h"
#include "oRis/Main/thrdNtv.h"


class SmaParser;
#include "oRis/Nodes/smaNd.h"
#include "oRis/Main/smaLex.h"
#include "oRis/Main/smaParse.h"

class ConstantParser;
#include "oRis/Main/cstLex.h"
#include "oRis/Main/cstParse.h"

#ifndef __MSDOS__
#include <sstream>
#else
#include <strstrea.h>
#endif

#include <time.h>
#include <string.h>
#include <fstream>

#include "Utility/system.h"
#include "Utility/timer.h"

#ifdef __MSDOS__
#define EXPORT_PREFIX extern "C" __declspec(dllexport)
#include "Main/ArApplication.h"
EXPORT_PREFIX ExternFunction getExternAReViFunction(const Strings & name);
EXPORT_PREFIX ExternFunction getExternAttrCbFunction(const Strings & name);
EXPORT_PREFIX ExternFunction getExternReflFunction(const Strings & name);
EXPORT_PREFIX ExternFunction getExternCntnrFunction(const Strings & name);
EXPORT_PREFIX ExternFunction getExternConvFunction(const Strings & name);
EXPORT_PREFIX ExternFunction getExternFilesFunction(const Strings & name);
EXPORT_PREFIX ExternFunction getExternFuzzyFunction(const Strings & name);
EXPORT_PREFIX ExternFunction getExternIpcFunction(const Strings & name);
EXPORT_PREFIX ExternFunction getExternMathsFunction(const Strings & name);
EXPORT_PREFIX ExternFunction getExternORisBaseFunction(const Strings & name);
EXPORT_PREFIX ExternFunction getExternSemaFunction(const Strings & name);
EXPORT_PREFIX ExternFunction getExternGuiFunction(const Strings & name);
EXPORT_PREFIX ExternFunction getExternTcpUdpFunction(const Strings & name);
EXPORT_PREFIX ExternFunction getExternObject2dFunction(const Strings & name);
#endif

//--------------------------------------------------------------------------//
// class Simulator                                                          //
//--------------------------------------------------------------------------//

Simulator * Simulator::_simul=(Simulator *)0;
struct sigaction Simulator::_newTimerAction;
struct sigaction Simulator::_oldTimerAction;
Word Simulator::_notifyClassWord(Word::create("notifyClass"));
Word Simulator::_notifyModuleWord(Word::create("notifyModule"));
Word Simulator::_globalDescriptionWord(Word::create("GlobalDescription"));

//-------- Friend methods --------------------------------------------------//
// inline Simulator * getSimul(void) // friend

//-------- Allocators ------------------------------------------------------//
Simulator::Simulator(const StringsVect & commandLine,
                     TextOutput * output,TextOutput * err)
 : BlockAllocatedObject(),
   _dynLibs(), _packages(), _replaceMap(), _pickingMode(Simulator::RANDOM),
   _interval(0), _clock(0), _readlnMutex((SimpleThreadMutex *)0),
   _maxDepth(100), _commandLine(commandLine),
   _installDir(commandLine[0].dirname().dirname()),
   _output(output), _err(err), _nbErrors(0),
   _isBad(false), _pathVect(), _agentMap(), _deletedMap(),
   _execVectFrom(), _execVectTo(), _execListSuspend(), _curExecutions(),
   _modelMap(), _moduleMap(), _parseResults(), _editCommand(),
   _onExit(), _controlCB((ControlCB)0), _controller((void *)0)
{
if(_simul) delete _simul;
else ::srand(::time((time_t *)0)); // first --> init random generator
_simul=this;
_readlnMutex=new SimpleThreadMutex;
readOrisPath();
readOrisEdit();
Strings libName("oRisBase");
#ifndef __MSDOS__
if(openDynLibrary(libName))
#endif
  {
  parseStrings(Simulator::_launchCode1);
  parseStrings(Simulator::_launchCode2);
  parseStrings(Simulator::_launchCode3);
  }
#ifndef  __MSDOS__
else
  {
  Strings message="Library '"+libName+"' not found, oRis can't start !";
  manageError(new Error(Error::RUNTIME,message));
  _isBad=true;
  }
#endif
Simulator::_newTimerAction.sa_handler=schedHandler;
sigemptyset(&Simulator::_newTimerAction.sa_mask);
Simulator::_newTimerAction.sa_flags=0;
::sigaction(SIGPROF,&Simulator::_newTimerAction,&Simulator::_oldTimerAction);
}

Simulator::~Simulator(void)
{
if(!_isBad)
  {
  SimpleVector<Strings *> tmp;
  _onExit.swap(tmp);           // avoid increasing _onExit
  while(tmp.getSize())
    {
    parseStrings(*tmp.getFront());
    delete tmp.readPopFront();
    }
  }
while(_onExit.getSize()) delete _onExit.readPopBack();
size_t i;
unsigned long timeOut=2000;
ThreadedNative * thNative;
for(i=0;i<_curExecutions.getSize();i++)
  {
  thNative=_curExecutions[i]->accessThreadedNative();
  if(thNative)
    {
    unsigned long limit=Timer::getTime()+timeOut;
    while(!thNative->taskEnded()&&(Timer::getTime()<limit))
      SimpleThread::yield();
    if(!thNative->taskEnded())
      {
      _curExecutions[i]->setThreadedNative((ThreadedNative *)0);
      delete thNative;
      }
    }
  }
for(i=0;i<_execVectFrom.getSize();i++)
  {
  if(_execVectFrom[i])
    {
    thNative=_execVectFrom[i]->accessThreadedNative();
    if(thNative)
      {
      unsigned long limit=Timer::getTime()+timeOut;
      while(!thNative->taskEnded()&&(Timer::getTime()<limit))
        SimpleThread::yield();
      if(!thNative->taskEnded())
        {
        _execVectFrom[i]->setThreadedNative((ThreadedNative *)0);
        delete thNative;
        }
      }
    }
  }
for(i=0;i<_execVectTo.getSize();i++)
  {
  if(_execVectTo[i])
    {
    thNative=_execVectTo[i]->accessThreadedNative();
    if(thNative)
      {
      unsigned long limit=Timer::getTime()+timeOut;
      while(!thNative->taskEnded()&&(Timer::getTime()<limit))
        SimpleThread::yield();
      if(!thNative->taskEnded())
        {
        _execVectTo[i]->setThreadedNative((ThreadedNative *)0);
        delete thNative;
        }
      }
    }
  }
ExecutionList::iterator it=_execListSuspend.begin();
for(;it!=_execListSuspend.end();it++)
  {
  if(*it)
    {
    thNative=(*it)->accessThreadedNative();
    if(thNative)
      {
      unsigned long limit=Timer::getTime()+timeOut;
      while(!thNative->taskEnded()&&(Timer::getTime()<limit))
        SimpleThread::yield();
      if(!thNative->taskEnded())
        {
        (*it)->setThreadedNative((ThreadedNative *)0);
        delete thNative;
        }
      }
    }
  }
// ThreadedNative::clearThreadedNativeStock();
_isBad=true;
setInterval(0L);
::sigaction(SIGPROF,&Simulator::_oldTimerAction,(struct sigaction *)0);
while(_curExecutions.getSize())    // clear executions
  {
  Execution::relaxExecution(_curExecutions.readPopBack());
  }
while(_execVectFrom.getSize())
  {
  if(_execVectFrom.getBack())
    Execution::relaxExecution(_execVectFrom.readPopBack());
  else _execVectFrom.popBack();
  }
while(_execVectTo.getSize())
  {
  if(_execVectTo.getBack())
    Execution::relaxExecution(_execVectTo.readPopBack());
  else _execVectTo.popBack();
  }
while(_execListSuspend.size())
  {
  Execution::relaxExecution(_execListSuspend.back());
  _execListSuspend.pop_back();
  }
delete _readlnMutex;
Execution::clearExecutionStock();
TmpValue::clearTmpValueStock();
LocalValue::clearLocalValueStock();
SubValue::clearSubValueStock();
SubValue::clearSubValueVectStock();
Context::clearContextStock();
Context::clearValueVectStock();
Agent * agent;
while(_agentMap.size()) // delete agents
  {
  agent=(*_agentMap.begin()).second; // strongClear=true --> attributes
  agent->clearReferences(true);      //                      are deleted
  if((*_agentMap.begin()).second==agent) // not auto-removed
    {
    _agentMap.erase(_agentMap.begin());
    delete agent;
    }
  }
while(_deletedMap.size()) // delete agents (only deleted in oRis)
  {
  agent=(*_deletedMap.begin()).second; // strongClear=true --> attributes
  agent->clearReferences(true);        //                      are deleted
  if((*_deletedMap.begin()).second==agent) // not auto-removed
    {
    _deletedMap.erase(_deletedMap.begin());
    delete agent;
    }
  }
TmpValue::clearTmpValueStock(); // some TmpValues should have been relaxed
ModelMap::iterator itM=_modelMap.begin();
for(;!(itM==_modelMap.end());itM++) // delete models
  delete (*itM).second;
ExecModMap::iterator itF=_moduleMap.begin();
for(;!(itF==_moduleMap.end());itF++) // delete functions
  (*itF).second->unReference();
while(ValueCallback::getMask()>0) // reset callback mask
  ValueCallback::unmask();
while(ValueCallback::getMask()<0)
  ValueCallback::mask();
while(_dynLibs.getSize()) // close dynamic libraries
  delete _dynLibs.readPopBack();
if(_simul==this) _simul=(Simulator *)0;
}

//-------- Inspectors ------------------------------------------------------//
// inline const StringsVect & Simulator::getPackages(void) const

// inline bool Simulator::hasPackage(const Strings & packName) const

const SymbolVect * Simulator::getSymbolVect(const Strings & name) const
{
SymbolMap::const_iterator it=_replaceMap.find(name);
if(it==_replaceMap.end()) return((const SymbolVect *)0);
else return(&(*it).second);
}

// inline Simulator::PickingMode Simulator::getPickingMode(void) const

// inline long Simulator::getInterval(void) const

// inline long unsigned int Simulator::getClock(void) const

// inline int Simulator::getMaxDepth(void) const

// inline const StringsVect & Simulator::getCommandLine(void) const

// inline const Strings & Simulator::getInstallDir(void) const

// inline const TextOutput * Simulator::getOutput(void) const

// inline const TextOutput * Simulator::getError(void) const

// inline int Simulator::getNbErrors(void) const

// inline bool Simulator::isBad(void) const

// inline const StringsVect & Simulator::getPathVect(void) const

Strings Simulator::locateFile(const Strings & fileName) const
{
Strings name(fileName);
FILE * file=::fopen(name,"rb");
for(unsigned int i=0;!file&&i<_pathVect.size();i++)
  {
  name=_pathVect[i]+fileName;
  file=::fopen(name,"rb");
  }
if(file) ::fclose(file);
else name=Strings("");
return(name);
}

// inline int Simulator::getNbAgents(void) const

// inline const AgentMap & Simulator::getAgentMap(void) const

const Agent * Simulator::getAgent(const AgentName & name) const
{
AgentMap::const_iterator it=_agentMap.find(name);
if(it==_agentMap.end()) return((const Agent *)0);
return((*it).second);
}

// inline int Simulator::getNbModels(void) const

// inline const ModelMap & Simulator::getModelMap(void) const

const Model * Simulator::getModel(const Word & name) const
{
ModelMap::const_iterator it=_modelMap.find(name);
if(it==_modelMap.end()) return((const Model *)0);
return((*it).second);
}

// inline int Simulator::getNbModules(void) const

// inline const ExecModMap & Simulator::getModuleMap(void) const

const ExecModule * Simulator::getModule(const Word & name) const
{
ExecModMap::const_iterator it=_moduleMap.find(name);
if(it==_moduleMap.end()) return((const ExecModule *)0);
return((*it).second);
}

Error * Simulator::tryToSetModule(const ExecModule * newModule) const
{
const ExecModule * oldModule=getModule(newModule->getName());
if(oldModule&&!oldModule->isCompatible(*newModule))
  {
  Strings message="Module '"+
                  newModule->getCompleteName()+
                  "' is incompatible with '"+
                  oldModule->getCompleteName()+
                  "'";
  return(new Error(Error::SEMANTICS,message));
  }
return((Error *)0);
}

ExternFunction Simulator::getExternFunction(const Strings & name) const
{                                                            // virtual
#ifdef __MSDOS__
  ExternFunction function;
  function=::getExternAReViFunction(name);    if(function) return(function);
  function=::getExternAttrCbFunction(name);   if(function) return(function);
  function=::getExternReflFunction(name);     if(function) return(function);
  function=::getExternCntnrFunction(name);    if(function) return(function);
  function=::getExternConvFunction(name);     if(function) return(function);
  function=::getExternFilesFunction(name);    if(function) return(function);
  function=::getExternFuzzyFunction(name);    if(function) return(function);
  function=::getExternIpcFunction(name);      if(function) return(function);
  function=::getExternMathsFunction(name);    if(function) return(function);
  function=::getExternORisBaseFunction(name); if(function) return(function);
  function=::getExternSemaFunction(name);     if(function) return(function);
  function=::getExternTcpUdpFunction(name);   if(function) return(function);
  function=::getExternGuiFunction(name);      if(function) return(function);
  function=::getExternObject2dFunction(name); if(function) return(function);
  return((ExternFunction)0); // not found
#else
Strings wrapName=DynLibInfo::buildWrapName(name);
ExternFunction function;
for(int i=_dynLibs.getSize();i--;)
  {
  function=_dynLibs[i]->getExternFunction(name,wrapName);
  if(function) return(function);
  }
return((ExternFunction)0);  // not found in libraries
#endif
}

Agent * Simulator::getNewAgent(Model * model,long instNum) const // virtual
{
Agent * agt;
if(model->isNative())
  {
  for(int i=_dynLibs.getSize();i--;)
    {
    agt=_dynLibs[i]->getNewAgent(model,instNum);
    if(agt) return(agt);
    }
  }
return(new Agent(model,instNum)); // not found in libraries
}

// inline const Execution * Simulator::getPreviousExecution(const Execution
//                                                          * exec) const

//-------- Comparisons -----------------------------------------------------//

//-------- Input/Output ----------------------------------------------------//
ostream & operator<<(ostream & stream,const Simulator & simul)
{
return(stream<<simul.print());
}

Strings Simulator::print(void) const
{
Strings str;
str+="\n//======================= Replacements =======================//\n\n";
  {
  /*
  SymbolMap::const_iterator it=_replaceMap.begin();
  for(;!(it==_replaceMap.end());it++)
    {
    str+="replace "+(*it).first+" by ";
    if((*it).second.code==SmaParser::STRING_VAL)
      str+=(*it).second.value.dump();
    else
      str+=(*it).second.value.print();
    str+="\n";
    }
  */
  }
str+="\n//======================= Declarations =======================//\n\n";
ModelSet modelSet;
ModelSet::const_iterator itMS;
  {
  ModelMap::const_iterator itM=_modelMap.begin();
  for(;!(itM==_modelMap.end());itM++)
    modelSet.insert((*itM).second);
  itMS=modelSet.begin();
  for(;!(itMS==modelSet.end());itMS++)
    str+="class "+(*itMS)->getName().getStrings()+";\n";
  }
  str+="\n//-------- Global Code ------------------------\n\n";
  {
  ExecModMap::const_iterator itM=_moduleMap.begin();
  for(;!(itM==_moduleMap.end());itM++)
    str+=(*itM).second->printProto()+"\n";
  }
  {
  itMS=modelSet.begin();
  for(;!(itMS==modelSet.end());itMS++)
    str+="//-------- Class "+(*itMS)->getName().getStrings()+
         " ------------------------\n\n"+
         (*itMS)->printDeclaration()+"\n";
  }
str+="\n//====================== Implementation ======================//\n\n";
str+="//-------- Global Code ------------------------\n\n";
  {
  ExecModMap::const_iterator itM=_moduleMap.begin();
  for(;!(itM==_moduleMap.end());itM++)
    str+=(*itM).second->print()+"\n";
  }
  {
  itMS=modelSet.begin();
  for(;!(itMS==modelSet.end());itMS++)
    str+="//-------- Class "+(*itMS)->getName().getStrings()+
         " ------------------------\n\n"+
         (*itMS)->printModules()+"\n";
  }
str+="\n//====================== Initialization ======================//\n\n";
  {
  str+="execute\n{\n";
  str+="setClock("+Strings(_clock)+");\n";
  str+="setPickingMode(";
  if(_pickingMode==Simulator::SERIAL) str+="serialPicking);\n";
  else str+="randomPicking);\n";
  str+="setInterval("+Strings(_interval)+");\n";
  str+="setMaxDepth("+Strings(_maxDepth)+");\n";
  AgentMap::const_iterator itA=_agentMap.begin();
  for(;!(itA==_agentMap.end());itA++)
    str+=(*itA).second->printInstanciation();
  str+="}\n\n";
  itA=_agentMap.begin();
  for(;!(itA==_agentMap.end());itA++)
    str+=(*itA).second->printDeclarations();
  itA=_agentMap.begin();
  for(;!(itA==_agentMap.end());itA++)
    str+=(*itA).second->printDefinitions();
  str+="execute\n{\nforce\n  {\n";
  for(itA=_agentMap.begin();!(itA==_agentMap.end());itA++)
    str+=(*itA).second->printInitialisations(2);
  str+="  }\n}\n\n";
  }
return(str);
}

bool Simulator::save(const Strings & fileName) const
{
ofstream file(fileName);
if(file.fail()) return(false);
file << print();
if(file.fail()) return(false);
file.close();
return(true);
}

Strings Simulator::printStack(void) const
{
Strings str;
for(unsigned int i=0;i<_curExecutions.getSize();i++)
  {
  if(i) str+="    ----------------\n";
  str+=_curExecutions[i]->printStack();
  }
return(str);
}

void Simulator::readOrisPath(void) // virtual
{
_pathVect.erase(_pathVect.begin(),_pathVect.end());
#ifndef __MSDOS__
_pathVect.push_back(_installDir);
_pathVect.push_back(_installDir+"Pkg/");
#endif
char * var=getenv("ORISPATH");
if(var) addToPath(Strings(var));
}

void Simulator::readOrisEdit(void)
{
char * var=getenv("ORISEDIT");
if(var) _editCommand=var;
}

void Simulator::editFile(const Strings & fileName) const
{
char command[0x100];
int i,j,k;
for(i=0,j=0;_editCommand[i];i++)
  {
  if((_editCommand[i]=='{')&&(_editCommand[i+1]=='}'))
    for(i++,k=0;fileName[k];command[j++]=fileName[k++]);
  else command[j++]=_editCommand[i];
  }
command[j]='\0';
if(j) System::launch(command);
}

istream * Simulator::openFile(Strings & fileName) const
{
Strings name(fileName);
// ifstream * file=new ifstream(name,ios::in|ios::nocreate);
ifstream * file=new ifstream(name,ios::in);
if(file->fail())
  {
  delete file;
  file=(ifstream *)0;
  for(unsigned int i=0;!file&&i<_pathVect.size();i++)
    {
    name=_pathVect[i]+fileName;
    file=new ifstream(name,ios::in);
    if(file->fail())
      {
      delete file;
      file=(ifstream *)0;
      }
    }
  }
if(file)
  {
  fileName=name;
  getSimul()->addToPath(fileName.dirname());
  return(file);
  }
else return((istream *)0);
}

void Simulator::showURL(const Strings & url,bool newWindow) const // virtual
{
Strings cmd;
cmd+="( TMP=`ls -l $HOME/.netscape/lock 2>/dev/null` ; ";
cmd+="  TMP=`echo $TMP | sed -e s%\"^.*netscape/lock \"%%g 2>/dev/null` ; ";
cmd+="  TMP=`echo $TMP | sed -e s%\"^.*:\"%%g 2>/dev/null` ; ";
cmd+="  if kill -0 $TMP >/dev/null 2>&1 ; then ";
cmd+="    netscape -remote 'openURL("+url;
if(newWindow) cmd+=",new-window";
cmd+=")' ; ";
cmd+="  else";
cmd+="    netscape "+url+" ; ";
cmd+="  fi ) &";
System::launch(cmd);
}

//-------- Modifiers -------------------------------------------------------//
// inline void Simulator::addPackage(const Strings & packName)

// inline void Simulator::setControlCB(ControlCB controlCB,void * controller)

void Simulator::replaceSymbol(const Strings & name,
                              const SymbolVect * symbolVect)
{
SymbolMap::iterator i=_replaceMap.find(name);
if(!(i==_replaceMap.end()))
  _replaceMap.erase(i);
if(symbolVect)
   _replaceMap.insert(SymbolMap::value_type(name,*symbolVect));
}

// inline void Simulator::setPickingMode(Simulator::PickingMode pickingMode)

void Simulator::setInterval(long interval)
{
_interval=interval;
if(_interval>0)
  {
  struct itimerval itv;
  itv.it_value.tv_sec=itv.it_interval.tv_sec
                     =(unsigned long)_interval/1000LU;
  itv.it_value.tv_usec=itv.it_interval.tv_usec
                     =1000LU*((unsigned long)_interval%1000LU);
  setitimer(ITIMER_PROF,&itv,(struct itimerval *)0);
  }
else
  {
  struct itimerval itv={{0LU,0LU},{0LU,0LU}};
  setitimer(ITIMER_PROF,&itv,(struct itimerval *)0);
  }
}

// inline void Simulator::setClock(long unsigned int clock)

// inline void Simulator::setMaxDepth(int maxDepth)

// inline TextOutput * Simulator::accessOutput(void)

// inline TextOutput * Simulator::accessError(void)

void Simulator::addToPath(const Strings & path) // virtual
{
const char * start=(const char *)path;
const char * current=start;
bool stop=false;
while(!stop)
  {
#ifndef __MSDOS__
  if((*current==':')||(*current==';')||(*current=='\0'))
#else
  if((*current==';')||(*current=='\0'))
#endif
  {
    if(current!=start)
      {
      char * str;
      int nb=current-start;
      if((*(current-1)=='/')||(*(current-1)=='\\'))
        {
        str=new char[nb+1];
        strncpy(str,start,nb);
        }
      else
        {
        str=new char[nb+2];
        strncpy(str,start,nb);
        nb++;
        }
      str[nb-1]='/';
      str[nb]='\0';
      Strings newPath(str);
      bool found=false;
      for(int i=_pathVect.size();!found&&i--;)
        if(_pathVect[i]==newPath) found=true;
      if(!found) _pathVect.push_back(newPath);
      delete []str;
      }
    start=current+1;
    }
  if(*current) current++;
  else stop=true;
  }
}

Agent * Simulator::accessAgent(const AgentName & name)
{
AgentMap::iterator it=_agentMap.find(name);
if(it==_agentMap.end()) return((Agent *)0);
return((*it).second);
}

void Simulator::setModel(Model * newModel,bool notify)
{
_modelMap.insert(ModelMap::value_type(newModel->getName(),newModel));
if(notify) _notifyChange(_notifyClassWord,newModel->getName());
}

// inline Model::ModelMap & Simulator::accessModelMap(void)

Model * Simulator::accessModel(const Word & name)
{
ModelMap::iterator it=_modelMap.find(name);
if(it==_modelMap.end()) return((Model *)0);
return((*it).second);
}

void Simulator::setModule(ExecModule * newModule,bool notify)
{
bool used=false;
ExecModMap::iterator it=_moduleMap.find(newModule->getName());
if(it==_moduleMap.end())                 // not found --> insert
  {
  _moduleMap.insert(ExecModMap::value_type(
                    newModule->getName(),newModule));
  used=true;
  newModule->reference();
  }
else                                     // found --> replace if needed
  {
  ExecModule * oldModule=(*it).second;
  if(oldModule!=newModule)               // not the same
    {
    oldModule->unReference();
    (*it).second=newModule;
    used=true;
    newModule->reference();
    }
  }
if(notify&&used) _notifyChange(_notifyModuleWord,newModule->getName());
}

// inline ExecModMap & Simulator::accessModuleMap(void)

ExecModule * Simulator::accessModule(const Word & name)
{
ExecModMap::iterator it=_moduleMap.find(name);
if(it==_moduleMap.end()) return((ExecModule *)0);
return((*it).second);
}

// inline Execution * Simulator::accessPreviousExecution(const Execution
//                                                       * exec)

//-------- Simulation Control ----------------------------------------------//
bool Simulator::openDynLibrary(const Strings & libName) // virtual
{
for(int i=_dynLibs.getSize();i--;)
  {
  if(_dynLibs[i]->getLibName()==libName) return(true);
  }
DynLibInfo * dynLib=new DynLibInfo(libName);
if(dynLib->fail())
  {
  delete dynLib;
  return(false);
  }
else
  {
  _dynLibs.pushBack(dynLib);
  return(true);
  }
}

int Simulator::parseFile(Strings & fileName)
{
int nbErrors=_nbErrors;
istream * file=openFile(fileName);
if(file)
  {
  SmaParser parser(fileName,file,(SmaParser *)0);
  parser.yyparse();
  delete file;
  }
else
  {
  Strings message="Can't open file '"+
                  fileName+"'";
  manageError(new Error(Error::SEMANTICS,message));
  }
return(_nbErrors-nbErrors);
}

int Simulator::parseStrings(const Strings & program)
{
int nbErrors=_nbErrors;
istringstream * input=new istringstream((const char *)program);
SmaParser parser("",input,(SmaParser *)0);
parser.yyparse();
delete input;
return(_nbErrors-nbErrors);
}

Value * Simulator::parseEval(const Strings & program)
{
getSimul()->_parseResults.pushBack(TmpValue::getTmpValue(ValueType()));
getSimul()->parseStrings(program);
return(getSimul()->_parseResults.readPopBack());
}

bool Simulator::parseConstant(Value * result,const Strings & constant)
{
istringstream * input=new istringstream((const char *)constant);
ConstantParser parser(input,result);
parser.yyparse();
delete input;
return(!parser.fail());
}

void Simulator::manageAgent(Agent * agent)
{
int pos;
bool found=false;
for(pos=_curExecutions.getSize();pos--;)
  {
  if((_curExecutions[pos]->getOwner()==agent)&&
     (_curExecutions[pos]->getType()==ExecModule::BEHAVIOR))
    {
    _curExecutions[pos]->allowRestart();
    found=true;
    break;
    }
  }
if(!found)
  {
  for(pos=_execVectFrom.getSize();pos--;)
    {
    if(_execVectFrom[pos]&&
       (_execVectFrom[pos]->getOwner()==agent)&&
       (_execVectFrom[pos]->getType()==ExecModule::BEHAVIOR))
      {
      _execVectFrom[pos]->allowRestart();
      found=true;
      break;
      }
    }
  if(!found)
    {
    for(pos=_execVectTo.getSize();pos--;)
      {
      if(_execVectTo[pos]&&
         (_execVectTo[pos]->getOwner()==agent)&&
         (_execVectTo[pos]->getType()==ExecModule::BEHAVIOR))
        {
        _execVectTo[pos]->allowRestart();
        found=true;
        break;
        }
      }
    if(!found)
      {
      ExecutionList::iterator it=_execListSuspend.begin();
      for(;it!=_execListSuspend.end();it++)
        {
        if(((*it)->getOwner()==agent)&&
           ((*it)->getType()==ExecModule::BEHAVIOR))
          {
          (*it)->allowRestart();
          _execVectTo.pushBack(*it);
          _execListSuspend.erase(it);
          found=true;
          break;
          }
        }
      if(!found)
        {
        ExecModule * module=agent->accessMainModule();
        Execution * execution=Execution::getExecution(agent,
                                                  module->getSpecialType());
        _curExecutions.pushBack(execution);
        execution->enterContext(module,agent);
        _curExecutions.popBack();
        _execVectTo.pushBack(execution);
        }
      }
    }
  }
}

void Simulator::unManageAgent(Agent * agent)
{
int pos;
for(pos=_curExecutions.getSize();pos--;)
  {
  if((_curExecutions[pos]->getOwner()==agent)&&
     (_curExecutions[pos]->getType()==ExecModule::BEHAVIOR))
    {
    _curExecutions[pos]->avoidRestart();
    return;
    }
  }
for(pos=_execVectFrom.getSize();pos--;)
  {
  if(_execVectFrom[pos]&&
     (_execVectFrom[pos]->getOwner()==agent)&&
     (_execVectFrom[pos]->getType()==ExecModule::BEHAVIOR))
    {
    _execVectFrom[pos]->avoidRestart();
    return;
    }
  }
for(pos=_execVectTo.getSize();pos--;)
  {
  if(_execVectTo[pos]&&
     (_execVectTo[pos]->getOwner()==agent)&&
     (_execVectTo[pos]->getType()==ExecModule::BEHAVIOR))
    {
    _execVectTo[pos]->avoidRestart();
    return;
    }
  }
ExecutionList::iterator it=_execListSuspend.begin();
for(;it!=_execListSuspend.end();it++)
  {
  if(((*it)->getOwner()==agent)&&
     ((*it)->getType()==ExecModule::BEHAVIOR))
    {
    (*it)->avoidRestart();
    return;
    }
  }
}

void Simulator::suspendAgent(Agent * agent)
{
int pos;
for(pos=_curExecutions.getSize();pos--;)
  {
  if((_curExecutions[pos]->getOwner()==agent)&&
     !_curExecutions[pos]->endReached())
    {
    if((_curExecutions[pos]->getType()==ExecModule::DESTRUCTOR)||
       (_curExecutions[pos]->getType()==ExecModule::CONSTRUCTOR))
      {
      Strings message="Critical section can't be broken during ";
      if(_curExecutions[pos]->getType()==ExecModule::DESTRUCTOR)
        message+="destruction";
      else // CONSTRUCTOR
        message+="construction";
      manageError(new Error(Error::WARNING,message));
      }
    else
      {
      if(_curExecutions[pos]->getCriticalLevel())
        {
        Strings message="Broken critical section";
        manageError(new Error(Error::WARNING,message));
        }
      _curExecutions[pos]->setSuspended(true);
      }
    }
  }
for(pos=_execVectFrom.getSize();pos--;)
  {
  if(_execVectFrom[pos]&&(_execVectFrom[pos]->getOwner()==agent))
    {
    _execVectFrom[pos]->setSuspended(true);
    _execListSuspend.push_back(_execVectFrom[pos]);
    _execVectFrom[pos]=(Execution *)0;
    }
  }
for(pos=_execVectTo.getSize();pos--;)
  {
  if(_execVectTo[pos]&&(_execVectTo[pos]->getOwner()==agent))
    {
    _execVectTo[pos]->setSuspended(true);
    _execListSuspend.push_back(_execVectTo[pos]);
    _execVectTo[pos]=(Execution *)0;
    }
  }
}

void Simulator::resumeAgent(Agent * agent)
{
for(int pos=_curExecutions.getSize();pos--;)
  {
  if(_curExecutions[pos]->getOwner()==agent)
    {
    _curExecutions[pos]->setSuspended(false);
    }
  }
ExecutionList::iterator it=_execListSuspend.begin();
while(it!=_execListSuspend.end())
  {
  if((*it)->getOwner()==agent)
    {
    (*it)->setSuspended(false);
    _execVectTo.pushBack(*it);
    _execListSuspend.erase(it++);
    }
  else it++;
  }
}

void Simulator::abortExecution(long execID)
{
Execution * exec;
int pos;
for(pos=_curExecutions.getSize();pos--;)
  {
  exec=_curExecutions[pos];
  if(exec&&(exec->getID()==execID))
    {
    if(!exec->endReached())
      {
      if((exec->getType()==ExecModule::DESTRUCTOR)||
         (exec->getType()==ExecModule::CONSTRUCTOR))
        {
        Strings message="Can't abort ";
        if(exec->getType()==ExecModule::DESTRUCTOR)
          message+="destruction";
        else // CONSTRUCTOR
          message+="construction";
        manageError(new Error(Error::WARNING,message));
        }
      else
        {
        if(exec->getCriticalLevel())
          {
          Strings message="Broken critical section";
          manageError(new Error(Error::WARNING,message));
          }
        exec->abort(false);
        }
      }
    return;
    }
  }
for(pos=_execVectFrom.getSize();pos--;)
  {
  exec=_execVectFrom[pos];
  if(exec&&(exec->getID()==execID))
    {
    exec->abort(true);
    if(exec->getType()!=ExecModule::BEHAVIOR)
      {
      Execution::relaxExecution(exec);
      _execVectFrom[pos]=(Execution *)0;
      }
    return;
    }
  }
for(pos=_execVectTo.getSize();pos--;)
  {
  exec=_execVectTo[pos];
  if(exec&&(exec->getID()==execID))
    {
    exec->abort(true);
    if(exec->getType()!=ExecModule::BEHAVIOR)
      {
      Execution::relaxExecution(exec);
      _execVectTo[pos]=(Execution *)0;
      }
    return;
    }
  }
ExecutionList::iterator it=_execListSuspend.begin();
for(;it!=_execListSuspend.end();it++)
  {
  exec=(*it);
  if(exec->getID()==execID)
    {
    exec->abort(true);
    if(exec->getType()!=ExecModule::BEHAVIOR)
      {
      Execution::relaxExecution(exec);
      _execListSuspend.erase(it);
      }
    return;
    }
  }
}

void Simulator::suspendExecution(long execID)
{
int pos;
for(pos=_curExecutions.getSize();pos--;)
  {
  if(_curExecutions[pos]&&(_curExecutions[pos]->getID()==execID))
    {
    if(!_curExecutions[pos]->endReached())
      {
      if((_curExecutions[pos]->getType()==ExecModule::DESTRUCTOR)||
         (_curExecutions[pos]->getType()==ExecModule::CONSTRUCTOR))
        {
        Strings message="Can't suspend ";
        if(_curExecutions[pos]->getType()==ExecModule::DESTRUCTOR)
          message+="destruction";
        else // CONSTRUCTOR
          message+="construction";
        manageError(new Error(Error::WARNING,message));
        }
      else
        {
        if(_curExecutions[pos]->getCriticalLevel())
          {
          Strings message="Broken critical section";
          manageError(new Error(Error::WARNING,message));
          }
        _curExecutions[pos]->setSuspended(true);
        }
      }
    return;
    }
  }
for(pos=_execVectFrom.getSize();pos--;)
  {
  if(_execVectFrom[pos]&&(_execVectFrom[pos]->getID()==execID))
    {
    _execVectFrom[pos]->setSuspended(true);
    _execListSuspend.push_back(_execVectFrom[pos]);
    _execVectFrom[pos]=(Execution *)0;
    return;
    }
  }
for(pos=_execVectTo.getSize();pos--;)
  {
  if(_execVectTo[pos]&&(_execVectTo[pos]->getID()==execID))
    {
    _execVectTo[pos]->setSuspended(true);
    _execListSuspend.push_back(_execVectTo[pos]);
    _execVectTo[pos]=(Execution *)0;
    return;
    }
  }
}

bool Simulator::resumeExecution(long execID)
{
for(int pos=_curExecutions.getSize();pos--;)
  {
  if(_curExecutions[pos]&&(_curExecutions[pos]->getID()==execID))
    {
    _curExecutions[pos]->setSuspended(false);
    return(true);
    }
  }
ExecutionList::iterator it=_execListSuspend.begin();
for(;it!=_execListSuspend.end();it++)
  {
  if((*it)->getID()==execID)
    {
    (*it)->setSuspended(false);
    _execVectTo.pushBack((*it));
    _execListSuspend.erase(it);
    return(true);
    }
  }
return(false);
}

bool Simulator::isSuspended(long execID)
{
for(int pos=_curExecutions.getSize();pos--;)
  {
  if(_curExecutions[pos]&&(_curExecutions[pos]->getID()==execID))
    {
    return(_curExecutions[pos]->isSuspended());
    }
  }
ExecutionList::iterator it=_execListSuspend.begin();
for(;it!=_execListSuspend.end();it++)
  {
  if((*it)->getID()==execID) return(true);
  }
return(false);
}

long Simulator::startModule(ExecModule * module,Agent * contextOwner,
                            Execution * initExec)
{
if((int)_curExecutions.getSize()>_maxDepth)
  {
  for(int param=module->getNbParams();param--;)
    initExec->popValue()->relax();
  if(initExec&&
     (module->getResultType().getSimpleType()!=ValueType::UNKNOWN))
    initExec->pushValue(TmpValue::getTmpValue(module->getResultType()));
  Strings message="Recursion too deep ("+
                  Strings(getSimul()->getMaxDepth())+
                  " reached)";
  manageError(new Error(Error::RUNTIME,message));
  return(0);
  }
else
  {
  Execution * execution=Execution::getExecution(contextOwner,
                                                module->getSpecialType());
  long execID=execution->getID();
  ValueType resultType(module->getResultType());
  if(initExec)
    {
    int param;
    for(param=module->getNbParams();param--;)
      execution->pushValue(initExec->accessValue(param));
    for(param=module->getNbParams();param--;)
      initExec->popValue();
    }
  _curExecutions.pushBack(execution);
  execution->enterContext(module,contextOwner);
  if(execution->getCriticalLevel()&& // only critical code is launched
    !execution->isFinished())
    {
    do
      {
      execution->accessContext()->eval();
      } while(execution->goOn());
    }
  _curExecutions.popBack();
  if(execution->isFinished())
    {
    if(execution->isAborted())
      {
      if(initExec) initExec->abort(false);
      }
    else if(resultType.getSimpleType()!=ValueType::UNKNOWN)
      {
      if(initExec)
        {
        initExec->pushValue(execution->popValue());
        }
      else if(_parseResults.getSize())
        {
        *_parseResults.accessBack()=*execution->lastValue();
        }
      }
    Execution::relaxExecution(execution);
    }
  else if(execution->isSuspended())
    _execListSuspend.push_back(execution);
  else
    _execVectTo.pushBack(execution);
  return(execID);
  }
}

// inline long Simulator::executeModule(ExecModule * module,
//                                      Agent * contextOwner,
//                                      Execution * initExec)

void Simulator::run(int nbSteps)
{
Execution * execution;
int nb,min,max,nbToGo;
do
  {
  _execVectFrom.swap(_execVectTo); // Now To is empty and From is full
  min=0;
  max=nb=_execVectFrom.getSize();
  while(nb--)
    {
    if((_pickingMode==Simulator::SERIAL)||
       Simulator::integerRand(2))
      {
      execution=_execVectFrom[min];
      _execVectFrom[min++]=(Execution *)0;
      }
    else
      {
      execution=_execVectFrom[--max];
      _execVectFrom[max]=(Execution *)0;
      }
    if(execution)
      {
      _curExecutions.pushBack(execution);
      execution->setGoOn(true);
      if(_interval>=0) // stopped by timer or cooperation
        {
        do
          {
          execution->accessContext()->eval();
          } while(execution->goOn());
        }
      else // deeply parallel
        {
        nbToGo=_interval;
        do
          {
          execution->accessContext()->eval();
          } while(((++nbToGo<0)||execution->getCriticalLevel())
                   &&execution->goOn());
        }
      _curExecutions.popBack();
      if(execution->isFinished())
        {
        if(execution->isAborted()&&execution->canRestart())
          {
          execution->abort(true);                // main() is ready to restart
          execution->cancelAbort();
          _execListSuspend.push_back(execution); // but is suspended
          }
        else Execution::relaxExecution(execution);
        }
      else if(execution->isSuspended())
        _execListSuspend.push_back(execution);
      else
        _execVectTo.pushBack(execution);
      if(_isBad) goto endRun; // emergency stop !
      }
    }
  _execVectFrom.raz();
  _clock++;
  } while(--nbSteps);
endRun: ; // emergency stop !
}

void Simulator::abortSignal(void)
{
for(int i=_curExecutions.getSize();i--;)
  {
  _curExecutions[i]->setGoOn(false);
  }
}

void Simulator::manageError(Error * error)
{
if(error)
  {
  switch(error->getType())
    {
    case Error::WARNING:
      {
      *_err << "\nWARNING" << endl;
      *_err << printStack() << error->print() << endl;
      }
      break;
    case Error::DEVICE:
      {
      *_err << "\nDEVICE" << endl;
      *_err << printStack() << error->print() << endl;
      }
      break;
    case Error::RUNTIME:
      {
      _nbErrors++;
      *_err << "\nRUN-TIME ERROR" << endl;
      *_err << printStack() << error->print() << endl;
      for(int pos=_curExecutions.getSize();pos--;)
        {
        if(_curExecutions[pos]->getType()==ExecModule::BEHAVIOR)
          {
          *_err << "To resume the 'main()' of '" 
                << _curExecutions[pos]->getOwner()->getName().print()
                << "' please parse :" << endl;
          *_err << "    execute { resume("
                << _curExecutions[pos]->getID()
                << "); }" << endl;;
          }
        _curExecutions[pos]->abort(false);
        }
      }
      break;
    case Error::BUG:
      {
      *_err << "\nCONGRATULATIONS... YOU HAVE FOUND A BUG !!!" << endl;
      *_err << error->print() << endl;
      _isBad=true;
      int pos;
      for(pos=_curExecutions.getSize();pos--;)
        {
        _curExecutions[pos]->mustDisappear();
        }
      for(pos=_execVectFrom.getSize();pos--;)
        {
        if(_execVectFrom[pos]) _execVectFrom[pos]->mustDisappear();
        }
      for(pos=_execVectTo.getSize();pos--;)
        {
        if(_execVectTo[pos]) _execVectTo[pos]->mustDisappear();
        }
      ExecutionList::iterator it=_execListSuspend.begin();
      while(it!=_execListSuspend.end())
        {
        (*it)->mustDisappear();
        }
      }
      break;
    default:
      {
      _nbErrors++;
      *_err << error->print() << endl;
      }
      break;
    }
  delete error;
  }
}

//-------- Agent Creation/Deletion -----------------------------------------//
void Simulator::createAgent(Value * created,bool isTransient,
                            Model * model,long instNum) // virtual
{
if(getAgent(AgentName(model->getName(),instNum)))
  {
  created->setAgent((Agent *)0);
  created->accessType().setModel(model);
  }
else
  {
  Agent * newAgent=model->createAgent(instNum);
  created->setAgent(newAgent);
  created->accessType().setModel(model);
  _agentMap.insert(AgentMap::value_type(newAgent->getName(),newAgent));
  newAgent->setTransient(isTransient);
  }
}

void Simulator::deleteAgent(Agent * agent) // virtual
{
_deletedMap.insert(AgentMap::value_type(agent->getName(),agent));
agent->kill();
if(!_isBad)
  {
  ExecModule * module=agent->accessDeleteModule();
  if(!module) module=agent->accessModel()->accessDeleteModule();
  startModule(module,agent);
  }
}

bool Simulator::tryToRemoveExecution(Execution * execution,Agent * agent)
{
if(!execution) return(false);
bool mustAbort=false;
if((execution->getOwner()==agent)&&
   (execution->getType()==ExecModule::BEHAVIOR))
  {
  execution->avoidRestart();
  mustAbort=true;
  }
if(execution->usesAgent(agent)&&
   !execution->isAborted()) // code is not ended
  {
  mustAbort=true;
  Strings message="Execution of '"+
                  execution->print()+
                  "' aborted";
  manageError(new Error(Error::WARNING,message));
  }
return(mustAbort);
}

void Simulator::removeAgent(Agent * agent) // virtual
{
agent->accessModel()->removeAgent(agent);
_agentMap.erase(agent->getName());
int pos;
for(pos=_curExecutions.getSize();pos--;)
  {
  if(tryToRemoveExecution(_curExecutions[pos],agent))
    _curExecutions[pos]->abort(false);
  }
for(pos=_execVectFrom.getSize();pos--;)
  {
  if(tryToRemoveExecution(_execVectFrom[pos],agent))
    {
    _execVectFrom[pos]->abort(true);
    Execution::relaxExecution(_execVectFrom[pos]);
    _execVectFrom[pos]=(Execution *)0;
    }
  }
for(pos=_execVectTo.getSize();pos--;)
  {
  if(tryToRemoveExecution(_execVectTo[pos],agent))
    {
    _execVectTo[pos]->abort(true);
    Execution::relaxExecution(_execVectTo[pos]);
    _execVectTo[pos]=(Execution *)0;
    }
  }
ExecutionList::iterator it=_execListSuspend.begin();
while(it!=_execListSuspend.end())
  {
  if(tryToRemoveExecution(*it,agent))
    {
    (*it)->abort(true);
    Execution::relaxExecution(*it);
    _execListSuspend.erase(it++);
    }
  else it++;
  }                            // strongClear=false --> attributes
agent->clearReferences(false); //                       are not deleted
}

void Simulator::removeDeletedAgent(Agent * agent) // virtual
{
_deletedMap.erase(agent->getName());
}

//-------- Utilities -------------------------------------------------------//
void Simulator::schedHandler(int /* sigNum */) // static
{
for(int i=getSimul()->_curExecutions.getSize();i--;)
  {
  if(!getSimul()->_curExecutions[i]->getCriticalLevel())
    getSimul()->_curExecutions[i]->setGoOn(false);
  }
if(getSimul()->_interval<=0)
  {
  struct itimerval itv={{0LU,0LU},{0LU,0LU}};
  setitimer(ITIMER_PROF,&itv,(struct itimerval *)0);
  }
}

// inline unsigned int Simulator::maxRand(void) // static

// inline unsigned int Simulator::integerRand(unsigned int i) // static

// inline double Simulator::realRand(void) // static

// inline double Simulator::cartToAngle(double x,double y) // static

// inline double Simulator::cartToDist(double x,double y) // static

// inline double Simulator::polToX(double d,double a) // static

// inline double Simulator::polToY(double d,double a) // static

// inline void Simulator::cartToPol(double & d,double & a,
//                                  double x,double y) // static

// inline void Simulator::polToCart(double & x,double & y,
//                                  double d,double a) // static

// inline double Simulator::betweenPi(double a) // static

void Simulator::embedThread(void (* task)(Context *),
                            Context * context) // static
{
if(context->accessExecution()->getCriticalLevel()) (*task)(context);
else
  {
  ThreadedNative * thNative=context->accessExecution()
                                   ->accessThreadedNative();
  if(thNative)
    {
    if(thNative->taskEnded())
      {
      ThreadedNative::relaxThreadedNative(thNative);
      context->accessExecution()->setThreadedNative((ThreadedNative *)0);
      }
    else
      {
      context->accessExecution()->setGoOn(false);
      context->moveCounter(-1);
      }
    }
  else
    {
    thNative=ThreadedNative::getThreadedNative();
    context->accessExecution()->setThreadedNative(thNative);
    thNative->startTask(task,context);
    context->accessExecution()->setGoOn(false);
    context->moveCounter(-1);
    }
  }
}

//-------- Native code -----------------------------------------------------//

// native void onExit(string command);
//
void Simulator::stOnExit(Context * context) // static
{
getSimul()->_onExit.pushBack(new Strings(context->getStrings(0)));
}

// native bool loadLibrary(string libName);
//
void Simulator::stLoadLibrary(Context * context) // static
{
const Strings & libName=context->getStrings(0);
bool result=getSimul()->openDynLibrary(libName);
if(!result)
  {
  Strings message="Library '"+libName+"' can't be loaded";
  getSimul()->manageError(new Error(Error::WARNING,message));
  }
context->returnInteger(result);
}

// native int parse(string program);
//
void Simulator::stParse(Context * context) // static
{
context->returnInteger(getSimul()->parseStrings(context->getStrings(0)));
}

// native int load(string fileName);
//
void Simulator::stLoad(Context * context) // static
{
Strings fileName(context->getStrings(0));
context->returnInteger(getSimul()->parseFile(fileName));
}

// native string[] parseEval(string program);
//
void Simulator::stParseEval(Context * context) // static
{
Value * value=getSimul()->parseEval(context->getStrings(0));
Value * result=TmpValue::getTmpValue(context->getModule()->getResultType());
SubValueVect * vect=SubValue::getSubValueVect();
vect->pushBack(SubValue::getSubValue(ValueType(ValueType::STRING),result));
vect->accessBack()->setStrings(value->getType().print());
vect->pushBack(SubValue::getSubValue(ValueType(ValueType::STRING),result));
vect->accessBack()->setStrings(value->print());
value->relax();
result->setValueVector(vect);
context->returnValue(result);
}

// native int eval(string code);
//
void Simulator::stEval(Context * context) // static
{
int nbErrors=getSimul()->_nbErrors;
Context * previous=context->accessPrevious();
const AttrVect & locals=previous->getModule()->getLocals();
Value * value=(Value *)0;
Agent * agent=(Agent *)0;
Strings code1,code2,code3,attrName,attrType,objName;
if(locals.size())
  {
  value=TmpValue::getTmpValue(ValueType(Model::getBaseModel()));
  getSimul()->createAgent(value,false,Model::getBaseModel(),0);
  agent=value->getAgent();
  getSimul()->startModule(Model::getBaseModel()->accessNewModule(),
                          agent,context->accessExecution());
  objName=agent->getName().print();
  }
if(previous->getOwner())
  code2=previous->getOwner()->getName().print()+"::";
code2+="execute\n{\n";
code3=";\n";
int i;
for(i=locals.size();i--;)
  {
  attrName=locals[i].getName().getStrings();
  attrType=locals[i].getType().print();
  code1+=attrType+" "+objName+"::"+attrName+";\n";
  code2+=attrType+" "+attrName+"=("+attrType+")"+
                                 previous->getLocal(i)->dump()+";\n";
  code3+=objName+"->"+attrName+"=("+attrType+")"+attrName+";\n";
  }
code3+="}\n";
getSimul()->parseStrings(code1+code2+context->getStrings(0)+code3);
int newErrors=getSimul()->_nbErrors-nbErrors;
if(!newErrors)
  {
  for(i=locals.size();i--;)
    {
    previous->accessLocal(i)->affectCast(
              *(agent->getAttribute(locals[i].getName()))->getValue());
    }
  }
if(value)
  {
  if(agent->isAlive()) getSimul()->deleteAgent(agent);
  value->relax();
  }
context->returnInteger(newErrors);
}

// native void save(string fileName);
//
void Simulator::stSave(Context * context) // static
{
getSimul()->save(context->getStrings(0));
}

// native int getPickingMode(void);
//
void Simulator::stGetPickingMode(Context * context) // static
{
context->returnInteger(getSimul()->_pickingMode==Simulator::SERIAL);
}

// native void setPickingMode(int mode);
//
void Simulator::stSetPickingMode(Context * context) // static
{
getSimul()->_pickingMode=(context->getInteger(0) ? Simulator::SERIAL
                                                 : Simulator::RANDOM);
}

// native int getInterval(void);
//
void Simulator::stGetInterval(Context * context) // static
{
context->returnInteger(getSimul()->_interval);
}

// native void setInterval(int interval);
//
void Simulator::stSetInterval(Context * context) // static
{
getSimul()->setInterval(context->getInteger(0));
}

// native void yield(void);
//
void Simulator::stYield(Context * context) // static
{
Execution * exec=context->accessExecution();
if((exec->getType()==ExecModule::DESTRUCTOR)||
   (exec->getType()==ExecModule::CONSTRUCTOR))
  {
  Strings message="Critical section can't be broken during ";
  if(exec->getType()==ExecModule::DESTRUCTOR)
    message+="destruction";
  else // CONSTRUCTOR
    message+="construction";
  getSimul()->manageError(new Error(Error::WARNING,message));
  }
else
  {
  if(exec->getCriticalLevel())
    {
    Strings message="Broken critical section";
    getSimul()->manageError(new Error(Error::WARNING,message));
    }
  exec->setGoOn(false);
  }
}

// native void softYield(void);
//
void Simulator::stSoftYield(Context * context) // static
{
Execution * exec=context->accessExecution();
if(!exec->getCriticalLevel())
  {
  exec->setGoOn(false);
  }
}

// native void getExecutionStack(Object[] & callers,string[] & modules,
//                               int execution);
//
void Simulator::stGetExecutionStack(Context * context) // static
{
long execID=context->getInteger(2);
const Execution * exec=(const Execution *)0;
int pos;
for(pos=getSimul()->_curExecutions.getSize();pos--;)
  {
  if(getSimul()->_curExecutions[pos]->getID()==execID)
    {
    exec=getSimul()->_curExecutions[pos];
    break;
    }
  }
if(!exec)
  {
  for(pos=getSimul()->_execVectFrom.getSize();pos--;)
    {
    if(getSimul()->_execVectFrom[pos]&&
       (getSimul()->_execVectFrom[pos]->getID()==execID))
      {
      exec=getSimul()->_execVectFrom[pos];
      break;
      }
    }
  if(!exec)
    {
    for(pos=getSimul()->_execVectTo.getSize();pos--;)
      {
      if(getSimul()->_execVectTo[pos]&&
         (getSimul()->_execVectTo[pos]->getID()==execID))
        {
        exec=getSimul()->_execVectTo[pos];
        break;
        }
      }
    if(!exec)
      {
      ExecutionList::iterator it=getSimul()->_execListSuspend.begin();
      for(;it!=getSimul()->_execListSuspend.end();it++)
        {
        if((*it)->getID()==execID)
          {
          exec=(*it);
          break;
          }
        }
      }
    }
  }
AgentVect * callersVect=new AgentVect;
StringsVect * modulesVect=new StringsVect;
if(exec)
  {
  exec->getStack(*callersVect,*modulesVect);
  if(context->getExecution()==exec) // remove this call
    {
    if(callersVect->getSize()) callersVect->popBack();
    if(modulesVect->size()) modulesVect->pop_back();
    }
  }
SubValueVect * vect=SubValue::getSubValueVect();
ValueType type(context->getLocal(0)->getType().getSimpleType());
unsigned int i;
for(i=0;i<callersVect->getSize();i++)
  {
  vect->pushBack(SubValue::getSubValue(type,context->accessLocal(0)));
  vect->accessBack()->setAgent((*callersVect)[i]);
  }
delete callersVect;
context->accessLocal(0)->setValueVector(vect);
vect=SubValue::getSubValueVect();
type.setSimpleType(context->getLocal(1)->getType().getSimpleType());
for(i=0;i<modulesVect->size();i++)
  {
  vect->pushBack(SubValue::getSubValue(type,context->accessLocal(1)));
  vect->accessBack()->setStrings((*modulesVect)[i]);
  }
context->accessLocal(1)->setValueVector(vect);
delete modulesVect;
}

// native void printStack(void);
//
void Simulator::stPrintStack(Context * /* context */) // static
{
Strings stackStr(getSimul()->printStack());
int limit=stackStr.len();
int nbEnd=0;
while(limit--)           // remove this call
  {
  if(stackStr[limit]=='\n')
    {
    if(++nbEnd==3) break;
    }
  }
*getSimul()->_output << stackStr.subString(0,limit);
if(limit) *getSimul()->_output << endl;
else *getSimul()->_output << flush;
}

// native int currentExecution(void);
//
void Simulator::stCurrentExecution(Context * context) // static
{
context->returnInteger(context->getExecution()->getID());
}

// native void abort(int execution);
//
void Simulator::stAbort(Context * context) // static
{
getSimul()->abortExecution(context->getInteger(0));
}

// native void suspend(int execution);
//
void Simulator::stSuspend(Context * context) // static
{
getSimul()->suspendExecution(context->getInteger(0));
}

// native void resume(int execution);
//
void Simulator::stResume(Context * context) // static
{
getSimul()->resumeExecution(context->getInteger(0));
}

// native bool isSuspended(int execution);
//
void Simulator::stIsSuspended(Context * context) // static
{
context->returnInteger(getSimul()->isSuspended(context->getInteger(0)));
}

// native int getMaxDepth(void);
//
void Simulator::stGetMaxDepth(Context * context) // static
{
context->returnInteger(getSimul()->getMaxDepth());
}

// native void setMaxDepth(int maxDepth);
//
void Simulator::stSetMaxDepth(Context * context) // static
{
getSimul()->setMaxDepth(context->getInteger(0));
}

// native int getClock(void);
//
void Simulator::stGetClock(Context * context) // static
{
context->returnInteger(getSimul()->getClock());
}

// native void setClock(int clk); // not documented
//
void Simulator::stSetClock(Context * context) // static
{
getSimul()->setClock(context->getInteger(0));
}

// native Object create(string objName); // not documented
//
void Simulator::stCreate(Context * context) // static
{
AgentName agtName(context->getStrings(0));
Model * model=getSimul()->accessModel(agtName.getModelName());
if(model)
  {
  if(model->getDeleteModule())
    {
    Value * result=TmpValue::getTmpValue(ValueType(model));
    getSimul()->createAgent(result,false,               // not transient
                            model,agtName.getInstNum());
    context->returnValue(result);
    }
  else
    {
    context->returnValue(TmpValue::getTmpValue(context->getModule()
                                             ->getResultType())); // not used
    Strings message="Class '"+
                    agtName.getModelName().getStrings()+
                    "' should have a destructor";
    getSimul()->manageError(new Error(Error::RUNTIME,message));
    }
  }
else
  {
  context->returnValue(TmpValue::getTmpValue(context->getModule()
                                             ->getResultType())); // not used
  Strings message="Class '"+
                  agtName.getModelName().getStrings()+
                  "' is unknown";
  getSimul()->manageError(new Error(Error::RUNTIME,message));
  }
}

// native Object getObject(string objName);
//
void Simulator::stGetObject(Context * context) // static
{
AgentName agtName(context->getStrings(0));
Agent * agent=getSimul()->accessAgent(agtName);
if(agent)
  context->returnAgent(agent,agent->accessModel());
else
  context->returnAgent((Agent *)0,Model::getBaseModel());
}

// native bool exist(Object obj);
//
void Simulator::stExist(Context * context) // static
{
const Agent * agt=context->getAgent(0);
context->returnInteger(agt&&agt->isAlive());
}

// native int numberOf(string className);
//
void Simulator::stNumberOf(Context * context) // static
{
const Word & name=Word::create(context->getStrings(0));
if(name.getStrings().len())
  {
  const Model * model=getSimul()->getModel(name);
  if(model) context->returnInteger(model->getNbAgents());
  else context->returnInteger(-1);
  }
else context->returnInteger(getSimul()->getNbAgents());
}

// native Object[] getAll(string className);
//
void Simulator::stGetAll(Context * context) // static
{
const Word & modelName=Word::create(context->getStrings(0));
const AgentMap * m=(const AgentMap *)0;
Model * model=(Model *)0;
if(modelName.getStrings().len())
  {
  model=getSimul()->accessModel(modelName);
  if(model) m=&model->getAgentMap();
  }
else
  {
  model=Model::getBaseModel();
  m=&model->getAgentMap(); // each kind of agent
  }
if(m)
  {
  Value * result=TmpValue::getTmpValue(ValueType(model,1));
  SubValueVect * agtVect=SubValue::getSubValueVect();
  agtVect->resizeIfNeeded(m->size());
  Agent * agt;
  AgentMap::const_iterator i=m->begin();
  for(;!(i==m->end());i++)
    {
    agt=(*i).second;
    if(agt!=context->getOwner())
      {
      agtVect->pushBack(SubValue::getSubValue(ValueType(agt->accessModel()),
                                              result));
      agtVect->accessBack()->setAgent(agt);
      }
    }
  result->setValueVector(agtVect);
  context->returnValue(result);
  }
else
  {
  Value * result=TmpValue::getTmpValue(context->getModule()->getResultType());
  context->returnValue(result);
  }
}

// native string[] getClasses(void);
//
void Simulator::stGetClasses(Context * context) // static
{
Value * result=TmpValue::getTmpValue(context->getModule()->getResultType());
SubValueVect * classVect=SubValue::getSubValueVect();
ModelMap::const_iterator itM=getSimul()->getModelMap().begin();
for(;!(itM==getSimul()->getModelMap().end());itM++)
  {
  classVect->pushBack(SubValue::getSubValue(ValueType(ValueType::STRING),
                                            result));
  classVect->accessBack()->setStrings((*itM).second->getName().getStrings());
  }
result->setValueVector(classVect);
context->returnValue(result);
}

// native int getTime(void);
//
void Simulator::stGetTime(Context * context) // static
{
context->returnInteger(Timer::getTime());
}

// native string getenv(string varName);
//
void Simulator::stGetenv(Context * context) // static
{
char * str=::getenv(context->getStrings(0));
#ifdef __MSDOS__
if(context->getStrings(0)==Strings("CLASSPATH"))
// we automatically add the location of the oRis java classes
  {
  static Strings ClassPath;
  if(str)
    ClassPath=Strings(str)+Strings(";")+ArApplication::classPath;
  else
    ClassPath=ArApplication::classPath;
  str=(char *)((const char *)ClassPath);
  }
#endif  
if(str) context->returnStrings(str);
else context->returnStrings(Strings());
}

// native void putenv(string varName,string value);
//
void Simulator::stPutenv(Context * context) // static
{
Strings str=context->getStrings(0)+"="+context->getStrings(1);
#ifndef __MSDOS__
::putenv((char *)(const char *)str); // const forgotten on Linux/PPC
#else
_putenv(str);
#endif
}

// native string systemName(bool detailed);
//
void Simulator::stSystemName(Context * context) // static
{
#ifndef __MSDOS__
if(context->getInteger(0))
  context->returnStrings(System::launchO("uname -a"));
else
  context->returnStrings(Strings("UNIX"));
#else
if(context->getInteger(0))
  context->returnStrings(System::launchO("ver"));
else
  context->returnStrings(Strings("MSDOS"));
#endif
}

// native void systemYield(void);
//
void Simulator::stSystemYield(Context * /* context */) // static
{
SimpleThread::yield();
}

// native int system(string command);
//
void Simulator::stSystem(Context * context) // static
{
Simulator::embedThread(Simulator::_systemTask,context);
}

void Simulator::_systemTask(Context * context) // static
{
context->returnInteger(System::launch(context->getStrings(0)));
}

// native void systemI(string command,string input);
//
void Simulator::stSystemI(Context * context) // static
{
Simulator::embedThread(Simulator::_systemITask,context);
}

void Simulator::_systemITask(Context * context) // static
{
System::launchI(context->getStrings(0),
                context->getStrings(1));
}

// native string systemO(string command);
//
void Simulator::stSystemO(Context * context) // static
{
Simulator::embedThread(Simulator::_systemOTask,context);
}

void Simulator::_systemOTask(Context * context) // static
{
context->returnStrings(System::launchO(context->getStrings(0)));
}

// native string systemE(string command);
//
void Simulator::stSystemE(Context * context) // static
{
Simulator::embedThread(Simulator::_systemETask,context);
}

void Simulator::_systemETask(Context * context) // static
{
context->returnStrings(System::launchE(context->getStrings(0)));
}

// native string systemIO(string command,string input);
//
void Simulator::stSystemIO(Context * context) // static
{
Simulator::embedThread(Simulator::_systemIOTask,context);
}

void Simulator::_systemIOTask(Context * context) // static
{
context->returnStrings(System::launchIO(context->getStrings(0),
                                        context->getStrings(1)));
}

// native string systemIE(string command,string input);
//
void Simulator::stSystemIE(Context * context) // static
{
Simulator::embedThread(Simulator::_systemIETask,context);
}

void Simulator::_systemIETask(Context * context) // static
{
context->returnStrings(System::launchIE(context->getStrings(0),
                                        context->getStrings(1)));
}

// native string systemOE(string command,string & err);
//
void Simulator::stSystemOE(Context * context) // static
{
Simulator::embedThread(Simulator::_systemOETask,context);
}

void Simulator::_systemOETask(Context * context) // static
{
Strings errorResult;
context->returnStrings(System::launchOE(context->getStrings(0),
                                        errorResult));
context->setStrings(1,errorResult);
}

// native string systemIOE(string command,string input,string & err);
//
void Simulator::stSystemIOE(Context * context) // static
{
Simulator::embedThread(Simulator::_systemIOETask,context);
}

void Simulator::_systemIOETask(Context * context) // static
{
Strings errorResult;
context->returnStrings(System::launchIOE(context->getStrings(0),
                                         context->getStrings(1),
                                         errorResult));
context->setStrings(2,errorResult);
}

// native string control(string command,string argument);
//
void Simulator::stControl(Context * context) // static
{
if(getSimul()->_controlCB)
  {
  context->returnStrings((*getSimul()->_controlCB)(getSimul(),
                                                   getSimul()->_controller,
                                                   context->getStrings(0),
                                                   context->getStrings(1)));
  }
else context->returnStrings("");
}

// native string[] commandLine(bool full);
//
void Simulator::stCommandLine(Context * context) // static
{
Value * result=TmpValue::getTmpValue(context->getModule()->getResultType());
SubValueVect * argVect=SubValue::getSubValueVect();
unsigned int i=2;
if(context->getInteger(0)) i=0;
for(;i<getSimul()->_commandLine.size();i++)
  {
  argVect->pushBack(SubValue::getSubValue(ValueType(ValueType::STRING),
                                          result));
  argVect->accessBack()->setStrings(getSimul()->_commandLine[i]);
  }
result->setValueVector(argVect);
context->returnValue(result);
}

// native int asciiCode(string char);
//
void Simulator::stAsciiCode(Context * context) // static
{
context->returnInteger((unsigned char)(context->getStrings(0)[0]));
}

// native string asciiChar(int code);
//
void Simulator::stAsciiChar(Context * context) // static
{
context->returnStrings(Strings((char)context->getInteger(0),(size_t)1));
}

// native string hexaString(int value);
//
void Simulator::stHexaString(Context * context) // static
{
static char buffer[0x10];
sprintf(buffer,"%lX",context->getInteger(0));
context->returnStrings(buffer);
}

// native string octalString(int value);
//
void Simulator::stOctalString(Context * context) // static
{
static char buffer[0x10];
sprintf(buffer,"%lo",context->getInteger(0));
context->returnStrings(buffer);
}

// native string binaryString(int value);
//
void Simulator::stBinaryString(Context * context) // static
{
static char buffer[33];
long value=context->getInteger(0);
for(int bit=32;bit--;)
  {
  buffer[31-bit]=(value&(1<<bit) ? '1' : '0');
  }
buffer[32]='\0';
context->returnStrings(buffer);
}

// native void showURL(string url,bool newWindow);
//
void Simulator::stShowURL(Context * context) // static
{
getSimul()->showURL(context->getStrings(0),context->getInteger(1));
}

// native string locateFile(string fileName);
//
void Simulator::stLocateFile(Context * context) // static
{
context->returnStrings(getSimul()->locateFile(context->getStrings(0)));
}

// native string readln(void);
//
void Simulator::stReadln(Context * context) // static
{
Simulator::embedThread(Simulator::_readlnTask,context);
}

void Simulator::_readlnTask(Context * context) // static
{
getSimul()->_readlnMutex->lock();
Strings str;
int c;
while(!feof(stdin))
  {
  c=getc(stdin);
  if((c=='\n')||(c==EOF)) break;
  else str+=c;
  }
context->returnStrings(str);
getSimul()->_readlnMutex->unlock();
}

void Simulator::_notifyChange(const Word & method,const Word & param)
{
Model * model=getSimul()->accessModel(_globalDescriptionWord);
if(model)
  {
  Agent * descr;
  Execution * initExec;
  ExecModule * module;
  Value * value;
  StlVector<AgentName> agtVect;
  AgentMap::const_iterator it=model->getAgentMap().begin();
  for(;it!=model->getAgentMap().end();it++)
    agtVect.push_back((*it).first);
  while(agtVect.size())
    {
    descr=getSimul()->accessAgent(agtVect.back());
    agtVect.pop_back();
    if(descr)
      {
      module=descr->accessModule(method);
      if(module->getType()!=ExecModule::PROTOTYPE)
        {
        initExec=Execution::getExecution(descr,ExecModule::NORMAL);
        value=TmpValue::getTmpValue(ValueType(ValueType::STRING));
        value->setStrings(param.getStrings());
        initExec->pushValue(value);
        getSimul()->executeModule(module,descr,initExec);
        Execution::relaxExecution(initExec);
        }
      }
    }
  }
}

