// @(#) ENIB/LI2 (c) - oRis - Version 2.3
//--------------------------------------------------------------------------//
// file        : smaLex.l                                                   //
// description : source file for the ENIB/LI2 class SmaLexer                //
// author(s)   : F. Harrouet                                                //
// copyright   : Laboratoire d'Informatique Industrielle (LI2)              //
//               Ecole Nationale d'Ingenieurs de Brest (ENIB)               //
//               CP 30815 - 29608 Brest Cedex - FRANCE                      //
//               Tel: (33) 02.98.05.66.31 , Fax: (33) 02.98.05.66.29        //
//               e-mail: li2@enib.fr                                        //
//--------------------------------------------------------------------------//
// Copyright (c) 1996-2000                                                  //
// Laboratoire d'Informatique Industrielle (LI2)                            //
// Ecole Nationale d'Ingenieurs de Brest (ENIB)                             //
//                                                                          //
// Permission to use, copy, modify, distribute and sell this software       //
// and its documentation for any purpose is hereby granted without fee,     //
// provided that the above copyright notice appear in all copies and        //
// that both that copyright notice and this permission notice appear        //
// in supporting documentation.                                             //
// ENIB/LI2 makes no representations about the suitability of this          //
// software for any purpose.                                                //
// It is provided "as is" without express or implied warranty.              //
//--------------------------------------------------------------------------//

%{
#include <iostream>
#include <stdlib.h>
#include <string.h>

class SmaParser;

#include "oRis/Main/simul.h"
#include "oRis/Main/errors.h"
#include "oRis/Nodes/smaNd.h"

#include "oRis/Main/smaLex.h"
#include "oRis/Main/smaParse.h"

#ifndef __MSDOS__
#define malloc(size) BlockAllocator::allocate(size);
#define free(ptr)    BlockAllocator::free(ptr);
#endif
%}

%name SmaLexer
%define INHERIT : public BlockAllocatedObject

%define IOSTREAM
%define MEMBERS \
        int _nbPack; \
        int _nbComment; \
        bool _replacement; \
        Strings _curStrings; \
        ParserSymbol _curSymbol; \
        SymbolList _symbolList; \
        int getReplacedLex(SmaParser * _parser); \
        int getLex(SmaParser * _parser); \
        int keyWord(SmaParser * parser,const unsigned char * text,int code); \
        int integer(SmaParser * parser,const unsigned char * text,int code); \
        int octal(SmaParser * parser,const unsigned char * text,int code); \
        int hexa(SmaParser * parser,const unsigned char * text,int code); \
        int real(SmaParser * parser,const unsigned char * text,int code); \
        int string(SmaParser * parser,const char * text,int code); \
        int agent(SmaParser * parser,const unsigned char * text,int code); \
        int identifier(SmaParser * parser,const unsigned char * text, \
                       int code); \
        void error(SmaParser * parser,const unsigned char * text);

%define CONSTRUCTOR_PARAM istream * stream
%define CONSTRUCTOR_CODE \
        yyin = stream; \
        _nbPack=0; \
        _nbComment=0; \
        _replacement=false;
%define LEX_PARAM SmaParser * _parser

%x commEnv
%x strEnv

space		[ \t\r]+
nl		[\n]
anInteger	[1-9][0-9]*|0
octalInteger	0[0-7]+
hexaInteger	0[xX][0-9A-Fa-f]+
r1		[0-9]*\.[0-9]+
r2		[0-9]+\.[0-9]*
r_exp		[eE][-+]?[0-9]+
aReal		{r1}|{r2}|{r1}{r_exp}|{r2}{r_exp}|{anInteger}{r_exp}
anAgent		[A-Za-z_][0-9a-zA-Z_]*\.[0-9]+
anIdent		[A-Za-z_][0-9a-zA-Z_]*

%%

 //-------- Punctuation --------------------------------------------------//

"{"		{return(keyWord(_parser,yytext,SmaParser::LB));}
"}"		{return(keyWord(_parser,yytext,SmaParser::RB));}
"("		{return(keyWord(_parser,yytext,SmaParser::LP));}
")"		{return(keyWord(_parser,yytext,SmaParser::RP));}
"["		{return(keyWord(_parser,yytext,SmaParser::LH));}
"]"		{return(keyWord(_parser,yytext,SmaParser::RH));}
"."		{return(keyWord(_parser,yytext,SmaParser::POINT));}
","		{return(keyWord(_parser,yytext,SmaParser::COMMA));}
";"		{return(keyWord(_parser,yytext,SmaParser::SEMICOLON));}
"::"		{return(keyWord(_parser,yytext,SmaParser::RANGE));}
":"		{return(keyWord(_parser,yytext,SmaParser::COLON));}
"->"		{return(keyWord(_parser,yytext,SmaParser::ARROW));}

 //-------- Global KeyWords ---------------------------------------------//

"include"	{return(keyWord(_parser,yytext,SmaParser::INCLUDE));}
"#include"	{return(keyWord(_parser,yytext,SmaParser::INCLUDE));}
"replace"	{return(keyWord(_parser,yytext,SmaParser::REPLACE));}
"#replace"	{return(keyWord(_parser,yytext,SmaParser::REPLACE));}
"by"		{return(keyWord(_parser,yytext,SmaParser::BY));}
"beginPackage"	{return(keyWord(_parser,yytext,SmaParser::BEGIN_PACK));}
"#beginPackage"	{return(keyWord(_parser,yytext,SmaParser::BEGIN_PACK));}
"endPackage"	{return(keyWord(_parser,yytext,SmaParser::END_PACK));}
"#endPackage"	{return(keyWord(_parser,yytext,SmaParser::END_PACK));}
"native"	{return(keyWord(_parser,yytext,SmaParser::NATIVE));}
"return"	{return(keyWord(_parser,yytext,SmaParser::RETURN));}
"int"		{return(keyWord(_parser,yytext,SmaParser::INTEGER));}
"float"		{return(keyWord(_parser,yytext,SmaParser::REAL));}
"string"	{return(keyWord(_parser,yytext,SmaParser::STRING));}
"void"		{return(keyWord(_parser,yytext,SmaParser::VOID));}
"execute"	{return(keyWord(_parser,yytext,SmaParser::EXECUTE));}
"start"		{return(keyWord(_parser,yytext,SmaParser::START));}
"force"		{return(keyWord(_parser,yytext,SmaParser::NOATTRCB));}

 //-------- Model KeyWords ----------------------------------------------//

"class"		{return(keyWord(_parser,yytext,SmaParser::CLASS));}
"new"		{return(keyWord(_parser,yytext,SmaParser::NEW));}
"NEW"		{return(keyWord(_parser,yytext,SmaParser::BIG_NEW));}
"main"		{return(keyWord(_parser,yytext,SmaParser::MAIN));}
"delete"	{return(keyWord(_parser,yytext,SmaParser::DELETE));}

 //-------- Instructions ------------------------------------------------//

"while"		{return(keyWord(_parser,yytext,SmaParser::WHILE));}
"do"		{return(keyWord(_parser,yytext,SmaParser::DO));}
"for"		{return(keyWord(_parser,yytext,SmaParser::FOR));}
"if"		{return(keyWord(_parser,yytext,SmaParser::IF));}
"else"		{return(keyWord(_parser,yytext,SmaParser::ELSE));}

"valueDump"	{return(keyWord(_parser,yytext,SmaParser::DUMP));}
"valueParse"	{return(keyWord(_parser,yytext,SmaParser::VALUE_OF));}

"insert"	{return(keyWord(_parser,yytext,SmaParser::INSERT));}
"pushFront"	{return(keyWord(_parser,yytext,SmaParser::PUSHFRONT));}
"pushBack"	{return(keyWord(_parser,yytext,SmaParser::PUSHBACK));}
"remove"	{return(keyWord(_parser,yytext,SmaParser::REMOVE));}
"popFront"	{return(keyWord(_parser,yytext,SmaParser::POPFRONT));}
"popBack"	{return(keyWord(_parser,yytext,SmaParser::POPBACK));}
"erase"		{return(keyWord(_parser,yytext,SmaParser::ERASE));}
"sort"		{return(keyWord(_parser,yytext,SmaParser::SORT));}
"front"		{return(keyWord(_parser,yytext,SmaParser::FRONT));}
"back"		{return(keyWord(_parser,yytext,SmaParser::BACK));}

"format"	{return(keyWord(_parser,yytext,SmaParser::FORMAT));}
"print"		{return(keyWord(_parser,yytext,SmaParser::PRINT));}
"println"	{return(keyWord(_parser,yytext,SmaParser::PRINTLN));}
"error"		{return(keyWord(_parser,yytext,SmaParser::ERROR));}
"warning"	{return(keyWord(_parser,yytext,SmaParser::WARNING));}

"length"	{return(keyWord(_parser,yytext,SmaParser::LENGTH));}
"seekMin"	{return(keyWord(_parser,yytext,SmaParser::SEEKMIN));}
"seekMax"	{return(keyWord(_parser,yytext,SmaParser::SEEKMAX));}
"seek"		{return(keyWord(_parser,yytext,SmaParser::SEEK));}
"deepSeek"	{return(keyWord(_parser,yytext,SmaParser::DEEPSEEK));}

 //-------- Logical ------------------------------------------------------//

"=="       	{return(keyWord(_parser,yytext,SmaParser::EQ));}
":=="       	{return(keyWord(_parser,yytext,SmaParser::DEEPEQ));}
"!="       	{return(keyWord(_parser,yytext,SmaParser::DIF));}
":!="       	{return(keyWord(_parser,yytext,SmaParser::DEEPDIF));}
"<?"       	{return(keyWord(_parser,yytext,SmaParser::MIN));}
">?"       	{return(keyWord(_parser,yytext,SmaParser::MAX));}
"<"       	{return(keyWord(_parser,yytext,SmaParser::INF));}
">"       	{return(keyWord(_parser,yytext,SmaParser::SUP));}
"<="       	{return(keyWord(_parser,yytext,SmaParser::INFEQ));}
">="       	{return(keyWord(_parser,yytext,SmaParser::SUPEQ));}
"!"       	{return(keyWord(_parser,yytext,SmaParser::NOT));}
"&&"       	{return(keyWord(_parser,yytext,SmaParser::AND));}
"||"       	{return(keyWord(_parser,yytext,SmaParser::OR));}

 //-------- Maths --------------------------------------------------------//

"="         	{return(keyWord(_parser,yytext,SmaParser::AFFECT));}
":="         	{return(keyWord(_parser,yytext,SmaParser::DEEPAFFECT));}
"++"       	{return(keyWord(_parser,yytext,SmaParser::INCREASE));}
"+="       	{return(keyWord(_parser,yytext,SmaParser::PLUSEQ));}
"+"       	{return(keyWord(_parser,yytext,SmaParser::PLUS));}
"--"       	{return(keyWord(_parser,yytext,SmaParser::DECREASE));}
"-="       	{return(keyWord(_parser,yytext,SmaParser::MINUSEQ));}
"-"       	{return(keyWord(_parser,yytext,SmaParser::MINUS));}
"*="       	{return(keyWord(_parser,yytext,SmaParser::MULTEQ));}
"*"       	{return(keyWord(_parser,yytext,SmaParser::MULT));}
"/="       	{return(keyWord(_parser,yytext,SmaParser::DIVEQ));}
"/"       	{return(keyWord(_parser,yytext,SmaParser::DIV));}
"%="       	{return(keyWord(_parser,yytext,SmaParser::MODEQ));}
"%"       	{return(keyWord(_parser,yytext,SmaParser::MODULO));}

 //-------- Bitwise ------------------------------------------------------//

"~"       	{return(keyWord(_parser,yytext,SmaParser::BITNOT));}
"&="       	{return(keyWord(_parser,yytext,SmaParser::BITANDEQ));}
"&"		{return(keyWord(_parser,yytext,SmaParser::BITAND));}
"|="		{return(keyWord(_parser,yytext,SmaParser::BITOREQ));}
"|"		{return(keyWord(_parser,yytext,SmaParser::BITOR));}
"^="		{return(keyWord(_parser,yytext,SmaParser::BITXOREQ));}
"^"		{return(keyWord(_parser,yytext,SmaParser::BITXOR));}
"<<="		{return(keyWord(_parser,yytext,SmaParser::LSHIFTEQ));}
"<<"		{return(keyWord(_parser,yytext,SmaParser::LSHIFT));}
">>="		{return(keyWord(_parser,yytext,SmaParser::RSHIFTEQ));}
">>"		{return(keyWord(_parser,yytext,SmaParser::RSHIFT));}

 //-------- Others -------------------------------------------------------//

"NONE"       	{return(keyWord(_parser,yytext,SmaParser::NONE));}
"?"       	{return(keyWord(_parser,yytext,SmaParser::RAND));}
"this"       	{return(keyWord(_parser,yytext,SmaParser::THIS));}
"that"       	{return(keyWord(_parser,yytext,SmaParser::THAT));}

{space}		{_parser->store((const char *)yytext);}
{nl}		{_parser->nextLine();}
{anIdent}	{return(identifier(_parser,yytext,SmaParser::IDENT));}

"#!".*    	{/* ignore */;}
"//".*    	{/* ignore */;}
"/*"    	{
                _parser->store((const char *)yytext);
                _nbComment++;
                BEGIN(commEnv);
                }
<commEnv>"/*"	{
                _parser->store((const char *)yytext);
                _nbComment++;
                }
<commEnv>"*/"	{
                _parser->store((const char *)yytext);
                if(!--_nbComment) BEGIN(INITIAL);
                }
<commEnv>"\n"	{_parser->nextLine();}
<commEnv><<EOF>> {
                _parser->store((const char *)yytext);
                BEGIN(INITIAL);
                return(0);
                }
<commEnv>.	{
                _parser->store((const char *)yytext);
                /* ignore */;
                }

"\""		{_curStrings=""; BEGIN(strEnv);}
<strEnv>"\""	{
                BEGIN(INITIAL);
                return(string(_parser,_curStrings,SmaParser::STRING_VAL));
                }
<strEnv>"\\a"	{_curStrings+="\a";}
<strEnv>"\\b"	{_curStrings+="\b";}
<strEnv>"\\f"	{_curStrings+="\f";}
<strEnv>"\\n"	{_curStrings+="\n";}
<strEnv>"\\r"	{_curStrings+="\r";}
<strEnv>"\\t"	{_curStrings+="\t";}
<strEnv>"\\v"	{_curStrings+="\v";}
<strEnv>"\\\\"	{_curStrings+="\\";}
<strEnv>"\\\""	{_curStrings+="\"";}
<strEnv>"\n"	{_curStrings+="\n";_parser->nextLine();}
<strEnv>"\\\n"	{_curStrings+="\n";_parser->nextLine();}
<strEnv>"\\".	{_curStrings+=yytext[1];}
<strEnv><<EOF>>	{
                BEGIN(INITIAL);
                return(string(_parser,_curStrings,SmaParser::STRING_VAL));
                }
<strEnv>.	{_curStrings+=(const char *)yytext;}

{aReal}  	{return(real(_parser,yytext,SmaParser::REAL_VAL));}
{anInteger}	{return(integer(_parser,yytext,SmaParser::INTEGER_VAL));}
{octalInteger}	{return(octal(_parser,yytext,SmaParser::INTEGER_VAL));}
{hexaInteger}	{return(hexa(_parser,yytext,SmaParser::INTEGER_VAL));}
{anAgent}  	{return(agent(_parser,yytext,SmaParser::AGENT_VAL));}

.		{error(_parser,yytext);}

 //----------------------------------------------------------------------//

%%

int SmaLexer::getReplacedLex(SmaParser * _parser)
{
int code;
do
  {
  if(_symbolList.empty())
    {
    code=yylex(_parser);
    if(code) _symbolList.push_back(_curSymbol);
    else return(0);
    }
  if(!_replacement)
    {
    const SymbolVect * symbolVect;
    do
      {
      code=_symbolList.front().code;
      if(code==SmaParser::IDENT)
        {
        symbolVect=getSimul()->getSymbolVect(_symbolList.front()
                                                        .value.getStrings());
        if(symbolVect)
          {
          _symbolList.pop_front();
          for(int i=symbolVect->size();i--;)
            _symbolList.push_front((*symbolVect)[i]);
          }
        }
      else symbolVect=(const SymbolVect *)0;
      } while(symbolVect&&_symbolList.size());
    }
  } while(_symbolList.empty());
code=_symbolList.front().code;
_parser->yylval=_symbolList.front();
_symbolList.pop_front();
if(code==SmaParser::IDENT)
  {
  Model * model=getSimul()->accessModel(
                Word::create(_parser->yylval.value.getStrings()));
  if(model)
    {
    _parser->yylval.model=model;
    code=_parser->yylval.code=SmaParser::MODELTYPE;
    }
  }
if(_replacement)
  {
  if(code==SmaParser::SEMICOLON) _replacement=false;
  }
else
  {
  if(code==SmaParser::REPLACE) _replacement=true;
  }
return(code);
}

int SmaLexer::getLex(SmaParser * _parser)
{
int code=getReplacedLex(_parser);
#ifdef __DEEPDEBUG__
extern bool AReViDebug ;
if (AReViDebug)
  cerr << "Lex ----> " << _parser->yylval.value << endl ;
#endif
switch(code)
  {
  case SmaParser::END_PACK:
    if(_nbPack) _nbPack--;
    else _parser->yyerror("No package to end");
    return(getLex(_parser));
  case SmaParser::BEGIN_PACK:
    {
    Strings packName;
    code=getReplacedLex(_parser);
    switch(code)
      {
      case SmaParser::IDENT:
        packName=_parser->yylval.value.getStrings();
        break;
      case SmaParser::MODELTYPE:
        packName=_parser->yylval.model->getName().getStrings();
        break;
      default:
        _parser->yyerror("parse error");
        return(getLex(_parser));
      }
    if(getSimul()->hasPackage(packName))
      {
      int pack=_nbPack;
      _nbPack++;
      do
        {
        code=getReplacedLex(_parser);
        if(code==SmaParser::BEGIN_PACK) _nbPack++;
        else if(code==SmaParser::END_PACK) _nbPack--;
        } while(code&&(_nbPack>pack));
      if(code) return(getLex(_parser));
      else return(code);
      }
    else
      {
      _nbPack++;
      getSimul()->addPackage(packName);
      return(getLex(_parser));
      }
    }
  default:
    return(code);
  }
}

int SmaLexer::keyWord(SmaParser * parser,const unsigned char * text,int code)
{
parser->store((const char *)text);
_curSymbol.code=code;
_curSymbol.value.setType(ValueType(ValueType::STRING));
_curSymbol.value.setStrings((const char *)text);
return(code);
}

int SmaLexer::integer(SmaParser * parser,const unsigned char * text,int code)
{
parser->store((const char *)text);
_curSymbol.code=code;
_curSymbol.value.setType(ValueType(ValueType::INTEGER));
_curSymbol.value.setInteger(atol((const char *)text));
return(code);
}

int SmaLexer::octal(SmaParser * parser,const unsigned char * text,int code)
{
parser->store((const char *)text);
_curSymbol.code=code;
_curSymbol.value.setType(ValueType(ValueType::INTEGER));
long nb;
sscanf((const char *)text,"%lo",&nb);
_curSymbol.value.setInteger(nb);
return(code);
}

int SmaLexer::hexa(SmaParser * parser,const unsigned char * text,int code)
{
parser->store((const char *)text);
_curSymbol.code=code;
_curSymbol.value.setType(ValueType(ValueType::INTEGER));
long nb;
sscanf((const char *)text,"%lx",&nb);
_curSymbol.value.setInteger(nb);
return(code);
}

int SmaLexer::real(SmaParser * parser,const unsigned char * text,int code)
{
parser->store((const char *)text);
_curSymbol.code=code;
_curSymbol.value.setType(ValueType(ValueType::REAL));
_curSymbol.value.setReal(atof((const char *)text));
return(code);
}

int SmaLexer::string(SmaParser * parser,const char * text,int code)
{
parser->store(Strings(text).dump());
_curSymbol.code=code;
_curSymbol.value.setType(ValueType(ValueType::STRING));
_curSymbol.value.setStrings(text);
return(code);
}

int SmaLexer::agent(SmaParser * parser,const unsigned char * text,int code)
{
parser->store((const char *)text);
AgentName agtName((const char *)text);
Agent * agent=getSimul()->accessAgent(agtName);
if(agent&&agent->isAlive())
  {
  _curSymbol.value.setType(ValueType(agent->accessModel()));
  _curSymbol.value.setAgent(agent);
  }
else
  {
  Strings message="Object '"+
                  agtName.print()+
                  "' is unknown";
  parser->manageError(new Error(Error::SEMANTICS,message));
  code=SmaParser::NONE;
  _curSymbol.value.setType(ValueType(ValueType::STRING));
  _curSymbol.value.setStrings("NONE");
  }
_curSymbol.code=code;
return(code);
}

int SmaLexer::identifier(SmaParser * parser,const unsigned char * text,
                         int code)
{
parser->store((const char *)text);
_curSymbol.code=code;
_curSymbol.value.setType(ValueType(ValueType::STRING));
_curSymbol.value.setStrings((const char *)text);
return(code);
}

void SmaLexer::error(SmaParser * parser,const unsigned char * text)
{
parser->store((const char *)text);
parser->yyerror("lexical error");
}

