// @(#) ENIB/LI2 (c) - oRis - Version 2.3
//--------------------------------------------------------------------------//
// file        : smaParse.y                                                 //
// description : source file for the ENIB/LI2 class SmaParser               //
// author(s)   : F. Harrouet                                                //
// copyright   : Laboratoire d'Informatique Industrielle (LI2)              //
//               Ecole Nationale d'Ingenieurs de Brest (ENIB)               //
//               CP 30815 - 29608 Brest Cedex - FRANCE                      //
//               Tel: (33) 02.98.05.66.31 , Fax: (33) 02.98.05.66.29        //
//               e-mail: li2@enib.fr                                        //
//--------------------------------------------------------------------------//
// Copyright (c) 1996-2000                                                  //
// Laboratoire d'Informatique Industrielle (LI2)                            //
// Ecole Nationale d'Ingenieurs de Brest (ENIB)                             //
//                                                                          //
// Permission to use, copy, modify, distribute and sell this software       //
// and its documentation for any purpose is hereby granted without fee,     //
// provided that the above copyright notice appear in all copies and        //
// that both that copyright notice and this permission notice appear        //
// in supporting documentation.                                             //
// ENIB/LI2 makes no representations about the suitability of this          //
// software for any purpose.                                                //
// It is provided "as is" without express or implied warranty.              //
//--------------------------------------------------------------------------//

// There is a Shift/Reduce conflict in IF ... ELSE
// It is normal, the only way to suppress it is to use a ENDIF
// or to impose LB ... RB
// Bison++ takes the decision to Reduce; this choice is the good one.

%{
#include <fstream>
#include <string.h>
#include <math.h>

class SmaParser;

#include "oRis/Nodes/nodes.h"
#include "oRis/Main/simul.h"
#include "oRis/Main/context.h"
#include "oRis/Main/errors.h"
#include "oRis/Main/execMod.h"
#include "oRis/Main/protoMod.h"
#include "oRis/Main/extMod.h"

#include "oRis/Main/parser.h"
#include "oRis/Main/smaLex.h"

#ifndef __MSDOS__
#define malloc(size) BlockAllocator::allocate(size);
#define free(ptr)    BlockAllocator::free(ptr);
#endif
%}

%name SmaParser
%define INHERIT : public BlockAllocatedObject

%define MEMBERS \
        SmaParser * _calling; \
        SmaLexer * _scanner; \
        Strings _fileName; \
        istream * _stream; \
        int _lineNum; \
        Strings _line; \
        SymbolVect _symbolVect; \
        Agent * _curAgent; \
        Model * _curModel; \
        ExecModule::SpecialType _curModuleType; \
        ExecModVect _curModules; \
        SimpleVector<Context *> _curContexts; \
        ValueType _curAttrType; \
        Parameter::ParamMode _curParamMode; \
        NodeVect _curNodes; \
        void init(Strings fileName,istream *stream,SmaParser * calling); \
        virtual ~SmaParser(void); \
        bool fileInUse(const Strings & fileName); \
        void setModule(void); \
        void setModuleNode(ExecModule * module,SmaNode * node); \
        void manageError(Error * error); \
        void nextLine(void); \
        void store(const char * text);

%define CONSTRUCTOR_PARAM \
        Strings fileName, \
        istream * stream, \
        SmaParser * calling

%define CONSTRUCTOR_CODE init(fileName,stream,calling);

%define LEX_BODY {return(_scanner->getLex(this));}

//-------- Tokens ----------------------------------------------------------//

// punctuation
%token LB RB LP RP LH RH POINT COMMA SEMICOLON RANGE COLON ARROW

// Global KeyWords
%token INCLUDE REPLACE BY BEGIN_PACK END_PACK
%token NATIVE RETURN
%token INTEGER REAL STRING VOID MODELTYPE
%token EXECUTE START NOATTRCB

// Model KeyWords
%token CLASS NEW BIG_NEW MAIN DELETE

// Instructions
%token WHILE DO FOR IF ELSE
%token DUMP VALUE_OF
%token INSERT PUSHFRONT PUSHBACK REMOVE POPFRONT POPBACK
%token ERASE SORT FRONT BACK
%token FORMAT PRINT PRINTLN ERROR WARNING
%token LENGTH SEEK SEEKMIN SEEKMAX DEEPSEEK

// Logical
%token EQ DEEPEQ DIF DEEPDIF MIN MAX INF SUP INFEQ SUPEQ NOT AND OR

// Maths
%token AFFECT DEEPAFFECT INCREASE PLUSEQ PLUS DECREASE MINUSEQ MINUS
%token MULTEQ MULT DIVEQ DIV MODEQ MODULO

// Bitwise
%token BITNOT BITANDEQ BITAND BITOREQ BITOR BITXOREQ BITXOR
%token LSHIFTEQ LSHIFT RSHIFTEQ RSHIFT

// Others
%token IDENT INTEGER_VAL REAL_VAL STRING_VAL AGENT_VAL
%token NONE RAND THIS THAT

//-------- Token types -----------------------------------------------------//

%define STYPE ParserSymbol

//-------- Priorities ------------------------------------------------------//

%nonassoc DELETE

%right    AFFECT DEEPAFFECT PLUSEQ MINUSEQ MULTEQ DIVEQ MODEQ BITANDEQ BITOREQ BITXOREQ LSHIFTEQ RSHIFTEQ

%right    RAND COLON

%left     MIN MAX

%left     OR
%left     AND

%left     BITOR
%left     BITXOR
%left     BITAND

%left     EQ DEEPEQ DIF DEEPDIF
%left     SUPEQ SUP INFEQ INF

%left     LSHIFT RSHIFT

%left     PLUS MINUS
%left     MULT DIV MODULO

%right    NOT BITNOT INCREASE DECREASE UPLUS UMINUS CAST

%left     ARROW POINT LH

%start program

%%

//-------- Program ---------------------------------------------------------//

program		: progElems
		| // empty
;

progElems	: progElem
		| progElems progElem
;

progElem	: replace
		| include
		| execModule
		| model
		| modelAttribute
		| agentAttribute
		| execute
		| SEMICOLON
		| error SEMICOLON
                   {
                   yyerrok;
                   setModule();
                   }
		| error RB
                   {
                   yyerrok;
                   setModule();
                   }
;

//-------- Replace ---------------------------------------------------------//

replace		: REPLACE identModelType BY replacementList SEMICOLON
                    {
                    getSimul()->replaceSymbol($2.value.getStrings(),
                                              &_symbolVect);
                    _symbolVect.erase(_symbolVect.begin(),_symbolVect.end());
                    }
		| REPLACE identModelType SEMICOLON
                    {
                    getSimul()->replaceSymbol($2.value.getStrings());
                    }
		| REPLACE error SEMICOLON {yyerrok;}
;

identModelType	: IDENT
                    {
                    $$.value.setType(ValueType(ValueType::STRING));
                    $$.value.setStrings($1.value.getStrings());
                    }
		| MODELTYPE
                    {
                    $$.value.setType(ValueType(ValueType::STRING));
                    $$.value.setStrings($1.model->getName().getStrings());
                    }
;

replacementList : replacementList replacement
                    {
                    _symbolVect.push_back($2);
                    }
		| // empty
                    {
                    _symbolVect.erase(_symbolVect.begin(),_symbolVect.end());
                    }
;

replacement	: // punctuation (without SEMICOLON)
		  LB | RB | LP | RP | LH | RH | POINT | COMMA
		| RANGE | COLON | ARROW
		  // Global KeyWords
		| INCLUDE | REPLACE | BY | BEGIN_PACK | END_PACK
		| NATIVE | RETURN
		| INTEGER | REAL | STRING | VOID | MODELTYPE
		| EXECUTE | START | NOATTRCB
		  // Model KeyWords
		| CLASS | NEW | BIG_NEW | MAIN | DELETE
		  // Instructions
		| WHILE | DO | FOR | IF | ELSE
		| DUMP | VALUE_OF
		| INSERT | PUSHFRONT | PUSHBACK | REMOVE | POPFRONT | POPBACK
		| ERASE | SORT | FRONT | BACK
		| FORMAT | PRINT | PRINTLN | ERROR | WARNING
		| LENGTH | SEEK | SEEKMIN | SEEKMAX | DEEPSEEK
		  // Logical
		| EQ | DEEPEQ | DIF | DEEPDIF | MIN | MAX
		| INF | SUP | INFEQ | SUPEQ | NOT | AND | OR
		  // Maths
		| AFFECT | DEEPAFFECT | INCREASE | PLUSEQ | PLUS | DECREASE
		| MINUSEQ | MINUS | MULTEQ | MULT | DIVEQ | DIV
		| MODEQ | MODULO
		  // Bitwise
		| BITNOT | BITANDEQ | BITAND | BITOREQ | BITOR
		| BITXOREQ | BITXOR | LSHIFTEQ | LSHIFT | RSHIFTEQ | RSHIFT
		  // Others
		| IDENT | INTEGER_VAL | REAL_VAL | STRING_VAL | AGENT_VAL
		| NONE | RAND | THIS | THAT
;

//-------- Include ---------------------------------------------------------//

include		: INCLUDE STRING_VAL
                    {
                    Strings fileName=$2.value.getStrings();
                    istream * file=getSimul()->openFile(fileName);
                    if(file)
                      {
                      if(fileInUse(fileName))
                        {
                        Strings message="Can't open file '"+
                                        fileName+
                                        "' (already open)";
                        manageError(new Error(Error::SEMANTICS,message));
                        }
                      else
                        {
                        SmaParser newParser(fileName,file,this);
                        newParser.yyparse();
                        }
                      delete file;
                      }
                    else
                      {
                      Strings message="Can't open file '"+
                                      fileName+"'";
                      manageError(new Error(Error::SEMANTICS,message));
                      }
                    }
		| INCLUDE error
                    {
                    yyerrok;
                    }
;

//-------- Simulator -------------------------------------------------------//

execute		: executeHead
                    {
                    _curContexts.pushBack(Context::getContext(
                                                   _curModules.accessBack(),
                                                   _curAgent,
                                                   (Context *)0));
                    }
                  algoBody
;

executeHead	: EXECUTE
                    {
                    _curModel=(Model *)0;
                    _curAgent=(Agent *)0;
                    Word moduleName(Word::create($1.value.getStrings()));
                    _curModules.pushBack(new ExecModule(moduleName,
                                                        ExecModule::EXECUTE,
                                                        ValueType(),
                                                        _curModel,_curAgent));
                    }
		| returnType EXECUTE
                    {
                    _curModel=(Model *)0;
                    _curAgent=(Agent *)0;
                    Word moduleName(Word::create($2.value.getStrings()));
                    _curModules.pushBack(new ExecModule(moduleName,
                                                        ExecModule::EXECUTE,
                                                        _curAttrType,
                                                        _curModel,_curAgent));
                    }
		| RANGE EXECUTE
                    {
                    _curModel=(Model *)0;
                    _curAgent=(Agent *)0;
                    Word moduleName(Word::create($2.value.getStrings()));
                    _curModules.pushBack(new ExecModule(moduleName,
                                                        ExecModule::EXECUTE,
                                                        ValueType(),
                                                        _curModel,_curAgent));
                    }
		| returnType RANGE EXECUTE
                    {
                    _curModel=(Model *)0;
                    _curAgent=(Agent *)0;
                    Word moduleName(Word::create($3.value.getStrings()));
                    _curModules.pushBack(new ExecModule(moduleName,
                                                        ExecModule::EXECUTE,
                                                        _curAttrType,
                                                        _curModel,_curAgent));
                    }
		| AGENT_VAL RANGE EXECUTE
                    {
                    _curModel=(Model *)0;
                    _curAgent=$1.value.getAgent();
                    Word moduleName(Word::create($3.value.getStrings()));
                    _curModules.pushBack(new ExecModule(moduleName,
                                                        ExecModule::EXECUTE,
                                                        ValueType(),
                                                        _curModel,_curAgent));
                    }
		| returnType AGENT_VAL RANGE EXECUTE
                    {
                    _curModel=(Model *)0;
                    _curAgent=$2.value.getAgent();
                    Word moduleName(Word::create($4.value.getStrings()));
                    _curModules.pushBack(new ExecModule(moduleName,
                                                        ExecModule::EXECUTE,
                                                        _curAttrType,
                                                        _curModel,_curAgent));
                    }
		| START
                    {
                    _curModel=(Model *)0;
                    _curAgent=(Agent *)0;
                    Word moduleName(Word::create($1.value.getStrings()));
                    _curModules.pushBack(new ExecModule(moduleName,
                                                        ExecModule::START,
                                                        ValueType(),
                                                        _curModel,_curAgent));
                    }
		| RANGE START
                    {
                    _curModel=(Model *)0;
                    _curAgent=(Agent *)0;
                    Word moduleName(Word::create($2.value.getStrings()));
                    _curModules.pushBack(new ExecModule(moduleName,
                                                        ExecModule::START,
                                                        ValueType(),
                                                        _curModel,_curAgent));
                    }
		| AGENT_VAL RANGE START
                    {
                    _curModel=(Model *)0;
                    _curAgent=$1.value.getAgent();
                    Word moduleName(Word::create($3.value.getStrings()));
                    _curModules.pushBack(new ExecModule(moduleName,
                                                        ExecModule::START,
                                                        ValueType(),
                                                        _curModel,_curAgent));
                    }
;

//-------- Model -----------------------------------------------------------//

model		: modelHead LB modelBody RB
                    {
                    _curModel=(Model *)0;
                    _curAgent=(Agent *)0;
                    }
		| modelHead SEMICOLON
                    {
                    _curModel=(Model *)0;
                    _curAgent=(Agent *)0;
                    }
		| CLASS error RB
                    {
                    yyerrok;
                    _curModel=(Model *)0;
                    _curAgent=(Agent *)0;
                    }
		| CLASS error SEMICOLON
                    {
                    yyerrok;
                    _curModel=(Model *)0;
                    _curAgent=(Agent *)0;
                    }
		| NATIVE CLASS error RB
                    {
                    yyerrok;
                    _curModel=(Model *)0;
                    _curAgent=(Agent *)0;
                    }
		| NATIVE CLASS error SEMICOLON
                    {
                    yyerrok;
                    _curModel=(Model *)0;
                    _curAgent=(Agent *)0;
                    }
;

modelHead	: stdModelHead
		| NATIVE stdModelHead
                    {
                    _curModel->setNative();
                    }
;

stdModelHead	: CLASS identModelType
                    {
                    _curAgent=(Agent *)0;
                    Word modelName(Word::create($2.value.getStrings()));
                    _curModel=getSimul()->accessModel(modelName);
                    if(!_curModel)
                      {
                      _curModel=new Model(modelName);
                      getSimul()->setModel(_curModel,true);
                      }
                    }
                  parents
;

parents		: COLON parentList
		| // empty
		| error SEMICOLON
                    {
                    yyerrok;
                    }
		| error LB
                    {
                    yyerrok;
                    }
;

parentList	: parent
		| parentList COMMA parent
;

parent		: MODELTYPE
		    {
                    Error * error=_curModel->tryToSetParent($1.model);
                    if(error) manageError(error);
                    else _curModel->setParent($1.model,true);
                    }
;

modelBody	: modelBody modelElem
		| // empty
;

modelElem	: SEMICOLON
		| returnType attributeList SEMICOLON
		| modelModule algoBody
		| modelModule SEMICOLON
                    {
                    if(_curModules.getBack()->getType()!=ExecModule::EXTERN)
                      { // its only a prototype
                      ExecModule * tmpModule=_curModules.accessBack()
                                             ->buildPrototype();
                      _curModules.accessBack()->mustDisappear()=true;
                      _curContexts.pushBack(Context::getContext(
                                                     _curModules.accessBack(),
                                                     _curAgent,
                                                     (Context *)0));
                      setModule(); // erase old module
                      _curModules.pushBack(tmpModule);
                      }
                    setModule();
                    }
                | error RB
                    {
                    yyerrok;
                    setModule();
                    }
                | error SEMICOLON
                    {
                    yyerrok;
                    setModule();
                    }
;

modelModule	: returnType moduleName
                    {
                    _curAgent=(Agent *)0;
                    Word moduleName(Word::create($2.value.getStrings()));
                    _curModules.pushBack(new ExecModule(moduleName,
                                                        _curModuleType,
                                                        _curAttrType,
                                                        _curModel,
                                                        _curAgent));
                    _curContexts.pushBack(Context::getContext(
                                                   _curModules.accessBack(),
                                                   _curAgent,
                                                   (Context *)0));
                    }
                  LP params RP
		| native returnType moduleName
                    {
                    Word moduleName(Word::create($3.value.getStrings()));
                    _curModules.pushBack(new ExternModule(moduleName,
                                                          _curModuleType,
                                                          _curAttrType,
                                                          _curModel,_curAgent,
                                                          $1.value
                                                            .getStrings()));
                    _curContexts.pushBack(Context::getContext(
                                                   _curModules.accessBack(),
                                                   _curAgent,
                                                   (Context *)0));
                    if(_curModules.accessBack()->mustDisappear())
                      {
                      Strings message="Native module '"+
                        ((ExternModule *)_curModules.getBack())
                                                    ->getNativeName()+
                        "' not found";
                      manageError(new Error(Error::SEMANTICS,message));
                      }
                    }
                  LP params RP
;


//-------- Attributes ------------------------------------------------------//

returnType	: complexType
		| VOID
                    {
                    _curAttrType=ValueType();
                    }
;

complexType	: simpleType hooks
;

hooks		: hooks LH RH
                    {
                    _curAttrType.setDepth(_curAttrType.getDepth()+1);
                    }
		| // empty
                    {
                    _curAttrType.setDepth(0);
                    }
;

simpleType	: INTEGER
                    {
                    _curAttrType=ValueType(ValueType::INTEGER);
                    }
		| REAL
                    {
                    _curAttrType=ValueType(ValueType::REAL);
                    }
		| STRING
                    {
                    _curAttrType=ValueType(ValueType::STRING);
                    }
		| MODELTYPE
                    {
                    _curAttrType=ValueType($1.model);
                    }
;

attributeList	: attribute
		| attributeList COMMA attribute
;

attribute	: largeIdent
                    {
                    if(_curAttrType==ValueType())
                      {
                      Strings message="Type '"+
                                      _curAttrType.print()+
                                      "' can't be given to '"+
                                      $1.value.getStrings()+
                                      "'";
                      manageError(new Error(Error::SEMANTICS,message));
                      }
                    else
                      {
                      Attribute attr(Word::create($1.value.getStrings()),
                                     _curAttrType,_curModel,_curAgent);
                      Error * error=_curModel->tryToSetAttribute(attr);
                      if(error) manageError(error);
                      else _curModel->setAttribute(attr,true);
                      }
                    }
;

modelAttribute	: returnType MODELTYPE RANGE largeIdent SEMICOLON
                    {
                    _curAgent=(Agent *)0;
                    if(_curAttrType==ValueType())
                      {
                      Strings message="Type '"+
                                      _curAttrType.print()+
                                      "' can't be given to '"+
                                      $2.model->getName().getStrings()+
                                      $3.value.getStrings()+
                                      $4.value.getStrings()+
                                      "'";
                      manageError(new Error(Error::SEMANTICS,message));
                      }
                    else
                      {
                      _curModel=$2.model;
                      Attribute attr(Word::create($4.value.getStrings()),
                                     _curAttrType,_curModel,_curAgent);
                      Error * error=_curModel->tryToSetAttribute(attr);
                      if(error) manageError(error);
                      else _curModel->setAttribute(attr,true);
                      _curModel=(Model *)0;
                      }
                    }
;

agentAttribute	: returnType AGENT_VAL RANGE largeIdent SEMICOLON
                    {
                    _curModel=(Model *)0;
                    if(_curAttrType==ValueType())
                      {
                      Strings message="Type '"+
                                      _curAttrType.print()+
                                      "' can't be given to '"+
                                      $2.model->getName().getStrings()+
                                      $3.value.getStrings()+
                                      $4.value.getStrings()+
                                      "'";
                      manageError(new Error(Error::SEMANTICS,message));
                      }
                    else
                      {
                      _curAgent=$2.value.getAgent();
                      Attribute attr(Word::create($4.value.getStrings()),
                                     _curAttrType,_curModel,_curAgent);
                      Error * error=_curAgent->tryToSetAttribute(attr);
                      if(error) manageError(error);
                      else _curAgent->setAttribute(attr,true);
                      _curAgent=(Agent *)0;
                      }
                    }
;

//-------- Execution Modules -----------------------------------------------//

moduleName	: largeIdent
                    {
                    _curModuleType=ExecModule::NORMAL;
                    $$.value.setType(ValueType(ValueType::STRING));
                    $$.value.setStrings($1.value.getStrings());
                    }
		| NEW
                    {
                    _curModuleType=ExecModule::CONSTRUCTOR;
                    $$.value.setType(ValueType(ValueType::STRING));
                    $$.value.setStrings($1.value.getStrings());
                    }
		| DELETE
                    {
                    _curModuleType=ExecModule::DESTRUCTOR;
                    $$.value.setType(ValueType(ValueType::STRING));
                    $$.value.setStrings($1.value.getStrings());
                    }
		| MAIN
                    {
                    _curModuleType=ExecModule::BEHAVIOR;
                    $$.value.setType(ValueType(ValueType::STRING));
                    $$.value.setStrings($1.value.getStrings());
                    }
;

largeIdent	: IDENT
                    {
                    $$.value.setType(ValueType(ValueType::STRING));
                    $$.value.setStrings($1.value.getStrings());
                    }
		| SEEK
                    {
                    $$.value.setType(ValueType(ValueType::STRING));
                    $$.value.setStrings($1.value.getStrings());
                    }
		| DEEPSEEK
                    {
                    $$.value.setType(ValueType(ValueType::STRING));
                    $$.value.setStrings($1.value.getStrings());
                    }
		| SEEKMIN
                    {
                    $$.value.setType(ValueType(ValueType::STRING));
                    $$.value.setStrings($1.value.getStrings());
                    }
		| SEEKMAX
                    {
                    $$.value.setType(ValueType(ValueType::STRING));
                    $$.value.setStrings($1.value.getStrings());
                    }
		| LENGTH
                    {
                    $$.value.setType(ValueType(ValueType::STRING));
                    $$.value.setStrings($1.value.getStrings());
                    }
		| INSERT
                    {
                    $$.value.setType(ValueType(ValueType::STRING));
                    $$.value.setStrings($1.value.getStrings());
                    }
		| PUSHFRONT
                    {
                    $$.value.setType(ValueType(ValueType::STRING));
                    $$.value.setStrings($1.value.getStrings());
                    }
		| PUSHBACK
                    {
                    $$.value.setType(ValueType(ValueType::STRING));
                    $$.value.setStrings($1.value.getStrings());
                    }
		| REMOVE
                    {
                    $$.value.setType(ValueType(ValueType::STRING));
                    $$.value.setStrings($1.value.getStrings());
                    }
		| POPFRONT
                    {
                    $$.value.setType(ValueType(ValueType::STRING));
                    $$.value.setStrings($1.value.getStrings());
                    }
		| POPBACK
                    {
                    $$.value.setType(ValueType(ValueType::STRING));
                    $$.value.setStrings($1.value.getStrings());
                    }
		| ERASE
                    {
                    $$.value.setType(ValueType(ValueType::STRING));
                    $$.value.setStrings($1.value.getStrings());
                    }
		| SORT
                    {
                    $$.value.setType(ValueType(ValueType::STRING));
                    $$.value.setStrings($1.value.getStrings());
                    }
;

execModule	: execModStart algoBody
		| execModStart SEMICOLON
                    {
                    if(_curModules.getBack()->getType()!=ExecModule::EXTERN)
                      { // its only a prototype
                      ExecModule * tmpModule=_curModules.accessBack()
                                             ->buildPrototype();
                      _curModules.accessBack()->mustDisappear()=true;
                      _curContexts.pushBack(Context::getContext(
                                                     _curModules.accessBack(),
                                                     (Agent *)0,
                                                     (Context *)0));
                      setModule(); // erase old module
                      _curModules.pushBack(tmpModule);
                      }
                    setModule();
                    }
		| returnType error RB
                    {
                    yyerrok;
                    setModule();
                    }
		| returnType error SEMICOLON
                    {
                    yyerrok;
                    setModule();
                    }
		| NATIVE error RB
                    {
                    yyerrok;
                    setModule();
                    }
		| NATIVE error SEMICOLON
                    {
                    yyerrok;
                    setModule();
                    }
;

execModStart	: execModStart2
                    {
                    if((_curModules.getBack()->getSpecialType()==
                        ExecModule::CONSTRUCTOR)&&
                       _curModules.getBack()->getOwner())
                      {
                      Strings message="'"+
                                      _curModules.getBack()->getCompleteName()+
                                      "' will never be used";
                      manageError(new Error(Error::WARNING,message));
                      }
                    }
;

execModStart2	: defModule
                    {
                    Word moduleName(Word::create($1.value.getStrings()));
                    _curModules.pushBack(new ExecModule(moduleName,
                                                        _curModuleType,
                                                        _curAttrType,
                                                        _curModel,_curAgent));
                    _curContexts.pushBack(Context::getContext(
                                                   _curModules.accessBack(),
                                                   _curAgent,
                                                   (Context *)0));
                    }
                  LP params RP
		| native defModule
                    {
                    Word moduleName(Word::create($2.value.getStrings()));
                    _curModules.pushBack(new ExternModule(moduleName,
                                                          _curModuleType,
                                                          _curAttrType,
                                                          _curModel,_curAgent,
                                                          $1.value
                                                            .getStrings()));
                    _curContexts.pushBack(Context::getContext(
                                                   _curModules.accessBack(),
                                                   _curAgent,
                                                   (Context *)0));
                    if(_curModules.accessBack()->mustDisappear())
                      {
                      Strings message="Native module '"+
                        ((ExternModule *)_curModules.getBack())
                                                    ->getNativeName()+
                        "' not found";
                      manageError(new Error(Error::SEMANTICS,message));
                      }
                    }
                  LP params RP
;

native		: NATIVE STRING_VAL
                    {
                    $$.value.setType(ValueType(ValueType::STRING));
                    $$.value.setStrings($2.value.getStrings());
                    }
		| NATIVE
                    {
                    $$.value.setType(ValueType(ValueType::STRING));
                    $$.value.setStrings(Strings());
                    }
;

defModule	: returnType largeIdent
                    {
                    _curModuleType=ExecModule::NORMAL;
                    _curModel=(Model *)0;
                    _curAgent=(Agent *)0;
                    $$.value.setType(ValueType(ValueType::STRING));
                    $$.value.setStrings($2.value.getStrings());
                    }
		| returnType MODELTYPE RANGE moduleName
                    {
                    _curAgent=(Agent *)0;
                    _curModel=$2.model;
                    $$.value.setType(ValueType(ValueType::STRING));
                    $$.value.setStrings($4.value.getStrings());
                    }
		| returnType AGENT_VAL RANGE moduleName
                    {
                    _curModel=(Model *)0;
                    _curAgent=$2.value.getAgent();
                    $$.value.setType(ValueType(ValueType::STRING));
                    $$.value.setStrings($4.value.getStrings());
                    }
;

params		: paramList
		| VOID
		| // empty # void
;

paramList	: param
		| paramList COMMA param
;

param		: complexType paramMode largeIdent
                    {
                    Parameter param(_curParamMode,_curAttrType,
                                    Word::create($3.value.getStrings()));
                    if(_curModules.getBack()->getLocalRank(param.getName())>=0)
                      {
                      Strings message="Parameter '"+
                                      param.getName().getStrings()+
                                      "' is already defined in '"+
                                      _curModules.getBack()->getCompleteName()+
                                      "'";
                      manageError(new Error(Error::SEMANTICS,message));
                      }
                    else _curModules.accessBack()->setParam(param);
                    }
;

paramMode	: BITAND
                    {
                    _curParamMode=Parameter::REFERENCE;
                    }
		| // empty
                    {
                    _curParamMode=Parameter::COPY;
                    }
;

algoBody	: LB instructionList RB
                    {
                    if(_curModules.getBack()->getType()==ExecModule::EXTERN)
                      {
                      delete $2.node;
                      Strings message=
                              "No code can be given to the native module '"+
                              _curModules.getBack()->getCompleteName()+
                              "'";
                      manageError(new Error(Error::SEMANTICS,message));
                      }
                    else
                      {
                      if($2.node->fail())
                        {
                        delete $2.node;
                        _curModules.accessBack()->mustDisappear()=true;
                        }
                      else setModuleNode(_curModules.accessBack(),$2.node);
                      }
                    setModule();
                    }
		| error RB
                    {
                    yyerrok;
                    setModule();
                    }
;

localDeclar	: complexType localVarList
                    {
                    $$.node=$2.node;
                    if($$.node) $$.node->check(_curContexts.accessBack(),
                                               _curModel);
                    }
;

localVarList    : localVar
                    {
                    if($1.node)
                      {
                      $$.node=new SeqNode(_lineNum,_fileName,",");
                      $$.node->addNode($1.node);
                      }
                    else $$.node=(SmaNode *)0;
                    }
		| localVarList COMMA localVar
                    {
                    if($1.node)
                      {
                      if($3.node)
                        {
                        $$.node=$1.node;
                        $$.node->addNode($3.node);
                        }
                      else $$.node=$1.node;
                      }
                    else
                      {
                      if($3.node)
                        {
                        $$.node=new SeqNode(_lineNum,_fileName,",");
                        $$.node->addNode($3.node);
                        }
                      else $$.node=(SmaNode *)0;
                      }
                    }
;

localVar	: localVarName
                    {
                    $$.node=(SmaNode *)0;
                    }
		| localVarName AFFECT expression
                    {
                    SmaNode * var=new VarNode(_lineNum,_fileName,
                                              $1.value.getStrings());
                    var->check(_curContexts.accessBack(),_curModel);
                    $$.node=new AffectNode(_lineNum,_fileName,
                                           $2.value.getStrings(),false);
                    $$.node->setNode(0,var);
                    $$.node->setNode(1,$3.node);
                    $$.node->check(_curContexts.accessBack(),_curModel);
                    }
;

localVarName	: largeIdent
                    {
                    Word name(Word::create($1.value.getStrings()));
                    if(_curModules.getBack()->getLocalRank(name)>=0)
                      {
                      Strings message="Attribute '"+
                                      name.getStrings()+
                                      "' is already declared";
                      manageError(new Error(Error::SEMANTICS,message));
                      }
                    else
                      {
                      _curModules.accessBack()
                                ->setLocal(Attribute(name,_curAttrType,
                                           (Model *)0,(Agent *)0));
                      }
                    $$.value.setType(ValueType(ValueType::STRING));
                    $$.value.setStrings(name.getStrings());
                    }
;

//-------- Instructions ----------------------------------------------------//

instructionList	: instrList2
                    {
                    $$.node=$1.node;
                    $$.node->check(_curContexts.accessBack(),_curModel);
                    }
;

instrList2	: instrList2 instruction
                    {
                    $$.node=$1.node;
                    if($2.node) $$.node->addNode($2.node);
                    }
		| // empty
                    {
                    $$.node=new SeqNode(_lineNum,_fileName,";");
                    }
;

instruction	: SEMICOLON
                    {
                    $$.node=(SmaNode *)0;
                    }
		| EXECUTE LB instructionList RB
                    {
                    $$.node=new CriticalNode(_lineNum,_fileName,
                                             $1.value.getStrings());
                    $3.node->insideCritical();
                    $$.node->setNode(0,$3.node);
                    $$.node->check(_curContexts.accessBack(),_curModel);
                    }
		| START startParam
                    {
                    Word moduleName(Word::create($1.value.getStrings()));
                    ExecModule * tmp=new ExecModule(moduleName,
                                                    ExecModule::START,
                                                    ValueType(),
                                                    _curModel,_curAgent);
                    for(int i=0;i<_curModules.getBack()->getNbLocals();i++)
                      {
                      tmp->setParam(Parameter(Parameter::COPY,
                                              _curModules.getBack()
                                                         ->getLocal(i)
                                                         ->getType(),
                                              _curModules.getBack()
                                                         ->getLocal(i)
                                                         ->getName()));
                      }
                    _curNodes.pushBack(new StartNode(_lineNum,_fileName,
                                                     $1.value.getStrings(),
                                                     tmp));
                    if($2.node) _curNodes.accessBack()->addNode($2.node);
                    _curModules.pushBack(tmp);
                    _curContexts.pushBack(Context::getContext(
                                                   _curModules.accessBack(),
                                                   _curAgent,
                                                   (Context *)0));
                    }
                  LB instructionList RB
                    {
                    if($5.node->fail())
                      {
                      delete $5.node;
                      _curModules.accessBack()->mustDisappear()=true;
                      }
                    else
                      {
                      _curModules.accessBack()->setNode($5.node);
                      }
                    _curModules.popBack();
                    Context::relaxContext(_curContexts.readPopBack());
                    $$.node=_curNodes.readPopBack();
                    $$.node->check(_curContexts.accessBack(),_curModel);
                    }
		| NOATTRCB LB instructionList RB
                    {
                    $$.node=new NoAttrCBNode(_lineNum,_fileName,
                                             $1.value.getStrings());
                    $3.node->insideNoAttrCB();
                    $$.node->setNode(0,$3.node);
                    $$.node->check(_curContexts.accessBack(),_curModel);
                    }
		| LB instructionList RB
                    {
                    $$.node=$2.node;
                    }
		| localDeclar SEMICOLON
                    {
                    $$.node=$1.node;
                    }
		| RETURN SEMICOLON
                    {
                    $$.node=new ReturnNode(_lineNum,_fileName,
                                           $1.value.getStrings());
                    $$.node->check(_curContexts.accessBack(),_curModel);
                    }
		| RETURN expression SEMICOLON
                    {
                    $$.node=new ReturnNode(_lineNum,_fileName,
                                           $1.value.getStrings());
                    $$.node->addNode($2.node);
                    $$.node->check(_curContexts.accessBack(),_curModel);
                    }
		| instrExpr SEMICOLON
                    {
                    $$.node=$1.node;
                    $$.node->setInstruction(true);
                    }
		| controlInstr
                    {
                    $$.node=$1.node;
                    }
		| error RB
                    {
                    yyerrok;
                    $$.node=new SeqNode(_lineNum,_fileName,";");
                    $$.node->check(_curContexts.accessBack(),_curModel);
                    }
		| error SEMICOLON
                    {
                    yyerrok;
                    $$.node=new SeqNode(_lineNum,_fileName,";");
                    $$.node->check(_curContexts.accessBack(),_curModel);
                    }
;

startParam	: LP expression RP
                    {
                    $$.node=$2.node;
                    }
		| // empty
                    {
                    $$.node=(SmaNode *)0;
                    }
;

//-------- Control Instructions --------

controlInstr	: WHILE LP expression RP instruction
                    {
                    $$.node=new WhileNode(_lineNum,_fileName,
                                          $1.value.getStrings());
                    $$.node->setNode(0,$3.node);
                    if($5.node) $$.node->addNode($5.node);
                    $$.node->check(_curContexts.accessBack(),_curModel);
                    }
		| DO instruction WHILE LP expression RP SEMICOLON
                    {
                    $$.node=new DoNode(_lineNum,_fileName,
                                       $1.value.getStrings(),
                                       $3.value.getStrings());
                    $$.node->setNode(0,$5.node);
                    if($2.node) $$.node->addNode($2.node);
                    $$.node->check(_curContexts.accessBack(),_curModel);
                    }
		| FOR LP forStart SEMICOLON expression SEMICOLON forEnd RP
                  instruction
                    {
                    $$.node=new ForNode(_lineNum,_fileName,
                                        $1.value.getStrings());
                    $$.node->setNode(0,$3.node);
                    $$.node->setNode(1,$5.node);
                    $$.node->setNode(2,$7.node);
                    if($9.node) $$.node->addNode($9.node);
                    $$.node->check(_curContexts.accessBack(),_curModel);
                    }
		| IF LP expression RP instruction
                    {
                    $$.node=new IfNode(_lineNum,_fileName,
                                       $1.value.getStrings(),
                                       "else");
                    $$.node->setNode(0,$3.node);
                    if($5.node) $$.node->addNode($5.node);
                    $$.node->check(_curContexts.accessBack(),_curModel);
                    }
		| IF LP expression RP instruction
                  ELSE instruction
                    {
                    $$.node=new IfNode(_lineNum,_fileName,
                                       $1.value.getStrings(),
                                       $6.value.getStrings());
                    $$.node->setNode(0,$3.node);
                    if($5.node) $$.node->addNode($5.node);
                    if($7.node)
                      {
                      if(!$5.node)
                        {
                        SmaNode * empty=new SeqNode(_lineNum,
                                                    _fileName,";");
                        empty->check(_curContexts.accessBack(),_curModel);
                        $$.node->addNode(empty);
                        }
                      $$.node->addNode($7.node);
                      }
                    $$.node->check(_curContexts.accessBack(),_curModel);
                    }
;

// !!! Shift/Reduce Conflict !!!
//  if(cond1)
//    if(cond2)
//      instr1;
//    else      // 'else' of 1st or 2nd 'if' ? --> 2nd, OK !
//      instr2;

forEnd		: forInstrList
                    {
                    $$.node=$1.node;
                    }
;

forStart	: forInstrList
                    {
                    $$.node=$1.node;
                    }
		| localDeclar
                    {
                    if($1.node) $$.node=$1.node;
                    else
                      {
                      $$.node=new SeqNode(_lineNum,_fileName,",");
                      $$.node->check(_curContexts.accessBack(),_curModel);
                      }
                    $$.node->setInstruction(false);
                    }
;

forInstrList	: forInstrList2
                    {
                    $$.node=$1.node;
                    $$.node->check(_curContexts.accessBack(),_curModel);
                    $$.node->setInstruction(false);
                    }
		| // empty
                    {
                    $$.node=new SeqNode(_lineNum,_fileName,",");
                    $$.node->check(_curContexts.accessBack(),_curModel);
                    }
;

forInstrList2	: forInstrList2 COMMA forInstr
                    {
                    $$.node=$1.node;
                    if($3.node) $$.node->addNode($3.node);
                    }
		| forInstr
                    {
                    $$.node=new SeqNode(_lineNum,_fileName,",");
                    if($1.node) $$.node->addNode($1.node);
                    }
;

forInstr	: instrExpr
                    {
                    $$.node=$1.node;
                    if($$.node) $$.node->setInstruction(false);
                    }
		| error SEMICOLON
                    {
                    yyerrok;
                    $$.node=new SeqNode(_lineNum,_fileName,",");
                    $$.node->check(_curContexts.accessBack(),_curModel);
                    }
;

//-------- Instructions or Expressions --------

instrExpr	: expression INCREASE
                    {
                    $$.node=new IncreaseNode(_lineNum,_fileName,
                                             $2.value.getStrings(),true);
                    $$.node->setNode(0,$1.node);
                    $$.node->check(_curContexts.accessBack(),_curModel);
                    }
		| INCREASE expression
                    {
                    $$.node=new IncreaseNode(_lineNum,_fileName,
                                             $1.value.getStrings(),false);
                    $$.node->setNode(0,$2.node);
                    $$.node->check(_curContexts.accessBack(),_curModel);
                    }
		| expression DECREASE
                    {
                    $$.node=new DecreaseNode(_lineNum,_fileName,
                                             $2.value.getStrings(),true);
                    $$.node->setNode(0,$1.node);
                    $$.node->check(_curContexts.accessBack(),_curModel);
                    }
		| DECREASE expression
                    {
                    $$.node=new DecreaseNode(_lineNum,_fileName,
                                             $1.value.getStrings(),false);
                    $$.node->setNode(0,$2.node);
                    $$.node->check(_curContexts.accessBack(),_curModel);
                    }
		| expression PLUSEQ expression
                    {
                    $$.node=new PlusEqNode(_lineNum,_fileName,
                                           $2.value.getStrings());
                    $$.node->setNode(0,$1.node);
                    $$.node->setNode(1,$3.node);
                    $$.node->check(_curContexts.accessBack(),_curModel);
                    }
		| expression MINUSEQ expression
                    {
                    $$.node=new MinusEqNode(_lineNum,_fileName,
                                            $2.value.getStrings());
                    $$.node->setNode(0,$1.node);
                    $$.node->setNode(1,$3.node);
                    $$.node->check(_curContexts.accessBack(),_curModel);
                    }
		| expression MULTEQ expression
                    {
                    $$.node=new MultEqNode(_lineNum,_fileName,
                                           $2.value.getStrings());
                    $$.node->setNode(0,$1.node);
                    $$.node->setNode(1,$3.node);
                    $$.node->check(_curContexts.accessBack(),_curModel);
                    }
		| expression DIVEQ expression
                    {
                    $$.node=new DivEqNode(_lineNum,_fileName,
                                          $2.value.getStrings());
                    $$.node->setNode(0,$1.node);
                    $$.node->setNode(1,$3.node);
                    $$.node->check(_curContexts.accessBack(),_curModel);
                    }
		| expression MODEQ expression
                    {
                    $$.node=new ModEqNode(_lineNum,_fileName,
                                          $2.value.getStrings());
                    $$.node->setNode(0,$1.node);
                    $$.node->setNode(1,$3.node);
                    $$.node->check(_curContexts.accessBack(),_curModel);
                    }
		| expression BITANDEQ expression
                    {
                    $$.node=new BitAndEqNode(_lineNum,_fileName,
                                             $2.value.getStrings());
                    $$.node->setNode(0,$1.node);
                    $$.node->setNode(1,$3.node);
                    $$.node->check(_curContexts.accessBack(),_curModel);
                    }
		| expression BITOREQ expression
                    {
                    $$.node=new BitOrEqNode(_lineNum,_fileName,
                                            $2.value.getStrings());
                    $$.node->setNode(0,$1.node);
                    $$.node->setNode(1,$3.node);
                    $$.node->check(_curContexts.accessBack(),_curModel);
                    }
		| expression BITXOREQ expression
                    {
                    $$.node=new BitXorEqNode(_lineNum,_fileName,
                                             $2.value.getStrings());
                    $$.node->setNode(0,$1.node);
                    $$.node->setNode(1,$3.node);
                    $$.node->check(_curContexts.accessBack(),_curModel);
                    }
		| expression LSHIFTEQ expression
                    {
                    $$.node=new LShiftEqNode(_lineNum,_fileName,
                                             $2.value.getStrings());
                    $$.node->setNode(0,$1.node);
                    $$.node->setNode(1,$3.node);
                    $$.node->check(_curContexts.accessBack(),_curModel);
                    }
		| expression RSHIFTEQ expression
                    {
                    $$.node=new RShiftEqNode(_lineNum,_fileName,
                                             $2.value.getStrings());
                    $$.node->setNode(0,$1.node);
                    $$.node->setNode(1,$3.node);
                    $$.node->check(_curContexts.accessBack(),_curModel);
                    }
		| expression AFFECT expression
                    {
                    $$.node=new AffectNode(_lineNum,_fileName,
                                           $2.value.getStrings(),false);
                    $$.node->setNode(0,$1.node);
                    $$.node->setNode(1,$3.node);
                    $$.node->check(_curContexts.accessBack(),_curModel);
                    }
		| expression DEEPAFFECT expression
                    {
                    $$.node=new AffectNode(_lineNum,_fileName,
                                           $2.value.getStrings(),true);
                    $$.node->setNode(0,$1.node);
                    $$.node->setNode(1,$3.node);
                    $$.node->check(_curContexts.accessBack(),_curModel);
                    }
		| moduleCall
                    {
                    $$.node=$1.node;
                    }
		| expression POINT SEEK LP // list
                             expression RP // elem
                    {
                    $$.node=new SeekNode(_lineNum,_fileName,
                                         $3.value.getStrings(),false);
                    $$.node->setNode(0,$1.node);
                    $$.node->setNode(1,$5.node);
                    $$.node->check(_curContexts.accessBack(),_curModel);
                    }
		| expression POINT DEEPSEEK LP // agent list
                             expression RP     // agent
                    {
                    $$.node=new SeekNode(_lineNum,_fileName,
                                         $3.value.getStrings(),true);
                    $$.node->setNode(0,$1.node);
                    $$.node->setNode(1,$5.node);
                    $$.node->check(_curContexts.accessBack(),_curModel);
                    }
		| expression POINT SEEKMIN LP RP // list
                    {
                    $$.node=new MinPosNode(_lineNum,_fileName,
                                           $3.value.getStrings());
                    $$.node->setNode(0,$1.node);
                    $$.node->check(_curContexts.accessBack(),_curModel);
                    }
		| expression POINT SEEKMAX LP RP // list
                    {
                    $$.node=new MaxPosNode(_lineNum,_fileName,
                                           $3.value.getStrings());
                    $$.node->setNode(0,$1.node);
                    $$.node->check(_curContexts.accessBack(),_curModel);
                    }
		| expression POINT LENGTH LP RP // list
                    {
                    $$.node=new LengthNode(_lineNum,_fileName,
                                           $3.value.getStrings());
                    $$.node->setNode(0,$1.node);
                    $$.node->check(_curContexts.accessBack(),_curModel);
                    }
		| DUMP LP expression RP
                    {
                    $$.node=new DumpNode(_lineNum,_fileName,
                                        $1.value.getStrings());
                    $$.node->setNode(0,$3.node);
                    $$.node->check(_curContexts.accessBack(),_curModel);
                    }
		| VALUE_OF LP expression COMMA // result
                              expression RP    // string
                    {
                    $$.node=new ValueOfNode(_lineNum,_fileName,
                                            $1.value.getStrings());
                    $$.node->setNode(0,$3.node);
                    $$.node->setNode(1,$5.node);
                    $$.node->check(_curContexts.accessBack(),_curModel);
                    }
		| instanciation
                    {
                    $$.node=$1.node;
                    }
		| DELETE expression // agent 
                    {
                    $$.node=new DeleteNode(_lineNum,_fileName,
                                           $1.value.getStrings());
                    $$.node->setNode(0,$2.node);
                    $$.node->check(_curContexts.accessBack(),_curModel);
                    }
		| expression POINT INSERT LP  // list
                             expression COMMA // position
                             expression RP    // data
                    {
                    $$.node=new InsertNode(_lineNum,_fileName,
                                           $3.value.getStrings());
                    $$.node->setNode(0,$1.node);
                    $$.node->setNode(1,$5.node);
                    $$.node->setNode(2,$7.node);
                    $$.node->check(_curContexts.accessBack(),_curModel);
                    }
		| expression POINT PUSHFRONT LP // list
                             expression RP      // data
                    {
                    $$.node=new PushFrontNode(_lineNum,_fileName,
                                              $3.value.getStrings());
                    $$.node->setNode(0,$1.node);
                    $$.node->setNode(1,$5.node);
                    $$.node->check(_curContexts.accessBack(),_curModel);
                    }
		| expression POINT PUSHBACK LP // list
                             expression RP     // data
                    {
                    $$.node=new PushBackNode(_lineNum,_fileName,
                                              $3.value.getStrings());
                    $$.node->setNode(0,$1.node);
                    $$.node->setNode(1,$5.node);
                    $$.node->check(_curContexts.accessBack(),_curModel);
                    }
		| expression POINT REMOVE LP // list
                             expression RP   // position
                    {
                    $$.node=new RemoveNode(_lineNum,_fileName,
                                           $3.value.getStrings());
                    $$.node->setNode(0,$1.node);
                    $$.node->setNode(1,$5.node);
                    $$.node->check(_curContexts.accessBack(),_curModel);
                    }
		| expression POINT POPFRONT LP RP // list
                    {
                    $$.node=new PopFrontNode(_lineNum,_fileName,
                                             $3.value.getStrings());
                    $$.node->setNode(0,$1.node);
                    $$.node->check(_curContexts.accessBack(),_curModel);
                    }
		| expression POINT POPBACK LP RP // list
                    {
                    $$.node=new PopBackNode(_lineNum,_fileName,
                                            $3.value.getStrings());
                    $$.node->setNode(0,$1.node);
                    $$.node->check(_curContexts.accessBack(),_curModel);
                    }
		| expression POINT ERASE LP RP // list
                    {
                    $$.node=new EraseNode(_lineNum,_fileName,
                                          $3.value.getStrings());
                    $$.node->setNode(0,$1.node);
                    $$.node->check(_curContexts.accessBack(),_curModel);
                    }
		| expression POINT SORT LP RP // list
                    {
                    $$.node=new SortNode(_lineNum,_fileName,
                                         $3.value.getStrings());
                    $$.node->setNode(0,$1.node);
                    $$.node->check(_curContexts.accessBack(),_curModel);
                    }
		| FORMAT LP
                    {
                    _curNodes.pushBack(new FormatNode(_lineNum,_fileName,
                                                      $1.value.getStrings()));
                    }
                  callParams RP
                    {
                    $$.node=_curNodes.readPopBack(); 
                    $$.node->check(_curContexts.accessBack(),_curModel);
                    }
		| PRINT LP
                    {
                    _curNodes.pushBack(new PrintNode(_lineNum,_fileName,
                                                     $1.value.getStrings(),
                                                     false));
                    }
                  callParams RP
                    {
                    $$.node=_curNodes.readPopBack(); 
                    $$.node->check(_curContexts.accessBack(),_curModel);
                    }
		| PRINTLN LP
                    {
                    _curNodes.pushBack(new PrintNode(_lineNum,_fileName,
                                                     $1.value.getStrings(),
                                                     true));
                    }
                  callParams RP
                    {
                    $$.node=_curNodes.readPopBack(); 
                    $$.node->check(_curContexts.accessBack(),_curModel);
                    }
		| ERROR LP
                    {
                    _curNodes.pushBack(new ErrorNode(_lineNum,_fileName,
                                                     $1.value.getStrings(),
                                                     false));
                    }
                  callParams RP
                    {
                    $$.node=_curNodes.readPopBack(); 
                    $$.node->check(_curContexts.accessBack(),_curModel);
                    }
		| WARNING LP
                    {
                    _curNodes.pushBack(new ErrorNode(_lineNum,_fileName,
                                                     $1.value.getStrings(),
                                                     true));
                    }
                  callParams RP
                    {
                    $$.node=_curNodes.readPopBack(); 
                    $$.node->check(_curContexts.accessBack(),_curModel);
                    }
;

//-------- Module Call --------

instanciation	: newHead
                    {
                    $$.node=_curNodes.readPopBack();
                    $$.node->check(_curContexts.accessBack(),_curModel);
                    }
		| newHead LP callParams RP
                    {
                    $$.node=_curNodes.readPopBack();
                    $$.node->check(_curContexts.accessBack(),_curModel);
                    }
;

newHead         : NEW MODELTYPE
                    {
                    _curNodes.pushBack(new NewNode(_lineNum,_fileName,
                                                   $1.value.getStrings(),
                                                   $2.model,
                                                   true,    // transient
                                                   false)); // no suffix
                    }
                | BIG_NEW MODELTYPE
                    {
                    _curNodes.pushBack(new NewNode(_lineNum,_fileName,
                                                   $1.value.getStrings(),
                                                   $2.model,
                                                   false,   // not transient
                                                   false)); // no suffix
                    }
		| NEW LP expression RP MODELTYPE
                    {
                    _curNodes.pushBack(new NewNode(_lineNum,_fileName,
                                                   $1.value.getStrings(),
                                                   $5.model,
                                                   true,   // transient
                                                   true)); // suffix given
                    _curNodes.accessBack()->addNode($3.node);
                    }
		| BIG_NEW LP expression RP MODELTYPE
                    {
                    _curNodes.pushBack(new NewNode(_lineNum,_fileName,
                                                   $1.value.getStrings(),
                                                   $5.model,
                                                   false,  // not transient
                                                   true)); // suffix given
                    _curNodes.accessBack()->addNode($3.node);
                    }
;

moduleCall	: callHead callParams RP
                    {
                    $$.node=_curNodes.readPopBack();
                    $$.node->check(_curContexts.accessBack(),_curModel);
                    }
;

callHead	: largeIdent LP
                    {
                    _curNodes.pushBack(new ModuleNode(_lineNum,
                                                      _fileName,
                                                      $1.value.getStrings(),
                                                      ""));
                    }
		| MODELTYPE RANGE moduleName LP
                    {
                    _curNodes.pushBack(new ModuleNode(_lineNum,
                                                      _fileName,
                                                      $3.value.getStrings(),
                                                      $1.model->getName()
                                                               .getStrings()));
                    }
		| RANGE largeIdent LP
                    {
                    _curNodes.pushBack(new ModuleNode(_lineNum,
                                                      _fileName,
                                                      $2.value.getStrings(),
                                                      $1.value.getStrings()));
                    }
		| expression ARROW largeIdent LP
                    {
                    SmaNode * node=new FarModuleNode(_lineNum,
                                                     _fileName,
                                                     $3.value.getStrings(),
                                                     "");
                    node->setNode(0,$1.node);
                    _curNodes.pushBack(node);
                    }
		| expression ARROW MODELTYPE RANGE largeIdent LP
                    {
                    SmaNode * node=new FarModuleNode(_lineNum,
                                                     _fileName,
                                                     $5.value.getStrings(),
                                                     $3.model->getName()
                                                              .getStrings());
                    node->setNode(0,$1.node);
                    _curNodes.pushBack(node);
                    }
;

callParams	: expressionList
		| // empty
		| error RP
                    {
                    yyerrok;
                    }
;

expressionList	: expression
                    {
                    _curNodes.accessBack()->addNode($1.node);
                    }
		| expressionList COMMA expression
                    {
                    _curNodes.accessBack()->addNode($3.node);
                    }
;

//-------- Expressions -----------------------------------------------------//

expression	: term
                    {
                    $$.node=$1.node;
                    }
		| expression MULT expression
                    {
                    $$.node=new MultNode(_lineNum,_fileName,
                                         $2.value.getStrings());
                    $$.node->setNode(0,$1.node);
                    $$.node->setNode(1,$3.node);
                    $$.node->check(_curContexts.accessBack(),_curModel);
                    }
		| expression DIV expression
                    {
                    $$.node=new DivNode(_lineNum,_fileName,
                                        $2.value.getStrings());
                    $$.node->setNode(0,$1.node);
                    $$.node->setNode(1,$3.node);
                    $$.node->check(_curContexts.accessBack(),_curModel);
                    }
		| expression MODULO expression
                    {
                    $$.node=new ModNode(_lineNum,_fileName,
                                        $2.value.getStrings());
                    $$.node->setNode(0,$1.node);
                    $$.node->setNode(1,$3.node);
                    $$.node->check(_curContexts.accessBack(),_curModel);
                    }
		| expression PLUS expression
                    {
                    $$.node=new PlusNode(_lineNum,_fileName,
                                         $2.value.getStrings());
                    $$.node->setNode(0,$1.node);
                    $$.node->setNode(1,$3.node);
                    $$.node->check(_curContexts.accessBack(),_curModel);
                    }
		| expression MINUS expression
                    {
                    $$.node=new MinusNode(_lineNum,_fileName,
                                          $2.value.getStrings());
                    $$.node->setNode(0,$1.node);
                    $$.node->setNode(1,$3.node);
                    $$.node->check(_curContexts.accessBack(),_curModel);
                    }
		| NOT expression
                    {
                    $$.node=new NotNode(_lineNum,_fileName,
                                        $1.value.getStrings());
                    $$.node->setNode(0,$2.node);
                    $$.node->check(_curContexts.accessBack(),_curModel);
                    }
		| expression AND expression
                    {
                    $$.node=new AndNode(_lineNum,_fileName,
                                        $2.value.getStrings());
                    $$.node->setNode(0,$1.node);
                    $$.node->setNode(1,$3.node);
                    $$.node->check(_curContexts.accessBack(),_curModel);
                    }
		| expression OR expression
                    {
                    $$.node=new OrNode(_lineNum,_fileName,
                                       $2.value.getStrings());
                    $$.node->setNode(0,$1.node);
                    $$.node->setNode(1,$3.node);
                    $$.node->check(_curContexts.accessBack(),_curModel);
                    }
		| BITNOT expression
                    {
                    $$.node=new BitNotNode(_lineNum,_fileName,
                                           $1.value.getStrings());
                    $$.node->setNode(0,$2.node);
                    $$.node->check(_curContexts.accessBack(),_curModel);
                    }
		| expression BITAND expression
                    {
                    $$.node=new BitAndNode(_lineNum,_fileName,
                                           $2.value.getStrings());
                    $$.node->setNode(0,$1.node);
                    $$.node->setNode(1,$3.node);
                    $$.node->check(_curContexts.accessBack(),_curModel);
                    }
		| expression BITOR expression
                    {
                    $$.node=new BitOrNode(_lineNum,_fileName,
                                          $2.value.getStrings());
                    $$.node->setNode(0,$1.node);
                    $$.node->setNode(1,$3.node);
                    $$.node->check(_curContexts.accessBack(),_curModel);
                    }
		| expression BITXOR expression
                    {
                    $$.node=new BitXorNode(_lineNum,_fileName,
                                           $2.value.getStrings());
                    $$.node->setNode(0,$1.node);
                    $$.node->setNode(1,$3.node);
                    $$.node->check(_curContexts.accessBack(),_curModel);
                    }
		| expression LSHIFT expression
                    {
                    $$.node=new LShiftNode(_lineNum,_fileName,
                                           $2.value.getStrings());
                    $$.node->setNode(0,$1.node);
                    $$.node->setNode(1,$3.node);
                    $$.node->check(_curContexts.accessBack(),_curModel);
                    }
		| expression RSHIFT expression
                    {
                    $$.node=new RShiftNode(_lineNum,_fileName,
                                           $2.value.getStrings());
                    $$.node->setNode(0,$1.node);
                    $$.node->setNode(1,$3.node);
                    $$.node->check(_curContexts.accessBack(),_curModel);
                    }
		| expression EQ expression
                    {
                    $$.node=new EqNode(_lineNum,_fileName,
                                       $2.value.getStrings(),false);
                    $$.node->setNode(0,$1.node);
                    $$.node->setNode(1,$3.node);
                    $$.node->check(_curContexts.accessBack(),_curModel);
                    }
		| expression DEEPEQ expression
                    {
                    $$.node=new EqNode(_lineNum,_fileName,
                                       $2.value.getStrings(),true);
                    $$.node->setNode(0,$1.node);
                    $$.node->setNode(1,$3.node);
                    $$.node->check(_curContexts.accessBack(),_curModel);
                    }
		| expression DIF expression
                    {
                    $$.node=new DifNode(_lineNum,_fileName,
                                        $2.value.getStrings(),false);
                    $$.node->setNode(0,$1.node);
                    $$.node->setNode(1,$3.node);
                    $$.node->check(_curContexts.accessBack(),_curModel);
                    }
		| expression DEEPDIF expression
                    {
                    $$.node=new DifNode(_lineNum,_fileName,
                                        $2.value.getStrings(),true);
                    $$.node->setNode(0,$1.node);
                    $$.node->setNode(1,$3.node);
                    $$.node->check(_curContexts.accessBack(),_curModel);
                    }
		| expression INF expression
                    {
                    $$.node=new InfNode(_lineNum,_fileName,
                                        $2.value.getStrings());
                    $$.node->setNode(0,$1.node);
                    $$.node->setNode(1,$3.node);
                    $$.node->check(_curContexts.accessBack(),_curModel);
                    }
		| expression SUP expression
                    {
                    $$.node=new SupNode(_lineNum,_fileName,
                                        $2.value.getStrings());
                    $$.node->setNode(0,$1.node);
                    $$.node->setNode(1,$3.node);
                    $$.node->check(_curContexts.accessBack(),_curModel);
                    }
		| expression INFEQ expression
                    {
                    $$.node=new InfEqNode(_lineNum,_fileName,
                                          $2.value.getStrings());
                    $$.node->setNode(0,$1.node);
                    $$.node->setNode(1,$3.node);
                    $$.node->check(_curContexts.accessBack(),_curModel);
                    }
		| expression SUPEQ expression
                    {
                    $$.node=new SupEqNode(_lineNum,_fileName,
                                          $2.value.getStrings());
                    $$.node->setNode(0,$1.node);
                    $$.node->setNode(1,$3.node);
                    $$.node->check(_curContexts.accessBack(),_curModel);
                    }
;

term		: LP expression RP
                    {
                    $$.node=$2.node;
                    $$.node->useParenthesis(true);
                    }
		| expression RAND expression COLON expression
                    {
                    $$.node=new CondNode(_lineNum,_fileName,
                                         $2.value.getStrings(),
                                         $4.value.getStrings());
                    $$.node->setNode(0,$1.node);
                    $$.node->setNode(1,$3.node);
                    $$.node->setNode(2,$5.node);
                    $$.node->check(_curContexts.accessBack(),_curModel);
                    }
		| expression MIN expression
                    {
                    $$.node=new MinNode(_lineNum,_fileName,
                                        $2.value.getStrings());
                    $$.node->setNode(0,$1.node);
                    $$.node->setNode(1,$3.node);
                    $$.node->check(_curContexts.accessBack(),_curModel);
                    }
		| expression MAX expression
                    {
                    $$.node=new MaxNode(_lineNum,_fileName,
                                        $2.value.getStrings());
                    $$.node->setNode(0,$1.node);
                    $$.node->setNode(1,$3.node);
                    $$.node->check(_curContexts.accessBack(),_curModel);
                    }
		| PLUS expression %prec UPLUS
                    {
                    $$.node=$2.node;
                    }
		| MINUS expression %prec UMINUS
                    {
                    $$.node=new UMinNode(_lineNum,_fileName,
                                         $1.value.getStrings());
                    $$.node->setNode(0,$2.node);
                    $$.node->check(_curContexts.accessBack(),_curModel);
                    }
		| cast expression %prec CAST
                    {
                    $$.node=_curNodes.readPopBack();
                    $$.node->setNode(0,$2.node);
                    $$.node->check(_curContexts.accessBack(),_curModel);
                    }
		| largeIdent
                    {
                    $$.node=new VarNode(_lineNum,_fileName,
                                        $1.value.getStrings());
                    $$.node->check(_curContexts.accessBack(),_curModel);
                    }
		| expression POINT FRONT LP RP // list
                    {
                    $$.node=new FrontNode(_lineNum,_fileName,
                                          $3.value.getStrings());
                    $$.node->setNode(0,$1.node);
                    $$.node->check(_curContexts.accessBack(),_curModel);
                    }
		| expression POINT BACK LP RP // list
                    {
                    $$.node=new BackNode(_lineNum,_fileName,
                                         $3.value.getStrings());
                    $$.node->setNode(0,$1.node);
                    $$.node->check(_curContexts.accessBack(),_curModel);
                    }
		| expression LH expression RH // list , int
                    {
                    $$.node=new IndexNode(_lineNum,_fileName,
                                          $2.value.getStrings(),
                                          $4.value.getStrings());
                    $$.node->setNode(0,$1.node);
                    $$.node->setNode(1,$3.node);
                    $$.node->check(_curContexts.accessBack(),_curModel);
                    }
		| expression ARROW largeIdent
                    {
                    $$.node=new FarVarNode(_lineNum,_fileName,
                                           $3.value.getStrings());
                    $$.node->setNode(0,$1.node);
                    $$.node->check(_curContexts.accessBack(),_curModel);
                    }
		| RAND
                    {
                    $$.node=new RealRndNode(_lineNum,_fileName,
                                            $1.value.getStrings());
                    $$.node->check(_curContexts.accessBack(),_curModel);
                    }
		| RAND LP expression RP // integer
                    {
                    $$.node=new IntRndNode(_lineNum,_fileName,
                                           $1.value.getStrings());
                    $$.node->setNode(0,$3.node);
                    $$.node->check(_curContexts.accessBack(),_curModel);
                    }
		| THIS
                    {
                    $$.node=new SelfNode(_lineNum,_fileName,
                                         $1.value.getStrings());
                    $$.node->check(_curContexts.accessBack(),_curModel);
                    }
		| THAT
                    {
                    $$.node=new CallerNode(_lineNum,_fileName,
                                           $1.value.getStrings());
                    $$.node->check(_curContexts.accessBack(),_curModel);
                    }
		| constantValue
                    {
                    $$.node=new ConstNode(_lineNum,_fileName,
                                          $1.value);
                    $$.node->check(_curContexts.accessBack(),_curModel);
                    }
		| instrExpr
                    {
                    $$.node=$1.node;
                    $$.node->setInstruction(false);
                    }
;

constantValue	: INTEGER_VAL
                    {
                    $$.value.setType(ValueType(ValueType::INTEGER));
                    $$.value.setInteger($1.value.getInteger());
                    }
		| REAL_VAL
                    {
                    $$.value.setType(ValueType(ValueType::REAL));
                    $$.value.setReal($1.value.getReal());
                    }
		| STRING_VAL
                    {
                    $$.value.setType(ValueType(ValueType::STRING));
                    $$.value.setStrings($1.value.getStrings());
                    }
		| AGENT_VAL
                    {
                    $$.value.setType($1.value.getType());
                    $$.value.setAgent($1.value.getAgent());
                    }
		| NONE
                    {
                    $$.value.setType(Model::getBaseModel());
                    $$.value.setAgent((Agent *)0);
                    }
		| cast LH RH %prec CAST
                    {
                    ValueType type=_curNodes.getBack()->getType();
                    delete _curNodes.readPopBack();
                    if(type.getDepth())
                      {
                      Value value(type);
                      $$.value=value;
                      }
                    else
                      {
                      Strings message="Type '"+
                                      type.print()+
                                      "' can't be given to '"+
                                      $2.value.getStrings()+
                                      $3.value.getStrings()+
                                      "'";
                      manageError(new Error(Error::SEMANTICS,message));
                      type.setDepth(1);
                      Value value(type);
                      $$.value=value;
                      }
                    }
		| LH valueList RH
                    {
                    if($2.value.evalVectType()) $$.value=$2.value;
                    else
                      {
                      Strings message="Type mismatch in list initialization";
                      manageError(new Error(Error::SEMANTICS,message));
                      Value value(ValueType(ValueType::INTEGER,1));
                      $$.value=value;
                      }
                    }
; 

cast		: LP complexType RP %prec CAST
                    {
                    _curNodes.pushBack(new CastNode(_lineNum,_fileName,
                                                    $1.value.getStrings(),
                                                    $3.value.getStrings(),
                                                    _curAttrType));
                    }
;

valueList	: listElem
                    {
                    ValueType type=$1.value.getType();
                    type.setDepth(type.getDepth()+1);
                    $$.value.setType(type);
                    $$.value.pushBack($1.value);
                    }
		| valueList COMMA listElem
                    {
                    $$.value=$1.value;
                    $$.value.pushBack($3.value);
                    }
;

listElem	: constantValue
                    {
                    $$.value=$1.value;
                    }
                | cast NONE %prec CAST
                    {
                    ValueType type=_curNodes.getBack()->getType();
                    delete _curNodes.readPopBack();
                    if(type.canCastTo(ValueType::AGENT))
                      {
                      Value value(type);
                      $$.value=value;
                      }
                    else
                      {
                      Strings message="Type '"+
                                      type.print()+
                                      "' can't be given to '"+
                                      $2.value.getStrings()+
                                      "'";
                      manageError(new Error(Error::SEMANTICS,message));
                      Value value(Model::getBaseModel());
                      $$.value=value;
                      }
                    }
		| PLUS INTEGER_VAL %prec UPLUS
                    {
                    $$.value.setType(ValueType(ValueType::INTEGER));
                    $$.value.setInteger($2.value.getInteger());
                    }
		| MINUS INTEGER_VAL %prec UMINUS
                    {
                    $$.value.setType(ValueType(ValueType::INTEGER));
                    $$.value.setInteger(-$2.value.getInteger());
                    }
		| PLUS REAL_VAL %prec UPLUS
                    {
                    $$.value.setType(ValueType(ValueType::REAL));
                    $$.value.setReal($2.value.getReal());
                    }
		| MINUS REAL_VAL %prec UMINUS
                    {
                    $$.value.setType(ValueType(ValueType::REAL));
                    $$.value.setReal(-$2.value.getReal());
                    }
;

%%

//----------------------------------------------------------------------//

void SmaParser::init(Strings fileName,istream * stream,SmaParser * calling)
{
_calling=calling;
_lineNum=1;
_fileName=fileName;
_stream=stream;
_curModel=(Model *)0;
_curAgent=(Agent *)0;
_curModuleType=ExecModule::NORMAL;
_scanner=new SmaLexer(_stream);
}

SmaParser::~SmaParser(void)
{
if(_scanner) delete _scanner;
while(_curContexts.getSize())
  {
  Context::relaxContext(_curContexts.readPopBack());
  }
while(_curNodes.getSize())
  {
  delete _curNodes.readPopBack();
  }
}

bool SmaParser::fileInUse(const Strings & fileName)
{
return((fileName==_fileName)||
       (_calling&&_calling->fileInUse(fileName)));
}

void SmaParser::setModule(void)
{
if(_curModules.getSize())
  {
  if(_curModules.accessBack()->mustDisappear())
    {
    delete _curModules.accessBack();
    }
  else if((_curModules.getBack()->getSpecialType()==ExecModule::EXECUTE)||
          (_curModules.getBack()->getSpecialType()==ExecModule::START))
    {
    getSimul()->startModule(_curModules.accessBack(),
                            _curModules.accessBack()->accessOwner());
    }
  else if(_curAgent)
    {
    Error * error=_curAgent->tryToSetModule(_curModules.getBack());
    if(error)
      {
      manageError(error);
      delete _curModules.accessBack();
      }
    else _curAgent->setModule(_curModules.accessBack(),true);
    }
  else if(_curModel)
    {
    Error * error=_curModel->tryToSetModule(_curModules.getBack());
    if(error)
      {
      manageError(error);
      delete _curModules.accessBack();
      }
    else _curModel->setModule(_curModules.accessBack(),true);
    }
  else
    {
    Error * error=getSimul()->tryToSetModule(_curModules.getBack());
    if(error)
      {
      manageError(error);
      delete _curModules.accessBack();
      }
    else getSimul()->setModule(_curModules.accessBack(),true);
    }
  _curModules.popBack();
  }
if(_curContexts.getSize())
  {
  Context::relaxContext(_curContexts.readPopBack());
  }
}

void SmaParser::setModuleNode(ExecModule * module,SmaNode * node)
{
const Model * model=module->getModel();
if(!model&&module->getOwner()) model=module->getOwner()->getModel();
if((module->getSpecialType()==ExecModule::CONSTRUCTOR)&&model)
  {
  SmaNode * seq=new SeqNode(_lineNum,_fileName,";");
  const Model * parent;
  bool called;
  for(unsigned int i=0;i<model->getParents().getSize();i++)
    {
    parent=model->getParents()[i];
    if(node->buildsParent(parent)) called=true;
    else
      {
      if(parent->getNewModule())
        {
        if(parent->getNewModule()->getNbParams()) called=false;
        else
          {
          SmaNode * call=new ModuleNode(_lineNum,
                                        _fileName,
                                        module->getName().getStrings(),
                                        parent->getName().getStrings());
          call->check(_curContexts.accessBack(),model);
          seq->addNode(call);
          called=true;
          }
        }
      else called=false;
      }
    if(!called)
      {
      Strings message="Module '"+
              module->getCompleteName()+
              "' must call the constructor of '"+
              parent->getName().getStrings()+
              "'";
      manageError(new Error(Error::SEMANTICS,message));
      }
    }
  if(seq->getNbNodes())
    {
    seq->addNode(node);
    seq->check(_curContexts.accessBack(),model);
    node=seq;
    }
  else delete seq; // nothing to add
  }
else if((module->getSpecialType()==ExecModule::DESTRUCTOR)&&model)
  {
  if(module->getOwner()&&node->destroysParent(model))
    {
    const Model * parent;
    for(unsigned int i=model->getParents().getSize();i--;)
      {
      parent=model->getParents()[i];
      if(node->destroysParent(parent))
        {
        Strings message="Destructor of '"+
                        parent->getName().getStrings()+
                        "' is called through '"+
                        model->getDeleteModule()->getCompleteName()+
                        "'";
        manageError(new Error(Error::SEMANTICS,message));
        }
      }
    }
  else
    {
    SmaNode * seq=new SeqNode(_lineNum,_fileName,";");
    seq->addNode((SmaNode *)0);
    const Model * parent;
    bool called;
    for(unsigned int i=model->getParents().getSize();i--;)
      {
      parent=model->getParents()[i];
      if(node->destroysParent(parent)) called=true;
      else
        {
        if(parent->getDeleteModule())
          {
          SmaNode * call=new ModuleNode(_lineNum,
                                        _fileName,
                                        module->getName().getStrings(),
                                        parent->getName().getStrings());
          call->check(_curContexts.accessBack(),model);
          seq->addNode(call);
          called=true;
          }
        else called=false;
        }
      if(!called)
        {
        Strings message="Module '"+
                        module->getCompleteName()+
                        "' must call the destructor of '"+
                        parent->getName().getStrings()+
                        "'";
        manageError(new Error(Error::SEMANTICS,message));
        }
      }
    if(seq->getNbNodes()>1)
      {
      seq->setNode(0,node);
      seq->check(_curContexts.accessBack(),model);
      node=seq;
      }
    else delete seq; // nothing to add
    }
  }
if((module->getResultType()!=ValueType())&&
   !node->containsReturn())
  {
  Strings message="Module '"+
                  module->getCompleteName()+
                  "' must return a '"+
                  module->getResultType().print()+
                  "'";
  manageError(new Error(Error::SEMANTICS,message));
  }
module->setNode(node);
}

void SmaParser::yyerror(char * text)
{
Strings message;
if(!::strcmp(text,"parse error")) message="Syntax error";
else message=text;
if(_line.len())
  message+="\n"+_line+"<---";
manageError(new Error(Error::SYNTAX,message));
}

void SmaParser::manageError(Error * error)
{
if(error)
  {
  if(_curModules.getSize()&&
     (error->getType()!=Error::WARNING))
    _curModules.accessBack()->mustDisappear()=true;
  error->setFileName(_fileName);
  error->setLineNumber(_lineNum);
  getSimul()->manageError(error);
  }
}

void SmaParser::nextLine(void)
{
_lineNum++;
_line=Strings("");
}

void SmaParser::store(const char * text)
{
_line+=Strings(text);
}

