// @(#) ENIB/LI2 (c) - oRis - Version 2.3
//--------------------------------------------------------------------------//
// file        : thrdNtv.cpp                                                //
// description : source file for the ENIB/LI2 class ThreadedNative          //
// author(s)   : F. Harrouet                                                //
// copyright   : Laboratoire d'Informatique Industrielle (LI2)              //
//               Ecole Nationale d'Ingenieurs de Brest (ENIB)               //
//               CP 30815 - 29608 Brest Cedex - FRANCE                      //
//               Tel: (33) 02.98.05.66.31 , Fax: (33) 02.98.05.66.29        //
//               e-mail: li2@enib.fr                                        //
//--------------------------------------------------------------------------//
// Copyright (c) 1996-2000                                                  //
// Laboratoire d'Informatique Industrielle (LI2)                            //
// Ecole Nationale d'Ingenieurs de Brest (ENIB)                             //
//                                                                          //
// Permission to use, copy, modify, distribute and sell this software       //
// and its documentation for any purpose is hereby granted without fee,     //
// provided that the above copyright notice appear in all copies and        //
// that both that copyright notice and this permission notice appear        //
// in supporting documentation.                                             //
// ENIB/LI2 makes no representations about the suitability of this          //
// software for any purpose.                                                //
// It is provided "as is" without express or implied warranty.              //
//--------------------------------------------------------------------------//

#include "oRis/Main/thrdNtv.h"

//--------------------------------------------------------------------------//
// class ThreadedNative                                                     //
//--------------------------------------------------------------------------//

SimpleVector<ThreadedNative *> ThreadedNative::_threadedNativeStock;

//-------- Static methods --------------------------------------------------//
ThreadedNative * ThreadedNative::getThreadedNative(void) // static
{
if(ThreadedNative::_threadedNativeStock.getSize())
  return(ThreadedNative::_threadedNativeStock.readPopBack());
else
  return(new ThreadedNative);
}

void ThreadedNative::relaxThreadedNative(ThreadedNative * thNative) // static
{
ThreadedNative::_threadedNativeStock.pushBack(thNative);
}

void ThreadedNative::clearThreadedNativeStock(void) // static
{
while(ThreadedNative::_threadedNativeStock.getSize())
  delete ThreadedNative::_threadedNativeStock.readPopBack();
}

//-------- Allocators ------------------------------------------------------//
ThreadedNative::ThreadedNative(void)
  : _thread(new SimpleThread(ThreadedNative::_taskLoop,this)),
    _mutex(new SimpleThreadMutex), _taskEnded(false),
    _task((void (*)(Context *))0), _context((Context *)0)
{
_thread->start();
}

ThreadedNative::~ThreadedNative(void)
{
_task=(void (*)(Context *))0;
delete _thread;
delete _mutex;
}

//-------- Inspectors ------------------------------------------------------//
// inline bool ThreadedNative::taskEnded(void) const

//-------- Modifiers -------------------------------------------------------//
void ThreadedNative::startTask(void (* task)(Context *),Context * context)
{
while(!_taskEnded) SimpleThread::yield();
_task=task;
_context=context;
_taskEnded=false;
_mutex->unlock();
}

//-------- Private methods -------------------------------------------------//
void * ThreadedNative::_taskLoop(void * data)
{
ThreadedNative * thNative=(ThreadedNative *)data;
bool again=true;
while(again)
  {
  thNative->_taskEnded=true;
  thNative->_mutex->lock();
  if(thNative->_task)
    {
    (*(thNative->_task))(thNative->_context);
    }
  }
return((void *)0);
}

