// @(#) ENIB/LI2 (c) - oRis - Version 2.3
//--------------------------------------------------------------------------//
// file        : value.cpp                                                  //
// description : source file for the ENIB/LI2 class Value                   //
// author(s)   : F. Harrouet & J. Tisseau                                   //
// copyright   : Laboratoire d'Informatique Industrielle (LI2)              //
//               Ecole Nationale d'Ingenieurs de Brest (ENIB)               //
//               CP 30815 - 29608 Brest Cedex - FRANCE                      //
//               Tel: (33) 02.98.05.66.31 , Fax: (33) 02.98.05.66.29        //
//               e-mail: li2@enib.fr                                        //
//--------------------------------------------------------------------------//
// Copyright (c) 1996-2000                                                  //
// Laboratoire d'Informatique Industrielle (LI2)                            //
// Ecole Nationale d'Ingenieurs de Brest (ENIB)                             //
//                                                                          //
// Permission to use, copy, modify, distribute and sell this software       //
// and its documentation for any purpose is hereby granted without fee,     //
// provided that the above copyright notice appear in all copies and        //
// that both that copyright notice and this permission notice appear        //
// in supporting documentation.                                             //
// ENIB/LI2 makes no representations about the suitability of this          //
// software for any purpose.                                                //
// It is provided "as is" without express or implied warranty.              //
//--------------------------------------------------------------------------//

#include "oRis/Main/value.h"
#include "oRis/Main/simul.h"
#include "oRis/Main/tmpVal.h"
#include "oRis/Main/errors.h"

//--------------------------------------------------------------------------//
// class ValueType                                                          //
//--------------------------------------------------------------------------//

//-------- Allocators ------------------------------------------------------//
// inline ValueType::ValueType(ValueType::SimpleType type,int depth)

// inline ValueType::ValueType(Model * model,int depth)

// inline ValueType::ValueType(const ValueType & type)

// inline ValueType::~ValueType(void)

//-------- Inspectors ------------------------------------------------------//
// inline ValueType::SimpleType ValueType::getSimpleType(void) const

// inline Model * ValueType::getModel(void) const

// inline int ValueType::getDepth(void) const

bool ValueType::canCastFrom(const ValueType & from) const
{
if(from._depth!=_depth) return(false);
if(_depth)
  {
  ValueType fromBaseType(from);
  fromBaseType._depth=0;
  ValueType toBaseType(*this);
  toBaseType._depth=0;
  return(toBaseType.canCastFrom(fromBaseType));
  }
switch(_type)
  {
  case ValueType::INTEGER:
  case ValueType::REAL:
    return((from._type==ValueType::INTEGER)||
           (from._type==ValueType::REAL));
  case ValueType::STRING:
    return(from._type==ValueType::STRING);
  case ValueType::AGENT:
    return((from._type==ValueType::AGENT)&&
           (!_model||
            (from._model&&from._model->isA(_model))));
  default:
    return(false);
  }
}

// inline bool ValueType::canCastTo(const ValueType & to) const

//-------- Comparisons -----------------------------------------------------//
// inline bool ValueType::operator==(const ValueType & type) const

// inline bool ValueType::operator!=(const ValueType & type) const

//-------- Comparisons -----------------------------------------------------//
ostream & operator<<(ostream & stream,const ValueType & type)
{
return(stream << type.print());
}

Strings ValueType::print(void) const
{
Strings str;
switch(_type)
  {
  case ValueType::INTEGER:
    str="int";
    break;
  case ValueType::REAL:
    str="float";
    break;
  case ValueType::STRING:
    str="string";
    break;
  case ValueType::AGENT:
    if(_model) str=_model->getName().getStrings();
    else str=Model::getBaseModel()->getName().getStrings();
    break;
  case ValueType::UNKNOWN:
    str="void";
    break;
  }
for(int i=_depth;i--;) str+="[]";
return(str);
}

//-------- Modifiers -------------------------------------------------------//
// inline void ValueType::setSimpleType(ValueType::SimpleType type)

// inline void ValueType::setModel(Model * model)

// inline Model * ValueType::accessModel(void)

// inline void ValueType::setDepth(int depth)

// inline ValueType & ValueType::operator=(const ValueType & type)


//--------------------------------------------------------------------------//
// class Value                                                              //
//--------------------------------------------------------------------------//

Word Value::_equalsToWord(Word::create("equalsTo")); // static
Word Value::_lessThanWord(Word::create("lessThan")); // static

//-------- Allocators ------------------------------------------------------//
// inline Value::Value(void)

// inline Value::Value(const ValueType & type)

Value::Value(const Value & value)
 : BlockAllocatedObject(),
   _type(value._type), _integer(value._integer), _real(value._real),
   _string(value._string), _agent(value._agent),
   _valueVector((SubValueVect *)0)
{
if(_agent) _agent->reference();
else if(value._valueVector)
  {
  _valueVector=SubValue::getSubValueVect();
  copyVector(*value._valueVector);
  }
}

Value::~Value(void)
{
if(_agent) _agent->unReference();
else if(_valueVector)
  {
  razVector();
  SubValue::relaxSubValueVect(_valueVector);
  }
}

//-------- Inspectors ------------------------------------------------------//
// inline bool Value::isLeftValue(void) const // virtual

// inline long Value::getInteger(void) const

// inline double Value::getReal(void) const

// inline const Strings & Value::getStrings(void) const

// inline Agent * Value::getAgent(void) const

// inline const ValueVector * Value::getValueVector(void) const

// inline int Value::getLen(void) const

// inline const Value & Value::getElem(int pos) const

// inline const Value & Value::getFront(void) const

// inline const Value & Value::getBack(void) const

int Value::findElem(const Value & value,bool deep) const
{
if(deep)
  {
  for(unsigned int i=0;i<_valueVector->getSize();i++)
    if((*_valueVector)[i]->deepEquals(value)) return(i);
  }
else
  {
  for(unsigned int i=0;i<_valueVector->getSize();i++)
    if(*(*_valueVector)[i]==value) return(i);
  }
return(-1); // not found
}

int Value::minPosition(void) const
{
int min=_valueVector->getSize()-1;
if(min<=0) return(min);
for(int i=min;i--;)
  if(*(*_valueVector)[i]<*(*_valueVector)[min]) min=i;
return(min);
}

int Value::maxPosition(void) const
{
int max=_valueVector->getSize()-1;
if(max<=0) return(max);
for(int i=max;i--;)
  if(*(*_valueVector)[max]<*(*_valueVector)[i]) max=i;
return(max);
}

bool Value::isNull(void) const
{
if(_valueVector) return(!_valueVector->getSize());
else
  {
  switch(_type.getSimpleType())
    {
    case ValueType::INTEGER:
      return(_integer==0);
    case ValueType::REAL:
      return(_real==0.0);
    case ValueType::STRING:
      return(_string.len()==0);
    case ValueType::AGENT:
      return(_agent ? 0 : 1);
    default:
      return(true);
    }
  }
}

// inline const ValueType & Value::getType(void) const

//-------- Comparisons -----------------------------------------------------//
bool Value::operator==(const Value & value) const
{
if(_valueVector)
  {
  if(_valueVector->getSize()!=value._valueVector->getSize()) return(false);
  for(unsigned int i=0;i<_valueVector->getSize();i++)
    if(*(*_valueVector)[i]!=*(*value._valueVector)[i]) return(false);
  return(true);
  }
else
  {
  switch(_type.getSimpleType())
    {
    case ValueType::INTEGER:
      if(value._type.getSimpleType()==ValueType::INTEGER)
        return(_integer==value._integer);
      else // REAL
        return((double)_integer==value._real);
    case ValueType::REAL:
      if(value._type.getSimpleType()==ValueType::REAL)
        return(_real==value._real);
      else // INTEGER
        return(_real==(double)value._integer);
    case ValueType::STRING:
      return(_string==value._string);
    case ValueType::AGENT:
      return(_agent==value._agent);
    default: // UNKOWN
      return(value._type.getSimpleType()==ValueType::UNKNOWN);
    }
  }
}

bool Value::deepEquals(const Value & value) const
{
if(_valueVector)
  {
  if(_valueVector->getSize()!=value._valueVector->getSize()) return(false);
  for(unsigned int i=0;i<_valueVector->getSize();i++)
    if(!(*_valueVector)[i]->deepEquals(*(*value._valueVector)[i]))
      return(false);
  return(true);
  }
else 
  {
  if(_agent&&_agent->isAlive())
    {
    ExecModule * module=_agent->accessModule(_equalsToWord);
    Execution * initExec=Execution::getExecution(_agent,ExecModule::NORMAL);
    Value * val=TmpValue::getTmpValue(value.getType());
    val->setAgent(value.getAgent());
    initExec->pushValue(val);
    getSimul()->executeModule(module,_agent,initExec);
    bool result=false;
    if(!initExec->isAborted())
      {
      val=initExec->popValue();
      result=val->getInteger();
      val->relax();
      }
    Execution::relaxExecution(initExec);
    return(result);
    }
  else
    {
    Strings str;
    if(_agent) str="deleted object '"+print()+"'";
    else str="'NONE'";
    Strings message="Can't use module 'equalsTo()' on "+str;
    getSimul()->manageError(new Error(Error::RUNTIME,message));
    return(false);
    }
  }
}

// inline bool Value::operator!=(const Value & value) const

bool Value::operator<(const Value & value) const
{
if(_valueVector)
  {
  unsigned int i;
  for(i=0;(i<_valueVector->getSize())&&(i<value._valueVector->getSize());i++)
    {
    if(*(*_valueVector)[i]<*(*value._valueVector)[i]) return(true);
    else if(*(*value._valueVector)[i]<*(*_valueVector)[i]) return(false);
    }
  if(i==_valueVector->getSize())
    return(i!=value._valueVector->getSize());
  return(false);
  }
else
  {
  switch(_type.getSimpleType())
    {
    case ValueType::INTEGER:
      if(value._type.getSimpleType()==ValueType::INTEGER)
        return(_integer<value._integer);
      else // REAL
        return((double)_integer<value._real);
    case ValueType::REAL:
      if(value._type.getSimpleType()==ValueType::REAL)
        return(_real<value._real);
      else // INTEGER
        return(_real<(double)value._integer);
    case ValueType::STRING:
      return(_string<value._string);
    case ValueType::AGENT:
      if(_agent&&_agent->isAlive())
        {
        ExecModule * module=_agent->accessModule(_lessThanWord);
        Execution * initExec=Execution::getExecution(_agent,
                                                     ExecModule::NORMAL);
        Value * val=TmpValue::getTmpValue(value.getType());
        val->setAgent(value.getAgent());
        initExec->pushValue(val);
        getSimul()->executeModule(module,_agent,initExec);
        bool result=false;
        if(!initExec->isAborted())
          {
          val=initExec->popValue();
          result=val->getInteger();
          val->relax();
          }
        Execution::relaxExecution(initExec);
        return(result);
        }
      else
        {
        Strings str;
        if(_agent) str="deleted object '"+print()+"'";
        else str="'NONE'";
        Strings message="Can't use module 'lessThan()' on "+str;
        getSimul()->manageError(new Error(Error::RUNTIME,message));
        return(false);
        }
    default:
      return(false);
    }
  }
}

// inline bool Value::operator>(const Value & value) const

// inline bool Value::operator<=(const Value & value) const

// inline bool Value::operator>=(const Value & value) const

//-------- Input/Output ----------------------------------------------------//
ostream & operator<<(ostream & stream,const Value & value)
{
return(stream << value.print());
}

Strings Value::print(void) const
{
if(_valueVector)
  {
  Strings str;
  if(!_valueVector->getSize()) str="("+_type.print()+")";
  str+="[";
  for(unsigned int i=0;i<_valueVector->getSize();i++)
    {
    if(i) str+=",";
    str+=(*_valueVector)[i]->print();
    }
  str+="]";
  return(str);
  }
else
  {
  switch(_type.getSimpleType())
    {
    case ValueType::INTEGER:
      return(Strings(_integer));
    case ValueType::REAL:
      return(Strings(_real));
    case ValueType::STRING:
      return(_string);
    case ValueType::AGENT:
      if(_agent) return(_agent->getName().print());
      else
        {
        if(_type.getModel()==Model::getBaseModel())
          return(Strings("NONE"));
        else
          return(Strings("(")+_type.print()+Strings(")NONE"));
        }
    default:
      Strings empty;
      return(empty);
    }
  }
}

Strings Value::dump(void) const
{
if(_valueVector)
  {
  Strings str;
  if(!_valueVector->getSize()) str="("+_type.print()+")";
  str+="[";
  for(unsigned int i=0;i<_valueVector->getSize();i++)
    {
    if(i) str+=",";
    str+=(*_valueVector)[i]->dump();
    }
  str+="]";
  return(str);
  }
else
  {
  switch(_type.getSimpleType())
    {
    case ValueType::STRING:
      return(_string.dump());
    case ValueType::REAL:
      return(Strings(_real,true));
    default:
      return(print());
    }
  }
}

//-------- Modifiers -------------------------------------------------------//
// inline void Value::reference(void) // virtual

// inline void Value::relax(void) // virtual

void Value::clearReferences(bool strongClear) // virtual
{
if(_agent&&!strongClear) _agent->unReference();
_agent=(Agent *)0;
if(_valueVector)
  {
  SubValue * subVal;
  while(_valueVector->getSize())
    {
    subVal=_valueVector->readPopBack();
    subVal->clearReferences(strongClear);
    SubValue::relaxSubValue(subVal);
    }
  }
}

// inline void Value::setType(const ValueType & type)

// inline ValueType & Value::accessType(void)

// inline bool Value::preset(void) // virtual

// inline void Value::postset(void) // virtual

void Value::razVector(void)
{
while(_valueVector->getSize())
  {
  SubValue::relaxSubValue(_valueVector->readPopBack());
  }
}

void Value::copyVector(const SubValueVect & valueVector)
{
razVector();
_valueVector->resizeIfNeeded(valueVector.getSize());
for(unsigned int i=0;i<valueVector.getSize();i++)
  {
  _valueVector->pushBack(SubValue::getSubValue(*valueVector[i],this));
  }
}

// inline void Value::setInteger(long integer) // virtual

// inline void Value::setReal(double real) // virtual

// inline void Value::setStrings(const Strings & string) // virtual

void Value::setAgent(Agent * agent) // virtual
{
if(_agent) _agent->unReference();
_agent=agent;
if(_agent) _agent->reference();
}

// inline void Value::setValueVectorCopy(const Value & valueVector) // virtual

void Value::setValueVectorCopyI2R(const SubValueVect & valueVector) // virtual
{
razVector();
_valueVector->resizeIfNeeded(valueVector.getSize());
ValueType type(ValueType::REAL,_type.getDepth()-1);
for(unsigned int i=0;i<valueVector.getSize();i++)
  {
  _valueVector->pushBack(SubValue::getSubValue(type,this));
  if(type.getDepth())
    _valueVector->accessBack()
                ->setValueVectorCopyI2R(*valueVector[i]->_valueVector);
  else _valueVector->accessBack()->_real=(double)(valueVector[i]->_integer);
  }
}

void Value::setValueVectorCopyR2I(const SubValueVect & valueVector) // virtual
{
razVector();
_valueVector->resizeIfNeeded(valueVector.getSize());
ValueType type(ValueType::INTEGER,_type.getDepth()-1);
for(unsigned int i=0;i<valueVector.getSize();i++)
  {
  _valueVector->pushBack(SubValue::getSubValue(type,this));
  if(type.getDepth())
    _valueVector->accessBack()
                ->setValueVectorCopyR2I(*valueVector[i]->_valueVector);
  else _valueVector->accessBack()->_integer=(long)(valueVector[i]->_real);
  }
}

// inline void Value::setValueVector(SubValueVect * valueVector) // virtual

// inline Value * Value::accessElem(int pos) // virtual

// inline Value * Value::accessFront(void) // virtual

// inline Value * Value::accessBack(void) // virtual

// inline void Value::insert(SubValue * value,int pos) // virtual

// inline void Value::insert(const Value & value,int pos) // virtual

// void Value::pushFront(SubValue * value) // virtual

// void Value::pushFront(const Value & value) // virtual

// void Value::pushBack(SubValue * value) // virtual

// void Value::pushBack(const Value & value) // virtual

// inline void Value::erase(int pos) // virtual

// inline void Value::readErase(Value & result,int pos) // virtual

// inline void Value::popFront(void) // virtual

// inline void Value::readPopFront(Value & result) // virtual

// inline void Value::popBack(void) // virtual

// inline void Value::readPopBack(Value & result) // virtual

void Value::appendVector(const SubValueVect & valueVector) // virtual
{
_valueVector->resizeIfNeeded(_valueVector->getSize()+valueVector.getSize());
for(unsigned int i=0;i<valueVector.getSize();i++)
  _valueVector->pushBack(SubValue::getSubValue(*valueVector[i],this));
}

bool Value::evalVectType(void)
{
ValueType commonType;
ValueType type;
for(unsigned int i=0;i<_valueVector->getSize();i++)
  {
  type=(*_valueVector)[i]->_type;
  if(!i) commonType=type; // first time
  else
    {
    if(type!=commonType)
      {
      if(type.getDepth()!=commonType.getDepth()) return(false);
      if(type.getSimpleType()==commonType.getSimpleType())
        {
        // type is AGENT or list of AGENT
        if(!type.canCastTo(commonType))
          {
          if(commonType.canCastTo(type)) commonType=type;
          else
            {
            commonType.setModel(Model::getBaseModel());
            }
          }
        // else keep commonType
        }
      else
        {
        switch(type.getSimpleType())
          {
          case ValueType::INTEGER:
            if(commonType.getSimpleType()!=ValueType::REAL) return(false);
            // else keep commonType
            break;
          case ValueType::REAL:
            if(commonType.getSimpleType()!=ValueType::INTEGER) return(false);
            else commonType=type;
            break;
          default:
            return(false);
          }
        }
      }
    }
  }
commonType.setDepth(commonType.getDepth()+1);
_type.setDepth(commonType.getDepth());
cast(commonType);
_type=commonType;
return(true);
}

void Value::sortVector(void) // virtual
{
if(_valueVector->getSize())
  {
  unsigned int i,j,min;
  for(i=0;i<_valueVector->getSize()-1;i++)
    {
    min=i;
    for(j=i+1;j<_valueVector->getSize();j++)
      {
      if(*(*_valueVector)[j]<*(*_valueVector)[min])
        min=j;
      }
    if(min!=i)
      {
      SubValue * tmp=(*_valueVector)[min];
      (*_valueVector)[min]=(*_valueVector)[i];
      (*_valueVector)[i]=tmp;
      }
    }
  }
}

Value & Value::operator=(const Value & value) // virtual
{
if(&value!=this)
  {
  _type=value._type;
  if(value._valueVector)
    {
    if(_agent)
      {
      _agent->unReference();
      _agent=(Agent *)0;
      }
    if(!_valueVector) _valueVector=SubValue::getSubValueVect();
    copyVector(*value._valueVector);
    }
  else
    {
    if(_valueVector)
      {
      razVector();
      SubValue::relaxSubValueVect(_valueVector);
      _valueVector=(SubValueVect *)0;
      }
    switch(_type.getSimpleType())
      {
      case ValueType::INTEGER:
        if(_agent)
          {
          _agent->unReference();
          _agent=(Agent *)0;
          }
        _integer=value._integer;
        break;
      case ValueType::REAL:
        if(_agent)
          {
          _agent->unReference();
          _agent=(Agent *)0;
          }
        _real=value._real;
        break;
      case ValueType::STRING:
        if(_agent)
          {
          _agent->unReference();
          _agent=(Agent *)0;
          }
        _string=value._string;
        break;
      case ValueType::AGENT:
        if(_agent) _agent->unReference();
        _agent=value._agent;
        if(_agent) _agent->reference();
        break;
      default:
        break;
      }
    }
  }
return(*this);
}

Value & Value::cast(const ValueType & type)
{
if(_valueVector)
  {
  ValueType newType(type);
  newType.setDepth(type.getDepth()-1);
  for(int i=_valueVector->getSize();i--;)
    (*_valueVector)[i]->cast(newType);
  if(_type!=type)
    {
    switch(_type.getSimpleType())
      {
      case ValueType::INTEGER:
        _type.setSimpleType(ValueType::REAL);
        break;
      case ValueType::REAL:
        _type.setSimpleType(ValueType::INTEGER);
        break;
      default:
        break;
      // !!! AGENT : the Model must NOT be changed in a cast !!!
      }
    }
  }
else
  {
  if(_type!=type)
    {
    switch(_type.getSimpleType())
      {
      case ValueType::INTEGER:
        _real=(double)_integer;
        _type.setSimpleType(ValueType::REAL);
        break;
      case ValueType::REAL:
        _integer=(long)_real;
        _type.setSimpleType(ValueType::INTEGER);
        break;
      default:
        break;
      // !!! AGENT : the Model must NOT be changed in a cast !!!
      }
    }       
  }
return(*this);
}

bool Value::deepCast(const ValueType & type)
{
if(_type.getDepth()!=type.getDepth()) return(false);
if(_valueVector)
  {
  ValueType subType(type);
  subType.setDepth(subType.getDepth()-1);
  for(int i=_valueVector->getSize();i--;)
    if(!(*_valueVector)[i]->deepCast(subType))
      return(false);
  // !!! AGENT : the Model must NOT be changed in a cast !!!
  if(_type.getSimpleType()!=ValueType::AGENT)
    _type=type;
  return(true);
  }
else
  {
  switch(_type.getSimpleType())
    {
    case ValueType::INTEGER:
      switch(type.getSimpleType())
        {
        case ValueType::INTEGER:
          return(true);
        case ValueType::REAL:
          _real=(double)_integer;
          _type.setSimpleType(ValueType::REAL);
          return(true);
        default:
          return(false);
        }
    case ValueType::REAL:
      switch(type.getSimpleType())
        {
        case ValueType::INTEGER:
          _integer=(long)_real;
          _type.setSimpleType(ValueType::INTEGER);
          return(true);
        case ValueType::REAL:
          return(true);
        default:
          return(false);
        }
    case ValueType::STRING:
      switch(type.getSimpleType())
        {
        case ValueType::STRING:
          return(true);
        default:
          return(false);
        }
    case ValueType::AGENT:
      switch(type.getSimpleType())
        {
        case ValueType::AGENT:
          if(!_agent)
            {
            _type=type;
            return(true);
            }
          // !!! AGENT : the Model must NOT be changed in a cast !!!
          return(!type.getModel()||
                 (_type.getModel()&&_type.getModel()->isA(type.getModel())));
        default:
          return(false);
        }
    default:
      return(false);
    }
  }
}

Value & Value::affectCast(const Value & value) // virtual
{
if(_type.getSimpleType()==value._type.getSimpleType())
  operator=(value); // !!! AGENT : the Model must NOT be changed !!!
else
  {
  ValueType oldType(_type);
  operator=(value);
  cast(oldType);
  }
return(*this);
}

//--------------------------------------------------------------------------//
// class SubValue                                                           //
//--------------------------------------------------------------------------//

SimpleVector<SubValue *> SubValue::_subValueStock;
SimpleVector<SubValueVect *> SubValue::_subValueVectStock;

//-------- Static methods --------------------------------------------------//
// inline SubValue * SubValue::getSubValue(const ValueType & type,
//                                         Value * upper) // static

// inline SubValue * SubValue::getSubValue(const Value & value,
//                                         Value * upper) // static

// inline void SubValue::relaxSubValue(SubValue * value) // static

void SubValue::clearSubValueStock(void) // static
{
while(SubValue::_subValueStock.getSize())
  {
  delete SubValue::_subValueStock.readPopBack();
  }
}

// inline SubValueVect * SubValue::getSubValueVect(void) // static

// inline void SubValue::relaxSubValueVect(SubValueVect * vect) // static

void SubValue::clearSubValueVectStock(void) // static
{
while(SubValue::_subValueVectStock.getSize())
  {
  delete SubValue::_subValueVectStock.readPopBack();
  }
}

