// @(#) ENIB/LI2 (c) - oRis - Version 2.3
//--------------------------------------------------------------------------//
// file        : affectNd.cpp                                               //
// description : source file for the ENIB/LI2 class AffectNode              //
// author(s)   : F. Harrouet                                                //
// copyright   : Laboratoire d'Informatique Industrielle (LI2)              //
//               Ecole Nationale d'Ingenieurs de Brest (ENIB)               //
//               CP 30815 - 29608 Brest Cedex - FRANCE                      //
//               Tel: (33) 02.98.05.66.31 , Fax: (33) 02.98.05.66.29        //
//               e-mail: li2@enib.fr                                        //
//--------------------------------------------------------------------------//
// Copyright (c) 1996-2000                                                  //
// Laboratoire d'Informatique Industrielle (LI2)                            //
// Ecole Nationale d'Ingenieurs de Brest (ENIB)                             //
//                                                                          //
// Permission to use, copy, modify, distribute and sell this software       //
// and its documentation for any purpose is hereby granted without fee,     //
// provided that the above copyright notice appear in all copies and        //
// that both that copyright notice and this permission notice appear        //
// in supporting documentation.                                             //
// ENIB/LI2 makes no representations about the suitability of this          //
// software for any purpose.                                                //
// It is provided "as is" without express or implied warranty.              //
//--------------------------------------------------------------------------//

#include "oRis/Nodes/affectNd.h"
#include "oRis/Main/model.h"
#include "oRis/Main/errors.h"
#include "oRis/Codes/affectCd.h"

//---------------------------------------------------------------------------//
// class AffectNode                                                          //
//---------------------------------------------------------------------------//

//-------- Allocators -------------------------------------------------------//
AffectNode::AffectNode(int lineNumber,const Strings & fileName,
                       const Strings & name,bool deep)
 : SmaNode(2,lineNumber,fileName,name,true), _deep(deep)
{}

AffectNode::AffectNode(const AffectNode & node)
 : SmaNode(node), _deep(node._deep)
{}

SmaNode * AffectNode::duplicate(void) const // virtual
{
return(new AffectNode(*this));
}

AffectNode::~AffectNode(void)
{}

//-------- Input/Output -----------------------------------------------------//
Strings AffectNode::_print(int) const // virtual
{
return(getNode(0)->print()+getName()+getNode(1)->print());
}

//-------- Evaluation -------------------------------------------------------//
bool AffectNode::check(const Context * context,const Model * model) // virtual
{
SmaNode::check(context,model);
if(!_fail)
  {
  if(_deep)
    {
    if(getNode(0)->getType().getDepth()||
       !getNode(0)->getType().canCastTo(ValueType(Model::getBaseModel()))||
       !getNode(1)->getType().canCastTo(ValueType(Model::getBaseModel())))
      {
      _fail=true;
      Strings message="Can't use '"+getName()+
                      "' between '"+
                      getNode(0)->getType().print()+
                      "' and '"+
                      getNode(1)->getType().print()+
                      "'";
      manageError(new Error(Error::SEMANTICS,message));
      }
    }
  else
    {
    if(!getNode(0)->isLeftValue())
      {
      _fail=true;
      Strings message="Left argument of '"+
                      getName()+
                      "' should be modifiable";
      manageError(new Error(Error::SEMANTICS,message));
      }
    if(!getNode(0)->getType().canCastFrom(getNode(1)->getType()))
      {
      _fail=true;
      Strings message="Can't use '"+getName()+
                      "' between '"+
                      getNode(0)->getType().print()+
                      "' and '"+
                      getNode(1)->getType().print()+
                      "'";
      manageError(new Error(Error::SEMANTICS,message));
      }
    }
  if(!_fail) _type=getNode(1)->getType();
  }
return(!_fail);
}

CodeVect * AffectNode::generateCode(void) // virtual
{
CodeVect * codeVect=new CodeVect;
for(int i=getNbNodes();i--;)
  {
  CodeVect * tmp=getNode(i)->generateCode();
  for(unsigned int j=0;j<tmp->getSize();j++)
    codeVect->pushBack((*tmp)[j]);
  delete tmp;
  }
if(_deep)
  {
  codeVect->pushBack(new Affect_Deep_Code(this));
  }
else if(getNode(0)->getType().getDepth())
  {
  if(getNode(0)->getType().getSimpleType()==
     getNode(1)->getType().getSimpleType())
    codeVect->pushBack(new Affect_V_V_Code(this));
  else
    {
    if(getNode(0)->getType().getSimpleType()==ValueType::INTEGER)
      codeVect->pushBack(new Affect_IV_RV_Code(this));
    else // REAL
      codeVect->pushBack(new Affect_RV_IV_Code(this));
    }
  }
else
  {
  switch(getNode(0)->getType().getSimpleType())
    {
    case ValueType::INTEGER:
      switch(getNode(1)->getType().getSimpleType())
        {
        case ValueType::INTEGER:
          codeVect->pushBack(new Affect_I_I_Code(this));
          break;
        case ValueType::REAL:
          codeVect->pushBack(new Affect_I_R_Code(this));
          break;
        default:
          break;
        }
      break;
    case ValueType::REAL:
      switch(getNode(1)->getType().getSimpleType())
        {
        case ValueType::REAL:
          codeVect->pushBack(new Affect_R_R_Code(this));
          break;
        case ValueType::INTEGER:
          codeVect->pushBack(new Affect_R_I_Code(this));
          break;
        default:
          break;
        }
      break;
    case ValueType::STRING:
      codeVect->pushBack(new Affect_S_S_Code(this));
      break;
    case ValueType::AGENT:
      codeVect->pushBack(new Affect_A_A_Code(this));
      break;
    default:
      break;
    }
  }
return(codeVect);
}

