// @(#) ENIB/LI2 (c) - oRis - Version 2.3
//--------------------------------------------------------------------------//
// file        : castNd.cpp                                                 //
// description : source file for the ENIB/LI2 class CastNode                //
// author(s)   : F. Harrouet                                                //
// copyright   : Laboratoire d'Informatique Industrielle (LI2)              //
//               Ecole Nationale d'Ingenieurs de Brest (ENIB)               //
//               CP 30815 - 29608 Brest Cedex - FRANCE                      //
//               Tel: (33) 02.98.05.66.31 , Fax: (33) 02.98.05.66.29        //
//               e-mail: li2@enib.fr                                        //
//--------------------------------------------------------------------------//
// Copyright (c) 1996-2000                                                  //
// Laboratoire d'Informatique Industrielle (LI2)                            //
// Ecole Nationale d'Ingenieurs de Brest (ENIB)                             //
//                                                                          //
// Permission to use, copy, modify, distribute and sell this software       //
// and its documentation for any purpose is hereby granted without fee,     //
// provided that the above copyright notice appear in all copies and        //
// that both that copyright notice and this permission notice appear        //
// in supporting documentation.                                             //
// ENIB/LI2 makes no representations about the suitability of this          //
// software for any purpose.                                                //
// It is provided "as is" without express or implied warranty.              //
//--------------------------------------------------------------------------//

#include "oRis/Nodes/castNd.h"
#include "oRis/Main/errors.h"
#include "oRis/Codes/utilCd.h"

//---------------------------------------------------------------------------//
// class CastNode                                                            //
//---------------------------------------------------------------------------//

//-------- Allocators -------------------------------------------------------//
CastNode::CastNode(int lineNumber,const Strings & fileName,
                   const Strings & name,const Strings & secondName,
                   const ValueType & type)
 : SmaNode(1,lineNumber,fileName,name,false),
   _secondName(secondName), _baseType()
{
_type=type;
}

CastNode::CastNode(const CastNode & node)
 : SmaNode(node), _secondName(node._secondName), _baseType(node._baseType)
{}

SmaNode * CastNode::duplicate(void) const // virtual
{
return(new CastNode(*this));
}

CastNode::~CastNode(void)
{}

//-------- Input/Output -----------------------------------------------------//
Strings CastNode::_print(int) const // virtual
{
return(getName()+_type.print()+_secondName+getNode(0)->print());
}

//-------- Evaluation -------------------------------------------------------//
bool CastNode::check(const Context * context,const Model * model) // virtual
{
SmaNode::check(context,model);
if(!_fail)
  {
  _baseType=_type;
  _baseType.setDepth(0);
  if(getNode(0)->getType().getDepth()==_type.getDepth())
    {
    ValueType fromBaseType(getNode(0)->getType());
    fromBaseType.setDepth(0);
    if(!fromBaseType.canCastTo(_baseType)&&
       !fromBaseType.canCastFrom(_baseType)&&
       ((fromBaseType.getSimpleType()!=ValueType::AGENT)||
        (_baseType.getSimpleType()!=ValueType::AGENT)))
      _fail=true;
    }
  else _fail=true;
  if(_fail)
    {
    Strings message="No cast from '"+
                    getNode(0)->getType().print()+
                    "' to '"+
                    _type.print()+
                    "'";
    manageError(new Error(Error::SEMANTICS,message));
    }
  }
return(!_fail);
}

CodeVect * CastNode::generateCode(void) // virtual
{
CodeVect * codeVect=getNode(0)->generateCode();
codeVect->pushBack(new CastCode(this,_type));
return(codeVect);
}

