// @(#) ENIB/LI2 (c) - oRis - Version 2.3
//--------------------------------------------------------------------------//
// file        : condNd.cpp                                                 //
// description : source file for the ENIB/LI2 class CondNode                //
// author(s)   : F. Harrouet                                                //
// copyright   : Laboratoire d'Informatique Industrielle (LI2)              //
//               Ecole Nationale d'Ingenieurs de Brest (ENIB)               //
//               CP 30815 - 29608 Brest Cedex - FRANCE                      //
//               Tel: (33) 02.98.05.66.31 , Fax: (33) 02.98.05.66.29        //
//               e-mail: li2@enib.fr                                        //
//--------------------------------------------------------------------------//
// Copyright (c) 1996-2000                                                  //
// Laboratoire d'Informatique Industrielle (LI2)                            //
// Ecole Nationale d'Ingenieurs de Brest (ENIB)                             //
//                                                                          //
// Permission to use, copy, modify, distribute and sell this software       //
// and its documentation for any purpose is hereby granted without fee,     //
// provided that the above copyright notice appear in all copies and        //
// that both that copyright notice and this permission notice appear        //
// in supporting documentation.                                             //
// ENIB/LI2 makes no representations about the suitability of this          //
// software for any purpose.                                                //
// It is provided "as is" without express or implied warranty.              //
//--------------------------------------------------------------------------//

#include "oRis/Nodes/condNd.h"
#include "oRis/Main/errors.h"
#include "oRis/Codes/utilCd.h"

//---------------------------------------------------------------------------//
// class CondNode                                                            //
//---------------------------------------------------------------------------//

//-------- Allocators -------------------------------------------------------//
CondNode::CondNode(int lineNumber,const Strings & fileName,
                   const Strings & name,const Strings & secondName)
 : SmaNode(3,lineNumber,fileName,name,false), _secondName(secondName)
{}

CondNode::CondNode(const CondNode & node)
 : SmaNode(node), _secondName(node._secondName)
{}

SmaNode * CondNode::duplicate(void) const // virtual
{
return(new CondNode(*this));
}

CondNode::~CondNode(void)
{}

//-------- Inspectors -------------------------------------------------------//
bool CondNode::isLeftValue(void) const // virtual
{
return(getNode(1)->isLeftValue()&&
       getNode(2)->isLeftValue()&&
       (getNode(1)->getType()==getNode(2)->getType()));
}

bool CondNode::isLocalValue(void) const // virtual
{
return(getNode(1)->isLocalValue()||
       getNode(2)->isLocalValue());
}

//-------- Input/Output -----------------------------------------------------//
Strings CondNode::_print(int) const // virtual
{
return(getNode(0)->print()+getName()+
       getNode(1)->print()+_secondName+
       getNode(2)->print());
}

//-------- Evaluation -------------------------------------------------------//
bool CondNode::check(const Context * context,const Model * model) // virtual
{
SmaNode::check(context,model);
if(!_fail)
  {
  ValueType type;
  if(getNode(0)->getType().getSimpleType()==ValueType::UNKNOWN)
    {
    _fail=true;
    Strings message="First argument of '"+
                    getName()+_secondName+
                    "' should not be 'void'";
    manageError(new Error(Error::SEMANTICS,message));
    }
  if(getNode(1)->getType().canCastFrom(getNode(2)->getType()))
    {
    type=getNode(1)->getType();
    if((getNode(1)->getType().getSimpleType()==ValueType::REAL)||
       (getNode(2)->getType().getSimpleType()==ValueType::REAL))
      type.setSimpleType(ValueType::REAL);
    }
  else if(getNode(1)->getType().canCastTo(getNode(2)->getType()))
    {
    type=getNode(2)->getType();
    if((getNode(1)->getType().getSimpleType()==ValueType::REAL)||
       (getNode(2)->getType().getSimpleType()==ValueType::REAL))
      type.setSimpleType(ValueType::REAL);
    }
  else
    {
    _fail=true;
    Strings message="No cast between '"+
                    getNode(1)->getType().print()+
                    "' and '"+
                    getNode(2)->getType().print()+
                    "' in '"+
                    getName()+_secondName+
                    "'";
    manageError(new Error(Error::SEMANTICS,message));
    }
  if(!_fail) _type=type;
  }
return(!_fail);
}

CodeVect * CondNode::generateCode(void) // virtual
{
CodeVect * codeVect=getNode(0)->generateCode();
CodeVect * tmp=getNode(1)->generateCode();
if(getNode(1)->getType()!=_type) tmp->pushBack(new CastCode(this,_type));
codeVect->pushBack(new JumpIfFalseCode(this,tmp->getSize()+1));
unsigned int j;
for(j=0;j<tmp->getSize();j++)
  codeVect->pushBack((*tmp)[j]);
delete tmp;
tmp=getNode(2)->generateCode();
if(getNode(2)->getType()!=_type) tmp->pushBack(new CastCode(this,_type));
codeVect->pushBack(new JumpCode(this,tmp->getSize()));
for(j=0;j<tmp->getSize();j++)
  codeVect->pushBack((*tmp)[j]);
delete tmp;
return(codeVect);
}

