// @(#) ENIB/LI2 (c) - oRis - Version 2.3
//--------------------------------------------------------------------------//
// file        : deleteNd.cpp                                               //
// description : source file for the ENIB/LI2 class DeleteNode              //
// author(s)   : F. Harrouet                                                //
// copyright   : Laboratoire d'Informatique Industrielle (LI2)              //
//               Ecole Nationale d'Ingenieurs de Brest (ENIB)               //
//               CP 30815 - 29608 Brest Cedex - FRANCE                      //
//               Tel: (33) 02.98.05.66.31 , Fax: (33) 02.98.05.66.29        //
//               e-mail: li2@enib.fr                                        //
//--------------------------------------------------------------------------//
// Copyright (c) 1996-2000                                                  //
// Laboratoire d'Informatique Industrielle (LI2)                            //
// Ecole Nationale d'Ingenieurs de Brest (ENIB)                             //
//                                                                          //
// Permission to use, copy, modify, distribute and sell this software       //
// and its documentation for any purpose is hereby granted without fee,     //
// provided that the above copyright notice appear in all copies and        //
// that both that copyright notice and this permission notice appear        //
// in supporting documentation.                                             //
// ENIB/LI2 makes no representations about the suitability of this          //
// software for any purpose.                                                //
// It is provided "as is" without express or implied warranty.              //
//--------------------------------------------------------------------------//

#include "oRis/Nodes/deleteNd.h"
#include "oRis/Main/errors.h"
#include "oRis/Codes/moduleCd.h"

//---------------------------------------------------------------------------//
// class DeleteNode                                                          //
//---------------------------------------------------------------------------//

//-------- Allocators -------------------------------------------------------//
DeleteNode::DeleteNode(int lineNumber,const Strings & fileName,
                       const Strings & name)
 : SmaNode(1,lineNumber,fileName,name,true)
{}

DeleteNode::DeleteNode(const DeleteNode & node)
 : SmaNode(node)
{}

SmaNode * DeleteNode::duplicate(void) const // virtual
{
return(new DeleteNode(*this));
}

DeleteNode::~DeleteNode(void)
{}

//-------- Input/Output -----------------------------------------------------//
Strings DeleteNode::_print(int) const // virtual
{
Strings str=getName();
if(!getNode(0)->usesParenthesis()) str+=" ";
str+=getNode(0)->print();
return(str);
}

//-------- Evaluation -------------------------------------------------------//
bool DeleteNode::check(const Context * context,const Model * model) // virtual
{
SmaNode::check(context,model);
if(!_fail)
  {
  if(!getNode(0)->getType().canCastTo(ValueType(ValueType::AGENT)))
    {
    _fail=true;
    Strings message="Argument of '"+
                    getName()+
                    "' should be an 'Object'";
    manageError(new Error(Error::SEMANTICS,message));
    }
  }
return(!_fail);
}

CodeVect * DeleteNode::generateVoidCode(void) // virtual
{
CodeVect * codeVect=getNode(0)->generateCode();
codeVect->pushBack(new DeleteCode(this));
return(codeVect);
}

