// @(#) ENIB/LI2 (c) - oRis - Version 2.3
//--------------------------------------------------------------------------//
// file        : difNd.cpp                                                  //
// description : source file for the ENIB/LI2 class DifNode                 //
// author(s)   : F. Harrouet                                                //
// copyright   : Laboratoire d'Informatique Industrielle (LI2)              //
//               Ecole Nationale d'Ingenieurs de Brest (ENIB)               //
//               CP 30815 - 29608 Brest Cedex - FRANCE                      //
//               Tel: (33) 02.98.05.66.31 , Fax: (33) 02.98.05.66.29        //
//               e-mail: li2@enib.fr                                        //
//--------------------------------------------------------------------------//
// Copyright (c) 1996-2000                                                  //
// Laboratoire d'Informatique Industrielle (LI2)                            //
// Ecole Nationale d'Ingenieurs de Brest (ENIB)                             //
//                                                                          //
// Permission to use, copy, modify, distribute and sell this software       //
// and its documentation for any purpose is hereby granted without fee,     //
// provided that the above copyright notice appear in all copies and        //
// that both that copyright notice and this permission notice appear        //
// in supporting documentation.                                             //
// ENIB/LI2 makes no representations about the suitability of this          //
// software for any purpose.                                                //
// It is provided "as is" without express or implied warranty.              //
//--------------------------------------------------------------------------//

#include "oRis/Nodes/difNd.h"
#include "oRis/Main/errors.h"
#include "oRis/Main/model.h"
#include "oRis/Codes/comparCd.h"

//---------------------------------------------------------------------------//
// class DifNode                                                             //
//---------------------------------------------------------------------------//

//-------- Allocators -------------------------------------------------------//
DifNode::DifNode(int lineNumber,const Strings & fileName,
                 const Strings & name,bool deep)
 : SmaNode(2,lineNumber,fileName,name,false), _deep(deep)
{}

DifNode::DifNode(const DifNode & node)
 : SmaNode(node), _deep(node._deep)
{}

SmaNode * DifNode::duplicate(void) const // virtual
{
return(new DifNode(*this));
}

DifNode::~DifNode(void)
{}

//-------- Input/Output -----------------------------------------------------//
Strings DifNode::_print(int) const // virtual
{
return(getNode(0)->print()+getName()+getNode(1)->print());
}

//-------- Evaluation -------------------------------------------------------//
bool DifNode::check(const Context * context,const Model * model) // virtual
{
SmaNode::check(context,model);
if(!_fail)
  {
  if(_deep)
    {
    if(getNode(0)->getType().canCastTo(ValueType(Model::getBaseModel()))&&
       getNode(1)->getType().canCastTo(ValueType(Model::getBaseModel())))
      _type=ValueType(ValueType::INTEGER);
    else
      _fail=true;
    }
  else
    {
    if(getNode(0)->getType().canCastFrom(getNode(1)->getType())||
       (getNode(0)->getType().canCastTo(ValueType(Model::getBaseModel()))&&
        getNode(1)->getType().canCastTo(ValueType(Model::getBaseModel()))))
      _type=ValueType(ValueType::INTEGER);
    else
      _fail=true;
    }
  if(_fail)
    {
    Strings message="Can't use '"+getName()+
                    "' between '"+
                    getNode(0)->getType().print()+
                    "' and '"+
                    getNode(1)->getType().print()+
                    "'";
    manageError(new Error(Error::SEMANTICS,message));
    }
  }
return(!_fail);
}

CodeVect * DifNode::generateCode(void) // virtual
{
CodeVect * codeVect=getNode(1)->generateCode();
CodeVect * tmp=getNode(0)->generateCode();
for(unsigned int j=0;j<tmp->getSize();j++)
  codeVect->pushBack((*tmp)[j]);
delete tmp;
if(_deep)
  {
  codeVect->pushBack(new Diff_Deep_Code(this));
  }
else if(getNode(0)->getType().getDepth())
  {
  codeVect->pushBack(new Diff_V_V_Code(this));
  }
else
  {
  switch(getNode(0)->getType().getSimpleType())
    {
    case ValueType::INTEGER:
      switch(getNode(1)->getType().getSimpleType())
        {
        case ValueType::INTEGER:
          codeVect->pushBack(new Diff_I_I_Code(this));
          break;
        case ValueType::REAL:
          codeVect->pushBack(new Diff_I_R_Code(this));
          break;
        default:
          break;
        }
      break;
    case ValueType::REAL:
      switch(getNode(1)->getType().getSimpleType())
        {
        case ValueType::REAL:
          codeVect->pushBack(new Diff_R_R_Code(this));
          break;
        case ValueType::INTEGER:
          codeVect->pushBack(new Diff_R_I_Code(this));
          break;
        default:
          break;
        }
      break;
    case ValueType::STRING:
      codeVect->pushBack(new Diff_S_S_Code(this));
      break;
    case ValueType::AGENT:
      codeVect->pushBack(new Diff_A_A_Code(this));
      break;
    default:
      break;
    }
  }
return(codeVect);
}

