// @(#) ENIB/LI2 (c) - oRis - Version 2.3
//--------------------------------------------------------------------------//
// file        : divNd.cpp                                                  //
// description : source file for the ENIB/LI2 class DivNode                 //
// author(s)   : F. Harrouet                                                //
// copyright   : Laboratoire d'Informatique Industrielle (LI2)              //
//               Ecole Nationale d'Ingenieurs de Brest (ENIB)               //
//               CP 30815 - 29608 Brest Cedex - FRANCE                      //
//               Tel: (33) 02.98.05.66.31 , Fax: (33) 02.98.05.66.29        //
//               e-mail: li2@enib.fr                                        //
//--------------------------------------------------------------------------//
// Copyright (c) 1996-2000                                                  //
// Laboratoire d'Informatique Industrielle (LI2)                            //
// Ecole Nationale d'Ingenieurs de Brest (ENIB)                             //
//                                                                          //
// Permission to use, copy, modify, distribute and sell this software       //
// and its documentation for any purpose is hereby granted without fee,     //
// provided that the above copyright notice appear in all copies and        //
// that both that copyright notice and this permission notice appear        //
// in supporting documentation.                                             //
// ENIB/LI2 makes no representations about the suitability of this          //
// software for any purpose.                                                //
// It is provided "as is" without express or implied warranty.              //
//--------------------------------------------------------------------------//

#include "oRis/Nodes/divNd.h"
#include "oRis/Main/errors.h"
#include "oRis/Codes/calculCd.h"

//---------------------------------------------------------------------------//
// class DivNode                                                             //
//---------------------------------------------------------------------------//

//-------- Allocators -------------------------------------------------------//
DivNode::DivNode(int lineNumber,const Strings & fileName,const Strings & name)
 : SmaNode(2,lineNumber,fileName,name,false)
{}

DivNode::DivNode(const DivNode & node)
 : SmaNode(node)
{}

SmaNode * DivNode::duplicate(void) const // virtual
{
return(new DivNode(*this));
}

DivNode::~DivNode(void)
{}

//-------- Input/Output -----------------------------------------------------//
Strings DivNode::_print(int) const // virtual
{
return(getNode(0)->print()+getName()+getNode(1)->print());
}

//-------- Evaluation -------------------------------------------------------//
bool DivNode::check(const Context * context,const Model * model) // virtual
{
SmaNode::check(context,model);
if(!_fail)
  {
  if(getNode(0)->getType().canCastTo(ValueType(ValueType::REAL))&&
     getNode(1)->getType().canCastTo(ValueType(ValueType::REAL)))
    {
    if((getNode(0)->getType().getSimpleType()==ValueType::REAL)||
       (getNode(1)->getType().getSimpleType()==ValueType::REAL))
      _type=ValueType(ValueType::REAL);
    else _type=ValueType(ValueType::INTEGER);
    }
  else
    {
    _fail=true;
    Strings message="Can't use '"+getName()+
                    "' between '"+
                    getNode(0)->getType().print()+
                    "' and '"+
                    getNode(1)->getType().print()+
                    "'";
    manageError(new Error(Error::SEMANTICS,message));
    }
  }
return(!_fail);
}

CodeVect * DivNode::generateCode(void) // virtual
{
CodeVect * codeVect=getNode(1)->generateCode();
CodeVect * tmp=getNode(0)->generateCode();
for(unsigned int j=0;j<tmp->getSize();j++)
  codeVect->pushBack((*tmp)[j]);
delete tmp;
if(getNode(0)->getType().getSimpleType()==ValueType::INTEGER)
  {
  if(getNode(1)->getType().getSimpleType()==ValueType::INTEGER)
    codeVect->pushBack(new Div_I_I_Code(this));
  else // REAL
    codeVect->pushBack(new Div_I_R_Code(this));
  }
else // REAL
  {
  if(getNode(1)->getType().getSimpleType()==ValueType::INTEGER)
    codeVect->pushBack(new Div_R_I_Code(this));
  else // REAL
    codeVect->pushBack(new Div_R_R_Code(this));
  }
return(codeVect);
}

