// @(#) ENIB/LI2 (c) - oRis - Version 2.3
//--------------------------------------------------------------------------//
// file        : doNd.cpp                                                   //
// description : source file for the ENIB/LI2 class DoNode                  //
// author(s)   : F. Harrouet                                                //
// copyright   : Laboratoire d'Informatique Industrielle (LI2)              //
//               Ecole Nationale d'Ingenieurs de Brest (ENIB)               //
//               CP 30815 - 29608 Brest Cedex - FRANCE                      //
//               Tel: (33) 02.98.05.66.31 , Fax: (33) 02.98.05.66.29        //
//               e-mail: li2@enib.fr                                        //
//--------------------------------------------------------------------------//
// Copyright (c) 1996-2000                                                  //
// Laboratoire d'Informatique Industrielle (LI2)                            //
// Ecole Nationale d'Ingenieurs de Brest (ENIB)                             //
//                                                                          //
// Permission to use, copy, modify, distribute and sell this software       //
// and its documentation for any purpose is hereby granted without fee,     //
// provided that the above copyright notice appear in all copies and        //
// that both that copyright notice and this permission notice appear        //
// in supporting documentation.                                             //
// ENIB/LI2 makes no representations about the suitability of this          //
// software for any purpose.                                                //
// It is provided "as is" without express or implied warranty.              //
//--------------------------------------------------------------------------//

#include "oRis/Nodes/doNd.h"
#include "oRis/Main/errors.h"
#include "oRis/Codes/utilCd.h"

//---------------------------------------------------------------------------//
// class DoNode                                                              //
//---------------------------------------------------------------------------//

//-------- Allocators -------------------------------------------------------//
DoNode::DoNode(int lineNumber,const Strings & fileName,const Strings & name,
               const Strings & secondName)
 : SmaNode(1,lineNumber,fileName,name,true), _secondName(secondName)
{}

DoNode::DoNode(const DoNode & node)
 : SmaNode(node), _secondName(node._secondName)
{}

SmaNode * DoNode::duplicate(void) const // virtual
{
return(new DoNode(*this));
}

DoNode::~DoNode(void)
{}

//-------- Inspectors -------------------------------------------------------//
bool DoNode::containsReturn(void) const // virtual
{
return(getNbNodes()>1&&getNode(1)->containsReturn());
}

bool DoNode::buildsParent(const Model * parent) const // virtual
{
return(getNbNodes()>1&&getNode(1)->buildsParent(parent));
}

bool DoNode::destroysParent(const Model * parent) const // virtual
{
return(getNbNodes()>1&&getNode(1)->destroysParent(parent));
}

//-------- Input/Output -----------------------------------------------------//
Strings DoNode::print(int column) const // virtual
{
Strings str;
int i;
for(i=0;i<column;i++) str+=" ";
str+=getName()+"\n";
column+=2;
for(i=0;i<column;i++) str+=" ";
str+="{\n";
if(getNbNodes()>1) str+=getNode(1)->print(column);
for(i=0;i<column;i++) str+=" ";
str+="} "+_secondName+"("+getNode(0)->print()+");\n";
return(str);
}

//-------- Evaluation -------------------------------------------------------//
bool DoNode::check(const Context * context,const Model * model) // virtual
{
SmaNode::check(context,model);
if(!_fail)
  {
  if(getNode(0)->getType().getSimpleType()==ValueType::UNKNOWN)
    {
    _fail=true;
    Strings message="Condition argument of '"+
                    getName()+" ... "+_secondName+
                    "' should not be 'void'";
    manageError(new Error(Error::SEMANTICS,message));
    }
  }
return(!_fail);
}

CodeVect * DoNode::generateVoidCode(void) // virtual
{
CodeVect * codeVect;
if(getNbNodes()>1) codeVect=getNode(1)->generateVoidCode();
else codeVect=new CodeVect;
CodeVect * tmp=getNode(0)->generateCode();
for(unsigned int j=0;j<tmp->getSize();j++)
  codeVect->pushBack((*tmp)[j]);
delete tmp;
codeVect->pushBack(new JumpIfTrueCode(this,-(codeVect->getSize()+1)));
return(codeVect);
}

