// @(#) ENIB/LI2 (c) - oRis - Version 2.3
//--------------------------------------------------------------------------//
// file        : farModNd.cpp                                               //
// description : source file for the ENIB/LI2 class FarVarNode              //
// author(s)   : F. Harrouet                                                //
// copyright   : Laboratoire d'Informatique Industrielle (LI2)              //
//               Ecole Nationale d'Ingenieurs de Brest (ENIB)               //
//               CP 30815 - 29608 Brest Cedex - FRANCE                      //
//               Tel: (33) 02.98.05.66.31 , Fax: (33) 02.98.05.66.29        //
//               e-mail: li2@enib.fr                                        //
//--------------------------------------------------------------------------//
// Copyright (c) 1996-2000                                                  //
// Laboratoire d'Informatique Industrielle (LI2)                            //
// Ecole Nationale d'Ingenieurs de Brest (ENIB)                             //
//                                                                          //
// Permission to use, copy, modify, distribute and sell this software       //
// and its documentation for any purpose is hereby granted without fee,     //
// provided that the above copyright notice appear in all copies and        //
// that both that copyright notice and this permission notice appear        //
// in supporting documentation.                                             //
// ENIB/LI2 makes no representations about the suitability of this          //
// software for any purpose.                                                //
// It is provided "as is" without express or implied warranty.              //
//--------------------------------------------------------------------------//

#include "oRis/Nodes/farModNd.h"
#include "oRis/Nodes/constNd.h"
#include "oRis/Main/errors.h"
#include "oRis/Main/simul.h"
#include "oRis/Main/context.h"
#include "oRis/Codes/moduleCd.h"

//---------------------------------------------------------------------------//
// class FarModuleNode                                                       //
//---------------------------------------------------------------------------//

//-------- Allocators -------------------------------------------------------//
FarModuleNode::FarModuleNode(int lineNumber,const Strings & fileName,
                             const Strings & name,const Strings & modelName)
 : SmaNode(1,lineNumber,fileName,name,true),
   _modelName(modelName), _virtual(false), _model((Model *)0)
{}

FarModuleNode::FarModuleNode(const FarModuleNode & node)
 : SmaNode(node), _modelName(node._modelName),
   _virtual(node._virtual), _model(node._model)
{}

SmaNode * FarModuleNode::duplicate(void) const // virtual
{
return(new FarModuleNode(*this));
}

FarModuleNode::~FarModuleNode(void)
{}

//-------- Input/Output -----------------------------------------------------//
Strings FarModuleNode::_print(int) const // virtual
{
Strings str=getNode(0)->print()+"->";
if(_modelName.len()) str+=_modelName+"::";
str+=getName()+"(";
for(int i=1;i<getNbNodes();i++)
  {
  if(i!=1) str+=",";
  str+=getNode(i)->print();
  }
str+=")";
return(str);
}

//-------- Evaluation -------------------------------------------------------//
bool FarModuleNode::check(const Context * context,const Model * model)
{                                                                   // virtual
SmaNode::check(context,model);
if(!_fail)
  {
  Error * error=(Error *)0;
  Strings name;
  if(_modelName.len()) name=_modelName+"::";
  name+=getName()+"()";
  const Model * farModel=(const Model *)0;
  if(!getNode(0)->getType().getDepth()&&
     getNode(0)->getType().canCastTo(ValueType(ValueType::AGENT)))
    farModel=getNode(0)->getType().getModel();
  if(farModel)
    {
    const ExecModule * module=(const ExecModule *)0;
    if(_modelName.len())
      {
      _virtual=false;
      _model=getSimul()->accessModel(Word::create(_modelName));
      if(_model&&farModel->isA(_model))
        module=_model->getModule(Word::create(getName()));
      }
    else
      {
      _virtual=true;
      const Agent * agent=(const Agent *)0;
      if(getNode(0)->isConst()) // call with a constant Agent name
        {
        agent=((ConstNode *)getNode(0))->getValue().getAgent();
        }
      if(agent) // not NONE
        module=agent->getModule(Word::create(getName())); // seek in agent
      else
        module=farModel->getModule(Word::create(getName())); // seek in model
      }
    if(module)
      {
      if(module->getSpecialType()!=ExecModule::NORMAL)
        {
        Strings message="Module '"+
                        name+
                        "' should not be called from '"+
                        context->getModule()->getCompleteName()+
                        "'";
        error=new Error(Error::SEMANTICS,message);
        }
      else
        {
        _type=module->getResultType();
        error=module->tryToCallFromNode(this,1);
        }
      }
    else
      {
      Strings message="Module '"+
                      name+
                      "' is unknown";
      error=new Error(Error::SEMANTICS,message);
      }
    }
  else
    {
    Strings message="Expression before '->"+
                    name+
                    "(...)' should be an 'Object'";
    error=new Error(Error::SEMANTICS,message);
    }
  if(error)
    {
    _fail=true;
    manageError(error);
    }
  }
return(!_fail);
}

CodeVect * FarModuleNode::generateCode(void) // virtual
{
CodeVect * codeVect=new CodeVect;
for(int i=getNbNodes();i--;)
  {
  CodeVect * tmp=getNode(i)->generateCode();
  for(unsigned int j=0;j<tmp->getSize();j++)
    codeVect->pushBack((*tmp)[j]);
  delete tmp;
  }
if(_virtual)
  codeVect->pushBack(new FarAgentModuleCode(this,Word::create(getName())));
else
  codeVect->pushBack(new FarModelModuleCode(this,Word::create(getName()),
                                            _model));
return codeVect;
}

