// @(#) ENIB/LI2 (c) - oRis - Version 2.3
//--------------------------------------------------------------------------//
// file        : farVarNd.cpp                                               //
// description : source file for the ENIB/LI2 class FarVarNode              //
// author(s)   : F. Harrouet                                                //
// copyright   : Laboratoire d'Informatique Industrielle (LI2)              //
//               Ecole Nationale d'Ingenieurs de Brest (ENIB)               //
//               CP 30815 - 29608 Brest Cedex - FRANCE                      //
//               Tel: (33) 02.98.05.66.31 , Fax: (33) 02.98.05.66.29        //
//               e-mail: li2@enib.fr                                        //
//--------------------------------------------------------------------------//
// Copyright (c) 1996-2000                                                  //
// Laboratoire d'Informatique Industrielle (LI2)                            //
// Ecole Nationale d'Ingenieurs de Brest (ENIB)                             //
//                                                                          //
// Permission to use, copy, modify, distribute and sell this software       //
// and its documentation for any purpose is hereby granted without fee,     //
// provided that the above copyright notice appear in all copies and        //
// that both that copyright notice and this permission notice appear        //
// in supporting documentation.                                             //
// ENIB/LI2 makes no representations about the suitability of this          //
// software for any purpose.                                                //
// It is provided "as is" without express or implied warranty.              //
//--------------------------------------------------------------------------//

#include "oRis/Nodes/farVarNd.h"
#include "oRis/Nodes/constNd.h"
#include "oRis/Main/errors.h"
#include "oRis/Main/model.h"
#include "oRis/Main/context.h"
#include "oRis/Codes/varCd.h"

//---------------------------------------------------------------------------//
// class FarVarNode                                                          //
//---------------------------------------------------------------------------//

//-------- Allocators -------------------------------------------------------//
FarVarNode::FarVarNode(int lineNumber,const Strings & fileName,
                       const Strings & name)
 : SmaNode(1,lineNumber,fileName,name,false)
{}

FarVarNode::FarVarNode(const FarVarNode & node)
 : SmaNode(node)
{}

SmaNode * FarVarNode::duplicate(void) const // virtual
{
return(new FarVarNode(*this));
}

FarVarNode::~FarVarNode(void)
{}

//-------- Inspectors -------------------------------------------------------//
bool FarVarNode::isLeftValue(void) const // virtual
{
return(true);
}

//-------- Input/Output -----------------------------------------------------//
Strings FarVarNode::_print(int) const // virtual
{
return(getNode(0)->print()+"->"+getName());
}

//-------- Evaluation -------------------------------------------------------//
bool FarVarNode::check(const Context * context,const Model * model) // virtual
{
SmaNode::check(context,model);
if(!_fail)
  {
  Error * error=(Error *)0;
  const Model * farModel=(const Model *)0;
  if(!getNode(0)->getType().getDepth()&&
     getNode(0)->getType().canCastTo(ValueType(ValueType::AGENT)))
    farModel=getNode(0)->getType().getModel();
  if(farModel)
    {
    const Attribute * attribute=(const Attribute *)0;
    const Agent * agent=(const Agent *)0;
    if(getNode(0)->isConst()) // call with a constant Agent name
      {
      agent=((ConstNode *)getNode(0))->getValue().getAgent();
      }
    if(agent) // not NONE
      attribute=agent->getAttribute(Word::create(getName())); // seek in agent
    else
      attribute=farModel->getAttribute(Word::create(getName())); // in model
    if(attribute) _type=attribute->getType();
    else
      {
      Strings message="Attribute '"+
                      farModel->getName().getStrings()+
                      "::"+
                      getName()+
                      "' is unknown";
      error=new Error(Error::SEMANTICS,message);
      }
    }
  else
    {
    Strings message="Expression before '->"+
                    getName()+
                    "' should be an 'Object'";
    error=new Error(Error::SEMANTICS,message);
    }
  if(error)
    {
    _fail=true;
    manageError(error);
    }
  }
return(!_fail);
}

CodeVect * FarVarNode::generateCode(void) // virtual
{
CodeVect * codeVect=getNode(0)->generateCode();
codeVect->pushBack(new FarVarCode(this,Word::create(getName())));
return codeVect;
}

