// @(#) ENIB/LI2 (c) - oRis - Version 2.3
//--------------------------------------------------------------------------//
// file        : forNd.cpp                                                  //
// description : source file for the ENIB/LI2 class ForNode                 //
// author(s)   : F. Harrouet                                                //
// copyright   : Laboratoire d'Informatique Industrielle (LI2)              //
//               Ecole Nationale d'Ingenieurs de Brest (ENIB)               //
//               CP 30815 - 29608 Brest Cedex - FRANCE                      //
//               Tel: (33) 02.98.05.66.31 , Fax: (33) 02.98.05.66.29        //
//               e-mail: li2@enib.fr                                        //
//--------------------------------------------------------------------------//
// Copyright (c) 1996-2000                                                  //
// Laboratoire d'Informatique Industrielle (LI2)                            //
// Ecole Nationale d'Ingenieurs de Brest (ENIB)                             //
//                                                                          //
// Permission to use, copy, modify, distribute and sell this software       //
// and its documentation for any purpose is hereby granted without fee,     //
// provided that the above copyright notice appear in all copies and        //
// that both that copyright notice and this permission notice appear        //
// in supporting documentation.                                             //
// ENIB/LI2 makes no representations about the suitability of this          //
// software for any purpose.                                                //
// It is provided "as is" without express or implied warranty.              //
//--------------------------------------------------------------------------//

#include "oRis/Nodes/forNd.h"
#include "oRis/Main/errors.h"
#include "oRis/Codes/utilCd.h"

//---------------------------------------------------------------------------//
// class ForNode                                                             //
//---------------------------------------------------------------------------//

//-------- Allocators -------------------------------------------------------//
ForNode::ForNode(int lineNumber,const Strings & fileName,const Strings & name)
 : SmaNode(3,lineNumber,fileName,name,true)
{}

ForNode::ForNode(const ForNode & node)
 : SmaNode(node)
{}

SmaNode * ForNode::duplicate(void) const // virtual
{
return(new ForNode(*this));
}

ForNode::~ForNode(void)
{}

//-------- Input/Output -----------------------------------------------------//
Strings ForNode::print(int column) const // virtual
{
Strings str;
int i;
for(i=0;i<column;i++) str+=" ";
str+=getName()+"("+getNode(0)->print()+";"
                  +getNode(1)->print()+";"
                  +getNode(2)->print()+")\n";
column+=2;
for(i=0;i<column;i++) str+=" ";
str+="{\n";
if(getNbNodes()>3) str+=getNode(3)->print(column);
for(i=0;i<column;i++) str+=" ";
str+="}\n";
return(str);
}

//-------- Evaluation -------------------------------------------------------//
bool ForNode::check(const Context * context,const Model * model) // virtual
{
SmaNode::check(context,model);
if(!_fail)
  {
  if(getNode(1)->getType().getSimpleType()==ValueType::UNKNOWN)
    {
    _fail=true;
    Strings message="Condition argument of '"+
                    getName()+
                    "' should not be 'void'";
    manageError(new Error(Error::SEMANTICS,message));
    }
  }
return(!_fail);
}

CodeVect * ForNode::generateVoidCode(void) // virtual
{
CodeVect * codeVect=getNode(0)->generateVoidCode();
unsigned int initSize=codeVect->getSize();
CodeVect * tmp=getNode(1)->generateCode();
unsigned int j;
for(j=0;j<tmp->getSize();j++)
  codeVect->pushBack((*tmp)[j]);
delete tmp;
CodeVect * body;
if(getNbNodes()>3) body=getNode(3)->generateVoidCode();
else body=new CodeVect;
tmp=getNode(2)->generateVoidCode();
for(j=0;j<tmp->getSize();j++)
  body->pushBack((*tmp)[j]);
delete tmp;
codeVect->pushBack(new JumpIfFalseCode(this,body->getSize()+1));
for(j=0;j<body->getSize();j++)
  codeVect->pushBack((*body)[j]);
delete body;
codeVect->pushBack(new JumpCode(this,-(codeVect->getSize()+1-initSize)));
return(codeVect);
}

