// @(#) ENIB/LI2 (c) - oRis - Version 2.3
//--------------------------------------------------------------------------//
// file        : ifNd.cpp                                                   //
// description : source file for the ENIB/LI2 class IfNode                  //
// author(s)   : F. Harrouet                                                //
// copyright   : Laboratoire d'Informatique Industrielle (LI2)              //
//               Ecole Nationale d'Ingenieurs de Brest (ENIB)               //
//               CP 30815 - 29608 Brest Cedex - FRANCE                      //
//               Tel: (33) 02.98.05.66.31 , Fax: (33) 02.98.05.66.29        //
//               e-mail: li2@enib.fr                                        //
//--------------------------------------------------------------------------//
// Copyright (c) 1996-2000                                                  //
// Laboratoire d'Informatique Industrielle (LI2)                            //
// Ecole Nationale d'Ingenieurs de Brest (ENIB)                             //
//                                                                          //
// Permission to use, copy, modify, distribute and sell this software       //
// and its documentation for any purpose is hereby granted without fee,     //
// provided that the above copyright notice appear in all copies and        //
// that both that copyright notice and this permission notice appear        //
// in supporting documentation.                                             //
// ENIB/LI2 makes no representations about the suitability of this          //
// software for any purpose.                                                //
// It is provided "as is" without express or implied warranty.              //
//--------------------------------------------------------------------------//

#include "oRis/Nodes/ifNd.h"
#include "oRis/Main/errors.h"
#include "oRis/Codes/utilCd.h"

//---------------------------------------------------------------------------//
// class IfNode                                                              //
//---------------------------------------------------------------------------//

//-------- Allocators -------------------------------------------------------//
IfNode::IfNode(int lineNumber,const Strings & fileName,const Strings & name,
               const Strings & secondName)
 : SmaNode(1,lineNumber,fileName,name,true), _secondName(secondName)
{}

IfNode::IfNode(const IfNode & node)
 : SmaNode(node), _secondName(node._secondName)
{}

SmaNode * IfNode::duplicate(void) const // virtual
{
return(new IfNode(*this));
}

IfNode::~IfNode(void)
{}

//-------- Inspectors -------------------------------------------------------//
bool IfNode::containsReturn(void) const // virtual
{
return((getNbNodes()==3)&&
       getNode(1)->containsReturn()&&
       getNode(2)->containsReturn());
}

bool IfNode::buildsParent(const Model * parent) const // virtual
{
return((getNbNodes()==3)&&
       getNode(1)->buildsParent(parent)&&
       getNode(2)->buildsParent(parent));
}

bool IfNode::destroysParent(const Model * parent) const // virtual
{
return((getNbNodes()==3)&&
       getNode(1)->destroysParent(parent)&&
       getNode(2)->destroysParent(parent));
}

//-------- Input/Output -----------------------------------------------------//
Strings IfNode::print(int column) const // virtual
{
Strings str;
int i;
for(i=0;i<column;i++) str+=" ";
str+=getName()+"("+getNode(0)->print()+")\n";
column+=2;
for(i=0;i<column;i++) str+=" ";
str+="{\n";
if(getNbNodes()>1) str+=getNode(1)->print(column);
for(i=0;i<column;i++) str+=" ";
str+="}\n";
if(getNbNodes()>2)
  {
  for(i=2;i<column;i++) str+=" ";
  str+=_secondName+"\n";
  for(i=0;i<column;i++) str+=" ";
  str+="{\n";
  str+=getNode(2)->print(column);
  for(i=0;i<column;i++) str+=" ";
  str+="}\n";
  }
return(str);
}

//-------- Evaluation -------------------------------------------------------//
bool IfNode::check(const Context * context,const Model * model) // virtual
{
SmaNode::check(context,model);
if(!_fail)
  {
  if(getNode(0)->getType().getSimpleType()==ValueType::UNKNOWN)
    {
    _fail=true;
    Strings message="Condition argument of '"+
                    getName()+
                    "' should not be 'void'";
    manageError(new Error(Error::SEMANTICS,message));
    }
  }
return(!_fail);
}

CodeVect * IfNode::generateVoidCode(void) // virtual
{
CodeVect * codeVect=getNode(0)->generateCode();
CodeVect * ifBlock;
if(getNbNodes()>1)
  {
  ifBlock=getNode(1)->generateVoidCode();
  if(!ifBlock->getSize())
    {
    delete ifBlock;
    ifBlock=(CodeVect *)0;
    }
  }
else ifBlock=(CodeVect *)0;
CodeVect * elseBlock;
if(getNbNodes()>2)
  {
  elseBlock=getNode(2)->generateVoidCode();
  if(!elseBlock->getSize())
    {
    delete elseBlock;
    elseBlock=(CodeVect *)0;
    }
  }
else elseBlock=(CodeVect *)0;
unsigned int j;
if(elseBlock)
  {
  if(ifBlock)
    {
    codeVect->pushBack(new JumpIfFalseCode(this,ifBlock->getSize()+1));
    for(j=0;j<ifBlock->getSize();j++)
      codeVect->pushBack((*ifBlock)[j]);
    delete ifBlock;
    codeVect->pushBack(new JumpCode(this,elseBlock->getSize()));
    for(j=0;j<elseBlock->getSize();j++)
      codeVect->pushBack((*elseBlock)[j]);
    delete elseBlock;
    }
  else
    {
    codeVect->pushBack(new JumpIfTrueCode(this,elseBlock->getSize()));
    for(j=0;j<elseBlock->getSize();j++)
      codeVect->pushBack((*elseBlock)[j]);
    delete elseBlock;
    }
  }
else
  {
  if(ifBlock)
    {
    codeVect->pushBack(new JumpIfFalseCode(this,ifBlock->getSize()));
    for(j=0;j<ifBlock->getSize();j++)
      codeVect->pushBack((*ifBlock)[j]);
    delete ifBlock;
    }
  else
    {
    codeVect->pushBack(new PopCode(this));
    }
  }
return(codeVect);
}

