// @(#) ENIB/LI2 (c) - oRis - Version 2.3
//--------------------------------------------------------------------------//
// file        : incrNd.cpp                                                 //
// description : source file for the ENIB/LI2 class IncreaseNode            //
// author(s)   : F. Harrouet                                                //
// copyright   : Laboratoire d'Informatique Industrielle (LI2)              //
//               Ecole Nationale d'Ingenieurs de Brest (ENIB)               //
//               CP 30815 - 29608 Brest Cedex - FRANCE                      //
//               Tel: (33) 02.98.05.66.31 , Fax: (33) 02.98.05.66.29        //
//               e-mail: li2@enib.fr                                        //
//--------------------------------------------------------------------------//
// Copyright (c) 1996-2000                                                  //
// Laboratoire d'Informatique Industrielle (LI2)                            //
// Ecole Nationale d'Ingenieurs de Brest (ENIB)                             //
//                                                                          //
// Permission to use, copy, modify, distribute and sell this software       //
// and its documentation for any purpose is hereby granted without fee,     //
// provided that the above copyright notice appear in all copies and        //
// that both that copyright notice and this permission notice appear        //
// in supporting documentation.                                             //
// ENIB/LI2 makes no representations about the suitability of this          //
// software for any purpose.                                                //
// It is provided "as is" without express or implied warranty.              //
//--------------------------------------------------------------------------//

#include "oRis/Nodes/incrNd.h"
#include "oRis/Main/errors.h"
#include "oRis/Codes/calculCd.h"

//---------------------------------------------------------------------------//
// class IncreaseNode                                                        //
//---------------------------------------------------------------------------//

//-------- Allocators -------------------------------------------------------//
IncreaseNode::IncreaseNode(int lineNumber,const Strings & fileName,
                           const Strings & name,bool postInc)
 : SmaNode(1,lineNumber,fileName,name,true), _postInc(postInc)
{}

IncreaseNode::IncreaseNode(const IncreaseNode & node)
 : SmaNode(node), _postInc(node._postInc)
{}

SmaNode * IncreaseNode::duplicate(void) const // virtual
{
return(new IncreaseNode(*this));
}

IncreaseNode::~IncreaseNode(void)
{}

//-------- Input/Output -----------------------------------------------------//
Strings IncreaseNode::_print(int) const // virtual
{
return(_postInc ? getNode(0)->print()+getName()
                : getName()+getNode(0)->print());
}

//-------- Evaluation -------------------------------------------------------//
bool IncreaseNode::check(const Context * context,const Model * model)
{                                                                   // virtual
SmaNode::check(context,model);
if(!_fail)
  {
  if(!getNode(0)->isLeftValue())
    {
    _fail=true;
    Strings message="Argument of '"+
                    getName()+
                    "' should be modifiable";
    manageError(new Error(Error::SEMANTICS,message));
    }
  if(!getNode(0)->getType().canCastTo(ValueType(ValueType::INTEGER)))
    {
    _fail=true;
    Strings message="Can't use '"+
                    getName()+
                    "' on a '"+
                    getNode(0)->getType().print()+
                    "'";
    manageError(new Error(Error::SEMANTICS,message));
    }
  if(!_fail) _type=getNode(0)->getType();
  }
return(!_fail);
}

CodeVect * IncreaseNode::generateCode(void) // virtual
{
CodeVect * codeVect=getNode(0)->generateCode();
if(getNode(0)->getType().getSimpleType()==ValueType::INTEGER)
  {
  if(_postInc) codeVect->pushBack(new PostIncr_I_Code(this));
  else codeVect->pushBack(new PreIncr_I_Code(this));
  }
else // REAL
  {
  if(_postInc) codeVect->pushBack(new PostIncr_R_Code(this));
  else codeVect->pushBack(new PreIncr_R_Code(this));
  }
return(codeVect);
}

CodeVect * IncreaseNode::generateVoidCode(void) // virtual
{
CodeVect * codeVect=getNode(0)->generateCode();
if(getNode(0)->getType().getSimpleType()==ValueType::INTEGER)
  codeVect->pushBack(new Incr_I_Code(this));
else // REAL
  codeVect->pushBack(new Incr_R_Code(this));
return(codeVect);
}

