// @(#) ENIB/LI2 (c) - oRis - Version 2.3
//--------------------------------------------------------------------------//
// file        : indexNd.cpp                                                //
// description : source file for the ENIB/LI2 class IndexNode               //
// author(s)   : F. Harrouet                                                //
// copyright   : Laboratoire d'Informatique Industrielle (LI2)              //
//               Ecole Nationale d'Ingenieurs de Brest (ENIB)               //
//               CP 30815 - 29608 Brest Cedex - FRANCE                      //
//               Tel: (33) 02.98.05.66.31 , Fax: (33) 02.98.05.66.29        //
//               e-mail: li2@enib.fr                                        //
//--------------------------------------------------------------------------//
// Copyright (c) 1996-2000                                                  //
// Laboratoire d'Informatique Industrielle (LI2)                            //
// Ecole Nationale d'Ingenieurs de Brest (ENIB)                             //
//                                                                          //
// Permission to use, copy, modify, distribute and sell this software       //
// and its documentation for any purpose is hereby granted without fee,     //
// provided that the above copyright notice appear in all copies and        //
// that both that copyright notice and this permission notice appear        //
// in supporting documentation.                                             //
// ENIB/LI2 makes no representations about the suitability of this          //
// software for any purpose.                                                //
// It is provided "as is" without express or implied warranty.              //
//--------------------------------------------------------------------------//

#include "oRis/Nodes/indexNd.h"
#include "oRis/Main/errors.h"
#include "oRis/Codes/listCd.h"

//---------------------------------------------------------------------------//
// class IndexNode                                                           //
//---------------------------------------------------------------------------//

//-------- Allocators -------------------------------------------------------//
IndexNode::IndexNode(int lineNumber,const Strings & fileName, 
                     const Strings & name,const Strings & secondName)
 : SmaNode(2,lineNumber,fileName,name,false), _secondName(secondName)
{}

IndexNode::IndexNode(const IndexNode & node)
 : SmaNode(node), _secondName(node._secondName)
{}

SmaNode * IndexNode::duplicate(void) const // virtual
{
return(new IndexNode(*this));
}

IndexNode::~IndexNode(void)
{}

//-------- Inspectors -------------------------------------------------------//
bool IndexNode::isLeftValue(void) const // virtual
{
if(getNode(0)->getType().canCastFrom(ValueType(ValueType::STRING)))
  return(false);
return(getNode(0)->isLeftValue());
}

bool IndexNode::isLocalValue(void) const // virtual
{
if(getNode(0)->getType().canCastFrom(ValueType(ValueType::STRING)))
  return(false);
return(getNode(0)->isLocalValue());
}

//-------- Input/Output -----------------------------------------------------//
Strings IndexNode::_print(int) const // virtual
{
return(getNode(0)->print()+getName()+getNode(1)->print()+_secondName);
}

//-------- Evaluation -------------------------------------------------------//
bool IndexNode::check(const Context * context,const Model * model) // virtual
{
SmaNode::check(context,model);
if(!_fail)
  {
  if(!getNode(1)->getType().canCastTo(ValueType(ValueType::INTEGER)))
    {
    _fail=true;
    Strings message="Argument of '"+
                    getName()+_secondName+
                    "' should be an 'int'";
    manageError(new Error(Error::SEMANTICS,message));

    }
  if(getNode(0)->getType().canCastFrom(ValueType(ValueType::STRING)))
    {
    if(!_fail) _type=getNode(0)->getType();
    }
  else if(getNode(0)->getType().getDepth())
    {
    if(!_fail)
      {
      _type=getNode(0)->getType();
      _type.setDepth(_type.getDepth()-1);
      }
    }
  else
    {
    _fail=true;
    Strings message="Expression before '"+
                    getName()+
                    "..."+
                    _secondName+
                    "' should be a 'string' or a list";
    manageError(new Error(Error::SEMANTICS,message));
    }
  }
return(!_fail);
}

CodeVect * IndexNode::generateCode(void) // virtual
{
CodeVect * codeVect=new CodeVect;
for(int i=0;i<getNbNodes();i++)
  {
  CodeVect * tmp=getNode(i)->generateCode();
  for(unsigned int j=0;j<tmp->getSize();j++)
    codeVect->pushBack((*tmp)[j]);
  delete tmp;
  }
if(getNode(0)->getType().canCastFrom(ValueType(ValueType::STRING)))
  {
  if(getNode(1)->getType().getSimpleType()==ValueType::INTEGER)
    codeVect->pushBack(new Index_S_I_Code(this));
  else // REAL
    codeVect->pushBack(new Index_S_R_Code(this));
  }
else
  {
  if(getNode(0)->isLeftValue())
    {
    if(getNode(1)->getType().getSimpleType()==ValueType::INTEGER)
      codeVect->pushBack(new IndexRef_I_Code(this));
    else // REAL
      codeVect->pushBack(new IndexRef_R_Code(this));
    }
  else
    {
    if(getNode(1)->getType().getSimpleType()==ValueType::INTEGER)
      codeVect->pushBack(new IndexCopy_I_Code(this));
    else // REAL
      codeVect->pushBack(new IndexCopy_R_Code(this));
    }
  }
return(codeVect);
}

