// @(#) ENIB/LI2 (c) - oRis - Version 2.3
//--------------------------------------------------------------------------//
// file        : infNd.cpp                                                  //
// description : source file for the ENIB/LI2 class InfNode                 //
// author(s)   : F. Harrouet                                                //
// copyright   : Laboratoire d'Informatique Industrielle (LI2)              //
//               Ecole Nationale d'Ingenieurs de Brest (ENIB)               //
//               CP 30815 - 29608 Brest Cedex - FRANCE                      //
//               Tel: (33) 02.98.05.66.31 , Fax: (33) 02.98.05.66.29        //
//               e-mail: li2@enib.fr                                        //
//--------------------------------------------------------------------------//
// Copyright (c) 1996-2000                                                  //
// Laboratoire d'Informatique Industrielle (LI2)                            //
// Ecole Nationale d'Ingenieurs de Brest (ENIB)                             //
//                                                                          //
// Permission to use, copy, modify, distribute and sell this software       //
// and its documentation for any purpose is hereby granted without fee,     //
// provided that the above copyright notice appear in all copies and        //
// that both that copyright notice and this permission notice appear        //
// in supporting documentation.                                             //
// ENIB/LI2 makes no representations about the suitability of this          //
// software for any purpose.                                                //
// It is provided "as is" without express or implied warranty.              //
//--------------------------------------------------------------------------//

#include "oRis/Nodes/infNd.h"
#include "oRis/Main/model.h"
#include "oRis/Main/errors.h"
#include "oRis/Codes/comparCd.h"

//---------------------------------------------------------------------------//
// class InfNode                                                             //
//---------------------------------------------------------------------------//

//-------- Allocators -------------------------------------------------------//
InfNode::InfNode(int lineNumber,const Strings & fileName,const Strings & name)
 : SmaNode(2,lineNumber,fileName,name,false)
{}

InfNode::InfNode(const InfNode & node)
 : SmaNode(node)
{}

SmaNode * InfNode::duplicate(void) const // virtual
{
return(new InfNode(*this));
}

InfNode::~InfNode(void)
{}

//-------- Input/Output -----------------------------------------------------//
Strings InfNode::_print(int) const // virtual
{
return(getNode(0)->print()+getName()+getNode(1)->print());
}

//-------- Evaluation -------------------------------------------------------//
bool InfNode::check(const Context * context,const Model * model) // virtual
{
SmaNode::check(context,model);
if(!_fail)
  {
  if(getNode(0)->getType().canCastTo(ValueType(Model::getBaseModel())))
    {
    if(!getNode(1)->getType().canCastTo(ValueType(Model::getBaseModel())))
      _fail=true;
    }
  else if(getNode(1)->getType().canCastTo(ValueType(Model::getBaseModel())))
    {
    _fail=true;
    }
  else if(!getNode(0)->getType().canCastFrom(getNode(1)->getType()))
    {
    _fail=true;
    }
  if(_fail)
    {
    Strings message="Can't use '"+getName()+
                    "' between '"+
                    getNode(0)->getType().print()+
                    "' and '"+
                    getNode(1)->getType().print()+
                    "'";
    manageError(new Error(Error::SEMANTICS,message));
    }
  else
    {
    _type=ValueType(ValueType::INTEGER);
    }
  }
return(!_fail);
}

CodeVect * InfNode::generateCode(void) // virtual
{
CodeVect * codeVect=getNode(1)->generateCode();
CodeVect * tmp=getNode(0)->generateCode();
for(unsigned int j=0;j<tmp->getSize();j++)
  codeVect->pushBack((*tmp)[j]);
delete tmp;
if(getNode(0)->getType().getDepth()||
   (getNode(0)->getType().getSimpleType()==ValueType::AGENT))
  {
  codeVect->pushBack(new Inf_Deep_Code(this));
  }
else
  {
  switch(getNode(0)->getType().getSimpleType())
    {
    case ValueType::INTEGER:
      switch(getNode(1)->getType().getSimpleType())
        {
        case ValueType::INTEGER:
          codeVect->pushBack(new Inf_I_I_Code(this));
          break;
        case ValueType::REAL:
          codeVect->pushBack(new Inf_I_R_Code(this));
          break;
        default:
          break;
        }
      break;
    case ValueType::REAL:
      switch(getNode(1)->getType().getSimpleType())
        {
        case ValueType::REAL:
          codeVect->pushBack(new Inf_R_R_Code(this));
          break;
        case ValueType::INTEGER:
          codeVect->pushBack(new Inf_R_I_Code(this));
          break;
        default:
          break;
        }
      break;
    case ValueType::STRING:
      codeVect->pushBack(new Inf_S_S_Code(this));
      break;
    default:
      break;
    }
  }
return(codeVect);
}

