// @(#) ENIB/LI2 (c) - oRis - Version 2.3
//--------------------------------------------------------------------------//
// file        : intRndNd.cpp                                               //
// description : source file for the ENIB/LI2 class IntRndNode              //
// author(s)   : F. Harrouet                                                //
// copyright   : Laboratoire d'Informatique Industrielle (LI2)              //
//               Ecole Nationale d'Ingenieurs de Brest (ENIB)               //
//               CP 30815 - 29608 Brest Cedex - FRANCE                      //
//               Tel: (33) 02.98.05.66.31 , Fax: (33) 02.98.05.66.29        //
//               e-mail: li2@enib.fr                                        //
//--------------------------------------------------------------------------//
// Copyright (c) 1996-2000                                                  //
// Laboratoire d'Informatique Industrielle (LI2)                            //
// Ecole Nationale d'Ingenieurs de Brest (ENIB)                             //
//                                                                          //
// Permission to use, copy, modify, distribute and sell this software       //
// and its documentation for any purpose is hereby granted without fee,     //
// provided that the above copyright notice appear in all copies and        //
// that both that copyright notice and this permission notice appear        //
// in supporting documentation.                                             //
// ENIB/LI2 makes no representations about the suitability of this          //
// software for any purpose.                                                //
// It is provided "as is" without express or implied warranty.              //
//--------------------------------------------------------------------------//

#include "oRis/Nodes/intRndNd.h"
#include "oRis/Main/errors.h"
#include "oRis/Main/context.h"
#include "oRis/Codes/valCd.h"

//---------------------------------------------------------------------------//
// class IntRndNode                                                          //
//---------------------------------------------------------------------------//

//-------- Allocators -------------------------------------------------------//
IntRndNode::IntRndNode(int lineNumber,const Strings & fileName,
                       const Strings & name)
 : SmaNode(1,lineNumber,fileName,name,false)
{}

IntRndNode::IntRndNode(const IntRndNode & node)
 : SmaNode(node)
{}

SmaNode * IntRndNode::duplicate(void) const // virtual
{
return(new IntRndNode(*this));
}

IntRndNode::~IntRndNode(void)
{}

//-------- Input/Output -----------------------------------------------------//

//-------- Evaluation -------------------------------------------------------//
bool IntRndNode::check(const Context * context,const Model * model) // virtual
{
SmaNode::check(context,model);
if(!_fail)
  {
  if(!getNode(0)->getType().canCastTo(ValueType(ValueType::INTEGER)))
    {
    _fail=true;
    Strings message="Argument of '"+
                    getName()+
                    "' should be an 'int'";
    manageError(new Error(Error::SEMANTICS,message));
    }
  else  _type=ValueType(ValueType::INTEGER);
  }
return(!_fail);
}

CodeVect * IntRndNode::generateCode(void) // virtual
{
CodeVect * codeVect=getNode(0)->generateCode();
if(getNode(0)->getType().getSimpleType()==ValueType::INTEGER)
  codeVect->pushBack(new IntRnd_I_Code(this));
else
  codeVect->pushBack(new IntRnd_R_Code(this));
return codeVect;
}

