// @(#) ENIB/LI2 (c) - oRis - Version 2.3
//--------------------------------------------------------------------------//
// file        : lengthNd.cpp                                               //
// description : source file for the ENIB/LI2 class LengthNode              //
// author(s)   : F. Harrouet                                                //
// copyright   : Laboratoire d'Informatique Industrielle (LI2)              //
//               Ecole Nationale d'Ingenieurs de Brest (ENIB)               //
//               CP 30815 - 29608 Brest Cedex - FRANCE                      //
//               Tel: (33) 02.98.05.66.31 , Fax: (33) 02.98.05.66.29        //
//               e-mail: li2@enib.fr                                        //
//--------------------------------------------------------------------------//
// Copyright (c) 1996-2000                                                  //
// Laboratoire d'Informatique Industrielle (LI2)                            //
// Ecole Nationale d'Ingenieurs de Brest (ENIB)                             //
//                                                                          //
// Permission to use, copy, modify, distribute and sell this software       //
// and its documentation for any purpose is hereby granted without fee,     //
// provided that the above copyright notice appear in all copies and        //
// that both that copyright notice and this permission notice appear        //
// in supporting documentation.                                             //
// ENIB/LI2 makes no representations about the suitability of this          //
// software for any purpose.                                                //
// It is provided "as is" without express or implied warranty.              //
//--------------------------------------------------------------------------//

#include "oRis/Nodes/lengthNd.h"
#include "oRis/Main/errors.h"
#include "oRis/Codes/listCd.h"

//---------------------------------------------------------------------------//
// class LengthNode                                                            //
//---------------------------------------------------------------------------//

//-------- Allocators -------------------------------------------------------//
LengthNode::LengthNode(int lineNumber,const Strings & fileName,
                       const Strings & name)
 : SmaNode(1,lineNumber,fileName,name,false)
{}

LengthNode::LengthNode(const LengthNode & node)
 : SmaNode(node)
{}

SmaNode * LengthNode::duplicate(void) const // virtual
{
return(new LengthNode(*this));
}

LengthNode::~LengthNode(void)
{}

//-------- Input/Output -----------------------------------------------------//
Strings LengthNode::_print(int) const // virtual
{
return(getNode(0)->print()+"."+getName()+"()");
}

//-------- Evaluation -------------------------------------------------------//
bool LengthNode::check(const Context * context,const Model * model) // virtual
{
SmaNode::check(context,model);
if(!_fail)
  {
  if(!getNode(0)->getType().getDepth()&&
     !getNode(0)->getType().canCastFrom(ValueType(ValueType::STRING)))
    {
    _fail=true;
    Strings message="Expression before '."+
                    getName()+
                    "()' should be a 'string' or a list";
    manageError(new Error(Error::SEMANTICS,message));
    }
  else
    {
    _type=ValueType(ValueType::INTEGER);
    }
  }
return(!_fail);
}

CodeVect * LengthNode::generateCode(void) // virtual
{
CodeVect * codeVect=getNode(0)->generateCode();
if(getNode(0)->getType().getDepth())
  codeVect->pushBack(new Length_V_Code(this));
else
  codeVect->pushBack(new Length_S_Code(this));
return(codeVect);
}

