// @(#) ENIB/LI2 (c) - oRis - Version 2.3
//--------------------------------------------------------------------------//
// file        : maxNd.cpp                                                  //
// description : source file for the ENIB/LI2 class MaxNode                 //
// author(s)   : F. Harrouet                                                //
// copyright   : Laboratoire d'Informatique Industrielle (LI2)              //
//               Ecole Nationale d'Ingenieurs de Brest (ENIB)               //
//               CP 30815 - 29608 Brest Cedex - FRANCE                      //
//               Tel: (33) 02.98.05.66.31 , Fax: (33) 02.98.05.66.29        //
//               e-mail: li2@enib.fr                                        //
//--------------------------------------------------------------------------//
// Copyright (c) 1996-2000                                                  //
// Laboratoire d'Informatique Industrielle (LI2)                            //
// Ecole Nationale d'Ingenieurs de Brest (ENIB)                             //
//                                                                          //
// Permission to use, copy, modify, distribute and sell this software       //
// and its documentation for any purpose is hereby granted without fee,     //
// provided that the above copyright notice appear in all copies and        //
// that both that copyright notice and this permission notice appear        //
// in supporting documentation.                                             //
// ENIB/LI2 makes no representations about the suitability of this          //
// software for any purpose.                                                //
// It is provided "as is" without express or implied warranty.              //
//--------------------------------------------------------------------------//

#include "oRis/Nodes/maxNd.h"
#include "oRis/Main/model.h"
#include "oRis/Main/errors.h"
#include "oRis/Codes/comparCd.h"
#include "oRis/Codes/utilCd.h"

//---------------------------------------------------------------------------//
// class MaxNode                                                             //
//---------------------------------------------------------------------------//

//-------- Allocators -------------------------------------------------------//
MaxNode::MaxNode(int lineNumber,const Strings & fileName,const Strings & name)
 : SmaNode(2,lineNumber,fileName,name,false)
{}

MaxNode::MaxNode(const MaxNode & node)
 : SmaNode(node)
{}

SmaNode * MaxNode::duplicate(void) const // virtual
{
return(new MaxNode(*this));
}

MaxNode::~MaxNode(void)
{}

//-------- Inspectors -------------------------------------------------------//
bool MaxNode::isLeftValue(void) const // virtual
{
return(getNode(0)->isLeftValue()&&
       getNode(1)->isLeftValue()&&
       (getNode(0)->getType()==getNode(1)->getType()));
}

bool MaxNode::isLocalValue(void) const // virtual
{
return(getNode(0)->isLocalValue()||
       getNode(1)->isLocalValue());
}

//-------- Input/Output -----------------------------------------------------//
Strings MaxNode::_print(int) const // virtual
{
return(getNode(0)->print()+getName()+getNode(1)->print());
}

//-------- Evaluation -------------------------------------------------------//
bool MaxNode::check(const Context * context,const Model * model) // virtual
{
SmaNode::check(context,model);
if(!_fail)
  {
  if(getNode(0)->getType().canCastFrom(getNode(1)->getType()))
    {
    _type=getNode(0)->getType();
    if((getNode(0)->getType().getSimpleType()==ValueType::REAL)||
       (getNode(1)->getType().getSimpleType()==ValueType::REAL))
      _type.setSimpleType(ValueType::REAL);
    }
  else if(getNode(0)->getType().canCastTo(getNode(1)->getType()))
    {
    _type=getNode(1)->getType();
    if((getNode(0)->getType().getSimpleType()==ValueType::REAL)||
       (getNode(1)->getType().getSimpleType()==ValueType::REAL))
      _type.setSimpleType(ValueType::REAL);
    }
  else
    {
    _fail=true;
    Strings message="No cast between '"+
                    getNode(1)->getType().print()+
                    "' and '"+
                    getNode(2)->getType().print()+
                    "' in '"+getName()+"'";
    manageError(new Error(Error::SEMANTICS,message));
    }
  }
return(!_fail);
}

CodeVect * MaxNode::generateCode(void) // virtual
{
CodeVect * codeVect=getNode(1)->generateCode();
if(getNode(1)->getType()!=_type)
  codeVect->pushBack(new CastCode(this,_type));
CodeVect * tmp=getNode(0)->generateCode();
for(unsigned int j=0;j<tmp->getSize();j++)
  codeVect->pushBack((*tmp)[j]);
delete tmp;
if(getNode(0)->getType()!=_type)
  codeVect->pushBack(new CastCode(this,_type));
if(getNode(0)->getType().getDepth()||
   (getNode(0)->getType().getSimpleType()==ValueType::AGENT))
  {
  codeVect->pushBack(new Max_Deep_Code(this));
  }
else
  {
  switch(_type.getSimpleType())
    {
    case ValueType::INTEGER:
      codeVect->pushBack(new Max_I_Code(this));
      break;
    case ValueType::REAL:
      codeVect->pushBack(new Max_R_Code(this));
      break;
    case ValueType::STRING:
      codeVect->pushBack(new Max_S_Code(this));
      break;
    default:
      break;
    }
  }
return(codeVect);
}

