// @(#) ENIB/LI2 (c) - oRis - Version 2.3
//--------------------------------------------------------------------------//
// file        : moduleNd.cpp                                               //
// description : source file for the ENIB/LI2 class ModuleNode              //
// author(s)   : F. Harrouet                                                //
// copyright   : Laboratoire d'Informatique Industrielle (LI2)              //
//               Ecole Nationale d'Ingenieurs de Brest (ENIB)               //
//               CP 30815 - 29608 Brest Cedex - FRANCE                      //
//               Tel: (33) 02.98.05.66.31 , Fax: (33) 02.98.05.66.29        //
//               e-mail: li2@enib.fr                                        //
//--------------------------------------------------------------------------//
// Copyright (c) 1996-2000                                                  //
// Laboratoire d'Informatique Industrielle (LI2)                            //
// Ecole Nationale d'Ingenieurs de Brest (ENIB)                             //
//                                                                          //
// Permission to use, copy, modify, distribute and sell this software       //
// and its documentation for any purpose is hereby granted without fee,     //
// provided that the above copyright notice appear in all copies and        //
// that both that copyright notice and this permission notice appear        //
// in supporting documentation.                                             //
// ENIB/LI2 makes no representations about the suitability of this          //
// software for any purpose.                                                //
// It is provided "as is" without express or implied warranty.              //
//--------------------------------------------------------------------------//

#include "oRis/Nodes/moduleNd.h"
#include "oRis/Main/errors.h"
#include "oRis/Main/simul.h"
#include "oRis/Main/context.h"
#include "oRis/Codes/moduleCd.h"

//---------------------------------------------------------------------------//
// class ModuleNode                                                          //
//---------------------------------------------------------------------------//

//-------- Allocators -------------------------------------------------------//
ModuleNode::ModuleNode(int lineNumber,const Strings & fileName,
                       const Strings & name,const Strings & secondName)
 : SmaNode(0,lineNumber,fileName,name,true),
   _secondName(secondName), _virtual(false), _global(false),
   _constructor(false), _destructor(false), _model((Model*)0)
{}

ModuleNode::ModuleNode(const ModuleNode & node)
 : SmaNode(node), _secondName(node._secondName), _virtual(node._virtual),
   _global(node._global), _constructor(node._constructor),
   _destructor(node._destructor), _model(node._model)
{}

SmaNode * ModuleNode::duplicate(void) const // virtual
{
return(new ModuleNode(*this));
}

ModuleNode::~ModuleNode(void)
{}

//-------- Ins[ectors -------------------------------------------------------//
bool ModuleNode::buildsParent(const Model * parent) const // virtual
{
if(parent->getName().getStrings()!=_secondName) return(false);
const ExecModule * module=parent->getNewModule();
return(module&&module->getName().getStrings()==getName());
}

bool ModuleNode::destroysParent(const Model * parent) const // virtual
{
if(parent->getName().getStrings()!=_secondName) return(false);
const ExecModule * module=parent->getDeleteModule();
return(module&&module->getName().getStrings()==getName());
}

//-------- Input/Output -----------------------------------------------------//
Strings ModuleNode::_print(int) const // virtual
{
Strings str;
if(_secondName.len())
  {
  if(_secondName!=Strings("::")) str+=_secondName;
  str+="::";
  }
str+=getName()+"(";
for(int i=0;i<getNbNodes();i++)
  {
  if(i) str+=",";
  str+=getNode(i)->print();
  }
str+=")";
return(str);
}

//-------- Evaluation -------------------------------------------------------//
bool ModuleNode::check(const Context * context,const Model * model) // virtual
{
SmaNode::check(context,model);
if(!_fail)
  {
  Error * error=(Error *)0;
  Strings name=_secondName;
  if(_secondName.len())
    {
    if(_secondName!=Strings("::")) name=_secondName;
    name+="::";
    }
  name+=getName()+"()";
  const ExecModule * module=(const ExecModule *)0;
  _constructor=false;
  _destructor=false;
  if(_secondName.len())
    {
    _virtual=false;
    if(_secondName==Strings("::"))
      {
      _global=true;
      module=getSimul()->getModule(Word::create(getName()));
      }
    else
      {
      _global=false;
      _model=getSimul()->accessModel(Word::create(_secondName));
      if((context->getOwner()&&
          context->getOwner()->getModel()->isA(_model))||
         (model&&model->isA(_model)))
        {
        module=_model->getModule(Word::create(getName()));
        if(module)
          {
          if(module==_model->getNewModule()) _constructor=true;
          else if(module==_model->getDeleteModule()) _destructor=true;
          }
        }
      }
    }
  else
    {
    if(context->getOwner())
      module=context->getOwner()->getModule(Word::create(getName()));
    else if(model)
      module=model->getModule(Word::create(getName()));
    if(module)
      {
      _virtual=true;
      _global=false;
      }
    else
      {
      _virtual=false;
      _global=true;
      module=getSimul()->getModule(Word::create(getName()));
      }
    }
  if(module)
    {
    const ExecModule * curModule=context->getModule();
    switch(curModule->getSpecialType())
      {
      case ExecModule::NORMAL:
        if(module->getSpecialType()!=ExecModule::NORMAL)
          {
          Strings message="Module '"+
                          name+
                          "' should not be called from '"+
                          curModule->getCompleteName()+
                          "'";
          error=new Error(Error::SEMANTICS,message);
          }
        break;
      case ExecModule::BEHAVIOR:
        if((module->getSpecialType()!=ExecModule::NORMAL)&&
           (module->getSpecialType()!=ExecModule::BEHAVIOR))
          {
          Strings message="Module '"+
                          name+
                          "' should not be called from '"+
                          curModule->getCompleteName()+
                          "'";
          error=new Error(Error::SEMANTICS,message);
          }
        break;
      case ExecModule::CONSTRUCTOR:
        if(module->getSpecialType()!=ExecModule::NORMAL)
          {
          const Model * curModel=model;
          if(!curModel) curModel=context->getOwner()->getModel();
          if((module->getSpecialType()!=ExecModule::CONSTRUCTOR)||
             !curModel->hasParent(Word::create(_secondName)))
            {
            Strings message="Module '"+
                            name+
                            "' should not be called from '"+
                            curModule->getCompleteName()+
                            "'";
            error=new Error(Error::SEMANTICS,message);
            }
          }
        break;
      case ExecModule::DESTRUCTOR:
        if(module->getSpecialType()!=ExecModule::NORMAL)
          {
          const Model * curModel=model;
          if(!curModel) curModel=context->getOwner()->getModel();
          if((module->getSpecialType()!=ExecModule::DESTRUCTOR)||
             ((_secondName!=curModel->getName().getStrings())&&
              !curModel->hasParent(Word::create(_secondName))))
            {
            Strings message="Module '"+
                            name+
                            "' should not be called from '"+
                            curModule->getCompleteName()+
                            "'";
            error=new Error(Error::SEMANTICS,message);
            }
          }
        break;
      default:
        break;
      }
    if(!error) error=module->tryToCallFromNode(this);
    if(!error) _type=module->getResultType();
    }
  else
    {
    Strings message="Module '"+
                    name+
                    "' is unknown";
    error=new Error(Error::SEMANTICS,message);
    }
  if(error)
    {
    _fail=true;
    manageError(error);
    }
  else
    {
    if(_global) _secondName="::";
    }
  }
return(!_fail);
}

CodeVect * ModuleNode::generateCode(void) // virtual
{
CodeVect * codeVect=new CodeVect;
for(int i=getNbNodes();i--;)
  {
  CodeVect * tmp=getNode(i)->generateCode();
  for(unsigned int j=0;j<tmp->getSize();j++)
    codeVect->pushBack((*tmp)[j]);
  delete tmp;
  }
if(_constructor)
  codeVect->pushBack(new ConstrModuleCode(this,_model));
else if(_destructor)
  codeVect->pushBack(new DestrModuleCode(this,_model));
else if(_virtual)
  codeVect->pushBack(new AgentModuleCode(this,Word::create(getName())));
else if(_global)
  codeVect->pushBack(new GlobalModuleCode(this,Word::create(getName())));
else
  codeVect->pushBack(new ModelModuleCode(this,Word::create(getName()),_model));
return codeVect;
}

