// @(#) ENIB/LI2 (c) - oRis - Version 2.3
//--------------------------------------------------------------------------//
// file        : newNd.cpp                                                  //
// description : source file for the ENIB/LI2 class NewNode                 //
// author(s)   : F. Harrouet                                                //
// copyright   : Laboratoire d'Informatique Industrielle (LI2)              //
//               Ecole Nationale d'Ingenieurs de Brest (ENIB)               //
//               CP 30815 - 29608 Brest Cedex - FRANCE                      //
//               Tel: (33) 02.98.05.66.31 , Fax: (33) 02.98.05.66.29        //
//               e-mail: li2@enib.fr                                        //
//--------------------------------------------------------------------------//
// Copyright (c) 1996-2000                                                  //
// Laboratoire d'Informatique Industrielle (LI2)                            //
// Ecole Nationale d'Ingenieurs de Brest (ENIB)                             //
//                                                                          //
// Permission to use, copy, modify, distribute and sell this software       //
// and its documentation for any purpose is hereby granted without fee,     //
// provided that the above copyright notice appear in all copies and        //
// that both that copyright notice and this permission notice appear        //
// in supporting documentation.                                             //
// ENIB/LI2 makes no representations about the suitability of this          //
// software for any purpose.                                                //
// It is provided "as is" without express or implied warranty.              //
//--------------------------------------------------------------------------//

#include "oRis/Nodes/newNd.h"
#include "oRis/Main/errors.h"
#include "oRis/Main/model.h"
#include "oRis/Codes/moduleCd.h"
#include "oRis/Codes/utilCd.h"

//---------------------------------------------------------------------------//
// class NewNode                                                             //
//---------------------------------------------------------------------------//

//-------- Allocators -------------------------------------------------------//
NewNode::NewNode(int lineNumber,const Strings & fileName,
                 const Strings & name,Model * model,
                 bool isTransient,bool suffixGiven)
 : SmaNode(0,lineNumber,fileName,name,true),
   _model(model), _isTransient(isTransient), _suffixGiven(suffixGiven)
{}

NewNode::NewNode(const NewNode & node)
 : SmaNode(node), _model(node._model),
   _isTransient(node._isTransient), _suffixGiven(node._suffixGiven)
{}

SmaNode * NewNode::duplicate(void) const // virtual
{
return(new NewNode(*this));
}

NewNode::~NewNode(void)
{}

//-------- Input/Output -----------------------------------------------------//
Strings NewNode::_print(int) const // virtual
{
Strings str;
str+=getName();
if(_suffixGiven)
  {
  str+="("+getNode(0)->print()+") "+_model->getName().getStrings();
  if(getNbNodes()>1)
    {
    str+="(";
    for(int i=1;i<getNbNodes();i++)
      {
      if(i>1) str+=",";
      str+=getNode(i)->print();
      }
    str+=")";
    }
  }
else
  {
  str+=" "+_model->getName().getStrings();
  if(getNbNodes())
    {
    str+="(";
    for(int i=0;i<getNbNodes();i++)
      {
      if(i) str+=",";
      str+=getNode(i)->print();
      }
    str+=")";
    }
  }
return(str);
}

//-------- Evaluation -------------------------------------------------------//
bool NewNode::check(const Context * context,const Model * model) // virtual
{
SmaNode::check(context,model);
if(!_fail)
  {
  if(_suffixGiven&&
     !getNode(0)->getType().canCastTo(ValueType(ValueType::INTEGER)))
    {
    Strings message="Suffix given in '"+
                    getName()+
                    "' should be an 'int'";
    manageError(new Error(Error::SEMANTICS,message));
    _fail=true;
    }
  if(!_model->getNewModule())
    {
    Strings message="Class '"+
                    _model->getName().getStrings()+
                    "' should have a constructor";
    manageError(new Error(Error::SEMANTICS,message));
    _fail=true;
    }
  else
    {
    Error * error=_model->getNewModule()
                        ->tryToCallFromNode(this,_suffixGiven ? 1 : 0);
    if(error)
      {
      manageError(error);
      _fail=true;
      }
    }
  if(!_model->getDeleteModule())
    {
    Strings message="Class '"+
                    _model->getName().getStrings()+
                    "' should have a destructor";
    manageError(new Error(Error::SEMANTICS,message));
    _fail=true;
    }
  if(!_fail) _type=ValueType(_model);
  }
return(!_fail);
}

CodeVect * NewNode::generateCode(void) // virtual
{
CodeVect * codeVect=new CodeVect;
codeVect->pushBack(new PushCode(this,_type)); // result
for(int i=getNbNodes();i--;)
  {
  CodeVect * tmp=getNode(i)->generateCode();
  for(unsigned int j=0;j<tmp->getSize();j++)
    codeVect->pushBack((*tmp)[j]);
  delete tmp;
  }
codeVect->pushBack(new NewCode(this,_model,_isTransient,_suffixGiven));
return codeVect;
}

