// @(#) ENIB/LI2 (c) - oRis - Version 2.3
//--------------------------------------------------------------------------//
// file        : noAtCBNd.cpp                                               //
// description : source file for the ENIB/LI2 class NoAttrCBNode            //
// author(s)   : F. Harrouet                                                //
// copyright   : Laboratoire d'Informatique Industrielle (LI2)              //
//               Ecole Nationale d'Ingenieurs de Brest (ENIB)               //
//               CP 30815 - 29608 Brest Cedex - FRANCE                      //
//               Tel: (33) 02.98.05.66.31 , Fax: (33) 02.98.05.66.29        //
//               e-mail: li2@enib.fr                                        //
//--------------------------------------------------------------------------//
// Copyright (c) 1996-2000                                                  //
// Laboratoire d'Informatique Industrielle (LI2)                            //
// Ecole Nationale d'Ingenieurs de Brest (ENIB)                             //
//                                                                          //
// Permission to use, copy, modify, distribute and sell this software       //
// and its documentation for any purpose is hereby granted without fee,     //
// provided that the above copyright notice appear in all copies and        //
// that both that copyright notice and this permission notice appear        //
// in supporting documentation.                                             //
// ENIB/LI2 makes no representations about the suitability of this          //
// software for any purpose.                                                //
// It is provided "as is" without express or implied warranty.              //
//--------------------------------------------------------------------------//

#include "oRis/Nodes/noAtCBNd.h"
#include "oRis/Codes/utilCd.h"

//---------------------------------------------------------------------------//
// class NoAttrCBNode                                                        //
//---------------------------------------------------------------------------//

//-------- Allocators -------------------------------------------------------//
NoAttrCBNode::NoAttrCBNode(int lineNumber,const Strings& fileName,
                           const Strings & name)
 : SmaNode(1,lineNumber,fileName,name,true)
{}

NoAttrCBNode::NoAttrCBNode(const NoAttrCBNode & node)
 : SmaNode(node)
{}

SmaNode * NoAttrCBNode::duplicate(void) const // virtual
{
return(new NoAttrCBNode(*this));
}

NoAttrCBNode::~NoAttrCBNode(void)
{}

//-------- Inspectors -------------------------------------------------------//
bool NoAttrCBNode::containsReturn(void) const // virtual
{
return(getNode(0)->containsReturn());
}

bool NoAttrCBNode::buildsParent(const Model * parent) const // virtual
{
return(getNode(0)->buildsParent(parent));
}

bool NoAttrCBNode::destroysParent(const Model * parent) const // virtual
{
return(getNode(0)->destroysParent(parent));
}

//-------- Input/Output -----------------------------------------------------//
Strings NoAttrCBNode::print(int column) const // virtual
{
Strings str;
int j;
for(j=0;j<column;j++) str+=" ";
str+=getName()+"\n";
column+=2;
for(j=0;j<column;j++) str+=" ";
str+="{\n";
str+=getNode(0)->print(column);
for(j=0;j<column;j++) str+=" ";
str+="}\n";
return(str);
}

//-------- Evaluation -------------------------------------------------------//
bool NoAttrCBNode::check(const Context * context,const Model * model)
{                                                                // virtual
SmaNode::check(context,model);
return(!_fail);
}

CodeVect * NoAttrCBNode::generateVoidCode(void) // virtual
{
CodeVect * codeVect=new CodeVect;
codeVect->pushBack(new BeginCriticalCode(this));
codeVect->pushBack(new BeginNoCBCode(this));
CodeVect * tmp=getNode(0)->generateVoidCode();
for(unsigned int j=0;j<tmp->getSize();j++)
  codeVect->pushBack((*tmp)[j]);
delete tmp;
codeVect->pushBack(new EndNoCBCode(this));
codeVect->pushBack(new EndCriticalCode(this));
return(codeVect);
}

