// @(#) ENIB/LI2 (c) - oRis - Version 2.3
//--------------------------------------------------------------------------//
// file        : PlusNd.C                                                   //
// description : source file for the ENIB/LI2 class PlusNode                //
// author(s)   : F. Harrouet                                                //
// copyright   : Laboratoire d'Informatique Industrielle (LI2)              //
//               Ecole Nationale d'Ingenieurs de Brest (ENIB)               //
//               CP 30815 - 29608 Brest Cedex - FRANCE                      //
//               Tel: (33) 02.98.05.66.31 , Fax: (33) 02.98.05.66.29        //
//               e-mail: li2@enib.fr                                        //
//--------------------------------------------------------------------------//
// Copyright (c) 1996-2000                                                  //
// Laboratoire d'Informatique Industrielle (LI2)                            //
// Ecole Nationale d'Ingenieurs de Brest (ENIB)                             //
//                                                                          //
// Permission to use, copy, modify, distribute and sell this software       //
// and its documentation for any purpose is hereby granted without fee,     //
// provided that the above copyright notice appear in all copies and        //
// that both that copyright notice and this permission notice appear        //
// in supporting documentation.                                             //
// ENIB/LI2 makes no representations about the suitability of this          //
// software for any purpose.                                                //
// It is provided "as is" without express or implied warranty.              //
//--------------------------------------------------------------------------//

#include "oRis/Nodes/plusNd.h"
#include "oRis/Main/errors.h"
#include "oRis/Codes/calculCd.h"

//---------------------------------------------------------------------------//
// class PlusNode                                                            //
//---------------------------------------------------------------------------//

//-------- Allocators -------------------------------------------------------//
PlusNode::PlusNode(int lineNumber,const Strings & fileName,
                   const Strings & name)
 : SmaNode(2,lineNumber,fileName,name,false)
{}

PlusNode::PlusNode(const PlusNode & node)
 : SmaNode(node)
{}

SmaNode * PlusNode::duplicate(void) const // virtual
{
return(new PlusNode(*this));
}

PlusNode::~PlusNode(void)
{}

//-------- Input/Output -----------------------------------------------------//
Strings PlusNode::_print(int) const // virtual
{
return(getNode(0)->print()+getName()+getNode(1)->print());
}

//-------- Evaluation -------------------------------------------------------//
bool PlusNode::check(const Context * context,const Model * model) // virtual
{
SmaNode::check(context,model);
if(!_fail)
  {
  if(getNode(0)->getType().getDepth()==getNode(1)->getType().getDepth())
    {
    if(getNode(0)->getType().getDepth())
      {
      if(getNode(0)->getType().canCastFrom(getNode(1)->getType()))
        {
        _type=getNode(0)->getType();
        }
      else
        {
        _fail=true;
        }
      }
    else
      {
      if((getNode(0)->getType().canCastFrom(ValueType(ValueType::REAL))&&
          getNode(1)->getType().canCastFrom(ValueType(ValueType::REAL)))||
         (getNode(0)->getType().canCastFrom(ValueType(ValueType::STRING))&&
          getNode(1)->getType().canCastFrom(ValueType(ValueType::STRING))))
        {
        if((getNode(0)->getType().getSimpleType()==ValueType::REAL)||
           (getNode(1)->getType().getSimpleType()==ValueType::REAL))
          _type=ValueType(ValueType::REAL);
        else _type=getNode(0)->getType();
        }
      else
        {
        _fail=true;
        }
      }
    }
  else
    {
    _fail=true;
    }
  if(_fail)
    {
    Strings message="Can't use '"+getName()+
                    "' between '"+
                    getNode(0)->getType().print()+
                    "' and '"+
                    getNode(1)->getType().print()+
                    "'";
    manageError(new Error(Error::SEMANTICS,message));
    }
  }
return(!_fail);
}

CodeVect * PlusNode::generateCode(void) // virtual
{
CodeVect * codeVect=getNode(1)->generateCode();
CodeVect * tmp=getNode(0)->generateCode();
for(unsigned int j=0;j<tmp->getSize();j++)
  codeVect->pushBack((*tmp)[j]);
delete tmp;
if(getNode(0)->getType().getDepth())
  {
  if(getNode(0)->getType().getSimpleType()==
     getNode(1)->getType().getSimpleType())
    codeVect->pushBack(new Plus_V_V_Code(this));
  else
    {
    if(getNode(0)->getType().getSimpleType()==ValueType::INTEGER)
      codeVect->pushBack(new Plus_IV_RV_Code(this));
    else // REAL
      codeVect->pushBack(new Plus_RV_IV_Code(this));
    }
  }
else
  {
  switch(getNode(0)->getType().getSimpleType())
    {
    case ValueType::INTEGER:
      switch(getNode(1)->getType().getSimpleType())
        {
        case ValueType::INTEGER:
          codeVect->pushBack(new Plus_I_I_Code(this));
          break;
        case ValueType::REAL:
          codeVect->pushBack(new Plus_I_R_Code(this));
          break;
        default:
          break;
        }
      break;
    case ValueType::REAL:
      switch(getNode(1)->getType().getSimpleType())
        {
        case ValueType::INTEGER:
          codeVect->pushBack(new Plus_R_I_Code(this));
          break;
        case ValueType::REAL:
          codeVect->pushBack(new Plus_R_R_Code(this));
          break;
        default:
          break;
        }
      break;
    case ValueType::STRING:
      codeVect->pushBack(new Plus_S_S_Code(this));
      break;
    default:
      break;
    }
  }
return(codeVect);
}

