// @(#) ENIB/LI2 (c) - oRis - Version 2.3
//--------------------------------------------------------------------------//
// file        : popBaNd.cpp                                                //
// description : source file for the ENIB/LI2 class PopBackNode             //
// author(s)   : F. Harrouet                                                //
// copyright   : Laboratoire d'Informatique Industrielle (LI2)              //
//               Ecole Nationale d'Ingenieurs de Brest (ENIB)               //
//               CP 30815 - 29608 Brest Cedex - FRANCE                      //
//               Tel: (33) 02.98.05.66.31 , Fax: (33) 02.98.05.66.29        //
//               e-mail: li2@enib.fr                                        //
//--------------------------------------------------------------------------//
// Copyright (c) 1996-2000                                                  //
// Laboratoire d'Informatique Industrielle (LI2)                            //
// Ecole Nationale d'Ingenieurs de Brest (ENIB)                             //
//                                                                          //
// Permission to use, copy, modify, distribute and sell this software       //
// and its documentation for any purpose is hereby granted without fee,     //
// provided that the above copyright notice appear in all copies and        //
// that both that copyright notice and this permission notice appear        //
// in supporting documentation.                                             //
// ENIB/LI2 makes no representations about the suitability of this          //
// software for any purpose.                                                //
// It is provided "as is" without express or implied warranty.              //
//--------------------------------------------------------------------------//

#include "oRis/Nodes/popBaNd.h"
#include "oRis/Main/errors.h"
#include "oRis/Codes/listCd.h"

//---------------------------------------------------------------------------//
// class PopBackNode                                                         //
//---------------------------------------------------------------------------//

//-------- Allocators -------------------------------------------------------//
PopBackNode::PopBackNode(int lineNumber,const Strings & fileName,
                         const Strings & name)
 : SmaNode(1,lineNumber,fileName,name,true)
{}

PopBackNode::PopBackNode(const PopBackNode & node)
 : SmaNode(node)
{}

SmaNode * PopBackNode::duplicate(void) const // virtual
{
return(new PopBackNode(*this));
}

PopBackNode::~PopBackNode(void)
{}

//-------- Input/Output -----------------------------------------------------//
Strings PopBackNode::_print(int) const // virtual
{
return(getNode(0)->print()+"."+getName()+"()");
}

//-------- Evaluation -------------------------------------------------------//
bool PopBackNode::check(const Context * context,const Model * model)
{                                                                  // virtual
SmaNode::check(context,model);
if(!_fail)
  {
  if(!getNode(0)->isLeftValue())
    {
    _fail=true;
    Strings message="Expression before '."+
                    getName()+
                    "()' should be modifiable";
    manageError(new Error(Error::SEMANTICS,message));
    }
  if(!getNode(0)->getType().getDepth())
    {
    _fail=true;
    Strings message="Expression before '."+
                    getName()+
                    "()' should be a list";
    manageError(new Error(Error::SEMANTICS,message));
    }
  if(!_fail)
    {
    _type=getNode(0)->getType();
    _type.setDepth(_type.getDepth()-1);
    }
  }
return(!_fail);
}

CodeVect * PopBackNode::generateCode(void) // virtual
{
CodeVect * codeVect=getNode(0)->generateCode();
codeVect->pushBack(new PopBackCode(this));
return(codeVect);
}

CodeVect * PopBackNode::generateVoidCode(void) // virtual
{
CodeVect * codeVect=getNode(0)->generateCode();
codeVect->pushBack(new PopBackVoidCode(this));
return(codeVect);
}

