// @(#) ENIB/LI2 (c) - oRis - Version 2.3
//--------------------------------------------------------------------------//
// file        : printNd.cpp                                                //
// description : source file for the ENIB/LI2 class PrintNode               //
// author(s)   : F. Harrouet                                                //
// copyright   : Laboratoire d'Informatique Industrielle (LI2)              //
//               Ecole Nationale d'Ingenieurs de Brest (ENIB)               //
//               CP 30815 - 29608 Brest Cedex - FRANCE                      //
//               Tel: (33) 02.98.05.66.31 , Fax: (33) 02.98.05.66.29        //
//               e-mail: li2@enib.fr                                        //
//--------------------------------------------------------------------------//
// Copyright (c) 1996-2000                                                  //
// Laboratoire d'Informatique Industrielle (LI2)                            //
// Ecole Nationale d'Ingenieurs de Brest (ENIB)                             //
//                                                                          //
// Permission to use, copy, modify, distribute and sell this software       //
// and its documentation for any purpose is hereby granted without fee,     //
// provided that the above copyright notice appear in all copies and        //
// that both that copyright notice and this permission notice appear        //
// in supporting documentation.                                             //
// ENIB/LI2 makes no representations about the suitability of this          //
// software for any purpose.                                                //
// It is provided "as is" without express or implied warranty.              //
//--------------------------------------------------------------------------//

#include "oRis/Nodes/printNd.h"
#include "oRis/Main/errors.h"
#include "oRis/Codes/utilCd.h"

//---------------------------------------------------------------------------//
// class PrintNode                                                           //
//---------------------------------------------------------------------------//

//-------- Allocators -------------------------------------------------------//
PrintNode::PrintNode(int lineNumber,const Strings & fileName,
                     const Strings & name,bool newLine)
 : SmaNode(0,lineNumber,fileName,name,true), _newLine(newLine)
{}

PrintNode::PrintNode(const PrintNode & node)
 : SmaNode(node), _newLine(node._newLine)
{}

SmaNode * PrintNode::duplicate(void) const // virtual
{
return(new PrintNode(*this));
}

PrintNode::~PrintNode(void)
{}

//-------- Input/Output -----------------------------------------------------//

//-------- Evaluation -------------------------------------------------------//
bool PrintNode::check(const Context * context,const Model * model) // virtual
{
SmaNode::check(context,model);
if(!_fail)
  {
  for(int i=0;i<getNbNodes();i++)
    {
    if(getNode(i)->getType().getSimpleType()==ValueType::UNKNOWN)
      _fail=true;
    }
  if(_fail)
    {
    Strings message="'"+getName()+
                    "(...)' should not have 'void' arguments";
    manageError(new Error(Error::SEMANTICS,message));
    }
  }
return(!_fail);
}

CodeVect * PrintNode::generateVoidCode(void) // virtual
{
CodeVect * codeVect=new CodeVect;
for(int i=getNbNodes();i--;)
  {
  CodeVect * tmp=getNode(i)->generateCode();
  for(unsigned int j=0;j<tmp->getSize();j++)
    codeVect->pushBack((*tmp)[j]);
  delete tmp;
  }
codeVect->pushBack(new PrintCode(this,getNbNodes(),_newLine));
return codeVect;
}

