// @(#) ENIB/LI2 (c) - oRis - Version 2.3
//--------------------------------------------------------------------------//
// file        : pushBaNd.cpp                                               //
// description : source file for the ENIB/LI2 class PushBackNode            //
// author(s)   : F. Harrouet                                                //
// copyright   : Laboratoire d'Informatique Industrielle (LI2)              //
//               Ecole Nationale d'Ingenieurs de Brest (ENIB)               //
//               CP 30815 - 29608 Brest Cedex - FRANCE                      //
//               Tel: (33) 02.98.05.66.31 , Fax: (33) 02.98.05.66.29        //
//               e-mail: li2@enib.fr                                        //
//--------------------------------------------------------------------------//
// Copyright (c) 1996-2000                                                  //
// Laboratoire d'Informatique Industrielle (LI2)                            //
// Ecole Nationale d'Ingenieurs de Brest (ENIB)                             //
//                                                                          //
// Permission to use, copy, modify, distribute and sell this software       //
// and its documentation for any purpose is hereby granted without fee,     //
// provided that the above copyright notice appear in all copies and        //
// that both that copyright notice and this permission notice appear        //
// in supporting documentation.                                             //
// ENIB/LI2 makes no representations about the suitability of this          //
// software for any purpose.                                                //
// It is provided "as is" without express or implied warranty.              //
//--------------------------------------------------------------------------//

#include "oRis/Nodes/pushBaNd.h"
#include "oRis/Main/errors.h"
#include "oRis/Codes/listCd.h"

//---------------------------------------------------------------------------//
// class PushBackNode                                                        //
//---------------------------------------------------------------------------//

//-------- Allocators -------------------------------------------------------//
PushBackNode::PushBackNode(int lineNumber,const Strings & fileName,
                           const Strings & name)
 : SmaNode(2,lineNumber,fileName,name,true), _listElemType()
{}

PushBackNode::PushBackNode(const PushBackNode & node)
 : SmaNode(node), _listElemType(node._listElemType)
{}

SmaNode * PushBackNode::duplicate(void) const // virtual
{
return(new PushBackNode(*this));
}

PushBackNode::~PushBackNode(void)
{}

//-------- Input/Output -----------------------------------------------------//
Strings PushBackNode::_print(int) const // virtual
{
return(getNode(0)->print()+"."+getName()+"("+getNode(1)->print()+")");
}

//-------- Evaluation -------------------------------------------------------//
bool PushBackNode::check(const Context * context,const Model * model)
{                                                                // virtual
SmaNode::check(context,model);
if(!_fail)
  {
  _listElemType=getNode(0)->getType();
  _listElemType.setDepth(_listElemType.getDepth()-1);
  if(!getNode(0)->isLeftValue())
    {
    _fail=true;
    Strings message="Expression before '."+
                    getName()+
                    "(...)' should be modifiable";
    manageError(new Error(Error::SEMANTICS,message));
    }
  if(!getNode(1)->getType().canCastTo(_listElemType))
    {
    _fail=true;
    Strings message="Expression before '."+
                    getName()+
                    "(...)' should be a list of '"+
                    getNode(1)->getType().print()+
                    "'";
    manageError(new Error(Error::SEMANTICS,message));
    }
  }
return(!_fail);
}

CodeVect * PushBackNode::generateVoidCode(void) // virtual
{
CodeVect * codeVect=getNode(1)->generateCode();
CodeVect * tmp=getNode(0)->generateCode();
unsigned int j;
for(j=0;j<tmp->getSize();j++)
  codeVect->pushBack((*tmp)[j]);
delete tmp;
if(getNode(1)->getType()==_listElemType)
  codeVect->pushBack(new PushBack_same_Code(this));
else
  codeVect->pushBack(new PushBack_diff_Code(this,_listElemType));
return(codeVect);
}

