// @(#) ENIB/LI2 (c) - oRis - Version 2.3
//--------------------------------------------------------------------------//
// file        : rShiftNd.cpp                                               //
// description : source file for the ENIB/LI2 class RShiftNode              //
// author(s)   : F. Harrouet                                                //
// copyright   : Laboratoire d'Informatique Industrielle (LI2)              //
//               Ecole Nationale d'Ingenieurs de Brest (ENIB)               //
//               CP 30815 - 29608 Brest Cedex - FRANCE                      //
//               Tel: (33) 02.98.05.66.31 , Fax: (33) 02.98.05.66.29        //
//               e-mail: li2@enib.fr                                        //
//--------------------------------------------------------------------------//
// Copyright (c) 1996-2000                                                  //
// Laboratoire d'Informatique Industrielle (LI2)                            //
// Ecole Nationale d'Ingenieurs de Brest (ENIB)                             //
//                                                                          //
// Permission to use, copy, modify, distribute and sell this software       //
// and its documentation for any purpose is hereby granted without fee,     //
// provided that the above copyright notice appear in all copies and        //
// that both that copyright notice and this permission notice appear        //
// in supporting documentation.                                             //
// ENIB/LI2 makes no representations about the suitability of this          //
// software for any purpose.                                                //
// It is provided "as is" without express or implied warranty.              //
//--------------------------------------------------------------------------//

#include "oRis/Nodes/rShiftNd.h"
#include "oRis/Main/errors.h"
#include "oRis/Codes/calculCd.h"

//---------------------------------------------------------------------------//
// class RShiftNode                                                          //
//---------------------------------------------------------------------------//

//-------- Allocators -------------------------------------------------------//
RShiftNode::RShiftNode(int lineNumber,const Strings & fileName,
                       const Strings & name)
 : SmaNode(2,lineNumber,fileName,name,false)
{}

RShiftNode::RShiftNode(const RShiftNode & node)
 : SmaNode(node)
{}

SmaNode * RShiftNode::duplicate(void) const // virtual
{
return(new RShiftNode(*this));
}

RShiftNode::~RShiftNode(void)
{}

//-------- Input/Output -----------------------------------------------------//
Strings RShiftNode::_print(int) const // virtual
{
return(getNode(0)->print()+getName()+getNode(1)->print());
}

//-------- Evaluation -------------------------------------------------------//
bool RShiftNode::check(const Context * context,const Model * model) // virtual
{
SmaNode::check(context,model);
if(!_fail)
  {
  if(getNode(0)->getType()!=ValueType(ValueType::INTEGER))
    {
    _fail=true;
    Strings message="Left argument of '"+
                    getName()+
                    "' should be an 'int'";
    manageError(new Error(Error::SEMANTICS,message));
    }
  if(getNode(1)->getType()!=ValueType(ValueType::INTEGER))
    {
    _fail=true;
    Strings message="Right argument of '"+
                    getName()+
                    "' should be an 'int'";
    manageError(new Error(Error::SEMANTICS,message));
    }
  if(!_fail) _type=getNode(0)->getType();
  }
return(!_fail);
}

CodeVect * RShiftNode::generateCode(void) // virtual
{
CodeVect * codeVect=getNode(1)->generateCode();
CodeVect * tmp=getNode(0)->generateCode();
for(unsigned int j=0;j<tmp->getSize();j++)
  codeVect->pushBack((*tmp)[j]);
delete tmp;
codeVect->pushBack(new RShiftCode(this));
return(codeVect);
}

