// @(#) ENIB/LI2 (c) - oRis - Version 2.3
//--------------------------------------------------------------------------//
// file        : removeNd.cpp                                               //
// description : source file for the ENIB/LI2 class RemoveNode              //
// author(s)   : F. Harrouet                                                //
// copyright   : Laboratoire d'Informatique Industrielle (LI2)              //
//               Ecole Nationale d'Ingenieurs de Brest (ENIB)               //
//               CP 30815 - 29608 Brest Cedex - FRANCE                      //
//               Tel: (33) 02.98.05.66.31 , Fax: (33) 02.98.05.66.29        //
//               e-mail: li2@enib.fr                                        //
//--------------------------------------------------------------------------//
// Copyright (c) 1996-2000                                                  //
// Laboratoire d'Informatique Industrielle (LI2)                            //
// Ecole Nationale d'Ingenieurs de Brest (ENIB)                             //
//                                                                          //
// Permission to use, copy, modify, distribute and sell this software       //
// and its documentation for any purpose is hereby granted without fee,     //
// provided that the above copyright notice appear in all copies and        //
// that both that copyright notice and this permission notice appear        //
// in supporting documentation.                                             //
// ENIB/LI2 makes no representations about the suitability of this          //
// software for any purpose.                                                //
// It is provided "as is" without express or implied warranty.              //
//--------------------------------------------------------------------------//

#include "oRis/Nodes/removeNd.h"
#include "oRis/Main/errors.h"
#include "oRis/Codes/listCd.h"

//---------------------------------------------------------------------------//
// class RemoveNode                                                          //
//---------------------------------------------------------------------------//

//-------- Allocators -------------------------------------------------------//
RemoveNode::RemoveNode(int lineNumber,const Strings & fileName,
                       const Strings & name)
 : SmaNode(2,lineNumber,fileName,name,true)
{}

RemoveNode::RemoveNode(const RemoveNode & node)
 : SmaNode(node)
{}

SmaNode * RemoveNode::duplicate(void) const // virtual
{
return(new RemoveNode(*this));
}

RemoveNode::~RemoveNode(void)
{}

//-------- Input/Output -----------------------------------------------------//
Strings RemoveNode::_print(int) const // virtual
{
return(getNode(0)->print()+"."+getName()+"("+getNode(1)->print()+")");
}

//-------- Evaluation -------------------------------------------------------//
bool RemoveNode::check(const Context * context,const Model * model) // virtual
{
SmaNode::check(context,model);
if(!_fail)
  {
  if(!getNode(0)->isLeftValue())
    {
    _fail=true;
    Strings message="Expression before '."+
                    getName()+
                    "()' should be modifiable";
    manageError(new Error(Error::SEMANTICS,message));
    }
  if(!getNode(0)->getType().getDepth())
    {
    _fail=true;
    Strings message="Expression before '."+
                    getName()+
                    "()' should be a list";
    manageError(new Error(Error::SEMANTICS,message));
    }
  if(!getNode(1)->getType().canCastTo(ValueType(ValueType::INTEGER)))
    {
    _fail=true;
    Strings message="Argument of '"+
                    getName()+
                    "' should be an 'int'";
    manageError(new Error(Error::SEMANTICS,message));
    }
  if(!_fail)
    {
    _type=getNode(0)->getType();
    _type.setDepth(_type.getDepth()-1);
    }
  }
return(!_fail);
}

CodeVect * RemoveNode::generateCode(void) // virtual
{
CodeVect * codeVect=getNode(0)->generateCode();
CodeVect * tmp=getNode(1)->generateCode();
unsigned int j;
for(j=0;j<tmp->getSize();j++)
  codeVect->pushBack((*tmp)[j]);
delete tmp;
if(getNode(1)->getType().getSimpleType()==ValueType::INTEGER)
  codeVect->pushBack(new Remove_I_Code(this));
else // REAL
  codeVect->pushBack(new Remove_R_Code(this));
return(codeVect);
}

CodeVect * RemoveNode::generateVoidCode(void) // virtual
{
CodeVect * codeVect=getNode(0)->generateCode();
CodeVect * tmp=getNode(1)->generateCode();
unsigned int j;
for(j=0;j<tmp->getSize();j++)
  codeVect->pushBack((*tmp)[j]);
delete tmp;
if(getNode(1)->getType().getSimpleType()==ValueType::INTEGER)
  codeVect->pushBack(new RemoveVoid_I_Code(this));
else // REAL
  codeVect->pushBack(new RemoveVoid_R_Code(this));
return(codeVect);
}

