// @(#) ENIB/LI2 (c) - oRis - Version 2.3
//--------------------------------------------------------------------------//
// file        : returnNd.cpp                                               //
// description : source file for the ENIB/LI2 class ReturnNode              //
// author(s)   : F. Harrouet                                                //
// copyright   : Laboratoire d'Informatique Industrielle (LI2)              //
//               Ecole Nationale d'Ingenieurs de Brest (ENIB)               //
//               CP 30815 - 29608 Brest Cedex - FRANCE                      //
//               Tel: (33) 02.98.05.66.31 , Fax: (33) 02.98.05.66.29        //
//               e-mail: li2@enib.fr                                        //
//--------------------------------------------------------------------------//
// Copyright (c) 1996-2000                                                  //
// Laboratoire d'Informatique Industrielle (LI2)                            //
// Ecole Nationale d'Ingenieurs de Brest (ENIB)                             //
//                                                                          //
// Permission to use, copy, modify, distribute and sell this software       //
// and its documentation for any purpose is hereby granted without fee,     //
// provided that the above copyright notice appear in all copies and        //
// that both that copyright notice and this permission notice appear        //
// in supporting documentation.                                             //
// ENIB/LI2 makes no representations about the suitability of this          //
// software for any purpose.                                                //
// It is provided "as is" without express or implied warranty.              //
//--------------------------------------------------------------------------//

#include "oRis/Nodes/returnNd.h"
#include "oRis/Main/errors.h"
#include "oRis/Main/context.h"
#include "oRis/Main/execMod.h"
#include "oRis/Codes/utilCd.h"

//---------------------------------------------------------------------------//
// class ReturnNode                                                          //
//---------------------------------------------------------------------------//

//-------- Allocators -------------------------------------------------------//
ReturnNode::ReturnNode(int lineNumber,const Strings& fileName,
                       const Strings & name)
 : SmaNode(0,lineNumber,fileName,name,true), _resultType()
{}

ReturnNode::ReturnNode(const ReturnNode & node)
 : SmaNode(node), _resultType(node._resultType)
{}

SmaNode * ReturnNode::duplicate(void) const // virtual
{
return(new ReturnNode(*this));
}

ReturnNode::~ReturnNode(void)
{}

//-------- Inspectors -------------------------------------------------------//
bool ReturnNode::containsReturn(void) const // virtual
{
return(true);
}

//-------- Input/Output -----------------------------------------------------//
Strings ReturnNode::_print(int) const // virtual
{
Strings str=getName();
if(getNbNodes())
  {
  if(!getNode(0)->usesParenthesis()) str+=" ";
  str+=getNode(0)->print();
  }
return(str);
}

//-------- Evaluation -------------------------------------------------------//
bool ReturnNode::check(const Context * context,const Model * model) // virtual
{
SmaNode::check(context,model);
if(!_fail)
  {
  _resultType=context->getModule()->getResultType();
  if(getNbNodes()&&(_resultType==ValueType()))
    {
    _fail=true;
    Strings message="Module '"+
                    context->getModule()->getCompleteName()+
                    "' should not return a value";
    manageError(new Error(Error::SEMANTICS,message));
    }
  else if((getNbNodes()&&!getNode(0)->getType().canCastTo(_resultType))||
     (!getNbNodes()&&(_resultType!=ValueType())))
    {
    _fail=true;
    Strings message="Module '"+
                    context->getModule()->getCompleteName()+
                    "' should return a '"+
                    _resultType.print()+
                    "'";
    manageError(new Error(Error::SEMANTICS,message));
    }
  }
return(!_fail);
}

CodeVect * ReturnNode::generateVoidCode(void) // virtual
{
CodeVect * codeVect;
if(getNbNodes())
  {
  codeVect=getNode(0)->generateCode();
  if(getNode(0)->getType()==_resultType)
    {
    if(getNode(0)->isLocalValue()) codeVect->pushBack(new CopyCode(this));
    }
  else codeVect->pushBack(new CastCode(this,_resultType));
  }
else codeVect=new CodeVect;
codeVect->pushBack(new ReturnCode(this,_noCBLevel,_criticalLevel));
return(codeVect);
}

