// @(#) ENIB/LI2 (c) - oRis - Version 2.3
//--------------------------------------------------------------------------//
// file        : seekNd.cpp                                                 //
// description : source file for the ENIB/LI2 class SeekNode                //
// author(s)   : F. Harrouet                                                //
// copyright   : Laboratoire d'Informatique Industrielle (LI2)              //
//               Ecole Nationale d'Ingenieurs de Brest (ENIB)               //
//               CP 30815 - 29608 Brest Cedex - FRANCE                      //
//               Tel: (33) 02.98.05.66.31 , Fax: (33) 02.98.05.66.29        //
//               e-mail: li2@enib.fr                                        //
//--------------------------------------------------------------------------//
// Copyright (c) 1996-2000                                                  //
// Laboratoire d'Informatique Industrielle (LI2)                            //
// Ecole Nationale d'Ingenieurs de Brest (ENIB)                             //
//                                                                          //
// Permission to use, copy, modify, distribute and sell this software       //
// and its documentation for any purpose is hereby granted without fee,     //
// provided that the above copyright notice appear in all copies and        //
// that both that copyright notice and this permission notice appear        //
// in supporting documentation.                                             //
// ENIB/LI2 makes no representations about the suitability of this          //
// software for any purpose.                                                //
// It is provided "as is" without express or implied warranty.              //
//--------------------------------------------------------------------------//

#include "oRis/Nodes/seekNd.h"
#include "oRis/Main/model.h"
#include "oRis/Main/errors.h"
#include "oRis/Codes/listCd.h"

//---------------------------------------------------------------------------//
// class SeekNode                                                            //
//---------------------------------------------------------------------------//

//-------- Allocators -------------------------------------------------------//
SeekNode::SeekNode(int lineNumber,const Strings & fileName,
                   const Strings & name,bool deep)
 : SmaNode(2,lineNumber,fileName,name,false), _listElemType(), _deep(deep)
{}

SeekNode::SeekNode(const SeekNode & node)
 : SmaNode(node), _listElemType(node._listElemType), _deep(node._deep)
{}

SmaNode * SeekNode::duplicate(void) const // virtual
{
return(new SeekNode(*this));
}

SeekNode::~SeekNode(void)
{}

//-------- Input/Output -----------------------------------------------------//
Strings SeekNode::_print(int) const // virtual
{
return(getNode(0)->print()+"."+getName()+"("+getNode(1)->print()+")");
}

//-------- Evaluation -------------------------------------------------------//
bool SeekNode::check(const Context * context,const Model * model) // virtual
{
SmaNode::check(context,model);
if(!_fail)
  {
  if(_deep)
    {
    if((getNode(0)->getType().getSimpleType()==ValueType::AGENT)&&
       (getNode(0)->getType().getDepth()>0)&&
       (getNode(1)->getType().getSimpleType()==ValueType::AGENT)&&
       (getNode(1)->getType().getDepth()==getNode(0)->getType().getDepth()-1))
      {
      _type=ValueType(ValueType::INTEGER);
      }
    else
      {
      _fail=true;
      Strings message="Can't use '."+getName()+
                      "(...)' between '"+
                      getNode(0)->getType().print()+
                      "' and '"+
                      getNode(1)->getType().print()+
                      "'";
      manageError(new Error(Error::SEMANTICS,message));
      }
    }
  else
    {
    if(getNode(0)->getType().canCastFrom(ValueType(ValueType::STRING)))
      {
      if(!getNode(1)->getType().canCastFrom(ValueType(ValueType::STRING)))
        {
        _fail=true;
        Strings message="Argument of '"+
                        getName()+
                        "' should be a 'string'";
        manageError(new Error(Error::SEMANTICS,message));
        }
      else _type=ValueType(ValueType::INTEGER);
      }
    else
      {
      _listElemType=getNode(0)->getType();
      _listElemType.setDepth(_listElemType.getDepth()-1);
      if(!_listElemType.canCastTo(getNode(1)->getType())&&
         ((_listElemType.getDepth()!=getNode(1)->getType().getDepth())||
          (_listElemType.getSimpleType()!=
           getNode(1)->getType().getSimpleType())))
        {
        _fail=true;
        Strings message="Expression before '."+
                        getName()+
                        "(...)' should be a list of '"+
                        getNode(1)->getType().print()+
                        "'";
        manageError(new Error(Error::SEMANTICS,message));
        }
      else _type=ValueType(ValueType::INTEGER);
      }
    }
  }
return(!_fail);
}

CodeVect * SeekNode::generateCode(void) // virtual
{
CodeVect * codeVect=getNode(1)->generateCode();
CodeVect * tmp=getNode(0)->generateCode();
for(unsigned int j=0;j<tmp->getSize();j++)
  codeVect->pushBack((*tmp)[j]);
delete tmp;
if(_deep)
  {
  codeVect->pushBack(new Seek_same_V_Code(this,true));
  }
else if(getNode(0)->getType().canCastFrom(ValueType(ValueType::STRING)))
  codeVect->pushBack(new Seek_S_Code(this));
else
  {
  if(getNode(1)->getType()==_listElemType)
    codeVect->pushBack(new Seek_same_V_Code(this,false));
  else
    codeVect->pushBack(new Seek_diff_V_Code(this,_listElemType));
  }
return(codeVect);
}

