// @(#) ENIB/LI2 (c) - oRis - Version 2.3
//--------------------------------------------------------------------------//
// file        : selfNd.cpp                                                 //
// description : source file for the ENIB/LI2 class SelfNode                //
// author(s)   : F. Harrouet                                                //
// copyright   : Laboratoire d'Informatique Industrielle (LI2)              //
//               Ecole Nationale d'Ingenieurs de Brest (ENIB)               //
//               CP 30815 - 29608 Brest Cedex - FRANCE                      //
//               Tel: (33) 02.98.05.66.31 , Fax: (33) 02.98.05.66.29        //
//               e-mail: li2@enib.fr                                        //
//--------------------------------------------------------------------------//
// Copyright (c) 1996-2000                                                  //
// Laboratoire d'Informatique Industrielle (LI2)                            //
// Ecole Nationale d'Ingenieurs de Brest (ENIB)                             //
//                                                                          //
// Permission to use, copy, modify, distribute and sell this software       //
// and its documentation for any purpose is hereby granted without fee,     //
// provided that the above copyright notice appear in all copies and        //
// that both that copyright notice and this permission notice appear        //
// in supporting documentation.                                             //
// ENIB/LI2 makes no representations about the suitability of this          //
// software for any purpose.                                                //
// It is provided "as is" without express or implied warranty.              //
//--------------------------------------------------------------------------//

#include "oRis/Nodes/selfNd.h"
#include "oRis/Main/errors.h"
#include "oRis/Main/context.h"
#include "oRis/Main/agent.h"
#include "oRis/Codes/valCd.h"

//---------------------------------------------------------------------------//
// class SelfNode                                                            //
//---------------------------------------------------------------------------//

//-------- Allocators -------------------------------------------------------//
SelfNode::SelfNode(int lineNumber,const Strings & fileName,
                   const Strings & name)
 : SmaNode(0,lineNumber,fileName,name,false)
{}

SelfNode::SelfNode(const SelfNode & node)
 : SmaNode(node)
{}

SmaNode * SelfNode::duplicate(void) const // virtual
{
return(new SelfNode(*this));
}

SelfNode::~SelfNode(void)
{}

//-------- Input/Output -----------------------------------------------------//
Strings SelfNode::_print(int) const // virtual
{
return(getName());
}

//-------- Evaluation -------------------------------------------------------//
bool SelfNode::check(const Context * context,const Model * model) // virtual
{
SmaNode::check(context,model);
if(!_fail)
  {
  if(context->getOwner())
    {
    _type=ValueType((Model *)context->getOwner()->getModel());
    }
  else if(model)
    {
    _type=ValueType((Model *)model);
    }
  else
    {
    _fail=true;
    Strings message="'"+getName()+
                    "' has no value in a global module";
    manageError(new Error(Error::SEMANTICS,message));
    }
  }
return(!_fail);
}

CodeVect * SelfNode::generateCode(void) // virtual
{
CodeVect * codeVect=new CodeVect;
codeVect->pushBack(new SelfCode(this));
return(codeVect);
}

