// @(#) ENIB/LI2 (c) - oRis - Version 2.3
//--------------------------------------------------------------------------//
// file        : seqNd.cpp                                                  //
// description : source file for the ENIB/LI2 class SeqNode                 //
// author(s)   : F. Harrouet                                                //
// copyright   : Laboratoire d'Informatique Industrielle (LI2)              //
//               Ecole Nationale d'Ingenieurs de Brest (ENIB)               //
//               CP 30815 - 29608 Brest Cedex - FRANCE                      //
//               Tel: (33) 02.98.05.66.31 , Fax: (33) 02.98.05.66.29        //
//               e-mail: li2@enib.fr                                        //
//--------------------------------------------------------------------------//
// Copyright (c) 1996-2000                                                  //
// Laboratoire d'Informatique Industrielle (LI2)                            //
// Ecole Nationale d'Ingenieurs de Brest (ENIB)                             //
//                                                                          //
// Permission to use, copy, modify, distribute and sell this software       //
// and its documentation for any purpose is hereby granted without fee,     //
// provided that the above copyright notice appear in all copies and        //
// that both that copyright notice and this permission notice appear        //
// in supporting documentation.                                             //
// ENIB/LI2 makes no representations about the suitability of this          //
// software for any purpose.                                                //
// It is provided "as is" without express or implied warranty.              //
//--------------------------------------------------------------------------//

#include "oRis/Nodes/seqNd.h"

//---------------------------------------------------------------------------//
// class SeqNode                                                             //
//---------------------------------------------------------------------------//

//-------- Allocators -------------------------------------------------------//
SeqNode::SeqNode(int lineNumber,const Strings & fileName,const Strings & name)
 : SmaNode(0,lineNumber,fileName,name,true)
{}

SeqNode::SeqNode(const SeqNode & node)
 : SmaNode(node)
{}

SmaNode * SeqNode::duplicate(void) const // virtual
{
return(new SeqNode(*this));
}

SeqNode::~SeqNode(void)
{}

//-------- Inspectors -------------------------------------------------------//
bool SeqNode::containsReturn(void) const // virtual
{
for(int i=getNbNodes();i--;)
  if(getNode(i)->containsReturn()) return(true);
return(false);
}

bool SeqNode::buildsParent(const Model * parent) const // virtual
{
for(int i=getNbNodes();i--;)
  if(getNode(i)->buildsParent(parent)) return(true);
return(false);
}

bool SeqNode::destroysParent(const Model * parent) const // virtual
{
for(int i=getNbNodes();i--;)
  if(getNode(i)->destroysParent(parent)) return(true);
return(false);
}

//-------- Input/Output -----------------------------------------------------//
Strings SeqNode::print(int column) const // virtual
{
Strings str;
for(int i=0;i<getNbNodes();i++)
  {
  if(i&&!isInstruction()) str+=getName();
  str+=getNode(i)->print(column);
  }
return(str);
}

//-------- Modifiers --------------------------------------------------------//
void SeqNode::setInstruction(bool isInstruction) // virtual
{
SmaNode::setInstruction(isInstruction);
for(int i=getNbNodes();i--;)
  if(getNode(i)) getNode(i)->setInstruction(isInstruction);
}

//-------- Evaluation -------------------------------------------------------//
bool SeqNode::check(const Context * context,const Model * model) // virtual
{
SmaNode::check(context,model);
return(!_fail);
}

CodeVect * SeqNode::generateVoidCode(void) // virtual
{
CodeVect * codeVect=new CodeVect;
for(int i=0;i<getNbNodes();i++)
  {
  CodeVect * tmp=getNode(i)->generateVoidCode();
  for(unsigned int j=0;j<tmp->getSize();j++)
    {
    codeVect->pushBack((*tmp)[j]);
    }
  delete tmp;
  }
return(codeVect);
}

