// @(#) ENIB/LI2 (c) - oRis - Version 2.3
//--------------------------------------------------------------------------//
// file        : smaNd.cpp                                                  //
// description : source file for the ENIB/LI2 class SmaNode                 //
// author(s)   : F. Harrouet                                                //
// copyright   : Laboratoire d'Informatique Industrielle (LI2)              //
//               Ecole Nationale d'Ingenieurs de Brest (ENIB)               //
//               CP 30815 - 29608 Brest Cedex - FRANCE                      //
//               Tel: (33) 02.98.05.66.31 , Fax: (33) 02.98.05.66.29        //
//               e-mail: li2@enib.fr                                        //
//--------------------------------------------------------------------------//
// Copyright (c) 1996-2000                                                  //
// Laboratoire d'Informatique Industrielle (LI2)                            //
// Ecole Nationale d'Ingenieurs de Brest (ENIB)                             //
//                                                                          //
// Permission to use, copy, modify, distribute and sell this software       //
// and its documentation for any purpose is hereby granted without fee,     //
// provided that the above copyright notice appear in all copies and        //
// that both that copyright notice and this permission notice appear        //
// in supporting documentation.                                             //
// ENIB/LI2 makes no representations about the suitability of this          //
// software for any purpose.                                                //
// It is provided "as is" without express or implied warranty.              //
//--------------------------------------------------------------------------//

#include "oRis/Nodes/smaNd.h"
#include "oRis/Main/simul.h"
#include "oRis/Main/errors.h"
#include "oRis/Codes/utilCd.h"

//---------------------------------------------------------------------------//
// class SmaNode                                                             //
//---------------------------------------------------------------------------//

//-------- Allocators -------------------------------------------------------//
SmaNode::SmaNode(int nbNodes,int lineNumber,const Strings & fileName,
                 const Strings & name,bool isInstruction)
 : BlockAllocatedObject(),
   _type(), _fail(false), _criticalLevel(0), _noCBLevel(0),
   _nodes(), _lineNumber(lineNumber), _fileName(fileName),
   _name(name), _isInstruction(isInstruction), _usesParenthesis(false),
   _isChecked(false)
{
for(int i=0;i<nbNodes;i++)
  _nodes.pushBack((SmaNode *)0);
}

SmaNode::SmaNode(const SmaNode & node)
 : BlockAllocatedObject(),
   _type(node._type), _fail(node._fail),
   _criticalLevel(node._criticalLevel), _noCBLevel(node._noCBLevel),
   _nodes(), _lineNumber(node._lineNumber), _fileName(node._fileName),
   _name(node._name), _isInstruction(node._isInstruction),
   _usesParenthesis(node._usesParenthesis), _isChecked(node._isChecked)
{
for(unsigned int i=0;i<node._nodes.getSize();i++)
  {
  if(node._nodes[i]) _nodes.pushBack(node._nodes[i]->duplicate());
  else _nodes.pushBack((SmaNode *)0);
  }
}

// virtual SmaNode * SmaNode::duplicate(void) const=0

SmaNode::~SmaNode(void)
{
for(unsigned int i=_nodes.getSize();i--;)
  if(_nodes[i]) delete _nodes[i];
}

//-------- Inspectors -------------------------------------------------------//
// inline int SmaNode::getLineNumber(void) const

// inline const Strings & SmaNode::getFileName(void) const

// inline const Strings & SmaNode::getName(void) const

// inline int SmaNode::getNbNodes(void) const

// inline SmaNode * SmaNode::getNode(int nodeNumber) const

// inline Value::ValueType SmaNode::getType(void) const

// inline bool SmaNode::depth(void) const

// inline bool SmaNode::isInstruction(void) const

// inline bool SmaNode::containsReturn(void) const // virtual

// inline bool SmaNode::buildsParent(const Model * parent) const // virtual

// inline bool SmaNode::destroysParent(const Model * parent) const // virtual

// inline bool SmaNode::isLeftValue(void) const // virtual

// inline bool SmaNode::isLocalValue(void) const // virtual

// inline bool SmaNode::isConst(void) const // virtual

// inline bool SmaNode::usesParenthesis(void) const

// inline bool SmaNode::isChecked(void) const

// inline bool SmaNode::fail(void) const

//-------- Input/Output -----------------------------------------------------//
ostream & operator<<(ostream & stream,const SmaNode & node)
{
return(stream << node.print());
}

Strings SmaNode::print(int column) const // virtual
{
Strings str;
if(_isInstruction) for(int j=0;j<column;j++) str+=" ";
if(_usesParenthesis) str+="(";
str+=_print(column);
if(_usesParenthesis) str+=")";
if(_isInstruction) str+=";\n";
return(str);
}

Strings SmaNode::_print(int) const // virtual
{
Strings str;
str+=_name+"(";
for(unsigned int i=0;i<_nodes.getSize();i++)
  {
  if(i) str+=",";
  str+=_nodes[i]->print();
  }
str+=")";
return(str);
}

//-------- Modifiers --------------------------------------------------------//
// inline void SmaNode::setInstruction(bool isInstruction) // virtual

void SmaNode::setOwner(Agent * owner) // virtual
{
for(int i=getNbNodes();i--;)
  if(getNode(i)) getNode(i)->setOwner(owner);
}

void SmaNode::insideCritical(void)
{
_criticalLevel++;
for(int i=getNbNodes();i--;)
  if(getNode(i)) getNode(i)->insideCritical();
}

void SmaNode::insideNoAttrCB(void)
{
_noCBLevel++;
_criticalLevel++;
for(int i=getNbNodes();i--;)
  if(getNode(i)) getNode(i)->insideNoAttrCB();
}

void SmaNode::addNode(SmaNode * node)
{
_nodes.pushBack(node);
}

void SmaNode::setNode(int nodeNumber,SmaNode * node)
{
if((nodeNumber<0)||
   (nodeNumber>=(int)_nodes.getSize())||
   (_nodes[nodeNumber]&&node))
  {
  Strings message="Can't set sub-node "+
                  Strings(nodeNumber)+
                  " in node '"+
                  getName()+
                  "'";
  manageError(new Error(Error::BUG,message));
  }
else _nodes[nodeNumber]=node;
}

//-------- Evaluation -------------------------------------------------------//
bool SmaNode::check(const Context *,const Model *) // virtual
{
for(unsigned int i=0;i<_nodes.getSize();i++)
  {
  if(!_nodes[i]||!_nodes[i]->_isChecked)
    {
    _fail=true;
    Strings message="Sub-node not checked in node '"+
                    getName()+
                    "'";
    manageError(new Error(Error::BUG,message));
    }
  else _fail=(_nodes[i]->_fail ? true : _fail);
  }
_isChecked=true;
return(!_fail);
}

SmaNode * SmaNode::optimize(void) // virtual
{
return(this);
}

CodeVect * SmaNode::generateCode(void) // virtual
{
Strings message="Can't use 'generateCode()' in node '"+
                getName()+
                "'";
manageError(new Error(Error::BUG,message));
return(new CodeVect);
}

CodeVect * SmaNode::generateVoidCode(void) // virtual
{
CodeVect * codeVect=generateCode();
if(_type!=ValueType(ValueType::UNKNOWN))
  codeVect->pushBack(new PopCode(this));
return(codeVect);
}
  
//-------- Error Management -------------------------------------------------//
void SmaNode::manageError(Error * error) const
{
if(error)
  {
  error->setFileName(_fileName);
  error->setLineNumber(_lineNumber);
  getSimul()->manageError(error);
  }
}

