// @(#) ENIB/LI2 (c) - oRis - Version 2.3
//--------------------------------------------------------------------------//
// file        : startNd.cpp                                                //
// description : source file for the ENIB/LI2 class StartNode               //
// author(s)   : F. Harrouet                                                //
// copyright   : Laboratoire d'Informatique Industrielle (LI2)              //
//               Ecole Nationale d'Ingenieurs de Brest (ENIB)               //
//               CP 30815 - 29608 Brest Cedex - FRANCE                      //
//               Tel: (33) 02.98.05.66.31 , Fax: (33) 02.98.05.66.29        //
//               e-mail: li2@enib.fr                                        //
//--------------------------------------------------------------------------//
// Copyright (c) 1996-2000                                                  //
// Laboratoire d'Informatique Industrielle (LI2)                            //
// Ecole Nationale d'Ingenieurs de Brest (ENIB)                             //
//                                                                          //
// Permission to use, copy, modify, distribute and sell this software       //
// and its documentation for any purpose is hereby granted without fee,     //
// provided that the above copyright notice appear in all copies and        //
// that both that copyright notice and this permission notice appear        //
// in supporting documentation.                                             //
// ENIB/LI2 makes no representations about the suitability of this          //
// software for any purpose.                                                //
// It is provided "as is" without express or implied warranty.              //
//--------------------------------------------------------------------------//

#include "oRis/Nodes/startNd.h"
#include "oRis/Main/execMod.h"
#include "oRis/Main/errors.h"
#include "oRis/Codes/moduleCd.h"
#include "oRis/Codes/varCd.h"
#include "oRis/Codes/affectCd.h"
#include "oRis/Codes/utilCd.h"

//---------------------------------------------------------------------------//
// class StartNode                                                           //
//---------------------------------------------------------------------------//

//-------- Allocators -------------------------------------------------------//
StartNode::StartNode(int lineNumber,const Strings& fileName,
                     const Strings & name,ExecModule * module)
 : SmaNode(0,lineNumber,fileName,name,true), _module(module)
{
if(_module) _module->reference();
}

StartNode::StartNode(const StartNode & node)
 : SmaNode(node), _module(node._module)
{
if(_module)
  {
  _module=_module->duplicate();
  _module->reference();
  }
}

SmaNode * StartNode::duplicate(void) const // virtual
{
return(new StartNode(*this));
}

StartNode::~StartNode(void)
{
if(_module) _module->unReference();
}

//-------- Inspectors -------------------------------------------------------//

//-------- Input/Output -----------------------------------------------------//
Strings StartNode::print(int column) const // virtual
{
Strings str;
int j;
if(isInstruction()) for(j=0;j<column;j++) str+=" ";
str+=getName();
if(getNbNodes()) str+="("+getNode(0)->print()+")\n";
else str+="\n";
column+=2;
for(j=0;j<column;j++) str+=" ";
str+="{\n";
if(_module)
  {
  str+=_module->printLocals(column);
  str+=_module->printCode(column);
  }
for(j=0;j<column;j++) str+=" ";
str+="}\n";
return(str);
}

//-------- Modifiers --------------------------------------------------------//
void StartNode::setOwner(Agent * owner) // virtual
{
if(_module) _module->setOwner(owner);
}

//-------- Evaluation -------------------------------------------------------//
bool StartNode::check(const Context * context,const Model * model) // virtual
{
SmaNode::check(context,model);
if(!_fail)
  {
  if(!_module||_module->mustDisappear()) _fail=true;
  if(getNbNodes())
    {
    if(!getNode(0)->isLeftValue())
      {
      _fail=true;
      Strings message="Argument of '"+
                      getName()+
                      "' should be modifiable";
      manageError(new Error(Error::SEMANTICS,message));
      }
    if(!getNode(0)->getType().canCastTo(ValueType(ValueType::INTEGER)))
      {
      _fail=true;
      Strings message="Argument of '"+getName()+
                      "' should be an 'int' not a '"+
                      getNode(0)->getType().print()+
                      "'";
      manageError(new Error(Error::SEMANTICS,message));
      }
    }
  }
return(!_fail);
}

CodeVect * StartNode::generateVoidCode(void) // virtual
{
CodeVect * codeVect=new CodeVect;
for(int i=_module->getNbParams();i--;)
  codeVect->pushBack(new LocalVarCode(this,i));
if(getNbNodes())
  {
  codeVect->pushBack(new StartCode(this,_module,true));
  CodeVect * tmp=getNode(0)->generateCode();
  for(unsigned int j=0;j<tmp->getSize();j++)
    codeVect->pushBack((*tmp)[j]);
  delete tmp;
  if(getNode(0)->getType().getSimpleType()==ValueType::INTEGER)
    codeVect->pushBack(new Affect_I_I_Code(this));
  else // REAL
    codeVect->pushBack(new Affect_R_I_Code(this));
  }
else
  {
  codeVect->pushBack(new StartCode(this,_module,false));
  }
return(codeVect);
}

