// @(#) ENIB/LI2 (c) - oRis - Version 2.3
//--------------------------------------------------------------------------//
// file        : valOfNd.cpp                                                //
// description : source file for the ENIB/LI2 class ValueOfNode             //
// author(s)   : F. Harrouet                                                //
// copyright   : Laboratoire d'Informatique Industrielle (LI2)              //
//               Ecole Nationale d'Ingenieurs de Brest (ENIB)               //
//               CP 30815 - 29608 Brest Cedex - FRANCE                      //
//               Tel: (33) 02.98.05.66.31 , Fax: (33) 02.98.05.66.29        //
//               e-mail: li2@enib.fr                                        //
//--------------------------------------------------------------------------//
// Copyright (c) 1996-2000                                                  //
// Laboratoire d'Informatique Industrielle (LI2)                            //
// Ecole Nationale d'Ingenieurs de Brest (ENIB)                             //
//                                                                          //
// Permission to use, copy, modify, distribute and sell this software       //
// and its documentation for any purpose is hereby granted without fee,     //
// provided that the above copyright notice appear in all copies and        //
// that both that copyright notice and this permission notice appear        //
// in supporting documentation.                                             //
// ENIB/LI2 makes no representations about the suitability of this          //
// software for any purpose.                                                //
// It is provided "as is" without express or implied warranty.              //
//--------------------------------------------------------------------------//

#include "oRis/Nodes/valOfNd.h"
#include "oRis/Main/errors.h"
#include "oRis/Codes/valCd.h"
#include "oRis/Codes/affectCd.h"
#include "oRis/Codes/utilCd.h"

//---------------------------------------------------------------------------//
// class ValueOfNode                                                         //
//---------------------------------------------------------------------------//

//-------- Allocators -------------------------------------------------------//
ValueOfNode::ValueOfNode(int lineNumber,const Strings & fileName,
                         const Strings & name)
 : SmaNode(2,lineNumber,fileName,name,true)
{}

ValueOfNode::ValueOfNode(const ValueOfNode & node)
 : SmaNode(node)
{}

SmaNode * ValueOfNode::duplicate(void) const // virtual
{
return(new ValueOfNode(*this));
}

ValueOfNode::~ValueOfNode(void)
{}

//-------- Input/Output -----------------------------------------------------//

//-------- Evaluation -------------------------------------------------------//
bool ValueOfNode::check(const Context * context,const Model * model) // virtual
{
SmaNode::check(context,model);
if(!_fail)
  {
  if(!getNode(0)->isLeftValue())
    {
    _fail=true;
    Strings message="First argument of '"+
                    getName()+
                    "(...)' should be modifiable";
    manageError(new Error(Error::SEMANTICS,message));
    }
  if(!getNode(1)->getType().canCastTo(ValueType(ValueType::STRING)))
    {
    _fail=true;
    Strings message="Second argument of '"+
                    getName()+
                    "(...)' should be a 'string'";
    manageError(new Error(Error::SEMANTICS,message));
    }
  if(!_fail) _type=ValueType(ValueType::INTEGER);
  }
return(!_fail);
}

CodeVect * ValueOfNode::generateCode(void) // virtual
{
CodeVect * codeVect=getNode(0)->generateCode(); // result
CodeVect * tmp=getNode(1)->generateCode();
for(unsigned int j=0;j<tmp->getSize();j++)
  codeVect->pushBack((*tmp)[j]);
delete tmp;
codeVect->pushBack(new ValueOfCode(this));
return(codeVect);
}

