// @(#) ENIB/LI2 (c) - oRis - Version 2.3
//--------------------------------------------------------------------------//
// file        : varNd.cpp                                                  //
// description : source file for the ENIB/LI2 class VarNode                 //
// author(s)   : F. Harrouet                                                //
// copyright   : Laboratoire d'Informatique Industrielle (LI2)              //
//               Ecole Nationale d'Ingenieurs de Brest (ENIB)               //
//               CP 30815 - 29608 Brest Cedex - FRANCE                      //
//               Tel: (33) 02.98.05.66.31 , Fax: (33) 02.98.05.66.29        //
//               e-mail: li2@enib.fr                                        //
//--------------------------------------------------------------------------//
// Copyright (c) 1996-2000                                                  //
// Laboratoire d'Informatique Industrielle (LI2)                            //
// Ecole Nationale d'Ingenieurs de Brest (ENIB)                             //
//                                                                          //
// Permission to use, copy, modify, distribute and sell this software       //
// and its documentation for any purpose is hereby granted without fee,     //
// provided that the above copyright notice appear in all copies and        //
// that both that copyright notice and this permission notice appear        //
// in supporting documentation.                                             //
// ENIB/LI2 makes no representations about the suitability of this          //
// software for any purpose.                                                //
// It is provided "as is" without express or implied warranty.              //
//--------------------------------------------------------------------------//

#include "oRis/Nodes/varNd.h"
#include "oRis/Main/errors.h"
#include "oRis/Main/model.h"
#include "oRis/Main/context.h"
#include "oRis/Codes/varCd.h"

//---------------------------------------------------------------------------//
// class VarNode                                                             //
//---------------------------------------------------------------------------//

//-------- Allocators -------------------------------------------------------//
VarNode::VarNode(int lineNumber,const Strings& fileName,const Strings & name)
 : SmaNode(0,lineNumber,fileName,name,false),
   _local(-1)
{}

VarNode::VarNode(const VarNode & node)
 : SmaNode(node), _local(node._local)
{}

SmaNode * VarNode::duplicate(void) const // virtual
{
return(new VarNode(*this));
}

VarNode::~VarNode(void)
{}

//-------- Inspectors -------------------------------------------------------//
bool VarNode::isLeftValue(void) const // virtual
{
return(true);
}

bool VarNode::isLocalValue(void) const // virtual
{
return(_local>=0);
}

//-------- Input/Output -----------------------------------------------------//
Strings VarNode::_print(int) const // virtual
{
return(getName());
}

//-------- Evaluation -------------------------------------------------------//
bool VarNode::check(const Context * context,const Model * model) // virtual
{
SmaNode::check(context,model);
if(!_fail)
  {
  const Attribute * attribute=(const Attribute *)0;
  _local=context->getModule()->getLocalRank(Word::create(getName()));
  if(_local<0)
    {
    if(context->getOwner())
      attribute=context->getOwner()->getAttribute(Word::create(getName()));
    else if(model)
      attribute=model->getAttribute(Word::create(getName()));
    }
  else attribute=context->getModule()->getLocal(_local);
  if(attribute) _type=attribute->getType();
  else
    {
    _fail=true;
    Strings message="Attribute '"+
                    getName()+
                    "' is unknown";
    manageError(new Error(Error::SEMANTICS,message));
    }
  }
return(!_fail);
}

CodeVect * VarNode::generateCode(void) // virtual
{
CodeVect * codeVect=new CodeVect;
if(_local<0)
  codeVect->pushBack(new VarCode(this,Word::create(getName())));
else
  codeVect->pushBack(new LocalVarCode(this,_local));
return codeVect;
}

