// @(#) ENIB/LI2 (c) - oRis - Version 2.3
//--------------------------------------------------------------------------//
// file        : whileNd.cpp                                                //
// description : source file for the ENIB/LI2 class WhileNode               //
// author(s)   : F. Harrouet                                                //
// copyright   : Laboratoire d'Informatique Industrielle (LI2)              //
//               Ecole Nationale d'Ingenieurs de Brest (ENIB)               //
//               CP 30815 - 29608 Brest Cedex - FRANCE                      //
//               Tel: (33) 02.98.05.66.31 , Fax: (33) 02.98.05.66.29        //
//               e-mail: li2@enib.fr                                        //
//--------------------------------------------------------------------------//
// Copyright (c) 1996-2000                                                  //
// Laboratoire d'Informatique Industrielle (LI2)                            //
// Ecole Nationale d'Ingenieurs de Brest (ENIB)                             //
//                                                                          //
// Permission to use, copy, modify, distribute and sell this software       //
// and its documentation for any purpose is hereby granted without fee,     //
// provided that the above copyright notice appear in all copies and        //
// that both that copyright notice and this permission notice appear        //
// in supporting documentation.                                             //
// ENIB/LI2 makes no representations about the suitability of this          //
// software for any purpose.                                                //
// It is provided "as is" without express or implied warranty.              //
//--------------------------------------------------------------------------//

#include "oRis/Nodes/whileNd.h"
#include "oRis/Main/errors.h"
#include "oRis/Codes/utilCd.h"

//---------------------------------------------------------------------------//
// class WhileNode                                                           //
//---------------------------------------------------------------------------//

//-------- Allocators -------------------------------------------------------//
WhileNode::WhileNode(int lineNumber,const Strings & fileName,
                     const Strings & name)
 : SmaNode(1,lineNumber,fileName,name,true)
{}

WhileNode::WhileNode(const WhileNode & node)
 : SmaNode(node)
{}

SmaNode * WhileNode::duplicate(void) const // virtual
{
return(new WhileNode(*this));
}

WhileNode::~WhileNode(void)
{}

//-------- Input/Output -----------------------------------------------------//
Strings WhileNode::print(int column) const // virtual
{
Strings str;
int i;
for(i=0;i<column;i++) str+=" ";
str+=getName()+"("+getNode(0)->print(column)+")\n";
column+=2;
for(i=0;i<column;i++) str+=" ";
str+="{\n";
if(getNbNodes()>1) str+=getNode(1)->print(column);
for(i=0;i<column;i++) str+=" ";
str+="}\n";
return(str);
}

//-------- Evaluation -------------------------------------------------------//
bool WhileNode::check(const Context * context,const Model * model) // virtual
{
SmaNode::check(context,model);
if(!_fail)
  {
  if(getNode(0)->getType().getSimpleType()==ValueType::UNKNOWN)
    {
    _fail=true;
    Strings message="Condition argument of '"+
                    getName()+
                    "' should not be 'void'";
    manageError(new Error(Error::SEMANTICS,message));
    }
  }
return(!_fail);
}

CodeVect * WhileNode::generateVoidCode(void) // virtual
{
CodeVect * codeVect=getNode(0)->generateCode();
CodeVect * tmp;
if(getNbNodes()>1) tmp=getNode(1)->generateVoidCode();
else tmp=new CodeVect;
codeVect->pushBack(new JumpIfFalseCode(this,tmp->getSize()+1));
for(unsigned int j=0;j<tmp->getSize();j++)
  codeVect->pushBack((*tmp)[j]);
delete tmp;
codeVect->pushBack(new JumpCode(this,-(codeVect->getSize()+1)));
return(codeVect);
}


