// @(#) ENIB/LI2 (c) - oRis - Version 2.3
//--------------------------------------------------------------------------//
// file        : orisCtrl.cpp                                               //
// description : source file for the ENIB/LI2 class OrisControl             //
// author(s)   : F. Harrouet                                                //
// copyright   : Laboratoire d'Informatique Industrielle (LI2)              //
//               Ecole Nationale d'Ingenieurs de Brest (ENIB)               //
//               CP 30815 - 29608 Brest Cedex - FRANCE                      //
//               Tel: (33) 02.98.05.66.31 , Fax: (33) 02.98.05.66.29        //
//               e-mail: li2@enib.fr                                        //
//--------------------------------------------------------------------------//
// Copyright (c) 1996-2000                                                  //
// Laboratoire d'Informatique Industrielle (LI2)                            //
// Ecole Nationale d'Ingenieurs de Brest (ENIB)                             //
//                                                                          //
// Permission to use, copy, modify, distribute and sell this software       //
// and its documentation for any purpose is hereby granted without fee,     //
// provided that the above copyright notice appear in all copies and        //
// that both that copyright notice and this permission notice appear        //
// in supporting documentation.                                             //
// ENIB/LI2 makes no representations about the suitability of this          //
// software for any purpose.                                                //
// It is provided "as is" without express or implied warranty.              //
//--------------------------------------------------------------------------//

#if defined(X_GUI) || defined(W_GUI)

#include "orisCtrl.h"
#include "oRis/Main/simul.h"
#include "Utility/tcpUdp.h"

#include <stdlib.h>
#include <signal.h>

//--------------------------------------------------------------------------//
// class OrisControl                                                        //
//--------------------------------------------------------------------------//

OrisControl * OrisControl::_orisControl=(OrisControl *)0;

//-------- Friend functions ------------------------------------------------//
OrisControl * getOrisControl(void) // friend
{
return(OrisControl::_orisControl);
}

//-------- Allocators ------------------------------------------------------//
OrisControl::OrisControl(const StringsVect & commandLine)
 : BlockAllocatedObject(),
   _commandLine(), _fileName(""), _currentPath(""),
   _state(OrisControl::STOP), _oldState((OrisControl::State)-1),
   _quit(false), _step(1), _oldStep(-1), _oldClock((unsigned long)-1),
   _textOutput(&cout), _textError(&cerr),
   _mainWindow((GuiWindow *)0), _mainMenu((GuiMenuBar *)0),
   _runStopItm((GuiMenuItem *)0), _runStopBtn((GuiButton *)0),
   _oneStepBtn((GuiButton *)0), _stateLabel((GuiLabel *)0),
   _clockLabel((GuiLabel *)0), _parseBtn((GuiButton *)0),
   _clearBtn((GuiButton *)0), _execBtn((GuiButton *)0),
   _startBtn((GuiButton *)0), _editor((GuiEditor *)0),
   _prevTask((void (*)(void *))0), _prevTaskData((void *)0)
{
if(_orisControl) delete _orisControl;
_orisControl=this;
_commandLine.push_back(commandLine[0]);
_commandLine.push_back("-gui");
unsigned int limit;
for(limit=1;limit<commandLine.size();limit++)
  {
  if((commandLine[limit]==Strings("-text"))||
     (commandLine[limit]==Strings("-gui")))
    {
    _commandLine[1]=commandLine[limit];
    }
  else if(commandLine[limit][0]=='-')
    {
    _textError << "unknown option '" << commandLine[limit] << "'" << endl;
    }
  else break;
  }
for(;limit<commandLine.size();limit++)
  _commandLine.push_back(commandLine[limit]);
if(_commandLine.size()>2) _fileName=_commandLine[2];
if(_commandLine[1]==Strings("-gui")) buildPanel();
getGuiManager()->show();
getGuiManager()->getTask(_prevTask,_prevTaskData);
newSimul();
struct sigaction act;
act.sa_handler=OrisControl::abortSignal;
sigemptyset(&act.sa_mask);
act.sa_flags=0;
sigaction(SIGINT,&act,(struct sigaction *)0);
if(_mainWindow)
  {
  if(_fileName.len()) load(_fileName);
  }
else if(_fileName.len())
  {
  load(_fileName);
  _state=OrisControl::RUN;
  }
else
  {
  _textError << "a file name is required in text mode" << endl;
  _quit=true;
  }
registerTask();
}

OrisControl::~OrisControl(void)
{
struct sigaction act;
act.sa_handler=SIG_DFL;
sigemptyset(&act.sa_mask);
act.sa_flags=0;
sigaction(SIGINT,&act,(struct sigaction *)0);
getGuiManager()->setTask(_prevTask,_prevTaskData);
if(getSimul()) delete getSimul();
if(_mainWindow) delete _mainWindow;
if(_orisControl==this) _orisControl=(OrisControl *)0;
}

//-------- Views Management ------------------------------------------------//
void OrisControl::buildPanel(void)
{
Strings title=SocketElem::getHostName();
if(title.len()) title=Strings("oRis (")+title+Strings(")");
else title=Strings("oRis");
_mainWindow=new GuiWindow(title);
_mainWindow->setCallback(OrisControl::resizeCB);
_mainWindow->setCloseCallback(OrisControl::quitCB);
_mainMenu=new GuiMenuBar(_mainWindow,true,true);
GuiMenu * menu;
GuiMenuItem * item;
GuiElement * elem;
                                             // Main menu
item=new GuiMenuItem(_mainMenu,"File");
  menu=new GuiMenu(item,false);
  elem=new GuiMenuItem(menu,"Load");
  elem->setCallback(OrisControl::loadCB);
  elem=new GuiMenuItem(menu,"Reload");
  elem->setCallback(OrisControl::reloadCB);
  elem=new GuiMenuSepar(menu);
  elem=new GuiMenuItem(menu,"New");
  elem->setCallback(OrisControl::newSimulCB);
  elem=new GuiMenuItem(menu,"Include");
  elem->setCallback(OrisControl::includeCB);
  elem=new GuiMenuSepar(menu);
  elem=new GuiMenuItem(menu,"Edit");
  elem->setCallback(OrisControl::editCB);
  // elem=new GuiMenuItem(menu,"Save");
  // elem->setCallback(OrisControl::saveCB);
  elem=new GuiMenuSepar(menu);
  elem=new GuiMenuItem(menu,"Quit");
  elem->setCallback(OrisControl::quitCB);
item=new GuiMenuItem(_mainMenu,"Control");
  menu=new GuiMenu(item,false);
  elem=_runStopItm=new GuiMenuItem(menu,"Stop");
  elem->setCallback(OrisControl::runStopCB);
  elem=new GuiMenuSepar(menu);
  elem=new GuiMenuItem(menu,"One step");
  elem->setCallback(OrisControl::oneStepCB);
  elem=new GuiMenuItem(menu,"Step size");
  elem->setCallback(OrisControl::chooseStepCB);
  elem=new GuiMenuSepar(menu);
  elem=new GuiMenuItem(menu,"Parse");
  elem->setCallback(OrisControl::parseCB);
  elem=new GuiMenuItem(menu,"Clear");
  elem->setCallback(OrisControl::clearCB);
  elem=new GuiMenuItem(menu,"execute{}");
  elem->setCallback(OrisControl::execCB);
  elem=new GuiMenuItem(menu,"start{}");
  elem->setCallback(OrisControl::startCB);
item=new GuiMenuItem(_mainMenu,"Tools");
  menu=new GuiMenu(item,false);
  elem=new GuiMenuItem(menu,"Browser");
  elem->setCallback(OrisControl::browserCB);
#ifdef OPEN_GL
  elem=new GuiMenuItem(menu,"3D Viewer");
  elem->setCallback(OrisControl::scene3dCB);
#endif
  elem=new GuiMenuItem(menu,"2D Viewer");
  elem->setCallback(OrisControl::scene2dCB);
  elem=new GuiMenuItem(menu,"Grapher");
  elem->setCallback(OrisControl::grapherCB);
  elem=new GuiMenuItem(menu,"Remote Control");
  elem->setCallback(OrisControl::remoteCtrlCB);
  elem=new GuiMenuItem(menu,"Mail Reader");
  elem->setCallback(OrisControl::readMailCB);
item=new GuiMenuItem(_mainMenu,"Help");
  menu=new GuiMenu(item,false);
  elem=new GuiMenuItem(menu,"About ...");
  elem->setCallback(OrisControl::aboutCB);
  elem=new GuiMenuItem(menu,"Manual");
    menu=new GuiMenu((GuiMenuItem *)elem,false);
    elem=new GuiMenuItem(menu,"Same window");
    elem->setCallback(OrisControl::manualCB,(void *)0);
    elem=new GuiMenuItem(menu,"New window");
    elem->setCallback(OrisControl::manualCB,(void *)1);
                                             // Buttons and labels
int x1=getGuiManager()->getMargin();
int y=menu->getY()+_mainMenu->getHeight()+getGuiManager()->getMargin();
elem=new GuiButton(_mainWindow," Reload ");
  elem->setCallback(OrisControl::reloadCB);
  elem->setX(x1);
  elem->setY(y);
  x1+=getGuiManager()->getMargin()+elem->getWidth();
elem=_runStopBtn=new GuiButton(_mainWindow," Stop ");
  elem->setCallback(OrisControl::runStopCB);
  elem->setX(x1);
  elem->setY(y);
  x1+=getGuiManager()->getMargin()+elem->getWidth();
elem=_stateLabel=new GuiLabel(_mainWindow," One step ",false,true);
  elem->setX(x1);
  elem->setY(y);
  x1+=getGuiManager()->getMargin()+elem->getWidth();
elem=new GuiButton(_mainWindow,"-");
  elem->setCallback(OrisControl::downStepCB);
  elem->setX(x1);
  elem->setY(y);
  x1+=elem->getWidth();
elem=_oneStepBtn=new GuiButton(_mainWindow," 65535 ");
  elem->setCallback(OrisControl::oneStepCB);
  elem->setX(x1);
  elem->setY(y);
  x1+=elem->getWidth();
elem=new GuiButton(_mainWindow,"+");
  elem->setCallback(OrisControl::upStepCB);
  elem->setX(x1);
  elem->setY(y);
  x1+=getGuiManager()->getMargin()+elem->getWidth();
elem=_clockLabel=new GuiLabel(_mainWindow," 4294967295 ",false,true);
  elem->setX(x1);
  elem->setY(y);
  x1+=getGuiManager()->getMargin()+elem->getWidth();
y+=getGuiManager()->getMargin()+elem->getHeight();
elem=_editor=new GuiEditor(_mainWindow,Strings());
  elem->setX(getGuiManager()->getMargin());
  elem->setY(y);
  elem->setHeight(12*getGuiManager()->getCharHeight());
y+=getGuiManager()->getMargin()+elem->getHeight();
int x2=getGuiManager()->getMargin();
Strings largeTitle=" execute {...} ";
elem=_parseBtn=new GuiButton(_mainWindow,largeTitle);
  elem->setCallback(OrisControl::parseCB);
  elem->setTitle(" Parse ");
  elem->setX(x2);
  elem->setY(y);
  x2+=getGuiManager()->getMargin()+elem->getWidth();
elem=_clearBtn=new GuiButton(_mainWindow,largeTitle);
  elem->setCallback(OrisControl::clearCB);
  elem->setTitle(" Clear ");
  elem->setX(x2);
  elem->setY(y);
  x2+=getGuiManager()->getMargin()+elem->getWidth();
elem=_execBtn=new GuiButton(_mainWindow,largeTitle);
  elem->setCallback(OrisControl::execCB);
  elem->setTitle(" execute {...} ");
  elem->setX(x2);
  elem->setY(y);
  x2+=getGuiManager()->getMargin()+elem->getWidth();
elem=_startBtn=new GuiButton(_mainWindow,largeTitle);
  elem->setCallback(OrisControl::startCB);
  elem->setTitle(" start {...} ");
  elem->setX(x2);
  elem->setY(y);
  x2+=getGuiManager()->getMargin()+elem->getWidth();
y+=getGuiManager()->getMargin()+elem->getHeight();
_mainWindow->setInWidth(x1>x2 ? x1 : x2);
_mainWindow->setInHeight(y);
}

void OrisControl::displayClock(void)
{
if(getSimul()->getClock()!=_oldClock)
  {
  if(_mainWindow)
    _clockLabel->setTitle(" "+Strings(getSimul()->getClock())+" ");
  _oldClock=getSimul()->getClock();
  }
}

void OrisControl::displayState(void)
{
if(_state!=_oldState)
  {
  if(_mainWindow)
    {
    switch(_state)
      {
      case OrisControl::BAD:
        _stateLabel->setTitle(" Invalid ");
        _runStopItm->setTitle("~~~~");
        _runStopBtn->setTitle(" ~~~~ ");
        break;
      case OrisControl::STOP:
        _stateLabel->setTitle(" Paused ");
        _runStopItm->setTitle("Run");
        _runStopBtn->setTitle(" Run ");
        break;
      case OrisControl::RUN:
        _stateLabel->setTitle(" Running ");
        _runStopItm->setTitle("Stop");
        _runStopBtn->setTitle(" Stop ");
        break;
      case OrisControl::ONESTEP:
        _stateLabel->setTitle(" One step ");
        _runStopItm->setTitle("Run");
        _runStopBtn->setTitle(" Run ");
        break;
      }
    }
  _oldState=_state;
  }
if(_step!=_oldStep)
  {
  if(_mainWindow) _oneStepBtn->setTitle(" "+Strings(_step)+" ");
  _oldStep=_step;
  }
}

void OrisControl::singleMessage(const Strings & message)
{
StringsVect msg;
msg.push_back(message);
StringsVect buttons;
buttons.push_back("  OK  ");
GuiButtonDialog dialog("oRis message",msg,buttons);
dialog.show();
dialog.ask();
}

bool OrisControl::singleQuestion(const Strings & question)
{
StringsVect msg;
msg.push_back(question);
StringsVect buttons;
buttons.push_back("  OK  ");
buttons.push_back(" Cancel ");
GuiButtonDialog dialog("oRis question",msg,buttons);
dialog.show();
return(dialog.ask()==0);
}

//-------- Simulation Control ----------------------------------------------//
void OrisControl::mainTask(void)
{
switch(_state)
  {
  case OrisControl::RUN:
    getSimul()->run(_step);
    displayClock();
    if(getSimul()->isBad())
      {
      _state=OrisControl::BAD;
      displayState();
      }
    break;
  case OrisControl::ONESTEP:
    getSimul()->run(_step);
    if(getSimul()->isBad()) _state=OrisControl::BAD;
    else _state=OrisControl::STOP;
    registerTask();
    break;
  default: // STOP or BAD
    break;
  }
}

void OrisControl::registerTask(void)
{
if(_quit)
  {
  getGuiManager()->setTask(OrisControl::_quitTask,(void *)0);
  }
else
  {
  switch(_state)
    {
    case OrisControl::RUN:
    case OrisControl::ONESTEP:
      getGuiManager()->setTask(OrisControl::_mainTask,(void *)0);
      break;
    default: // STOP or BAD
      getGuiManager()->setTask(_prevTask,_prevTaskData);
      break;
    }
  }
displayState();
displayClock();
}

void OrisControl::newSimul(void)
{
Simulator * simul=new Simulator(_commandLine,&_textOutput,&_textError);
                                              // the old one is destroyed
simul->setControlCB(OrisControl::controlCB,(void *)this);
if(simul->isBad()) _state=OrisControl::BAD;
else if(_state==OrisControl::BAD)
  _state=OrisControl::STOP;
if(_mainWindow)
  {
  Strings title=SocketElem::getHostName();
  if(title.len()) title=Strings("oRis (")+title+Strings(")");
  else title=Strings("oRis");
  _mainWindow->setTitle(title);
  }
registerTask();
}

void OrisControl::load(const Strings & fileName)
{
_fileName=fileName;
if(_commandLine.size()<=2) _commandLine.push_back(_fileName);
else _commandLine[2]=_fileName;
newSimul();
if(!getSimul()->isBad()) getSimul()->parseFile(_fileName);
if(getSimul()->isBad()) _state=OrisControl::BAD;
if(_mainWindow)
  {
  Strings title=SocketElem::getHostName();
  if(title.len()) title=Strings("oRis (")+title+Strings(")");
  else title=Strings("oRis");
  _mainWindow->setTitle(title+Strings(" - ")+_fileName.basename());
  }
registerTask();
}

void OrisControl::save(const Strings & fileName)
{
if(!getSimul()->save(fileName))
  {
  _textError << "Can't save in '"+fileName+"'" << endl;
  }
}

//-------- Callback Methods ------------------------------------------------//
Strings OrisControl::controlCB(Simulator *,void *,
                               const Strings & command,const Strings & arg)
{
if(command==Strings("quit"))
  {
  if(arg==Strings("yes"))
    {
    _orisControl->_quit=true;
    _orisControl->registerTask();
    return("yes");
    }
  else
    {
    OrisControl::quitCB((GuiElement *)0,(void *)0);
    if(_orisControl->_quit) return("yes");
    else return("no");
    }
  }
else if(command==Strings("state"))
  {
  switch(_orisControl->_state)
    {
    case OrisControl::BAD:
      return("bad");
    case OrisControl::STOP:
      return("stop");
    case OrisControl::RUN:
      return("run");
    case OrisControl::ONESTEP:
      return("one step");
    }
  }
else if(command==Strings("run"))
  {
  switch(_orisControl->_state)
    {
    case OrisControl::BAD:
      _orisControl->singleMessage("oRis can't restart after errors !");
      break;
    case OrisControl::STOP:
      _orisControl->_state=OrisControl::RUN;
      _orisControl->registerTask();
      break;
    default:
      break;
    }
  return("");
  }
else if(command==Strings("stop"))
  {
  if(_orisControl->_state==OrisControl::RUN)
    {
    _orisControl->_state=OrisControl::STOP;
    _orisControl->registerTask();
    }
  return("");
  }
else if(command==Strings("one step"))
  {
  OrisControl::oneStepCB((GuiElement *)0,(void *)0);
  return("");
  }
else if(command==Strings("get step"))
  {
  return(Strings(_orisControl->_step));
  }
else if(command==Strings("set step"))
  {
  int step;
  if((sscanf(arg,"%d",&step)==1)&&
     (step>0)&&(step<=65536))
    {
    _orisControl->_step=step;
    _orisControl->displayState();
    return(arg);
    }
  else return("bad argument");
  }
else if(command==Strings("get shape"))
  {
  if(_orisControl->_mainWindow)
    {
    return(Strings("[")+
           Strings(_orisControl->_mainWindow->getX())+
           Strings(",")+
           Strings(_orisControl->_mainWindow->getY())+
           Strings(",")+
           Strings(_orisControl->_mainWindow->getWidth())+
           Strings(",")+
           Strings(_orisControl->_mainWindow->getHeight())+
           Strings("]"));
    }
  else
    {
    return("command '"+command+"' not supported in text mode");
    }
  }
else if(command==Strings("set shape"))
  {
  if(_orisControl->_mainWindow)
    {
    int x,y,w,h;
    if(sscanf(arg,"[%d,%d,%d,%d]",&x,&y,&w,&h)==4)
      {
      _orisControl->_mainWindow->setX(x);
      _orisControl->_mainWindow->setY(y);
      _orisControl->_mainWindow->setWidth(w);
      _orisControl->_mainWindow->setHeight(h);
      return(arg);
      }
    else return("bad argument");
    }
  else
    {
    return("command '"+command+"' not supported in text mode");
    }
  }
else if(command==Strings("about"))
  {
  OrisControl::aboutCB((GuiElement *)0,(void *)0);
  return("");
  }
else if(command==Strings("manual"))
  {
  bool newWindow=true;
  if((arg==Strings("false"))||(arg==Strings("0"))) newWindow=false;
  OrisControl::manualCB((GuiElement *)0,(void *)newWindow);
  return("");
  }
return("unknown control");
}

void OrisControl::aboutCB(GuiElement *,void *)
{
Strings version;
StringsVect msg;
msg.push_back("o R i s : Active Object Simulator");
msg.push_back("");
msg.push_back("  - Version 2.3");
msg.push_back("  - Build date : May, 16th, 2001");
msg.push_back("  - Copyright (c) 1996-2001");
msg.push_back("_____________________________________________");
msg.push_back("");
msg.push_back("Laboratoire d'Informatique Industrielle (LI2)");
msg.push_back("Ecole Nationale d'Ingenieurs de Brest (ENIB)");
msg.push_back("  - CP 30815 - 29608 Brest Cedex - FRANCE");
msg.push_back("  - Tel: +33 (0)298 05 66 31");
msg.push_back("  - Fax: +33 (0)298 05 66 29");
msg.push_back("_____________________________________________");
msg.push_back("");
msg.push_back("Contact: Fabrice HARROUET");
msg.push_back("  - e-mail: harrouet@enib.fr");
msg.push_back("  - url: http://www.enib.fr/~harrouet/");
StringsVect buttons;
GuiButtonDialog dialog("oRis - About",msg,buttons);
dialog.show();
dialog.ask();
}

void OrisControl::manualCB(GuiElement *,void * data)
{
bool newWindow=data ? true : false;
Strings url="file:"+getSimul()->getInstallDir()+"Doc/index.html";
getSimul()->showURL(url,newWindow);
}

void OrisControl::quitCB(GuiElement *,void *)
{
if(_orisControl->singleQuestion("Do you really want to quit oRis ?"))
  {
  _orisControl->_quit=true;
  _orisControl->registerTask();
  }
}

void OrisControl::runStopCB(GuiElement *,void *)
{
switch(_orisControl->_state)
  {
  case OrisControl::BAD:
    _orisControl->singleMessage("oRis can't restart after errors !");
    break;
  case OrisControl::STOP:
    _orisControl->_state=OrisControl::RUN;
    break;
  case OrisControl::RUN:
  case OrisControl::ONESTEP:
    _orisControl->_state=OrisControl::STOP;
    break;
  default:
    break;
  }
_orisControl->registerTask();
}

void OrisControl::oneStepCB(GuiElement *,void *)
{
if(_orisControl->_state==OrisControl::BAD)
  _orisControl->singleMessage("oRis can't restart after errors !");
else
  _orisControl->_state=OrisControl::ONESTEP;
_orisControl->registerTask();
}

void OrisControl::newSimulCB(GuiElement *,void *)
{
if(_orisControl->singleQuestion(
   "Do you really want to clear the current simulation ?"))
  {
  _orisControl->newSimul();
  _orisControl->_fileName="";
  while(_orisControl->_commandLine.size()>2)
    _orisControl->_commandLine.pop_back();
  }
}

void OrisControl::includeCB(GuiElement *,void *)
{
if(_orisControl->_state==OrisControl::BAD)
  _orisControl->singleMessage("oRis can't restart after errors !");
else
  {
  GuiFileChooser chooser("oRis - include",
                          "Include oRis file");
  if(_orisControl->_currentPath.len())
    chooser.setInitPath(_orisControl->_currentPath);
  chooser.show();
  Strings answer;
  if(chooser.ask(answer))
    {
    getSimul()->parseStrings("include \""+answer+"\"");
    if(getSimul()->isBad())
      {
      _orisControl->_state=OrisControl::BAD;
      _orisControl->registerTask();
      }
    }
  _orisControl->_currentPath=chooser.getPath();
  }
}

void OrisControl::loadCB(GuiElement *,void *)
{
GuiFileChooser chooser("oRis - load",
                        "Load oRis simulation");
if(_orisControl->_currentPath.len())
  chooser.setInitPath(_orisControl->_currentPath);
chooser.show();
Strings answer;
if(chooser.ask(answer)) _orisControl->load(answer);
_orisControl->_currentPath=chooser.getPath();
}

void OrisControl::reloadCB(GuiElement * calling,void * data)
{
if(_orisControl->_fileName.len())
  _orisControl->load(_orisControl->_fileName);
else
  OrisControl::loadCB(calling,data);
}

void OrisControl::saveCB(GuiElement *,void *)
{
GuiFileChooser chooser("oRis - save",
                        "Save current simulation");
if(_orisControl->_currentPath.len())
  chooser.setInitPath(_orisControl->_currentPath);
chooser.show();
Strings answer;
if(chooser.ask(answer)) _orisControl->save(answer);
_orisControl->_currentPath=chooser.getPath();
}

void OrisControl::parseCB(GuiElement *,void *)
{
if(_orisControl->_state==OrisControl::BAD)
  _orisControl->singleMessage("oRis can't restart after errors !");
else
  {
  getSimul()->parseStrings(_orisControl->_editor->getContent());
  if(getSimul()->isBad())
    {
    _orisControl->_state=OrisControl::BAD;
    _orisControl->registerTask();
    }
  }
}

void OrisControl::clearCB(GuiElement *,void *)
{
_orisControl->_editor->setContent("");
}

void OrisControl::execCB(GuiElement *,void *)
{
Strings content=_orisControl->_editor->getContent();
if(content.len()&&(content[content.len()-1]!='\n')) content+='\n';
_orisControl->_editor->setContent(content+"::execute\n{\n\n}\n");
}

void OrisControl::startCB(GuiElement *,void *)
{
Strings content=_orisControl->_editor->getContent();
if(content.len()&&(content[content.len()-1]!='\n')) content+='\n';
_orisControl->_editor->setContent(content+"::start\n{\n\n}\n");
}

void OrisControl::editCB(GuiElement *,void *)
{
GuiFileChooser chooser("oRis - edit",
                        "Edit text file");
if(_orisControl->_currentPath.len())
  chooser.setInitPath(_orisControl->_currentPath);
chooser.show();
Strings answer;
if(chooser.ask(answer)) getSimul()->editFile(answer);
_orisControl->_currentPath=chooser.getPath();
}

void OrisControl::upStepCB(GuiElement *,void *)
{
_orisControl->_step*=2;
if(_orisControl->_step>65536) _orisControl->_step=65536;
_orisControl->displayState();
}

void OrisControl::downStepCB(GuiElement *,void *)
{
_orisControl->_step/=2;
if(_orisControl->_step<1) _orisControl->_step=1;
_orisControl->displayState();
}

void OrisControl::chooseStepCB(GuiElement *,void *)
{
StringsVect msg;
msg.push_back("Enter a value in [1,65536]");
msg.push_back("for the simulation step");
GuiTextDialog dialog("oRis - step",msg);
dialog.show();
int newStep=0;
bool stop=false;
do
  {
  dialog.setInitText(Strings(_orisControl->_step));
  Strings answer;
  if(dialog.ask(answer))
    {
    newStep=::atoi(answer);
    if((newStep>0)&&(newStep<=65536))
      {
      _orisControl->_step=newStep;
      stop=true;
      }
    else stop=false;
    }
  else stop=true;
  } while(!stop);
_orisControl->displayState();
}

void OrisControl::browserCB(GuiElement *,void *)
{
getSimul()->parseStrings("\
  include \"browser.pkg\" \
  execute{new GlobalBrowser->show();} \
  ");
}

void OrisControl::scene2dCB(GuiElement *,void *)
{
getSimul()->parseStrings("\
  include \"object2d.pkg\" \
  execute{new Viewer2d->show();} \
  ");
}

#ifdef OPEN_GL
void OrisControl::scene3dCB(GuiElement *,void *)
{
getSimul()->parseStrings("\
  include \"object3d.pkg\" \
  execute{new Viewer3d->show();} \
  ");
}
#endif

void OrisControl::grapherCB(GuiElement *,void *)
{
getSimul()->parseStrings("\
  include \"grapher.pkg\" \
  execute{new Grapher->show();} \
  ");
}

void OrisControl::remoteCtrlCB(GuiElement *,void *)
{
getSimul()->parseStrings("\
  include \"remote.pkg\" \
  execute{new RemotePanel->show();} \
  ");
}

void OrisControl::readMailCB(GuiElement *,void *)
{
getSimul()->parseStrings("\
  include \"mailRead.pkg\" \
  execute{new MailReader->show();} \
  ");
}

void OrisControl::resizeCB(GuiElement *,void *)
{
_orisControl->_parseBtn->setY(_orisControl->_mainWindow->getInHeight()-
                              _orisControl->_parseBtn->getHeight()-
                              getGuiManager()->getMargin());
_orisControl->_clearBtn->setY(_orisControl->_parseBtn->getY());
_orisControl->_execBtn->setY(_orisControl->_parseBtn->getY());
_orisControl->_startBtn->setY(_orisControl->_parseBtn->getY());
_orisControl->_editor->setWidth(_orisControl->_mainWindow->getInWidth()-
                                2*getGuiManager()->getMargin());
_orisControl->_editor->setHeight(_orisControl->_parseBtn->getY()-
                                 _orisControl->_editor->getY()-
                                 getGuiManager()->getMargin());
}

void OrisControl::abortSignal(int) // static
{
getSimul()->abortSignal();
_orisControl->_state=OrisControl::STOP;
if(!_orisControl->_mainWindow)
  _orisControl->_quit=true;
_orisControl->registerTask();
}

void OrisControl::_mainTask(void *) // static
{
_orisControl->mainTask();
if(_orisControl->_prevTask)
  (*_orisControl->_prevTask)(_orisControl->_prevTaskData);
}

void OrisControl::_quitTask(void *) // static
{
getGuiManager()->exitMainLoop();
}

#endif // X_GUI || W_GUI

