//-------------------------------------------------------------------------
//
// File    	:  fuzzyPkg.h
// Description  :  fuzzy pkg for oRis
//
// Author(s)   	: P. Reignier
// Date         : 14/09/99
// Copyright	: Laboratoire d'Informatique Industrielle (LI2)
//		  Ecole Nationale d'Ingenieurs de Brest (ENIB)
//		  CP 15 - 29608 Brest Cedex - FRANCE
//		  Tel: (33) 98.05.66.31, Fax: (33) 98.05.66.29
//		  e-mail: li2@li2.enib.fr
//-------------------------------------------------------------------------
// Copyright (c) 1996
// Laboratoire d'Informatique Industrielle (LI2)
// Ecole Nationale d'Ingenieurs de Brest (ENIB)
//
// Permission to use, copy, modify, distribute and sell this software
// and its documentation for any purpose is hereby granted without fee,
// provided that the above copyright notice appear in all copies and
// that both that copyright notice and this permission notice appear
// in supporting documentation.
// ENIB/LI2 makes no representations about the suitability of this
// software for any purpose.
// It is provided "as is" without express or implied warranty.
//-------------------------------------------------------------------------

#ifndef FUZZYPKG_H
#define FUZZYPKG_H

#ifndef __MSDOS__
// #include <stl_config.h>
#endif

#include "Utility/stlVect.h"
#include "oRis/Main/agent.h"
#include "point2D.h"
#include "oRis/Main/dLibInfo.h"
#include "fuzzySet.h"

class Value ;

//--------------------------------------------------------------------------//
// LookUp table for functions                                               //
//--------------------------------------------------------------------------//

ExternFunction getFunction(const Strings & name);

//--------------------------------------------------------------------------//
// LookUp table for agents                                                  //
//--------------------------------------------------------------------------//

Agent * getgent(Model * model,long instNum);

//--------------------------------------------------------------------------//
// OFuzzySet
//--------------------------------------------------------------------------//

class Model ;

class OFuzzySet : public Agent
{
public:
  OFuzzySet(Model* model, int instNum) ;
  virtual ~OFuzzySet(void) ;

  void setCrispValue(double value) ;
  void setFuzzyValue(const StlVector< Point2D >& value) ;
  void setValue(const FuzzySet* fyzzySet) ;
  
  virtual double getCrispValue(void) const ;
  virtual const StlVector<Point2D>& getFuzzyValue() const ;
  virtual bool isCrisp(void) const ;
  
  virtual void saturate(double a) {_set->saturate(a);_modified->setInteger(1);}
  virtual void product(double a) {_set->product(a);_modified->setInteger(1);}
  virtual void combineMax(const FuzzySet& set) {_set->combineMax(set);_modified->setInteger(1);}
  virtual void combineMin(const FuzzySet& set) {_set->combineMin(set);_modified->setInteger(1);}
  virtual void combineSum(const FuzzySet& set) {_set->combineSum(set);_modified->setInteger(1);}

  virtual double mu(double x) const {return _set->mu(x);}
  virtual double integral(void) const {return _set->integral();}
  virtual double xIntegral(void) const {return _set->xIntegral();}
  virtual FuzzySet* accessFuzzySet(void) { return _set ; }
  virtual const FuzzySet* getFuzzySet(void) const {return _set ; }

  virtual bool maxMu(StlVector<Point2D>& set) const ;
  virtual double maxMin(const FuzzySet& set) const {return _set->maxMin(set);}

  virtual void simplify(void) {_set->simplify();}
  virtual void normalize(void) {_set->normalize();_modified->setInteger(1);}

  static void staticDelete(Context* context) ;
  static void staticSetCrispValue(Context* context) ;
  static void staticSetFuzzyValue(Context* context) ;
  static void staticGetCrispValue(Context* context) ;
  static void staticGetFuzzyValue(Context* context) ;
  static void staticIsCrisp(Context* context) ;

  static void staticSaturate(Context* context) ;
  static void staticProduct(Context* context) ;
  static void staticCombineMax(Context* context) ;
  static void staticCombineMin(Context* context) ;
  static void staticCombineSum(Context* context) ;
  static void staticMu(Context* context) ;
  static void staticIntegral(Context* context) ;
  static void staticXIntegral(Context* context) ;
  static void staticMaxMu(Context* context) ;
  static void staticMaxMin(Context* context) ;
  static void staticSimplify(Context* context) ;
  static void staticNormalize(Context* context) ;
  static void staticNot(Context* context) ;
  static void staticMoreOrLess(Context* context) ;
  static void staticVery(Context* context) ;

protected:
  FuzzySet* _set ;
  Value* _modified ; // value modified each time the set is modified
                     // it allows to use AttrCB on fuzzySets
} ;

//----------------------------------------------------------------------//
// Modifiers
//----------------------------------------------------------------------//

void staticNot(Context*) ;
void staticVery(Context*) ;
void staticMoreOrLess(Context*) ;

#endif
