//-------------------------------------------------------------------------
//
// File    	:  point2D.h
// Description  :  
//
// Author(s)   	: P. Reignier
// Date         : 09/03/97
// Copyright	: Laboratoire d'Informatique Industrielle (LI2)
//		  Ecole Nationale d'Ingenieurs de Brest (ENIB)
//		  CP 15 - 29608 Brest Cedex - FRANCE
//		  Tel: (33) 98.05.66.31, Fax: (33) 98.05.66.29
//		  e-mail: li2@li2.enib.fr
//-------------------------------------------------------------------------
// Copyright (c) 1996
// Laboratoire d'Informatique Industrielle (LI2)
// Ecole Nationale d'Ingenieurs de Brest (ENIB)
//
// Permission to use, copy, modify, distribute and sell this software
// and its documentation for any purpose is hereby granted without fee,
// provided that the above copyright notice appear in all copies and
// that both that copyright notice and this permission notice appear
// in supporting documentation.
// ENIB/LI2 makes no representations about the suitability of this
// software for any purpose.
// It is provided "as is" without express or implied warranty.
//-------------------------------------------------------------------------


#ifndef _POINT2D
#define _POINT2D

#include <iostream>
#include "Utility/stlUtil.h"
#include "Utility/blockAlc.h"

class Point2D : public BlockAllocatedObject
{
public:

  // Constructors

  Point2D(double x = 0.0, double y = 0.0) ;
  Point2D(const Point2D &p) ;    // copy constructor

  // Destructors

  virtual ~Point2D(void) ;

  // Operators

  friend ostream& operator<< (ostream& flot, const Point2D& p) ;  // injection
  friend bool operator== (const Point2D& p1, const Point2D& p2) ; // equality 
  friend bool operator!= (const Point2D& p1, const Point2D& p2) ; // difference
  friend Point2D& operator+ (const Point2D& p1, const Point2D& p2) ; 
  // addition
  bool operator<(const Point2D& p1) const 
    { 
      return _x < p1._x  || _y < p1._y ;
    }
  
  Point2D& operator=(const Point2D& p) ; // affectation

  // Consultation

  inline double getX() const {return(_x) ;} ; 
  inline double getY() const {return(_y) ;} ;

  // Modifications
  
  inline void setX(double x) {_x = x ;} ;
  inline void setY(double y) {_y = y ;} ;

  static const double epsilon ;

private :

  // Utilities

  void _copy (const Point2D&) ;

  // Data

  double _x ; // abscisse
  double _y ; // ordonnee
} ;

#endif

