%{
//-------------------------------------------------------------------------
//
// File    	:  fuzzyParse.y
// Description  :  Syntaxic analysis of fuzzy rules
//
// Author(s)   	: P. Reignier
// Date         : 01/04/98
// Copyright	: Laboratoire d'Informatique Industrielle (LI2)
//		  Ecole Nationale d'Ingenieurs de Brest (ENIB)
//		  CP 15 - 29608 Brest Cedex - FRANCE
//		  Tel: (33) 98.05.66.31, Fax: (33) 98.05.66.29
//		  e-mail: li2@li2.enib.fr
//-------------------------------------------------------------------------
// Copyright (c) 1996
// Laboratoire d'Informatique Industrielle (LI2)
// Ecole Nationale d'Ingenieurs de Brest (ENIB)
//
// Permission to use, copy, modify, distribute and sell this software
// and its documentation for any purpose is hereby granted without fee,
// provided that the above copyright notice appear in all copies and
// that both that copyright notice and this permission notice appear
// in supporting documentation.
// ENIB/LI2 makes no representations about the suitability of this
// software for any purpose.
// It is provided "as is" without express or implied warranty.
//-------------------------------------------------------------------------

#include <iostream> 
#include <fstream>
#include <stdio.h>

#include "oRis/Main/errors.h"    
#include "oRis/Main/simul.h"
#include "Utility/strings.h"
#include "fuzzyController.h"
#include "Utility/strings.h"

class FuzzyParserLoad ;
#include "fuzzyScanLoad.h"

#ifndef __MSDOS__
#define malloc(size) BlockAllocator::allocate(size);
#define free(ptr)    BlockAllocator::free(ptr);
#endif
%}

%name FuzzyParserLoad
%define INHERIT : public BlockAllocatedObject

%define MEMBERS FuzzyLexerLoad* _scanner ;\
        Strings _line ; \
        FuzzyController* _controller; \
        int _lineNumber ;\
	void initParser(istream*, FuzzyController*) ;\
        virtual ~FuzzyParserLoad(void) ;

%define CONSTRUCTOR_PARAM istream* stream, \
	  FuzzyController* controller
	 
%define CONSTRUCTOR_CODE initParser(stream, controller)

%define LEX_BODY   {return _scanner->yylex(this) ;}

%start Program

%token <string> IF 
%token <string> IS 
%token <string> THEN 
%token <string> TERM 
%token <string> AND 
%token <string> OR 
%token <string> PV 
%token <string> V 
%token <string> PO 
%token <string> PF
%token <string> DELTA

%union {
  Strings* string ;   
}


%type <string> ListRules 
%type <string> ListActions
%type <string> VariableCond VariableConcl
%type <string> ListModifiers
%type <string> TermComplex
%type <string> Rules
%type <string> Prop
%type <string> PropElem
%type <string> Action
%type <string> Program


%left OR
%left AND
%right MOD
%%

Program : ListRules
;

/*----------------------------------------------------------------------*/
/*                                                                      */
/* RULES DEFINITIONS                                                    */
/*                                                                      */
/*----------------------------------------------------------------------*/


ListRules :	ListRules Rules 
                {
		  _controller->addRule(*$2) ;
		}
                | /* empty */ {
		}
;


Rules : 	IF Prop THEN ListActions PV 
                {
		  $$ = new Strings ;
		  *$$ = *$1 + *$2 + *$3 + *$4 + *$5 ;
		  delete $1 ;
		  delete $2 ;
		  delete $3 ;
		  delete $4 ;
		  delete $5 ;
		}
;


Prop :		PropElem 
                {
		  $$ = new Strings ;
		  *$$ = *$1 ;
		  delete $1 ;
		}
                | PO Prop PF
                {
		  $$ = new Strings ;
		  *$$ = *$1 + *$2 + *$3 ;
		  delete $1 ;
		  delete $2 ;
		  delete $3 ;
		}
		| Prop AND Prop 
                {
		  $$ = new Strings ;
		  *$$ = *$1 + *$2 + *$3 ;
		  delete $1 ;
		  delete $2 ;
		  delete $3 ;
		}
		| Prop OR Prop 
                {
		  $$ = new Strings ;
		  *$$ = *$1 + *$2 + *$3 ;
		  delete $1 ;
		  delete $2 ;
		  delete $3 ;
		}
;

PropElem :	VariableCond IS TermComplex 
                {
		  $$ = new Strings ;
		  *$$ = *$1 + *$2 + *$3 ;
		  delete $1 ;
		  delete $2 ;
		  delete $3 ;
                }
;


TermComplex	: TERM
                {
		  $$ = new Strings ;
		  *$$ = *$1 ;
		  delete $1 ;
		}
                | ListModifiers TERM
                {
		  $$ = new Strings ;
		  *$$ = *$1 + *$2 ;
		  delete $1 ;
		  delete $2 ;
		}
;

ListModifiers : ListModifiers TERM
              {
		$$ = new Strings ;
		*$$ = *$1 + *$2 ;
		delete $1 ;
		delete $2 ;
	      }
              | TERM
              {
		$$ = new Strings ;
		*$$ = *$1 ;
		delete $1 ;
	      }

VariableCond : TERM 
             { 
		$$ = new Strings ;
		*$$ = *$1 ;
		delete $1 ;
	     }
             | DELTA TERM
             {
		$$ = new Strings ;
		*$$ = *$1 + *$2 ;
		delete $1 ;
		delete $2 ;
	     }
; 

VariableConcl : TERM 
              { 
		$$ = new Strings ;
		*$$ = *$1 ;
		delete $1 ;
	      }
              | DELTA TERM
              {
		$$ = new Strings ;
		*$$ = *$1 + *$2 ;
		delete $1 ;
		delete $2 ;
	      }

ListActions :	Action {
		$$ = new Strings ;
		*$$ = *$1 ;
		delete $1 ;
                }
                | ListActions AND ListActions 
                {
		  $$ = new Strings ;
		  *$$ = *$1 + *$2 + *$3 ;
		  delete $1 ;
		  delete $2 ;
		  delete $3 ;
		}		  
;

Action : 	VariableConcl IS TermComplex 
                {
		  $$ = new Strings ;
		  *$$ = *$1 + *$2 + *$3 ;
		  delete $1 ;
		  delete $2 ;
		  delete $3 ;
		}
;


%%

//----------------------------------------------------------------------//
//
// C++ CODE
//
//----------------------------------------------------------------------//

//------FuzzyParserLoad::yyerror()------------------------------------------//

void FuzzyParserLoad::yyerror (char *)
{
  char nb[100] ;
  sprintf(nb, "%d", _lineNumber) ;
  Strings message = _line + "<--- : Syntax error line " + nb ;
  getSimul()->manageError(new Error(Error::SYNTAX, message)); 
  return ;
}
		
//------FuzzyParserLoad::initParser()---------------------------------------//

void FuzzyParserLoad::initParser(istream *stream, FuzzyController* controller)
{
  _controller = controller ;
  _scanner=new FuzzyLexerLoad(stream) ; 
  _line = Strings("") ;
  _lineNumber = 0 ;
}


//----------------------------------------------------------------------//

FuzzyParserLoad::~FuzzyParserLoad(void)
{
  delete _scanner ;
}



