// @(#) ENIB/LI2 (c) - oRis - Version 2.1
//--------------------------------------------------------------------------//
// file        : filesPkg.cpp                                               //
// description : source file for the ENIB/LI2 oRis package files.pkg        //
// author(s)   : P. Reignier                                                //
// copyright   : Laboratoire d'Informatique Industrielle (LI2)              //
//               Ecole Nationale d'Ingenieurs de Brest (ENIB)               //
//               CP 30815 - 29608 Brest Cedex - FRANCE                      //
//               Tel: (33) 02.98.05.66.31 , Fax: (33) 02.98.05.66.29        //
//               e-mail: li2@enib.fr                                        //
//--------------------------------------------------------------------------//
// Copyright (c) 1996,1999                                                  //
// Laboratoire d'Informatique Industrielle (LI2)                            //
// Ecole Nationale d'Ingenieurs de Brest (ENIB)                             //
//                                                                          //
// Permission to use, copy, modify, distribute and sell this software       //
// and its documentation for any purpose is hereby granted without fee,     //
// provided that the above copyright notice appear in all copies and        //
// that both that copyright notice and this permission notice appear        //
// in supporting documentation.                                             //
// ENIB/LI2 makes no representations about the suitability of this          //
// software for any purpose.                                                //
// It is provided "as is" without express or implied warranty.              //
//--------------------------------------------------------------------------//

#include "oRis/Main/simul.h"
#include "oRis/Main/tmpVal.h"
#include "fuzzyPkg.h"
#include "point2D.h"
#include "fuzzyController.h"
#include "oRis/Main/model.h"
#include "oRis/Main/context.h"
#include "fuzzySet.h"
#include "Utility/stlVect.h"
#include "oRis/Main/errors.h"
#include "oRis/Main/context.h"

#include <stdio.h>

#ifndef __MSDOS__
#define EXPORT_PREFIX extern "C"
#else
#include <windows.h>
#define EXPORT_PREFIX extern "C" __declspec(dllexport)
#define openingLibrary    openingFuzzyLibrary
#define closingLibrary    closingFuzzyLibrary
#define getExternFunction getExternFuzzyFunction
#define getNewAgent       getNewFuzzyAgent
#endif
 
//--------------------------------------------------------------------------//
// Opening and closing the library                                          //
//--------------------------------------------------------------------------//
 
EXPORT_PREFIX void openingLibrary(void)
{
  // nothing to do
}
 
EXPORT_PREFIX void closingLibrary(void)
{
  // nothing to do
}

//--------------------------------------------------------------------------//
// LookUp table for functions                                               //
//--------------------------------------------------------------------------//

EXPORT_PREFIX ExternFunction getExternFunction(const Strings & name)
{

  //----------------------------------------------------------------------//
  // Fuzzy Set
  //----------------------------------------------------------------------//

  if(name==Strings("FuzzySet::setCrispValue()")) 
    return(OFuzzySet::staticSetCrispValue);
  if(name==Strings("FuzzySet::setFuzzyValue()")) 
    return(OFuzzySet::staticSetFuzzyValue);
  if(name==Strings("FuzzySet::getCrispValue()")) 
    return(OFuzzySet::staticGetCrispValue);
  if(name==Strings("FuzzySet::getFuzzyValue()")) 
    return(OFuzzySet::staticGetFuzzyValue);
  if(name==Strings("FuzzySet::isCrisp()")) 
    return(OFuzzySet::staticIsCrisp);
  if(name==Strings("FuzzySet::_delete()")) return(OFuzzySet::staticDelete);
  if(name==Strings("FuzzySet::saturate()")) return(OFuzzySet::staticSaturate);
  if(name==Strings("FuzzySet::product()")) return(OFuzzySet::staticProduct);
  if(name==Strings("FuzzySet::combineMax()")) 
    return(OFuzzySet::staticCombineMax);
  if(name==Strings("FuzzySet::combineMin()")) 
    return(OFuzzySet::staticCombineMin);
  if(name==Strings("FuzzySet::combineSum()")) 
    return(OFuzzySet::staticCombineSum);
  if(name==Strings("FuzzySet::mu()")) 
    return(OFuzzySet::staticMu);
  if(name==Strings("FuzzySet::integral()")) 
    return(OFuzzySet::staticIntegral);
  if(name==Strings("FuzzySet::xIntegral()")) 
    return(OFuzzySet::staticXIntegral);
  if(name==Strings("FuzzySet::maxMu()")) 
    return(OFuzzySet::staticMaxMu);
  if(name==Strings("FuzzySet::maxMin()")) 
    return(OFuzzySet::staticMaxMin);
  if(name==Strings("FuzzySet::simplify()")) 
    return(OFuzzySet::staticSimplify);
  if(name==Strings("FuzzySet::normalize()")) 
    return(OFuzzySet::staticNormalize);
  if(name==Strings("FuzzySet::very()")) 
    return(OFuzzySet::staticVery);
  if(name==Strings("FuzzySet::moreOrLess()")) 
    return(OFuzzySet::staticMoreOrLess);
  if(name==Strings("FuzzySet::not()")) 
    return(OFuzzySet::staticNot);

  //----------------------------------------------------------------------//
  // Fuzzy Controller
  //----------------------------------------------------------------------//
  
  if (name==Strings("FuzzyController::_delete()"))
    return(FuzzyController::staticDestroy) ;
  if (name==Strings("FuzzyController::_compile()"))
    return(FuzzyController::staticCompile) ;
  if (name==Strings("FuzzyController::addRulesFromFile()"))
    return(FuzzyController::staticAddRulesFromFile) ;

  return((ExternFunction)0); // not found
}

//--------------------------------------------------------------------------//
// LookUp table for agents                                                  //
//--------------------------------------------------------------------------//

static Word fuzzySetWord(Word::create("FuzzySet"));
static Word fuzzyControllerWord(Word::create("FuzzyController"));

EXPORT_PREFIX Agent * getNewAgent(Model * model,long instNum)
{
  if(model->isA(::fuzzySetWord)) return(new OFuzzySet(model,instNum));
  if(model->isA(::fuzzyControllerWord))
                                 return(new FuzzyController(model,instNum));

  return((Agent *)0); // not found
}

//----------------------------------------------------------------------//
//
// OFuzzySet
//
//----------------------------------------------------------------------//

OFuzzySet::OFuzzySet(Model* model, int instNum) :
  Agent(model, instNum)
{
  _set = new FuzzySet ;
  _modified = accessAttribute(Word::create("modified"))->accessValue() ;
  _modified->setInteger(0) ;
}

//----------------------------------------------------------------------//

OFuzzySet::~OFuzzySet(void)
{
  if (_set)
    delete _set ;

  _set = (FuzzySet *) 0;
}

//----------------------------------------------------------------------//

void OFuzzySet::setCrispValue(double value)
{
  _set->setValue(value) ;
  _modified->setInteger(1) ;
}

//----------------------------------------------------------------------//

void OFuzzySet::setFuzzyValue(const StlVector< Point2D >& value)
{
  _set->setValue(value) ;
  _modified->setInteger(1) ;
}

//----------------------------------------------------------------------//

void OFuzzySet::setValue(const FuzzySet* fuzzySet) 
{
  if (_set)
    delete _set ;
  _set = new FuzzySet(*fuzzySet) ;
  _modified->setInteger(1) ;
}

//----------------------------------------------------------------------//

double OFuzzySet::getCrispValue(void) const 
{
  double result ;
  result = _set->getCrispValue() ;
  return (result) ;
}

//----------------------------------------------------------------------//

const StlVector<Point2D>&  OFuzzySet::getFuzzyValue(void) const 
{
  if (_set->getType() != FuzzySet::FUZZY)
    // there is no fuzzy value associated
    {
      Strings message=getName().print()+
                      " has no fuzzy value";
      getSimul()->manageError(new Error(Error::RUNTIME,message)) ;
    }

  return _set->getFuzzyValue() ;
}

//----------------------------------------------------------------------//

bool OFuzzySet::isCrisp(void) const
{
  return _set->getType() == FuzzySet::CRISP ;
}

//----------------------------------------------------------------------//

bool OFuzzySet::maxMu(StlVector<Point2D>& vect) const
{
  return( _set->maxMu(vect)) ;
}

//----------------------------------------------------------------------//

void OFuzzySet::staticSetFuzzyValue(Context* context)
{
  OFuzzySet* oFuzzySet = (OFuzzySet *) context->accessOwner() ;

  StlVector< Point2D > vect ;
  const SubValueVect & pointVect = *context->getValueVector(0) ;
  int size = pointVect.getSize() ;
  
  Point2D point ;
  int i ;
  for (i=0; i<size; i++)
    {
      point.setX(pointVect[i]->getElem(0).getReal()) ;
      point.setY(pointVect[i]->getElem(1).getReal()) ;
      vect.push_back(point) ;
    }

  // we check the points are increasing
  bool correct = true ;
  for (i=1; i<size && correct; i++)
    correct = (vect[i].getX() >= vect[i-1].getX()) ;
  
  if (!correct)
    {
      Strings message=oFuzzySet->getName().print()+
                      " : abscissa values should always increase";
      getSimul()->manageError(new Error(Error::RUNTIME,message)) ;
    }

  oFuzzySet->setFuzzyValue(vect) ;
}

//----------------------------------------------------------------------//

void OFuzzySet::staticSetCrispValue(Context* context) 
{
  OFuzzySet* oFuzzySet = (OFuzzySet *) context->accessOwner() ;
  oFuzzySet->setCrispValue(context->getReal(0)) ;
}

//----------------------------------------------------------------------//

void OFuzzySet::staticGetCrispValue(Context* context) 
{
  const OFuzzySet* oFuzzySet = (const OFuzzySet *) context->getOwner() ;

  double result = oFuzzySet->getCrispValue() ;
  context->returnReal(result) ;
}

//----------------------------------------------------------------------//

void OFuzzySet::staticIsCrisp(Context* context) 
{
  const OFuzzySet* oFuzzySet = (const OFuzzySet *) context->getOwner() ;
  bool result = oFuzzySet->isCrisp() ;
  context->returnInteger(result) ;
}

//----------------------------------------------------------------------//

void OFuzzySet::staticGetFuzzyValue(Context* context) 
{
  const OFuzzySet* oFuzzySet = (const OFuzzySet *) context->getOwner() ;
  const StlVector< Point2D > vect = oFuzzySet->getFuzzyValue(); 

  if (oFuzzySet->isCrisp())
  {
      Strings message=oFuzzySet->getName().print()+ "is not fuzzy" ;
      getSimul()->manageError(new Error(Error::RUNTIME,message)) ;
      return ;
  }

  ValueType resultType(context->getModule()->getResultType());
  TmpValue * result=TmpValue::getTmpValue(resultType);
  SubValueVect * points=SubValue::getSubValueVect();
  for (unsigned int i=0; i<vect.size(); i++)
    {
      SubValue * point=SubValue::getSubValue(ValueType(ValueType::REAL,1),
					     result);
      points->pushBack(point);
      SubValueVect * coords=SubValue::getSubValueVect();
      coords->pushBack(SubValue::getSubValue(ValueType(ValueType::REAL),
					     point));
      coords->accessBack()->setReal(vect[i].getX());
      coords->pushBack(SubValue::getSubValue(ValueType(ValueType::REAL),
					     point));
      coords->accessBack()->setReal(vect[i].getY());
      point->setValueVector(coords);
    }
  result->setValueVector(points);
  context->returnValue(result);
}

//----------------------------------------------------------------------//

void OFuzzySet::staticDelete(Context * context) 
{
  OFuzzySet* oFuzzySet = (OFuzzySet *) context->accessOwner() ;
  if (oFuzzySet->_set)
    delete oFuzzySet->_set ;
  oFuzzySet->_set = (FuzzySet *) 0 ;
}

//----------------------------------------------------------------------//

void OFuzzySet::staticSaturate(Context* context) 
{
  OFuzzySet* oFuzzySet = (OFuzzySet *) context->accessOwner() ;
  oFuzzySet->saturate(context->getReal(0)) ;
}

//----------------------------------------------------------------------//

void OFuzzySet::staticProduct(Context* context) 
{
  OFuzzySet* oFuzzySet = (OFuzzySet *) context->accessOwner() ;
  oFuzzySet->product(context->getReal(0)) ;
}

//----------------------------------------------------------------------//

void OFuzzySet::staticCombineMax(Context* context) 
{
  OFuzzySet* oFuzzySet = (OFuzzySet *) context->accessOwner() ;
  const FuzzySet* set = ((const OFuzzySet *) context->getAgent(0))->getFuzzySet() ;
  if ((set->getType() == FuzzySet::CRISP) || oFuzzySet->isCrisp())
    getSimul()->manageError(new Error(Error::RUNTIME, "combineMax : sets should not be crisp")) ;
  else
    oFuzzySet->combineMax(* set) ;
}

//----------------------------------------------------------------------//

void OFuzzySet::staticCombineMin(Context* context) 
{
  OFuzzySet* oFuzzySet = (OFuzzySet *) context->accessOwner() ;
  const FuzzySet* set = ((const OFuzzySet *) context->getAgent(0))->getFuzzySet() ;
  if ((set->getType() == FuzzySet::CRISP) || oFuzzySet->isCrisp())
    getSimul()->manageError(new Error(Error::RUNTIME, "combineMin : sets should not be crisp")) ;
  else
    oFuzzySet->combineMin(* set) ;
}

//----------------------------------------------------------------------//

void OFuzzySet::staticCombineSum(Context* context) 
{
  OFuzzySet* oFuzzySet = (OFuzzySet *) context->accessOwner() ;
  const FuzzySet* set = ((const OFuzzySet *) context->getAgent(0))->getFuzzySet() ;
  if ((set->getType() == FuzzySet::CRISP) || oFuzzySet->isCrisp())
    getSimul()->manageError(new Error(Error::RUNTIME, "combineSum : sets should not be crisp")) ;
  else
    oFuzzySet->combineSum(* set) ;
}

//----------------------------------------------------------------------//

void OFuzzySet::staticMu(Context* context) 
{
  const OFuzzySet* oFuzzySet = (const OFuzzySet *) context->getOwner() ;
  context->returnReal(oFuzzySet->mu(context->getReal(0))) ;
}

//----------------------------------------------------------------------//

void OFuzzySet::staticIntegral(Context* context) 
{
  const OFuzzySet* oFuzzySet = (const OFuzzySet *) context->getOwner() ;
  context->returnReal(oFuzzySet->integral()) ;
}

//----------------------------------------------------------------------//

void OFuzzySet::staticXIntegral(Context* context) 
{
  OFuzzySet* oFuzzySet = (OFuzzySet *) context->accessOwner() ;
  context->returnReal(oFuzzySet->xIntegral()) ;
}

//----------------------------------------------------------------------//

void OFuzzySet::staticMaxMu(Context* context) 
{
  OFuzzySet* oFuzzySet = (OFuzzySet *) context->accessOwner() ;
  StlVector<Point2D> vect ;
  int succeed ;

  succeed = oFuzzySet->maxMu(vect) ;

  if (succeed)
    {
      Value * result=context->accessLocal(0) ;
      SubValueVect * points=SubValue::getSubValueVect();
      for (unsigned int i=0; i<vect.size(); i++)
	{
	  SubValue * point=SubValue::getSubValue(ValueType(ValueType::REAL,1),
						 result);
	  points->pushBack(point);
	  SubValueVect * coords=SubValue::getSubValueVect();
	  coords->pushBack(SubValue::getSubValue(ValueType(ValueType::REAL),
						 point));
	  coords->accessBack()->setReal(vect[i].getX());
	  coords->pushBack(SubValue::getSubValue(ValueType(ValueType::REAL),
						 point));
	  coords->accessBack()->setReal(vect[i].getY());
	  point->setValueVector(coords);
	}
      result->setValueVector(points);
    }

  context->returnInteger(succeed) ;
}

//----------------------------------------------------------------------//

void OFuzzySet::staticMaxMin(Context* context) 
{
  const OFuzzySet* oFuzzySet = (const OFuzzySet *) context->getOwner() ;
  double val = oFuzzySet->maxMin(* ((const OFuzzySet *) 
				    context->getAgent(0))->getFuzzySet()) ;
  context->returnReal(val) ;
}

//----------------------------------------------------------------------//

void OFuzzySet::staticSimplify(Context* context) 
{
  OFuzzySet* oFuzzySet = (OFuzzySet *) context->accessOwner() ;
  oFuzzySet->simplify() ;
}

//----------------------------------------------------------------------//

void OFuzzySet::staticNormalize(Context* context) 
{
  OFuzzySet* oFuzzySet = (OFuzzySet *) context->accessOwner() ;
  oFuzzySet->normalize() ;
}


//----------------------------------------------------------------------//

void OFuzzySet::staticVery(Context* context)
{
  const OFuzzySet* oset = (const OFuzzySet *) context->getOwner() ;
  const FuzzySet* set = oset->getFuzzySet() ;
  FuzzySet result = set->fVery() ;

  Simulator* simul = getSimul() ;
  Model* model = getSimul()->accessModel(Word::create("FuzzySet")) ;
  TmpValue *fuzzySet = TmpValue::getTmpValue(ValueType(model));

  simul->createAgent(fuzzySet, true, model, 0) ;
  simul->startModule(model->accessNewModule(),
		     fuzzySet->getAgent()) ;
  ((OFuzzySet *) fuzzySet->getAgent())->setValue(&result) ;
  context->returnValue(fuzzySet) ;
}


//----------------------------------------------------------------------//

void OFuzzySet::staticMoreOrLess(Context* context)
{
  const OFuzzySet* oset = (const OFuzzySet *) context->getOwner() ;
  const FuzzySet* set = oset->getFuzzySet() ;
  FuzzySet result = set->fMoreOrLess() ;

  Simulator* simul = getSimul() ;
  Model* model = getSimul()->accessModel(Word::create("FuzzySet")) ;
  TmpValue *fuzzySet = TmpValue::getTmpValue(ValueType(model));

  simul->createAgent(fuzzySet, true, model, 0) ;
  simul->startModule(model->accessNewModule(),
		     fuzzySet->getAgent()) ;
  ((OFuzzySet *) fuzzySet->getAgent())->setValue(&result) ;
  context->returnValue(fuzzySet) ;
}

//----------------------------------------------------------------------//

void OFuzzySet::staticNot(Context* context)
{
  const OFuzzySet* oset = (const OFuzzySet *) context->getOwner() ;
  const FuzzySet* set = oset->getFuzzySet() ;
  FuzzySet result = set->fNot() ;

  Simulator* simul = getSimul() ;
  Model* model = getSimul()->accessModel(Word::create("FuzzySet")) ;
  TmpValue *fuzzySet = TmpValue::getTmpValue(ValueType(model));

  simul->createAgent(fuzzySet, true, model, 0) ;
  simul->startModule(model->accessNewModule(),
   		     fuzzySet->getAgent()) ;
  ((OFuzzySet *) fuzzySet->getAgent())->setValue(&result) ;
  context->returnValue(fuzzySet) ;
}

