%{
//-------------------------------------------------------------------------
//
// File    	:  fuzzyScan.l
// Description  :  Lexical analysis of fuzzy rules
//
// Author(s)   	: P. Reignier
// Date         : 01/04/98
// Copyright	: Laboratoire d'Informatique Industrielle (LI2)
//		  Ecole Nationale d'Ingenieurs de Brest (ENIB)
//		  CP 15 - 29608 Brest Cedex - FRANCE
//		  Tel: (33) 98.05.66.31, Fax: (33) 98.05.66.29
//		  e-mail: li2@li2.enib.fr
//-------------------------------------------------------------------------
// Copyright (c) 1996
// Laboratoire d'Informatique Industrielle (LI2)
// Ecole Nationale d'Ingenieurs de Brest (ENIB)
//
// Permission to use, copy, modify, distribute and sell this software
// and its documentation for any purpose is hereby granted without fee,
// provided that the above copyright notice appear in all copies and
// that both that copyright notice and this permission notice appear
// in supporting documentation.
// ENIB/LI2 makes no representations about the suitability of this
// software for any purpose.
// It is provided "as is" without express or implied warranty.
//-------------------------------------------------------------------------

#include <string.h>
#include <fstream>

#include "Utility/strings.h"
#include "fuzzyFNod.h"

class FuzzyParser ;
#include "fuzzyScan.h"
#include "fuzzyParse.h"

#ifndef __MSDOS__
#define malloc(size) BlockAllocator::allocate(size);
#define free(ptr)    BlockAllocator::free(ptr);
#endif
%}
%name FuzzyLexer
%define INHERIT : public BlockAllocatedObject
%define IOSTREAM
%define LEX_PARAM FuzzyParser *parser
%define CONSTRUCTOR_PARAM istream* stream
%define CONSTRUCTOR_CODE yyin = stream
%define CLASS FuzzyLexer

%x comment

blanc		[ \t]+
digit		[0-9]
letter		[_a-zA-Z]
term		{letter}({letter}|{digit})*
%%


{blanc}		;
"\n"		{parser->incNbLine() ; parser->_line=Strings(""); }
"if"		{parser->_line+=Strings((char *) yytext);
		parser->_line+=Strings(" "); return FuzzyParser::IF ;}
"is"		{parser->_line+=Strings((char *) yytext);
		parser->_line+=Strings(" "); return FuzzyParser::IS ;}
"then"		{parser->_line+=Strings((char *) yytext);
		parser->_line+=Strings(" "); return FuzzyParser::THEN ;}
"and"		{parser->_line+=Strings((char *) yytext);
		parser->_line+=Strings(" "); return FuzzyParser::AND ;}
"or"		{parser->_line+=Strings((char *) yytext);
		parser->_line+=Strings(" "); return FuzzyParser::OR ;}
"("		{parser->_line+=Strings((char *) yytext);
		parser->_line+=Strings(" "); return FuzzyParser::PO ;}
")"		{parser->_line+=Strings((char *) yytext);		
		parser->_line+=Strings(" "); return FuzzyParser::PF ;}
";"		{parser->_line+=Strings((char *) yytext);
		parser->_line+=Strings(" "); return FuzzyParser::PV ;}
"delta"		{parser->_line+=Strings((char *) yytext);
		parser->_line+=Strings(" "); return FuzzyParser::DELTA ;}
{term}		{parser->yylval.string = new Strings((char *) yytext); 
		parser->_line +=Strings((char *) yytext);
		parser->_line+=Strings(" "); return FuzzyParser::TERM ;}
"//"		{BEGIN(comment) ;}
<comment>[^\n]* /* eats the comment */
<comment>"\n"	{parser->incNbLine(); parser->_line[0]='\0'; BEGIN(0) ;}
.		/* eats any non matched caracters */
%%
