%{
//-------------------------------------------------------------------------
//
// File    	:  fuzzyScan.l
// Description  :  Lexical analysis of fuzzy rules
//
// Author(s)   	: P. Reignier
// Date         : 01/04/98
// Copyright	: Laboratoire d'Informatique Industrielle (LI2)
//		  Ecole Nationale d'Ingenieurs de Brest (ENIB)
//		  CP 15 - 29608 Brest Cedex - FRANCE
//		  Tel: (33) 98.05.66.31, Fax: (33) 98.05.66.29
//		  e-mail: li2@li2.enib.fr
//-------------------------------------------------------------------------
// Copyright (c) 1996
// Laboratoire d'Informatique Industrielle (LI2)
// Ecole Nationale d'Ingenieurs de Brest (ENIB)
//
// Permission to use, copy, modify, distribute and sell this software
// and its documentation for any purpose is hereby granted without fee,
// provided that the above copyright notice appear in all copies and
// that both that copyright notice and this permission notice appear
// in supporting documentation.
// ENIB/LI2 makes no representations about the suitability of this
// software for any purpose.
// It is provided "as is" without express or implied warranty.
//-------------------------------------------------------------------------

#include <string.h>
#include <fstream>

#include "Utility/strings.h"
#include "fuzzyController.h"

class FuzzyParserLoad ;
#include "fuzzyScanLoad.h"
#include "fuzzyParseLoad.h"

#ifndef __MSDOS__
#define malloc(size) BlockAllocator::allocate(size);
#define free(ptr)    BlockAllocator::free(ptr);
#endif
%}
%name FuzzyLexerLoad
%define INHERIT : public BlockAllocatedObject
%define IOSTREAM
%define LEX_PARAM FuzzyParserLoad *parser
%define CONSTRUCTOR_PARAM istream* stream
%define CONSTRUCTOR_CODE yyin = stream
%define MEMBERS void addLexem(unsigned char * yytext, FuzzyParserLoad* parser);
%define CLASS FuzzyLexerLoad

%x comment

blanc		[ \t]+
digit		[0-9]
letter		[_a-zA-Z]
term		{letter}({letter}|{digit})*
%%


{blanc}		;
"\n"		{parser->_line=Strings(""); parser->_lineNumber++;}
"if"		{parser->_line+=Strings((const char *) yytext);
		 parser->_line+=Strings(" "); 
                 addLexem(yytext, parser) ;
                 return FuzzyParserLoad::IF ;}
"is"		{parser->_line+=Strings((const char *) yytext);
                 addLexem(yytext, parser) ;
		 parser->_line+=Strings(" "); 
                 return FuzzyParserLoad::IS ;}
"then"		{parser->_line+=Strings((const char *) yytext);
		 parser->_line+=Strings(" "); 
                 addLexem(yytext, parser) ;
                 return FuzzyParserLoad::THEN ;}
"and"		{parser->_line+=Strings((const char *) yytext);
		 parser->_line+=Strings(" "); 
                 addLexem(yytext, parser) ;
                 return FuzzyParserLoad::AND ;}
"or"		{parser->_line+=Strings((const char *) yytext);
	      	 parser->_line+=Strings(" "); 
                 addLexem(yytext, parser) ;
                 return FuzzyParserLoad::OR ;}
"("		{parser->_line+=Strings((const char *) yytext);
		 parser->_line+=Strings(" "); 
                 addLexem(yytext, parser) ;
                 return FuzzyParserLoad::PO ;}
")"		{parser->_line+=Strings((const char *) yytext);		
		 parser->_line+=Strings(" "); 
                 addLexem(yytext, parser) ;
                 return FuzzyParserLoad::PF ;}
";"		{parser->_line+=Strings((const char *) yytext);
                 addLexem(yytext, parser) ;
		 parser->_line+=Strings(" "); return FuzzyParserLoad::PV ;}
"delta"		{parser->_line+=Strings((const char *) yytext);
		 parser->_line+=Strings(" "); 
                 addLexem(yytext, parser) ;
                 return FuzzyParserLoad::DELTA ;}
{term}		{parser->_line +=Strings((const char *) yytext);
		 parser->_line+=Strings(" "); 
                 addLexem(yytext, parser) ;
                 return FuzzyParserLoad::TERM ;}
"//".*		/* eats comments */
.		/* eats any non matched caracters */
%%

void FuzzyLexerLoad::addLexem(unsigned char* txt, FuzzyParserLoad* parser)
{
  Strings* result = new Strings((const char *) txt) ;
  *result += Strings(" ") ;
  parser->yylval.string = result ;
}
