//-------------------------------------------------------------------------
//
// File    	:  linguisFNod.C
// Description  :  linguistic part of a rule (condition or conclusion)
//
// Author(s)   	: P. Reignier
// Date         : 23/03/98
// Copyright	: Laboratoire d'Informatique Industrielle (LI2)
//		  Ecole Nationale d'Ingenieurs de Brest (ENIB)
//		  CP 15 - 29608 Brest Cedex - FRANCE
//		  Tel: (33) 98.05.66.31, Fax: (33) 98.05.66.29
//		  e-mail: li2@li2.enib.fr
//-------------------------------------------------------------------------
// Copyright (c) 1996
// Laboratoire d'Informatique Industrielle (LI2)
// Ecole Nationale d'Ingenieurs de Brest (ENIB)
//
// Permission to use, copy, modify, distribute and sell this software
// and its documentation for any purpose is hereby granted without fee,
// provided that the above copyright notice appear in all copies and
// that both that copyright notice and this permission notice appear
// in supporting documentation.
// ENIB/LI2 makes no representations about the suitability of this
// software for any purpose.
// It is provided "as is" without express or implied warranty.
//-------------------------------------------------------------------------

#ifndef __MSDOS__
// #include <stl_config.h>
#endif

#include "linguisFNod.h"
#include "fuzzyFNod.h"
#include "nodeData.h"
#include "Utility/strings.h"
#include "oRis/Main/attrib.h"
#include "oRis/Main/agent.h"
#include "oRis/Main/value.h"
#include "oRis/Main/simul.h"
#include "oRis/Main/errors.h"

//----------------------------------------------------------------------//
//
// Constructors
//
//----------------------------------------------------------------------//

//------linguisFNod::linguisFNod(....) ---------------------------------//

LinguisFNod::LinguisFNod (Strings data) 
  : FuzzyFNod() 
{
  _lingName = data ;
  _signature = Strings("linguistic : ") + data ; 
} 

//------linguisFNod::linguisFNod(linguisFNod&)--------------------------//

LinguisFNod::LinguisFNod (const LinguisFNod& node) : FuzzyFNod(node) 
{
  _lingName = node._lingName ;
}

//----------------------------------------------------------------------//
//
// Destructor
//
//----------------------------------------------------------------------//

//------linguisFNod::~linguisFNod()-------------------------------------//

LinguisFNod::~LinguisFNod(void) {}


//----------------------------------------------------------------------//
//
// Code generation
//
//----------------------------------------------------------------------//

void LinguisFNod::generate(FNodData& data) 
{
  const Agent* agt = data.getVar() ;
  const SubValueVect* vect ;
	
  // we first check that the linguistic term describes the variable
  if (data.isDerivative())
    vect = agt->getAttribute(Word::create("_derivateVocabulary"))->getValue()
      ->getValueVector() ;
  else
    vect = agt->getAttribute(Word::create("_vocabulary"))->getValue()
                                                         ->getValueVector() ;

  bool found = false ;
  size_t indice=0;
  for (size_t i=0; i<vect->getSize() && !found; i++)
    {
      if ((*vect)[i]->getAgent()->getAttribute(Word::create("_name"))
                                ->getValue()->getStrings() == _lingName)
	{
	  found = true ;
	  indice = i ;
	}
    }
  
  if (!found)
    {
      if (data.isDerivative())
	getSimul()->manageError(new Error(Error::RUNTIME, 
					  _lingName + " is not in the " +
					  "derivative vocabulary of " + 
					  agt->getName().print())) ;
      else
	getSimul()->manageError(new Error(Error::RUNTIME, 
					  _lingName + " is not in the " +
					  "vocabulary of " + 
					  agt->getName().print())) ;
      data.setError(true) ;
      return ;
    }
  

  // we generate the code to get the fuzzy set associated to the linguistic 
  // term
  Strings tmpVar = data.allocateTmpFuzzyVar() ;
  data.appendProg("FuzzySet " + tmpVar) ;
  data.appendProg(" = " + (*vect)[indice]->getAgent()
		  ->getName().print() +
		  "->getValue();\n") ;
  data.setResult(tmpVar) ;
}

