//-------------------------------------------------------------------------
//
// File    	:  point2D.cpp
// Description  :  point 2D
//
// Author(s)   	: P. Reignier
// Date         : 20/03/98
// Copyright	: Laboratoire d'Informatique Industrielle (LI2)
//		  Ecole Nationale d'Ingenieurs de Brest (ENIB)
//		  CP 15 - 29608 Brest Cedex - FRANCE
//		  Tel: (33) 98.05.66.31, Fax: (33) 98.05.66.29
//		  e-mail: li2@li2.enib.fr
//-------------------------------------------------------------------------
// Copyright (c) 1996
// Laboratoire d'Informatique Industrielle (LI2)
// Ecole Nationale d'Ingenieurs de Brest (ENIB)
//
// Permission to use, copy, modify, distribute and sell this software
// and its documentation for any purpose is hereby granted without fee,
// provided that the above copyright notice appear in all copies and
// that both that copyright notice and this permission notice appear
// in supporting documentation.
// ENIB/LI2 makes no representations about the suitability of this
// software for any purpose.
// It is provided "as is" without express or implied warranty.
//-------------------------------------------------------------------------


#include <iostream>
#include <math.h>

#include "point2D.h"

const double Point2D::epsilon = 0.0001 ;


//----------------------------------------------------------------------//
//
//   Constructors
//
//----------------------------------------------------------------------//

//------Point2D::Point2D(x,y)-------------------------------------------//

Point2D::Point2D (double x, double y)
 : BlockAllocatedObject()
{
  _x = x ;
  _y = y ;
}


//------Point2D::Point2D(Point2D)---------------------------------------//

Point2D::Point2D(const Point2D& p) 
 : BlockAllocatedObject()
{
  _copy(p) ;
}


//----------------------------------------------------------------------//
//
// Destructors
//
//----------------------------------------------------------------------//


//-------Point2D::~Point2D()--------------------------------------------//

Point2D::~Point2D (void)
{}


//----------------------------------------------------------------------//
//
// Operators
//
//----------------------------------------------------------------------//

//------operator<<()----------------------------------------------------//

ostream& operator<<(ostream& flot, const Point2D& p)
{
  flot << p._x << " " << p._y  ;
  return flot ;
}


//------operator==()----------------------------------------------------//

bool operator==(const Point2D& p1, const Point2D& p2)
{
  double deltaX = fabs(p1.getX() - p2.getX()) ;
  double deltaY = fabs(p2.getY() - p2.getY()) ;
  if ((deltaX <= Point2D::epsilon) && (deltaY <= Point2D::epsilon))
    return (true) ;
  else
    return (false) ;
}

//------operator==()----------------------------------------------------//

bool operator!=(const Point2D& p1, const Point2D& p2)
{
  return ! (p1 == p2) ;
}

//------operator+()-----------------------------------------------------//

Point2D& operator+(const Point2D& p1, const Point2D& p2)
{
  Point2D *result = new Point2D ;
  
  result->setX(p1.getX() + p2.getX()) ;
  result->setY(p1.getY() + p2.getY()) ;

  return (*result) ;
}


//------Point2D::operator=()--------------------------------------------//

Point2D& Point2D::operator=(const Point2D& p) 
{
  if (this != &p)
    _copy (p) ;

  return *this ;
}


//----------------------------------------------------------------------//
//
// Utilities
//
//----------------------------------------------------------------------//

//------Point2D::_copy--------------------------------------------------//

void Point2D::_copy (const Point2D& p)
{
  _x = p.getX() ;
  _y = p.getY() ;
}




