//-------------------------------------------------------------------------
//
// File    	:  varConclFNod.C
// Description  :  fuzzy variable in the right part of a rule (Concl part)
//
// Author(s)   	: P. Reignier
// Date         : 30/03/98
// Copyright	: Laboratoire d'Informatique Industrielle (LI2)
//		  Ecole Nationale d'Ingenieurs de Brest (ENIB)
//		  CP 15 - 29608 Brest Cedex - FRANCE
//		  Tel: (33) 98.05.66.31, Fax: (33) 98.05.66.29
//		  e-mail: li2@li2.enib.fr
//-------------------------------------------------------------------------
// Copyright (c) 1996
// Laboratoire d'Informatique Industrielle (LI2)
// Ecole Nationale d'Ingenieurs de Brest (ENIB)
//
// Permission to use, copy, modify, distribute and sell this software
// and its documentation for any purpose is hereby granted without fee,
// provided that the above copyright notice appear in all copies and
// that both that copyright notice and this permission notice appear
// in supporting documentation.
// ENIB/LI2 makes no representations about the suitability of this
// software for any purpose.
// It is provided "as is" without express or implied warranty.
//-------------------------------------------------------------------------

#ifndef __MSDOS__
// #include <stl_config.h>
#endif

#include "varConclFNod.h"
#include "fuzzyFNod.h"
#include "nodeData.h"
#include "Utility/strings.h"
#include "fuzzyController.h"
#include "oRis/Main/attrib.h"
#include "oRis/Main/agent.h"
#include "oRis/Main/value.h"
#include "oRis/Main/simul.h"
#include "oRis/Main/errors.h"

//----------------------------------------------------------------------//
//
// Constructors
//
//----------------------------------------------------------------------//

//------varConclFNod::varConclFNod(....) -------------------------------//

VarConclFNod::VarConclFNod (Strings varName) : FuzzyFNod() 
{
  _variable = varName ;
  _signature = "variable (left)" + varName ;
} 

//------varConclFNod::varConclFNod(varConclFNod&)-----------------------//

VarConclFNod::VarConclFNod (const VarConclFNod& node) : FuzzyFNod(node) 
{
  _variable = node._variable ;
}

//----------------------------------------------------------------------//
//
// Destructor
//
//----------------------------------------------------------------------//

//------varConclFNod::~varConclFNod()-----------------------------------//

VarConclFNod::~VarConclFNod(void) {}


//----------------------------------------------------------------------//
//
// Code generation
//
//----------------------------------------------------------------------//

void VarConclFNod::generate(FNodData& data)
{
  const Value* vars = data.getFuzzyController()->
    getAttribute(Word::create("_variables"))->
    getValue() ;
  const SubValueVect* vect = vars->getValueVector() ;
  
  size_t size = vect->getSize() ;
  bool found = false ;
  size_t indice = 0 ;

  for(size_t i=0; i<size && !found; i++)
    {
      Agent* agt = (*vect)[i]->getAgent() ;
      if (agt->getAttribute(Word::create("_name"))->getValue()->getStrings()
          == _variable)
	    {
	      found = true ;
	      indice = i ;
	      data.setVar(agt) ;
        data.accessFuzzyController()->addUsedVariable(agt) ;
	    }
    }

  if (found)
    {
      Strings tmpVar = data.allocateTmpFuzzyVar() ;
      data.appendProg("FuzzySet " + tmpVar) ;
      if (data.isDerivative())
      {
        data.appendProg(Strings("=") + 
		      (*vect)[indice]->getAgent()->getName().print() + 
		      Strings("->getFuzzyDerivative();\n")) ;
      }
      else
      {
        data.appendProg(Strings("=") + 
		        (*vect)[indice]->getAgent()->getName().print() + 
		        Strings("->getFuzzyValue();\n")) ;
      }
      data.setResult(tmpVar) ;
    }
  else
    {
      data.setError(true) ;
      getSimul()->manageError(new Error(Error::RUNTIME,
		"Fuzzy : " + _variable + " is not a variable of " +	
			      data.getFuzzyController()->getName().print())) ;
    }
}

