%-------------------------------------------------------------------------
%
% File    	:  fuzzy.pl
% Description   :  support prolog au systeme flou
%
% Author(s)   	: P. Reignier
% Date          : 16/03/98
% Copyright	: Laboratoire d'Informatique Industrielle (LI2)
%		  Ecole Nationale d'Ingenieurs de Brest (ENIB)
%		  CP 15 - 29608 Brest Cedex - FRANCE
%		  Tel: (33) 98.05.66.31, Fax: (33) 98.05.66.29
%		  e-mail: li2@li2.enib.fr
%-------------------------------------------------------------------------
% Copyright (c) 1996
% Laboratoire d'Informatique Industrielle (LI2)
% Ecole Nationale d'Ingenieurs de Brest (ENIB)
%
% Permission to use, copy, modify, distribute and sell this software
% and its documentation for any purpose is hereby granted without fee,
% provided that the above copyright notice appear in all copies and
% that both that copyright notice and this permission notice appear
% in supporting documentation.
% ENIB/LI2 makes no representations about the suitability of this
% software for any purpose.
% It is provided "as is" without express or implied warranty.
%-------------------------------------------------------------------------

% predicats predefinis en C :
%   fuzzyConf(Name,Type,Valeur)
%      avec Name = nom du controleur
%      avec Type = et,ou,ainsiQue,implication,decodage
%      et Valeur respectivement (min,produit,produitBorne,produitDrastic), 
%                               (max,sommeBornee,sommeDrastic),
%                               (somme,max),
%                               (mandani,larsen),
%                               (max,moyenneMax,centre)

getFuzzyLing(PredName, VarName, LingName, Value) :-
	PredName =.. ListPredName,
	conc(ListPredName,[variable(VarName,_,ListeLing)],ListPredCall),
	PredCall =.. ListPredCall,
	call(PredCall),
	Ling =.. [LingName,Value],
	extractFuzzyLing(Ling,ListeLing).

extractFuzzyLing(Ling,[Ling|_]) :- !.
extractFuzzyLing(Ling,[_|Q]) :- extractFuzzyLing(Ling,Q).

%----------------------------------------------------------------------

% the value of the var is crisp 
getFuzzyValue(PredName,VarName,Value) :-
	PredName =.. ListPredName,
	conc(ListPredName,[valeur(VarName,crisp,_,Value)],ListPredCall),
	PredCall =.. ListPredCall,
	call(PredCall).

% the value of the var is fuzzy
getFuzzyValue(PredName,VarName,Value) :-
	PredName =.. ListPredName,
	conc(ListPredName,[valeur(VarName,fuzzy,Value,_)],ListPredCall),
	PredCall =.. ListPredCall,
	call(PredCall).

% the value of the var is fuzzy and crisp : we choose the fuzzy value
getFuzzyValue(PredName,VarName,Value) :-
	PredName =.. ListPredName,
	conc(ListPredName,[valeur(VarName,both,Value,_)],ListPredCall),
	PredCall =.. ListPredCall,
	call(PredCall).

%----------------------------------------------------------------------

% the value of the var is crisp 
getFuzzyDerivate(PredName,VarName,Value) :-
	PredName =.. ListPredName,
	conc(ListPredName,[derivee(VarName,crisp,_,Value)],ListPredCall),
	PredCall =.. ListPredCall,
	call(PredCall).

% the value of the var is fuzzy
getFuzzyDerivate(PredName,VarName,Value) :-
	PredName =.. ListPredName,
	conc(ListPredName,[derivee(VarName,fuzzy,Value,_)],ListPredCall),
	PredCall =.. ListPredCall,
	call(PredCall).

% the value of the var is fuzzy and crisp : we choose the fuzzy value
getFuzzyDerivate(PredName,VarName,Value) :-
	PredName =.. ListPredName,
	conc(ListPredName,[derivee(VarName,both,Value,_)],ListPredCall),
	PredCall =.. ListPredCall,
	call(PredCall).

%----------------------------------------------------------------------

% the value of the var is crisp 
setFuzzyValue(PredName, VarName, crisp,_,CrispValue) :-
	PredName =.. ListPredName,
	conc(ListPredName,[valeur(VarName,crisp,_,CrispValue)],ListPredCall),
	PredCall =.. ListPredCall,
	call(PredCall).

% the value of the var is fuzzy
setFuzzyValue(PredName, VarName,fuzzy,FuzzyValue,_) :-
	PredName =.. ListPredName,
	conc(ListPredName,[valeur(VarName,fuzzy,FuzzyValue,_)],ListPredCall),
	PredCall =.. ListPredCall,
	call(PredCall).

% the value of the var is both 
setFuzzyValue(PredName, VarName,both,FuzzyValue,CrispValue) :-
	PredName =.. ListPredName,
	conc(ListPredName,[valeur(VarName,both,FuzzyValue,CrispValue)],
	     ListPredCall),
	PredCall =.. ListPredCall,
	call(PredCall).

%----------------------------------------------------------------------

% the value of the var is crisp 
setFuzzyDerivate(PredName, VarName, crisp,_,CrispValue) :-
	PredName =.. ListPredName,
	conc(ListPredName,[derivee(VarName,crisp,_,CrispValue)],ListPredCall),
	PredCall =.. ListPredCall,
	call(PredCall).

% the value of the var is fuzzy
setFuzzyDerivate(PredName, VarName,fuzzy,FuzzyValue,_) :-
	PredName =.. ListPredName,
	conc(ListPredName,[derivee(VarName,fuzzy,FuzzyValue,_)],ListPredCall),
	PredCall =.. ListPredCall,
	call(PredCall).

% the value of the var is both 
setFuzzyDerivate(PredName, VarName,both,FuzzyValue,CrispValue) :-
	PredName =.. ListPredName,
	conc(ListPredName,[derivee(VarName,both,FuzzyValue,CrispValue)],
	     ListPredCall),
	PredCall =.. ListPredCall,
	call(PredCall).

%----------------------------------------------------------------------

fuzzyPredefined([[X1,0],[X2,1],[X3,0]],triangle(X1,X2,X3)).
fuzzyPredefined([[X1,0],[X2,1],[X3,1],[X4,0]],trapeze(X1,X2,X3,X4)).
fuzzyPredefined([[X1,0],[X2,1]],rampe_haut(X1,X2)).
fuzzyPredefined([[X1,1],[X2,0]],rampe_bas(X1,X2)).

